export default {
  hil: 3,
  render: {
    view_template: '@eab-aip/ai-template-split-view/ai-template-split-view'
  },
  runtime: {
    startup: [
      '[reference]{ /list_api }',
      '[reference]{ /detail }',
      '[reference]{ /detail_header_controls }',
      '[reference]{ /filter_form }'
    ],
    tasks: {
      view: {
        $is: 'task',
        model_override: {
          title: 'Resources',
          description: '',
          list_zone: {
            header: 'People'
          },
          detail_zone: {
            header: '[value]{ /title |or| -- }'
          }
        }
      },
      location: {
        $is: 'task',
        success: [
          {
            $is: 'assignment',
            settings: {
              target: {
                id: 'route_list'
              }
            },
            model_fallback: '-- continue --',
            before_guards: ['[guard]{ [reference]{ /location/value/route } like /list }']
          },
          {
            $is: 'assignment',
            settings: {
              target: {
                id: 'route_detail'
              }
            },
            model_fallback: '-- continue --',
            before_guards: ['[guard]{ [reference]{ /location/value/route } like /detail }']
          }
        ]
      },
      route_list: {
        $is: 'task',
        model: null,
        success: ['[assignment]{ /detail }', '[assignment]{ /show_left }']
      },
      route_detail: {
        $is: 'mutate',
        model_import: {
          id: 'list'
        },
        settings: {
          mutation_type: 'table'
        },
        type: {
          mutations: {
            filter_by: {
              0: {
                comparator: '==',
                field: '/id',
                value: '[route]{ :id }'
              }
            }
          },
          fields: {
            id: {
              render_type: 'text'
            }
          }
        },
        after_modifiers: ['[f1]{ /nth 0 }'],
        success: ['[assignment]{ /view }', '[assignment]{ /detail }', '[assignment]{ /show_right }']
      },
      list_api: {
        $is: 'api',
        settings: {
          uri: '[constant]{ @stc_api_base_uri }/api/students/[constant]{ @sgd_var__student_nk }/support_team'
        },
        after_modifiers: [
          {
            $is: 'task',
            model: '[value]{ /team }'
          },
          {
            $is: 'task',
            model: async (model) => {
              return { val: [...new Map(model.map((item) => [item.id, item])).values()] };
            }
          },
          {
            $is: 'series',
            model_override: {
              full_name: '[value]{ /first_name } \\s [value]{ /last_name }',
              title: '[value]{ /first_name } \\s [value]{ /last_name } \\s [value]{ /formatted_pronouns }',
              description: {
                $is: 'condition',
                settings: {
                  if: {
                    comparator: '==',
                    left: '[value] {/work_phone |or| --}',
                    right: '--'
                  },
                  is_true: '[value]{ /email |or| --}',
                  is_false: {
                    $is: 'condition',
                    settings: {
                      if: {
                        comparator: '==',
                        left: '[value] {/email |or| --}',
                        right: '--'
                      },
                      is_true: '[value]{ /work_phone}',
                      is_false: '[value]{ /email }\\s|\\s[value]{ /work_phone }'
                    }
                  }
                }
              },
              badges: {
                $is: 'condition',
                settings: {
                  if: {
                    comparator: 'of',
                    left: '[value] {/relationship_type}',
                    right: 'array'
                  },
                  is_true: '[value] {/relationship_type}',
                  is_false: {
                    0: '[value]{ /relationship_type }'
                  }
                }
              },
              appointment: '#/my/appointment-new',
              courses: {
                $is: 'condition',
                settings: {
                  if: {
                    comparator: '==',
                    left: '[value] {/courses |or| --}',
                    right: '--'
                  },
                  is_true: '--',
                  is_false: {
                    $is: 'series',
                    settings: {
                      path: '/courses'
                    },
                    model: '[value] {/name}',
                    after_modifiers: ['[f1] {join ,\\s}']
                  }
                }
              },
              terms: {
                $is: 'condition',
                settings: {
                  if: {
                    comparator: '==',
                    left: '[value] {/courses |or| --}',
                    right: '--'
                  },
                  is_true: '--',
                  is_false: {
                    $is: 'series',
                    settings: {
                      path: '/courses'
                    },
                    model: '[value] {/term_name}',
                    after_modifiers: ['[f1]{ /unique }', '[f1] {join ,\\s}']
                  }
                }
              },
              availability: {
                $is: 'condition',
                settings: {
                  if: {
                    comparator: '==',
                    left: '[value] {/availability |or| --}',
                    right: '--'
                  },
                  is_true: '[]',
                  is_false: {
                    $is: 'series',
                    settings: {
                      path: '/availability'
                    },
                    model: {
                      id: '[value] {/id}',
                      title: '[join] { [value]{/days} \\s [value]{/times}} ',
                      purpose: '[value] {/purpose |or| --}',
                      dates: '[value] {/dates |or| --}',
                      meeting_url_or_phone: '[value] {/meeting_url_or_phone |or| --}',
                      details: '[value] {/details |or| --}',
                      meeting_types: {
                        $is: 'condition',
                        settings: {
                          if: {
                            comparator: 'of',
                            left: '[value] {/meeting_types}',
                            right: 'empty'
                          },
                          is_true: '--',
                          is_false: {
                            $is: 'series',
                            settings: {
                              path: '/meeting_types'
                            },
                            model: '[value] {/name}',
                            after_modifiers: ['[f1] {join ,\\s}']
                          }
                        }
                      },
                      care_unit: '[value] {/care_unit/name |or| --}',
                      location: '[value] {/location/name}',
                      services: {
                        $is: 'condition',
                        settings: {
                          if: {
                            comparator: 'of',
                            left: '[value] {/services}',
                            right: 'empty'
                          },
                          is_true: '--',
                          is_false: {
                            $is: 'series',
                            settings: {
                              path: '/services'
                            },
                            model: '[value] {/name}',
                            after_modifiers: ['[f1] {join ,\\s}']
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        ],
        success: ['[assignment]{ /list }']
      },
      list: {
        $is: 'task',
        model_import: {
          id: 'list_api'
        },
        after_modifiers: [
          {
            $is: 'mutate',
            settings: {
              mutation_type: 'table'
            },
            type: {
              fields: {
                relationship_type: {
                  data_type: 'list'
                }
              },
              mutations: {
                filter_by: {
                  $is: 'mutate',
                  model_import: {
                    id: 'submit_filter'
                  },
                  settings: {
                    mutation_type: 'filter_form'
                  }
                }
              }
            }
          }
        ],
        type: {
          fields: {
            title: {
              render_type: 'link',
              settings: {
                href: '#/my/host/resources/people/detail/[local]{ /id }'
              },
              value: {
                in: '[reference]{ /show_right }'
              }
            },
            full_name: {
              render_type: 'text'
            },
            description: {
              render_type: 'text'
            },
            actions: {
              render_type: 'factory-controls',
              searchable: false,
              fields: {
                item_menu: {
                  render_type: 'menu',
                  attributes: {
                    aesthetic: 'icon',
                    placement: 'bottom'
                  },
                  settings: {
                    icon: 'more'
                  },
                  source: {
                    out: [
                      {
                        value: 1,
                        title: 'View Details'
                      }
                    ]
                  },
                  value: {
                    in: {
                      $is: 'proxy',
                      task: {
                        // parse the value from the menu `(value = 1 is "View Details")
                        //
                        $is: 'algorithm',
                        settings: {
                          // the $event is the value chosen
                          //
                          active_algorithm: '[value]{ /$event }',
                          algorithms: {
                            1: {
                              $is: 'proxy',
                              task: {
                                $is: 'task',
                                success: [
                                  // the entity in the row is the value - we cannot use local because
                                  // we are sending data out of the type in to other tasks
                                  //
                                  '[goto]{ #/my/host/resources/people/detail/[value]{ /id } }',
                                  '[assignment]{ /show_right }'
                                ]
                              }
                            }
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }
      },
      detail: {
        $is: 'task',
        type: {
          fields: {
            switch_panel: {
              value: {
                in: '[reference]{ /show_left }'
              }
            },
            badges: {
              render_type: 'badges',
              hidden_title: true
            },
            email: {
              render_type: 'link',
              settings: {
                $is: 'condition',
                settings: {
                  if: {
                    comparator: '==',
                    left: '[value]{ /email }',
                    right: ''
                  },
                  is_true: {
                    icon: 'envelope',
                    href: '#',
                    label: '--'
                  },
                  is_false: {
                    icon: 'envelope',
                    href: 'mailto:[value]{ /email }',
                    raw_label: '[value]{ /email }'
                  }
                }
              },
              attributes: {
                $is: 'condition',
                settings: {
                  if: {
                    comparator: '==',
                    left: '[value]{ /email }',
                    right: ''
                  },
                  is_true: {
                    disabled: true
                  },
                  is_false: {
                    disabled: false
                  }
                }
              },
              hidden_title: true
            },
            work_phone: {
              render_type: 'link',
              settings: {
                $is: 'condition',
                settings: {
                  if: {
                    comparator: '==',
                    left: '[value]{ /work_phone }',
                    right: ''
                  },
                  is_true: {
                    icon: 'telephone',
                    href: '#',
                    label: '--'
                  },
                  is_false: {
                    icon: 'telephone',
                    href: 'tel:[value]{ /work_phone }',
                    label: '[value]{ /work_phone }'
                  }
                }
              },
              attributes: {
                $is: 'condition',
                settings: {
                  if: {
                    comparator: '==',
                    left: '[value]{ /work_phone }',
                    right: ''
                  },
                  is_true: {
                    disabled: true
                  },
                  is_false: {
                    target: '_system',
                    disabled: false
                  }
                }
              },
              hidden_title: true
            },
            appointment: {
              hidden: {
                $is: 'condition',
                settings: {
                  if: {
                    comparator: 'of',
                    left: '[value] {/availability}',
                    right: 'empty'
                  },
                  is_true: true,
                  is_false: false
                }
              },
              render_type: 'link',
              settings: {
                label: 'Schedule an Appointment',
                icon: 'people'
              },
              hidden_title: true
            },
            courses: {
              render_type: 'text',
              hidden: {
                $is: 'condition',
                settings: {
                  if: {
                    comparator: '==',
                    left: '[value] {/courses |or| --}',
                    right: '--'
                  },
                  is_true: true,
                  is_false: false
                }
              }
            },
            terms: {
              render_type: 'text',
              hidden: {
                $is: 'condition',
                settings: {
                  if: {
                    comparator: '==',
                    left: '[value] {/courses |or| --}',
                    right: '--'
                  },
                  is_true: true,
                  is_false: false
                }
              }
            },
            availability: {
              hidden: {
                $is: 'condition',
                settings: {
                  if: {
                    comparator: 'of',
                    left: '[value] {/availability}',
                    right: 'empty'
                  },
                  is_true: true,
                  is_false: false
                }
              },
              render_type: 'list-viewer',
              fields: {
                title: {
                  render_type: 'text',
                  title: 'Appointment Days and Times'
                },
                dates: {
                  render_type: 'text'
                },
                location: {
                  render_type: 'text'
                },
                care_unit: {
                  render_type: 'text'
                },
                services: {
                  render_type: 'text'
                },
                meeting_types: {
                  render_type: 'text',
                  title: 'Meeting Type'
                },
                purpose: {
                  render_type: 'text',
                  title: 'Appointment Type'
                },
                meeting_url_or_phone: {
                  render_type: 'text',
                  title: 'Phone/URL',
                  style: {
                    'overflow-wrap': 'break-word'
                  }
                },
                details: {
                  render_type: 'text',
                  title: 'Special Instructions'
                }
              },
              settings: {
                item_title_path: '/title'
              }
            }
          }
        }
      },
      filter_form: {
        $is: 'task',
        type: {
          fields: {
            relationship_type: {
              render_type: 'select-tags',
              hidden_title: true,
              settings: {
                multiple: true
              },
              source: {
                out: {
                  $is: 'task',
                  model_import: {
                    id: 'list_api'
                  },
                  after_modifiers: [
                    {
                      $is: 'series',
                      model: '[value]{ /relationship_type }'
                    },
                    '[f1]{ /flatten 1 }',
                    '[f1]{ /unique }',
                    {
                      $is: 'series',
                      model: {
                        value: '[value]{ }',
                        title: {
                          $is: 'value',
                          after_modifiers: ['[f1]{ /word }']
                        }
                      }
                    }
                  ]
                }
              }
            }
          }
        }
      },
      submit_filter: {
        $is: 'task',
        success: ['[assignment]{ /filter_form }', '[assignment]{ /list }']
      },
      detail_header_controls: {
        $is: 'task',
        model: {
          show_list_view: true
        },
        type: {
          fields: {
            show_list_view: {
              render_type: 'button',
              attributes: {
                aesthetic: 'icon',
                'x-control': 'detail-panel-mobile-back'
              },
              settings: {
                icon: 'go_back'
              },
              value: {
                in: '[reference]{ /show_left }'
              }
            }
          }
        }
      }
    }
  }
};
