
import { html, nothing, render } from '../../../hip/library/lit-html/lit-html.js';
import { dedupingMixin } from '../../../hip/library/polymer/polymer.js';
import { HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import { BehaviorBehavior } from '../../../eip/element/hi-behavior-behavior/hi-behavior-behavior.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import adt from '../../../pip/library/pi-adt/pi-adt.js';
import $ip from '../../../pip/library/pi-ip/pi-ip.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export function BaseBehaviorFactory(Base) {
  class BehaviorFactory2 extends BehaviorBehavior(Base) {
    constructor() {
      super(...arguments);
      this.fieldMutations = adt.mutation("fields");
      this.model = {};
      this.namePrefix = "";
      this.clearValues = false;
      this.childOf = "";
      this._willRender = true;
      this.__hasRenderCompleted = false;
      this.valueSource = "both";
      this.__internalType = false;
    }
    static get properties() {
      return {
        valueSource: {
          type: String,
          attribute: "value-source"
        },
        for: {
          type: String
        },
        namePrefix: {
          type: String,
          attribute: "name-prefix"
        },
        value: {
          type: Object
        },
        type: {
          type: Object
        },
        clearValues: {
          type: Boolean,
          attribute: "clear-values"
        }
      };
    }
    static get autoBinding() {
      return {
        signal: "type-changed",
        field: "type"
      };
    }
    connectedCallback() {
      super.connectedCallback();
      this._registerBehavior("BehaviorFactory");
      dom.setAttrs(this, {
        exclude: true,
        hidden: true
      });
    }
    createRenderRoot() {
      return this;
    }
    hasRender(group, render_type) {
      return $ip.di.has(group, render_type);
    }
    get hasRenderCompleted() {
      return this.__hasRenderCompleted;
    }
    renderBy(group, render_type) {
      return $ip.di.inject(group, render_type);
    }
    updated(props) {
      const fields = this._renderFields();
      const fieldsHash = this.childOf + f1.join("!", f1.map(f1.read("/name", ""), fields));
      const hasChangedType = props.has("type") && !f1.is.empty(props.get("type"));
      const hasNewTypeShape = hasChangedType && this.__fieldsHash !== fieldsHash;
      const isParentFactory = f1.is.empty(this.childOf);
      const isTypeNew = !this.__internalType;
      if (isParentFactory && props.has("value")) {
        this.model = f1.clone(this.value);
      } else if (isParentFactory && isTypeNew && props.has("type")) {
        this.__eraseModel();
      } else if (hasNewTypeShape && fields && this.clearValues) {
        this.__clearFieldValues(fields);
        this.__captureType(fieldsHash);
      }
      if (!this._willRender) {
        return;
      }
      const previousEl = this.previousElementSibling || document.body;
      const targetElement = previousEl.getAttribute("gid") === this.for ? previousEl : this.for ? HIP.browser.queryOne(this.for, this.ns) : this.parentElement;
      if (!targetElement) {
        return;
      }
      const changeRequested = props.size > 0;
      if (changeRequested) {
        this.__hasRenderCompleted = false;
        render(this._render(fields, this.type), targetElement);
        this.__hasRenderCompleted = true;
        this._dispatchProcess("render");
      }
      if (hasChangedType) {
        this.__internalType = false;
      }
    }
    __clearFieldValues(fields) {
      if (f1.is.empty(fields)) {
        return;
      }
      this.model = f1.safe({}, this.model);
      f1.each((field) => {
        const valueModelPath = field.model_path;
        const modelPath = this.namePrefix + valueModelPath;
        this.model = f1.write(modelPath, null, this.model);
        f1.write("/value/out", null, field);
      }, fields);
    }
    __captureType(fieldsHash) {
      this.__fieldsHash = fieldsHash;
    }
    __eraseModel() {
      if (this.valueSource === "model") {
        return;
      }
      f1.each((v, k, dict) => {
        delete dict[k];
      }, this.model);
    }
    retargetFieldEvent(event, field) {
      event.stopImmediatePropagation();
      const type = f1.safe({}, this.type);
      const fieldName = field.name;
      const valueShallow = f1.read("/detail/value", void 0, event);
      const valueDeep = f1.read("/detail/value/value", void 0, event);
      const value = f1.is.in("value", f1.safe({}, valueShallow)) ? valueDeep : valueShallow;
      const isOriginatingEvent = !f1.is.in("originElement", event.detail);
      if (isOriginatingEvent) {
        f1.write(f1.path(this.namePrefix + field.model_path), value, this.model);
        f1.write("/value/out", value, field);
        if (f1.is.in("fields", type)) {
          type.fields[fieldName] = field;
        }
      }
      const detail = {
        originElement: event.detail.srcElement,
        path: event.composedPath(),
        value,
        field,
        type
      };
      const modelDetail = {
        originElement: event.detail.srcElement,
        path: event.composedPath(),
        value: this.model,
        field,
        type
      };
      this.__internalType = true;
      this._dispatchEvent(`field-${field.name}-trigger-ended`, detail);
      this._dispatchEvent("field-trigger-ended", detail);
      this._dispatchEvent("model-update-ended", modelDetail);
      return detail;
    }
    retargetFieldStateEvent(event, field, stateName) {
      event.stopImmediatePropagation();
      const type = this.type;
      const valueShallow = f1.read("/detail/value", void 0, event);
      const valueDeep = f1.read("/detail/value/value", void 0, event);
      const value = f1.is.in("value", f1.safe({}, valueShallow)) ? valueDeep : valueShallow;
      const state = {
        name: stateName,
        value
      };
      const isOriginatingEvent = !f1.is.in("originElement", event.detail);
      if (isOriginatingEvent) {
        f1.write(`/state/${stateName}/value/out`, value, field);
        if (f1.is.in("fields", type)) {
          type.fields[field.name] = field;
        }
      }
      const detail = {
        originElement: event.detail.srcElement,
        path: event.composedPath(),
        state,
        field,
        type
      };
      this.__internalType = true;
      this._dispatchEvent(`field-state-${field.name}-${stateName}-trigger-ended`, detail);
      this._dispatchEvent("field-state-trigger-ended", detail);
      return detail;
    }
    proxyRetargetFieldEvent(event) {
      event.stopImmediatePropagation();
      const field = event.detail.field;
      const modelDetail = {
        originElement: event.detail.srcElement,
        path: event.composedPath(),
        value: this.model,
        field
      };
      this._dispatchEvent(`field-${field.name}-trigger-ended`, event.detail);
      this._dispatchEvent("field-trigger-ended", event.detail);
      this._dispatchEvent("model-update-ended", modelDetail);
      return event.detail;
    }
    proxyRetargetFieldStateEvent(event) {
      event.stopImmediatePropagation();
      const field = event.detail.field;
      this._dispatchEvent(`field-state-${field.name}-trigger-ended`, event.detail);
      this._dispatchEvent("field-state-trigger-ended", event.detail);
      return event.detail;
    }
    _renderFields() {
      return this.fieldMutations.options({
        namePrefix: this.namePrefix
      }).invoke(f1.read("/type", {}, this));
    }
    _render(fields, type) {
      return nothing;
    }
    _renderValue(renderFn, field) {
      const entity = this.value;
      const value = this._fieldValue(field, entity);
      return this.renderValue(renderFn, field, entity, value);
    }
    renderValue(renderFn, field, entity, value) {
      return renderFn(f1.read("/substitution/value", value, field), entity);
    }
    _fieldValue(field, value) {
      const valueModelPath = field.model_path;
      const modelPath = this.namePrefix + valueModelPath;
      const readModel = f1.read(modelPath, void 0, f1.safe({}, this.model));
      if (this.valueSource !== "field" && void 0 !== readModel) {
        return readModel;
      }
      const readFieldValue = f1.read("/value/out", void 0, field);
      if (this.valueSource !== "model" && void 0 !== readFieldValue) {
        this.model = f1.write(modelPath, readFieldValue, f1.safe({}, this.model));
        return readFieldValue;
      }
      return null;
    }
    renderDescription(field) {
      if (!field.description) {
        return nothing;
      }
      return html`
        <p gid="form_input__desc__${field.id}" slot="description" class="clr-gray_80-txt text-italic" .ns=${this.ns}>
          ${field.description}
        </p>
      `;
    }
    _renderHint(field) {
      if (!field.hint) {
        return nothing;
      }
      const iconId = `form_input__hint__${field.id}`;
      const ns = this.ns;
      return html`
        <hi-icon
          glyph="info_circle"
          mode="content"
          size="medium"
          class="mrsx"
          slot="controls"
          gid=${iconId}
          id=${iconId}
          .ns=${ns}
        ></hi-icon>
        <hi-tooltip slot="controls" placement="left" for=${iconId} .ns=${ns}>${field.hint}</hi-tooltip>
      `;
    }
    _renderCopyable(field) {
      if (!field.copyable) {
        return nothing;
      }
      return html`
        <hi-button
          size="small"
          aesthetic="icon"
          slot="controls"
          .ns=${this.ns}
          @trigger-ended=${this.__copyableButton__onTriggerEnded}
        >
          <hi-icon glyph="copy" size="small"></hi-icon>
          <hi-hint placement="right">Copy ${field.title}</hi-hint>
        </hi-button>
      `;
    }
    __copyableButton__onTriggerEnded(e) {
      const inputElement = e.detail?.srcElement?.closest(
        '[x-marker="field-label"]'
      )?.nextElementSibling;
      if (!inputElement) {
        return;
      }
      const copyText = f1.is.in("valueKey", inputElement) ? inputElement["value"][inputElement.valueKey] : f1.is.in("value", inputElement) ? inputElement.value : inputElement.localName === "dd" ? inputElement.children[0]["value"] || inputElement.children[0].textContent : inputElement.textContent;
      navigator.clipboard.writeText(`${copyText || ""}`.trim());
    }
  }
  return HIP.factory.mixin(BehaviorFactory2);
}
export const BehaviorFactory = dedupingMixin(BaseBehaviorFactory);
