
import { afterNextRender } from '../../../hip/library/polymer/polymer.js';
import { dedupingMixin } from '../../../hip/library/polymer/polymer.js';
import '../../../hip/library/polymer/polymer.js';
import { BehaviorBehavior } from '../../../eip/element/hi-behavior-behavior/hi-behavior-behavior.js';
import HIL from '../../../aip/library/ai-transpiler/ai-transpiler.js';
import {
  HIP
} from '../../../eip/element/hi-hip/hi-hif.js';
import c11n from '../../../pip/library/pi-c11n/pi-c11n.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';
export function BaseBehaviorPrefab(Base) {
  class BehaviorPrefab2 extends BehaviorBehavior(Base) {
    static get properties() {
      return {
        level: {
          type: Number
        },
        context: {
          type: Object
        },
        requestNoHeader: {
          type: Boolean,
          value: false
        },
        requestNoFooter: {
          type: Boolean,
          value: false
        }
      };
    }
    static get observers() {
      return ["__contextWatcher(context)"];
    }
    connectedCallback() {
      super.connectedCallback();
      this.style.display = "block";
      this.setAttribute("role", "region");
      this._registerBehavior("BehaviorPrefab");
      this.__attachLocationSync();
    }
    initialize() {
      if (super.initialize) {
        super.initialize();
      }
    }
    uninitialize() {
      this.__disconnectedViewCallback();
    }
    updated(changedProperties) {
      super.updated(changedProperties);
      if (changedProperties.has("context")) {
        this.__writeContextToHeader(changedProperties.get("context"));
      }
    }
    createRenderRoot() {
      afterNextRender(this, function() {
        this.__connectedViewCallback();
      });
      return this;
    }
    __contextWatcher(context) {
      this.__writeContextToHeader(context);
    }
    _attachDom(dom2) {
      HIP.util.applyNS(dom2.children, this.getAttribute("ns") || "");
      this.appendChild(dom2);
      afterNextRender(this, function() {
        this.__connectedViewCallback();
      });
    }
    __writeContextToHeader(context) {
      const header = HIP.db.gid.queryOne("_view_header", this.ns);
      if (header) {
        const _context = HIP.db.gid.queryOne("context", this.ns).output;
        header.context = ifn.mergeWith(
          "clobber",
          {
            description: "",
            title: ""
          },
          _context || {}
        );
      }
    }
    tetherPresentation(script) {
      script = this.__reflectBucketToPrefab(script);
      script = this.__hoistContextToPrefab("title", script);
      script = this.__hoistContextToPrefab("description", script);
      return script;
    }
    __reflectBucketToPrefab(script) {
      f1.each((bucket, _name) => {
        const name = bucket.name || _name;
        bucket.$bindings = [...bucket.$bindings || [], `{ ${name} } =>> { prefab.${name} }`];
      }, script.bucket || {});
      return script;
    }
    __hoistContextToPrefab(key, script) {
      const value = ifn.get(`system.${key}`, script);
      if (!value) {
        return script;
      }
      const hasKey = f1.reduce(
        (_hasKey, keyGroup) => {
          return _hasKey && ifn.get(`bucket.context.${keyGroup}.${key}`, script) != null;
        },
        true,
        ["$initial_value", "$reactive_values", "$computed_values", "$overwrite_values", "initial_value"]
      );
      if (!hasKey) {
        ifn.set(`bucket.context.$computed_values.${key}`, script, value);
      }
      return script;
    }
    includeBehaviors(script) {
      return script;
    }
    _createSubView(subViewRequest) {
      return HIP.fn.createSubView(this.parentElement, subViewRequest);
    }
    __connectedViewCallback() {
      this._isReady = true;
      this._connectedViewCallback();
    }
    _connectedViewCallback() {
    }
    __disconnectedViewCallback() {
      this.__detachLocationSync();
      this._disconnectedViewCallback();
    }
    _disconnectedViewCallback() {
    }
    __attachLocationSync() {
      const ns = this.ns;
      const location = HIP.db.gid.queryOne("location", ns);
      if (location) {
        this.__location__onOutputChangedBound = this.__location__onOutputChanged.bind(this);
        if (this.__location__onOutputChangedBound) {
          location.addEventListener("output-changed", this.__location__onOutputChangedBound);
        }
      }
    }
    __detachLocationSync() {
      const ns = this.ns;
      const location = HIP.db.gid.queryOne("location", ns);
      if (location) {
        if (this.__location__onOutputChangedBound) {
          location.removeEventListener("output-changed", this.__location__onOutputChangedBound);
        }
        this.__location__onOutputChangedBound = void 0;
      }
    }
    __location__onOutputChanged(e) {
      this._onLocationChanged(e.detail.value);
    }
    _onLocationChanged(_) {
    }
    onRouteChanged() {
      if (this._onRouteChanged) {
        this._onRouteChanged();
      }
    }
  }
  return HIP.factory.mixin(BehaviorPrefab2);
}
export const BehaviorPrefab = dedupingMixin(BaseBehaviorPrefab);
HIP["BehaviorPrefab"] = BehaviorPrefab;
HIP.fn.createSubView = function createSubView(viewEl, subViewRequest) {
  const { subViewRegionEl, regionTarget } = subViewRequest;
  const uninitialized = Promise.all(
    f1.map((el) => {
      f1.pipe(
        [f1.filter((el2) => el2.localName === "ai-view"), f1.each((el2) => el2.remove())],
        f1.to.array(subViewRegionEl.children) || []
      );
      return el.uninitialize();
    }, Array.from(subViewRegionEl.children) || [])
  );
  const regionGroup = ifn.get("output", HIP.db.gid.queryOne("region_group", viewEl.ns));
  if (!regionGroup) {
    return uninitialized.then(() => {
      Promise.resolve();
    });
  }
  const regionDeclaration = regionGroup[regionTarget];
  if (!regionDeclaration) {
    return uninitialized.then(() => {
      Promise.resolve();
    });
  }
  const regionRequest = regionDeclaration.view;
  if (!regionRequest) {
    return uninitialized.then(() => {
      Promise.resolve();
    });
  }
  const customizationPaths = c11n.readRefs(regionDeclaration);
  const viewDisallowed = c11n.isRuleOff("allowed", customizationPaths);
  if (viewDisallowed) {
    return uninitialized.then(() => {
      Promise.resolve();
    });
  }
  const scriptLoader = uninitialized.then(() => {
    return !regionRequest.uri ? Promise.resolve(regionRequest.script || {}) : HIL.services.network.loadHIL(regionRequest.uri).then((hilScript) => ifn.merge(hilScript || {}, regionRequest.script || {}));
  });
  return scriptLoader.then((script) => {
    const subViewEl = document.createElement("ai-view");
    const environment = ifn.get("setting", HIP.db.gid.queryOne("environment", viewEl.ns));
    const parameters = ifn.get("parameters", HIP.db.gid.queryOne("location", viewEl.ns));
    const source = ifn.get("output", HIP.db.gid.queryOne("source", viewEl.ns));
    const viewSystem = ifn.get("system", source);
    const system = {
      environment
    };
    dom.setAttrs(subViewEl, {
      c11n: f1.join(",", customizationPaths),
      ns: `${viewEl.ns}-${regionTarget}`,
      level: viewEl.level + 1,
      gid: "_sub_view"
    });
    f1.each(
      (systemKey) => {
        if (viewSystem[systemKey] != null) {
          system[systemKey] = viewSystem[systemKey];
        }
      },
      ["release", "mode"]
    );
    subViewEl["pointCutScript"] = viewEl.pointCutScript;
    subViewEl["autoRunScript"] = ifn.merge(
      {
        system,
        environment: ifn.merge({}, ifn.get("environment", source)),
        constants: ifn.merge({}, ifn.get("constants", source)),
        configuration: ifn.merge({}, ifn.get("configuration", source)),
        parameters: ifn.merge({}, ifn.get("parameters", source), parameters)
      },
      script
    );
    const bindingDSL = HIL.dsl.assignment();
    f1.each((cloneRule) => {
      const { $from_ref, $to_ref } = cloneRule || {};
      if ($from_ref == null || $to_ref == null) {
        return;
      }
      const [fromInstanceName, fromField, fromPath] = $from_ref.split(bindingDSL.symbolTuple);
      const fromInstance = HIP.db.gid.queryOne(fromInstanceName, viewEl.ns) || {};
      const instanceValue = ifn.get(`${fromField || "output"}.${fromPath}`, fromInstance);
      const [toInstanceName, toField, toPath] = $to_ref.split(bindingDSL.symbolTuple);
      if (instanceValue != null) {
        ifn.set(`bucket.${toInstanceName}.initial_value.${toPath}`, subViewEl["autoRunScript"], instanceValue);
      }
    }, ifn.get("interop.cloned_memory", regionDeclaration) || []);
    subViewRegionEl.appendChild(subViewEl);
    subViewEl.style = "width:100%";
    const screenEl = HIP.db.gid.queryOne("_screen", viewEl.ns);
    if (screenEl) {
      screenEl.state("main");
    }
    return subViewRegionEl;
  });
};
