
import { HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import { html, nothing } from '../../../hip/library/lit-html/lit-html.js';
import navigation from '../../../pip/library/pi-navigation/pi-navigation.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import { BehaviorBehavior } from '../../../eip/element/hi-behavior-behavior/hi-behavior-behavior.js';
export function AsLitTemplateHILC(p) {
  return p;
}
export function BaseBehaviorTemplate(Base) {
  return class BehaviorTemplate extends BehaviorBehavior(Base) {
    constructor() {
      super(...arguments);
      this.requestNoHeader = false;
      this.requestNoFooter = false;
      this.__isOn = false;
      this.__runtimeSubscriber = {
        id: Symbol(),
        load: () => {
          this.requestUpdate();
        },
        invoke: () => {
          this.requestUpdate();
        }
      };
    }
    static get properties() {
      return {
        level: {
          type: Number
        }
      };
    }
    isOn() {
      return this.__isOn;
    }
    get defaultRuntime() {
      return this._defaultRuntimeShared(
        f1.merge(
          "over",
          {
            startup: [],
            tasks: {}
          },
          this._defaultRuntime()
        )
      );
    }
    _defaultRuntime() {
      return {};
    }
    _defaultRuntimeShared(runtimeHIL) {
      const viewTaskPath = "/tasks/view";
      f1.write(
        viewTaskPath,
        f1.merge(
          "over",
          {
            $is: "task",
            proxy: {
              out: {
                $is: "reference",
                id: "view_head",
                field: "/context"
              }
            }
          },
          f1.read(viewTaskPath, {}, runtimeHIL)
        ),
        runtimeHIL
      );
      runtimeHIL = f1.reduce(
        (_runtimeHIL, id) => f1.merge(
          "onto",
          {
            tasks: {
              [id]: {
                $is: "task"
              }
            }
          },
          _runtimeHIL
        ),
        runtimeHIL,
        this._injectInstances()
      );
      runtimeHIL = f1.merge(
        "onto",
        {
          startup: [
            {
              $is: "reference",
              id: "view"
            }
          ]
        },
        runtimeHIL
      );
      return runtimeHIL;
    }
    connectedCallback() {
      super.connectedCallback();
      this.style.display = "block";
      this.setAttribute("role", "region");
      this._registerBehavior("BehaviorTemplate");
    }
    createRenderRoot() {
      return this;
    }
    __injectInstances(runtime) {
      const instanceRecord = f1.reduce(
        (_, id) => {
          _[id] = id;
          return _;
        },
        {},
        this._injectInstances()
      );
      return f1.compact(f1.map((id) => runtime.workspace.queryInstance(id), instanceRecord));
    }
    _injectInstances() {
      return [];
    }
    _injectInstance(id) {
      const instanceI = this.runtime.workspace.queryInstance(id);
      return !instanceI ? { type: {}, value: [], settings: {} } : instanceI.state();
    }
    connectedViewCallback() {
      this.__isOn = true;
      this.runtime.subscribe(this.__runtimeSubscriber);
      this._instances = this.__injectInstances(this.runtime);
      this._connectedViewCallback();
    }
    _connectedViewCallback() {
    }
    disconnectedViewCallback() {
      this._disconnectedViewCallback();
      this.runtime.unsubscribe(this.__runtimeSubscriber.id);
      this.__isOn = false;
    }
    _disconnectedViewCallback() {
    }
    render() {
      if (!this.runtime || !this._instances) {
        return this._renderStateError({
          title: "This view is not ready to pull data yet"
        });
      }
      return this._render();
    }
    _render() {
      return nothing;
    }
    _renderStateEmpty(message = {
      title: "No Content",
      details: "Once you provide model for the template, you can see the template with the content"
    }) {
      return html`
        <hi-alert mode="presentation">
          ${message.title}
          <span slot="details">${message.details}</span>
        </hi-alert>
      `;
    }
    _renderStateError(message = {
      title: "Oh no!",
      details: "Something went wrong. Please make sure that default configurations are provided"
    }) {
      return html`
        <hi-alert mode="presentation" theme="error">
          ${message.title}
          <span slot="details">${message.details}</span>
        </hi-alert>
      `;
    }
    injectErrorsIntoView(formId, instanceValue) {
      const errors = f1.read("/metadata/errors", void 0, instanceValue);
      const message = f1.read("/metadata/message", void 0, instanceValue);
      if (errors) {
        const authorizationFormEl = HIP.db.gid.queryOne(formId, this.ns);
        if (authorizationFormEl) {
          authorizationFormEl.validateCustomValidity(errors);
        }
      } else if (message) {
        HIP.fn.notify({
          status: "error",
          title: message
        });
      }
    }
    async stateChange() {
      const runtime = this.runtime;
      const locationI = runtime.workspace.queryInstance("location");
      if (locationI) {
        await locationI.invoke(navigation.state());
      }
      if (runtime.dsl) {
        await runtime.invokeLifeCycle(runtime.dsl, "state_change");
      }
    }
  };
}
export const BehaviorTemplate = BaseBehaviorTemplate;
