
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import { BehaviorBuilder } from '../../../aip/element/ai-behavior-builder/ai-behavior-builder.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import './ai-visitor-container-accordion.js';
import './ai-visitor-container-fallback.js';
export class BuilderContainerElement extends BehaviorBuilder(BehaviorElement(AsHIC(PolymerElement))) {
  static get is() {
    return "ai-builder-container";
  }
  static get properties() {
    return {
      defaultTagRenderType: {
        type: String,
        value: "accordion"
      }
    };
  }
  insertContainer(containerEl, builderEl) {
    if (containerEl) {
      f1.each((childEl) => {
        builderEl._insertElement(childEl);
      }, f1.to.array(containerEl.children));
    }
  }
  buildContainers(_input, inputBuilder) {
    const fragment = document.createDocumentFragment();
    const fields = f1.read("/properties", {}, _input);
    if (f1.is.empty(fields)) {
      return fragment;
    }
    let activeTags = this.__activeTags;
    const relevantTags = this.__relevantTags(activeTags, fields);
    activeTags = this.__applyDefaults__activeTags(activeTags, relevantTags);
    activeTags = this.__includeGroupRenderTypes__activeTags("tab", activeTags, relevantTags);
    activeTags = this.__includeGroupRenderTypes__activeTags("accordion", activeTags, relevantTags);
    const fieldGroups = this.__createFieldGroups(relevantTags, fields);
    let parentFieldGroups = this.__createParentFieldGroups(fieldGroups, activeTags);
    parentFieldGroups = f1.is.empty(parentFieldGroups) ? (
      this.__buildOneParentFieldGroup(fieldGroups)
    ) : parentFieldGroups;
    if (fieldGroups["__ungrouped__"]) {
      this.__iterateFieldGroup(
        fieldGroups["__ungrouped__"],
        fragment,
        (_fieldType, fieldParentEl, containerBuilder, tagType) => {
          this.__renderField(_fieldType, fieldParentEl, containerBuilder, inputBuilder, tagType);
        }
      );
    }
    this.__iterateFieldGroups(parentFieldGroups, (_parentFieldGroup) => {
      this.__iterateFieldGroup(_parentFieldGroup, fragment, (_fieldGroup, fieldGroupParentEl) => {
        this.__iterateFieldGroup(
          _fieldGroup,
          fieldGroupParentEl,
          (_fieldType, fieldParentEl, containerBuilder, tagType) => {
            this.__renderField(_fieldType, fieldParentEl, containerBuilder, inputBuilder, tagType);
          }
        );
      });
    });
    return fragment;
  }
  __iterateFieldGroups(fieldGroups, fn) {
    for (const tagName in fieldGroups) {
      const fieldGroup = fieldGroups[tagName];
      if (!fieldGroup) {
        continue;
      } else if (tagName === "__ungrouped__") {
        continue;
      }
      fn(fieldGroup, tagName, fieldGroups);
    }
  }
  __renderField(_fieldType, fieldParentEl, containerBuilder, inputBuilder, tagType) {
    let fieldEl = inputBuilder(_fieldType);
    if (!fieldEl) {
      return;
    }
    if (fieldEl.nodeType === Node.DOCUMENT_FRAGMENT_NODE) {
      fieldEl = fieldEl.children[0];
    }
    containerBuilder.buildField(fieldEl, fieldParentEl, _fieldType, tagType);
  }
  __iterateFieldGroup(parentFieldGroup, parentEl, fieldGroupRenderer) {
    const tagType = parentFieldGroup.tag;
    const containerBuilder = HIP.visitor.queryField(tagType, "container");
    if (!containerBuilder) {
      return parentEl;
    }
    const containerEl = containerBuilder.buildTag(parentEl, tagType);
    const orderedFields = this.__orderFields(parentFieldGroup.fields);
    for (const index in orderedFields) {
      const fieldGroup = orderedFields[index];
      if (!fieldGroup) {
        continue;
      }
      fieldGroupRenderer(fieldGroup, containerEl, containerBuilder, tagType);
    }
    return containerEl;
  }
  __deriveAppropriateTag(fieldTag, activeTags) {
    const filteredTags = fieldTag.split(",").map((tagName) => tagName.trim()).filter((tag) => activeTags[tag]).filter((tag) => activeTags[tag].render_type);
    return f1.nth(0, filteredTags);
  }
  __relevantTags(activeTags, fields) {
    const relevantTags = {};
    for (const id in fields) {
      const fieldType = fields[id];
      const fieldTag = fieldType[`$tag`] || fieldType[`tag`];
      if (fieldTag) {
        const hasMultipleTags = fieldTag.includes(",");
        if (hasMultipleTags) {
          const filteredTag = this.__deriveAppropriateTag(fieldTag, activeTags);
          relevantTags[filteredTag] = activeTags[filteredTag];
        } else if (activeTags[fieldTag].render_type) {
          relevantTags[fieldTag] = activeTags[fieldTag];
        }
      }
    }
    return relevantTags;
  }
  __createParentFieldGroups(fieldGroups, activeTags) {
    const parentFieldGroups = {};
    this.__iterateFieldGroups(fieldGroups, (fieldGroup) => {
      const subordinateTag = fieldGroup.tag;
      const subordinateTagName = subordinateTag.$tag;
      if (subordinateTagName) {
        const activeTag = activeTags[subordinateTagName];
        parentFieldGroups[subordinateTagName] = parentFieldGroups[subordinateTagName] || {
          render_type: activeTag.render_type || "fallback",
          tag: activeTag,
          fields: []
        };
        parentFieldGroups[subordinateTagName].fields.push(fieldGroup);
      }
    });
    return parentFieldGroups;
  }
  __createFieldGroups(activeTags, fields) {
    const fieldGroups = {
      __ungrouped__: {
        render_type: "fallback",
        fields: [],
        tag: {
        }
      }
    };
    for (const id in fields) {
      const fieldType = fields[id];
      const fieldTag = fieldType[`$tag`] || fieldType[`tag`] || "__ungrouped__";
      const hasMultipleTags = fieldTag.includes(",");
      let filteredTag = "";
      if (hasMultipleTags) {
        filteredTag = this.__deriveAppropriateTag(fieldTag, activeTags);
      } else {
        filteredTag = fieldTag;
      }
      const fieldTagGroup = fieldGroups[filteredTag] = fieldGroups[filteredTag] || {
        fields: [],
        tag: {}
      };
      fieldTagGroup.tag = activeTags[filteredTag] || {};
      fieldTagGroup.fields = [...fieldTagGroup.fields, fieldType];
    }
    return fieldGroups;
  }
  __buildOneParentFieldGroup(fieldGroups) {
    const parentFieldGroup = {
      render_type: "fallback",
      fields: [],
      tag: {
        render_type: "fallback"
      }
    };
    this.__iterateFieldGroups(fieldGroups, (fieldGroup) => {
      parentFieldGroup.fields.push(fieldGroup);
    });
    return {
      "top-level": parentFieldGroup
    };
  }
  __applyDefaults__activeTags(activeTags, relevantTags) {
    for (const tagName in relevantTags) {
      const tag = activeTags[tagName];
      if (tag) {
        tag.render_type = tag.render_type || this.defaultTagRenderType;
      }
    }
    return activeTags;
  }
  __includeGroupRenderTypes__activeTags(group, activeTags, relevantTags) {
    const hasGroup = f1.reduce(
      (_hasGroup, fieldGroup) => {
        return _hasGroup || fieldGroup["render_type"] === group;
      },
      false,
      activeTags
    );
    if (!hasGroup) {
      return activeTags;
    }
    const groupRenderType = `${group}-group`;
    const tagGroup = f1.values(activeTags).find((tag) => tag.render_type === groupRenderType);
    if (tagGroup) {
      return activeTags;
    }
    const newGroup = activeTags[`${groupRenderType}-auto`] = activeTags[`${groupRenderType}-auto`] || {
      name: `${groupRenderType}-auto`,
      render_type: groupRenderType,
      title: "",
      tagged: []
    };
    for (const tagName in relevantTags) {
      const relevantTag = relevantTags[tagName];
      if (relevantTag.render_type === group) {
        relevantTag.$tag = newGroup.name;
        newGroup.tagged.push(relevantTag);
      }
    }
    return activeTags;
  }
  get __activeTags() {
    const tagGroupEl = HIP.db.gid.queryOne("tag_group", this.ns);
    const activeTags = f1.read("/input", {}, f1.safe({}, tagGroupEl));
    return activeTags;
  }
  __orderFields(fields) {
    return f1.to.array(fields).sort((a, b) => (a.order | 0) - (b.order | 0));
  }
}
HIP.factory.element(BuilderContainerElement);
