
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import '../../../aip/element/ai-builder-container/ai-builder-container.js';
import { BehaviorBuilder } from '../../../aip/element/ai-behavior-builder/ai-behavior-builder.js';
import './ai-visitor-form-multiline.js';
import './ai-visitor-form-fallback.js';
import './ai-visitor-form-control.js';
import './ai-visitor-form-dnd.js';
import './ai-visitor-form-select.js';
import './ai-visitor-form-range.js';
import './ai-visitor-form-child.js';
import './ai-visitor-form-grid.js';
import './ai-visitor-form-control-dropdown.js';
import {
  __createInputDescription,
  __createInputGroup,
  __createInputTitle,
  __createInputClear,
  __createInputHint,
  __getElementPath
} from './ai-support-input.js';
export class BuilderFormElement extends BehaviorBuilder(BehaviorElement(AsHIC(PolymerElement))) {
  static get is() {
    return "ai-builder-form";
  }
  static get properties() {
    return {
      form: {
        type: String
      },
      mode: {
        type: String
      },
      overridePath: {
        type: String
      }
    };
  }
  __setFormId() {
    if (!this.form) {
      this.form = this.parentElement ? this.parentElement["form"] : "";
    }
  }
  get __builderContainer() {
    this.__containerBuilderEl = this.__containerBuilderEl || document.createElement("ai-builder-container");
    this.__containerBuilderEl.ns = this.ns;
    return this.__containerBuilderEl;
  }
  _inputWatcher(_input) {
    this.__setFormId();
    const containerFragment = this.__builderContainer.buildContainers(_input, (fieldType) => {
      return this.__createFormElement(fieldType);
    });
    this.__builderContainer.insertContainer(containerFragment, this);
  }
  __createFormElement(fieldType) {
    if (fieldType.hidden) {
      return;
    }
    const inputVisitor = HIP.visitor.queryField(fieldType, "form");
    if (!inputVisitor) {
      return;
    }
    const formEl = this.mode === "display" && f1.read("/attributes/is_form_container", false, inputVisitor) !== true ? this.__createDisplayModeElement(fieldType, inputVisitor) : this.__createEditModeElement(fieldType, inputVisitor);
    if (formEl == null) {
      return;
    }
    if (f1.read("/attributes/is_form_container", false, inputVisitor) === true) {
      return formEl;
    }
    const inputGroupEl = __createInputGroup(fieldType, this);
    const hintTemplate = __createInputHint(fieldType);
    const clearTemplate = __createInputClear(fieldType, formEl);
    const labelEl = __createInputTitle(fieldType, hintTemplate, clearTemplate, this);
    const descriptionEl = __createInputDescription(fieldType, formEl);
    if (labelEl) {
      inputGroupEl.appendChild(labelEl);
    }
    if (formEl) {
      inputGroupEl.appendChild(formEl);
    }
    if (descriptionEl) {
      inputGroupEl.appendChild(descriptionEl);
    }
    this.__interlinkSourceValue(fieldType, inputGroupEl, formEl);
    this.__interlinkSinkValue(fieldType, inputGroupEl, formEl);
    return inputGroupEl;
  }
  __interlinkSourceValue(fieldType, inputGroupEl, formEl) {
    const source = f1.read("/$value/$configure/relationship/source", null, fieldType);
    if (!source) {
      return;
    }
    const sourceEl = HIP.browser.queryOne(source, this.ns);
    const inputValueProp = formEl.valueProperty || "value";
    const sourceValueEvent = formEl.autoBinding.signal || "output-changed";
    const bindingEl = document.createElement("hi-binding");
    bindingEl.source = sourceEl;
    bindingEl.signal = sourceValueEvent;
    bindingEl.sink = formEl;
    bindingEl.field = inputValueProp;
    bindingEl.mapFn = ({ input }) => {
      return f1.is.empty(input) ? null : f1.is.in(fieldType.name, input) ? input[fieldType.name] : input;
    };
    inputGroupEl.appendChild(bindingEl);
  }
  __interlinkSinkValue(fieldType, inputGroupEl, formEl) {
    const sink = f1.read("/$value/$configure/relationship/sink", null, fieldType);
    if (!sink) {
      return;
    }
    const sinkEl = HIP.browser.queryOne(sink, this.ns);
    const inputValueEvent = formEl.valueEvent || "value-changed";
    const sinkBindingField = sinkEl.autoBinding.field || "invoke";
    const bindingEl = document.createElement("hi-binding");
    bindingEl.source = formEl;
    bindingEl.signal = inputValueEvent;
    bindingEl.sink = sinkEl;
    bindingEl.field = sinkBindingField;
    bindingEl.mapFn = ({ input }) => {
      return { [fieldType.name]: input };
    };
    inputGroupEl.appendChild(bindingEl);
  }
  __createDisplayModeElement(fieldType, inputVisitor) {
    const formEl = this.__createEditModeElement(fieldType, inputVisitor);
    formEl.readonly = true;
    return formEl;
  }
  __createEditModeElement(fieldType, inputVisitor) {
    const inputEl = inputVisitor.build(fieldType, this);
    if (!inputEl) {
      return;
    }
    const formAesthetic = this.parentElement.getAttribute("aesthetic");
    inputEl.setAttribute("aesthetic", formAesthetic || "none");
    const isFormInput = f1.is.function(inputEl.validate) || HIP.util.doesExtend(inputEl, "BehaviorForm");
    if (!isFormInput) {
      return inputEl;
    }
    if (fieldType.icon) {
      inputEl.setAttribute("icon", fieldType.icon);
    }
    f1.each((attr) => dom.setAttr(inputEl, attr, !!fieldType[attr]), ["required", "readonly", "locked"]);
    return this.__identifyFormElement(fieldType, inputEl);
  }
  __identifyFormElement(fieldType, formEl) {
    const gid = `${this.gid}__${fieldType.name}`;
    const name = __getElementPath(fieldType, this);
    const formNSAttributes = formEl.hasAttribute("exclude") ? {} : {
      ns: this.ns,
      form: this.form
    };
    dom.setAttrs(
      formEl,
      f1.merge(
        "over",
        {
          placeholder: fieldType.placeholder || "",
          nullable: fieldType.nullable || false,
          blankable: fieldType.blankable || false,
          id: gid,
          name,
          gid
        },
        {
          ...formNSAttributes,
          ...fieldType.attributes || {}
        }
      )
    );
    const initialValue = fieldType.$initial_value || fieldType.initial_value;
    if (initialValue != null) {
      if (formEl.hasAttribute("values") || formEl.values != null) {
        formEl.values = initialValue;
      } else {
        formEl.setAttribute("value", initialValue);
      }
    }
    if (fieldType.theme) {
      formEl.setAttribute("theme", fieldType.theme);
    }
    return formEl;
  }
}
HIP.factory.element(BuilderFormElement);
