
import {
  __attachOptionsList,
  __createInputHint,
  __createInputClear,
  __createInputTitle,
  __createInputDescription,
  __createInputGroup
} from './ai-support-input.js';
import { HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export function buildFormControls(fieldType, builderEl) {
  const selectEl = document.createElement("hi-select");
  dom.setAttrs(selectEl, {
    required: true,
    name: fieldType.name,
    form: builderEl.form,
    ns: builderEl.ns,
    mode: "value"
  });
  if (fieldType.initial_value) {
    selectEl.setAttribute("value", fieldType.initial_value);
  }
  const choicesBuilder = document.createElement("ai-builder-select");
  choicesBuilder.gid = `${fieldType.name}_select__builder`;
  choicesBuilder.id = `${fieldType.name}_select__builder`;
  choicesBuilder.ns = builderEl.ns;
  __attachOptionsList(choicesBuilder, fieldType, builderEl);
  selectEl.appendChild(choicesBuilder);
  const dropdownEl = document.createElement("hi-dropdown");
  dropdownEl.appendChild(selectEl);
  dom.setAttrs(dropdownEl, {
    placeholder: fieldType.placeholder || "Select an option",
    required: fieldType.required,
    "disallow-responsiveness": fieldType.disallow_responsiveness,
    name: fieldType.name,
    "cap-height": true,
    ns: builderEl.ns,
    exclude: true,
    kind: "value"
  });
  const inputGroupEl = __createInputGroup(fieldType, builderEl);
  const hintTemplate = __createInputHint(fieldType);
  const clearTemplate = __createInputClear(fieldType, dropdownEl);
  const labelEl = __createInputTitle(fieldType, hintTemplate, clearTemplate, builderEl);
  const descriptionEl = __createInputDescription(fieldType, dropdownEl);
  if (labelEl) {
    inputGroupEl.appendChild(labelEl);
  }
  if (dropdownEl) {
    inputGroupEl.appendChild(dropdownEl);
  }
  if (descriptionEl) {
    inputGroupEl.appendChild(descriptionEl);
  }
  const formBuilder = document.createElement("ai-builder-form");
  dom.setAttrs(formBuilder, {
    name: fieldType.name,
    gid: fieldType.name,
    ns: builderEl.ns,
    aesthetic: "invisible",
    form: builderEl.form
  });
  selectEl.addEventListener("value-changed", () => {
    const typeGroupEl = HIP.db.gid.queryOne("type_group", builderEl.ns);
    const typeGroup = f1.read("/output", {}, typeGroupEl);
    formBuilder.input = typeGroup[selectEl.value];
  });
  const section = document.createElement("section");
  section.appendChild(inputGroupEl);
  section.appendChild(formBuilder);
  return { section, selectEl };
}
