
import { HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';
function __formElementInputChanged(formElement, fieldType, newInput = []) {
  const validInputSet = formElement && fieldType;
  if (!validInputSet) {
    return;
  }
  formElement.fieldType = fieldType;
  const isRepeater = formElement.nodeName === "HI-REPEAT";
  const inputHoistingKey = "properties";
  const doesNeedHoisting = !newInput.hasOwnProperty(inputHoistingKey);
  const nonHoistedInput = newInput;
  if (isRepeater || !doesNeedHoisting) {
    formElement.input = nonHoistedInput;
    return;
  }
  formElement.input = {
    [inputHoistingKey]: newInput
  };
}
function __attachOptionsList(exportBuilder, fieldType, inputEl) {
  const ns = inputEl.ns;
  const sourceGID = f1.read("/$configure/relationship/instance", void 0, fieldType);
  const optionsGID = fieldType.options;
  const sourceBucket = f1.is.string(sourceGID) ? HIP.db.gid.queryOne(sourceGID, ns) : void 0;
  const optionsBucket = f1.is.string(optionsGID) ? HIP.db.gid.queryOne(optionsGID, ns) : void 0;
  const fieldTypeOptions = ifn.is(Object, fieldType.options) || f1.is.array(fieldType.options) ? Object.values(fieldType.options || {}) : void 0;
  const isSourceLocal = f1.is.array(fieldTypeOptions) && fieldTypeOptions.length > 0;
  const isSourceInstance = sourceBucket != null || optionsBucket != null;
  const isSourceURI = typeof fieldType.options === "string";
  if (isSourceInstance) {
    const inputBucket = sourceBucket || optionsBucket;
    const returnSource = f1.is.string(exportBuilder);
    if (returnSource) {
      return inputBucket;
    }
    const isRepeater = exportBuilder.nodeName === "HI-REPEAT";
    exportBuilder.input = isRepeater ? inputBucket.output || [] : {
      properties: inputBucket.output || []
    };
    const existingValue = inputBucket.output;
    if (existingValue) {
      __formElementInputChanged(exportBuilder, fieldType, existingValue);
    }
    inputBucket.addEventListener("output-changed", (event) => {
      __formElementInputChanged(exportBuilder, fieldType, event.detail.value);
    });
    return inputBucket;
  } else if (isSourceURI) {
    const httpChoices = document.createElement("hi-http");
    const dnFnInit = document.createElement("di-fn-init");
    dom.setAttrs(httpChoices, {
      bind: exportBuilder.gid,
      uri: fieldType.options,
      ns: inputEl.ns,
      auto: true,
      gid: `${exportBuilder.gid}_http`,
      id: `${exportBuilder.gid}_http`
    });
    dom.setAttrs(dnFnInit, {
      "hoist-path": "properties",
      gid: `${exportBuilder.gid}_http_init`,
      id: `${exportBuilder.gid}_http_init`,
      ns: inputEl.ns
    });
    httpChoices.appendChild(dnFnInit);
    inputEl.appendChild(httpChoices);
    return httpChoices;
  } else if (isSourceLocal) {
    exportBuilder.input = {
      properties: fieldTypeOptions.sort((a, b) => a.order - b.order)
    };
    return fieldTypeOptions;
  }
}
function __applyAttributes(fieldType, el) {
  if (fieldType.format_type != null) {
    dom.setAttrs(el, {
      formatterRule: f1.read("/$configure/format_type", {}, fieldType),
      formatter: fieldType.format_type
    });
  }
  if (fieldType.parse_type != null) {
    dom.setAttrs(el, {
      parserRule: f1.read("/$configure/parse_type", {}, fieldType),
      parser: fieldType.parse_type
    });
  }
  if (fieldType.minimum_length != null) {
    el.setAttribute("minlength", fieldType.minimum_length);
  }
  if (fieldType.maximum_length != null) {
    el.setAttribute("maxlength", fieldType.maximum_length);
  }
  if (fieldType.minimum != null) {
    el.setAttribute("min", fieldType.minimum);
  }
  if (fieldType.maximum != null) {
    el.setAttribute("max", fieldType.maximum);
  }
}
function __createInputHint(fieldType) {
  return !fieldType.hint ? "" : `<hi-icon id="${fieldType.name}_label_hint"
          kind="info_circle" mode="content" type="material" slot="controls"></hi-icon>
        <hi-tooltip for="${fieldType.name}_label_hint"
          placement="top" slot="controls">${fieldType.hint}</hi-tooltip>`;
}
function __isBooleanField(fieldType) {
  return fieldType.render_type === "boolean" || fieldType.render_type === "bi-state" || fieldType.render_type === "tri-state";
}
function __createInputTitle(fieldType, hintTemplate, clearTemplate, builderEl) {
  if (fieldType.render_type === "hidden") {
    return;
  }
  const templateEl = document.createElement("template");
  const booleanRenderType = __isBooleanField(fieldType);
  templateEl.innerHTML = `
    <hi-label class="field-label ${booleanRenderType ? "field-label--boolean" : "height-1"}" ${fieldType.required ? 'required = "required"' : ""}>
    <span>${fieldType.title}</span>
    ${hintTemplate}
    ${clearTemplate}
    </hi-label>
  `;
  const labelEl = templateEl.content.firstElementChild;
  labelEl.ns = builderEl.ns;
  return labelEl;
}
function __createInputClear(fieldType, inputEl) {
  return fieldType.clearable !== true ? "" : `<hi-button gid="button_${inputEl.form}__clear__${inputEl.gid}" size="small" aesthetic="ghost" slot="controls" class="field-clear-button">
        <hi-icon kind="layers_clear" type="material"></hi-icon>
        <hi-hint placement="top">Clear ${fieldType.title}</hi-hint>
        <span class="show-ax">Clear ${fieldType.title}</span>
       </hi-button>
       <hi-binding id="binding_${inputEl.form}__clear__${inputEl.gid}"
          source="button_${inputEl.form}__clear__${inputEl.gid}"
          sink="${inputEl.gid}"
          field="clear">
       </hi-binding>`;
}
function __createInputDescription(fieldType, inputEl) {
  if (fieldType.render_type === "hidden") {
    return;
  } else if (!fieldType.description) {
    return;
  }
  const descriptionEl = document.createElement("p");
  const descriptionId = `${fieldType.name}_label_description`;
  descriptionEl.innerHTML = fieldType.description;
  HIP.util.includeListAttribute(inputEl, "aria-labelledby", descriptionId);
  dom.setAttrs(descriptionEl, {
    class: "clr-gray_80-txt mtsx",
    id: descriptionId
  });
  return descriptionEl;
}
function __createInputGroup(fieldType, builderEl) {
  const inputGroupEl = document.createElement("hi-input-group");
  if (fieldType.attributes.hasOwnProperty("c11n")) {
    inputGroupEl.c11n = fieldType.attributes.c11n;
  }
  inputGroupEl.form = builderEl.form;
  inputGroupEl.ns = builderEl.ns;
  return inputGroupEl;
}
function __getElementPath(fieldType, builderEl) {
  return builderEl.overridePath ? `${builderEl.overridePath}.${fieldType.name}` : fieldType.path || fieldType.name;
}
export {
  __attachOptionsList,
  __applyAttributes,
  __createInputHint,
  __createInputTitle,
  __createInputDescription,
  __isBooleanField,
  __createInputClear,
  __createInputGroup,
  __getElementPath,
  __formElementInputChanged
};
