
import '../../../eip/element/hi-input/hi-input.js';
import { HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import { html } from '../../../hip/library/lit-html/lit-html.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
const entity = "form";
HIP.visitor.add({
  entity,
  render_type: "form_child",
  attributes: {
    is_form_container: true
  },
  build: (inputType, builderEl) => {
    const builderFormEl = document.createElement("ai-builder-form");
    builderFormEl.input = inputType;
    dom.setAttrs(builderFormEl, {
      gid: `${builderEl.gid}__child_${inputType.name}`,
      form: builderEl.form,
      ns: builderEl.ns
    });
    const fieldsetEl = document.createElement("hi-fieldset");
    fieldsetEl.appendChild(builderFormEl);
    dom.setAttrs(fieldsetEl, {
      heading: inputType.title || inputType.name || "Sub Section",
      form: builderEl.form,
      ns: builderEl.ns
    });
    return fieldsetEl;
  }
});
HIP.visitor.add({
  entity,
  render_type: "form_child_list",
  attributes: {
    is_form_container: true
  },
  build: (inputType, builderEl) => {
    const fieldsetEl = document.createElement("hi-fieldset");
    dom.setAttrs(fieldsetEl, {
      heading: inputType.title || inputType.name || "Sub Section",
      form: builderEl.form,
      ns: builderEl.ns
    });
    function removeIteratorItems(e) {
      const iteratorIndex = parseInt(`${e.detail.srcElement.value}`);
      const formEl2 = e.detail.srcElement.closest("hi-form");
      const formInput = formEl2.input;
      const apiData = f1.read(f1.path(inputType.path), [], formInput);
      const removedList = apiData.filter((item, index) => iteratorIndex !== index);
      f1.write(f1.path(inputType.path), removedList, formInput);
      dom.setAttr(loopEl, "value-path", inputType.path);
      loopEl.input = removedList.filter((_) => !_.idx);
      if (formEl2) {
        formEl2.refresh();
      }
    }
    const loopEl = document.createElement("vi-loop");
    const formEl = document.querySelector("hi-form#_record_form");
    loopEl.id = `_form_${inputType.name}__iterator, _form__iterator`;
    loopEl.gid = `_form_${inputType.name}__iterator, _form__iterator`;
    loopEl.ns = builderEl.ns;
    loopEl.class = `mbl`;
    loopEl.context = {};
    loopEl.input = f1.read(f1.path(inputType.path), [], formEl.input);
    loopEl.setAttribute("value-path", inputType.path);
    loopEl.template = {
      render() {
        return (obj, index) => {
          const name = f1.read("/name", "unnamed", obj);
          const blockId = `block_${name}_${index}`;
          const formId = `_form_${name}_${index}`;
          const buttonId = `_button_${name}_${index}`;
          return html`<li>
            <ai-block l-kind="hort" l-align="end" l-gutter id="${blockId}" gid="${blockId}">
              <ai-builder-form
                id="${formId}"
                gid="${formId}"
                form="${builderEl.form}"
                ns="${builderEl.ns}"
                override-path="${inputType.path}[${index}]"
                .input="${inputType}"
              >
              </ai-builder-form>
              <div l-size="1" l-align="top">
                <hi-button
                  id="${buttonId}"
                  gid="${buttonId}"
                  aesthetic="icon"
                  theme="red"
                  value="${index}"
                  ns="${builderEl.ns}"
                  @trigger-ended=${removeIteratorItems}
                >
                  <hi-icon kind="cross_filled" theme="red"></hi-icon>
                  <hi-hint>Delete</hi-hint>
                </hi-button>
              </div>
            </ai-block>
          </li>`;
        };
      }
    };
    const iteratorRenderer = document.createElement("div");
    iteratorRenderer.id = `${inputType.name}__iterator_renderer, __iterator_renderer`;
    iteratorRenderer.gid = `${inputType.name}__iterator_renderer, __iterator_renderer`;
    const addButtonEl = document.createElement("hi-button");
    dom.setAttrs(addButtonEl, {
      gid: `_add_button_${inputType.name}`,
      id: `_add_button_${inputType.name}`,
      class: "mtsx mlsx",
      ns: builderEl.ns,
      size: "small"
    });
    addButtonEl.innerHTML = `
      <hi-icon kind="plus"></hi-icon>
      <span>Add ${fieldsetEl.heading}</span>
    `;
    addButtonEl.addEventListener("trigger-ended", (e) => {
      const formEl2 = e.detail.srcElement.closest("hi-form");
      formEl2.$.valueGroup.evaluate("form-submitted");
      formEl2.input = formEl2.$.valueGroup.output;
      const apiData = f1.read(f1.path(inputType.path), [], formEl2.input);
      const addIndex = [
        ...apiData,
        {
          idx: f1.ascending()
        }
      ];
      if (formEl2.input) {
        f1.write(f1.path(inputType.path), addIndex, formEl2.input);
      }
      loopEl.input = addIndex;
      loopEl.setAttribute("value-path", inputType.path);
      if (formEl2) {
        formEl2.refresh();
      }
    });
    iteratorRenderer.appendChild(loopEl);
    fieldsetEl.appendChild(iteratorRenderer);
    fieldsetEl.appendChild(addButtonEl);
    return fieldsetEl;
  }
});
