
import '../../../eip/element/hi-input/hi-input.js';
import { HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import { __applyAttributes, __getElementPath } from './ai-support-input.js';
const entity = "form";
HIP.visitor.add({
  entity,
  render_type: "slider",
  build: (fieldType) => {
    const el = document.createElement("hi-slider");
    dom.setAttrs(el, {
      name: fieldType.name
    });
    return el;
  }
});
HIP.visitor.add({
  entity,
  render_type: "datepicker",
  build: (fieldType, builderEl) => {
    const dropdownEl = document.createElement("hi-dropdown");
    const datepickerEl = document.createElement("hi-datepicker");
    dom.setAttrs(dropdownEl, {
      placeholder: fieldType.placeholder || "Select Date",
      kind: "value",
      exclude: true
    });
    dom.setAttrs(datepickerEl, {
      required: fieldType.required,
      name: __getElementPath(fieldType, builderEl),
      form: builderEl.form,
      ns: builderEl.ns
    });
    if (fieldType.initial_value) {
      datepickerEl.value = fieldType.initial_value;
    }
    dropdownEl.appendChild(datepickerEl);
    return dropdownEl;
  }
});
HIP.visitor.add({
  entity,
  render_type: "calendar",
  build: (fieldType) => {
    const el = document.createElement("hi-datepicker");
    dom.setAttrs(el, {
      name: fieldType.name
    });
    return el;
  }
});
const inputPermittedTypes = HIP.factory.queryOne("hi-input").permittedTypes;
const fieldTypes = [
  ...Object.keys(inputPermittedTypes),
  "string"
];
fieldTypes.forEach((render_type) => {
  HIP.visitor.add({
    entity,
    render_type,
    build: (fieldType) => {
      const el = document.createElement("hi-input");
      __applyAttributes(fieldType, el);
      dom.setAttrs(el, {
        name: fieldType.name,
        type: render_type
      });
      const isTextOrString = render_type === "text" || render_type === "string";
      if (isTextOrString && fieldType.mask != null) {
        const childEl = document.createElement("hi-validation-mask");
        el.appendChild(childEl);
        dom.setAttrs(childEl, {
          placeholder: fieldType.placeholder,
          name: fieldType.name,
          rule: fieldType.mask
        });
      }
      return el;
    }
  });
});
const booleanBuilder = (fieldType) => {
  const el = document.createElement("hi-checkbox");
  el.setAttribute("name", fieldType.name);
  el.setAttribute("kind", fieldType.render_type === "tri-state" ? "tri-state" : "bi-state");
  if (f1.is.boolean(fieldType.initial_value)) {
    dom.flipAttr(el, "checked", fieldType.initial_value);
  }
  return el;
};
HIP.visitor.add({
  entity,
  render_type: "boolean",
  build: (fieldType) => booleanBuilder(fieldType)
});
HIP.visitor.add({
  entity,
  render_type: "bi-state",
  build: (fieldType) => booleanBuilder(fieldType)
});
HIP.visitor.add({
  entity,
  render_type: "tri-state",
  build: (fieldType) => {
    const el = booleanBuilder(fieldType);
    if (fieldType.initial_value == null) {
      dom.flipAttr(el, "indeterminate", true);
    }
    return el;
  }
});
HIP.visitor.add({
  entity,
  render_type: "toggle",
  build: (fieldType, builderEl) => {
    const toggleEl = document.createElement("hi-toggle");
    const icon = f1.read("/$configure/icon", null, fieldType);
    const kind = f1.read("/$configure/kind", "", fieldType);
    if (icon) {
      toggleEl.icon = icon;
    }
    dom.setAttrs(toggleEl, {
      name: fieldType.name,
      form: builderEl.form,
      ns: builderEl.ns,
      kind
    });
    dom.setAttr(toggleEl, "on", f1.format("boolean", {}, fieldType.initial_value));
    return toggleEl;
  }
});
HIP.visitor.add({
  entity,
  render_type: "fallback",
  build: (fieldType) => {
    const el = document.createElement("hi-input");
    dom.setAttrs(el, {
      type: fieldType.render_type || "text",
      name: fieldType.name
    });
    __applyAttributes(fieldType, el);
    return el;
  }
});
