
import { __attachOptionsList, __getElementPath, __formElementInputChanged } from './ai-support-input.js';
import '../../../eip/element/hi-dropdown/hi-dropdown.js';
import '../../../eip/element/hi-combobox/hi-combobox.js';
import '../../../eip/element/hi-select/hi-select.js';
import '../../../eip/element/hi-radio/hi-radio-group.js';
import '../../../eip/element/hi-radio/hi-radio.js';
import { HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
const entity = "form";
const createSelectEl = (fieldType, builderEl) => {
  const selectEl = document.createElement("hi-select");
  dom.setAttrs(selectEl, {
    required: fieldType.required,
    name: __getElementPath(fieldType, builderEl),
    form: builderEl.form,
    ns: builderEl.ns,
    mode: "value",
    nullable: fieldType.nullable,
    blankable: fieldType.blankable,
    "only-value": "only-value"
  });
  const { multiple } = f1.read("/attributes", {}, fieldType);
  const isMultiSelection = f1.is.boolean(multiple) ? multiple : fieldType.max_choices >= 2 || fieldType.max_choices === -1;
  selectEl.max = fieldType.max_choices;
  if (isMultiSelection) {
    selectEl.setAttribute("multiple", "multiple");
  }
  selectEl.allowKeySearch = true;
  if (fieldType.initial_value) {
    if (isMultiSelection) {
      selectEl.values = fieldType.initial_value;
    } else {
      selectEl.value = fieldType.initial_value;
    }
  }
  const choicesBuilder = document.createElement("ai-builder-select");
  choicesBuilder.gid = `${fieldType.name}_select__builder`;
  choicesBuilder.id = `${fieldType.name}_select__builder`;
  choicesBuilder.ns = builderEl.ns;
  __attachOptionsList(choicesBuilder, fieldType, builderEl);
  selectEl.appendChild(choicesBuilder);
  return selectEl;
};
HIP.visitor.add({
  entity,
  render_type: "radio",
  build: (fieldType, builderEl) => {
    const radioGroupEl = document.createElement("hi-radio-group");
    const groupId = `group-radio-${fieldType.name.replace(/\./g, "-")}`;
    dom.setAttrs(radioGroupEl, {
      required: fieldType.required,
      name: __getElementPath(fieldType, builderEl),
      form: builderEl.form,
      ns: builderEl.ns,
      group: groupId
    });
    radioGroupEl.value = fieldType.initial_value;
    const radioBuilderEl = document.createElement("ai-builder-select");
    radioBuilderEl.gid = `${fieldType.name}_radio__builder`;
    radioBuilderEl.id = `${fieldType.name}_radio__builder`;
    radioBuilderEl.ns = builderEl.ns;
    radioBuilderEl.fieldType = fieldType;
    __attachOptionsList(radioBuilderEl, fieldType, builderEl);
    radioGroupEl.appendChild(radioBuilderEl);
    return radioGroupEl;
  }
});
HIP.visitor.add({
  entity,
  render_type: "list",
  build: (fieldType, builderEl) => {
    return createSelectEl(fieldType, builderEl);
  }
});
HIP.visitor.add({
  entity,
  render_type: "dropdown",
  build: (fieldType, builderEl) => {
    const dropdownEl = document.createElement("hi-dropdown");
    const selectEl = createSelectEl(fieldType, builderEl);
    selectEl.setAttribute("aesthetic", "dropdown");
    dom.setAttrs(dropdownEl, {
      placeholder: fieldType.placeholder || "Select an option",
      name: fieldType.path,
      "cap-height": true,
      ns: builderEl.ns,
      kind: "value",
      excluded: true
    });
    dropdownEl.appendChild(selectEl);
    if (fieldType.initial_value) {
      dropdownEl.removeAttribute("placeholder-shown");
    }
    return dropdownEl;
  }
});
HIP.visitor.add({
  entity,
  render_type: "typeahead",
  build: (fieldType, builderEl) => {
    const repeatTemplate = `<template>
      <hi-result value="[[ object.value ]]"
        label="[[ object.title ]]">
      </hi-result>
    </template>`;
    const resultGroupEl = document.createElement("hi-result-group");
    const typeaheadEl = document.createElement("hi-typeahead");
    const repeatEl = document.createElement("hi-repeat");
    dom.setAttrs(resultGroupEl, {
      ns: builderEl.ns
    });
    dom.setAttrs(typeaheadEl, {
      "preserve-value": true,
      form: builderEl.form,
      name: __getElementPath(fieldType, builderEl),
      ns: builderEl.ns,
      kind: "edit",
      "search-delay": 250
    });
    dom.setAttrs(repeatEl, {
      gid: `${fieldType.name}_typeahead__repeat`,
      id: `${fieldType.name}_typeahead__repeat`,
      "value-attr": "title",
      ns: builderEl.ns,
      as: "object"
    });
    repeatEl.innerHTML = repeatTemplate;
    resultGroupEl.appendChild(repeatEl);
    typeaheadEl.appendChild(resultGroupEl);
    const source = __attachOptionsList(repeatEl, fieldType, builderEl);
    if (source.invoke && typeof source.invoke === "function") {
      dom.setAttrs(typeaheadEl, {
        mode: "search"
      });
      const searchKey = fieldType.search_key || "name";
      typeaheadEl.addEventListener("search-value-changed", () => {
        if (typeaheadEl.value === typeaheadEl.searchValue) {
          return;
        }
        if (typeaheadEl.searchValue && typeaheadEl.searchValue.length >= typeaheadEl.min) {
          source.invoke({ [searchKey]: typeaheadEl.searchValue });
        }
      });
    }
    return typeaheadEl;
  }
});
HIP.visitor.add({
  entity,
  render_type: "tree",
  build: (fieldType, builderEl) => {
    const treeEl = document.createElement("hi-tree-chooser");
    dom.setAttrs(treeEl, {
      gid: `${builderEl.gid}__${fieldType.name}`,
      id: `${builderEl.gid}__${fieldType.name}`,
      name: fieldType.path,
      form: builderEl.form,
      ns: builderEl.ns
    });
    __attachOptionsList(treeEl, fieldType, builderEl);
    return treeEl;
  }
});
const __handleComboboxDynamicInputChanged = (comboboxEl, fieldType, fieldTypeUpdate, sourceBucket) => {
  if (!sourceBucket) {
    return __formElementInputChanged(comboboxEl, fieldType, fieldTypeUpdate);
  }
  const paginationData = sourceBucket.metadata;
  const newFieldTypeUpdate = paginationData != null ? { properties: fieldTypeUpdate, metadata: paginationData } : fieldTypeUpdate;
  return __formElementInputChanged(comboboxEl, fieldType, newFieldTypeUpdate);
};
const __handleComboboxDynamic = (comboboxEl, fieldType, builderEl) => {
  comboboxEl.setAttribute("kind", "dynamic");
  comboboxEl.setAttribute("persist-search", "persist-search");
  const sourceBucket = __attachOptionsList("returnSource", fieldType, builderEl);
  const existingInput = sourceBucket.output;
  if (existingInput) {
    __handleComboboxDynamicInputChanged(comboboxEl, fieldType, existingInput, sourceBucket);
  }
  sourceBucket.addEventListener("output-changed", (event) => {
    const newValue = event.detail.value;
    return __handleComboboxDynamicInputChanged(comboboxEl, fieldType, newValue, event.srcElement);
  });
  const searchKey = fieldType.search_key || fieldType.name;
  comboboxEl.addEventListener("search-value-changed", (event) => {
    const searchValue = event.detail.value;
    const hasSearchValue = !f1.is.nil(searchValue);
    const hasMinField = !f1.is.nil(fieldType.min);
    const isSearchValueEmpty = hasSearchValue && f1.is.empty(searchValue);
    if (isSearchValueEmpty) {
      sourceBucket.invoke();
    } else if (hasSearchValue && hasMinField && searchValue.length >= fieldType.min) {
      sourceBucket.invoke({
        [searchKey]: searchValue
      });
    }
  });
  return comboboxEl;
};
const __handleComboboxStatic = (comboboxEl, fieldType, builderEl) => {
  __attachOptionsList(comboboxEl, fieldType, builderEl);
  return comboboxEl;
};
HIP.visitor.add({
  entity,
  render_type: "combobox",
  build: (fieldType, builderEl) => {
    const comboboxEl = document.createElement("hi-combobox");
    dom.setAttrs(comboboxEl, {
      gid: `${builderEl.gid}__${fieldType.name}`,
      id: `${builderEl.gid}__${fieldType.name}`,
      name: fieldType.path,
      form: builderEl.form,
      ns: builderEl.ns
    });
    const isMultiSelection = fieldType.max_choices >= 2 || fieldType.max_choices === -1;
    comboboxEl.max = fieldType.max_choices;
    if (isMultiSelection) {
      comboboxEl.setAttribute("multiple", "multiple");
    }
    if (fieldType.initial_value) {
      if (isMultiSelection) {
        comboboxEl.values = fieldType.initial_value;
      } else {
        comboboxEl.value = fieldType.initial_value;
      }
    }
    const isDynamic = fieldType.attributes.kind === "dynamic";
    if (isDynamic) {
      return __handleComboboxDynamic(comboboxEl, fieldType, builderEl);
    }
    return __handleComboboxStatic(comboboxEl, fieldType, builderEl);
  }
});
