
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';
import { BehaviorBuilder } from '../../../aip/element/ai-behavior-builder/ai-behavior-builder.js';
import './ai-visitor-grid-fallback.js';
import './ai-visitor-grid-status.js';
import './ai-visitor-grid-tasks.js';
export class BuilderGridElement extends BehaviorBuilder(BehaviorElement(AsHIC(PolymerElement))) {
  static get is() {
    return "ai-builder-grid";
  }
  static get properties() {
    return {
      allowColumns: {
        type: Object,
        notify: true
      },
      sortBy: {
        type: String,
        notify: true
      },
      sortOrder: {
        type: String,
        notify: true
      }
    };
  }
  _inputWatcher(_input) {
    const properties = f1.read("/properties", {}, _input);
    this.allowColumns = this.__calculateAllowColumns(properties);
    const sortOrder = this.__calculateSortOrder(properties);
    const sortBy = this.__calculateSortBy(properties);
    if (sortOrder != null) {
      this.sortOrder = sortOrder;
    }
    if (sortBy != null) {
      this.sortBy = sortBy;
    }
    const fields = f1.sortBy((v) => f1.to.integer(v.order), Object.values(properties) || []);
    for (let i = 0; i < fields.length; i++) {
      const fieldType = fields[i];
      const id = fieldType.name;
      if (!fieldType) {
        continue;
      }
      fieldType.ns = this.ns;
      const inputVisitor = HIP.visitor.queryField(fieldType, "grid");
      if (!inputVisitor) {
        continue;
      }
      const columnEl = inputVisitor.build(fieldType, id, this);
      if (!columnEl) {
        continue;
      }
      this._insertElement(columnEl);
    }
  }
  __calculateAllowColumns(properties) {
    return Object.values(
      f1.map(
        (field) => field.name,
        f1.filter(
          (_) => !!_,
          f1.filter((field) => !field.hidden, properties)
        )
      )
    );
  }
  __calculateSortBy(properties) {
    return this.__calculateFieldValue("sort_by", properties);
  }
  __calculateSortOrder(properties) {
    return this.__calculateFieldValue("sort_order", properties);
  }
  __calculateFieldValue(fieldValue, properties) {
    return ifn.first(
      Object.values(
        f1.map(
          (field) => field.name,
          f1.filter((field) => field[fieldValue] != null, properties)
        )
      ) || properties
    );
  }
}
HIP.factory.element(BuilderGridElement);
