
import { HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
const entity = "grid";
export const createColumnEl = (entity2, fieldType, id) => {
  const columnEl = document.createElement("hi-column");
  columnEl.allowSort = typeof fieldType.sortable === "boolean" ? fieldType.sortable : true;
  columnEl.id = fieldType.name || fieldType.id || id;
  columnEl.type = fieldType.render_type;
  columnEl.heading = fieldType.title;
  columnEl.hint = fieldType.hint;
  columnEl.order = fieldType.order;
  columnEl.fieldType = fieldType;
  dom.setAttrs(columnEl, fieldType.attributes);
  return columnEl;
};
HIP.visitor.add({
  entity,
  render_type: "template",
  build: (fieldType, id) => {
    const columnEl = createColumnEl(entity, fieldType, id);
    columnEl.innerHTML = fieldType.template;
    return columnEl;
  }
});
HIP.visitor.add({
  entity,
  render_type: "tooltip",
  build: (fieldType, id) => {
    const columnEl = createColumnEl(entity, fieldType, id);
    const uuid = f1.id();
    const rowColumnId = `${fieldType.name}-column-row-${uuid}`;
    columnEl.innerHTML = `<template>
      <span id="${rowColumnId}">
        [[object.${fieldType.title}]]
      </span>
      <hi-tooltip heading="${fieldType.title}"
        for="${rowColumnId}">
        ${fieldType.template}
      </hi-tooltip>
    </template>`;
    return columnEl;
  }
});
HIP.visitor.add({
  entity,
  render_type: "boolean",
  build: (fieldType, id) => {
    const columnEl = createColumnEl(entity, fieldType, id);
    columnEl.innerHTML = `<template>
      <hi-if comparator='==' value-left='[[object.${fieldType.name}]]' value-right='true'>
        <hi-icon kind='checkbox_checked_fill' aria-label='true' size="medium"></hi-icon>
      </hi-if>
      <hi-if comparator='==' value-left='[[object.${fieldType.name}]]' value-right='false'>
        <hi-icon kind='checkbox_unchecked' aria-label='false' size="medium"></hi-icon>
      </hi-if>
    </template>`;
    return columnEl;
  }
});
const lookupCellBuilder = (fieldType, id) => {
  const columnEl = createColumnEl(entity, fieldType, id);
  const options = fieldType.options;
  let optionsDict = options;
  if (f1.is.array(fieldType.options)) {
    optionsDict = {};
    const n = fieldType.options.length;
    for (let i = 0; i < n; i++) {
      optionsDict[fieldType.options[i]["value"]] = fieldType.options[i]["title"];
    }
  }
  const formatTypeAttr = fieldType.format_type != null ? `formatter="${fieldType.format_type}"` : "";
  fieldType.optionsDictionary = optionsDict;
  columnEl.innerHTML = `<template>
      <hi-text value-dictionary="[[ reference.fieldType.optionsDictionary ]]"
        value="[[ object.${fieldType.name} ]]"
        ${formatTypeAttr}>
      </hi-text>
    </template>
  `;
  return columnEl;
};
HIP.visitor.add({
  entity,
  render_type: "list",
  build: lookupCellBuilder
});
HIP.visitor.add({
  entity,
  render_type: "dropdown",
  build: lookupCellBuilder
});
HIP.visitor.add({
  entity,
  render_type: "text",
  build: (fieldType, id, builderEl) => {
    const columnEl = createColumnEl(entity, fieldType, id);
    const formatTypeAttr = fieldType.format_type != null ? `formatter="${fieldType.format_type}"` : "";
    const valueSink = typeof fieldType.$sink === "object" ? fieldType.$sink.name : fieldType.$sink;
    const binding = `<hi-binding
      source="editable_${fieldType.name}__[[cell_index]]__[[row_index]]"
      sink="${valueSink}"
      signal="submit-ended"
      ns=${builderEl.ns}>
    </hi-binding>`;
    if (fieldType.editable) {
      columnEl.innerHTML = `<template>
        <hi-inline-edit ns=${builderEl.ns}
          gid="editable_${fieldType.name}__[[cell_index]]__[[row_index]]"
          internal-grid
          value="[[ object ]]"
          value-key= "${fieldType.name}"
          auto-height>
          <hi-text slot="display">
          </hi-text>
          <hi-input slot="edit" type="text" internal-grid>
          </hi-input>
        </hi-inline-edit>
        ${binding}
      </template>
    `;
    } else {
      columnEl.innerHTML = `<template>
        <hi-text value="[[ object.${fieldType.name} ]]"
          ${formatTypeAttr}>
        </hi-text>
      </template>
    `;
    }
    return columnEl;
  }
});
HIP.visitor.add({
  entity,
  render_type: "fallback",
  build: (fieldType, id) => {
    return createColumnEl(entity, fieldType, id);
  }
});
