
import { createColumnEl } from './ai-visitor-grid-fallback.js';
import { HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
const entity = "grid";
HIP.visitor.add({
  entity,
  render_type: "badge",
  build: (fieldType, id) => {
    const columnEl = createColumnEl(entity, fieldType, id);
    columnEl.innerHTML = `<template>
      <hi-badge kind="indicator"
        size="large"
        heading="[[object.${fieldType.title}]]"
        theme="[[object.${fieldType.name}]]">
      </hi-badge>
    </template>`;
    return columnEl;
  }
});
HIP.visitor.add({
  entity,
  render_type: "tag",
  build: (fieldType, id) => {
    const columnEl = createColumnEl(entity, fieldType, id);
    const itemPathPeek = f1.read("/$configure/render_type/item_path", fieldType.name, fieldType);
    const itemPath = itemPathPeek.length > 0 ? `object.${itemPathPeek}` : "object";
    columnEl.innerHTML = `<template>
      <hi-tag>
        [[ ${itemPath} ]]
      </hi-tag>
    </template>`;
    return columnEl;
  }
});
HIP.visitor.add({
  entity,
  render_type: "tags",
  build: (fieldType, id, builderGridEl) => {
    const columnEl = createColumnEl(entity, fieldType, id);
    const itemPathPeek = f1.read("/$configure/render_type/item_path", fieldType.name, fieldType);
    const itemPath = itemPathPeek.length > 0 ? `item.${itemPathPeek}` : "item";
    columnEl.innerHTML = `<template>
      <hi-repeat ns$="${builderGridEl.ns}"
        items="[[ object.${fieldType.name} ]]">
        <template>
          <hi-tag>
            [[ ${itemPath} ]]
          </hi-tag>
        </template>
      </hi-repeat>
    </template>`;
    return columnEl;
  }
});
HIP.visitor.add({
  entity,
  render_type: "icon",
  build: (fieldType, id) => {
    const iconKind = fieldType.kind || "info_circle";
    const columnEl = createColumnEl(entity, fieldType, id);
    columnEl.innerHTML = `<template>
      <hi-icon kind="${iconKind}"
        title="[[object.${fieldType.title}]]">
      </hi-icon>
    </template>`;
    return columnEl;
  }
});
