
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';
import { BehaviorBuilder } from '../../../aip/element/ai-behavior-builder/ai-behavior-builder.js';
import './ai-visitor-message-temporary.js';
import './ai-visitor-message-embedded.js';
export class BuilderMessageElement extends BehaviorBuilder(BehaviorElement(AsHIC(PolymerElement))) {
  static get is() {
    return "ai-builder-message";
  }
  _inputWatcher(_input) {
    const messages = Array.from(_input || []);
    const filteredMessages = ifn.filter((o) => !!o, messages);
    f1.each((messageDeclaration) => {
      const messageEl = this.__createMessageElement(messageDeclaration);
      if (!messageEl) {
        return;
      }
      this.__includeMessageElement(messageDeclaration, messageEl);
      this.__includeMessageFlushBinding(messageDeclaration, messageEl);
    }, filteredMessages);
  }
  __createMessageElement(messageDeclaration) {
    messageDeclaration.render_type = messageDeclaration.$is;
    const inputVisitor = HIP.visitor.queryField(messageDeclaration, "message");
    if (!inputVisitor) {
      return;
    }
    const messageEl = inputVisitor.build(messageDeclaration, this);
    if (!messageEl) {
      return;
    }
    const failOverId = `${this.id}__${messageDeclaration.$is}__${messageDeclaration.$instance}__message`;
    const id = messageDeclaration.id || failOverId;
    dom.setAttrs(
      messageEl,
      ifn.merge(
        {
          gid: id,
          id
        },
        ifn.get("$configure.$is", messageDeclaration) || {}
      )
    );
    messageEl.__messageDeclaration__ = messageDeclaration;
    return messageEl;
  }
  __includeMessageElement(messageDeclaration, messageEl) {
    this._insertElement(messageEl);
  }
  __includeMessageFlushBinding(messageDeclaration, messageEl) {
    const inputVisitor = HIP.visitor.queryField(messageDeclaration, "message");
    if (!inputVisitor) {
      return;
    }
    const bindingEl = document.createElement("hi-binding");
    const bindingId = `binding-${inputVisitor.message_endpoint}-${messageDeclaration.$instance}`;
    dom.setAttrs(bindingEl, {
      id: bindingId,
      gid: bindingId,
      field: inputVisitor.message_endpoint,
      source: messageDeclaration.$instance,
      sink: inputVisitor.messageSink(messageEl),
      ns: this.ns
    });
    this._insertElement(bindingEl);
    if (messageDeclaration.$output || messageDeclaration.$strategy) {
      const outputBindingEl = document.createElement("hi-binding");
      const { instance: outputSinkInstance } = HIP.kernel.normalizeReference(messageDeclaration.$output);
      outputBindingEl.ns = this.ns;
      outputBindingEl.source = bindingId;
      outputBindingEl.signal = "invoke-ended";
      outputBindingEl.sink = outputSinkInstance;
      outputBindingEl.mapFn = function(packet) {
        const control = packet.event.detail.rtn.type;
        const instanceRef = ifn.get(`$strategy.value.${control}`, messageDeclaration);
        if (instanceRef) {
          HIP.kernel.assignToReference(this.ns, instanceRef, packet.output);
        }
        return packet.output;
      };
      this._insertElement(outputBindingEl);
    }
  }
}
HIP.factory.element(BuilderMessageElement);
