
import { HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
const assignDefaultValue = (optionType, parentFieldType, id) => {
  const typeAliases = f1.read("/aliases", {}, parentFieldType);
  const valueAliasType = typeAliases.value;
  const titleAliasType = typeAliases.title;
  const valueAlias = valueAliasType ? optionType[valueAliasType.name] : void 0;
  const titleAlias = titleAliasType ? optionType[titleAliasType.name] : void 0;
  const primaryValue = valueAlias || optionType.value;
  const value = primaryValue !== void 0 ? primaryValue : optionType.name || optionType.id || id;
  return {
    textContent: titleAlias || optionType.title || optionType.label,
    value
  };
};
HIP.visitor.add({
  entity: "select",
  render_type: "fallback",
  build: (optionType, parentFieldType, id) => {
    const { textContent, value } = assignDefaultValue(optionType, parentFieldType, id);
    const optionEl = document.createElement("hi-option");
    dom.flipAttr(optionEl, "selected", !!optionType.selected);
    optionEl.setAttribute("value", value + "");
    optionEl.textContent = textContent;
    return [optionEl];
  }
});
HIP.visitor.add({
  entity: "select",
  render_type: "radio",
  build: (optionType, parentFieldType, id) => {
    const { textContent, value } = assignDefaultValue(optionType, parentFieldType, id);
    const labelEl = document.createElement("hi-label");
    const radioEl = document.createElement("hi-radio");
    const uniqueId = `select-radio-${f1.id()}`;
    labelEl.textContent = textContent;
    dom.setAttrs(labelEl, {
      for: uniqueId
    });
    dom.setAttrs(radioEl, {
      selected: !!optionType.selected,
      id: uniqueId,
      value
    });
    return [radioEl, labelEl];
  }
});
