
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import { html } from '../../../hip/library/lit-html/lit-html.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import { BehaviorBuilder } from '../../../aip/element/ai-behavior-builder/ai-behavior-builder.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import './ai-visitor-repeater-fallback.js';
import './ai-visitor-template-fallback.js';
import './ai-visitor-template-card.js';
import './ai-visitor-repeater-card-list-default.js';
export class BuilderTemplateElement extends BehaviorBuilder(BehaviorElement(AsHIC(PolymerElement))) {
  static get is() {
    return "ai-builder-template";
  }
  static get observers() {
    return ["__inputWatcher(input, template)"];
  }
  static get properties() {
    return {
      mode: {
        type: String,
        value: "template"
      },
      template: {
        type: String,
        value: "fallback"
      },
      for: {
        type: String
      },
      host: {
        type: String
      },
      method: {
        type: String
      },
      forChild: {
        type: String
      },
      litFn: {
        type: Object,
        notify: true
      }
    };
  }
  _inputWatcher(_input) {
    const fieldType = _input;
    const fieldProperties = ifn.get("properties", fieldType);
    if (f1.is.empty(fieldProperties)) {
      return;
    }
    const hostElExternal = HIP.browser.queryOne(this.host, this.ns);
    const modeType = this.mode === "template" ? "item" : "repeater";
    const cardTemplates = ifn.get(`render_hints.template[${modeType}]`, fieldProperties);
    const inputVisitor = HIP.visitor.query(
      {
        visitor_type: cardTemplates || this.template
      },
      this.mode
    );
    let method;
    let hostEl;
    if (inputVisitor) {
      hostEl = inputVisitor;
      method = "build";
    } else if (hostElExternal) {
      method = this.method;
      if (method in hostElExternal) {
        hostEl = hostElExternal;
      } else if (method in hostElExternal.output) {
        hostEl = hostElExternal.output;
      }
    }
    const isInvalidTemplateBuilder = !hostEl || typeof hostEl[method] !== "function";
    if (isInvalidTemplateBuilder) {
      method = "__forcedVisitorFallback";
      hostEl = this;
    }
    const forChildEl = HIP.browser.queryOne(this.forChild, this.ns);
    const hasLitFnForRepeater = forChildEl && forChildEl.litFn;
    const _litFn = hostEl[method](fieldProperties, fieldType, this);
    const litFn = this.litFn = hasLitFnForRepeater ? _litFn(forChildEl.litFn) : (
      _litFn((...args) => this.__widgetBuilderFn(...args))
    );
    const litEl = HIP.browser.queryOne(this.for, this.ns);
    if (litEl) {
      litEl.litFn = litFn;
    }
  }
  __forcedVisitorFallback(recordType) {
    const fallbackVisitor = HIP.visitor.queryField(
      {
        visitor_type: "fallback"
      },
      this.mode
    );
    return fallbackVisitor.build(recordType);
  }
  __widgetBuilderFn(taskFieldType, o, index) {
    const layoutKind = taskFieldType.render_type === "tasks_menu" ? "vert" : "hort";
    const layoutOrientation = taskFieldType.render_type === "tasks_menu" ? "left" : "center";
    return html`
      <div
        id="_${taskFieldType.name}_regions__singular_task__widgets_group__${index}"
        l-kind="${layoutKind}"
        l-align="${layoutOrientation}"
        l-width-fixed
      >
        <ai-builder-widget
          gid="_${taskFieldType.name}_regions__singular_task__widgets_group__${index}__builder"
          .value="${o}"
          ns="${this.ns}"
        >
        </ai-builder-widget>
      </div>
      <hi-binding
        source="source"
        sink="_${taskFieldType.name}_regions__singular_task__widgets_group__${index}__builder"
        ns="${this.ns}"
      >
        <di-fn-get path="region.${taskFieldType.$region || "record_row"}.tasks.singular"></di-fn-get>
      </hi-binding>
    `;
  }
}
HIP.factory.element(BuilderTemplateElement);
