
import { html, nothing } from '../../../hip/library/lit-html/lit-html.js';
import { HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
const __definitionListTemplate = (typeFieldsKeys, record) => {
  return typeFieldsKeys.filter((fieldType) => fieldType.render_type !== "tasks").map((fieldType) => {
    if (!ifn.is(Object, fieldType)) {
      return html`${nothing}`;
    }
    const field = fieldType.title || "";
    let value = ifn.get(fieldType.name, record);
    if (f1.is.nil(value)) {
      value = "";
    }
    const formattedValue = HIP.fn.format(value, fieldType.format_type, ifn.get("$configure.format_type", fieldType));
    return html`
        <dt>${field}${__createInputHint(fieldType)}</dt>
        <dd>${formattedValue !== "" && formattedValue != null ? formattedValue : html`<span>&nbsp;</span>`}</dd>
      `;
  });
};
const __createInputHint = (fieldType) => {
  const hintId = `${fieldType.name}_label_hint`;
  return !fieldType.hint ? nothing : html`<hi-icon id=${hintId} kind="info_circle" mode="content" type="material"></hi-icon>
        <hi-tooltip for=${hintId} placement="right">${fieldType.hint}</hi-tooltip>`;
};
const __widgetTemplate = (widgetBuilderFn, typeFieldsKeys, record) => {
  const taskFieldTypes = typeFieldsKeys.filter((fieldType) => fieldType.render_type === "tasks");
  return widgetBuilderFn && taskFieldTypes && taskFieldTypes.length > 0 ? html`<footer>
        ${ifn.map((taskFieldType, index) => widgetBuilderFn(taskFieldType, record, index), taskFieldTypes)}
      </footer>` : nothing;
};
const __orderedTypeFields = (hilFields) => {
  const typeFields = ifn.is(Object, hilFields) ? Object.values(hilFields) || [] : Array.from(hilFields || []);
  return typeFields.filter((o) => !o.hidden).sort((a, b) => a.order || 0 - b.order || 0);
};
export { __definitionListTemplate, __widgetTemplate, __orderedTypeFields };
