
import { html, nothing } from '../../../hip/library/lit-html/lit-html.js';
import { repeat } from '../../../hip/library/lit-html/directives/repeat.js';
import { unsafeHTML } from '../../../hip/library/lit-html/directives/unsafe-html.js';
import { classMap } from '../../../hip/library/lit-html/directives/class-map.js';
import { HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';
const entity = "template";
const willInclude = (condition, template) => {
  return condition ? template : nothing;
};
const createDataList = (data, hilType, isHort) => {
  let datalist = [];
  if (data.input && data.input.length > 0) {
    datalist = data.input;
  } else {
    datalist = Object.keys(data.__otherPropData).map((key) => {
      const title = hilType.properties[key].title || key;
      const value = data.__otherPropData[key] || " ";
      return {
        title,
        value
      };
    });
  }
  const definitionBlockTmpl = (item) => {
    return html`<dt>${item.title}</dt>
      <dd>${item.value !== "" ? item.value : html`<span>&nbsp;</span>`}</dd>`;
  };
  return html`${repeat(
    datalist,
    (item) => isHort ? html`<div l-size="12" l-sml-size="3">${definitionBlockTmpl(item)}</div>` : html`${definitionBlockTmpl(item)}`
  )}`;
};
const buildCardSections = ({ data, tasks, widgetBuilderFn, input, index, templateConfig, hilType, hilFields }) => {
  const isHorizontal = templateConfig && templateConfig.layout === "horizontal";
  const isVertical = !isHorizontal;
  const hasBadges = ifn.is("array", data.badges) && data.badges.length > 0;
  const header = html`<section class="hi-card__section hi-card__header">
    ${willInclude(data.title, html`<p class="hi-card__title">${data.title}</p>`)}
    ${willInclude(data.sub_title, html`<span class="hi-card__subtitle">${data.sub_title}</span>`)}
    ${willInclude(
    data.status,
    html`<div class="hi-card__badge-wrapper"><hi-badge aesthetic="light">${data.status}</hi-badge></div>`
  )}
    ${hasBadges ? html`<div class="hi-card__badge-wrapper">
          ${data.badges.map(
    (badge) => html`<hi-badge aesthetic="light" theme="${badge.theme || "primary"}">${badge.title}</hi-badge>`
  )}
        </div>` : nothing}
    ${willInclude(data.description, html`<p class="hi-card__description">${unsafeHTML(data.description)}</p>`)}
  </section>`;
  const thumbnail = html`${willInclude(
    data.initials,
    html`<div class="hi-card__section hi-card__thumbnail">
      <hi-avatar size="medium" initials="${data.initials}"></hi-avatar>
    </div>`
  )} `;
  const image = html`${willInclude(
    data.image,
    html` <section class="hi-card__section hi-card__thumbnail hi-card__image">
      <hi-image uri="${data.image}"></hi-image>
    </section>`
  )}`;
  const actions = html`<section class="hi-card__section hi-card__actions hi-card__toolbar">
    ${willInclude(
    tasks.record_row.length > 0,
    ifn.map((taskFieldType) => widgetBuilderFn(taskFieldType, input, index), tasks.record_row)
  )}
  </section>`;
  const viewDetails = html`<section class="hi-card__section hi-card__actions">
    ${willInclude(
    tasks.record_row_details.length > 0,
    ifn.map((taskFieldType) => widgetBuilderFn(taskFieldType, input, index), tasks.record_row_details)
  )}
  </section>`;
  const menu = html`<section class="hi-card__section hi-card__section--prefix">
    ${willInclude(
    tasks.record_row_menu.length > 0,
    ifn.map((taskFieldType) => widgetBuilderFn(taskFieldType, input, index), tasks.record_row_menu)
  )}
  </section>`;
  const reveal = html`<section class="hi-card__section">
    ${willInclude(
    data.additional_content,
    html`<hi-reveal kind="additional">
        <p class="hi-card__description" slot="additional-content">${data.additional_content}</p>
      </hi-reveal>`
  )}
  </section>`;
  const dataList = isHorizontal ? html`<section class="hi-card__section hi-card__data-list">
        <dl l-kind="hort" kind>${createDataList(data, hilType, isHorizontal)}</dl>
      </section>` : html`<section class="hi-card__section hi-card__data-list">
        <dl kind="data">${createDataList(data, hilType, isHorizontal)}</dl>
      </section>`;
  const layout = {
    "hi-card--hort": isHorizontal
  };
  return {
    header,
    thumbnail,
    image,
    actions,
    viewDetails,
    menu,
    reveal,
    dataList,
    layout
  };
};
const formatDataByFormatType = (input, hilTypeSet, typeMap) => {
  return ifn.map(
    (fieldType) => {
      const fieldName = fieldType.name;
      const fieldValue = input[fieldName];
      return fieldType && fieldType.format_type ? HIP.fn.format(fieldValue, fieldType.format_type, ifn.get("$configure.format_type", fieldType)) : fieldValue;
    },
    ifn.filter((field) => !field.hidden, hilTypeSet)
  );
};
const cardContentBuilder = (builderFn) => {
  return (hilFields, hilType) => {
    const typeAliases = ifn.get("aliases", hilType);
    const typeMap = ifn.filter((field) => !field.hidden, typeAliases);
    const templateConfig = ifn.get("render_hints.$configure.item.attributes", hilFields);
    return (widgetBuilderFn) => {
      return (input, index) => {
        const tasks = {
          record_row_details: ifn.filter(
            (field) => field.render_type === "tasks_details",
            Object.values(hilType.properties)
          ),
          record_row: ifn.filter(
            (field) => field.render_type === "tasks",
            Object.values(hilType.properties)
          ),
          record_row_menu: ifn.filter(
            (field) => field.render_type === "tasks_menu",
            Object.values(hilType.properties)
          )
        };
        const otherProps = ifn.filter((field) => field.$alias == null, hilType.properties);
        const formattedDataAliases = formatDataByFormatType(input, hilType.aliases);
        const formattedDataProperties = formatDataByFormatType(input, otherProps);
        const data = formattedDataAliases;
        data.__otherPropData = formattedDataProperties;
        const templateDef = {
          data,
          tasks,
          widgetBuilderFn,
          index,
          templateConfig,
          hilType,
          hilFields
        };
        return builderFn(templateDef);
      };
    };
  };
};
HIP.visitor.add({
  entity,
  render_type: "card-basic-template",
  build: cardContentBuilder((e) => {
    const { header, layout } = buildCardSections(e);
    return html` <article class="hi-card ${classMap(layout)}">${header}</article> `;
  })
});
HIP.visitor.add({
  entity,
  render_type: "card-avatar-template",
  build: cardContentBuilder((e) => {
    const { thumbnail, header, layout } = buildCardSections(e);
    return html` <article class="hi-card ${classMap(layout)}">${thumbnail} ${header}</article> `;
  })
});
HIP.visitor.add({
  entity,
  render_type: "card-actions-template",
  build: cardContentBuilder((e) => {
    const { header, actions, layout } = buildCardSections(e);
    return html` <article class="hi-card ${classMap(layout)}">${header} ${actions}</article> `;
  })
});
HIP.visitor.add({
  entity,
  render_type: "card-details-template",
  build: cardContentBuilder((e) => {
    const { header, viewDetails, layout } = buildCardSections(e);
    return html` <article class="hi-card hi-card--interactive ${classMap(layout)}">${header} ${viewDetails}</article> `;
  })
});
HIP.visitor.add({
  entity,
  render_type: "card-menu-template",
  build: cardContentBuilder((e) => {
    const { menu, header, layout } = buildCardSections(e);
    return html` <article class="hi-card ${classMap(layout)}">${menu} ${header}</article> `;
  })
});
HIP.visitor.add({
  entity,
  render_type: "card-image-template",
  build: cardContentBuilder((e) => {
    const { image, header, actions, layout } = buildCardSections(e);
    return html` <article class="hi-card ${classMap(layout)}">${image} ${header} ${actions}</article> `;
  })
});
HIP.visitor.add({
  entity,
  render_type: "card-expandable-template",
  build: cardContentBuilder((e) => {
    const { header, reveal, actions, layout } = buildCardSections(e);
    return html` <article class="hi-card ${classMap(layout)}">${header} ${reveal} ${actions}</article> `;
  })
});
HIP.visitor.add({
  entity,
  render_type: "card-datalist-template",
  build: cardContentBuilder((e) => {
    const { header, dataList, actions, layout } = buildCardSections(e);
    return html` <article class="hi-card ${classMap(layout)}">${header} ${dataList} ${actions}</article> `;
  })
});
