
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import { Debouncer } from '../../../hip/library/polymer/polymer.js';
import { idlePeriod } from '../../../hip/library/polymer/polymer.js';
import '../../../aip/element/ai-expression/ai-expression.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';
import { BehaviorBuilder } from '../../../aip/element/ai-behavior-builder/ai-behavior-builder.js';
import '../../../aip/element/ai-behavior-builder/ai-behavior-builder.js';
import '../../../aip/element/ai-builder-container/ai-builder-container.js';
import './ai-visitor-widget-datepicker.js';
import './ai-visitor-widget-control.js';
import './ai-visitor-widget-select.js';
export class BuilderWidgetElement extends BehaviorBuilder(BehaviorElement(AsHIC(PolymerElement))) {
  constructor() {
    super();
    this.__addFieldExpression();
  }
  static get is() {
    return "ai-builder-widget";
  }
  static get properties() {
    return {
      value: {
        type: Object
      },
      pullSource: {
        type: String
      },
      pullField: {
        type: String
      }
    };
  }
  static get observers() {
    return ["__valueWatcher(value)"];
  }
  __addFieldExpression() {
    const exprEl = this.__fieldsExprEl = document.createElement("ai-expression");
    exprEl.hidden = true;
    exprEl.ns = this.ns;
  }
  __valueWatcher() {
    this.__valueDebouncer = Debouncer.debounce(this.__valueDebouncer, idlePeriod, () => {
      this.flush();
    });
  }
  get __builderContainer() {
    this.__containerBuilderEl = this.__containerBuilderEl || document.createElement("ai-builder-container");
    this.__containerBuilderEl.ns = this.ns;
    return this.__containerBuilderEl;
  }
  _inputWatcher(input) {
    const widgets = Array.from(f1.safe([], input));
    const filteredWidgets = ifn.filter((_) => !!_, widgets);
    const orderedWidgets = f1.sortBy((o) => (o.order | 0) + 1, filteredWidgets);
    const containerFragment = this.__builderContainer.buildContainers(
      { properties: orderedWidgets },
      (widgetDeclaration) => {
        return this.__createWrapperElement(widgetDeclaration, document.createDocumentFragment());
      }
    );
    this.__builderContainer.insertContainer(containerFragment, this);
  }
  __createWrapperElement(widgetDeclaration, containerEl) {
    this.__calculateWidgetRenderType(widgetDeclaration);
    const widgetEl = this.__createWidgetElement(widgetDeclaration, this);
    if (!widgetEl) {
      return;
    }
    this.__applyConditionalTetheredGid(widgetDeclaration);
    this.__includeWidget(widgetEl, containerEl);
    this.__applyValue(widgetDeclaration, widgetEl);
    this.__applyConditionalExpressions(widgetDeclaration, widgetEl);
    this.__applyFieldsExpressions(widgetDeclaration, widgetEl);
    this.__applyLabel(widgetDeclaration, widgetEl);
    this.__includeWidgetValueStrategyBinding(widgetDeclaration, widgetEl);
    this.__includeWidgetValuePayloadBinding(widgetDeclaration, widgetEl);
    this.__includeWidgetStateInputBinding(widgetDeclaration, widgetEl);
    this.__includeWidgetStateOutputBinding(widgetDeclaration, widgetEl);
    return containerEl;
  }
  __applyConditionalTetheredGid(widgetDeclaration) {
    const conditionalTetherId = ifn.get("_runtime.conditional_tether_id", widgetDeclaration);
    if (conditionalTetherId == null) {
      return;
    }
    const widgetIsNotTethered = this.gid.indexOf(conditionalTetherId) < 0;
    if (widgetIsNotTethered) {
      this.gid = `${this.gid},${conditionalTetherId}`;
    }
  }
  __calculateWidgetRenderType(widgetDeclaration) {
    const widgetBehaviorAliases = {
      download: "download",
      operation: "button",
      navigation: "link",
      toggle: "toggle",
      button: "button",
      flag: "toggle",
      link: "link",
      flyout: "flyout",
      dropdown: "dropdown"
    };
    widgetDeclaration.render_type = widgetBehaviorAliases[widgetDeclaration.$is] || widgetDeclaration.$is || "button";
  }
  __createWidgetElement(widgetDeclaration, parentEl) {
    const inputVisitor = HIP.visitor.queryField(widgetDeclaration, "widget");
    if (!inputVisitor) {
      return;
    }
    const widgetEl = inputVisitor.build(widgetDeclaration, parentEl);
    if (!widgetEl) {
      return;
    }
    const failOverId = `${this.id}__${widgetDeclaration.$is}__task_widget`;
    const id = widgetDeclaration.id || failOverId;
    const generateUniqueId = id + "_" + f1.id();
    dom.setAttrs(
      widgetEl,
      ifn.merge(
        {
          theme: widgetDeclaration.theme || "primary",
          layout: widgetDeclaration.layout,
          class: "mas",
          gid: generateUniqueId,
          id: generateUniqueId
        },
        ifn.get("$configure.$is", widgetDeclaration) || {}
      )
    );
    widgetEl["__widgetDeclaration__"] = widgetDeclaration;
    return widgetEl;
  }
  __includeWidget(widgetEl, containerEl) {
    const insertEl = widgetEl["__containerEl__"] ? widgetEl["__containerEl__"] : widgetEl;
    this._insertElement(insertEl, containerEl);
  }
  __applyValue(widgetDeclaration, widgetEl) {
    let value;
    if (this.value != null) {
      value = this.value;
    } else if (widgetDeclaration.initial_value != null) {
      value = widgetDeclaration.initial_value;
    }
    const hasValue = value != null;
    if (hasValue) {
      widgetEl["value"] = value;
      widgetEl["__associatedRecord__"] = value;
    }
    if (hasValue && widgetDeclaration.value_path != null && widgetEl.localName === "hi-toggle") {
      widgetEl["on"] = ifn.get(widgetDeclaration.value_path, value);
    }
  }
  __applyConditionalExpressions(widgetDeclaration, widgetEl) {
    const conditionalGroupId = ifn.get("_runtime.conditional_group_id", widgetDeclaration);
    if (conditionalGroupId == null) {
      return;
    }
    const conditionalGroupEl = HIP.browser.queryOne(conditionalGroupId, this.ns);
    const expressionsEls = f1.to.array(
      f1.read("/children", [], f1.safe({}, conditionalGroupEl))
    );
    const value = widgetEl["value"] == null ? true : widgetEl["value"];
    f1.each((expression) => {
      expression.evaluate(widgetEl, value);
    }, expressionsEls);
  }
  __applyFieldsExpressions(widgetDeclaration, widgetEl) {
    const value = widgetEl.value == null ? true : widgetEl.value;
    const exprEl = this.__fieldsExprEl;
    ifn.each((expression, field) => {
      exprEl.field = field;
      exprEl.value = expression;
      exprEl.argument = value;
      exprEl.ns = this.ns;
      exprEl.evaluate(widgetEl);
    }, widgetDeclaration.$fields || {});
  }
  __includeWidgetValueStrategyBinding(widgetDeclaration, widgetEl) {
    const payloadGid = ifn.get("_runtime.payload_gid", widgetDeclaration) || ifn.get("_runtime._pipe_gid", widgetDeclaration);
    if (payloadGid == null) {
      return;
    }
    if (!widgetDeclaration.$strategy) {
      return;
    }
    const inputVisitor = HIP.visitor.queryField(widgetDeclaration, "widget");
    if (!inputVisitor) {
      return;
    }
    if (!inputVisitor.strategyAction) {
      return;
    }
    const bindingEl = document.createElement("hi-binding");
    bindingEl.source = widgetEl;
    bindingEl.signal = inputVisitor.activityEvent || "trigger-ended";
    bindingEl.sink = "noop";
    const pullSource = this.pullSource || this.getAttribute("pull-source") || "";
    const pullField = this.pullField || this.getAttribute("pull-field") || "";
    if (pullSource) {
      bindingEl.pullSource = pullSource;
      bindingEl.pullField = pullField;
    }
    inputVisitor.strategyAction(widgetDeclaration, widgetEl, bindingEl);
    this._insertElement(bindingEl);
  }
  __includeWidgetValuePayloadBinding(widgetDeclaration, widgetEl) {
    const inputVisitor = HIP.visitor.queryField(widgetDeclaration, "widget");
    if (!inputVisitor) {
      return;
    }
    const payloadGid = ifn.get("_runtime.payload_gid", widgetDeclaration) || ifn.get("_runtime._pipe_gid", widgetDeclaration);
    if (payloadGid != null) {
      const bindingEl = document.createElement("hi-binding");
      bindingEl.source = widgetEl;
      bindingEl.signal = inputVisitor.activityEvent || "trigger-ended";
      bindingEl.field = "invoke";
      bindingEl.sink = payloadGid;
      const pullSource = this.pullSource || this.getAttribute("pull-source");
      const pullField = this.pullField || this.getAttribute("pull-field");
      if (pullSource) {
        bindingEl.pullSource = pullSource;
        bindingEl.pullField = pullField;
      }
      this._insertElement(bindingEl);
    }
  }
  __includeWidgetStateInputBinding(widgetDeclaration, widgetEl) {
    const widgetStateInputRef = ifn.get("_runtime.widget_state_input_ref", widgetDeclaration);
    if (widgetStateInputRef != null) {
      const bindingEl = document.createElement("hi-binding");
      bindingEl.sourceProperty = widgetStateInputRef.property;
      bindingEl.source = widgetStateInputRef.instance;
      bindingEl.signal = widgetStateInputRef.field;
      bindingEl.sink = widgetEl;
      bindingEl.field = widgetEl.autoBinding.field || widgetEl.autoBinding["slot"];
      this._insertElement(bindingEl);
    }
  }
  __includeWidgetStateOutputBinding(widgetDeclaration, widgetEl) {
    const widgetStateOutputRef = ifn.get("_runtime.widget_state_output_ref", widgetDeclaration);
    if (widgetStateOutputRef != null) {
      const bindingEl = document.createElement("hi-binding");
      bindingEl.source = widgetEl;
      bindingEl.signal = widgetEl.autoBinding.signal;
      bindingEl.sink = widgetStateOutputRef.instance;
      bindingEl.field = widgetStateOutputRef.field;
      bindingEl.sinkProperty = widgetStateOutputRef.property;
      this._insertElement(bindingEl);
    }
  }
  __applyLabel(widgetDeclaration, widgetEl) {
    const textOrPath = widgetDeclaration.title || widgetDeclaration.label;
    if (!textOrPath) {
      return;
    }
    const isPropertyLookup = textOrPath.indexOf(".") === 0;
    const text = isPropertyLookup ? ifn.get(textOrPath, widgetEl.value || {}) || "" : textOrPath;
    if (widgetEl["__labelEl__"]) {
      widgetEl["__labelEl__"].textContent = text;
    }
  }
  flush() {
    super.flush();
    this.builtEls.filter((el) => el["__widgetDeclaration__"] != null).map((widgetEl) => {
      this.__applyValue(widgetEl["__widgetDeclaration__"], widgetEl);
      this.__applyConditionalExpressions(widgetEl["__widgetDeclaration__"], widgetEl);
      this.__applyLabel(widgetEl["__widgetDeclaration__"], widgetEl);
    });
  }
}
HIP.factory.element(BuilderWidgetElement);
