
import { HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
const entity = "widget";
const includeLabel = (widgetEl) => {
  const labelEl = document.createElement("hi-label");
  dom.setAttrs(labelEl, {
    class: "text text-sz-m mrsx",
    for: widgetEl.id
  });
  widgetEl["__labelEl__"] = labelEl;
  return labelEl;
};
const includeSelectEl = (declaration, builderEl, wrapperEl) => {
  const selectEl = document.createElement("hi-select");
  dom.setAttrs(selectEl, {
    ns: builderEl["ns"],
    mode: "value"
  });
  const builderSelectEl = document.createElement("ai-builder-select");
  dom.setAttrs(builderSelectEl, {
    gid: `${declaration.name}__select_builder`,
    id: `${declaration.name}__select_builder`,
    ns: builderEl["ns"]
  });
  const sourceInstance = ifn.get("$configure.relationship.instance", declaration);
  const sourceDataToBuilderEl = document.createElement("hi-binding");
  dom.setAttrs(sourceDataToBuilderEl, {
    sink: builderSelectEl.gid,
    source: sourceInstance,
    ns: builderEl["ns"]
  });
  builderSelectEl.fieldType = declaration.$type_ref;
  selectEl.appendChild(sourceDataToBuilderEl);
  selectEl.appendChild(builderSelectEl);
  wrapperEl.appendChild(selectEl);
  selectEl["__containerEl__"] = wrapperEl;
  return selectEl;
};
const includeBindingContext = (widgetDeclaration, widgetEl, bindingEl, getWidgetState) => {
  return (bindingContext) => {
    const ns = bindingEl.ns;
    const widgetState = getWidgetState();
    const strategyActionInstanceName = ifn.get(`value.${widgetState}`, widgetDeclaration.$strategy);
    const input = widgetDeclaration.value_path ? ifn.set(widgetDeclaration.value_path, widgetEl["__associatedRecord__"] || {}, widgetState) : widgetState;
    if (strategyActionInstanceName) {
      HIP.kernel.assignToReference(ns, strategyActionInstanceName, input);
    }
    return bindingContext;
  };
};
HIP.visitor.add({
  entity,
  render_type: "toggle",
  build: (widgetDeclaration) => {
    const toggleGroupEl = document.createElement("hi-toggle-group");
    const toggleEl = document.createElement("hi-toggle");
    if (widgetDeclaration.layout) {
      toggleGroupEl.layout = widgetDeclaration.layout;
    }
    if (widgetDeclaration.kind) {
      toggleEl.kind = widgetDeclaration.kind;
    }
    if (widgetDeclaration.icon) {
      toggleEl["icon"] = widgetDeclaration.icon;
    }
    toggleEl["__containerEl__"] = toggleGroupEl;
    toggleGroupEl.appendChild(toggleEl);
    toggleGroupEl.appendChild(includeLabel(toggleEl));
    return toggleEl;
  },
  strategyAction: (widgetDeclaration, widgetEl, bindingEl) => {
    bindingEl.mapFn = includeBindingContext(widgetDeclaration, widgetEl, bindingEl, () => widgetEl["on"]);
    return bindingEl;
  }
});
HIP.visitor.add({
  entity,
  render_type: "flyout",
  activityEvent: "selected-ended",
  build: (widgetDeclaration, widgetBuilderEl) => {
    const flyoutEl = document.createElement("hi-flyout");
    dom.setAttrs(flyoutEl, {
      ns: widgetBuilderEl.ns,
      aesthetic: "chrome",
      alignment: "left",
      kind: "value"
    });
    if (widgetDeclaration.icon) {
      flyoutEl.icon = widgetDeclaration.icon;
    }
    return includeSelectEl(widgetDeclaration, widgetBuilderEl, flyoutEl);
  },
  strategyAction: (widgetDeclaration, widgetEl, bindingEl) => {
    bindingEl.mapFn = includeBindingContext(widgetDeclaration, widgetEl, bindingEl, () => widgetEl["value"]);
    return bindingEl;
  }
});
HIP.visitor.add({
  entity,
  render_type: "dropdown",
  build: (widgetDeclaration, widgetBuilderEl) => {
    const dropdownEl = document.createElement("hi-dropdown");
    dom.setAttrs(dropdownEl, {
      ns: widgetBuilderEl.ns,
      placeholder: widgetDeclaration.placeholder || "Select an option",
      "cap-height": true,
      kind: "value"
    });
    return includeSelectEl(widgetDeclaration, widgetBuilderEl, dropdownEl);
  },
  strategyAction: (widgetDeclaration, widgetEl, bindingEl) => {
    bindingEl.mapFn = includeBindingContext(widgetDeclaration, widgetEl, bindingEl, () => widgetEl["value"]);
    return bindingEl;
  }
});
