
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import ess from '../../../pip/library/pi-ess/pi-ess.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import { BehaviorIO } from '../../../eip/element/hi-behavior-io/hi-behavior-io.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
export class ChannelElement extends BehaviorIO(BehaviorElement(AsHIC(PolymerElement))) {
  constructor(...args) {
    super(args);
    this.__onMessageBound = this.__onMessage.bind(this);
  }
  static get is() {
    return "ai-channel";
  }
  static get properties() {
    return {
      mode: {
        type: String
      },
      topic: {
        type: String,
        observer: "__topicChanged",
        notify: true
      }
    };
  }
  static get api() {
    return ["destroy"];
  }
  static get valueEvent() {
    return "output-changed";
  }
  static get autoBinding() {
    return {
      signal: "output-changed",
      field: "invoke"
    };
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    this._unlistenMessage();
    this.__onMessageBound = void 0;
  }
  destroy() {
    this._unlistenMessage();
    this.set("input", null);
  }
  __topicChanged() {
    this._unlistenMessage();
    this._listenMessage();
  }
  _inputChanged(input) {
    this._writeMessage(input);
  }
  __onMessage(event) {
    this.__mutex__ = true;
    this.set("input", event.data);
    this.__mutex__ = false;
  }
  _listenMessage() {
    if (this.mode === "producer") {
      return;
    }
    if (f1.is.nil(this.__onMessageBound)) {
      return;
    }
    return this.__listener = ess.on(
      {
        site: "ai-lucid",
        source: this.topic,
        action: "invoke"
      },
      this.__onMessageBound
    );
  }
  _unlistenMessage() {
    if (this.__listener) {
      ess.off(this.__listener);
      this.__listener = void 0;
    }
  }
  _writeMessage(data) {
    if (this.mode === "consumer") {
      return;
    }
    if (this.__mutex__) {
      return;
    }
    ess.send({
      site: "ai-lucid",
      source: this.topic,
      action: "invoke",
      producer: this,
      data
    });
  }
}
export default HIP.factory.element(ChannelElement);
