
import { afterNextRender } from '../../../hip/library/polymer/polymer.js';
import { animationFrame } from '../../../hip/library/polymer/polymer.js';
import { Debouncer } from '../../../hip/library/polymer/polymer.js';
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
export class ExpressionElement extends BehaviorElement(AsHIC(PolymerElement)) {
  static get is() {
    return "ai-expression";
  }
  static get properties() {
    return {
      coercion: {
        type: String,
        reflectToAttribute: true
      },
      argument: {
        type: Object
      },
      field: {
        type: String,
        reflectToAttribute: true
      },
      value: {
        type: Object,
        notify: true
      }
    };
  }
  static get api() {
    return ["evaluate", "flush"];
  }
  static get autoBinding() {
    return {
      signal: "evaluate-ended",
      field: "evaluate"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("hidden", "hidden");
    afterNextRender(this, function() {
      if (this.parentElement) {
        this.evaluate(this.parentElement);
      }
    });
  }
  flush() {
    this.__flushDebouncer = Debouncer.debounce(this.__flushDebouncer, animationFrame, () => {
      this.evaluate();
    });
  }
  evaluate(overrideEl, argument) {
    const el = overrideEl || this.parentElement;
    const field = this.field;
    if (!(el && field)) {
      return "";
    }
    argument = argument != null ? argument : this.argument != null ? this.argument : void 0;
    const value = this.value;
    const selfInstance = argument;
    const expressionValue = HIP.kernel.resolveExpression(value, this, selfInstance);
    if (expressionValue == null) {
      return "";
    }
    const isPureExpression = this.children.length <= 0;
    if (isPureExpression) {
      this.__assign(el, expressionValue);
      return expressionValue;
    }
    const conditionValue = argument != null ? argument : expressionValue;
    const safeValue = HIP.kernel.applyConditions(this.children, conditionValue, this);
    if (safeValue == null) {
      if (this.coercion == "boolean") {
        this.__assign(el, null);
      }
      return "";
    }
    const resolveSafeValue = HIP.kernel.resolveExpression(expressionValue, this, selfInstance);
    if (resolveSafeValue != null) {
      this.__assign(el, resolveSafeValue);
    }
    return safeValue;
  }
  __assign(el, _value) {
    const value = HIP.kernel.coerce(_value, this.coercion);
    const field = this.field;
    HIP.kernel.assign(el, field, value);
    this._dispatchModification(el, {
      field,
      value
    });
  }
  toString() {
    return `${this.field} = ${this.value};`;
  }
}
HIP.factory.element(ExpressionElement);
