
import { html, nothing } from '../../../hip/library/lit-html/lit-html.js';
import { setAttributes } from '../../../eip/element/hi-lit/hi-lit.js';
import { styleMap } from '../../../hip/library/lit-html/directives/style-map.js';
import $ip from '../../../pip/library/pi-ip/pi-ip.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
$ip.di.register("control", "button", {
  id: "factory-control-button",
  render(field, factory) {
    const ns = factory.ns;
    return (value, model) => {
      const { title, hint } = field;
      const { icon = "" } = f1.read("/settings", {}, field);
      return html`<hi-button
        data-attrs=${setAttributes(field.attributes)}
        style=${styleMap(f1.read("/style", {}, field))}
        .ns=${ns}
        .value=${value}
        .model=${model}
        @trigger-ended=${(e) => factory.retargetFieldEvent(e, field)}
      >
        ${icon ? html`<hi-icon glyph=${icon} type="material"></hi-icon>` : nothing}
        <span>${title}</span>
        ${hint ? html`<hi-hint>${hint}</hi-hint>` : nothing}
      </hi-button>`;
    };
  }
});
$ip.di.register("control", "link", {
  id: "factory-control-link",
  render(field, factory) {
    return (value, model) => {
      const { title, hint } = field;
      const { icon = "", href = "" } = f1.read("/settings", {}, field);
      return html`<hi-link
        href=${href}
        data-attrs=${setAttributes(field.attributes)}
        style=${styleMap(f1.read("/style", {}, field))}
        .ns=${factory.ns}
        .value=${value}
        .model=${model}
        @trigger-ended=${(e) => factory.retargetFieldEvent(e, field)}
      >
        ${icon ? html`<hi-icon glyph=${icon} type="material"></hi-icon>` : nothing}
        <span>${title}</span>
        ${hint ? html`<hi-icon glyph="info" type="material"><hi-hint>${hint}</hi-hint></hi-icon>` : nothing}
      </hi-link>`;
    };
  }
});
$ip.di.register("control", "select", {
  id: "factory-control-select",
  render(field, factory) {
    const options = f1.read("/source/out", [], field);
    const { multiple = false } = f1.read("/settings", {}, field);
    return (value, model) => {
      return html`<hi-dropdown kind="value" .ns=${factory.ns}>
        <hi-select
          mode="value"
          data-attrs=${setAttributes(field.attributes)}
          style=${styleMap(f1.read("/style", {}, field))}
          .ns=${factory.ns}
          .value=${value}
          .model=${model}
          ?multiple=${multiple}
          @value-changed=${(e) => factory.retargetFieldEvent(e, field)}
        >
          ${options.map((option) => html`<hi-option .value=${option.value}> ${option.title} </hi-option>`)}
        </hi-select>
      </hi-dropdown>`;
    };
  }
});
$ip.di.register("control", "menu", {
  id: "factory-control-menu",
  render(field, factory) {
    const ns = factory.ns;
    const { title, hint } = field;
    const options = f1.read("/source/out", [], field);
    const { icon = "" } = f1.read("/settings", {}, field);
    function buttonControl__triggerEnded(e) {
      this.nextElementSibling?.activate("", e);
    }
    return (value, model) => {
      return html`
        <hi-button
          .ns=${ns}
          data-attrs=${setAttributes(field.attributes)}
          style=${styleMap(f1.read("/style", {}, field))}
          @trigger-ended=${buttonControl__triggerEnded}
        >
          ${icon ? html`<hi-icon glyph=${icon} type="material"></hi-icon>` : nothing}
          <span>${title}</span>
          ${hint ? html`<hi-hint>${hint}</hi-hint>` : nothing}
        </hi-button>
        <hi-menu
          cap-height
          data-attrs=${setAttributes(field.attributes)}
          style=${styleMap(f1.read("/style", {}, field))}
          .ns=${factory.ns}
          .value=${value}
          .model=${model}
          @selected-ended=${(e) => factory.retargetFieldEvent(e, field)}
        >
          ${options.map((option) => {
        return html`<hi-pick value=${option.value}> ${option.title} </hi-pick>`;
      })}
        </hi-menu>
      `;
    };
  }
});
