
import { setAttributes, writeAlways } from '../../../eip/element/hi-lit/hi-lit.js';
import { ifDefined as ifDef } from '../../../hip/library/lit-html/directives/if-defined.js';
import { styleMap } from '../../../hip/library/lit-html/directives/style-map.js';
import { repeat } from '../../../hip/library/lit-html/directives/repeat.js';
import { html } from '../../../hip/library/lit-html/lit-html.js';
import $ip from '../../../pip/library/pi-ip/pi-ip.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
$ip.di.register("edit", "combobox", {
  id: "factory-edit-combobox",
  render(field, factory) {
    const descriptionId = field.description ? `form_select__desc__${field.name}` : void 0;
    return (value) => {
      const options = f1.read("/source/out", [], field);
      const { multiple, dynamic = false, placeholder = "" } = f1.read("/settings", {}, field);
      const maxLength = f1.safe(0, field.maximum_length);
      const isMultiple = f1.is.boolean(multiple) ? multiple : maxLength >= 2 || maxLength <= -1;
      const vals = isMultiple ? f1.is.array(value) ? value : f1.compact([value]) : void 0;
      const val = isMultiple ? void 0 : f1.is.array(value) ? f1.nth(0, value) : value;
      const kind = dynamic ? "dynamic" : "static";
      return html`<hi-combobox
        kind=${kind}
        form=${factory.form}
        name=${factory.namePrefix + field.name}
        model-path=${factory.namePrefix + field.model_path}
        max=${ifDef(field.maximum_length)}
        placeholder=${ifDef(placeholder)}
        aria-labelledby=${ifDef(descriptionId)}
        data-attrs=${setAttributes(field.attributes)}
        style=${styleMap(f1.read("/style", {}, field))}
        .values=${writeAlways(vals)}
        .value=${writeAlways(val)}
        .input=${options}
        .ns=${factory.ns}
        ?required=${field.required}
        ?nullable=${field.nullable}
        ?blankable=${field.blankable}
        ?multiple=${isMultiple}
        @commit-ended=${(e) => factory.retargetFieldEvent(e, field)}
        @search-value-changed=${(e) => factory.retargetFieldStateEvent(e, field, "search_value")}
      ></hi-combobox>`;
    };
  }
});
["select", "select-tags"].forEach((renderType) => {
  $ip.di.register("edit", renderType, {
    id: `factory-edit-${renderType}`,
    render(field, factory) {
      const ns = factory.ns;
      const descriptionId = field.description ? `form_select__desc__${field.name}` : void 0;
      return (value) => {
        const options = f1.read("/source/out", [], field);
        const {
          iterator = "select",
          placeholder = "",
          generator,
          multiple
        } = f1.read("/settings", {}, field);
        const maxLength = f1.safe(0, field.maximum_length);
        const isMultiple = f1.is.boolean(multiple) ? multiple : maxLength >= 2 || maxLength <= -1;
        const input = generator ? void 0 : options;
        return html`<hi-dropdown
          placeholder=${ifDef(placeholder)}
          name=${factory.namePrefix + field.name}
          model-path=${factory.namePrefix + field.model_path}
          aria-labelledby=${ifDef(descriptionId)}
          data-attrs=${setAttributes(field.attributes)}
          style=${styleMap(f1.read("/style", {}, field))}
          kind=${isMultiple ? "sticky" : "value"}
          ?disabled=${field.read_only}
          cap-height
          exclude
          .ns=${ns}
          ?required=${field.required}
        >
          <hi-select
            only-value
            mode="value"
            allow-key-search
            form=${factory.form}
            name=${factory.namePrefix + field.name}
            model-path=${factory.namePrefix + field.model_path}
            .value=${writeAlways(value)}
            .ns=${ns}
            ?multiple=${isMultiple}
            ?required=${field.required}
            ?nullable=${field.nullable}
            ?blankable=${field.blankable}
            ?readonly=${field.read_only}
            @commit-ended=${(e) => factory.retargetFieldEvent(e, field)}
          >
            <hi-iterator .generator=${generator} .iterator=${iterator} .input=${input} .ns=${ns}></hi-iterator>
          </hi-select>
        </hi-dropdown>`;
      };
    }
  });
});
["boolean", "bi-state", "tri-state"].forEach((render_type) => {
  const kind = render_type === "tri-state" ? "tri-state" : "bi-state";
  $ip.di.register("edit", render_type, {
    id: "factory-edit-boolean",
    render: (field, factory) => {
      const descriptionId = field.description ? `form_${render_type}__desc__${field.name}` : void 0;
      return (value) => {
        const indeterminate = field.render_type == "tri-state" && !f1.is.boolean(value);
        return html`<hi-checkbox
          form=${factory.form}
          name=${factory.namePrefix + field.name}
          model-path=${factory.namePrefix + field.model_path}
          kind=${kind}
          aria-labelledby=${ifDef(descriptionId)}
          data-attrs=${setAttributes(field.attributes)}
          style=${styleMap(f1.read("/style", {}, field))}
          .value=${writeAlways(value)}
          .ns=${factory.ns}
          ?nullable=${field.nullable}
          ?blankable=${field.blankable}
          ?required=${field.required}
          ?readonly=${field.read_only}
          ?indeterminate=${indeterminate}
          @commit-ended=${(e) => factory.retargetFieldEvent(e, field)}
        >
        </hi-checkbox>`;
      };
    }
  });
});
$ip.di.register("edit", "toggle", {
  id: "factory-edit-toggle",
  render: (field, factory) => {
    const descriptionId = field.description ? `form_toggle__desc__${field.name}` : void 0;
    return (value) => {
      return html`<hi-toggle
        form=${factory.form}
        name=${factory.namePrefix + field.name}
        model-path=${factory.namePrefix + field.model_path}
        aria-labelledby=${ifDef(descriptionId)}
        data-attrs=${setAttributes(field.attributes)}
        style=${styleMap(f1.read("/style", {}, field))}
        .on=${writeAlways(value)}
        .ns=${factory.ns}
        ?required=${field.required}
        ?nullable=${field.nullable}
        ?blankable=${field.blankable}
        ?readonly=${field.read_only}
        @commit-ended=${(e) => factory.retargetFieldEvent(e, field)}
      >
      </hi-toggle>`;
    };
  }
});
$ip.di.register("edit", "radio", {
  id: "factory-edit-radio",
  render(field, factory) {
    const descriptionId = field.description ? `form_radio__desc__${field.name}` : void 0;
    const group = `radio-group-${factory.form}-${field.name}`;
    const ns = factory.ns;
    return (value) => {
      const options = f1.read("/source/out", [], field);
      const {} = f1.read("/settings", {}, field);
      return html`<hi-radio-group
        form=${factory.form}
        name=${factory.namePrefix + field.name}
        model-path=${factory.namePrefix + field.model_path}
        group=${group}
        aria-labelledby=${ifDef(descriptionId)}
        data-attrs=${setAttributes(field.attributes)}
        style=${styleMap(f1.read("/style", {}, field))}
        .value=${writeAlways(value)}
        .ns=${ns}
        ?required=${field.required}
        ?nullable=${field.nullable}
        ?blankable=${field.blankable}
        ?readonly=${field.read_only}
        @commit-ended=${(e) => factory.retargetFieldEvent(e, field)}
      >
        ${repeat(
        options,
        (opt) => opt.value,
        (opt) => html`
            <hi-radio group=${group} .ns=${ns} .value=${opt.value}></hi-radio>
            <hi-label .ns=${ns}> ${opt.title} </hi-label>
          `
      )}
      </hi-radio-group>`;
    };
  }
});
