
import { setAttributes, writeAlways } from '../../../eip/element/hi-lit/hi-lit.js';
import { ifDefined as ifDef } from '../../../hip/library/lit-html/directives/if-defined.js';
import { styleMap } from '../../../hip/library/lit-html/directives/style-map.js';
import { html, nothing } from '../../../hip/library/lit-html/lit-html.js';
import { renderInput } from './ai-edit-support.js';
import $ip from '../../../pip/library/pi-ip/pi-ip.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
["iso_date", "date"].forEach((renderType) => {
  $ip.di.register("edit", renderType, {
    id: `factory-edit-${renderType}`,
    render(field, factory) {
      const { ns, form } = factory;
      const {
        placeholder = "Select a date",
        min,
        max,
        format_type,
        disallow_initial_value,
        disabled_below_min,
        disabled_above_max,
        highlight_dates
      } = f1.read("/settings", {}, field);
      return (value) => {
        return html` <hi-dropdown
          kind="value"
          hug-width
          form=${form}
          name=${factory.namePrefix + field.name}
          model-path=${factory.namePrefix + field.model_path}
          placeholder=${ifDef(placeholder)}
          data-attrs=${setAttributes(field.attributes)}
          style=${styleMap(f1.read("/style", {}, field))}
          .ns=${ns}
          ?required=${field.required}
        >
          <hi-datepicker
            min=${ifDef(min)}
            max=${ifDef(max)}
            format-type=${ifDef(format_type)}
            ?disabled-below-min=${disabled_below_min}
            ?disabled-above-max=${disabled_above_max}
            ?disallow-initial-value=${disallow_initial_value}
            ?required=${field.required}
            ?nullable=${field.nullable}
            ?blankable=${field.blankable}
            ?readonly=${field.read_only}
            .value=${writeAlways(value)}
            .highlightDates=${writeAlways(highlight_dates)}
            @commit-ended=${(e) => factory.retargetFieldEvent(e, field)}
          >
          </hi-datepicker>
        </hi-dropdown>`;
      };
    }
  });
});
["iso_time", "time"].forEach((renderType) => {
  $ip.di.register("edit", renderType, {
    id: `factory-edit-${renderType}`,
    render(field, factory) {
      const { ns, form } = factory;
      const {
        placeholder = "",
        time_start = "12:00 AM",
        time_end = "12:00 PM",
        time_step = "15",
        time_zone = ""
      } = f1.read("/settings", {}, field);
      return (value) => {
        return html` <hi-dropdown
          placeholder=${ifDef(placeholder)}
          name=${factory.namePrefix + field.name}
          model-path=${factory.namePrefix + field.model_path}
          data-attrs=${setAttributes(field.attributes)}
          style=${styleMap(f1.read("/style", {}, field))}
          form=${form}
          kind="value"
          cap-height
          .ns=${ns}
        >
          <hi-select
            only-value
            mode="value"
            allow-key-search
            form=${form}
            .value=${writeAlways(value)}
            .ns=${ns}
            ?required=${field.required}
            ?nullable=${field.nullable}
            ?blankable=${field.blankable}
            ?readonly=${field.read_only}
            @commit-ended=${(e) => factory.retargetFieldEvent(e, field)}
          >
            <hi-times
              form=${form}
              .ns=${ns}
              .value=${writeAlways(value)}
              time-start="${time_start}"
              time-step="${time_step}"
              time-end="${time_end}"
              time-zone=${ifDef(time_zone)}
            ></hi-times>
          </hi-select>
        </hi-dropdown>`;
      };
    }
  });
});
["iso_date_time", "date_time"].forEach((renderType) => {
  $ip.di.register("edit", renderType, {
    id: `factory-edit-${renderType}`,
    render(field, factory) {
      const { mask } = f1.read("/settings", {}, field);
      const { time_zone } = f1.read("/settings", {}, field);
      const timeZoneRule = time_zone ? f1.read("/time_zone", {}, field.settings) : void 0;
      const maskRule = mask ? html`<hi-validation-mask rule=${mask}></hi-validation-mask>` : nothing;
      return (value) => {
        return renderInput(
          value,
          field,
          factory,
          {
            step: 60,
            type: "date_time",
            "time-zone": timeZoneRule
          },
          maskRule
        );
      };
    }
  });
});
const listGenerator = {
  year: (settings) => {
    return f1.range(1, f1.read("/min_year", 1930, settings), f1.read("/max_year", 2050, settings));
  },
  day: (settings) => {
    return f1.range(1, f1.read("/min_day", 1, settings), f1.read("/max_day", 31, settings));
  },
  month: () => {
    return [
      "January",
      "February",
      "March",
      "April",
      "May",
      "June",
      "July",
      "August",
      "September",
      "October",
      "November",
      "December"
    ].map((name, value) => ({
      name,
      value
    }));
  }
};
["year", "month", "day"].forEach((render_type) => {
  $ip.di.register("edit", render_type, {
    id: `factory-edit-${render_type}`,
    render(field, factory) {
      const settings = f1.read("/settings", {}, field);
      const { ns, form } = factory;
      const { placeholder = f1.word(`Select a ${render_type}`), multiple = false } = settings;
      const listFn = listGenerator[render_type];
      const list = listFn(settings);
      return (value) => {
        return html`<hi-dropdown
          placeholder=${ifDef(placeholder)}
          name=${factory.namePrefix + field.name}
          model-path=${factory.namePrefix + field.model_path}
          data-attrs=${setAttributes(field.attributes)}
          style=${styleMap(f1.read("/style", {}, field))}
          form=${form}
          .ns=${ns}
          kind="value"
          cap-height
          hug-width
        >
          <hi-select
            only-value
            mode="value"
            allow-key-search
            form=${form}
            .ns=${ns}
            .value=${writeAlways(value)}
            ?multiple=${multiple}
            ?required=${field.required}
            ?nullable=${field.nullable}
            ?blankable=${field.blankable}
            ?readonly=${field.read_only}
            @commit-ended=${(e) => factory.retargetFieldEvent(e, field)}
          >
            ${list.map((year) => html` <hi-option value="${year}">${year}</hi-option>`)}
          </hi-select>
        </hi-dropdown>`;
      };
    }
  });
});
