
import { setAttributes, writeAlways } from '../../../eip/element/hi-lit/hi-lit.js';
import { ifDefined as ifDef } from '../../../hip/library/lit-html/directives/if-defined.js';
import { styleMap } from '../../../hip/library/lit-html/directives/style-map.js';
import { html } from '../../../hip/library/lit-html/lit-html.js';
import { renderAttributes } from './ai-edit-support.js';
import $ip from '../../../pip/library/pi-ip/pi-ip.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
$ip.di.register("edit", "code", {
  id: `factory-edit-code`,
  render(field, factory) {
    return (value) => {
      const { language = "json" } = f1.read("/settings", {}, field);
      return html`<hi-code
        form=${factory.form}
        name=${factory.namePrefix + field.name}
        model-path=${factory.namePrefix + field.model_path}
        data-attrs=${setAttributes(renderAttributes(field))}
        style=${styleMap(f1.read("/style", {}, field))}
        .value=${writeAlways(f1.safe("", value))}
        .language=${language}
        .ns=${factory.ns}
        ?required=${!!field.required}
        ?readonly=${!!field.read_only}
        ?nullable=${!!field.nullable}
        ?blankable=${field.blankable}
        @commit-ended=${(e) => factory.retargetFieldEvent(e, field)}
      ></hi-code>`;
    };
  }
});
$ip.di.register("edit", "content", {
  id: `factory-edit-content`,
  render(field, factory) {
    return (value) => {
      return html`<hi-content
        .html=${value}
        data-attrs=${setAttributes(field.attributes)}
        style=${styleMap(f1.read("/style", {}, field))}
      ></hi-content>`;
    };
  }
});
["editor", "html", "html_text"].forEach((render_type) => {
  $ip.di.register("edit", render_type, {
    id: `factory-edit-${render_type}`,
    render(field, factory) {
      return (value) => {
        const descriptionId = field.description ? `form_textarea__desc__${field.name}` : void 0;
        const { mode = "minimal" } = f1.read("/settings", {}, field);
        return html`<hi-editor
          form=${factory.form}
          name=${factory.namePrefix + field.name}
          model-path=${factory.namePrefix + field.model_path}
          aria-labelledby=${ifDef(descriptionId)}
          data-attrs=${setAttributes(renderAttributes(field))}
          style=${styleMap(f1.read("/style", {}, field))}
          .value=${writeAlways(value)}
          .ns=${factory.ns}
          .mode=${mode}
          ?required=${!!field.required}
          ?readonly=${!!field.read_only}
          ?nullable=${!!field.nullable}
          ?blankable=${field.blankable}
          @commit-ended=${(e) => factory.retargetFieldEvent(e, field)}
        ></hi-editor>`;
      };
    }
  });
});
