
import { setAttributes, writeAlways } from '../../../eip/element/hi-lit/hi-lit.js';
import { ifDefined as ifDef } from '../../../hip/library/lit-html/directives/if-defined.js';
import { styleMap } from '../../../hip/library/lit-html/directives/style-map.js';
import { html, nothing } from '../../../hip/library/lit-html/lit-html.js';
import { renderInput, renderAttributes } from './ai-edit-support.js';
import $ip from '../../../pip/library/pi-ip/pi-ip.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
[
  "datetime-local",
  "username",
  "password",
  "integer",
  "decimal",
  "number",
  "hidden",
  "search",
  "text",
  "week",
  "badge"
].forEach((render_type) => {
  $ip.di.register("edit", render_type, {
    id: `factory-edit-${render_type}`,
    render(field, factory) {
      const { mask } = f1.read("/settings", {}, field);
      const maskRule = mask ? html`<hi-validation-mask rule=${mask}></hi-validation-mask>` : nothing;
      return (value) => {
        return renderInput(value, field, factory, {}, maskRule);
      };
    }
  });
});
$ip.di.register("edit", "currency", {
  id: `factory-edit-currency`,
  render(field, factory) {
    return (value) => {
      return renderInput(value, field, factory, {
        step: 0.01
      });
    };
  }
});
$ip.di.register("edit", "percentage", {
  id: `factory-edit-percentage`,
  render(field, factory) {
    return (value) => {
      return renderInput(value, field, factory, {
        step: 1
      });
    };
  }
});
$ip.di.register("edit", "telephone", {
  id: `factory-edit-telephone`,
  render(field, factory) {
    return (value) => {
      return renderInput(
        value,
        field,
        factory,
        {},
        html`<hi-validation-mask rule="(&&&) &&&-&&&&"></hi-validation-mask>`
      );
    };
  }
});
$ip.di.register("edit", "email", {
  id: `factory-edit-email`,
  render(field, factory) {
    const pattern = f1.patterns.email.source;
    return (value) => {
      return renderInput(value, field, factory, {
        placeholder: "example@example.com",
        pattern
      });
    };
  }
});
["url", "link"].forEach((render_type) => {
  $ip.di.register("edit", render_type, {
    id: `factory-edit-${render_type}`,
    render(field, factory) {
      const pattern = f1.patterns.url.source;
      return (value) => {
        return renderInput(value, field, factory, {
          placeholder: "https://example.com",
          pattern
        });
      };
    }
  });
});
$ip.di.register("edit", "unknown", {
  id: `factory-edit-unknown`,
  render(field, factory) {
    return (value) => {
      return renderInput(value, field, factory, {
        type: "text"
      });
    };
  }
});
["textarea", "text_list"].forEach((render_type) => {
  $ip.di.register("edit", render_type, {
    id: `factory-edit-${render_type}`,
    render(field, factory) {
      return (value) => {
        const descriptionId = field.description ? `form_${render_type}__desc__${field.name}` : void 0;
        return html`<hi-textarea
          form=${factory.form}
          name=${factory.namePrefix + field.name}
          model-path=${factory.namePrefix + field.model_path}
          aria-labelledby=${ifDef(descriptionId)}
          data-attrs=${setAttributes(renderAttributes(field))}
          style=${styleMap(f1.read("/style", {}, field))}
          .value=${writeAlways(value)}
          .ns=${factory.ns}
          ?required=${!!field.required}
          ?readonly=${!!field.read_only}
          ?nullable=${!!field.nullable}
          ?blankable=${field.blankable}
          @commit-ended=${(e) => factory.retargetFieldEvent(e, field)}
        ></hi-textarea>`;
      };
    }
  });
});
["dictionary", "array"].forEach((render_type) => {
  $ip.di.register("edit", render_type, {
    id: `factory-edit-${render_type}`,
    render(field, factory) {
      return (value) => {
        const fallbackValue = render_type === "dictionary" ? f1.safe({}, value) : f1.safe([], value);
        return html`<hi-input-list
          form=${factory.form}
          name=${factory.namePrefix + field.name}
          model-path=${factory.namePrefix + field.model_path}
          data-attrs=${setAttributes(renderAttributes(field))}
          style=${styleMap(f1.read("/style", {}, field))}
          kind=${render_type}
          .value=${writeAlways(fallbackValue)}
          .ns=${factory.ns}
          ?required=${!!field.required}
          ?readonly=${!!field.read_only}
          @commit-ended=${(e) => {
          if (e.detail.srcElement.localName == "hi-input-list") {
            return factory.retargetFieldEvent(e, field);
          }
          return;
        }}
        ></hi-input-list>`;
      };
    }
  });
});
