
import { setAttributes } from '../../../eip/element/hi-lit/hi-lit.js';
import { ifDefined as ifDef } from '../../../hip/library/lit-html/directives/if-defined.js';
import { styleMap } from '../../../hip/library/lit-html/directives/style-map.js';
import { repeat } from '../../../hip/library/lit-html/directives/repeat.js';
import { nothing, html } from '../../../hip/library/lit-html/lit-html.js';
import $ip from '../../../pip/library/pi-ip/pi-ip.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
const R = {
  findAllSlashes: /\//g
};
const services = {
  commitValueAndUpdate(event, factory, field, value) {
    f1.write("/value/out", value, field);
    const formFactory = factory["parentFactory"];
    if (!formFactory) {
      return;
    }
    formFactory.retargetFieldEvent(event, field);
    formFactory.requestUpdate("type");
  },
  factorySignature(prefix, value, field, factory) {
    const subForm = `form--${prefix}--${(factory.namePrefix || "name_prefix").replace(R.findAllSlashes, "-")}--${factory.ns || "ns"}--${field.name}`;
    const childOf = `${subForm}--${value}`;
    return {
      subForm,
      childOf
    };
  }
};
$ip.di.register("edit", "toggle-form", {
  id: "factory-edit-toggle-form",
  is_render_control: true,
  renderControlled(field, factory) {
    const descriptionId = field.description ? `form_select_form__desc__${field.name}` : void 0;
    const select__onCommitEnded = (e) => {
      services.commitValueAndUpdate(e, factory, field, e.detail.value);
    };
    return (value, valueModel) => {
      const { subForm, childOf } = services.factorySignature("toggle-form", value, field, factory);
      const { alignment = "", clear_values = false } = f1.read("/settings", {}, field);
      const typeHeader = f1.read("/title", f1.word(field.name), field);
      const fields = f1.read("/fields", f1.read("/source/out", {}, field), field);
      const type = {
        fields
      };
      return {
        input: html`<hi-toggle
          form=${factory.form}
          name=${factory.namePrefix + field.name}
          model-path=${factory.namePrefix + field.model_path}
          ?required=${field.required}
          .ns=${factory.ns}
          .on=${value}
          aria-labelledby=${ifDef(descriptionId)}
          @commit-ended=${select__onCommitEnded}
        >
        </hi-toggle>`,
        controlled: html`${!value ? nothing : html`<hi-fieldset
                gid=${subForm}
                heading=${typeHeader}
                .ns=${factory.ns}
                .alignment=${alignment}
              ></hi-fieldset>
              <ai-factory-editor
                for=${subForm}
                form=${factory.form}
                gid="force-paint"
                .model=${factory.model}
                .value=${valueModel}
                .ns=${factory.ns}
                .type=${type}
                .valueSource=${factory.valueSource}
                .namePrefix=${factory.namePrefix}
                .childOf=${childOf}
                ?clear-values=${clear_values}
                @field-trigger-ended=${(e) => factory.proxyRetargetFieldEvent(e)}
                @field-state-trigger-ended=${(e) => factory.proxyRetargetFieldStateEvent(e)}
              ></ai-factory-editor>`}`
      };
    };
  }
});
$ip.di.register("edit", "select-form", {
  id: "factory-edit-select-form",
  is_render_control: true,
  renderControlled(field, factory) {
    const descriptionId = field.description ? `form_select_form__desc__${field.name}` : void 0;
    const fields = f1.read("/fields", f1.read("/source/out", {}, field), field);
    const options = f1.to.array(
      f1.map((type, typeKey) => {
        return {
          title: f1.read("/title", f1.word(typeKey), type),
          value: typeKey
        };
      }, fields)
    );
    const select__onCommitEnded = (e) => {
      services.commitValueAndUpdate(e, factory, field, e.detail.value);
    };
    return (value, valueModel) => {
      const { subForm, childOf } = services.factorySignature("select-form", value, field, factory);
      const {
        placeholder = "",
        alignment = "",
        clear_values = false
      } = f1.read("/settings", {}, field);
      const finalModelPath = factory.namePrefix + field.model_path;
      const valueType = f1.safe({ fields: {} }, f1.read(f1.path(value), void 0, fields));
      const subFields = f1.read("/fields", f1.read("/source/out", {}, valueType), valueType);
      const selectedType = {
        fields: subFields
      };
      const typeHeader = f1.read("/title", f1.word(value), selectedType);
      const fieldExists = f1.read("/fields", void 0, selectedType);
      return {
        input: html`<hi-dropdown
          placeholder=${ifDef(placeholder)}
          name=${factory.namePrefix + field.name}
          model-path=${finalModelPath}
          kind="value"
          cap-height
          exclude
          aria-labelledby=${ifDef(descriptionId)}
          data-attrs=${setAttributes(field.attributes)}
          style=${styleMap(f1.read("/style", {}, field))}
          .ns=${factory.ns}
          ?required=${field.required}
        >
          <hi-select
            form=${factory.form}
            name=${factory.namePrefix + field.name}
            model-path=${finalModelPath}
            mode="value"
            only-value
            ?required=${field.required}
            .ns=${factory.ns}
            .value=${value}
            @commit-ended=${select__onCommitEnded}
          >
            ${repeat(
          options,
          (opt) => opt.value,
          (opt) => html`<hi-option .value=${opt.value}> ${opt.title} </hi-option>`
        )}
          </hi-select>
        </hi-dropdown>`,
        controlled: html`${f1.is.empty(fieldExists) ? nothing : html` <hi-fieldset
                gid=${subForm}
                heading=${typeHeader}
                .alignment=${alignment}
                .ns=${factory.ns}
              ></hi-fieldset>
              <ai-factory-editor
                for=${subForm}
                form=${factory.form}
                .model=${factory.model}
                .value=${valueModel}
                .ns=${factory.ns}
                .type=${selectedType}
                .namePrefix=${factory.namePrefix}
                .valueSource=${factory.valueSource}
                .childOf=${childOf}
                ?clear-values=${clear_values}
                @field-trigger-ended=${(e) => factory.proxyRetargetFieldEvent(e)}
                @field-state-trigger-ended=${(e) => factory.proxyRetargetFieldStateEvent(e)}
              ></ai-factory-editor>`}`
      };
    };
  }
});
$ip.di.register("edit", "sub-form", {
  id: "factory-edit-sub-form",
  is_render_isolated: true,
  render(field, factory) {
    return (value, valueModel) => {
      const { subForm, childOf } = services.factorySignature("sub-form", value, field, factory);
      const fields = f1.read("/fields", f1.read("/source/out", {}, field), field);
      const typeHeader = f1.read("/title", f1.word(field.name), field);
      const {} = f1.read("/settings", {}, field);
      const type = {
        fields
      };
      return html`
        <hi-fieldset heading=${typeHeader}>
          ${factory.renderDescription(field)}
          <hi-panel gid=${subForm} .ns=${factory.ns}></hi-panel>
        </hi-fieldset>
        <ai-factory-editor
          for=${subForm}
          form=${factory.form}
          .ns=${factory.ns}
          .type=${type}
          .childOf=${childOf}
          .value=${valueModel}
          .model=${factory.model}
          .valueSource=${factory.valueSource}
          .namePrefix=${factory.namePrefix}
          @field-trigger-ended=${(e) => factory.proxyRetargetFieldEvent(e)}
          @field-state-trigger-ended=${(e) => factory.proxyRetargetFieldStateEvent(e)}
        ></ai-factory-editor>
      `;
    };
  }
});
$ip.di.register("edit", "list-form", {
  id: "factory-edit-list-form",
  is_render_isolated: true,
  render(field, factory) {
    const fields = f1.read("/fields", f1.read("/source/out", {}, field), field);
    const typeHeader = f1.read("/title", f1.word(field.name), field);
    const settings = f1.safe({}, field.settings);
    const type = {
      fields
    };
    let list = void 0;
    let valuePtr = void 0;
    const removeFromForm = (event) => {
      if (!list) {
        return;
      }
      list.splice(event.detail.value, 1);
      event.detail.value = list;
      services.commitValueAndUpdate(event, factory, field, list);
    };
    const addToForm = (event) => {
      if (!list) {
        return;
      }
      const newObject = f1.clone(f1.read("/item_template", f1.copy({}), settings));
      list.push(newObject);
      event.detail.value = list;
      services.commitValueAndUpdate(event, factory, field, list);
    };
    return (value, valueModel) => {
      list = f1.safe(value, list);
      if (valuePtr && valuePtr !== value) {
        list = value;
      }
      list = f1.safe([], list);
      valuePtr = list;
      return html`
        <hi-fieldset heading=${typeHeader}>
          ${factory.renderDescription(field)}
          <hi-accordion
            mode="allow-any-opened"
            class="mtm"
            .ns=${factory.ns}
            data-attrs=${setAttributes(field.attributes)}
            style=${styleMap(f1.read("/style", {}, field))}
          >
            ${list.map((entity, index) => {
        const subNamePrefix = `${factory.namePrefix}${field.model_path}/${index}/`;
        const { subForm, childOf } = services.factorySignature(
          `form-list-${subNamePrefix.replace(/\//g, "-")}`,
          value,
          field,
          factory
        );
        const itemTitle = f1.is.in("item_title_path", settings) ? f1.read(settings.item_title_path, `${typeHeader} (${index})`, entity) : `${typeHeader} (${index})`;
        return html` <hi-bar ?opened=${0 <= index} .ns=${factory.ns}>${itemTitle}</hi-bar>
                <hi-note .ns=${factory.ns} allow-overflow>
                  <hi-panel gid="${subForm}" .ns=${factory.ns}></hi-panel>
                  <div class="mtm">
                    <hi-button
                      size="small"
                      aesthetic="secondary"
                      theme="orange"
                      @trigger-ended=${removeFromForm}
                      .value=${index}
                      .ns=${factory.ns}
                    >
                      Remove <span class="hide-ax">${typeHeader} #${index}</span>
                    </hi-button>
                  </div>
                  <ai-factory-editor
                    for=${subForm}
                    form=${factory.form}
                    .type=${type}
                    .value=${entity}
                    .ns=${factory.ns}
                    .childOf=${childOf}
                    .model=${factory.model}
                    .valueSource=${factory.valueSource}
                    .namePrefix=${subNamePrefix}
                    @field-trigger-ended=${(e) => services.commitValueAndUpdate(e, factory, field, list)}
                  ></ai-factory-editor>
                </hi-note>`;
      })}
          </hi-accordion>
          <hi-button size="small" class="mtm" aesthetic="secondary" .ns=${factory.ns} @trigger-ended=${addToForm}>
            Add ${typeHeader}
          </hi-button>
        </hi-fieldset>
      `;
    };
  }
});
