
import { LitElement } from '../../../hip/library/lit-element/lit-element.js';
import { html, nothing } from '../../../hip/library/lit-html/lit-html.js';
import { AsLitHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import { BehaviorFactory } from '../../../aip/element/ai-behavior-factory/ai-behavior-factory.js';
import '../../../aip/element/ai-behavior-factory/ai-behavior-factory.js';
import '../../../eip/element/hi-button/hi-button.js';
import { FactoryEditElement } from '../../../aip/element/ai-factory-edit/ai-factory-edit.js';
import './ai-editor-artifact-container.js';
import adt from '../../../pip/library/pi-adt/pi-adt.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class FactoryEditorElement extends BehaviorFactory(BehaviorElement(AsLitHIC(LitElement))) {
  constructor() {
    super(...arguments);
    this.editFactory = new FactoryEditElement();
    this.groupingMutation = adt.mutation("grouping");
    this.includeSubmit = false;
    this.submitLabel = "Submit";
    this.includeReset = false;
    this.resetLabel = "Clear";
    this.includeCancel = false;
    this.cancelLabel = "Cancel";
    this.form = `factory-form-${f1.id()}`;
  }
  static get is() {
    return "ai-factory-editor";
  }
  static get properties() {
    return {
      includeSubmit: {
        type: Boolean,
        attribute: "include-submit"
      },
      submitLabel: {
        type: String,
        attribute: "submit-label"
      },
      includeReset: {
        type: Boolean,
        attribute: "include-reset"
      },
      resetLabel: {
        type: String,
        attribute: "reset-label"
      },
      includeCancel: {
        type: Boolean,
        attribute: "include-cancel"
      },
      cancelLabel: {
        type: String,
        attribute: "cancel-label"
      },
      form: {
        type: String
      }
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.__retarget__fieldTriggerEndedBound = this.__retarget__fieldTriggerEnded.bind(this);
    this.__retarget__fieldStateTriggerEndedBound = this.__retarget__fieldStateTriggerEnded.bind(this);
    this.__retarget__modelUpdateEndedBound = this.__retarget__modelUpdateEnded.bind(this);
    this.editFactory.addEventListener("field-trigger-ended", this.__retarget__fieldTriggerEndedBound);
    this.editFactory.addEventListener("field-state-trigger-ended", this.__retarget__fieldStateTriggerEndedBound);
    this.editFactory.addEventListener("model-update-ended", this.__retarget__modelUpdateEndedBound);
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    this.editFactory.removeEventListener("field-trigger-ended", this.__retarget__fieldTriggerEndedBound);
    this.editFactory.removeEventListener("field-state-trigger-ended", this.__retarget__fieldStateTriggerEndedBound);
    this.editFactory.removeEventListener("model-update-ended", this.__retarget__modelUpdateEndedBound);
  }
  __retarget__fieldTriggerEnded(e) {
    this._dispatchProcess("field-trigger", e.detail);
  }
  __retarget__fieldStateTriggerEnded(e) {
    this._dispatchProcess("field-state-trigger", e.detail);
  }
  __retarget__modelUpdateEnded(e) {
    this._dispatchProcess("model-update", e.detail);
  }
  _render(fields, type) {
    const ns = this.ns;
    const { none, groupings } = this.groupingMutation.options({
      tags: f1.read("/type/tags", {}, this)
    }).invoke(fields);
    return html`
      ${none.list.map((field) => this.__renderInputGroup(field))}
      ${groupings.length > 0 ? html`<hi-accordion mode="allow-any-opened" .ns=${ns} class="mtl">
            ${groupings.map((group, index) => {
      return html`
                <hi-bar ?opened=${0 === index} .ns=${ns}>${group.title}</hi-bar>
                <hi-note allow-overflow .ns=${ns}>
                  ${group.list.map((field) => this.__renderInputGroup(field))}
                </hi-note>
              `;
    })}
          </hi-accordion>` : nothing}
      ${this.__renderFormActionButtons()}
    `;
  }
  __renderFormActionButtons() {
    const ns = this.ns;
    const submitButton = this.includeSubmit ? html`<hi-button .ns=${ns} slot="controls" type="submit">${this.submitLabel}</hi-button>` : nothing;
    const resetButton = this.includeReset ? html`<hi-button .ns=${ns} aesthetic="secondary" slot="controls" type="reset">${this.resetLabel}</hi-button>` : nothing;
    const cancelButton = this.includeCancel ? html`<hi-button .ns=${ns} aesthetic="secondary" slot="controls" type="cancel">${this.cancelLabel}</hi-button>` : nothing;
    return html` ${submitButton}${cancelButton}${resetButton} `;
  }
  __renderInputGroup(field) {
    const inputTmpl = this.renderField(field);
    if (inputTmpl === nothing) {
      return nothing;
    }
    const renderFn = this.renderBy("edit", field.render_type);
    if (renderFn && renderFn.is_render_isolated) {
      return inputTmpl;
    }
    const isInputHidden = field.render_type === "hidden";
    const isInputRenderControl = renderFn && renderFn.is_render_control;
    return html`
      <hi-input-group ?hidden=${isInputHidden} form=${this.form} .ns=${this.ns}>
        ${this.__renderLabel(field)} ${isInputRenderControl ? inputTmpl["input"] : inputTmpl}
        ${this.renderDescription(field)}
      </hi-input-group>
      ${isInputRenderControl ? inputTmpl["controlled"] : nothing}
    `;
  }
  __renderLabel(field) {
    return html`
      <hi-label
        x-marker="field-label"
        ?required=${field.required}
        .ns=${this.ns}
        style="display: flex; align-items: center;"
      >
        <span class="prsxx">${field.title}</span>
        ${this._renderHint(field)} ${this._renderCopyable(field)} ${this.__renderClearable(field)}
      </hi-label>
    `;
  }
  __renderClearable(field) {
    if (!field.clearable || field.read_only) {
      return nothing;
    }
    return html`
      <hi-button
        size="small"
        aesthetic="icon"
        slot="controls"
        .ns=${this.ns}
        style="position: inherit;"
        @trigger-ended=${this.__clearableButton__onTriggerEnded}
      >
        <hi-icon glyph="layers_clear" type="material" size="small"></hi-icon>
        <hi-hint placement="right">Clear ${field.title}</hi-hint>
        <span class="show-ax">Clear ${field.title}</span>
      </hi-button>
    `;
  }
  __clearableButton__onTriggerEnded(e) {
    const inputElement = e.detail?.srcElement?.closest("hi-label")?.nextElementSibling;
    if (!inputElement) {
      return;
    }
    inputElement.clear();
  }
  renderField(field) {
    this.editFactory.namePrefix = this.namePrefix;
    this.editFactory.value = this.value;
    this.editFactory.model = this.model;
    this.editFactory.type = this.type;
    this.editFactory.form = this.form;
    this.editFactory.ns = this.ns;
    this.editFactory["parentFactory"] = this;
    return this.editFactory.renderField(field);
  }
}
export default HIP.factory.element(FactoryEditorElement);
