
import { LitElement } from '../../../hip/library/lit-element/lit-element.js';
import { html, nothing } from '../../../hip/library/lit-html/lit-html.js';
import { writeAlways } from '../../../eip/element/hi-lit/hi-lit.js';
import { AsLitHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import { BehaviorFactory } from '../../../aip/element/ai-behavior-factory/ai-behavior-factory.js';
import { FactoryEditElement } from '../../../aip/element/ai-factory-edit/ai-factory-edit.js';
import { FactoryViewElement } from '../../../aip/element/ai-factory-view/ai-factory-view.js';
import '../../../aip/element/ai-behavior-factory/ai-behavior-factory.js';
import '../../../eip/element/hi-inline-edit/hi-inline-edit.js';
import '../../../eip/element/hi-button/hi-button.js';
import '../../../eip/element/hi-label/hi-label.js';
import adt from '../../../pip/library/pi-adt/pi-adt.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class FactoryInlineElement extends BehaviorFactory(BehaviorElement(AsLitHIC(LitElement))) {
  constructor() {
    super(...arguments);
    this.editFactory = new FactoryEditElement();
    this.viewFactory = new FactoryViewElement();
    this.groupingMutation = adt.mutation("grouping");
    this.form = `factory-form-${f1.id()}`;
  }
  static get is() {
    return "ai-factory-inline";
  }
  static get properties() {
    return {
      form: {
        type: String
      }
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.__retarget__fieldTriggerEndedBound = this.__retarget__fieldTriggerEnded.bind(this);
    this.editFactory.addEventListener("field-trigger-ended", this.__retarget__fieldTriggerEndedBound);
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    this.editFactory.removeEventListener("field-trigger-ended", this.__retarget__fieldTriggerEndedBound);
  }
  __retarget__fieldTriggerEnded(e) {
    this._dispatchProcess("field-trigger", e.detail);
  }
  _render(fields, type) {
    const ns = this.ns;
    const { none, groupings } = this.groupingMutation.options({
      tags: f1.read("/type/tags", {}, this)
    }).invoke(fields);
    return html`
      <dl kind="normal">${none.list.map((field) => this.__renderDataItem(field))}</dl>

      ${groupings.length > 0 ? html`<hi-accordion mode="allow-any-opened" .ns=${ns} class="mtl">
            ${groupings.map((group, index) => {
      return html`
                <hi-bar ?opened=${0 <= index} .ns=${ns}>${group.title}</hi-bar>
                <hi-note allow-overflow .ns=${ns}> ${group.list.map((field) => this.__renderDataItem(field))} </hi-note>
              `;
    })}
          </hi-accordion>` : nothing}
    `;
  }
  __renderDataItem(field) {
    const inlineEditTmpl = this.renderField(field);
    if (inlineEditTmpl === nothing) {
      return nothing;
    }
    const isInputHidden = field.render_type === "hidden";
    return html`
      <hi-input-group ?hidden=${isInputHidden} form=${this.form} .ns=${this.ns}>
        ${this.__renderLabel(field)} ${inlineEditTmpl} ${this.renderDescription(field)}
      </hi-input-group>
    `;
  }
  __renderLabel(field) {
    return html`
      <hi-label
        x-marker="field-label"
        ?required=${field.required}
        .ns=${this.ns}
        style="display: flex; align-items: center;"
      >
        <span class="prsxx" style="margin-right: auto">${field.title}</span>
        ${this._renderHint(field)} ${this._renderCopyable(field)}
      </hi-label>
    `;
  }
  renderField(field) {
    const editField = f1.write("/attributes/slot", "edit", f1.clone(field));
    const viewField = f1.write("/attributes/slot", "display", f1.clone(field));
    this.editFactory.namePrefix = this.namePrefix;
    this.editFactory.value = this.value;
    this.editFactory.model = this.model;
    this.editFactory.type = this.type;
    this.editFactory.form = this.form;
    this.editFactory.ns = this.ns;
    this.editFactory["parentFactory"] = this;
    this.viewFactory.namePrefix = this.namePrefix;
    this.viewFactory.type = this.type;
    this.viewFactory.value = this.value;
    this.viewFactory.model = this.model;
    this.viewFactory.ns = this.ns;
    this.viewFactory["parentFactory"] = this;
    const editTmpl = this.editFactory.renderField(editField);
    const viewTmpl = this.viewFactory.renderField(viewField);
    if (editTmpl === nothing || viewTmpl === nothing) {
      return nothing;
    }
    const value = this._fieldValue(field, this.value);
    return html`<hi-inline-edit
      .value=${writeAlways(value)}
      @value-changed=${(e) => this.__inlineEdit__onValueChanged(e, field)}
      >${viewTmpl}${editTmpl}</hi-inline-edit
    > `;
  }
  __inlineEdit__onValueChanged(e, field) {
    const viewFactory = this.viewFactory["parentFactory"];
    const editFactory = this.editFactory["parentFactory"];
    if (editFactory || viewFactory) {
      viewFactory.retargetFieldEvent(e, field);
      editFactory.retargetFieldEvent(e, field);
      this.__updateValueFromModel(field);
      f1.write(`/type/fields/${field.name}`, field, viewFactory);
      viewFactory.requestUpdate("type");
      editFactory.requestUpdate("type");
    }
  }
  __updateValueFromModel(field) {
    const valueModelPath = field.model_path;
    const modelPath = this.namePrefix + valueModelPath;
    const readModelValue = f1.read(modelPath, void 0, this.model);
    f1.write(modelPath, readModelValue, f1.safe({}, this.value));
  }
}
HIP.factory.element(FactoryInlineElement);
