
import { setAttributes, writeAlways } from '../../../eip/element/hi-lit/hi-lit.js';
import { styleMap } from '../../../hip/library/lit-html/directives/style-map.js';
import { html } from '../../../hip/library/lit-html/lit-html.js';
import adt from '../../../pip/library/pi-adt/pi-adt.js';
import $ip from '../../../pip/library/pi-ip/pi-ip.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
["select", "combobox", "radio"].forEach((type) => {
  $ip.di.register("view", type, {
    id: `factory-view-${type}`,
    render(field, factory) {
      const ns = factory.ns;
      const options = f1.read("/source/out", [], field);
      const optRec = f1.to.record(f1.always("value"), options);
      return (value) => {
        let { generator, multiple = false } = f1.read("/settings", {}, field);
        if (type === "radio") {
          multiple = false;
        }
        const isEqual = adt.comparators.isEq.condition;
        const matchedValues = multiple ? f1.intersection(
          f1.map((opt) => opt.value, options),
          f1.is.array(value) ? value : [value]
        ) : f1.map(
          (opt) => opt.value,
          f1.filter((opt) => isEqual(opt.value, value), options)
        );
        f1.write("/style/margin-right", "0.25rem", field);
        const emptyMessage = f1.read("/substitution/out_of_bounds", value || "", field);
        const isEmpty = matchedValues.length <= 0 && f1.is.string(emptyMessage) && emptyMessage.length > 0;
        const values = f1.map((val) => optRec[`${val}`], matchedValues);
        const titles = isEmpty ? [emptyMessage] : f1.compact(f1.map((_) => f1.is.empty(_.title) ? void 0 : _.title, values));
        return f1.map(
          (title) => html`<hi-badge
            size="small"
            allow-ellipsis
            data-attrs=${setAttributes(field.attributes)}
            style=${styleMap(f1.read("/style", {}, field))}
          >
            <hi-text
              name=${factory.namePrefix + field.name}
              model-path=${factory.namePrefix + field.model_path}
              allow-ellipsis
              .ns=${ns}
              .value=${title}
              .generator=${generator}
              ?disallow-hint=${f1.is.boolean(field.hintable) ? !field.hintable : false}
            ></hi-text>
          </hi-badge>`,
          titles
        );
      };
    }
  });
});
export const renderCheckmark = (value, field, factory) => {
  const { case_true, case_false, case_default } = f1.read("/settings", {}, field);
  return html`<hi-checkmark
    name=${factory.namePrefix + field.name}
    model-path=${factory.namePrefix + field.model_path}
    allow-ellipsis
    .value=${writeAlways(value)}
    .ns=${factory.ns}
    .caseDefault=${case_default}
    .caseFalse=${case_false}
    .caseTrue=${case_true}
    data-attrs=${setAttributes(field.attributes)}
    style=${styleMap(f1.read("/style", {}, field))}
    ?disallow-hint=${f1.is.boolean(field.hintable) ? !field.hintable : false}
  ></hi-checkmark>`;
};
["boolean", "bi-state", "toggle"].forEach((type) => {
  $ip.di.register("view", type, {
    id: `factory-view-${type}`,
    render(field, factory) {
      const { case_true, case_false, case_default } = f1.read("/settings", {}, field);
      return (value) => {
        return renderCheckmark(value, field, factory);
      };
    }
  });
});
$ip.di.register("view", "tri-state", {
  id: `factory-view-tri-state`,
  render(field, factory) {
    return (value) => {
      const { case_true, case_false, case_default } = f1.read("/settings", {}, field);
      return html`<hi-checkmark
        kind="tri-state"
        name=${factory.namePrefix + field.name}
        model-path=${factory.namePrefix + field.model_path}
        allow-ellipsis
        .value=${writeAlways(value)}
        .ns=${factory.ns}
        .caseDefault=${case_default}
        .caseFalse=${case_false}
        .caseTrue=${case_true}
        data-attrs=${setAttributes(field.attributes)}
        style=${styleMap(f1.read("/style", {}, field))}
        ?disallow-hint=${f1.is.boolean(field.hintable) ? !field.hintable : false}
      ></hi-checkmark>`;
    };
  }
});
$ip.di.register("view", "flag", {
  id: `factory-view-flag`,
  render(field, factory) {
    return (value) => {
      return html`<hi-toggle
        name=${factory.namePrefix + field.name}
        model-path=${factory.namePrefix + field.model_path}
        .on=${writeAlways(value)}
        .ns=${factory.ns}
        data-attrs=${setAttributes(field.attributes)}
        style=${styleMap(field.style)}
        @commit-ended=${(e) => factory.retargetFieldEvent(e, field)}
      ></hi-toggle>`;
    };
  }
});
$ip.di.register("view", "select-tags", {
  id: `factory-view-select-tags`,
  render(field, factory) {
    const ns = factory.ns;
    const options = f1.read("/source/out", [], field);
    const optRec = f1.to.record(f1.always("value"), options);
    return (value) => {
      const { multiple = false } = f1.read("/settings", {}, field);
      const isEqual = adt.comparators.isEq.condition;
      const matchedValues = multiple ? f1.intersection(
        f1.map((opt) => opt.value, options),
        f1.is.array(value) ? value : [value]
      ) : f1.map(
        (opt) => opt.value,
        f1.filter((opt) => isEqual(opt.value, value), options)
      );
      const values = f1.map((val) => optRec[`${val}`], matchedValues);
      return html`<hi-tag-group .ns=${ns} @close-tag-ended=${(e) => factory.retargetFieldEvent(e, field)}>
        ${f1.map(
        ({ value: value2, title }) => html`<hi-tag
            size="small"
            allow-close
            .value=${value2}
            .ns=${ns}
            data-attrs=${setAttributes(field.attributes)}
            style=${styleMap(f1.read("/style", {}, field))}
            >${title}</hi-tag
          >`,
        values
      )}
      </hi-tag-group>`;
    };
  }
});
