
import { unsafeHTML } from '../../../hip/library/lit-html/directives/unsafe-html.js';
import { html, nothing } from '../../../hip/library/lit-html/lit-html.js';
import { setAttributes, writeAlways } from '../../../eip/element/hi-lit/hi-lit.js';
import { ifDefined as ifDef } from '../../../hip/library/lit-html/directives/if-defined.js';
import { styleMap } from '../../../hip/library/lit-html/directives/style-map.js';
import { IteratorElement } from '../../../eip/element/hi-iterator/hi-iterator.js';
import adt from '../../../pip/library/pi-adt/pi-adt.js';
import $ip from '../../../pip/library/pi-ip/pi-ip.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
$ip.di.register("view", "link", {
  id: `factory-view-link`,
  render(field, factory) {
    return (value, entity) => {
      const { label = "", raw_label = "", href, icon = "" } = f1.read("/settings", {}, field);
      const _href = f1.safe(f1.is.string(value) ? value : "#", href);
      const _label = raw_label ? raw_label : label ? f1.word(label) : _href;
      return html`<hi-link
        href=${_href}
        .ns=${factory.ns}
        .value=${entity}
        data-attrs=${setAttributes(field.attributes)}
        style=${styleMap(f1.read("/style", {}, field))}
        @trigger-ended=${(e) => factory.retargetFieldEvent(e, field)}
      >
        ${icon ? html`<hi-icon glyph=${icon} class="mrsxx"></hi-icon>` : nothing}
        <span>${_label}</span>
      </hi-link>`;
    };
  }
});
$ip.di.register("view", "code", {
  id: `factory-view-code`,
  render(field, factory) {
    return (value) => {
      const { language = "json" } = f1.read("/settings", {}, field);
      return html`<hi-code
        disabled
        data-attrs=${setAttributes(field.attributes)}
        style=${styleMap(f1.read("/style", {}, field))}
        .value=${ifDef(value)}
        .language=${language}
        .ns=${factory.ns}
      ></hi-code>`;
    };
  }
});
["editor", "html", "content"].forEach((render_type) => {
  $ip.di.register("view", render_type, {
    id: `factory-view-${render_type}`,
    render(field, factory) {
      return (value, entity) => {
        const content = adt.expression.evaluate(value, entity);
        const id = `render_type-${render_type}-${f1.id()}`;
        const attrs = f1.write("/gid", id, field.attributes);
        return html`<hi-panel
            .ns=${factory.ns}
            data-attrs=${setAttributes(attrs)}
            style=${styleMap(f1.read("/style", {}, field))}
          >
            ${unsafeHTML(content)}
          </hi-panel>
          ${field.hintable === true ? html`<hi-tooltip for=${attrs.gid} .ns=${factory.ns}>${unsafeHTML(content)}</hi-tooltip>` : nothing}`;
      };
    }
  });
});
$ip.di.register("view", "html_text", {
  id: `factory-view-html_text`,
  render(field, factory) {
    return (value) => {
      return html`<hi-text
        formatter="textarea"
        name=${factory.namePrefix + field.name}
        model-path=${factory.namePrefix + field.model_path}
        data-attrs=${setAttributes(field.attributes)}
        style=${styleMap(f1.read("/style", {}, field))}
        .value=${writeAlways(f1.format("html_text", {}, value))}
        .formatterRule=${ifDef(field.settings)}
        ?disallow-hint=${f1.is.boolean(field.hintable) ? !field.hintable : false}
      ></hi-text>`;
    };
  }
});
$ip.di.register("view", "color", {
  id: `factory-view-color`,
  list: IteratorElement.generatorDict("color"),
  render() {
    return (value) => {
      const option = {
        value,
        title: this.list[value]
      };
      return html`<div style="display: flex; align-items: center; gap: var(--hi-px-4)">
        <hi-color size="large" .value=${option.value} aria-label="color swatch for ${option.title}"></hi-color>
        <span>${option.title}</span>
      </div>`;
    };
  }
});
$ip.di.register("view", "icon", {
  id: `factory-view-icon`,
  list: IteratorElement.generatorDict("icon"),
  render() {
    return (value) => {
      const option = {
        value,
        title: this.list[value]
      };
      return html`<div style="display: flex; align-items: center; gap: var(--hi-px-4)">
        <hi-icon size="large" glyph=${option.value} aria-label="icon for ${option.title}"></hi-icon>
        <span>${option.title}</span>
      </div>`;
    };
  }
});
