
import { html } from '../../../hip/library/lit-html/lit-html.js';
import { setAttributes, writeAlways } from '../../../eip/element/hi-lit/hi-lit.js';
import { ifDefined as ifDef } from '../../../hip/library/lit-html/directives/if-defined.js';
import { styleMap } from '../../../hip/library/lit-html/directives/style-map.js';
import $ip from '../../../pip/library/pi-ip/pi-ip.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
const splitNewline = /\n|\n\r/;
[
  "datetime-local",
  "percentage",
  "telephone",
  "password",
  "currency",
  "username",
  "integer",
  "decimal",
  "number",
  "hidden",
  "search",
  "email",
  "text",
  "week",
  "time",
  "url"
].forEach((render_type) => {
  $ip.di.register("view", render_type, {
    id: `factory-view-${render_type}`,
    render(field, factory) {
      return (value) => {
        return html`<hi-text
          name=${factory.namePrefix + field.name}
          model-path=${factory.namePrefix + field.model_path}
          data-attrs=${setAttributes(field.attributes)}
          style=${styleMap(f1.read("/style", {}, field))}
          .ns=${factory.ns}
          .value=${writeAlways(value)}
          .formatterRule=${ifDef(field.settings)}
          formatter=${render_type}
          ?secure=${render_type === "password"}
          ?disallow-hint=${f1.is.boolean(field.hintable) ? !field.hintable : false}
        ></hi-text>`;
      };
    }
  });
});
$ip.di.register("view", "textarea", {
  id: `factory-view-textarea`,
  render(field, factory) {
    return (value) => {
      return html`<hi-text
        formatter="textarea"
        name=${factory.namePrefix + field.name}
        model-path=${factory.namePrefix + field.model_path}
        data-attrs=${setAttributes(field.attributes)}
        style=${styleMap(f1.read("/style", {}, field))}
        .ns=${factory.ns}
        .value=${writeAlways(value)}
        .formatterRule=${ifDef(field.settings)}
        ?disallow-hint=${f1.is.boolean(field.hintable) ? !field.hintable : false}
      ></hi-text>`;
    };
  }
});
$ip.di.register("view", "unknown", {
  id: `factory-view-unknown`,
  render(field, factory) {
    return (value) => {
      return html`<hi-text
        formatter="text"
        name=${factory.namePrefix + field.name}
        model-path=${factory.namePrefix + field.model_path}
        data-attrs=${setAttributes(field.attributes)}
        style=${styleMap(f1.read("/style", {}, field))}
        .ns=${factory.ns}
        .value=${writeAlways(value)}
        .formatterRule=${ifDef(field.settings)}
        ?disallow-hint=${f1.is.boolean(field.hintable) ? !field.hintable : false}
      ></hi-text>`;
    };
  }
});
const renderTextListItem = (value, field, factory) => {
  return html` <hi-text
    formatter=${field.render_type}
    name=${factory.namePrefix + field.name}
    model-path=${factory.namePrefix + field.model_path}
    data-attrs=${setAttributes(field.attributes)}
    style=${styleMap(f1.read("/style", {}, field))}
    .ns=${factory.ns}
    .value=${writeAlways(value)}
    .formatterRule=${ifDef(field.settings)}
    ?disallow-hint=${f1.is.boolean(field.hintable) ? !field.hintable : false}
  ></hi-text>`;
};
["array", "text_list"].forEach((render_type) => {
  $ip.di.register("view", render_type, {
    id: `factory-view-${render_type}`,
    render(field, factory) {
      return (value) => {
        const valueList = f1.is.string(value) ? value.split(splitNewline) : f1.is.array(value) ? value : [];
        return html`<ul kind="no-dot" data-attrs=${setAttributes(field.attributes)}>
          ${f1.compact(valueList).map((v) => {
          return html`<li>${renderTextListItem(v, field, factory)}</li>`;
        })}
        </ul>`;
      };
    }
  });
});
$ip.di.register("view", "dictionary", {
  id: `factory-view-dictionary`,
  render(field, factory) {
    return (value) => {
      const record = f1.compact(Object.entries(value));
      return html`<ul kind="no-dot" data-attrs=${setAttributes(field.attributes)}>
        ${record.map((_) => {
        return html`<li><strong>${f1.word(_[0])}</strong>: ${_[1]}</li>`;
      })}
      </ul>`;
    };
  }
});
const renderBadge = (value, field, factory) => {
  return html`<hi-badge
    class="mrsx"
    data-attrs=${setAttributes(field.attributes)}
    style=${styleMap(f1.read("/style", {}, field))}
    .ns=${factory.ns}
    >${value}</hi-badge
  >`;
};
$ip.di.register("view", "badge", {
  id: `factory-view-badge`,
  render(field, factory) {
    return (value) => {
      return renderBadge(value, field, factory);
    };
  }
});
$ip.di.register("view", "badges", {
  id: `factory-view-badges`,
  render(field, factory) {
    return (value) => {
      const valueList = f1.is.array(value) ? value : [];
      return f1.compact(valueList).map((v) => {
        return renderBadge(v, field, factory);
      });
    };
  }
});
