
import { LitElement } from '../../../hip/library/lit-element/lit-element.js';
import { html, nothing } from '../../../hip/library/lit-html/lit-html.js';
import { AsLitHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import { BehaviorFactory } from '../../../aip/element/ai-behavior-factory/ai-behavior-factory.js';
import '../../../aip/element/ai-behavior-factory/ai-behavior-factory.js';
import { FactoryViewElement } from '../../../aip/element/ai-factory-view/ai-factory-view.js';
import './ai-viewer-artifact-container.js';
import adt from '../../../pip/library/pi-adt/pi-adt.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class FactoryViewerElement extends BehaviorFactory(BehaviorElement(AsLitHIC(LitElement))) {
  constructor() {
    super(...arguments);
    this.viewFactory = new FactoryViewElement();
    this.groupingMutation = adt.mutation("grouping");
  }
  static get is() {
    return "ai-factory-viewer";
  }
  connectedCallback() {
    super.connectedCallback();
    this.__retarget__fieldTriggerEndedBound = this.__retarget__fieldTriggerEnded.bind(this);
    this.viewFactory.addEventListener("field-trigger-ended", this.__retarget__fieldTriggerEndedBound);
    this.__retarget__modelUpdateEndedBound = this.__retarget__modelUpdateEnded.bind(this);
    this.viewFactory.addEventListener("model-update-ended", this.__retarget__modelUpdateEndedBound);
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    this.viewFactory.removeEventListener("field-trigger-ended", this.__retarget__fieldTriggerEndedBound);
    this.viewFactory.removeEventListener("model-update-ended", this.__retarget__modelUpdateEndedBound);
  }
  __retarget__fieldTriggerEnded(e) {
    this._dispatchProcess("field-trigger", e.detail);
  }
  __retarget__modelUpdateEnded(e) {
    this._dispatchProcess("model-update", e.detail);
  }
  _render(fields, type) {
    const ns = this.ns;
    const { none, groupings } = this.groupingMutation.options({
      tags: f1.read("/type/tags", {}, this)
    }).invoke(fields);
    return html`
      <dl kind="normal">${none.list.map((field) => this.__renderDataItem(field))}</dl>
      ${groupings.length > 0 ? html`<hi-accordion mode="allow-any-opened" .ns=${ns} class="mtl">
            ${groupings.map((group, index) => {
      return html`
                <hi-bar ?opened=${0 <= index} .ns=${ns}>${group.title}</hi-bar>
                <hi-note allow-overflow .ns=${ns}>
                  <dl kind="normal">${group.list.map((field) => this.__renderDataItem(field))}</dl>
                </hi-note>
              `;
    })}
          </hi-accordion>` : nothing}
    `;
  }
  __renderDataItem(field) {
    const viewEl = this.renderField(field);
    if (viewEl === nothing) {
      return nothing;
    }
    return html`
      ${field.hidden_title ? nothing : html` <dt x-marker="field-label">
            <span class="prsxx" style="margin-right: auto">${field.title}</span>
            ${this._renderHint(field)} ${this._renderCopyable(field)}
          </dt>`}
      <dd>${viewEl} ${this.renderDescription(field)}</dd>
    `;
  }
  renderField(field) {
    this.viewFactory.namePrefix = this.namePrefix;
    this.viewFactory.type = this.type;
    this.viewFactory.value = this.value;
    this.viewFactory.model = this.model;
    this.viewFactory.ns = this.ns;
    this.viewFactory["parentFactory"] = this;
    return this.viewFactory.renderField(field);
  }
}
HIP.factory.element(FactoryViewerElement);
