
import { renderCheckmark } from '../../../aip/element/ai-factory-view/ai-view-artifact-choice.js';
import { setAttributes } from '../../../eip/element/hi-lit/hi-lit.js';
import { styleMap } from '../../../hip/library/lit-html/directives/style-map.js';
import { html, nothing } from '../../../hip/library/lit-html/lit-html.js';
import $ip from '../../../pip/library/pi-ip/pi-ip.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
const R = {
  findAllSlashes: /\//g
};
const services = {
  factorySignature(prefix, value, field, factory) {
    const subViewer = `viewer--${prefix}--${(factory.namePrefix || "name_prefix").replace(R.findAllSlashes, "-")}--${factory.ns || "ns"}--${field.name}`;
    const childOf = `${subViewer}--${value}`;
    return {
      subViewer,
      childOf
    };
  }
};
["list-viewer", "list-form"].forEach((render_type) => {
  $ip.di.register("view", render_type, {
    id: `factory-view-${render_type}`,
    is_render_isolated: true,
    render(field, factory) {
      const fields = f1.read("/fields", f1.read("/source/out", {}, field), field);
      const typeHeader = f1.read("/title", f1.word(field.name), field);
      const settings = f1.safe({}, field.settings);
      const type = {
        fields
      };
      let list = void 0;
      return (value, valueModel) => {
        list = f1.safe(f1.safe([], value), list);
        return html`
          <hi-accordion
            mode="allow-any-opened"
            class="mtm"
            .ns=${factory.ns}
            data-attrs=${setAttributes(field.attributes)}
            style=${styleMap(f1.read("/style", {}, field))}
          >
            ${list.map((entity, index) => {
          const subNamePrefix = `${factory.namePrefix}${field.model_path}/${index}/`;
          const { subViewer, childOf } = services.factorySignature(
            `viewer-list-${subNamePrefix.replace(/\//g, "-")}`,
            value,
            field,
            factory
          );
          const itemTitle = f1.is.in("item_title_path", settings) ? f1.read(settings.item_title_path, `${typeHeader}`, entity) : `${typeHeader}`;
          return html` <hi-bar .ns=${factory.ns}>${itemTitle}</hi-bar>
                <hi-note .ns=${factory.ns} allow-overflow>
                  <hi-panel gid="${subViewer}" .ns=${factory.ns}></hi-panel>
                  <ai-factory-viewer
                    for=${subViewer}
                    gid="force-paint"
                    .type=${type}
                    .value=${entity}
                    .ns=${factory.ns}
                    .childOf=${childOf}
                    .model=${factory.model}
                    .namePrefix=${subNamePrefix}
                  ></ai-factory-viewer>
                </hi-note>`;
        })}
          </hi-accordion>
        `;
      };
    }
  });
});
$ip.di.register("view", "select-form", {
  id: "factory-view-select-form",
  render(field, factory) {
    return (value, valueModel) => {
      const fields = f1.read("/fields", f1.read("/source/out", {}, field), field);
      const valueType = f1.safe({ fields: {} }, f1.read(f1.path(value), void 0, fields));
      const subFields = f1.read("/fields", f1.read("/source/out", {}, valueType), valueType);
      const selectedType = {
        fields: subFields
      };
      return html` <span>${value}</span>
        <ai-factory-viewer
          gid="force-paint"
          .type=${selectedType}
          .value=${valueModel}
          .ns=${factory.ns}
          .model=${factory.model}
          .namePrefix=${factory.namePrefix + field.name}
        ></ai-factory-viewer>`;
    };
  }
});
["toggle-form", "sub-form"].forEach((render_type) => {
  $ip.di.register("view", render_type, {
    id: `factory-view-${render_type}`,
    render(field, factory) {
      return (value, valueModel) => {
        const fields = f1.read("/fields", f1.read("/source/out", {}, field), field);
        const type = {
          fields
        };
        if (f1.is.boolean(value) && !value) {
          return nothing;
        }
        return html`${render_type === "toggle-form" ? renderCheckmark(value, field, factory) : nothing}
          <ai-factory-viewer
            gid="force-paint"
            .type=${type}
            .value=${valueModel}
            .ns=${factory.ns}
            .model=${factory.model}
            .namePrefix=${factory.namePrefix + field.name}
          ></ai-factory-viewer>`;
      };
    }
  });
});
