
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';
export class GuardError extends Error {
  constructor(guard, ...params) {
    super(...params);
    this.guard = guard;
    if (f1.is.in("captureStackTrace", Error) && f1.is.function(Error.captureStackTrace)) {
      Error.captureStackTrace(this, GuardError);
    }
  }
}
export class GuardElement extends BehaviorElement(AsHIC(PolymerElement)) {
  static get is() {
    return "ai-guard";
  }
  static get properties() {
    return {
      comparator: {
        type: String,
        value: "=="
      },
      compareSource: {
        type: String
      },
      compareField: {
        type: String
      },
      comparePath: {
        type: String
      },
      valueSource: {
        type: String
      },
      valueField: {
        type: String
      },
      valuePath: {
        type: String
      },
      value: {
        type: Object
      },
      failureValue: {
        type: Object,
        value: void 0
      },
      affect: {
        type: String,
        value: "error"
      },
      symbol: {
        type: String
      }
    };
  }
  static get observers() {
    return [
      "__settingsWatcher(comparator, value, compareSource, compareField, pullSource, pullField)",
      "__deprecatedWatcher(symbol)"
    ];
  }
  static get api() {
    return ["condition"];
  }
  static get autoBinding() {
    return {
      signal: "condition-ended",
      field: "value"
    };
  }
  get purpose() {
    return "guard";
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("hidden", "hidden");
  }
  condition(input) {
    const runComparator = HIP.kernel.runComparator;
    const comparator = this.comparator;
    const rightSet = this.__getValue(this.compareSource, this.compareField, input);
    const leftSet = this.__getValue(this.valueSource, this.valueField, this.value);
    const rightValues = this.__ensureArray(rightSet);
    const leftValues = this.__ensureArray(leftSet);
    const leftSizeLetter = this.__getSizeLetter(leftValues);
    const rightSizeLetter = this.__getSizeLetter(rightValues);
    const compareRule = `${leftSizeLetter}-${rightSizeLetter}`;
    const rightN = rightValues.length;
    const leftN = leftValues.length;
    if (compareRule.indexOf("0") >= 0) {
      return this.__initiateFailure(`values required: (${leftN}, ${rightN})`);
    } else if (leftSizeLetter === "N" && rightSizeLetter === "N" && leftN !== rightN) {
      return this.__initiateFailure(`subset not permitted: (${leftN}, ${rightN})`);
    }
    const n = compareRule === "1-1" ? 1 : Math.max(leftN, rightN);
    let i = 0;
    let j = 0;
    let k = 0;
    for (; k < n; k += 1) {
      const rightValue = this.comparePath ? ifn.get(this.comparePath, rightValues[j]) : rightValues[j];
      const leftValue = this.valuePath ? ifn.get(this.valuePath, leftValues[i]) : leftValues[i];
      if (!runComparator(comparator, leftValue, rightValue)) {
        return this.__initiateFailure(`no match: (${leftValue} ${comparator} ${rightValue})`);
      }
      if (leftSizeLetter === "N") {
        i += 1;
      }
      if (rightSizeLetter === "N") {
        j += 1;
      }
    }
    this.__repairFailure();
    return input;
  }
  __getSizeLetter(list) {
    return list.length > 1 ? "N" : list.length < 1 ? "0" : "1";
  }
  __getValue(instance, field, defaultValue) {
    if (instance == null) {
      return defaultValue;
    }
    return HIP.kernel.resolveReference(
      {
        instance,
        field
      },
      this
    );
  }
  __ensureArray(_set) {
    return f1.is.array(_set) ? _set : [_set];
  }
  __initiateFailure(_message) {
    switch (this.affect) {
      case "error":
        throw new GuardError(this, _message);
      case "disable":
        if (this.parentElement) {
          this.parentElement.setAttribute("disabled", "disabled");
        }
        break;
      case "empty":
      default:
        return this.failureValue;
    }
  }
  __repairFailure() {
    switch (this.affect) {
      case "disable":
        if (this.parentElement) {
          this.parentElement.removeAttribute("disabled");
        }
        break;
      default:
        break;
    }
  }
  __deprecatedWatcher(symbol) {
    this.comparator = symbol;
  }
  __settingsWatcher() {
    this._dispatchNotification("modified");
  }
  toString() {
    return `(${this.compareSource || ""}.${this.compareField || ""}.${this.comparePath || ""}) ${this.comparator} ${this.valueSource || ""}.${this.valueField || ""}.${this.valuePath || ""}`;
  }
}
HIP.factory.element(GuardElement);
