import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import '../../../aip/element/ai-behavior-prefab/ai-behavior-prefab.js';
import '../../../eip/element/hi-component/hi-component.js';

/**
 * @customElement
 * @polymer
 * @appliesMixin BehaviorPrefab
 * @appliesMixin BehaviorElement
 */
class Prefab404Element extends HIP.BehaviorPrefab(HIP.BehaviorElement(PolymerElement)) {
  static get is() {
    return 'ai-prefab-404';
  }

  static get template() {
    return html`
      <style>
        header .hero-oops-header{
          display: block;
          font-size: calc(100vh / 3);
          font-weight: bold;
          line-height: calc(100vh / 3 + 1vh);
          letter-spacing: calc(-6.5vh);
          z-index: 1;
          position: relative;
        }
        .hero-oops-header span{
          --oops--shadow: rgba(228, 173, 234, .7);
          /* --oops--shadow: rgba(171, 71, 188, .1); */
          color: var(--hi-color--magenta-30);
          text-shadow:
            -2px 0px 4px rgba(0,0,0,1),
            .5px .5px var(--oops--shadow),
            1px 1px var(--oops--shadow),
            1.5px 1.5px var(--oops--shadow),
            2px 2px var(--oops--shadow),
            2.5px 2.5px var(--oops--shadow),
            3px 3px  var(--oops--shadow),
            3.5px 3.5px var(--oops--shadow);
          transition: color ease-in-out .3s, text-shadow ease-in-out .3s;
        }

        .hero-oops-header:hover span{
          text-shadow:
            2px 0px 4px rgba(0,0,0,1),
            -.5px -.5px var(--oops--shadow),
            -1px -1px var(--oops--shadow),
            -1.5px -1.5px var(--oops--shadow),
            -2px -2px var(--oops--shadow),
            -2.5px -2.5px var(--oops--shadow),
            -3px -3px  var(--oops--shadow),
            -3.5px -3.5px var(--oops--shadow);
        }

        main > section{
          position: relative;
          top: calc(-3vh);
          z-index: 0;
        }
      </style>
      <ai-screen gid="_screen">
        <ai-state gid="_state__main"
          states="main">
          <header class="text-center">
            <p aria-label="Oops!" class="hero-oops-header text-center max-w-600 maa">
              <span aria-hidden="true">O</span>
              <span aria-hidden="true">o</span>
              <span aria-hidden="true">p</span>
              <span aria-hidden="true">s</span>
              <span aria-hidden="true">!</span>
            </p>
          </header>
          <main>
          <section l-kind="hort"
            l-justify="center">
            <hi-tile class="max-w-600 text-center">
              <h1 class="text-bold clr-gray_100-txt ptm">Page not found</h1>
              <p class="pbm text-sz-l">Unfortunately there was a problem fulfilling your request.</p>
              <div l-kind="hort" l-align="top" class="mtl">
                <div l-size="4" class="ptl">
                  <hi-button aesthetic="primary" href$="[[__getHomePageRoute()]]">
                    <hi-icon kind="home"></hi-icon>
                    <span>Go Home</span>
                  </hi-button>
                </div>
                <div l-size="8" class="bdr-solid bls clr-gray_20-bdr text-left plm">
                  <p class="text-bold clr-gray_80-txt pbm">History of pages you visited</h4>
                  <ul kind="no-dot">
                    <hi-repeat gid="_history__repeat"
                    input="[[__readHistoryLinks()]]"
                    as="historyItem">
                      <template>
                          <li class="pbm">
                            <p l-size="2" class="text-sz-s clr-gray_80-txt">[[historyItem.time]]</p>
                            <hi-link href$="[[historyItem.uri]]"
                              class="text-sz-l text-ellipsis">
                                [[historyItem.title]]
                            </hi-link>
                          </li>
                      </template>
                    </hi-repeat>
                  </ul>
                </div>
              </div>
            </hi-tile>
          </section>
        </main>
        </ai-state>
      </ai-screen>
    `;
  }

  get defaultScript() {
    return {
      system: {
        state: 'main'
      },
      configuration: {},
      bucket: {
        context: {
          $reactive_values: {
            title: '',
            description: ''
          }
        },
        index: {}
      }
    };
  }

  constructor() {
    super();
    this.requestNoHeader = true;
  }

  __readHistoryLinks() {
    const getHistoryFromStorage = window.sessionStorage.getItem('ahp-history');
    if (getHistoryFromStorage != null) {
      return JSON.parse(getHistoryFromStorage);
    }
    return [];
  }

  __getHomePageRoute() {
    return window.location.href.split('#')[0];
  }
}

HIP.factory.element(Prefab404Element);
