import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';

/**
 * @customElement
 * @polymer
 * @appliesMixin BehaviorPrefab
 * @appliesMixin BehaviorElement
 */
class PrefabAnalysisFilterElement extends HIP.BehaviorPrefab(HIP.BehaviorElement(PolymerElement)) {
  static get is() {
    return 'ai-prefab-analysis-filter';
  }

  static get template() {
    return html`
      <ai-screen gid="_screen" l-kind="hort">
        <ai-state gid="_state_main" states="main" class="plm prm">
          <section class="mtsxxx pbm">
            <hi-form
              gid="_filter_form"
              kind="filter"
              form="filter_record_manager_[[ uuid ]]"
              endpoint="api"
              disable-http
              aesthetic="invisible"
              alignment="[[context.alignment]]"
              column-count="[[context.columnCount]]"
              flush
            >
              <ai-builder-form gid="_filter_form__builder"> </ai-builder-form>
              <hi-button
                id="_filter_form__clear_button"
                class="mrsx"
                aesthetic="secondary"
                type="reset"
                slot="controls"
              >
                Clear
              </hi-button>

              <hi-button id="_filter_form__submit_button" type="submit" slot="controls"> Filter </hi-button>
            </hi-form>
          </section>

          <hi-binding source="_filter_form__clear_button" sink="_filter_form__submit_button"> </hi-binding>
          <hi-binding source="query_records" sink="_filter_form"> </hi-binding>
          <hi-binding source="query_records_type" sink="_filter_form__builder"> </hi-binding>
        </ai-state>
      </ai-screen>
    `;
  }

  get defaultScript() {
    return {
      system: {
        state: 'main'
      },
      action: {
        query_applied: {}
      },
      bucket: {
        context: {
          $reactive_values: {
            alignment: 'hort',
            columnCount: 4
          }
        },
        query_records: {}
      }
    };
  }

  includeBehaviors(script) {
    script = this.__reflectMode(script);
    return script;
  }

  /**
   * mode = immediate, use the filter-update method of hi-form to filter continuously
   * mode = submission, use the form-submit method of hi-form to filter
   */
  __reflectMode(script) {
    const mode = ifn.get('system.mode', script) === 'immediate' ? 'immediate' : 'submission';

    const isSubmissionButtonsHidden = mode === 'immediate';
    const filterFormKind = mode === 'immediate' ? 'filter' : 'submit';
    const queryAppliedEvent =
      mode === 'immediate'
        ? '{ _filter_form.output-changed } =>> { query_applied }'
        : '{ _filter_form.submit-ended } =>> { query_applied }';

    return ifn.mergeWith('append', script, {
      system: {
        mode
      },
      life_cycle: {
        initialize: {
          $elements: {
            _filter_form__clear_button: {
              hidden: isSubmissionButtonsHidden
            },
            _filter_form__submit_button: {
              hidden: isSubmissionButtonsHidden
            },
            _filter_form: {
              kind: filterFormKind
            }
          }
        }
      },
      action: {
        query_applied: {
          $bindings: [queryAppliedEvent]
        }
      }
    });
  }
}

HIP.factory.element(PrefabAnalysisFilterElement);
