import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';

/**
 * @customElement
 * @polymer
 * @appliesMixin BehaviorPrefab
 * @appliesMixin BehaviorElement
 */
class PrefabAnalysisGraphElement extends HIP.BehaviorPrefab(HIP.BehaviorElement(PolymerElement)) {
  static get is() {
    return 'ai-prefab-analysis-graph';
  }

  static get template() {
    return html`
      <ai-screen gid="_screen" l-kind="hort" on-mode-ended="__onModeEnded">
        <ai-state gid="_state_main" states="main" l-kind="vert">
          <section class="mam">
            <ai-block id="_analysis_graph__block" gid="_analysis_graph__block" class="mla mra">
              <!-- analysis_graph -->
            </ai-block>
          </section>
        </ai-state>
        <ai-state gid="_external_interface" kind="logic">
          <hi-binding source="records" sink="_analysis_graph" field="input"> </hi-binding>
        </ai-state>
      </ai-screen>
    `;
  }

  get defaultScript() {
    return {
      system: {
        state: 'main'
      },
      configuration: {},
      action: {
        selected_record: {}
      },
      bucket: {
        context: {},
        records: {},
        display: {}
      }
    };
  }

  __onModeEnded(mode) {
    this.createPrefabGraph(mode);
  }

  createPrefabGraph(mode) {
    const displayBlockEl = this.$._analysis_graph__block;
    const existingGraph = displayBlockEl.children[0];

    if (existingGraph) {
      existingGraph.remove();
    }

    const modeValue = mode.detail ? mode.detail.activeMode : mode;
    const createGraphEl = document.createElement(`vi-${modeValue}`);
    const displayBucketEl = HIP.db.gid.queryOne('display', this.ns);

    createGraphEl.ns = this.ns;
    createGraphEl.id = '_analysis_graph';
    createGraphEl.gid = '_analysis_graph';
    createGraphEl.setAttribute('class', 'mla mra');
    createGraphEl.setAttribute('for', '_records_grid');

    // TODO move this elsewhere, or use HIP.data.()
    ifn.each((value, key) => {
      createGraphEl[key] = value;
    }, displayBucketEl.output);

    displayBlockEl.appendChild(createGraphEl);
    createGraphEl.flush();
  }
}

HIP.factory.element(PrefabAnalysisGraphElement);
