import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import '../../../aip/element/ai-behavior-prefab/ai-behavior-prefab.js';
import '../../../eip/element/hi-component/hi-component.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';

/**
 * @customElement
 * @polymer
 * @appliesMixin BehaviorPrefab
 * @appliesMixin BehaviorElement
 */
class PrefabAnalysisTableElement extends HIP.BehaviorPrefab(HIP.BehaviorElement(PolymerElement)) {
  static get is() {
    return 'ai-prefab-analysis-table';
  }

  static get template() {
    return html`
      <ai-screen gid="_screen">
        <ai-state gid="_state_main"
          states="main">

          <!-- grid -->
          <main l-kind="hort">

            <hi-panel id="_records_regions__container"
              gid="_records_regions__container">
              <header class="nhsxx plsx ptsxxx pbsxxx bts bbs clr-blue_30-bdr bdr-solid clr-gray_10-bkg">
                <!-- region: _records_regions -->
                <ai-block id="_records_regions"
                  gid="_records_regions"
                  l-size="12"
                  l-kind="hort"
                  l-width-fixed
                  l-align="baseline">
                  <div id="_records_regions__none_task__widget_group">
                    <ai-builder-widget gid="_records_regions__none_task__widget_group__builder">
                    </ai-builder-widget>
                  </div>
                  <div id="_records_regions__bulk_task__widget_group">
                    <ai-builder-widget gid="_records_regions__bulk_task__widget_group__builder">
                    </ai-builder-widget>
                  </div>
                  <div id="_records_regions__singular_task__widget_group">
                    <ai-builder-widget gid="_records_regions__singular_task__widget_group__builder">
                    </ai-builder-widget>
                  </div>
                </ai-block>
              </header>
            </hi-panel>

            <section id="_grid_section"
              l-size="12">
              <!-- allow-row-select -->
              <hi-grid id="_records_grid"
                gid="_records_grid"
                row-stripe="3">
                <di-fn-sort gid="_records_grid__fn_sort">
                </di-fn-sort>
                <di-fn-paginate gid="_records_grid__fn_paginate"
                  page-index="0"
                  page-size="10">
                </di-fn-paginate>
                <ai-builder-grid gid="_records_grid__builder">
                </ai-builder-grid>
              </hi-grid>

              <footer l-kind="hort"
                class="mtmx mbmx">
                <div l-kind="hort"
                  l-hug="end"
                  l-align="baseline"
                  l-justify="right"
                  l-width-fixed>
                  <hi-dropdown kind="value"
                    id="_pagination__dropdown"
                    gid="_pagination__dropdown"
                    placeholder="Page Size"
                    aesthetic="minimal"
                    class="mlsx mrsx">
                    <hi-select gid="_records_grid_paginate_length_select">
                      <ai-builder-select gid="_records_grid_paginate_length_select__builder">
                      </ai-builder-select>
                    </hi-select>
                  </hi-dropdown>
                  <hi-text id="_pagination__subText"
                    gid="_pagination__subText"
                    class="text-sz-m clr-gray_80-txt">
                    items per page |
                  </hi-text>
                  <!-- force this from toString -->
                  <hi-text id="_pagination__text"
                    gid="_pagination__text"
                    class="text-sz-m clr-gray_80-txt mlsx mrsxx">
                  </hi-text>
                  <hi-pagination id="_records_grid_pagination"
                    gid="_records_grid_pagination"
                    page-size="10"
                    max="60">
                  </hi-pagination>
                </div>
              </footer>

            </main>

          </section>

          <hi-binding source="_records_grid_paginate_length_select"
            signal="value-changed"
            sink="noop">
            <hi-pipe sink="_records_grid__fn_paginate"
              field="pageSize">
            </hi-pipe>
            <hi-pipe sink="_records_grid_pagination"
              field="pageSize">
            </hi-pipe>
            <hi-pipe sink="_records_grid__fn_paginate"
              field="pageIndex"
              constant="0">
            </hi-pipe>
            <hi-pipe sink="_records_grid_pagination"
              field="pageIndex"
              constant="0">
            </hi-pipe>
            <hi-pipe pull-source="_records_grid_pagination"
              pull-field="toString"
              sink="_pagination__text">
            </hi-pipe>
          </hi-binding>

          <hi-binding source="source"
            sink="noop">
            <hi-pipe sink="_records_regions__none_task__widget_group__builder">
              <di-fn-get path="region.records.tasks.none"></di-fn-get>
            </hi-pipe>
            <hi-pipe sink="_records_regions__singular_task__widget_group__builder">
              <di-fn-get path="region.records.tasks.singular"></di-fn-get>
              <hi-pipe sink="_records_grid"
                field="allow-row-select">
                output = input.length > 0;
              </hi-pipe>
            </hi-pipe>
            <hi-pipe sink="_records_regions__bulk_task__widget_group__builder">
              <di-fn-get path="region.records.tasks.bulk"></di-fn-get>
              <hi-pipe sink="_records_grid"
                field="allow-row-select">
                output = input.length > 0;
              </hi-pipe>
            </hi-pipe>
          </hi-binding>

          <hi-binding source="_records_grid"
            signal="values-changed"
            sink="noop">
            <hi-pipe sink="_records_regions__singular_task__widget_group__builder"
              field="disabled">
              output = input.length !== 1;
            </hi-pipe>
            <hi-pipe sink="_records_regions__bulk_task__widget_group__builder"
              field="disabled">
              output = input.length < 1;
            </hi-pipe>
            <hi-pipe pull-source="_records_grid"
              pull-field="inputSize"
              sink="_records_grid_pagination"
              field="max">
            </hi-pipe>
            <hi-pipe pull-source="_records_grid_pagination"
              pull-field="toString"
              sink="_pagination__text">
            </hi-pipe>
          </hi-binding>


          <hi-binding source="_records_grid"
            signal="values-changed"
            sink="active_records">
          </hi-binding>

          <hi-binding source="initialize"
            constant="0"
            sink="_records_grid_pagination"
            field="pageIndex">
          </hi-binding>

          <hi-binding source="_records_grid__builder"
            signal="allow-columns-changed"
            sink="_records_grid"
            field="allowColumns">
          </hi-binding>

          <hi-binding source="_records_grid__builder"
            signal="sort-by-changed"
            sink="_records_grid"
            field="sortBy">
          </hi-binding>

          <hi-binding source="_records_grid__builder"
            signal="sort-order-changed"
            sink="_records_grid"
            field="sortOrder">
          </hi-binding>

          <hi-binding source="_records_grid_pagination"
            sink="_records_grid__fn_paginate"
            field="pageIndex">
          </hi-binding>

          <hi-binding source="_records_grid_paginate_length_select"
            pull-source="_records_grid_pagination"
            pull-field="toString"
            sink="_pagination__text">
          </hi-binding>

          <hi-binding source="records_type"
            sink="_records_grid__builder">
          </hi-binding>

        </ai-state>

        <ai-state gid="_external_interface"
          kind="logic">
          <hi-binding source="records"
            sink="_records_grid">
          </hi-binding>
        </ai-state>

      </ai-screen>
    `;
  }

  get properties() {
    return {
      uuid: {
        type: String,
        value: () => f1.id()
      }
    };
  }

  get defaultInterlink() {
    return {
      interlink: {
        templates: {
          by_zone_id: {
            title: {
              zone_type: 'title'
            },
            records: {
              zone_type: 'widget',
              widget_template: {
                input_mode: 'none'
              }
            },
            _records_grid: {
              zone_type: 'content',
              content_mapping: {
                zone_bucket: 'records'
              }
            },
            record_row: {
              zone_type: 'widget',
              widget_template: {
                input_mode: 'singular'
              }
            },
            record_row_menu: {
              zone_type: 'widget',
              widget_template: {
                input_mode: 'singular'
              }
            },
            _records_grid_pagination: {
              zone_type: 'control'
            }
          }
        }
      }
    };
  }

  get defaultScript() {
    return {
      system: {
        state: 'main'
      },
      region: {
        records: {
          title: 'Records Region',
          name: 'records',
          task_binding_template: {
            pullSource: '_records_grid',
            pullField: 'values'
          }
        }
      },
      bucket: {
        pagination: {
          initial_value: {
            properties: [
              {
                value: 10,
                title: '10',
                selected: true
              },
              {
                value: 20,
                title: '20'
              },
              {
                value: 30,
                title: '30'
              },
              {
                value: 50,
                title: '50'
              }
            ]
          },
          $output: ['{ |> _records_grid_paginate_length_select__builder }']
        },
        active_records: {},
        context: {},
        records: {}
      }
    };
  }

  includeBehaviors(script) {
    script = this.__includePaginationBehaviors(script);
    return script;
  }

  /**
   * disabled until the prefab gets updated if new definitions are loaded
   */
  __includePaginationBehaviors(script) {
    const paginationIOName = ifn.first(ifn.get('behavior.pagination', script) || []);
    const paginationIO = ifn.get(`io.${paginationIOName}`, script);
    const isDefaultPaginationEnabled = paginationIOName && paginationIO;

    return isDefaultPaginationEnabled ? this.__enableDefaultPagination(script, paginationIOName) : script;
  }

  __enableDefaultPagination(script, paginationIOName) {
    return ifn.mergeWith('append', script, {
      life_cycle: {
        initialize: {
          $elements: {
            _pagination__text: {
              hidden: true
            },
            _pagination__subText: {
              hidden: true
            },
            _pagination__dropdown: {
              hidden: true
            },
            _records_grid: {
              disallowSort: true
            },
            _records_grid__fn_sort: {
              disabled: true
            },
            _records_grid__fn_paginate: {
              disabled: true
            },
            _records_grid_pagination: {
              kind: 'indeterminate'
            }
          },
          $bindings: [
            `_records_grid_pagination:jump-to-ended =>> ${paginationIOName}:invokeLink`,
            `${paginationIOName} =>> prefab:enablePaginationControls <=< ${paginationIOName}:metadata`
          ]
        }
      }
    });
  }

  /**
   *
   */
  enablePaginationControls(metadata) {
    const links = ifn.get('links', metadata);
    const paginationEl = this.$._records_grid_pagination;

    if (links) {
      paginationEl.toggleDisabledIndeterminate(links);
    }
  }
}

HIP.factory.element(PrefabAnalysisTableElement);
