import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';

/**
 * @customElement
 * @polymer
 * @appliesMixin BehaviorPrefab
 * @appliesMixin BehaviorElement
 */
class PrefabDocumentElement extends HIP.BehaviorPrefab(HIP.BehaviorElement(PolymerElement)) {
  static get is() {
    return 'ai-prefab-document';
  }

  static get template() {
    return html`
      <ai-screen gid="_screen">
        <ai-state gid="_state_main" states="main">
          <main>
            <header>
              <h1 id="header">[[ context.title ]]</h1>
            </header>

            <!-- uses innerHTML -->
            <section id="content"></section>

            <!-- default to hidden and only show if there is data -->
            <table id="table" hidden>
              <thead>
                <tr>
                  <th>[[ context.notes__title_left ]]</th>
                  <th>[[ context.notes__title_right ]]</th>
                </tr>
              </thead>
              <tbody id="addendum__tbody"></tbody>
              <tbody id="location__tbody"></tbody>
            </table>
          </main>

          <!-- <hi-binding source="location"
            sink="parameters__repeat">
            <di-fn-get path="parameters"></di-fn-get>
          </hi-binding> -->
        </ai-state>
      </ai-screen>
    `;
  }

  get defaultScript() {
    return {
      system: {
        state: 'main'
      },
      configuration: {},
      bucket: {
        context: {
          $reactive_values: {
            notes__title_left: 'Supporting Notes',
            notes__title_right: 'Information',
            title: '',
            content: ''
          }
        },
        addendum: {}
      }
    };
  }

  constructor() {
    super();
    this.requestNoHeader = true;
  }

  _connectedViewCallback() {
    this.__followBucket('location', 'parameters');
    this.__followBucket('addendum');
  }

  __followBucket(name, getPath = '') {
    const bucketEl = HIP.db.gid.queryOne(name, this.ns);
    if (!bucketEl) {
      return;
    }

    this.__applyParameters(name, ifn.get(getPath, bucketEl.output));
    bucketEl.addEventListener('output-changed', (e) => {
      this.__applyParameters(name, ifn.get(getPath, e.detail.value));
    });
  }

  __applyParameters(name, parameters) {
    if (!parameters) {
      return;
    }

    this.$.table.hidden = false;

    if (parameters.content) {
      this.$.content.innerHTML = parameters.content;
    } else if (parameters.message) {
      this.$.content.innerHTML = parameters.message;
    }

    // TODO should I update the actual title (what about the header bar?)
    if (parameters.title) {
      this.$.header.innerHTML = parameters.title;
    }

    let hideTable = true;

    for (const key in parameters) {
      if (key === 'content' || key === 'title' || key === 'message') {
        continue;
      }

      hideTable = false;

      const tr = document.createElement('tr');
      const tdLeft = document.createElement('td');
      const tdRight = document.createElement('td');

      tdLeft.innerText = key;
      tdRight.innerText = parameters[key];

      tr.appendChild(tdLeft);
      tr.appendChild(tdRight);
      this.$[`${name}__tbody`].appendChild(tr);
    }

    this.$.table.hidden = hideTable;
  }

  /**
   *
   * apply content on context as HTML
   */
  _alterContext(context) {
    // NOTE potential XSS attach, upgrade to LitElement for LitHTML support
    if (context.content != null) {
      this.$.content.innerHTML = context.content;
    }

    return context;
  }
}

HIP.factory.element(PrefabDocumentElement);
