import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';

/**
 * @customElement
 * @polymer
 * @appliesMixin BehaviorPrefab
 * @appliesMixin BehaviorElement
 * @note requires @eab-old/elements
 */
class PrefabFeedElement extends HIP.BehaviorPrefab(HIP.BehaviorElement(PolymerElement)) {
  static get is() {
    return 'ai-prefab-feed';
  }

  static get template() {
    return html`
      <ai-screen gid="_screen" l-kind="hort">
        <ai-state gid="_state_main" states="main" l-kind="vert" class="plm prm">
          <section l-kind="hort" class="mtm">
            <section l-size="12">
              <hi-combobox id="_combobox" gid="_combobox"> </hi-combobox>
            </section>
          </section>

          <section l-kind="hort" class="mtm">
            <section l-size="12" id="_template_target" gid="_template_target"></section>
            <hi-if id="noevents_message" hidden kind="simple" l-size="12">
              <h2 class="text-center">There are no events for this student.</h2>
            </hi-if>
            <aside l-size="2"></aside>
          </section>

          <section l-kind="hort" class="mtm">
            <aside l-size="5"></aside>
            <section l-size="2">
              <hi-switch value="button" id="show_button">
                <hi-case value="button">
                  <hi-button id="_next_button" gid="_next_button">
                    <span>[[ context.next_button_label ]]</span>
                    <hi-icon kind="chevron_right" aria-label="[[ context.next_button_label ]]"></hi-icon>
                  </hi-button>
                </hi-case>
                <hi-case value="message">
                  <p>End</p>
                </hi-case>
                <hi-case value="noevents"> </hi-case>
              </hi-switch>
            </section>
            <aside l-size="5"></aside>
          </section>

          <hi-render id="_template__render" gid="_template__render" for="_template_target"></hi-render>

          <ai-builder-template
            id="_repeater__template_builder"
            gid="_repeater__template_builder"
            mode="repeater"
            for="_template__render"
            for-child="_item__template_builder"
          ></ai-builder-template>
          <ai-builder-template
            id="_item__template_builder"
            gid="_item__template_builder"
            mode="template"
          ></ai-builder-template>

          <hi-binding source="context" sink="noop">
            <hi-pipe sink="_item__template_builder" field="template">
              <di-fn-get path="template.item"></di-fn-get>
            </hi-pipe>
            <hi-pipe sink="_repeater__template_builder" field="template">
              <di-fn-get path="template.repeater"></di-fn-get>
            </hi-pipe>
          </hi-binding>

          <hi-binding source="records_type" sink="noop">
            <hi-pipe sink="_item__template_builder"></hi-pipe>
            <hi-pipe sink="_repeater__template_builder"></hi-pipe>
          </hi-binding>

          <hi-binding source="records" sink="_template__render"> </hi-binding>

          <hi-binding source="context" sink="noop">
            <hi-pipe sink="_combobox" field="input">
              <di-fn-get path="filter_options"></di-fn-get>
            </hi-pipe>
          </hi-binding>

          <hi-binding source="records" sink="noevents_message" field="hidden">
            output = (!!input && input.length == 0 ) ? false : true
          </hi-binding>

          <hi-binding source="load_next_io" sink="show_button" field="value">
            output = (input && input.length < 10) ? 'message' : 'button';
          </hi-binding>

          <hi-binding source="load_initial_io" sink="show_button" field="value">
            output = (input && input.length === 0) ? 'noevents': (input.length < 10 ? 'message' : 'button');
          </hi-binding>
        </ai-state>
        <ai-state kind="logic" gid="_external_interface"> </ai-state>
      </ai-screen>
    `;
  }

  get defaultScript() {
    return {
      system: {
        state: 'main'
      },
      state: {
        main: {
          $elements: {
            load_initial_io: {
              disabled: false
            },
            load_next_io: {
              disabled: false
            }
          }
        }
      },
      action: {
        collect_params_replace: {
          $startup: {
            $state: 'main'
          }
        },
        collect_params_append: {}
      },
      bucket: {
        context: {
          $reactive_values: {
            template: {
              repeater: 'tiles',
              item: 'fallback'
            },
            next_button_label: 'Next'
          }
        },
        records: {}
      },
      io: {
        load_initial_io: {
          $input: '(( load_initial ))',
          $output: '(( records ))'
        },
        load_next_io: {
          $input: '(( load_next ))',
          $output: '(( records:append ))'
        }
      }
    };
  }
}

HIP.factory.element(PrefabFeedElement);
