
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class PrefabListReorderElement extends HIP.BehaviorPrefab(HIP.BehaviorElement(LitElement)) {
  constructor() {
    super(...arguments);
    this.enableDrag = true;
    this.__validTypes = [];
    this.__internalList = [];
    this.__rankAttr = "order";
    this.__onRecordsOutputChangedBound = this.__onRecordsOutputChanged.bind(this);
    this.__onRecordsTypeOutputChangedBound = this.__onRecordsTypeOutputChanged.bind(this);
  }
  static get is() {
    return "ai-prefab-list-reorder";
  }
  static get properties() {
    return {
      enableDrag: {
        type: Boolean,
        reflect: true,
        attribute: "enable-drag"
      }
    };
  }
  get defaultScript() {
    return {
      system: {
        state: "main"
      },
      action: {
        reorder: {},
        drop: {},
        drag: {},
        swap: {}
      },
      bucket: {
        records: {},
        activeRecord: {},
        orderedRecords: {}
      },
      type: {
        records_type: {}
      }
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.ns = this.ns || this.getAttribute("ns");
    this.__setTitleAndDescription();
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    if (this.__drake) {
      this.__drake.off("drop");
      this.__drake.destroy();
    }
    this.__drake = void 0;
  }
  __onRecordsTypeOutputChanged(event) {
    const props = f1.values(f1.read("/detail/value/properties", {}, event));
    const typeMap = props.filter((prop) => !prop.hidden);
    this.__validTypes = typeMap.sort((a, b) => (a.order || 0) - (b.order || 0));
  }
  __onRecordsOutputChanged(event) {
    const records = f1.read("/detail/value", [], event);
    if (!f1.is.empty(records)) {
      this.updateInternalList(records);
    }
  }
  __setTitleAndDescription() {
    const header = this.getInstance("_view_header");
    const contextBucket = this.getInstance("context");
    if (!header) {
      return;
    }
    const titleIsMissing = f1.is.empty(header.context);
    if (!titleIsMissing) {
      return;
    }
    header.context = contextBucket.output;
  }
  async firstUpdated() {
    this.getInstance("records").addEventListener("output-changed", this.__onRecordsOutputChangedBound);
    this.getInstance("records_type").addEventListener("output-changed", this.__onRecordsTypeOutputChangedBound);
    await this.updateComplete;
    this.__initializeDrag();
  }
  updated(changedProperties) {
    super.updated(changedProperties);
    if (changedProperties.has("enableDrag")) {
      if (!this.enableDrag) {
        this.__drake?.destroy();
      }
    }
  }
  __initializeDrag() {
    const dragContainer = this.querySelector("#dragula-container");
    if (!dragContainer) {
      return;
    }
    const dragula2 = window.dragula;
    this.__drake = dragula2([dragContainer], {
      moves: function(el) {
        return !!el && el?.hasAttribute("draggable-item");
      },
      copy: false,
      ignoreInputTextSelection: true,
      direction: "vertical"
    });
    this.__drake.on("drop", (el) => {
      const itemId = parseInt(el.dataset.id || "0");
      const newPosition = Array.from(el.parentNode?.children || []).findIndex((item) => el === item);
      const oldPosition = this.__internalList.findIndex((item) => `${item.id}` === `${itemId}`);
      const droppedItem = this.__internalList[oldPosition];
      droppedItem[this.__rankAttr] = newPosition;
      const removeOldItem = this.__internalList.splice(oldPosition, 1);
      const insertItemInNewPosition = this.__internalList.splice(f1.max(newPosition - 1, 0), 0, removeOldItem[0]);
      this.__internalList = this.__reIndexList();
      this.__drake.cancel(true);
      this.reorderAction(itemId, newPosition);
      this.sortListAndUpdate();
    });
  }
  render() {
    return html`
      <ai-screen gid="_screen" l-kind="hort">
        <ai-state gid="_state_main" states="main"> ${this.__renderListTemplate()} </ai-state>
      </ai-screen>
    `;
  }
  __shouldGrowColumn(columnIndex) {
    const rankTypeIndex = 0;
    const firstIndex = 0;
    const firstIndexWhenRankEnabled = 1;
    if (this.__isRankingEnabled(this.__validTypes[rankTypeIndex])) {
      return columnIndex === firstIndexWhenRankEnabled;
    }
    return columnIndex === firstIndex;
  }
  __isRankingEnabled(type) {
    return f1.read("/rank", false, type);
  }
  __isRankColumn(columnIndex, type) {
    const rankColumnIndex = 0;
    return columnIndex === rankColumnIndex && this.__isRankingEnabled(type);
  }
  __renderListTemplate() {
    return html`
      <section>
        <ul kind="no-dot" id="dragula-container" class="bdr-solid bts brs bls clr-gray_40-bdr bdr-round-s">
          <li class="text-normal bdr-solid bbsx clr-blue_30-bdr clr-gray_10-bkg" l-kind="hort" l-align="center">
            <div l-size="1" class="pam max-w-48 min-w-48"></div>
            <div l-size="grow" l-kind="hort" l-align="center">
              ${this.__validTypes.map(
      (type, i) => html`${this.__renderCellTemplate(type.title, this.__shouldGrowColumn(i))}`
    )}
            </div>
            <div l-size="1" class="pam max-w-64 min-w-64"></div>
          </li>
          ${this.__internalList.length > 0 ? html`${this.__internalList.map((item, index) => this.__renderListItemTemplate(item, index))}` : html`<div l-kind="hort" l-width-fixed l-align="center" l-justify="center" class="min-h-320">
                <hi-indicator></hi-indicator><span>Loading...</span>
              </div>`}
        </ul>
      </section>
    `;
  }
  __renderListItemDragHandleColumn() {
    return html` <div l-size="1" class="max-w-48 min-w-48 prm" l-kind l-width-fixed l-align="center">
      <div class="handle prm pam" style="cursor:move" drag-handle ?hidden="${!this.enableDrag}">
        <hi-icon kind="draggable" theme="gray-dark" size="small" style="cursor:move"></hi-icon>
      </div>
    </div>`;
  }
  __getColumnValue(item, type, columnIndex, rowIndex) {
    if (this.__isRankColumn(columnIndex, type)) {
      return rowIndex + 1;
    }
    return f1.read(`/${type.name.replace(/\./g, "/")}`, void 0, item);
  }
  __renderListItemCellTemplateColumns(item, rowIndex) {
    return html`<div l-size="grow" l-kind="hort" l-align="center">
      ${this.__validTypes.map((type, columnIndex) => {
      const columnValue = this.__getColumnValue(item, type, columnIndex, rowIndex);
      return html`${this.__renderCellTemplate(
        this.__formatValue(columnValue, type),
        this.__shouldGrowColumn(columnIndex)
      )}`;
    })}
    </div>`;
  }
  __renderListItemMoveButtonColumns(item, itemIndex) {
    const isUpDisabled = itemIndex === 0;
    const isDownDisabled = itemIndex === this.__internalList.length - 1;
    return html`<div l-size="1" class="max-w-64 min-w-64 prm" l-kind l-width-fixed l-align="center">
      <hi-button-group kind="small">
        <hi-button
          aesthetic="ghost"
          layout-icon
          size="small"
          @trigger-ended="${this.moveUp(item.id, itemIndex, item)}"
          ?disabled=${isUpDisabled}
        >
          <hi-icon kind="chevron_up" size="small"></hi-icon>
          <hi-hint>Move Up</hi-hint>
        </hi-button>
        <hi-button
          aesthetic="ghost"
          layout-icon
          size="small"
          @trigger-ended="${this.moveDown(item.id, itemIndex, item)}"
          ?disabled=${isDownDisabled}
        >
          <hi-icon kind="chevron_down" size="small"></hi-icon>
          <hi-hint>Move Down</hi-hint>
        </hi-button>
      </hi-button-group>
    </div>`;
  }
  __renderListItemTemplate(item, itemIndex) {
    return html`
      <li
        class="bdr-solid bbs clr-gray_40-bdr ${this.__updatedItemKey === item.id ? "updated" : ""}"
        data-index="${itemIndex}"
        data-id="${item.id}"
        draggable-item
        l-kind="hort"
        l-align="center"
      >
        ${this.__renderListItemDragHandleColumn()} ${this.__renderListItemCellTemplateColumns(item, itemIndex)}
        ${this.__renderListItemMoveButtonColumns(item, itemIndex)}
      </li>
    `;
  }
  __renderCellTemplate(value, shouldGrow) {
    return html`<div class="pam text-capitalize" l-size="${shouldGrow ? "grow" : 2}">${value}</div>`;
  }
  __formatValue(value, type) {
    return f1.format(type.format_type || "fallback", {}, value);
  }
  __swapRank(leftIndex, rightIndex) {
    const field = this.__rankAttr;
    const swapRank = this.__internalList[leftIndex][field];
    this.__internalList[leftIndex][field] = this.__internalList[rightIndex][field];
    this.__internalList[rightIndex][field] = swapRank;
    const swappedItems = [
      {
        id: this.__internalList[leftIndex].id,
        [field]: this.__internalList[leftIndex][field]
      },
      {
        id: this.__internalList[rightIndex].id,
        [field]: this.__internalList[rightIndex][field]
      }
    ];
    this.swapAction(swappedItems);
  }
  __reIndexList() {
    return f1.map((item, index) => {
      return {
        ...item,
        [`${this.__rankAttr}`]: index + 1
      };
    }, this.__internalList);
  }
  __sortList() {
    this.__internalList = f1.sort(
      (a, b) => f1.read(this.rankLens, 0, a) - f1.read(this.rankLens, 0, b),
      this.__internalList
    );
  }
  moveUp(id, order, item) {
    return () => {
      this.__swapRank(order, order - 1);
      this.reorderAction(id, order - 1);
    };
  }
  moveDown(id, order, item) {
    return () => {
      this.__swapRank(order, order + 1);
      this.reorderAction(id, order + 1);
    };
  }
  reorderAction(id, order) {
    this.__updatedItemKey = id;
    const reorderAction = this.getInstance("reorder");
    if (reorderAction) {
      reorderAction.invoke({
        id,
        [this.__rankAttr]: order
      });
    }
    this.sortListAndUpdate();
  }
  swapAction(swappedItems) {
    const swapAction = this.getInstance("swap");
    if (swapAction) {
      swapAction.invoke(swappedItems);
    }
  }
  updateInternalList(list = []) {
    if (f1.is.in("rank", list[0] || {})) {
      this.__rankAttr = "rank";
    }
    this.__internalList = list;
    this.sortListAndUpdate();
  }
  sortListAndUpdate() {
    this.__sortList();
    this.updateOrderedRecords();
    this.requestUpdate();
  }
  updateOrderedRecords() {
    const orderedBucket = this.getInstance("orderedRecords");
    const reIndexList = this.__reIndexList();
    if (orderedBucket) {
      orderedBucket.invoke(reIndexList);
    }
  }
  getInstance(instanceName) {
    return HIP.db.gid.queryOne(instanceName, this.ns);
  }
  get rankLens() {
    return f1.lens(`/${this.__rankAttr}`);
  }
}
HIP.factory.element(PrefabListReorderElement);
