import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
/* eslint-disable lit-a11y/anchor-is-valid */

/**
 * @customElement
 * @polymer
 * @appliesMixin BehaviorPrefab
 * @appliesMixin BehaviorElement
 */
class PrefabLoginElement extends HIP.BehaviorPrefab(HIP.BehaviorElement(PolymerElement)) {
  static get is() {
    return 'ai-prefab-login';
  }

  static get template() {
    return html`
      <style>
        .prefab-login--main-container {
          max-width: 22.5rem !important;
          margin-top: calc(var(--hi-px-32) * 2);
          box-shadow: var(--hi-color--shadow-standard);
        }
        .prefab-login--aside,
        .prefab-login--main {
          width: 100%;
        }
        .prefab-login--main header {
          border-bottom: var(--hi-px-1) solid var(--hi-color--gray-30);
        }
        .prefab-login--header {
          display: flex;
          flex-direction: column;
          align-items: center;
        }
        .prefab-login--aside-image {
          display: none;
        }
        @media only screen and (min-width: 45rem) {
          .prefab-login--main-container {
            max-width: 45rem !important;
          }
          .prefab-login--aside,
          .prefab-login--main {
            width: 50%;
          }
          .prefab-login--aside-image {
            display: block;
          }
        }
      </style>
      <ai-screen gid="_screen">
        <ai-state gid="_state_main" states="main">
          <section l-kind="hort" l-width-fixed l-justify="center" class="prefab-login--main-container mla mra">
            <aside class="prefab-login--aside clr-university-txt clr-university-bkg text-center">
              <div l-kind="vert">
                <img
                  class="prefab-login--aside-image mbm"
                  src$="[[context.university_image_uri]]"
                  alt=""
                  hidden$="[[__isUniversityImageMissing(context.university_image_uri)]]"
                />
                <div class="pbm plm prm">
                  <h1 class="clr-university-txt">[[__tenantTitle(context)]]</h1>
                  <p class="ptsx">[[context.tenant.description]]</p>
                </div>
              </div>
            </aside>
            <main class="prefab-login--main min-h-360">
              <header class="prefab-login--header pbm text-center">
                <img
                  class="prefab-login--main-logo"
                  src$="[[context.university_logo_uri]]"
                  alt=""
                  hidden$="[[__isUniversityImageMissing(context.university_logo_uri)]]"
                />
                <h2 class="ptm">[[ __productTitle(context) ]]</h2>
                <p class="clr-gray_80-txt">[[ context.product.description ]]</p>
              </header>
              <section class="mll mrl">
                <hi-alert id="_issue" gid="_issue" kind="hide" theme="error" hidden> [[ issue ]] </hi-alert>
                <hi-form id="_login_form" gid="_login_form" aesthetic="invisible" endpoint="api" disable-http flush>
                  <ai-builder-form gid="_login_form__builder"> </ai-builder-form>
                  <hi-button
                    gid="_login_form__authenticate_button"
                    type="submit"
                    theme="university"
                    slot="controls"
                    class="mlz mrz"
                    stretched
                  >
                    [[ context.login_form__authenticate_label ]]
                  </hi-button>
                </hi-form>
                <hi-binding source="form_type" sink="_login_form__builder"> </hi-binding>
                <hi-binding source="form" sink="_login_form"> </hi-binding>
              </section>
              <section class="mll mrl mtsx">
                <ai-block
                  gid="_below_form__block"
                  region="below_form__widget_group__builder"
                  ns$="[[ ns ]]"
                  class="mtm mbm"
                >
                  <div id="_below_form_none_task__widget_group__builder" class="width-100">
                    <ai-builder-widget gid="_below_form_none_task__widget_group__builder" ns$="[[ ns ]]">
                    </ai-builder-widget>
                  </div>
                </ai-block>
                <hi-binding source="source" sink="_below_form_none_task__widget_group__builder" ns$="[[ ns ]]">
                  <di-fn-get path="region.below_form.tasks.none"> </di-fn-get>
                </hi-binding>
              </section>
            </main>
          </section>
          <footer
            class="prefab-login--footer width-100 text-center pam"
            l-kind="hort"
            l-justify="center"
            l-align="center"
            l-width-fixed
          >
            <hi-eab></hi-eab>
            <!-- deprecated -->
            <ul kind="no-dot" class="mlm">
              <hi-repeat input="[[ context.links ]]">
                <template>
                  <li class="text-center display-inline mrsx">
                    <a href$="[[ item.uri ]]" rel="external nofollow" size="small">[[ item.title ]]</a>
                  </li>
                </template>
              </hi-repeat>
            </ul>
            <ul kind="no-dot" class="mlm" l-kind="hort" l-width-fixed l-align="center">
              <li class="text-center mrm">
                <ai-block gid="_login_footer">
                  <ai-builder-widget gid="_login_footer_none_task__widget_group__builder"> </ai-builder-widget>
                </ai-block>
              </li>
              <li class="text-center mrm">
                <hi-link href="https://eab.com" external target="_blank">EAB Home</hi-link>
              </li>
              <li class="text-center mrm">
                <hi-link href="https://eab.com/privacy-policy-2/" external target="_blank">Privacy Policy</hi-link>
              </li>
            </ul>
            <hi-binding source="source" sink="_login_footer_none_task__widget_group__builder">
              <di-fn-get path="region.login_footer.tasks.none"></di-fn-get>
            </hi-binding>
          </footer>
          <hi-binding source="_login_form" sink="authenticate"> </hi-binding>
        </ai-state>
      </ai-screen>
    `;
  }

  static get properties() {
    return {
      issue: {
        type: String,
        observer: '__issueChanged'
      },
      /**
       * The login io id which drives authentication.
       * We need this for internal authentication mechanisms
       */
      loginServiceId: String
    };
  }

  get defaultScript() {
    return {
      system: {
        state: 'main'
      },
      action: {
        authenticate: {}
      },
      bucket: {
        context: {
          $reactive_values: {
            login_form__authenticate_label: 'Login',
            university_image_uri:
              'https://attachment.eab.com/wp-content/uploads/2019/07/Student-Success-dark-blue-background.png',
            university_logo_uri: '',
            product: {
              title: 'Login'
            },
            tenant: {
              title: 'Welcome',
              description: 'Login to get started'
            }
          }
        },
        form: {}
      }
    };
  }

  constructor() {
    super();
    this.requestNoHeader = true;
  }

  connectedCallback() {
    super.connectedCallback();
    ess.session('clear');
  }

  includeBehaviors(script) {
    const loginServiceId = ifn.first(ifn.get('behavior.login', script) || []);

    if (!loginServiceId) {
      return script;
    }

    this.loginServiceId = loginServiceId;
    return ifn.merge(
      {
        action: {
          authenticate: {}
        },
        io: {
          [loginServiceId]: {
            method: 'post',
            manual: true,
            allowFail: true,
            $ahp: {
              network: {
                intercept_errors: false
              }
            },
            $input: '(( <| authenticate ))',
            $finish: [`=>> user_space:login__onLoginSuccess  <-< ${loginServiceId}`],
            $error: [`=>> user_space:login__onLoginFailure <-< ${loginServiceId}`]
          }
        }
      },
      script
    );
  }

  _connectedViewCallback() {
    // DEPRECATED we can just reference them off of the prefab itself
    const userSpace = HIP.db.gid.queryOne('user_space', this.ns) || {};

    userSpace.login__onLoginSuccess = this.__onLoginSuccess.bind(this);
    userSpace.login__onLoginFailure = this.__onLoginFailure.bind(this);
  }

  __onLoginSuccess(loginServiceId) {
    const loginIO = HIP.db.gid.queryOne(loginServiceId, this.ns);
    const data = ifn.get('output', loginIO) || ifn.get('output.data', loginIO) || ifn.get('data', loginIO);

    ess.session('initiate', 'success', data);
  }

  __onLoginFailure(loginServiceId) {
    const loginIO = HIP.db.gid.queryOne(loginServiceId, this.ns);
    const data = ifn.get('output', loginIO) || ifn.get('output.metadata', loginIO) || ifn.get('metadata', loginIO);

    this.informOnError(data);
    ess.session('save', 'failure', data);
  }

  informOnError(metadata) {
    const errors = ifn.get('errors', metadata);
    if (!f1.is.empty(errors)) {
      this.$._login_form.validateCustomValidity(errors);
    }
  }

  __isUniversityImageMissing(uri) {
    return !uri;
  }

  __tenantTitle(context) {
    return ifn.get('tenant.title', context) || context.title;
  }

  /**
   * @deprecated sub_title, use product.title
   */
  __productTitle(context) {
    return ifn.get('product.title', context) || context.sub_title;
  }

  __issueChanged() {
    const noIssue = f1.is.empty(this.issue);
    HIP.util.toggleAttribute(this.$._issue, 'hidden', noIssue, 'hidden');
  }
}

HIP.factory.element(PrefabLoginElement);
