import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';

/**
 * @customElement
 * @polymer
 * @appliesMixin BehaviorPrefab
 * @appliesMixin BehaviorElement
 */
class PrefabLogoutElement extends HIP.BehaviorPrefab(HIP.BehaviorElement(PolymerElement)) {
  static get is() {
    return 'ai-prefab-logout';
  }

  static get template() {
    return html`
      <ai-screen gid="_screen">
        <ai-state gid="_state_main" states="main" l-kind="hort" class="mte">
          <main l-kind="vert" l-hug="center" class="prefab-logout--main bdr-round-m min-w-240 max-w-480">
            <header class="mbmxx">
              <h1>[[ context.title ]]</h1>
              <p>[[ context.description ]]</p>
              <p>
                [[ context.return_label_prefix ]]<hi-link
                  id="_go_back_link"
                  gid="_go_back_link"
                  aesthetic="secondary"
                  class="mbsxx"
                  on-trigger-ended="__returnButton__onTriggerEnded"
                  href="#"
                  >[[ context.return_label ]]</hi-link
                >
              </p>
            </header>

            <section>
              <hi-button id="_logout_button" gid="_logout_button" class="mbsxx">
                <hi-icon kind="external2"></hi-icon>
                <span>[[ context.logout_label ]]</span>
              </hi-button>
            </section>
          </main>

          <hi-binding source="_logout_button" sink="logout"> </hi-binding>
        </ai-state>
      </ai-screen>
    `;
  }

  get defaultScript() {
    return {
      system: {
        state: 'main'
      },
      configuration: {},
      behavior: {},
      action: {
        logout: {}
      },
      bucket: {
        context: {
          $reactive_values: {
            title: 'Logout',
            description: 'If you logout, you will end your session and return to the login screen.',
            logout_label: 'Logout',
            return_label_prefix: 'You can return to',
            return_label: 'where you came from.'
          }
        }
      }
    };
  }

  constructor() {
    super();
    this.requestNoHeader = true;
  }

  includeBehaviors(script) {
    const logoutServiceId = ifn.first(ifn.get('behavior.logout', script) || []);

    if (!logoutServiceId) {
      return script;
    }

    return ifn.merge(
      {
        action: {
          logout: {
            $do: [`=>> ${logoutServiceId}:disabled <-< false`],
            $finish: [`{ ${logoutServiceId} }`, `=>> ${logoutServiceId}:disabled <-< true`]
          }
        },
        io: {
          [logoutServiceId]: {
            overrideSignal: 'invoke-ended',
            allowRedirect: true,
            allowFail: true,
            manual: true,
            $finish: [`{ prefab.__onLogoutSuccess }`],
            $error: [`{ prefab.__onLogoutFailure }`]
          }
        }
      },
      script
    );
  }

  _connectedViewCallback() {
    const userSpace = HIP.db.gid.queryOne('user_space', this.ns);

    userSpace.logout__onLogoutSuccess = this.__onLogoutSuccess.bind(this);
    userSpace.logout__onLogoutFailure = this.__onLogoutFailure.bind(this);
  }

  __onLogoutSuccess() {
    ess.session('clear');
  }

  __onLogoutFailure() {
    ess.session('clear');
  }

  __returnButton__onTriggerEnded() {
    window.history.back();
  }
}

HIP.factory.element(PrefabLogoutElement);
