import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';

/**
 * @customElement
 * @polymer
 * @appliesMixin BehaviorPrefab
 * @appliesMixin BehaviorElement
 */
class PrefabRecordListElement extends HIP.BehaviorPrefab(HIP.BehaviorElement(PolymerElement)) {
  static get is() {
    return 'ai-prefab-record-list';
  }

  static get template() {
    return html`
      <ai-screen id="_screen" gid="_screen">
        <ai-state gid="_state_main" states="main">
          <header l-kind="hort">
            <ai-block gid="_records_header" l-kind="hort">
              <div id="_records_header_none_task__widget_group">
                <ai-builder-widget gid="_records_header_none_task__widget_group__builder"> </ai-builder-widget>
              </div>
            </ai-block>
            <hi-binding source="source" sink="noop">
              <hi-pipe sink="_records_header_none_task__widget_group__builder">
                <di-fn-get path="region.records_header.tasks.none"></di-fn-get>
              </hi-pipe>
            </hi-binding>
          </header>

          <hi-panel class="mtm" id="_template_target" gid="_template_target" role="region"></hi-panel>
          <hi-render id="_template__render" gid="_template__render" for="_template_target"></hi-render>

          <ai-builder-template
            id="_repeater__template_builder"
            gid="_repeater__template_builder"
            mode="repeater"
            for="_template__render"
            for-child="_item__template_builder"
          ></ai-builder-template>
          <ai-builder-template id="_item__template_builder" gid="_item__template_builder" mode="template">
          </ai-builder-template>

          <hi-binding source="context" sink="noop">
            <hi-pipe sink="_item__template_builder" field="template">
              <di-fn-get path="template.item"></di-fn-get>
            </hi-pipe>
            <hi-pipe sink="_repeater__template_builder" field="template">
              <di-fn-get path="template.repeater"></di-fn-get>
            </hi-pipe>
          </hi-binding>

          <hi-binding source="records_type" sink="noop">
            <!-- we have to setup item, because repeater expects a lit-fn on it -->
            <hi-pipe sink="_item__template_builder">
              <di-fn-get gid="_item__template_builder__type__get_children" path="aliases.children"></di-fn-get>
            </hi-pipe>
            <hi-pipe sink="_repeater__template_builder"></hi-pipe>
          </hi-binding>

          <hi-binding source="records" sink="_template__render"> </hi-binding>

          <!-- mode hierarchy will pass the children alias type to the child -->
          <hi-binding
            source="_screen"
            signal="active-mode-changed"
            sink="_item__template_builder__type__get_children"
            field="disabled"
            map-fn="[[ __modeToTemplateChildren ]]"
          >
          </hi-binding>
        </ai-state>
        <ai-state kind="logic" gid="_external_interface"> </ai-state>
      </ai-screen>
    `;
  }

  get defaultScript() {
    return {
      system: {
        state: 'main',
        /**
         * - list, where item and repeater get the same type
         * - hierarchy, where the child is provided the children alias type
         */
        mode: 'list'
      },
      bucket: {
        context: {
          $reactive_values: {
            template: {
              repeater: 'fallback',
              item: 'fallback'
            }
          }
        },
        records: {}
      }
    };
  }

  /**
   * Used to turn parsing the type for the item builder-template
   *
   * 1. if the mode is hierarchy, we want the `disabled` false so that we descend the records_type to get a new type for the children
   * 2. if the mode is list (default), we use the records_type as the item type and the repeater type just shares it (does not need special knowledge)
   *
   * - hierarchy, we need two types (repeater, item)
   * - list, the object type applies to the item and not the repeater, the repeater is generic
   */
  __modeToTemplateChildren(binding) {
    const activeMode = binding.input;

    switch (activeMode) {
      case 'hierarchy':
        return false;
      case 'list':
        return true;
    }
  }
}

HIP.factory.element(PrefabRecordListElement);
