import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import '../../../aip/element/ai-behavior-prefab/ai-behavior-prefab.js';
import '../../../eip/element/hi-component/hi-component.js';

// eslint-disable-next-line prettier/prettier
import navigation from '../../../pip/library/pi-navigation/pi-navigation.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';

/**
 * @customElement
 * @polymer
 * @appliesMixin BehaviorPrefab
 * @appliesMixin BehaviorElement
 */
class PrefabRecordManagerElement extends HIP.BehaviorPrefab(HIP.BehaviorElement(PolymerElement)) {
  static get is() {
    return 'ai-prefab-record-manager';
  }

  static get template() {
    return html`
      <ai-screen gid="_screen">
        <ai-state states="main, empty" disabled>
          <!-- filter form -->
          <hi-drawer direction="right" heading="Filter Records" id="_filter_form__container" gid="_filter_form__container">
            <hi-form gid="_filter_form" form="filter_record_manager_[[ uuid ]]" kind="filter" endpoint="api" disable-http
              aesthetic="invisible" alignment="vert" column-count="1"  flush>
              <ai-builder-form gid="_filter_form__builder">
              </ai-builder-form>
              <hi-button id="_filter_form__submit_button" class="mtm" type="submit" slot="controls">
                Filter
              </hi-button>
              <hi-button id="_filter_form__clear_button" class="mrsx mtm" aesthetic="secondary" type="reset" slot="controls">
                Clear
              </hi-button>
            </hi-form>
          </hi-drawer>
          <hi-panel id="_records_regions__container" l-size="12" l-kind="hort" l-width-fixed l-align="center"
            gid="_records_regions__container">
            <!-- region: _records_regions -->
            <ai-block id="_records_regions" gid="_records_regions" l-size="10" l-kind="hort" l-width-fixed l-align="center">
              <div id="_records_regions__none_task__widget_group">
                <ai-builder-widget gid="_records_regions__none_task__widget_group__builder">
                </ai-builder-widget>
              </div>
              <div id="_records_regions__bulk_task__widget_group">
                <ai-builder-widget gid="_records_regions__bulk_task__widget_group__builder">
                </ai-builder-widget>
              </div>
              <div id="_records_regions__singular_task__widget_group">
                <ai-builder-widget gid="_records_regions__singular_task__widget_group__builder">
                </ai-builder-widget>
              </div>
            </ai-block>
            <div id="_filter_button_container" gid="_filter_button_container" l-hug="end">
              <hi-button aesthetic="ghost" id="_filter_button_[[ uuid ]]" gid="_filter_button_[[ uuid ]]">
                <hi-icon kind="filter" mode="presentation" inherit-color></hi-icon><span>Filter</span>
                <hi-badge size="medium" id="_active_filter_indicator" gid="_active_filter_indicator" kind="indicator"
                  theme="red" hidden heading="Active"></hi-badge>
              </hi-button>
            </div>
        </hi-panel>
        </ai-state>
        <ai-state gid="_state_main" states="main" disabled>
          <!-- grid -->
          <main l-kind="hort" class="mtm">
            <section id="_grid_section" l-size="12">
              <!-- allow-row-select -->
              <hi-grid id="_records_grid" aesthetic="standard" gid="_records_grid" row-stripe="3">
                <di-fn-sort gid="_records_grid__fn_sort">
                </di-fn-sort>
                <di-fn-paginate gid="_records_grid__fn_paginate" page-index="0" page-size="10">
                </di-fn-paginate>
                <ai-builder-grid gid="_records_grid__builder">
                </ai-builder-grid>
              </hi-grid>

              <footer l-kind="hort" class="mtmx mbmx">
                <div l-kind="hort" l-hug="end" l-align="baseline" l-justify="right" l-width-fixed>
                  <hi-dropdown kind="value" id="_pagination__dropdown" gid="_pagination__dropdown" placeholder="Page Size"
                    aesthetic="minimal" class="mlsx mrsx">
                    <hi-select gid="_records_grid_paginate_length_select" aesthetic="dropdown">
                      <ai-builder-select gid="_records_grid_paginate_length_select__builder">
                      </ai-builder-select>
                    </hi-select>
                  </hi-dropdown>
                  <hi-text id="_pagination__subText" gid="_pagination__subText" class="text-sz-m clr-gray_80-txt">
                    items per page |
                  </hi-text>
                  <!-- force this from toString -->
                  <hi-text id="_pagination__text" gid="_pagination__text" class="text-sz-m clr-gray_80-txt mlsx mrsxx">
                  </hi-text>
                  <hi-pagination id="_records_grid_pagination" gid="_records_grid_pagination" page-size="10" max="60">
                  </hi-pagination>
                </div>
              </footer>

          </main>

          </section>

          <hi-binding source="_records_grid_paginate_length_select" signal="value-changed" sink="noop">
            <hi-pipe sink="_records_grid__fn_paginate" field="pageSize">
            </hi-pipe>
            <hi-pipe sink="_records_grid_pagination" field="pageSize">
            </hi-pipe>
            <hi-pipe sink="_records_grid__fn_paginate" field="pageIndex" constant="0">
            </hi-pipe>
            <hi-pipe sink="_records_grid_pagination" field="pageIndex" constant="0">
            </hi-pipe>
            <hi-pipe pull-source="_records_grid_pagination" pull-field="toString" sink="_pagination__text">
            </hi-pipe>
          </hi-binding>

          <hi-binding source="_filter_button_[[ uuid ]]" sink="_filter_form__container"></hi-binding>

          <hi-binding source="_filter_form__clear_button" sink="_filter_form__submit_button">
          </hi-binding>

          <hi-binding source="context" sink="noop">
            <hi-pipe sink="_records_grid" field="caption">
              <di-fn-get path=".caption"></di-fn-get>
            </hi-pipe>
          </hi-binding>

          <hi-binding source="source" sink="noop">
            <hi-pipe sink="_records_regions__none_task__widget_group__builder">
              <di-fn-get path="region.records.tasks.none"></di-fn-get>
            </hi-pipe>
            <hi-pipe sink="_records_regions__singular_task__widget_group__builder">
              <di-fn-get path="region.records.tasks.singular"></di-fn-get>
            </hi-pipe>
            <hi-pipe sink="_records_regions__bulk_task__widget_group__builder">
              <di-fn-get path="region.records.tasks.bulk"></di-fn-get>
            </hi-pipe>
            <hi-pipe sink="_records_grid" field="allow-row-select" inject="source" map-fn="[[ __needsGridSelectionRows ]]">
            </hi-pipe>
          </hi-binding>

          <hi-binding source="_records_grid" signal="values-changed" sink="noop">
            <hi-pipe sink="_records_regions__singular_task__widget_group__builder" field="disabled">
              output = input.length !== 1;
            </hi-pipe>
            <hi-pipe sink="_records_regions__bulk_task__widget_group__builder" field="disabled">
              output = input.length < 1; </hi-pipe> <hi-pipe pull-source="_records_grid" pull-field="inputSize"
                sink="_records_grid_pagination" field="max">
            </hi-pipe>
            <hi-pipe pull-source="_records_grid_pagination" pull-field="toString" sink="_pagination__text">
            </hi-pipe>
          </hi-binding>

          <hi-binding source="_records_grid" signal="values-changed" sink="active_records">
          </hi-binding>

          <hi-binding source="initialize" constant="0" sink="_records_grid_pagination" field="pageIndex">
          </hi-binding>

          <hi-binding source="_records_grid__builder" signal="allow-columns-changed" sink="_records_grid"
            field="allowColumns">
          </hi-binding>

          <hi-binding source="_records_grid__builder" signal="sort-by-changed" sink="_records_grid" field="sortBy">
          </hi-binding>

          <hi-binding source="_records_grid__builder" signal="sort-order-changed" sink="_records_grid" field="sortOrder">
          </hi-binding>

          <hi-binding source="_records_grid_pagination" sink="_records_grid__fn_paginate" field="pageIndex">
          </hi-binding>

          <hi-binding source="_records_grid_paginate_length_select" pull-source="_records_grid_pagination"
            pull-field="toString" sink="_pagination__text">
          </hi-binding>

          <hi-binding source="records_type" sink="_records_grid__builder">
          </hi-binding>
        </ai-state>

        <hi-binding source="filter_records_type" sink="_filter_form__builder">
        </hi-binding>

        <ai-state gid="_external_interface" kind="logic">
          <ai-bucket id="_records_filter" gid="_records_filter">
            <di-fn-filter gid="_records_filter__fn_filter">
            </di-fn-filter>
          </ai-bucket>

          <hi-binding source="records" sink="_records_filter">
          </hi-binding>
          <hi-binding source="_records_filter" sink="_records_grid">
          </hi-binding>
        </ai-state>
        <ai-state states="empty" disabled l-kind="vert" l-width-fixed l-align="center">
            <h3>[[ context.empty_state_heading ]]</h3>
            <p>[[ context.empty_state_description ]]</p>
        </ai-state>
        <!-- Display active indicator if filters are applied -->
        <hi-binding source="filter_applied" sink="_active_filter_indicator" field="hidden">
          output = Object.values(input || {}).filter((v) => v != '').length == 0;
        </hi-binding>
      </ai-screen>
    `;
  }

  static get properties() {
    return {
      uuid: {
        type: String,
        value: () => f1.id()
      }
    };
  }

  get defaultScript() {
    return {
      system: {
        state: 'main'
      },
      life_cycle: {
        initialize: {
          $bindings: [
            '_filter_form:submit-ended =>> filter_applied',
            '_records_grid:request-sort-ended =>> sort_applied',
            // reset pagination
            `{ _records_grid.request-sort-ended } =>> { prefab.goToLink } <-< first`
          ]
        }
      },
      action: {
        filter_applied: {},
        sort_applied: {
          $modifiers: [
            {
              $is: 'di/fn/map',
              $configure: {
                kind: 'object',
                host: 'prefab',
                method: 'serializeSortParams'
              }
            }
          ]
        }
      },
      region: {
        records: {
          title: 'Records Region',
          name: 'records',
          task_binding_template: {
            pullSource: '_records_grid',
            pullField: 'values'
          }
        }
      },
      io: {
        _records_output_proxy_for_state: {
          $is: 'memory',
          $modifiers: [
            {
              $is: 'di/fn/map',
              $configure: {
                kind: 'object',
                host: 'prefab',
                method: 'calculateNewStateOfPrefab'
              }
            }
          ]
        }
      },
      bucket: {
        pagination: {
          initial_value: {
            properties: [
              {
                value: 10,
                title: '10',
                selected: true
              },
              {
                value: 20,
                title: '20'
              },
              {
                value: 30,
                title: '30'
              },
              {
                value: 50,
                title: '50'
              }
            ]
          },
          $output: ['{ |> _records_grid_paginate_length_select__builder }']
        },
        active_records: {},
        filter_records: {},
        context: {
          initial_value: {
            caption: 'Principle data grid.',
            empty_state_heading: 'No Results Found',
            empty_state_description: ''
          }
        },
        records: {
          $finish: [
            '=>> { _records_output_proxy_for_state } <=< { records }',
            '=>> { _screen.state } <=< { _records_output_proxy_for_state }'
          ]
        }
      }
    };
  }

  includeBehaviors(script) {
    script = this.__includeRestBehaviors(script);
    script = this.__includePaginationBehaviors(script);
    script = this.__includeFilterBehaviors(script);
    script = this.__includeSortBehaviors(script);

    return script;
  }

  /**
   * disabled until the prefab gets updated if new definitions are loaded
   */
  __includePaginationBehaviors(script) {
    const paginationIOName = ifn.first(ifn.get('behavior.pagination', script) || []);
    const paginationIO = ifn.get(`io.${paginationIOName}`, script);

    const restIOName = ifn.first(ifn.get('behavior.rest', script) || []);
    const restIO = ifn.get(`io.${restIOName}`, script);

    const readIOName = !restIOName || !restIO ? paginationIOName : `read_${restIOName}`;

    if (!readIOName || !paginationIOName || !paginationIO) {
      return script;
    }

    this.readIOName = readIOName;

    const doesPaginationIOExist = paginationIOName && paginationIO;
    const paginationConfiguration = ifn.get(`$configure.pagination`, paginationIO);

    if (doesPaginationIOExist && paginationConfiguration) {
      return paginationConfiguration.kind === 'determinate'
        ? this.__enableDeterminatePagination(script, this.readIOName)
        : script;
    } else {
      return doesPaginationIOExist ? this.__enableInDeterminatePagination(script, readIOName) : script;
    }
  }

  __enableDeterminatePagination(script, readIOName) {
    return ifn.mergeWith('append', script, {
      life_cycle: {
        initialize: {
          $elements: {
            _pagination__text: {
              hidden: true
            },
            _pagination__subText: {
              hidden: true
            },
            _pagination__dropdown: {
              hidden: true
            },
            _records_grid: {
              disallowSort: true
            },
            _records_grid__fn_sort: {
              disabled: true
            },
            _records_grid__fn_paginate: {
              disabled: true,
              pageIndex: 0
            },
            _records_grid_pagination: {
              kind: 'determinate',
              pageSize: 0,
              pageIndex: 0,
              max: 0
            }
          },
          $bindings: [
            ` ${readIOName} =>> _records_grid_pagination:input <=< ${readIOName}:metadata`,
            `_records_grid_pagination:jump-to-ended =>> prefab:invokeLink`
          ]
        }
      }
    });
  }

  __enableInDeterminatePagination(script, readIOName) {
    return ifn.mergeWith('append', script, {
      life_cycle: {
        initialize: {
          $elements: {
            _pagination__text: {
              hidden: true
            },
            _pagination__subText: {
              hidden: true
            },
            _pagination__dropdown: {
              hidden: true
            },
            _records_grid: {
              disallowSort: true
            },
            _records_grid__fn_sort: {
              disabled: true
            },
            _records_grid_paginate_length_select: {
              disabled: true
            },
            _records_grid__fn_paginate: {
              disabled: true
            },
            _records_grid_pagination: {
              kind: 'indeterminate'
            }
          },
          $bindings: [
            `_records_grid_pagination:jump-to-ended =>> prefab:invokeLink`,
            `${readIOName} =>> prefab:enablePaginationControls <=< ${readIOName}:metadata`
          ]
        }
      }
    });
  }
  enablePaginationControls(metadata) {
    const links = ifn.get('links', metadata);
    const paginationEl = this.$._records_grid_pagination;

    if (links) {
      paginationEl.toggleDisabledIndeterminate(links);
    }
  }

  __includeFilterBehaviors(script) {
    const filterIOName = ifn.first(ifn.get('behavior.filter', script) || []);
    const filterIO = ifn.get(`io.${filterIOName}`, script);

    const restIOName = ifn.first(ifn.get('behavior.rest', script) || []);
    const restIO = ifn.get(`io.${restIOName}`, script);

    const readIOName = !restIOName || !restIO ? filterIOName : `read_${restIOName}`;

    const isDefaultFilterEnabled = filterIOName && filterIO;

    return isDefaultFilterEnabled
      ? this.__enableDefaultFilter(script, readIOName)
      : ifn.mergeWith('append', script, {
          action: {
            filter_applied: {
              $finish: ['{ _records_filter__fn_filter.value }', '{ _records_filter.flush ~@ }']
            }
          }
        });
  }

  __enableDefaultFilter(script, readIOName) {
    return ifn.mergeWith('append', script, {
      life_cycle: {
        initialize: {
          $bindings: [`{ filter_applied } =>> { prefab.clearSort }`]
        }
      },
      action: {
        filter_applied: {
          $bindings: [`{ filter_applied } =>> { ${readIOName} }`]
        }
      }
    });
  }

  __includeSortBehaviors(script) {
    const sortIOName = ifn.first(ifn.get('behavior.sort', script) || []);
    const sortIO = ifn.get(`io.${sortIOName}`, script);

    const restIOName = ifn.first(ifn.get('behavior.rest', script) || []);
    const restIO = ifn.get(`io.${restIOName}`, script);

    const readIOName = !restIOName || !restIO ? sortIOName : `read_${restIOName}`;

    const isDefaultFilterEnabled = sortIOName && sortIO;

    return isDefaultFilterEnabled ? this.__enableDefaultSort(script, readIOName) : script;
  }

  __enableDefaultSort(script, readIOName) {
    return ifn.mergeWith('append', script, {
      life_cycle: {
        initialize: {
          $elements: {
            _records_grid: {
              disallowSort: false
            }
          }
        }
      },
      action: {
        sort_applied: {
          $output: [`{ ${readIOName} }`]
        }
      }
    });
  }

  serializeSortParams(params) {
    const prefix = this.__sortOrderSign(params.sortOrder);
    const sortBy = this.__sortByField(params.sortBy);

    return {
      ordering: `${prefix}${sortBy}`
    };
  }

  __sortOrderSign(sortOrder) {
    return sortOrder === 'desc' ? '-' : '';
  }

  __sortByField(sortBy) {
    return sortBy == null ? '' : sortBy;
  }

  clearSort() {
    const gridEl = HIP.db.gid.queryOne('_records_grid', this.ns);
    const ioEl = HIP.db.gid.queryOne(this.readIOName, this.ns);

    // reset interface
    gridEl.sortBy = undefined;
    gridEl.sortOrder = 'asc';

    // reset uri
    const [base] = ioEl.uri.split('?');
    ioEl.uri = base;
  }

  /**
   * @param {*} records bucket output
   */
  calculateNewStateOfPrefab(records) {
    if (records == null || records.length === 0) {
      return 'empty';
    }
    return 'main';
  }

  /**
   * invoke a link on the `metadata.links` dictionary
   * @param {*} linkName the link name
   */
  invokeLink(linkName) {
    const ioEl = HIP.db.gid.queryOne(this.readIOName, this.ns);
    if (!ioEl) {
      return Promise.resolve({});
    }

    ioEl.disabledExpressions = true;

    const link = this.goToLink(linkName);
    if (!link) {
      return Promise.resolve(ioEl.output);
    }

    const didChangingURIInvoke = ioEl.auto === true;
    return didChangingURIInvoke ? Promise.resolve(ioEl.output) : ioEl.invoke(ioEl.arguments);
  }

  goToLink(linkName) {
    const io = HIP.db.gid.queryOne(this.readIOName, this.ns);
    if (!io) {
      return '';
    }

    const linkURI = ifn.get(`metadata.links.${linkName}`, io);

    return linkURI != null ? (io.uri = linkURI) : '';
  }

  __includeRestBehaviors(script) {
    const restIOName = ifn.first(ifn.get('behavior.rest', script) || []);
    const restIO = ifn.get(`io.${restIOName}`, script);

    if (!restIOName || !restIO) {
      return script;
    }

    script = this.__browseRestIO(script, restIO, restIOName);

    const restConfigure = ifn.mergeWith(
      'clobber',
      {
        // - tasks_menu, load update and delete in a menu
        // - tasks, load update and delete in row
        // aesthetic: 'tasks_menu',
        aesthetic: 'tasks_menu',
        record_base_uri: '{ location[parent_hash] }',
        uri_scheme: 'single-record',
        record_view_name_key: '.title',
        record_navigation_key: 'id',
        requested_verbs: 'crude',
        record_name: 'Record',
        uri_qualifier: '',
        record_ref: '' // use `{ .id }` to lookup the key to send to the editor (replaces record_navigation_key)
      },
      ifn.get(`io.${restIOName}.$additional`, script) || {},
      ifn.get(`io.${restIOName}.$configure.rest`, script) || {}
    );

    if (restConfigure.recordBaseURI != null) {
      restConfigure.record_base_uri = restConfigure.recordBaseURI;
    }

    if (restConfigure.recordViewNameKey != null) {
      restConfigure.record_view_name_key = restConfigure.recordViewNameKey;
    }
    if (restConfigure.record_view_name_key[0] !== '.') {
      // support key format (we need .) but should not be fixed at this level
      restConfigure.record_view_name_key = `.${restConfigure.record_view_name_key}`;
    }

    if (restConfigure.recordNavigationKey != null) {
      restConfigure.record_navigation_key = restConfigure.recordNavigationKey;
    } else if (restConfigure.recordId != null) {
      restConfigure.record_navigation_key = restConfigure.recordId;
    } else if (restConfigure.requestId != null) {
      restConfigure.record_navigation_key = restConfigure.requestId;
    }

    if (restConfigure.requestedVerbs != null) {
      restConfigure.requested_verbs = restConfigure.requestedVerbs;
    }

    if (restConfigure.recordName != null) {
      restConfigure.record_name = restConfigure.recordName;
    }

    const verbs = ifn.get('requested_verbs', restConfigure).toLowerCase();
    const hasVerb = this.__hasVerb;

    // TODO add type based on methods - need to handle merging keys

    ifn.each(
      (restVerb) => {
        if (hasVerb(verbs, restVerb)) {
          script = this.__operationRestIO(script, restIO, restVerb, restConfigure);
        }
      },
      ['create', 'read', 'update', 'delete', 'edit']
    );

    return script;
  }

  __browseRestIO(script, restIO, restIOName) {
    return ifn.mergeWith('append', script, {
      io: {
        [`read_${restIOName}`]: {
          uri: `${restIO.uri}`,
          $output: '{ |> records }',
          $startup: {
            $life_cycle: 'initialize'
          }
        },
        [restIOName]: {
          $is: 'memory',
          $finish: [`{ read_${restIOName} }`]
        }
      }
    });
  }

  __hasVerb(verbs, letter) {
    return verbs.indexOf(letter[0]) >= 0;
  }

  __operationRestIO(script, restIO, restVerb, restConfigure) {
    const aesthetic = ifn.get('aesthetic', restConfigure) || '';
    const operationRegionIOFn = this[`__operationRestIO__${aesthetic}`];

    return ifn.is(Function, operationRegionIOFn)
      ? operationRegionIOFn.call(this, script, restIO, restVerb, restConfigure)
      : script;
  }

  __operationRestIO__tasks_menu(script, restIO, restVerb, restConfigure) {
    const uriQualifier = ifn.get('uri_qualifier', restConfigure) || '';
    const recordNavigationKey = ifn.get('record_navigation_key', restConfigure);
    const recordNavigationReference = ifn.get('record_ref', restConfigure);
    const id =
      restVerb === 'create'
        ? 'new'
        : recordNavigationReference != null
        ? recordNavigationReference
        : `(( ${restVerb}_record.${recordNavigationKey} ))`;

    const input_mode = restVerb === 'create' ? 'none' : 'singular';

    const prefabName = restVerb === 'read' || restVerb === 'delete' ? 'viewer' : 'editor';

    // they will already have this and we should not add a second
    //
    // if (restVerb === 'edit' || restVerb === 'delete' || restVerb === 'update') {
    //   script = ifn.mergeWith('append', script, {
    //     type: {
    //       records_type: {
    //         properties: {
    //           actions: {
    //             name: 'actions',
    //             sortable: false,
    //             render_type: 'tasks_menu',
    //             $region: 'record_row_menu'
    //           }
    //         }
    //       }
    //     }
    //   });
    // }

    const $region = restVerb === 'create' ? 'records' : restVerb === 'read' ? 'record_row' : 'record_row_menu';

    const title =
      (restVerb === 'read' ? restConfigure.record_view_name_key : '') ||
      `${f1.capitalize(restVerb)} ${restConfigure.record_name}`;

    const breadURI = navigation.uri.concatenate([
      restConfigure.record_base_uri,
      prefabName,
      restVerb,
      `${id}`,
      uriQualifier
    ]);

    return ifn.mergeWith('append', script, {
      action: {
        [`${restVerb}_record`]: {
          $widget: {
            $is: 'link',
            icon: 'link',
            title,
            $region,
            input_mode
          },
          $do: `=>> { location.goto } <-< ${breadURI}`
        }
      }
    });
  }

  __operationRestIO__tasks(script, restIO, restVerb, restConfigure) {
    const restIOName = ifn.first(ifn.get('behavior.rest', script) || []);
    const uriQualifier = ifn.get('uri_qualifier', restConfigure) || '';
    const recordNavigationKey = ifn.get('record_navigation_key', restConfigure);
    const recordNavigationReference = ifn.get('record_ref', restConfigure);
    const id =
      restVerb === 'create'
        ? 'new'
        : recordNavigationReference != null
        ? recordNavigationReference
        : `(( ${restVerb}_record.${recordNavigationKey} ))`;

    const input_mode = restVerb === 'create' ? 'none' : 'singular';

    const prefabName = restVerb === 'read' || restVerb === 'delete' ? 'viewer' : 'editor';

    const $region = restVerb === 'create' ? 'records' : restVerb === 'read' ? 'record_row' : 'record_row_tasks';

    let breadURI;
    switch (restConfigure.uri_scheme) {
      case 'parent-child':
        /*eslint no-undef: 0*/
        breadURI = navigation.uri.concatenate([
          restConfigure.record_base_uri,
          `${id}`,
          `editor`,
          restVerb,
          uriQualifier
        ]);
        break;
      case 'single-record':
      default:
        /*eslint no-undef: 0*/
        breadURI = navigation.uri.concatenate([
          restConfigure.record_base_uri,
          prefabName,
          restVerb,
          `${id}`,
          uriQualifier
        ]);
        break;
    }

    let $doGoTo = `=>> { location.goto } <-< ${breadURI}`;
    let $is = 'button';
    let icon = 'link';
    let layout = '';
    let title = `${f1.capitalize(restVerb)} ${restConfigure.record_name || ''}`;
    let kind = '';

    switch (restVerb) {
      case 'update':
        icon = 'edit';
        kind = 'icon';
        layout = 'icon';
        break;
      case 'create':
        icon = 'plus';
        break;
      case 'delete':
        $doGoTo = `{ delete_${restIOName} }`;
        $is = 'button';
        icon = 'trash';
        kind = 'icon';
        layout = 'icon';
        break;
      case 'read':
        title = restConfigure.record_view_name_key;
        break;
      default:
        break;
    }

    if (restVerb === 'delete') {
      const breadDeleteURI = navigation.uri.concatenate([restIO.uri, `${restConfigure.record_ref}@trailing_slash`]);

      script = ifn.mergeWith('append', script, {
        io: {
          [`delete_${restIOName}`]: {
            uri: breadDeleteURI,
            $finish: `{ read_${restIOName} ~@ }`,
            method: 'delete',
            manual: true,
            allowFail: true
          }
        }
      });
    }

    // TODO what if the type already has actions?
    if (restVerb === 'edit' || restVerb === 'delete' || restVerb === 'update') {
      script = ifn.mergeWith('append', script, {
        type: {
          records_type: {
            $mixin: [
              {
                type: '#/type/record_manager__action__records_type'
              }
            ]
          },
          record_manager__action__records_type: {
            properties: [
              {
                name: 'actions',
                $region: 'record_row_tasks',
                render_type: 'tasks',
                sortable: false,
                order: 9999
              }
            ]
          }
        }
      });
    }

    return ifn.mergeWith('append', script, {
      action: {
        [`${restVerb}_record`]: {
          $do: [$doGoTo],
          $widget: {
            $is,
            icon,
            title,
            $region,
            input_mode,
            kind,
            layout
          }
        }
      }
    });
  }

  /**
   * if there are singular or bulk tasks, then show grid checkbox selections
   */
  __needsGridSelectionRows(context) {
    const source = context.input;
    const recordsTasks = ifn.get('region.records.tasks', source) || {
      singular: [],
      bulk: []
    };

    return recordsTasks.bulk.length > 0 || recordsTasks.singular.length > 0;
  }
}

HIP.factory.element(PrefabRecordManagerElement);
