import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import '../../../aip/element/ai-behavior-prefab/ai-behavior-prefab.js';
import '../../../eip/element/hi-component/hi-component.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';

/**
 * @customElement
 * @polymer
 * @appliesMixin BehaviorPrefab
 * @appliesMixin BehaviorElement
 */
class PrefabRecordViewerElement extends HIP.BehaviorPrefab(HIP.BehaviorElement(PolymerElement)) {
  static get is() {
    return 'ai-prefab-record-viewer';
  }

  static get template() {
    return html`
      <ai-screen gid="_screen">
        <ai-state gid="_state_main" states="main" l-size="12" class="display-block">
          <section l-kind="hort" l-width-fixed>
            <hi-button
              id="_delete_record__button"
              gid="_delete_record__button"
              size="small"
              theme="red"
              class="mtm mlsx"
              hidden
            >
              [[ context.record__delete_button__label ]]
            </hi-button>
          </section>

          <section id="_template_target" gid="_template_target"></section>

          <ai-builder-template
            id="_template__builder"
            gid="_template__builder"
            for="_template__render"
          ></ai-builder-template>
          <hi-render id="_template__render" gid="_template__render" for="_template_target"></hi-render>

          <hi-binding source="context" field="template" sink="_template__builder">
            <di-fn-get path="template"></di-fn-get>
          </hi-binding>
          <hi-binding source="record_type" sink="_template__builder"> </hi-binding>

          <hi-binding source="record" sink="_template__render"> </hi-binding>
          <!-- force render -->
          <!-- this is excessive, but if everything is not available we do not render -->
          <hi-binding source="context" sink="_template__render" field="flush" stop-value> </hi-binding>
          <hi-binding source="record_type" sink="_template__render" field="flush" stop-value> </hi-binding>
          <!-- force render -->

          <hi-binding source="_delete_record__button" sink="delete_record"> </hi-binding>
        </ai-state>
      </ai-screen>
    `;
  }

  get defaultScript() {
    return {
      system: {
        state: 'main'
      },
      life_cycle: {
        initialize: {
          $elements: {
            _delete_record__button: {
              hidden: true
            }
          }
        }
      },
      state: {
        main: {
          $finish: ['(( _template__render ))']
        }
      },
      action: {
        delete_record: {}
      },
      bucket: {
        context: {
          $reactive_values: {
            template: 'definition-inline',
            record__delete_button__label: 'Delete Record'
          }
        },
        record: {}
      }
    };
  }

  includeBehaviors(script) {
    const restIOName = ifn.first(ifn.get('behavior.rest', script) || []);
    const restIO = ifn.get(`io.${restIOName}`, script);
    const restMethod = ifn.get(`system.mode`, script);

    if (!restIOName || !restIO) {
      return script;
    }

    const restConfigure = ifn.merge(
      {
        record_base_uri: 'auto'
      },
      ifn.get(`io.${restIOName}.$configure.rest`, script) || {}
    );

    script = this.__linkIncludeBehavior(script, restIO, restIOName, restConfigure);
    script = this.__readIncludeBehavior(script, restIO, restIOName, restConfigure);

    if (!restMethod) {
      return script;
    }

    return restMethod === 'delete' ? this.__deleteIncludeBehavior(script, restIO, restIOName, restConfigure) : script;
  }

  __deleteIncludeBehavior(script, restIO, restIOName) {
    return ifn.merge(script, {
      io: {
        [restIOName]: {
          method: 'delete',
          uri: `${restIO.uri}/${this.__getRecordKey(restIO)}@trailing_slash`,
          $input: '(( delete_record ))',
          $do: [`=>> (( location:back ))`]
        }
      },
      life_cycle: {
        initialize: {
          $elements: {
            _delete_record__button: {
              hidden: false
            }
          }
        }
      }
    });
  }

  __linkIncludeBehavior(script, restIO, restIOName, restConfigure) {
    const baseURI =
      restConfigure.record_base_uri === 'auto'
        ? ifn.first(location.hash.split('/viewer'))
        : restConfigure.record_base_uri;

    return ifn.merge(script, {
      action: {
        go_to_editor: {
          $do: [`=>> location:goto <=< ${baseURI}/editor/edit/${this.__getRecordKey(restIO)}`]
        },
        go_to_manager: {
          $do: [`=>> location:goto <=< ${baseURI}/manager/`]
        }
      }
    });
  }

  __readIncludeBehavior(script, restIO, restIOName) {
    return ifn.merge(script, {
      io: {
        [`read_${restIOName}`]: {
          uri: `${restIO.uri}/${this.__getRecordKey(restIO)}@trailing_slash`,
          $output: '(( record ))',
          $startup: {
            $state: 'main'
          }
        }
      }
    });
  }

  /**
   * get the record key to lookup the record on
   * @param {*} restIO
   */
  __getRecordKey(restIO) {
    // @deprecated
    const recordId =
      ifn.get('$configure.rest.record_id', restIO) ||
      ifn.get('$configure.rest.recordPath', restIO) ||
      ifn.get('$configure.rest.recordId', restIO) ||
      ifn.get('$additional.recordId', restIO) ||
      ifn.get('$additional.key', restIO);

    if (recordId != null) {
      const reducedRecordId = recordId.replace('((', '').replace('))', '');

      return `(( ${reducedRecordId} ))`;
    }

    // something like `{ :id }` instead of `location.paramters.id`
    return ifn.get('$configure.rest.record_ref', restIO);
  }

  _alterContext(context) {
    context = super._alterContext(context);

    if (!context.related_link) {
      context.related_title = '';
    }

    return context;
  }
}

HIP.factory.element(PrefabRecordViewerElement);
