import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import '../../../eip/element/hi-component/hi-component.js';
import '../../../eip/element/hi-binding/hi-binding.js';
import '../../../eip/element/hi-pipe/hi-pipe.js';
import '../../../eip/element/hi-button/hi-button.js';
import '../../../eip/element/hi-alert/hi-alert.js';
import '../../../aip/element/ai-behavior-prefab/ai-behavior-prefab.js';
import '../../../aip/element/ai-screen/ai-screen.js';
import '../../../aip/element/ai-state/ai-state.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';

/**
 * @customElement
 * @polymer
 * @appliesMixin BehaviorPrefab
 * @appliesMixin BehaviorElement
 */
class PrefabSearchElement extends HIP.BehaviorPrefab(HIP.BehaviorElement(PolymerElement)) {
  static get is() {
    return 'ai-prefab-search';
  }

  static get template() {
    return html`
      <ai-screen gid="_screen">
        <ai-state gid="_view_search" states="main">
          <div l-kind l-width-fixed>
            <div
              id="_search_form_loading"
              l-no-wrap
              l-kind="hort"
              l-align="center"
              l-justify="center"
              class="mtl width-100"
            >
              <hi-indicator chameleon class="mrm"></hi-indicator>
              Loading Search Form!
            </div>
            <div class="prefab-search-form">
              <!-- Form Fields -->
              <hi-form
                id="_search_form"
                class="max-w-360 min-w-360 ptz"
                gid="_search_form"
                aesthetic="invisible"
                disable-http=""
                hidden
              >
                <ai-builder-form gid="_search_form_builder"> </ai-builder-form>
                <hi-button gid="_search_button" type="submit" slot="controls">
                  [[ context.search_button_label ]]
                </hi-button>
                <hi-button gid="_reset_button" type="reset" slot="controls" aesthetic="secondary">
                  [[ context.reset_button_label ]]
                </hi-button>
              </hi-form>
            </div>
            <div class="prefab-search-results width-100">
              <section aria-live="polite" aria-atomic="true">
                <div class="pam text-sz-l min-w-360" id="_search_results_heading" hidden>
                  <div l-kind l-width-fixed l-justify="between" l-align="center">
                    <div class="text-sz-l text-bold">
                      <hi-label gid="_search_results_count_label"></hi-label>
                      <span>[[ context.results_title ]]</span>
                    </div>
                    <div>
                      <ai-block gid="_header_records" region="records_header">
                        <div id="_header_records__none_task__widget_group" hidden>
                          <ai-builder-widget gid="_header_records__none_task__widget_group__builder">
                          </ai-builder-widget>
                        </div>
                        <div
                          id="_header_records__singular_task__widget_group"
                          l-kind
                          l-width-fixed
                          l-align="center"
                          hidden
                        >
                          <ai-builder-widget gid="_header_records__singular_task__widget_group__builder">
                          </ai-builder-widget>
                        </div>
                        <div id="_header_records__bulk_task__widget_group" l-kind l-width-fixed l-align="center" hidden>
                          <div id="_selected_records_info" hidden>
                            <hi-label gid="_selected_records_count_label"></hi-label>
                            <span class="mrsxxx">[[ context.selected_title ]]</span>
                          </div>
                          <ai-builder-widget gid="_header_records__bulk_task__widget_group__builder">
                          </ai-builder-widget>
                        </div>
                      </ai-block>
                    </div>
                  </div>
                  <div
                    id="_warning_message_no_selected_records"
                    class="clr-red_30-txt text-sz-m text-right text-italic ptsx"
                    hidden
                  >
                    [[ context.error_message_no_selected ]]
                  </div>
                </div>
              </section>
              <div l-kind l-justify="center" class="pam" id="_search_results_placeholder" hidden>
                <span class="pal clr-gray_10-bkg max-w-480 text-center bdr-round-e mtl">
                  <hi-icon style="font-size:12em" kind="search" theme="gray-light"></hi-icon>
                  <p class="text-sz-l text-bold mtl">[[ context.results_placeholder_title ]]</p>
                  <p class="text-sz-m text-italic clr-gray_80-txt">[[ context.results_placeholder_tip ]]</p>
                </span>
              </div>
              <div l-kind l-justify="center" class="pam" id="_search_results_empty" hidden>
                <span class="pal clr-gray_10-bkg max-w-480 text-center bdr-round-e mtl">
                  <hi-icon style="font-size:12em" kind="warning" theme="gray-light"></hi-icon>
                  <p class="text-sz-l text-bold mtl">[[ context.no_result_title ]]</p>
                  <p class="text-sz-m text-italic clr-gray_80-txt">[[ context.no_result_tip ]]</p>
                </span>
              </div>
              <div class="pos-relative pam ptz height-100">
                <div class="min-w-360 pin-top pos-absolute width-100" id="_search_results_loading_indicator" hidden>
                  <div class="ptm" l-kind="hort" l-align="center" l-justify="center" l-width-fixed>
                    <hi-indicator chameleon class="mrm"></hi-indicator>
                    Fetching results!
                  </div>
                </div>
                <!-- Results Grid -->
                <hi-grid
                  gid="_search_results_grid"
                  theme="[[context.results_grid_theme]]"
                  sticky-header=""
                  aesthetic="standard"
                  hidden
                >
                  <di-fn-sort field="[[context.sorting.default_sort_key]]"></di-fn-sort>
                  <ai-builder-grid gid="_search_results_builder"> </ai-builder-grid>
                </hi-grid>
                <!-- Pagination -->
                <div l-kind="hort" l-justify="right" l-width-fixed>
                  <hi-pagination
                    gid="_search_results_pagination"
                    page-index="{{context.pagination.page_index}}"
                    range-size="[[context.pagination.range_size]]"
                    page-size="[[context.pagination.page_size]]"
                    hidden
                  >
                  </hi-pagination>
                </div>
              </div>
            </div>
          </div>
        </ai-state>
        <ai-state kind="logic" gid="_external_interface" l-size="0">
          <hi-binding
            source="selected_records"
            signal="output-changed"
            sink="_header_records__bulk_task__widget_group__builder"
            field="value"
          >
          </hi-binding>
          <hi-binding source="_search_results_grid" signal="values-changed" sink="selected_records">
            <hi-pipe sink="_selected_records_count_label" field="value"> output=input.length || 'None' </hi-pipe>
          </hi-binding>
          <hi-binding source="source" sink="noop">
            <hi-pipe sink="_header_records__none_task__widget_group__builder">
              <di-fn-get path="region.records_header.tasks.none"></di-fn-get>
            </hi-pipe>
            <hi-pipe sink="_header_records__singular_task__widget_group__builder">
              <di-fn-get path="region.records_header.tasks.singular"></di-fn-get>
              <hi-pipe sink="_search_results_grid" field="allow-row-select"> output = input.length > 0; </hi-pipe>
            </hi-pipe>
            <hi-pipe sink="_header_records__bulk_task__widget_group__builder">
              <di-fn-get path="region.records_header.tasks.bulk"></di-fn-get>
              <hi-pipe sink="_search_results_grid" field="allow-row-select"> output = input.length > 0; </hi-pipe>
            </hi-pipe>
            <hi-pipe sink="_header_records__bulk_task__widget_group__builder">
              <di-fn-get path="region.records_header.tasks.bulk"></di-fn-get>
              <hi-pipe sink="_selected_records_info" field="hidden"> output = (input.length == 0); </hi-pipe>
            </hi-pipe>
          </hi-binding>
          <!-- bucket bindings -->
          <hi-binding source="form_fields_type" sink="_search_form_builder"> </hi-binding>
          <hi-binding source="results_type" sink="_search_results_builder"> </hi-binding>
          <hi-binding
            source="_search_results_builder"
            signal="allow-columns-changed"
            field="allowColumns"
            sink="_search_results_grid"
          >
          </hi-binding>
          <hi-binding source="_search_form" signal="submit-ended" sink="proxy_search_action"> </hi-binding>
          <hi-binding source="results" signal="output-changed" sink="noop">
            <hi-pipe sink="_search_results_grid" field="clearValues"></hi-pipe>
            <hi-pipe sink="_search_results_grid">
              <di-fn-get path="results"></di-fn-get>
            </hi-pipe>
            <hi-pipe sink="_search_results_pagination" field="max">
              <di-fn-get path="count"></di-fn-get>
            </hi-pipe>
            <hi-pipe sink="_search_results_count_label" field="value"> output=output.count || 'No' </hi-pipe>
            <hi-pipe sink="_search_results_empty" field="hidden"> output=output.count>0 </hi-pipe>
            <hi-pipe sink="_search_results_grid" field="hidden">
              output=(output.count==undefined||output.count==0)
            </hi-pipe>
            <hi-pipe sink="_search_results_pagination" field="hidden">
              output=(output.count==undefined||output.count==0)
            </hi-pipe>
            <hi-pipe
              pull-source="source"
              sink="_header_records__bulk_task__widget_group"
              field="hidden"
              map-fn="[[ __hideBulkAction ]]"
            >
            </hi-pipe>
          </hi-binding>
          <!-- Store searched form data for pagination/sort query-->
          <hi-binding source="search_action" sink="noop">
            <hi-pipe sink="form_data"></hi-pipe>
            <hi-pipe sink="noop" inject='["_search_results_grid"]'> _search_results_grid.sortBy = undefined </hi-pipe>
            <hi-pipe constant="0" sink="_search_results_pagination" field="pageIndex"> </hi-pipe>
          </hi-binding>
          <!-- Pagination uses from and size params. Using current search query to build pagination -->
          <hi-binding
            gid="_search_pagination_binding"
            source="_search_results_pagination"
            signal="page-index-changed"
            pull-source="form_data"
            sink="paginate_results"
            inject='["context"]'
          >
            const pageSize = context.input.pagination.page_size || 50; output={...output, from: $source.pageIndex *
            pageSize, size: pageSize}
          </hi-binding>
          <!-- Sorting -->
          <!-- Sorting uses sort=prop:asc/desc. Using current search query to build sorting -->
          <hi-binding source="_search_results_grid" signal="request-sort-ended" pull-source="form_data" sink="noop">
            output={...output, sort: \`\${event.detail.sortBy}:\${event.detail.sortOrder}\`}
            <hi-pipe sink="form_data"></hi-pipe>
            <hi-pipe sink="sort_results"></hi-pipe>
            <hi-pipe constant="0" sink="_search_results_pagination" field="pageIndex"> </hi-pipe>
          </hi-binding>
          <!-- action bindings -->
          <hi-binding source="_reset_button" sink="noop">
            <hi-pipe sink="_search_form" field="reset"></hi-pipe>
            <hi-pipe sink="clear_action"></hi-pipe>
          </hi-binding>
        </ai-state>
      </ai-screen>
    `;
  }

  get defaultScript() {
    return {
      system: {
        state: 'main'
      },
      life_cycle: {
        initialize: {
          $do: [
            '=>> _search_results_loading_indicator:hidden <=< true',
            '=>> _search_results_heading:hidden <=< true',
            '=>> _search_results_empty:hidden <=< true',
            '=>> _search_results_grid:hidden <=< true',
            '=>> _search_results_pagination:hidden <=< true',
            '((search_request))',
            '((search_response))'
          ]
        }
      },
      action: {
        proxy_search_action: {
          $finish: ['{search_action}'],
          $is: 'proxy'
        },
        search_action: {
          $do: [
            '=>> _search_results_placeholder:hidden <=< true',
            '=>> _search_results_heading:hidden <=< true',
            '=>> _search_results_loading_indicator:hidden <=< false',
            '=>> _search_results_empty:hidden <=< true',
            '((search_query))'
          ]
        },
        paginate_results: {
          $do: ['=>> _search_results_loading_indicator:hidden <=< false', '((search_query))']
        },
        sort_results: {
          $do: ['=>> _search_results_loading_indicator:hidden <=< false', '((search_query))']
        },
        clear_action: {}
      },
      bucket: {
        context: {
          $reactive_values: {
            search_button_label: 'Search',
            reset_button_label: 'Clear',
            results_title: 'results found',
            search_title: 'Search',
            selected_title: 'selected',
            results_grid_theme: 'blue',
            no_result_title: 'Nothing Found!',
            no_result_tip: 'Check your keywords or try adjusting the filters',
            results_placeholder_title: 'Search with keywords to preview results',
            results_placeholder_tip: "Try searching for e.g. 'Math 101'",
            error_message_no_selected: 'Error: Please select at least one record from the Results',
            sorting: {
              default_sort_key: 'id'
            },
            pagination: {
              page_index: 0,
              range_size: 0,
              page_size: 10,
              max: 250
            }
          }
        },
        form_fields: {
          type: {
            $ref: '#/type/search_request'
          }
        },
        results: {
          type: {
            $ref: '#/type/search_response'
          }
        },
        selected_records: {},
        form_data: {}
      },
      io: {
        search_query: {
          method: 'get',
          $do: [
            '=>> _search_results_empty:hidden <=< true',
            '=>> _search_results_loading_indicator:hidden <=< false',
            '=>> _search_results_heading:hidden <=< true',
            '=>> _search_results_grid:hidden <=< true'
          ],
          $done: [
            '=>> _search_results_loading_indicator:hidden <=< true',
            '=>> _search_results_heading:hidden <=< false',
            '{ results }'
          ]
        },
        search_request: {
          $do: [
            '=>> _search_form:hidden <=< true',
            '=>> _search_results_heading:hidden <=< true',
            '=>> _search_results_placeholder:hidden <=< true',
            '=>> _search_form_loading:hidden <=< false'
          ],
          $done: [
            '=>> _search_form:hidden <=< false',
            '=>> _search_results_heading:hidden <=< true',
            '=>> _search_form_loading:hidden <=< true',
            '=>> _search_results_placeholder:hidden <=< false'
          ]
        },
        search_response: {}
      }
    };
  }

  __hideBulkAction(context) {
    const emptyResults = 0;
    const noResults = ifn.getWithDefault(emptyResults, 'event.detail.value.count', context) === emptyResults;
    if (noResults) {
      return true;
    }
    return ifn.getWithDefault(emptyResults, 'input.region.records_header.tasks.bulk.length', context) === emptyResults;
  }

  constructor() {
    super();
  }
}

HIP.factory.element(PrefabSearchElement);
