import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';

// eslint-disable-next-line prettier/prettier
import navigation from '../../../pip/library/pi-navigation/pi-navigation.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';

/**
 * @customElement
 * @polymer
 * @appliesMixin BehaviorPrefab
 * @appliesMixin BehaviorElement
 */
class PrefabTOCElement extends HIP.BehaviorPrefab(HIP.BehaviorElement(PolymerElement)) {
  static get is() {
    return 'ai-prefab-toc';
  }

  static get template() {
    return html`
      <ai-screen gid="_screen">
        <ai-state gid="_state_main">
          <hi-tile-group layout="grid">
            <hi-repeat gid="_group__repeat" as="group">
              <template>
                <article l-cell l-size="12" l-sml-size="6" l-med-size="4" l-lrg-size="3">
                  <hi-tile>
                    <header class="pbsxx">
                      <h3>[[ group.title ]]</h3>
                    </header>

                    <section>
                      <ul kind="index">
                        <hi-repeat
                          gid="_path__repeat"
                          input="[[ __readNavigationItems(group) ]]"
                          ns$="[[ ns ]]"
                          as="navItem"
                        >
                          <template>
                            <li class="pbsxx" on-click="__onClick__listItem">
                              <hi-link href$="[[ __buildHREF(navItem, group) ]]"> [[ navItem.title ]] </hi-link>
                              <div l-kind="hort" l-width-fixed>
                                <hi-repeat
                                  gid="_path__repeat__tags__repeat"
                                  input="[[ __readItemBadges(navItem) ]]"
                                  ns$="[[ ns ]]"
                                  as="badge"
                                >
                                  <template>
                                    <hi-badge
                                      class="mrsx pbsx text-plain"
                                      theme$="[[ badge.theme ]]"
                                      aesthetic="light"
                                      size="small"
                                    >
                                      [[ badge.title ]]
                                    </hi-badge>
                                  </template>
                                </hi-repeat>
                              </div>
                              <p hidden$="[[ ! navItem.description ]]" class="mtsx text-plain">
                                [[ navItem.description ]]
                              </p>
                            </li>
                          </template>
                        </hi-repeat>
                      </ul>
                    </section>
                  </hi-tile>
                </article>
              </template>
            </hi-repeat>
          </hi-tile-group>

          <hi-binding source="index" sink="_group__repeat"> </hi-binding>
        </ai-state>
        <ai-state kind="logic" gid="_external_interface"> </ai-state>
      </ai-screen>
    `;
  }

  get defaultScript() {
    return {
      system: {
        state: 'main'
      },
      bucket: {
        context: {},
        index: {}
      }
    };
  }

  includeBehaviors(script) {
    this.isDirect = (ifn.get('behavior.direct_uri', script) || []).length > 0;

    return script;
  }

  __onClick__listItem(e) {
    const aTag = e.currentTarget.querySelector('a');
    if (aTag) {
      aTag.click();
    }
  }

  __readNavigationItems(group) {
    return group.routes || group.navigation_items || [];
  }

  __readItemBadges(navItem) {
    return navItem.badges || navItem.flags || [];
  }

  __buildHREF(navItem, groupItem) {
    const uri = (groupItem.uri || '') + (navItem.uri || '');
    const amendedURI = uri.indexOf('http') === 0 ? uri : uri.indexOf('#') === 0 ? uri : `#${uri}`;

    const baseURI = this.isDirect ? uri : amendedURI;
    const finalURI = `${baseURI}${navItem.goto_uri || ''}`;

    /*eslint no-undef: 0 */
    return navigation.uri.write($ip.constants.read(finalURI));
  }
}

HIP.factory.element(PrefabTOCElement);
