import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import '../../../aip/element/ai-behavior-prefab/ai-behavior-prefab.js';
import '../../../eip/element/hi-component/hi-component.js';

// eslint-disable-next-line prettier/prettier
import HIL from '../../../aip/library/ai-transpiler/ai-transpiler.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';

/**
 * @customElement
 * @polymer
 * @appliesMixin BehaviorPrefab
 * @appliesMixin BehaviorElement
 */
class PrefabUploadElement extends HIP.BehaviorPrefab(HIP.BehaviorElement(PolymerElement)) {
  static get is() {
    return 'ai-prefab-upload';
  }

  static get template() {
    return html`
      <ai-screen gid="_screen">
        <ai-state gid="_state_main" states="main">
          <hi-upload
            id="_upload"
            gid="_upload"
            class="mtlxx mbmxxx mll mrl"
            on-file-completed-ended="__onFileCompletedEnded"
            on-file-errored-ended="__onFileErroredEnded"
            file-content-key$="[[ context.upload.parameters.required.file_content_key ]]"
            file-name-key$="[[ context.upload.parameters.required.file_name_key ]]"
            parameters="[[ context.upload.parameters.optional ]]"
            headers="[[ context.upload.headers ]]"
            uri$="[[ context.upload.uri ]]"
          >
          </hi-upload>

          <hi-alert id="_read_mode_message" class="mbm" hidden>
            This feature is not available in read only mode
          </hi-alert>

          <h1 class="text-sz-l">[[ context.labels.uploads_list__header ]]</h1>
        </ai-state>
      </ai-screen>
    `;
  }

  get defaultScript() {
    return {
      system: {
        state: 'main'
      },
      action: {
        completed_upload: {},
        errored_upload: {},
        deleted_upload: {}
      }
    };
  }

  includeBehaviors(script) {
    this.__isReadOnly = 'read' === ifn.get('system.mode', script);
    return script;
  }

  _connectedViewCallback() {
    HIP.util.toggleAttribute(this.$._read_mode_message, 'hidden', !this.__isReadOnly, 'hidden');
    HIP.util.toggleAttribute(this.$._upload, 'hidden', this.__isReadOnly, 'hidden');
    this.$._upload.style.display = this.__isReadOnly ? 'none' : 'initial';
  }

  __deletedUpload(event) {
    this.__messageEvent(event, 'deleted_upload');
  }

  __onFileCompletedEnded(event) {
    this.__messageEvent(event, 'completed_upload');
  }

  __onFileErroredEnded(event) {
    this.__messageEvent(event, 'errored_upload');
  }

  __messageEvent(event, actionName) {
    const actionEl = HIP.db.gid.queryOne(actionName, this.ns);

    if (actionName.indexOf('error') >= 0) {
      const errorReasons = ifn.get('detail.error', event);
      const errorsMessage = ifn.reduce(
        (s, o, field) => {
          return s + `(${field}) ${o.join('. ')}`;
        },
        '',
        HIL.dsl.api.conformErrors(errorReasons)
      );

      HIP.fn.notify({
        status: 'error',
        title: 'Upload',
        details: errorsMessage
      });
    } else if (actionEl) {
      const dataElement =
        ifn.get('model.__data.upload', event) || // for deleted
        ifn.get('detail.file', event) || // for completed
        {};

      actionEl.invoke(dataElement);
    }
  }
}

HIP.factory.element(PrefabUploadElement);
