import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import '../../../aip/element/ai-behavior-prefab/ai-behavior-prefab.js';
import '../../../eip/element/hi-component/hi-component.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';

/**
 * @customElement
 * @polymer
 * @appliesMixin BehaviorPrefab
 * @appliesMixin BehaviorElement
 */
class PrefabViewsListElement extends HIP.BehaviorPrefab(HIP.BehaviorElement(PolymerElement)) {
  static get is() {
    return 'ai-prefab-views-list';
  }

  static get template() {
    return html`
      <ai-screen gid="_screen">
        <ai-state gid="_state_sub_region" l-kind$="[[context.alignment]]" states="main">
          <hi-repeat id="_sub_region__repeat" gid="_sub_region__repeat" as="sub_view">
            <template>
              <ai-block gid="[[ __subViewName(sub_view) ]]" ns="[[ ns ]]" region="main"> </ai-block>
            </template>
          </hi-repeat>
        </ai-state>
        <ai-state gid="_external_interface" kind="logic">
          <hi-binding source="region_group" sink="_sub_region__repeat"></hi-binding>
        </ai-state>
      </ai-screen>
    `;
  }

  get defaultScript() {
    return {
      system: {
        state: 'main'
      },
      region: {},
      bucket: {
        context: {
          $reactive_values: {
            alignment: 'vert'
          }
        },
        items: {}
      }
    };
  }

  includeBehaviors(language) {
    const prefabMode = ifn.get('system.mode', language);

    switch (prefabMode) {
      case 'dynamic':
        language = ifn.merge(language, {
          bucket: {
            items: {
              $output: ['(( region_group ))']
            }
          }
        });
        break;

      case 'static':
      default:
        break;
    }

    return language;
  }

  __subViewName(sub_view) {
    const regionTarget = ifn.get('name', sub_view);
    const subViewName = `_sub_view__${regionTarget}`;

    // TODO replace with beforeNextRender (or afterNextRender)
    setTimeout(() => {
      this._createSubView({
        subViewRegionEl: HIP.db.gid.queryOne(subViewName, this.ns),
        regionTarget
      });
    });

    return subViewName;
  }
}

HIP.factory.element(PrefabViewsListElement);
