import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';

/**
 * @customElement
 * @polymer
 * @appliesMixin BehaviorPrefab
 * @appliesMixin BehaviorElement
 */
class PrefabViewsTabbedElement extends HIP.BehaviorPrefab(HIP.BehaviorElement(PolymerElement)) {
  static get is() {
    return 'ai-prefab-views-tabbed';
  }

  static get template() {
    return html`
      <ai-screen gid="_screen">
        <ai-state id="_state_controls" states="main,no_tab" l-kind="hort">
          <hi-tab-group
            id="_tabs__tab_group"
            gid="_tabs__tab_group"
            on-selected-ended="__tabList__onSelectedEnded"
            class="mtmx"
          >
            <hi-repeat gid="_tabs__tab_group__repeat" as="tab">
              <template>
                <hi-tab
                  gid="_tab"
                  ns$="[[ ns ]]"
                  value$="[[ tab.name ]]"
                  for="sub-view-region"
                  disabled$="[[ tab.disabled ]]"
                >
                  [[ tab.title ]]
                </hi-tab>
              </template>
            </hi-repeat>
          </hi-tab-group>

          <hi-binding source="region_group" sink="_tabs__tab_group__repeat" map-fn="[[ __mapRegionGroupToRepeat ]]">
          </hi-binding>

          <hi-binding source="_tabs__tab_group" signal="selected-ended" sink="activated_view">
            <di-fn-get path="detail.value"></di-fn-get>
            <di-fn-init hoist-path="view_name"></di-fn-init>
          </hi-binding>
        </ai-state>

        <ai-state gid="_state_sub_view" states="main" l-kind="vert">
          <ai-block gid="sub-view-region" id="sub-view-region" region="sub_view"> </ai-block>
        </ai-state>

        <ai-state gid="_state_pending" states="no_tab" l-kind="vert">
          <!-- show a message of empty -->
        </ai-state>

        <ai-state kind="logic" gid="_external_interface"> </ai-state>
      </ai-screen>
    `;
  }

  get defaultScript() {
    return {
      system: {
        state: 'main'
      },
      action: {
        activated_view: {}
      },
      bucket: {
        active_view: {
          $do: ['=>> { _tabs__tab_group.value } <=< { active_view.*.view_name }']
        },
        default_view: {
          $do: [
            '=>> { _tabs__tab_group.value } <=< { default_view.*.view_name } <?> null == { active_view.*.view_name }'
          ]
        }
      },
      state: {
        no_tab: {}
      }
    };
  }

  _connectedViewCallback() {
    this._changeViewByLocation();
    this._assignTabNavigation();
  }

  includeBehaviors(script) {
    script = this.__includeDynamicRegions(script);
    return script;
  }

  __includeDynamicRegions(script) {
    const regionBehaviorName = ifn.first(ifn.get('behavior.regions', script) || ifn.get('behavior.tabs', script) || []);
    if (regionBehaviorName == null) {
      return script;
    }

    const regionBehaviorIO = ifn.get(`io.${regionBehaviorName}`, script);
    if (regionBehaviorIO == null) {
      return script;
    }

    return ifn.mergeWith('append', script, {
      io: {
        [regionBehaviorIO]: {
          $output: ['{ region_group }']
        }
      }
    });
  }

  __tabList__onSelectedEnded(e) {
    const regionTarget = e.detail.value;
    if (!regionTarget) {
      return;
    }

    this._createSubView({
      subViewRegionEl: HIP.browser.queryOne('sub-view-region', this.ns),
      regionTarget
    });
  }

  __mapRegionGroupToRepeat(bindingContext) {
    return ifn.filter(
      (region) => region.view && (region.view.uri || !f1.is.empty(region.view.script)),
      bindingContext.input
    );
  }

  _onLocationChanged(location) {
    this._changeViewByLocation(location);
  }

  /**
   * if context requests tabs to connect to navigation
   */
  _assignTabNavigation() {
    const contextEl = HIP.db.gid.queryOne('context', this.ns);
    const context = contextEl.output || {};
    const useNavigationTabs = !!context.use_tab_navigation;

    if (useNavigationTabs) {
      this.$._tabs__tab_group.setAttribute('uri-state', 'tab-state');
    }
  }

  /**
   *
   */
  _changeViewByLocation(location = HIP.db.gid.queryOne('location', this.ns)) {
    if (this._isReady !== true) {
      return;
    }

    const ns = this.ns;
    const activeViewEl = HIP.db.gid.queryOne('active_view', ns);
    const view_name =
      ifn.get('parameters.view_name', location) ||
      ifn.get('parameters.tab_name', location) ||
      ifn.get(`parameters.tab-state`, location);

    if (view_name) {
      activeViewEl.invoke({ view_name });
    }
  }
}

HIP.factory.element(PrefabViewsTabbedElement);
