import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';

/**
 * @customElement
 * @polymer
 * @appliesMixin BehaviorPrefab
 * @appliesMixin BehaviorElement
 */
class PrefabViewsViewElement extends HIP.BehaviorPrefab(HIP.BehaviorElement(PolymerElement)) {
  static get is() {
    return 'ai-prefab-views-view';
  }

  static get template() {
    return html`
      <ai-screen gid="_screen">
        <ai-state gid="_state_sub_view" states="main" l-kind="vert">
          <ai-block gid="_sub_view_region" region="sub_view"> </ai-block>
        </ai-state>

        <ai-state gid="_state_pending" states="no_view">
          <!-- show a message of empty -->
        </ai-state>
      </ai-screen>
    `;
  }

  get defaultScript() {
    return {
      system: {
        state: 'no_view'
      },
      action: {
        activated_view: {
          $input: '{ active_view }'
        }
      },
      bucket: {
        active_view: {
          $do: ['{ prefab.activeView__invokeEnded }']
        }
      },
      state: {
        no_view: {}
      }
    };
  }

  _connectedViewCallback() {
    const ns = this.ns;
    const locationEl = HIP.db.gid.queryOne('location', ns);

    this._changeView(locationEl);
  }

  includeBehaviors(script) {
    script = this.__includeDynamicRegions(script);
    return script;
  }

  __includeDynamicRegions(script) {
    const regionBehaviorName = ifn.first(
      ifn.get('behavior.regions', script) || ifn.get('behavior.views', script) || []
    );
    if (regionBehaviorName == null) {
      return script;
    }

    const regionBehaviorIO = ifn.get(`io.${regionBehaviorName}`, script);
    if (regionBehaviorIO == null) {
      return script;
    }

    return ifn.mergeWith('append', script, {
      io: {
        [regionBehaviorIO]: {
          $output: ['{ region_group }']
        }
      }
    });
  }

  activeView__invokeEnded(value) {
    const regionTarget = ifn.get('view_name', value);
    if (!regionTarget) {
      return;
    }

    this._createSubView({
      subViewRegionEl: HIP.db.gid.queryOne('_sub_view_region', this.ns),
      regionTarget
    });
  }

  /**
   *
   */
  _onLocationChanged(location) {
    this._changeView(location);
  }

  /**
   *
   */
  _changeView(location) {
    if (this._isReady !== true) {
      return;
    }

    const ns = this.ns;
    const activeViewEl = HIP.db.gid.queryOne('active_view', ns);
    const view_name = ifn.get('parameters.view_name', location) || ifn.get('parameters.tab_name', location);

    activeViewEl.invoke({
      view_name
    });
  }
}

HIP.factory.element(PrefabViewsViewElement);
