
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import { BehaviorIO } from '../../../eip/element/hi-behavior-io/hi-behavior-io.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';
export class RelationshipElement extends BehaviorIO(BehaviorElement(AsHIC(PolymerElement))) {
  static get is() {
    return "ai-relationship";
  }
  static get properties() {
    return {
      kind: {
        type: String,
        value: "auto",
        reflectToAttribute: true
      }
    };
  }
  static get valueEvent() {
    return "output-changed";
  }
  static get autoBinding() {
    return {
      signal: "output-changed",
      field: "invoke"
    };
  }
  invoke(input) {
    const typeInstance = HIP.browser.queryOne(`${this.id}_type`, this.ns);
    const typeFields = f1.read("/output/properties", {}, typeInstance);
    const singleStream = Promise.resolve(input);
    return this.__parseTypeRelationshipFields(typeFields, input, singleStream).then(
      (_input) => super.invoke(_input)
    );
  }
  __parseTypeRelationshipFields(typeFields, input, singleStream, depth = { __last_field_name: "" }) {
    const relationships = this.__discoverRelationships(typeFields);
    const relationshipHTTP = ifn.reduce(
      (_singleStream, relationship) => {
        return new Promise((resolve) => {
          _singleStream.then(() => this.__parseRelationship(relationship, input, _singleStream, depth)).then(() => resolve(input));
        });
      },
      singleStream,
      relationships
    );
    return relationshipHTTP;
  }
  __discoverRelationships(typeFields) {
    const ns = this.ns;
    return f1.filter(
      (_) => !!_,
      f1.map((field) => {
        const configuration = f1.read("/$configure/relationship", void 0, field);
        if (!configuration) {
          return void 0;
        }
        const instance = configuration["instance"];
        const instanceEl = ifn.first(HIP.browser.queryAll(instance, ns) || []);
        if (!instanceEl) {
          return void 0;
        }
        return {
          instanceEl,
          field
        };
      }, typeFields)
    );
  }
  __parseRelationship(relationship, input, singleStream, depth) {
    const { instanceEl, field } = relationship;
    const fieldName = field.name;
    const kind = this.__discoverKind(input);
    const data = kind === "collection" ? f1.to.array(input) : [input];
    return f1.reduce(
      (_singleStream, o) => {
        return new Promise((resolve, reject) => {
          _singleStream.then(() => {
            const _depth = Object.assign(
              {},
              {
                [`__${depth.__last_field_name}`]: depth,
                ...o
              },
              {
                __last_field_name: fieldName
              }
            );
            instanceEl.manual = true;
            instanceEl.parameters = _depth;
            const failedURIExpression = "/";
            const isValidURI = failedURIExpression !== (instanceEl.uri || "");
            if (!isValidURI) {
              return Promise.resolve();
            }
            return instanceEl.invoke().then((crossLinkedAPI) => {
              ifn.set(fieldName, o, crossLinkedAPI);
              return field.properties != null ? this.__parseTypeRelationshipFields(field.properties, crossLinkedAPI, _singleStream, _depth) : Promise.resolve(crossLinkedAPI);
            });
          }).then((v) => {
            instanceEl.uri = "";
            return v;
          }).then(resolve).catch(reject);
        });
      },
      singleStream,
      data
    );
  }
  __discoverKind(input) {
    const isArrayOfObjects = f1.is.array(input) && f1.is.record(ifn.first(input));
    return this.kind === "auto" ? isArrayOfObjects ? "collection" : "object" : this.kind;
  }
}
HIP.factory.element(RelationshipElement);
