
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { Debouncer } from '../../../hip/library/polymer/polymer.js';
import { idlePeriod } from '../../../hip/library/polymer/polymer.js';
import { BehaviorContainer } from '../../../eip/element/hi-behavior-container/hi-behavior-container.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import '../../../eip/element/hi-indicator/hi-indicator.js';
import '../../../aip/element/ai-state/ai-state.js';
export class ScreenElement extends BehaviorContainer(BehaviorElement(AsHIC(PolymerElement))) {
  static get is() {
    return "ai-screen";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        .state--error,
        .state--loading {
          background-image: linear-gradient(
            45deg,
            #dddddd 4.55%,
            #ffffff 4.55%,
            #ffffff 50%,
            #dddddd 50%,
            #dddddd 54.55%,
            #ffffff 54.55%,
            #ffffff 100%
          );
          background-size: 15.56px 15.56px;
          border: var(--hi-px-2) solid var(--hi-clr--gray-8);
          justify-content: center;
          align-items: center;
          display: flex;
        }

        .state--error {
          color: var(--hi-clr--error);
        }
        .state--loading {
          color: var(--hi-clr--primary);
        }
      </style>
      <slot id="items"></slot>
      <ai-state states="loading" class="state--loading palx mtl" disabled>
        <div style="text-align: center">
          <h2>Loading...</h2>
        </div>
      </ai-state>
      <ai-state states="error" class="state--error palx mtl" disabled>
        <div style="text-align: center">
          <h2>Error</h2>
          <p>An issue happened when retrieving your data, please try again later...</p>
        </div>
      </ai-state>
    `;
  }
  static get properties() {
    return {
      activeState: {
        type: String,
        notify: true,
        reflectToAttribute: true
      },
      activeMode: {
        type: String,
        notify: true,
        reflectToAttribute: true
      },
      previousState: {
        type: String
      },
      previousMode: {
        type: String
      }
    };
  }
  static get autoBinding() {
    return {
      signal: "state-ended",
      field: "state"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("gid", "_screen, screen");
    this.setAttribute("id", "_screen");
  }
  flush() {
    this.__flushDebouncer = Debouncer.debounce(this.__flushDebouncer, idlePeriod, () => {
      this.__updateScreen();
    });
  }
  state(_activeState) {
    if (_activeState === void 0) {
      return this.activeState;
    } else if (this.activeState === _activeState) {
      return;
    }
    const previousState = this.previousState = this.activeState;
    const activeState = this.activeState = _activeState;
    this.__updateScreen();
    this._dispatchProcess("state", {
      previousState,
      activeState
    });
    return activeState;
  }
  mode(_activeMode) {
    if (_activeMode === void 0) {
      return this.activeMode;
    } else if (this.activeMode === _activeMode) {
      return;
    }
    const previousMode = this.previousMode = this.activeMode;
    const activeMode = this.activeMode = _activeMode;
    this.__relinkStateBindings();
    this._dispatchProcess("mode", {
      previousMode,
      activeMode
    });
    return activeMode;
  }
  _onSlotChanged() {
    super._onSlotChanged();
    this.__updateScreen();
  }
  __updateScreen() {
    if (this.activeState != null) {
      this.__updateChildrenState();
      this.__refreshStateBindings();
    }
  }
  __updateChildrenState(_activeState = this.activeState) {
    const globalStates = this.slotChildren.filter((el) => !el.hasAttribute("states"));
    const shadow = this.shadowRoot;
    const childNodes = f1.to.array(shadow.childNodes);
    const stateStates = [...this.slotChildren, ...childNodes].filter((el) => {
      if (el.hasAttribute) {
        return el.hasAttribute("states");
      }
    });
    globalStates.forEach((el) => {
      if (el.activate) {
        el.activate();
      }
    });
    stateStates.forEach((el) => {
      this.__processStates(el, _activeState);
    });
  }
  __processStates(el, _activeState) {
    const enabledStates = `${el.getAttribute("states")}`.split(",").map((s) => s.trim());
    const isEnabled = enabledStates.reduce((_isEnabled, permittedState) => {
      return _isEnabled || permittedState === _activeState;
    }, false);
    if (isEnabled) {
      if (el.activate) {
        el.activate();
      }
    } else {
      if (el.deactivate) {
        el.deactivate();
      }
    }
  }
  __relinkStateBindings() {
    this.__bindingsDebounce = Debouncer.debounce(this.__bindingsDebounce, idlePeriod, () => {
      this.slotChildren.forEach((el) => {
        if (el.relinkBindings) {
          el.relinkBindings();
        }
      });
    });
  }
  __refreshStateBindings() {
    this.__bindingsDebounce = Debouncer.debounce(this.__bindingsDebounce, idlePeriod, () => {
      this.slotChildren.forEach((el) => {
        if (el.refreshBindings) {
          el.refreshBindings();
        }
      });
    });
  }
}
HIP.factory.element(ScreenElement);
