
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { animationFrame } from '../../../hip/library/polymer/polymer.js';
import { Debouncer } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class StateElement extends BehaviorElement(AsHIC(PolymerElement)) {
  static get is() {
    return "ai-state";
  }
  static get template() {
    return html`
      <style>
        :host {
          width: 100%;
        }
      </style>
      <slot></slot>
    `;
  }
  static get properties() {
    return {
      kind: {
        type: String
      },
      states: {
        type: String
      }
    };
  }
  static get autoBinding() {
    return {
      signal: "disabled-changed",
      field: "disabled"
    };
  }
  static get api() {
    return ["deactivate", "activate"];
  }
  connectedCallback() {
    super.connectedCallback();
    if (this.kind === "logic") {
      this.setAttribute("hidden", "hidden");
    } else {
      this.setAttribute("role", "region");
    }
  }
  activate() {
    this.disabled = false;
  }
  deactivate() {
    this.disabled = true;
  }
  relinkBindings() {
    this.__relinkBindingDebouncer = Debouncer.debounce(this.__relinkBindingDebouncer, animationFrame, () => {
      f1.to.array(this.children).filter((el) => el.nodeName === "HI-BINDING").forEach((el) => {
        el.relink();
      });
    });
  }
  refreshBindings() {
    this.__reflectDisabledState(false);
  }
  _disabledChanged(_disabled) {
    super._disabledChanged(_disabled);
    this.__reflectDisabledState(_disabled);
    if (this.kind == null) {
      this.style.display = _disabled ? "none" : "";
      dom.flipAttr(this, "hidden", _disabled);
    }
  }
  __reflectDisabledState(_disabled) {
    this.__disabledDebouncer = Debouncer.debounce(this.__disabledDebouncer, animationFrame, () => {
      f1.to.array(this.children).filter((el) => el.nodeName === "HI-BINDING").forEach((el) => {
        el.disabled = _disabled;
      });
    });
  }
}
HIP.factory.element(StateElement);
