
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { BehaviorTemplate, AsLitTemplateHILC } from '../../../aip/element/ai-behavior-template/ai-behavior-template.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
export class Template404Element extends BehaviorTemplate(AsLitTemplateHILC(LitElement)) {
  static get is() {
    return "ai-template-404";
  }
  _defaultRuntime() {
    return {
      tasks: {
        view_404: {
          $is: "task"
        }
      }
    };
  }
  _injectInstances() {
    return ["view_404"];
  }
  _render() {
    return html`
      <style>
        main.template-404__main > section.template-404__text_container {
          position: relative;
          z-index: 0;
        }

        .template-404__number_container {
          vertical-align: middle;
          text-align: center;
          font-weight: 800;
          margin: 1em;
        }

        .template-404__number_container > span {
          display: inline-block;
          position: relative;
        }

        .template-404__number_container > span.template-404__number_four {
          font-size: 19em;
          color: var(--hi-clr--red-5);
        }

        .template-404__number_container > span.template-404__number_zero {
          width: 14em;
          height: 14em;
          border-radius: 999px;
          background: linear-gradient(
              -45deg,
              transparent 0%,
              rgba(0, 0, 0, 0.06) 50%,
              transparent 51%,
              transparent 100%
            ),
            linear-gradient(to top right, var(--hi-clr--red-1), var(--hi-clr--red-3), var(--hi-clr--red-5));
          overflow: hidden;
          animation: bgshadow 5s infinite;
        }
        .template-404__number_container > span.template-404__number_zero:before {
          content: '';
          display: block;
          position: absolute;
          transform: rotate(45deg);
          width: 5.5em;
          height: 5.5em;
          background-color: transparent;
          left: 0px;
          bottom: 0px;
          background: linear-gradient(
              95deg,
              transparent 0%,
              transparent 8%,
              rgba(0, 0, 0, 0.07) 9%,
              transparent 50%,
              transparent 100%
            ),
            linear-gradient(
              85deg,
              transparent 0%,
              transparent 19%,
              rgba(0, 0, 0, 0.05) 20%,
              rgba(0, 0, 0, 0.07) 91%,
              transparent 92%,
              transparent 100%
            );
        }
        .template-404__number_container > span.template-404__number_zero:after {
          content: '';
          display: block;
          position: absolute;
          border-radius: 999px;
          width: 8em;
          height: 8em;
          left: 3em;
          bottom: 3em;
          background: #fdfaf5;
          box-shadow: -2px 2px 2px 0px rgba(0, 0, 0, 0.1);
        }

        span.template-404__screen-reader-text {
          position: absolute;
          top: -9999em;
          left: -9999em;
        }

        @keyframes bgshadow {
          0% {
            box-shadow: inset -160px 160px 0px 5px rgba(0, 0, 0, 0.4);
          }
          45% {
            box-shadow: inset 0px 0px 0px 0px rgba(0, 0, 0, 0.1);
          }
          55% {
            box-shadow: inset 0px 0px 0px 0px rgba(0, 0, 0, 0.1);
          }
          100% {
            box-shadow: inset 160px -160px 0px 5px rgba(0, 0, 0, 0.4);
          }
        }
      </style>
      <header class="template-404__header">
        <section class="template-404__number_container">
          <span class="template-404__number_four"> 4 </span>
          <span class="template-404__number_zero">
            <span class="template-404__screen-reader-text"> 0 </span>
          </span>
          <span class="template-404__number_four"> 4 </span>
        </section>
      </header>
      <main class="template-404__main pam">
        <section class="template-404__text_container">
          <h1 class="template-404__text_title text-center">...Oops!</h1>
          <p class="template-404__text_description text-sz-l mtm text-center">
            You didn't break the internet. But we can't find what you are looking for.<br />
            Below are the list of pages to navigate to based on your history:
          </p>
          <div class="template-404__home_btn_container ptm pbm text-center">
            <hi-button aesthetic="primary" @trigger-ended=${this.__getHomePageRoute}>
              <hi-icon kind="home" type="material"></hi-icon>
              <span>Home</span>
            </hi-button>
          </div>
          <div class="template-404__history_list bdr-solid bts clr-gray_20-bdr max-w-600 mla mra ptsxx">
            <ul kind="no-dot">
              ${this.__readHistoryLinks().map(
      (item) => html` <li
                  class="pam bdr-solid bas clr-gray_20-bdr clr-gray_10-bkg text-center"
                >
                  <hi-link href=${item.uri}> ${item.title} </hi-link>
                  <p class="text-sz-s clr-blue_50-txt">${item.time}</p>
                </li>`
    )}
            </ul>
          </div>
        </section>
      </main>
    `;
  }
  __readHistoryLinks() {
    const getHistoryFromStorage = window.sessionStorage.getItem("ahp-history");
    if (getHistoryFromStorage != null) {
      return JSON.parse(getHistoryFromStorage);
    }
    return [];
  }
  __getHomePageRoute() {
    return window.location.href.split("#")[0];
  }
}
export default HIP.factory.element(Template404Element);
