
import { Box } from '../../../aip/library/ai-lucid/ai-lucid.js';
import { html } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { dedupingMixin } from '../../../hip/library/polymer/polymer.js';
import {
  HIP
} from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorBehavior } from '../../../eip/element/hi-behavior-behavior/hi-behavior-behavior.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export function BaseBehaviorTemplateAuth(Base) {
  class BehaviorTemplateAuth2 extends BehaviorBehavior(Base) {
    _defaultRuntime() {
      return {
        tasks: {
          view: {
            model_default: {
              verify_authorization_button_title: "Authorize",
              separator_content: "Or",
              hide_eab_logo: true
            }
          }
        }
      };
    }
    _injectInstances() {
      return ["view", "authorization", "verify_authorization", "authorization_controls"];
    }
    _renderHeader() {
      return html`
        <ai-view-head
          gid="view_head"
          id="view_head"
          slot="header"
          level=${this.level}
          .ns=${this.ns}
          .runtime=${this.runtime}
        >
        </ai-view-head>
      `;
    }
    _renderFooter() {
      return html`
        <ai-view-foot
          gid="view_foot"
          id="view_foot"
          slot="footer"
          level=${this.level}
          .ns=${this.ns}
          .runtime=${this.runtime}
        >
        </ai-view-foot>
      `;
    }
    _renderForm(uniqueId) {
      const ns = this.ns;
      const instances = this._instances;
      const { value, type } = instances.authorization.state();
      const viewValue = instances.view.value();
      return html`<hi-form
          gid="authorization_form"
          form=${uniqueId}
          .ns=${ns}
          @submit-ended=${this.__form__onSubmitEnded}
          layout="flush"
          disable-http
        >
          <hi-panel
            gid="${uniqueId}-form"
            .ns=${ns}
            style="max-height: 35vh; overflow: auto; padding-left: var(--hi-spacing--small-xx)"
          ></hi-panel>
          <hi-button type="submit" slot="controls" class="maz" stretched
            >${viewValue.verify_authorization_button_title}</hi-button
          >
        </hi-form>
        <ai-factory-editor
          for="${uniqueId}-form"
          gid="force-paint"
          form=${uniqueId}
          .ns=${ns}
          .value=${value}
          .type=${type}
          @field-trigger-ended=${(e) => this.runtime.interlink.proxyFieldValueIn(e.detail)}
          @field-state-trigger-ended=${(e) => this.runtime.interlink.proxyFieldStateValueIn(e.detail)}
          include-submit
        ></ai-factory-editor>`;
    }
    _renderSeparator() {
      const instances = this._instances;
      const { value } = instances.view.state();
      const authorizationControlsType = instances.authorization_controls.type();
      const needsSeparator = !f1.is.empty(f1.read("/fields", null, authorizationControlsType));
      return needsSeparator ? html`<hi-separator value=${value.separator_content}></hi-separator>` : nothing;
    }
    _renderEABLogo() {
      const instances = this._instances;
      const { value } = instances.view.state();
      const needsLogo = !value.hide_eab_logo;
      return needsLogo ? html`<section class="mts mbs"></section><hi-eab class="to-center"></hi-eab></section>` : nothing;
    }
    _renderAuthorizationControls(uniqueId) {
      const instances = this._instances;
      const { value, type } = instances.authorization_controls.state();
      const ns = this.ns;
      return html`
        <hi-panel
          gid="${uniqueId}-controls"
          .ns=${ns}
          x-hie
          class="toolbar mtm mbm"
          style="justify-content: center"
        ></hi-panel>
        <ai-factory-control
          gid="force-paint"
          for="${uniqueId}-controls"
          .ns=${ns}
          .type=${type}
          .value=${value}
          @field-trigger-ended=${(e) => this.runtime.interlink.proxyFieldValueIn(e.detail)}
        ></ai-factory-control>
      `;
    }
    __form__onSubmitEnded(e) {
      const instances = this._instances;
      const formEntity = e.detail.value;
      const submitButtonEl = e.currentTarget.$.submitButton;
      if (!instances.verify_authorization) {
        return;
      }
      const authorization = instances.verify_authorization.invoke(formEntity);
      submitButtonEl.waitFor(authorization);
      authorization.then((_) => {
        if (Box.hasError(_)) {
          this._authorize__onFailure(instances.verify_authorization);
        } else {
          this._authorize__onSuccess(instances.verify_authorization);
        }
      }).catch(() => this._authorize__onFailure(instances.verify_authorization));
    }
    _authorize__onSuccess(verifyAuthorizationI) {
    }
    _authorize__onFailure(verifyAuthorizationI) {
      const instanceError = verifyAuthorizationI.error();
      const jobError = f1.read("/context/error", {}, instanceError);
      this.injectErrorsIntoView("authorization_form", jobError);
    }
    _styleRegion() {
      const regionEl = this.closest("[x-region]");
      if (regionEl) {
        regionEl.style.background = "radial-gradient(var(--hi-clr--blue-3), var(--hi-clr--blue-5))";
      }
    }
    _styleCentralBoxPosition() {
      this.className = "display-block bdr-solid bas clr-white-bkg clr-gray_60-bdr width-100 pamx bdr-round-m max-w-480";
      this.style.boxShadow = "var(--hi-tile--shadow)";
      this.style.transform = "translate(-50%, -50%)";
      this.style.position = "absolute";
      this.style.left = "50%";
      this.style.top = "50%";
    }
    _styleCentralBox() {
      this._styleCentralBoxPosition();
      this.style.maxHeight = "90vh";
      this.style.overflowY = "auto";
      this.style.overflowX = "none";
    }
  }
  return HIP.factory.mixin(BehaviorTemplateAuth2);
}
export const BehaviorTemplateAuth = dedupingMixin(BaseBehaviorTemplateAuth);
