
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { BehaviorTemplate, AsLitTemplateHILC } from '../../../aip/element/ai-behavior-template/ai-behavior-template.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import '../../../aip/element/ai-factory-editor/ai-factory-editor.js';
import '../../../eip/element/hi-button/hi-button.js';
import '../../../eip/element/hi-form/hi-form.js';
import { BehaviorTemplateAuth } from './ai-behavior-auth.js';
import ess from '../../../pip/library/pi-ess/pi-ess.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class TemplateAuthLoginElement extends BehaviorTemplateAuth(BehaviorTemplate(AsLitTemplateHILC(LitElement))) {
  constructor() {
    super();
    this.requestNoHeader = true;
    this.requestNoFooter = true;
  }
  static get is() {
    return "ai-template-auth-login";
  }
  _defaultRuntime() {
    const banner_image = "https://attachment.eab.com/wp-content/uploads/2019/07/Student-Success-dark-blue-background.png";
    return f1.merge("over", super._defaultRuntime(), {
      tasks: {
        view: {
          model_default: {
            university_image: "",
            banner_image
          }
        }
      }
    });
  }
  connectedCallback() {
    super.connectedCallback();
    this._styleCentralBox();
    this._styleRegion();
    ess.session("clear");
  }
  _authorize__onSuccess(verifyAuthorizationI) {
    super._authorize__onSuccess(verifyAuthorizationI);
    ess.session("initiate", "success", verifyAuthorizationI.value());
  }
  _authorize__onFailure(verifyAuthorizationI) {
    super._authorize__onFailure(verifyAuthorizationI);
    ess.session("save", "failure", verifyAuthorizationI.error());
  }
  _render() {
    const uniqueId = "auth-login-form";
    const instances = this._instances;
    const viewValue = instances.view.value();
    return html`
      <style>
        .auth-login--uni-image {
          --auth-login--margin: var(--hi-spacing--medium-x);
          --auth-login--margin-neg: calc(-1 * var(--auth-login--margin));

          object-fit: cover;
          height: 7rem;
          margin: var(--auth-login--margin-neg) var(--auth-login--margin-neg) 0 var(--auth-login--margin-neg);
          width: calc(100% + (2 * var(--auth-login--margin)));
          max-width: calc(100% + (2 * var(--auth-login--margin)));
        }

        .auth-login--banner-left {
          --auth-login--offset: var(--hi-spacing--medium-x);

          transform: translate(calc(-100% - var(--auth-login--offset)), 0);
          position: absolute;
          top: 10%;
        }

        .auth-login--banner-right {
          transform: translate(100%, 0);
          position: absolute;
          top: 10%;
        }
      </style>

      <div ?hidden=${!viewValue.banner_image} class="auth-login--banner-left">
        <img
          src=${viewValue.banner_image}
          alt="the left half of a cartoon of people learning and building"
          style="clip-path: polygon(75% 0, 100% 0, 100% 100%, 25% 100%);"
          ?hidden=${!viewValue.banner_image}
        />
      </div>
      <div ?hidden=${!viewValue.banner_image} class="auth-login--banner-right">
        <img
          src=${viewValue.banner_image}
          alt="the right half of a cartoon of people learning and building"
          style="clip-path: polygon(0 0, 75% 0, 25% 100%, 0 100%);"
          ?hidden=${!viewValue.banner_image}
        />
      </div>

      <section>
        <img
          src=${viewValue.university_image}
          alt="the institution building and/or logo"
          class="auth-login--uni-image mbm bdr-round-m bdr-solid bas clr-white-bdr"
          ?hidden=${!viewValue.university_image}
        />

        ${this._renderHeader()} ${this._renderForm(uniqueId)} ${this._renderSeparator()}
        ${this._renderAuthorizationControls(uniqueId)} ${this._renderEABLogo()}
      </section>
      ${this._renderFooter()}
    `;
  }
  _styleCentralBox() {
    super._styleCentralBoxPosition();
  }
}
export default HIP.factory.element(TemplateAuthLoginElement);
