
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { BehaviorTemplate, AsLitTemplateHILC } from '../../../aip/element/ai-behavior-template/ai-behavior-template.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorTemplateAuth } from './ai-behavior-auth.js';
import ess from '../../../pip/library/pi-ess/pi-ess.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class TemplateAuthVerificationElement extends BehaviorTemplateAuth(
  BehaviorTemplate(AsLitTemplateHILC(LitElement))
) {
  constructor() {
    super();
    this.maxCountdown = 3;
    this.requestNoHeader = true;
    this.requestNoFooter = true;
  }
  static get is() {
    return "ai-template-auth-verification";
  }
  _defaultRuntime() {
    return {
      tasks: {
        authorization: {
          $is: "task"
        }
      }
    };
  }
  static get properties() {
    return {
      countdown: {
        type: Number
      }
    };
  }
  _injectInstances() {
    return ["view", "authorization", "verify_authorization"];
  }
  connectedCallback() {
    super.connectedCallback();
    this._styleCentralBox();
    this._styleRegion();
    ess.session("clear");
  }
  _connectedViewCallback() {
    const instances = this._instances;
    if (instances.authorization && f1.read("/job/state/run_count", 0, instances.authorization) > 0) {
      return this._initiateCountdown();
    }
    this.runtime.subscribe({
      id: Symbol(),
      invoke: (job) => {
        if ("authorization" === job.task.id) {
          this._initiateCountdown();
        }
      }
    });
  }
  _render() {
    return html` ${this._renderHeader()}
      <hi-alert mode="presentation">
        <span>Login Verification in Progress...</span>
        <span slot="details">
          ${f1.is.nil(this.countdown) ? html`Your session is being started.` : html`Your session will start in ${this.countdown} seconds.`}
        </span>
      </hi-alert>
      ${this._renderFooter()}`;
  }
  _initiateCountdown() {
    this.countdown = this.maxCountdown;
    const intervalToken = setInterval(() => {
      if (this.countdown > 0) {
        this.countdown -= 1;
        return;
      }
      const instances = this._instances;
      clearInterval(intervalToken);
      ess.session("initiate", "success", instances.authorization.value());
      if (instances.verify_authorization) {
        instances.verify_authorization.invoke();
      }
    }, 1e3);
  }
}
export default HIP.factory.element(TemplateAuthVerificationElement);
