
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { BehaviorTemplate, AsLitTemplateHILC } from '../../../aip/element/ai-behavior-template/ai-behavior-template.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import '../../../vip/element/vi-chart/vi-chart.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class TemplateChartElement extends BehaviorTemplate(AsLitTemplateHILC(LitElement)) {
  static get is() {
    return "ai-template-chart";
  }
  _injectInstances() {
    return ["entities", "entity_details"];
  }
  _defaultRuntime() {
    return {
      startup: [],
      tasks: {
        view: {
          $is: "task",
          model_default: {
            title: "Chart Title"
          }
        },
        entities: {
          $is: "task",
          settings: {
            chart: {
              chart_type: "bar",
              width: "600px",
              height: "600px",
              allow_toggle: false,
              allow_table_action: false,
              table_action_text: "Details"
            }
          }
        },
        entity_details: {
          $is: "task"
        }
      }
    };
  }
  _render() {
    const instanceI = this.runtime.workspace.queryInstance("entities");
    if (!instanceI) {
      return nothing;
    }
    const { type, settings } = instanceI.state();
    const chartSettings = f1.read("/chart", {}, settings);
    const entityValue = instanceI.value();
    const chartGid = `template-chart`;
    const ns = this.ns;
    return html`
      <section id="templateChartContainer" style="max-width:${chartSettings.width}; max-height:${chartSettings.height}">
        <vi-chart
          .ns=${ns}
          gid=${chartGid}
          kind=${chartSettings.chart_type}
          .type=${type}
          .input=${entityValue}
          library="chart"
          mode="hil"
          ?allow-toggle=${chartSettings.allow_toggle}
          ?allow-table-action=${chartSettings.allow_table_action}
          ?table-action-text=${chartSettings.table_action_text}
          @row-action-ended=${this.__chart__onTriggerEnded}
          @trigger-ended=${this.__chart__onTriggerEnded}
        >
        </vi-chart>
      </section>
    `;
  }
  __chart__onTriggerEnded(e) {
    const entityDetailsI = this._instances.entity_details;
    if (!entityDetailsI) {
      return;
    }
    entityDetailsI.invoke(e.detail);
  }
}
export default HIP.factory.element(TemplateChartElement);
