
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { unsafeHTML } from '../../../hip/library/lit-html/directives/unsafe-html.js';
import { BehaviorTemplate, AsLitTemplateHILC } from '../../../aip/element/ai-behavior-template/ai-behavior-template.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class TemplateChecklistElement extends BehaviorTemplate(AsLitTemplateHILC(LitElement)) {
  constructor() {
    super(...arguments);
    this.hasBeenFiltered = false;
    this.selectedItem = void 0;
    this.filteredChecklistItems = [];
  }
  static get is() {
    return "ai-template-checklist";
  }
  static get properties() {
    return {
      hasBeenFiltered: {
        type: Boolean
      },
      selectedItem: {
        type: Object
      },
      drilledIn: {
        type: Boolean,
        attribute: "drilled-in",
        reflect: true
      },
      filteredChecklistItems: {
        type: Array,
        attribute: "filtered-checklist-items"
      }
    };
  }
  _injectInstances() {
    return ["checklistItems", "filter", "completeChecklistItem"];
  }
  _render() {
    const instances = this._instances;
    const checklistItemsValue = f1.safe({}, instances.checklistItems.value());
    const filterValue = f1.safe({}, instances.filter.value());
    const checklistItems = !f1.is.empty(filterValue) ? this.filteredChecklistItems : f1.to.array(checklistItemsValue);
    return html`
      <style>
        .checklist__layout-split-2 {
          --left-panel-width: 34rem;
          --right-panel-width: 100%;
        }

        .checklist__layout-split-2--left-panel {
          display: flex;
          flex-direction: column;
        }

        .checklist__button--complete {
          display: flex;
          justify-content: flex-start;
        }

        .checklist__ring--background {
          background-color: var(--hi-color--white);
          border: 1px solid var(--hi-color--blue-30);
          border-radius: 50%;
          position: absolute;
        }

        .checklist__ring--single--size {
          width: 1rem;
          height: 1rem;
        }

        .checklist__ring--single--position {
          left: -25px;
        }

        .checklist__ring--single--completed {
          border: none;
          left: -27px;
        }

        .checklist__ring--all--size {
          width: 2rem;
          height: 2rem;
        }

        .checklist__ring--all--position {
          left: -14px;
          bottom: 0px;
        }

        .checklist__ring--all--completed {
          border: none;
        }

        .checklist-card {
          display: flex;
          align-items: center;
        }

        .checklist__card--ring-container--circle {
          width: 1rem;
          height: 1rem;
          background-color: var(--hi-color--blue-30);
          border-radius: 50%;
        }

        .checklist__card--ring-container--ring {
          border: 3px solid var(--hi-color--blue-30);
          border-radius: 30px;
          height: 1.5rem;
          width: 1.5rem;
          position: absolute;
          top: -0.25rem;
          left: -0.25rem;
          animation: checklist__pulsate 1s ease-out;
          animation-iteration-count: infinite;
          opacity: 0;
        }

        .checklist__card--ring-container--circle--completed {
          background-color: var(--hi-clr--green-3);
        }

        .checklist__card--ring-container--ring--completed {
          border: 3px solid var(--hi-clr--green-3);
        }

        .checklist__card--content {
          display: flex;
          flex-direction: column;
          justify-content: center;
        }

        .checklist__card--content--header {
          display: flex;
        }

        .checklist__card--controls {
          display: flex;
          justify-content: flex-end;
        }

        .checklist__placeholder {
          font-size: 8em;
          display: flex;
          flex-direction: column;
          height: 100%;
          width: 100%;
          justify-content: center;
          align-items: center;
        }

        @keyframes checklist__pulsate {
          0% {
            transform: scale(0.1, 0.1);
            opacity: 0;
          }
          50% {
            opacity: 1;
          }
          100% {
            transform: scale(1.2, 1.2);
            opacity: 0;
          }
        }
      </style>
      <div>
        <yi-layout-split-2
          class="layout-split-2"
          .auto=${false}
          @trigger-ended=${this.__layout__onTriggerEnded}
          .drilledIn=${this.drilledIn}
          panel-to-show=${this.drilledIn ? "right" : "left"}
        >
          <div slot="leftPanel">
            ${!f1.is.empty(filterValue) ? html`
                  <div class="left-panel-container">
                    <div>
                      <hi-input-group class="width-100">
                        <hi-label>
                          ${filterValue.name}
                          <hi-icon kind="info_circle" id="filter-info"></hi-icon>
                          <hi-tooltip heading=${filterValue.tooltip.heading} for="filter-info" placement="right">
                            ${filterValue.tooltip.body}
                          </hi-tooltip>
                        </hi-label>
                        <hi-combobox
                          id="filter-combobox"
                          .disabled=${filterValue.options.length === 0}
                          placeholder=${filterValue.options.length > 0 ? filterValue.placeholder.optionsExist : filterValue.placeholder.optionsDoNotExist}
                          .input=${filterValue.options}
                        >
                        </hi-combobox>
                      </hi-input-group>
                    </div>
                    <div class="mtm" l-kind="hort" l-justify="left">
                      <div class="min-w-240" l-kind="hort" l-size="3" l-align="start">
                        <hi-button
                          stretched
                          class="mrsx"
                          aesthetic="secondary"
                          @trigger-ended=${this.__submitButton__onTriggerEnded}
                        >
                          Submit
                        </hi-button>
                        <hi-button
                          stretched
                          class="mlsx"
                          aesthetic="ghost"
                          @trigger-ended=${this.__clearButton__onTriggerEnded}
                        >
                          Clear
                        </hi-button>
                      </div>
                    </div>
                  </div>
                ` : nothing}
            ${checklistItems.length > 0 ? html`<div class="bdr-solid blsx clr-blue_30-bdr">
                  ${checklistItems.map((checklistItem, index) => {
      const { id, title, completed } = checklistItem;
      const isSelectedItem = this.selectedItem && this.selectedItem.id === id;
      return html`
                      <div
                        class="checklist-card mam pos-relative${isSelectedItem ? " bdr-solid brsx clr-blue_30-bdr" : ""}"
                      >
                        ${isSelectedItem ? html`
                              <div class="checklist__ring--single--position pos-absolute">
                                <div class="pos-relative">
                                  <div
                                    class="checklist__card--ring-container--ring${completed ? " checklist__card--ring-container--ring--completed" : ""}"
                                  ></div>
                                  <div
                                    class="checklist__card--ring-container--circle${completed ? " checklist__card--ring-container--circle--completed" : ""}"
                                  ></div>
                                </div>
                              </div>
                              ${completed ? html`<hi-icon
                                    class="checklist__ring--background checklist__ring--single--position checklist__ring--single--completed pos-absolute"
                                    kind="success"
                                    theme="green"
                                    size="large"
                                  ></hi-icon>` : html``}
                            ` : html` ${completed ? html`
                                  <hi-icon
                                    class="checklist__ring--background checklist__ring--single--position checklist__ring--single--completed pos-absolute"
                                    kind="success"
                                    theme="green"
                                    size="large"
                                  ></hi-icon>
                                ` : html`<div
                                  class="checklist__ring--background pos-absolute checklist__ring--single--size checklist__ring--single--position"
                                ></div>`}`}

                        <article
                          class="width-100 hi-card hi-card--hort mrm${isSelectedItem ? " bdr-solid basx clr-blue_30-bdr" : ""}"
                        >
                          <div class="checklist__card--content width-100">
                            <div class="checklist__card--content--header">
                              <div class="hi-card__section hi-card__thumbnail">
                                <hi-icon kind="reminder" size="large-x"></hi-icon>
                              </div>
                              <div class="hi-card__section hi-card__header">
                                <p class="text-bold">${title}</p>
                              </div>
                              <div class="checklist__card--controls width-100 hi-card__actions mtm mbm">
                                <hi-button
                                  class="mlsx mrsx"
                                  aesthetic="icon"
                                  @trigger-ended=${this.__viewDetailsButton__onTriggerEnded}
                                  .value=${index}
                                >
                                  <hi-hint placement="left">View${isSelectedItem ? "ing" : ""} Details</hi-hint>
                                  <hi-icon kind="${isSelectedItem ? "eye" : "chevron_right"}"></hi-icon>
                                </hi-button>
                              </div>
                            </div>
                          </div>
                        </article>
                      </div>
                    `;
    })}
                  ${this.__areAllFilteredChecklistItemsCompleted() ? html` <hi-icon
                        class="checklist__ring--background checklist__ring--all--position checklist__ring--all--completed pos-absolute"
                        kind="success"
                        theme="green"
                        size="large-xx"
                      ></hi-icon>` : html`<div
                        class="checklist__ring--background pos-absolute checklist__ring--all--size checklist__ring--all--position"
                      ></div>`}
                </div>` : html`${this.__determineEmptyState()}`}
          </div>
          <div slot="rightPanel" class="height-100 pam">
            ${this.selectedItem ? html`<h3>${this.selectedItem.title}</h3>

                  <div class="mtm mbm">
                    ${this.filteredChecklistItems.length > 0 ? unsafeHTML(this.selectedItem.content) : "No Items Found"}
                  </div>

                  <div class="checklist__button--complete mtm mbsx mlsx">
                    ${this.selectedItem.href && this.selectedItem.label ? html` <hi-button class="mrsx" aesthetic="link" href=${this.selectedItem.href}>
                          ${this.selectedItem.label}
                        </hi-button>` : html``}
                  </div>
                  <div class="checklist__button--complete mtsx mlsx">
                    <hi-button
                      aesthetic="primary"
                      .value=${this.selectedItem.id}
                      @trigger-ended=${this.__selectedItem__onTriggerEnded}
                    >
                      ${this.selectedItem.completed ? "Undo" : "Complete"}
                    </hi-button>
                  </div>` : html`
                  <div class="checklist__placeholder">
                    <hi-icon type="material" kind="search_v2" theme="gray"></hi-icon>
                    <div class="text-sz-m clr-gray_80-txt mtm mbm">Results will show up here.</div>
                  </div>
                `}
          </div>
        </yi-layout-split-2>
      </div>
    `;
  }
  __selectedItem__onTriggerEnded(e) {
    const instances = this._instances;
    const id = e.detail.value;
    if (!instances.completeChecklistItem) {
      return;
    }
    instances.completeChecklistItem.invoke(id);
  }
  __filterResources(value) {
    if (typeof value === "undefined") {
      this.filteredChecklistItems = [];
      this.hasBeenFiltered = false;
    } else {
      this.hasBeenFiltered = true;
    }
    const instances = this._instances;
    const checklistItemsValue = f1.safe({}, instances.checklistItems.value());
    const filterValue = f1.safe({}, instances.filter.value());
    const key = filterValue.key;
    this.filteredChecklistItems = f1.to.array(checklistItemsValue).filter((checklistItem) => {
      return checklistItem[key] === value;
    });
  }
  __submitButton__onTriggerEnded() {
    const filterCombobox = document.getElementById("filter-combobox");
    const value = filterCombobox.value;
    this.drilledIn = false;
    this.__filterResources(value);
    this.selectedItem = void 0;
    this.drilledIn = false;
  }
  __areAllFilteredChecklistItemsCompleted() {
    const instances = this._instances;
    const checklistItemsValue = f1.safe({}, instances.checklistItems.value());
    const filterValue = f1.safe({}, instances.filter.value());
    const checklistItems = !f1.is.empty(filterValue) ? this.filteredChecklistItems : f1.to.array(checklistItemsValue);
    return checklistItems.filter((checklistItem) => {
      return checklistItem.completed;
    }).length === checklistItems.length;
  }
  __viewDetailsButton__onTriggerEnded(event) {
    const index = event.detail.value;
    const instances = this._instances;
    const checklistItemsValue = f1.safe({}, instances.checklistItems.value());
    const filterValue = f1.safe({}, instances.filter.value());
    const checklistItems = !f1.is.empty(filterValue) ? this.filteredChecklistItems : f1.to.array(checklistItemsValue);
    const selectedItem = checklistItems[index];
    this.selectedItem = selectedItem;
    this.drilledIn = true;
  }
  __clearButton__onTriggerEnded() {
    const filterCombobox = document.getElementById("filter-combobox");
    filterCombobox.value = void 0;
    this.selectedItem = void 0;
    this.drilledIn = false;
    this.__filterResources(void 0);
  }
  __layout__onTriggerEnded() {
    this.drilledIn = false;
  }
  __determineEmptyState() {
    const instances = this._instances;
    const checklistItemsValue = f1.safe({}, instances.checklistItems.value());
    const filterValue = f1.safe({}, instances.filter.value());
    const checklistItems = f1.to.array(checklistItemsValue);
    if (!f1.is.empty(filterValue) && this.hasBeenFiltered && checklistItems.length > 0) {
      return html`<hi-alert class="mtm mbm" theme="warning" mode="presentation">
        No Checklist Items Found
        <span slot="details"> There are currently no checklist items for you to review from this institution. </span>
      </hi-alert>`;
    } else {
      return html``;
    }
  }
}
HIP.factory.element(TemplateChecklistElement);
