
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { BehaviorTemplate, AsLitTemplateHILC } from '../../../aip/element/ai-behavior-template/ai-behavior-template.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class TemplateContentElement extends BehaviorTemplate(AsLitTemplateHILC(LitElement)) {
  static get is() {
    return "ai-template-content";
  }
  static get properties() {
    return {
      editing: {
        type: Boolean
      },
      preview: {
        type: String
      }
    };
  }
  get content() {
    const instances = this._instances;
    return f1.safe({}, instances.content.value());
  }
  _injectInstances() {
    return ["content", "submit_content"];
  }
  __handleEditButtonTriggerEnded() {
    this.editing = true;
    this.preview = this.content.article;
  }
  __handleEditorValueChanged(event) {
    this.preview = event.detail.value;
  }
  __handleCancelButtonTriggerEnded() {
    this.editing = false;
    this.preview = this.content.article;
  }
  __handleSaveButtonTriggerEnded() {
    this.editing = false;
    if (this._instances.submit_content) {
      this._instances.submit_content.invoke({ content: this.preview });
    }
  }
  _render() {
    return html`
      <div class="pos-relative">
        ${this.editing ? html`
              <hi-editor
                class="mbm"
                value=${this.content.article}
                @value-changed=${this.__handleEditorValueChanged}
              ></hi-editor>
              <hi-button @trigger-ended=${this.__handleSaveButtonTriggerEnded}>Save</hi-button>
              <hi-button aesthetic="secondary" @trigger-ended=${this.__handleCancelButtonTriggerEnded}
                >Cancel</hi-button
              >
              <h2 class="mtl">Preview:</h2>
            ` : html`
              ${this.content.readonly ? nothing : html`<hi-button
                    class="pos-absolute pin-top pin-right"
                    aesthetic="icon"
                    @trigger-ended=${this.__handleEditButtonTriggerEnded}
                  >
                    <hi-icon kind="edit"></hi-icon>
                    <hi-hint>Edit</hi-hint>
                  </hi-button>`}
            `}
        <hi-content .html=${this.editing ? this.preview : this.content.article}></hi-content>
      </div>
    `;
  }
}
export default HIP.factory.element(TemplateContentElement);
