
import { html } from '../../../hip/library/lit-element/lit-element.js';
import { dedupingMixin } from '../../../hip/library/polymer/polymer.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorBehavior } from '../../../eip/element/hi-behavior-behavior/hi-behavior-behavior.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { BehaviorEntityList } from './ai-behavior-entity-list.js';
import { BehaviorScreenQuery } from '../../../yip/element/yi-behavior-screen-query/yi-behavior-screen-query.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export function BaseBehaviorEntityListFilter(Base) {
  class BehaviorEntityListFilter2 extends BehaviorEntityList(BehaviorScreenQuery(BehaviorBehavior(Base))) {
    constructor() {
      super(...arguments);
      this.viewState = "initial";
      this.activePanel = "search";
      this.__filterSelected = {};
      this._renderSearchResultState = {
        loading(title, description, headingLevel) {
          return html`
          <section class="template-entity-list-filter-message-container">
            <div class="mam">
              <hi-title .level=${headingLevel} title=${title}></hi-title>
              <p>${description}</p>
            </div>
          </section>
        `;
        },
        initial(title, description, headingLevel) {
          return html`
          <section class="template-entity-list-filter-message-container">
            <div class="mam">
              <hi-title .level=${headingLevel} title=${title}></hi-title>
              <p>${description}</p>
            </div>
          </section>
        `;
        },
        results(title, description, headingLevel) {
          return this._renderList();
        }
      };
    }
    static get properties() {
      return {
        viewState: {
          type: String
        },
        activePanel: {
          type: String,
          attribute: "active-panel",
          reflect: true
        }
      };
    }
    _injectInstances() {
      return ["entity_filters", "entities", "entities_bulk_actions"];
    }
    _defaultRuntime() {
      return {
        startup: [],
        tasks: {
          view: {
            $is: "task",
            model_default: {
              filter_heading: "Search Preferences",
              list_heading: "Search Results"
            },
            settings: {
              workflow: {
                disallow_borders: true,
                settings: {
                  hidden: {
                    visibility_control_left: true,
                    visibility_control_right: true,
                    visibility_control_main: true
                  }
                }
              }
            }
          },
          entities: {
            $is: "task",
            settings: {
              list: {
                messages: {
                  state_initial: {
                    title: "Enter your search criteria",
                    description: ""
                  },
                  state_loading: {
                    title: "Fetching Results",
                    description: ""
                  },
                  state_empty: {
                    title: "No Results Found",
                    description: "Adjust your filter preferences"
                  }
                }
              }
            }
          },
          entity_filters: {
            $is: "task",
            settings: {
              form: {
                submit_label: "Filter",
                include_cancel: false,
                include_reset: true
              }
            }
          }
        }
      };
    }
    _connectedViewCallback() {
      this.runtime.subscribe({
        id: Symbol(),
        invoke: (job) => {
          if ("entities" === job.task.id) {
            this.viewState = "results";
          }
          this.requestUpdate("viewState");
        }
      });
    }
    firstUpdated() {
      const workspace = this.runtime?.workspace;
      const viewI = workspace?.queryInstance("view");
      if (!viewI) {
        return;
      }
      const { value: viewIValue } = viewI.state();
      const queryMode = f1.read("/query_mode", "search", viewIValue);
      if (queryMode === "filter") {
        this.viewState = "results";
        this.requestUpdate("viewState");
      }
    }
    _renderSearchForm() {
      const formI = this.runtime.workspace.queryInstance("entity_filters");
      if (!formI) {
        return nothing;
      }
      const { type: formType, value: formValue, settings: formSettings } = formI.state();
      const submitLabel = f1.read("/form/submit_label", "Filter", formSettings);
      const cancelButton = f1.read("/form/include_cancel", false, formSettings);
      const resetButton = f1.read("/form/include_reset", false, formSettings);
      return html`
        <hi-form
          gid="entity_list_filter_form"
          id="filter-list-form"
          name="filter-list-form"
          form="filter-list-form"
          layout="flush"
          .ns=${this.ns}
          @submit-ended=${this._entityListFilterForm__onSubmitEnded}
          @reset-ended=${this._entityListFilterForm__onResetEnded}
        ></hi-form>
        <ai-factory-editor
          for="entity_list_filter_form"
          gid="force-paint"
          form="filter-list-form"
          submit-label=${submitLabel}
          include-submit
          ?include-reset=${resetButton}
          ?include-cancel=${cancelButton}
          .ns=${this.ns}
          .type=${formType}
          .value=${formValue}
          @field-trigger-ended=${(e) => this.runtime.interlink.proxyFieldValueIn(e.detail)}
          @field-state-trigger-ended=${(e) => this.runtime.interlink.proxyFieldStateValueIn(e.detail)}
        ></ai-factory-editor>
      `;
    }
    _renderSearchResults(subHeadingLevel) {
      const listI = this.runtime.workspace.queryInstance("entities");
      if (!listI) {
        return nothing;
      }
      const { settings: listSettings } = listI.state();
      const title = f1.read(`/list/messages/state_${this.viewState}/title`, "", listSettings);
      const description = f1.read(`/list/messages/state_${this.viewState}/description`, "", listSettings);
      return html`${this._renderSearchResultState[this.viewState].call(
        this,
        title,
        description,
        subHeadingLevel + 1
      )} `;
    }
    _renderSearchFormAndResultsButtons() {
      const panelToggles = [
        {
          region: "search",
          label: "Preferences",
          triggerEnded: () => this._handlePanelToggle("search")
        },
        {
          region: "results",
          label: "Results",
          triggerEnded: () => this._handlePanelToggle("results")
        }
      ];
      return html`
        <section class="filter-footer">
          <hi-button-group aesthetic="none" layout="tight">
            ${panelToggles.map(
        (panel) => html`<hi-button
                .aesthetic=${panel.region === this.activePanel ? "primary" : "ghost"}
                bordered
                size="small"
                class="filter-footer-button"
                @trigger-ended=${panel.triggerEnded}
                >${panel.label}</hi-button
              >`
      )}
          </hi-button-group>
        </section>
      `;
    }
    _handlePanelToggle(panel) {
      this.activePanel = panel;
      const event = new CustomEvent("panel-changed", { detail: panel });
      this.dispatchEvent(event);
    }
    async _entityListFilterForm__onSubmitEnded(e) {
      this.__filterSelected = e.detail.value;
      this._submitButtonEl = e.currentTarget.$.submitButton;
      this._submitButtonEl.locked = true;
      await this._runSearchFilter();
    }
    async _entityListFilterForm__onResetEnded() {
      this.__filterSelected = {};
      await this._runSearchFilter();
    }
    async _runSearchFilter(filterParams = {}) {
      const filter = f1.merge("over", f1.merge("over", {}, this.__filterSelected), filterParams);
      const apiInstance = this.runtime.workspace.queryInstance("filter_api");
      const listTaskInstance = this.runtime.workspace.queryInstance("entities");
      if (!apiInstance || !listTaskInstance) {
        return;
      }
      const response = await apiInstance.invoke(filter);
      this.viewState = "loading";
      this.requestUpdate("viewState");
      this.activePanel = "results";
      const metadata = f1.read("/val/metadata", {}, response);
      const data = f1.read("/val/data", [], response);
      const mutations = f1.merge("over", filterParams, metadata);
      return listTaskInstance.typeInjection({
        mutations
      }).invoke(data).finally(() => {
        this._submitButtonEl.locked = false;
      });
    }
    _renderStyles() {
      return html`<style>
        .template-entity-list-filter-message-container {
          height: 100%;
          display: grid;
          justify-items: center;
          border: var(--hi-px-1) solid var(--hi-clr--gray-4);
        }

        .template-entity-list-filter-container {
          margin-bottom: calc(var(--hi-px-48) + var(--hi-px-8));
          max-height: 100%;
        }

        .template-entity-list-filter-container,
        .template-entity-list-container,
        .template-entity-filter-container {
          height: 100%;
          display: flex;
          flex-direction: column;
        }

        .filter-footer {
          display: inline-block;
          padding: var(--hi-px-8);
          border-top: var(--hi-px-1) solid var(--hi-clr--gray-3);
          background-color: var(--hi-clr--white);
          z-index: var(--hi-depth--content-hover);
          width: 100%;
          position: absolute;
          bottom: 0;
          left: 0;
        }

        .filter-footer-button {
          flex: 1 1 0%;
          min-width: unset;
        }
      </style>`;
    }
  }
  return HIP.factory.mixin(BehaviorEntityListFilter2);
}
export const BehaviorEntityListFilter = dedupingMixin(BaseBehaviorEntityListFilter);
