
import { BehaviorBehavior } from '../../../eip/element/hi-behavior-behavior/hi-behavior-behavior.js';
import adt from '../../../pip/library/pi-adt/pi-adt.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import { html, nothing } from '../../../hip/library/lit-html/lit-html.js';
export function BaseBehaviorEntityList(Base) {
  return class BehaviorEntityList extends BehaviorBehavior(Base) {
    constructor() {
      super(...arguments);
      this.fieldMutation = adt.mutation("fields");
    }
    _injectInstance(id) {
      const instanceI = this.runtime.workspace.queryInstance(id);
      return !instanceI ? { type: {}, value: [], settings: {} } : instanceI.state();
    }
    _renderList() {
      const instances = this._instances;
      const { type, value, settings } = instances.entities.state();
      const { list: listSettings, allow_preprocessed_input, disallow_header } = settings;
      const templateURI = f1.read("/template", "", settings);
      const mode = f1.read("/mode", void 0, settings);
      const templateResource = this.runtime.templates.get(templateURI);
      const template = templateResource ? templateResource : nothing;
      const bulkMenuActions = this.__computeActionsList("entities_bulk_actions");
      const rowMenuActions = this.__computeActionsList("entities_row_actions");
      return html`<vi-list
        gid="list"
        fit-height
        .bulkActions=${bulkMenuActions}
        .rowActions=${rowMenuActions}
        .settings=${listSettings}
        .template=${template}
        .input=${value}
        .type=${type}
        .mode=${mode}
        .allowPreprocessedInput=${allow_preprocessed_input}
        .disallowHeader=${disallow_header}
        @update-mutations-ended=${this._list__onUpdateMutationsEnded}
        @bulk-action-ended=${this.__list__onBulkActionEnded}
        @row-action-ended=${this.__list__onRowActionEnded}
        @pagination-ended=${this.__list__onPaginationEnded}
        @add-row-ended=${this.__list__onAddRowEnded}
        @selections-changed=${this.__list__onSelectionChanged}
        @selection-changed=${this.__list__onSelectionChanged}
        @commit-ended=${this.__list__onCommitEnded}
      ></vi-list>`;
    }
    __list__onSelectionChanged(e) {
      const { value } = e.detail;
      const selectionTaskI = this.runtime.workspace.queryInstance("entities_selected");
      if (selectionTaskI) {
        selectionTaskI.invoke(value);
      }
    }
    __list__onBulkActionEnded(e) {
      const actionModel = e.detail.value;
      const { selections, action } = actionModel;
      this.__updateField__listActions("entities_bulk_actions", selections, action, selections);
      this.__updateField__listActions("entities_bulk_actions", selections, "action_list", actionModel);
    }
    __updateField__listActions(taskId, selections, action, value) {
      const { type } = this._injectInstance(taskId);
      const field = f1.read(`/fields/${action}`, null, type);
      if (!field) {
        return;
      }
      this.runtime.interlink.proxyFieldValueIn({
        field,
        value
      });
    }
    __computeActionsList(instanceName) {
      const actionsListSource = f1.read(
        "/type/fields/action_list/source/out/value",
        null,
        this._injectInstance(instanceName)
      );
      if (actionsListSource) {
        return actionsListSource;
      }
      const type = f1.read("/type", {}, this._injectInstance(instanceName));
      const actionsList = this.fieldMutation.invoke(type);
      return f1.map(
        (field) => ({
          value: field.name,
          title: field.title
        }),
        actionsList
      );
    }
    async _list__onUpdateMutationsEnded(e) {
    }
    __list__onAddRowEnded(e) {
      const addEntityI = this.runtime.workspace.queryInstance("add_entity");
      if (!addEntityI) {
        return;
      }
      addEntityI.invoke();
    }
    __list__onCommitEnded(e) {
    }
    __list__onRowActionEnded(e) {
      const { selections, action } = e.detail.value;
      const { type } = this._injectInstance("entities_row_actions");
      const field = f1.read(`/fields/${action}`, {}, type);
      const value = selections;
      this.runtime.interlink.proxyFieldValueIn({
        field,
        value
      });
    }
    async __list__onPaginationEnded(e) {
    }
  };
}
export const BehaviorEntityList = BaseBehaviorEntityList;
