
import { html } from '../../../hip/library/lit-element/lit-element.js';
import { dedupingMixin } from '../../../hip/library/polymer/polymer.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorBehavior } from '../../../eip/element/hi-behavior-behavior/hi-behavior-behavior.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { BehaviorEntityTable } from './ai-behavior-entity-table.js';
import { BehaviorScreenQuery } from '../../../yip/element/yi-behavior-screen-query/yi-behavior-screen-query.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export function BaseBehaviorEntityTableFilter(Base) {
  class BehaviorEntityTableFilter2 extends BehaviorEntityTable(BehaviorScreenQuery(BehaviorBehavior(Base))) {
    constructor() {
      super(...arguments);
      this.viewState = "initial";
      this.__filterSelected = {};
      this._renderSearchResultState = {
        loading(title, description, headingLevel) {
          return html`
          <section class="template-entity-table-filter-message-container">
            <div class="mam">
              <hi-title .level=${headingLevel} title=${title}></hi-title>
              <p>${description}</p>
            </div>
          </section>
        `;
        },
        initial(title, description, headingLevel) {
          return html`
          <section class="template-entity-table-filter-message-container">
            <div class="mte">
              <hi-title .level=${headingLevel} title=${title}></hi-title>
              <p>${description}</p>
            </div>
          </section>
        `;
        },
        results(title, description, headingLevel) {
          return this._renderTable();
        }
      };
    }
    static get properties() {
      return {
        viewState: {
          type: String
        }
      };
    }
    _injectInstances() {
      return ["entity_filters", "entities", "entities_bulk_actions"];
    }
    _defaultRuntime() {
      return {
        startup: [],
        tasks: {
          view: {
            $is: "task",
            model_default: {
              filter_heading: "Filter Preferences",
              grid_heading: "Search Results"
            },
            settings: {
              workflow: {
                disallow_borders: true,
                settings: {
                  hidden: {
                    visibility_control_left: true,
                    visibility_control_right: true,
                    visibility_control_main: true
                  }
                }
              }
            }
          },
          entities: {
            $is: "task",
            settings: {
              grid: {
                messages: {
                  state_initial: {
                    title: "Enter your search criteria",
                    description: ""
                  },
                  state_loading: {
                    title: "Fetching Results",
                    description: ""
                  },
                  state_empty: {
                    title: "No Results Found",
                    description: "Adjust your filter preferences"
                  }
                }
              }
            }
          },
          entity_filters: {
            $is: "task",
            settings: {
              form: {
                submit_label: "Filter",
                include_cancel: false,
                include_reset: true
              }
            }
          }
        }
      };
    }
    _connectedViewCallback() {
      this.runtime.subscribe({
        id: Symbol(),
        invoke: (job) => {
          if ("entities" === job.task.id) {
            this.viewState = "results";
          }
          this.requestUpdate("viewState");
        }
      });
    }
    firstUpdated() {
      const workspace = this.runtime?.workspace;
      const viewI = workspace?.queryInstance("view");
      if (!viewI) {
        return;
      }
      const { value: viewIValue } = viewI.state();
      const queryMode = f1.read("/query_mode", "search", viewIValue);
      if (queryMode === "filter") {
        this.viewState = "results";
        this.requestUpdate("viewState");
      }
    }
    _renderSearchForm() {
      const formI = this.runtime.workspace.queryInstance("entity_filters");
      if (!formI) {
        return nothing;
      }
      const { type: formType, value: formValue, settings: formSettings } = formI.state();
      const submitLabel = f1.read("/form/submit_label", "Filter", formSettings);
      const cancelButton = f1.read("/form/include_cancel", false, formSettings);
      const resetButton = f1.read("/form/include_reset", false, formSettings);
      return html`
        <hi-form
          gid="entity_table_filter_form"
          id="filter-table-form"
          name="filter-table-form"
          form="filter-table-form"
          layout="flush"
          .ns=${this.ns}
          @submit-ended=${this._entityTableFilterForm__onSubmitEnded}
          @reset-ended=${this._entityTableFilterForm__onResetEnded}
        ></hi-form>
        <ai-factory-editor
          for="entity_table_filter_form"
          gid="force-paint"
          form="filter-table-form"
          submit-label=${submitLabel}
          include-submit
          ?include-reset=${resetButton}
          ?include-cancel=${cancelButton}
          .ns=${this.ns}
          .type=${formType}
          .value=${formValue}
          @field-trigger-ended=${(e) => this.runtime.interlink.proxyFieldValueIn(e.detail)}
          @field-state-trigger-ended=${(e) => this.runtime.interlink.proxyFieldStateValueIn(e.detail)}
        ></ai-factory-editor>
      `;
    }
    _renderSearchResults(subHeadingLevel) {
      const gridI = this.runtime.workspace.queryInstance("entities");
      if (!gridI) {
        return nothing;
      }
      const { settings: gridSettings } = gridI.state();
      const title = f1.read(`/grid/messages/state_${this.viewState}/title`, "", gridSettings);
      const description = f1.read(`/grid/messages/state_${this.viewState}/description`, "", gridSettings);
      return html`${this._renderSearchResultState[this.viewState].call(
        this,
        title,
        description,
        subHeadingLevel + 1
      )} `;
    }
    async _entityTableFilterForm__onSubmitEnded(e) {
      this.__filterSelected = e.detail.value;
      this._submitButtonEl = e.currentTarget.$.submitButton;
      this._submitButtonEl.locked = true;
      await this._runSearchFilter();
    }
    async _entityTableFilterForm__onResetEnded() {
      this.__filterSelected = {};
      await this._runSearchFilter();
    }
    async _runSearchFilter(filterParams = {}) {
      const filter = f1.merge("over", f1.merge("over", {}, this.__filterSelected), filterParams);
      const apiInstance = this.runtime.workspace.queryInstance("filter_api");
      const gridTaskInstance = this.runtime.workspace.queryInstance("entities");
      if (!apiInstance || !gridTaskInstance) {
        return;
      }
      const response = await apiInstance.invoke(filter);
      this.viewState = "loading";
      this.requestUpdate("viewState");
      const metadata = f1.read("/val/metadata", {}, response);
      const data = f1.read("/val/data", [], response);
      const mutations = f1.merge("over", filterParams, metadata);
      return gridTaskInstance.typeInjection({
        mutations
      }).invoke(data).finally(() => this._submitButtonEl.locked = false);
    }
    _renderStyles() {
      return html`<style>
        .template-entity-table-filter-message-container {
          display: grid;
          justify-items: center;
          border: var(--hi-px-1) solid var(--hi-clr--gray-4);
          min-height: calc((var(--hi-table--row-height) * 10) + var(--hi-table--column-height));
        }
      </style>`;
    }
  }
  return HIP.factory.mixin(BehaviorEntityTableFilter2);
}
export const BehaviorEntityTableFilter = dedupingMixin(BaseBehaviorEntityTableFilter);
