
import { ifDefined } from '../../../hip/library/lit-html/directives/if-defined.js';
import { BehaviorBehavior } from '../../../eip/element/hi-behavior-behavior/hi-behavior-behavior.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import { Box as B } from '../../../aip/library/ai-lucid/ai-lucid.js';
import adt from '../../../pip/library/pi-adt/pi-adt.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import { html } from '../../../hip/library/lit-html/lit-html.js';
export function BaseBehaviorEntityTable(Base) {
  return class BehaviorEntityTable extends BehaviorBehavior(Base) {
    constructor() {
      super(...arguments);
      this.fieldMutation = adt.mutation("fields");
    }
    _injectInstance(id) {
      const instanceI = this.runtime.workspace.queryInstance(id);
      return !instanceI ? { type: {}, value: [], settings: {} } : instanceI.state();
    }
    _renderTable() {
      const instances = this._instances;
      const { value: entitiesValue, type: entitiesType, settings: entitiesSettings } = instances.entities.state();
      const { caption, allow_preprocessed_input, grid: gridSettings } = entitiesSettings;
      const bulkMenuActions = this.__computeActionsList("entities_bulk_actions");
      const rowMenuActions = this.__computeActionsList("entities_row_actions");
      const selections = this.__computeSelectionList();
      return html`<vi-grid
        gid="grid"
        fit-height
        .bulkActions=${bulkMenuActions}
        .rowActions=${rowMenuActions}
        .allowPreprocessedInput=${allow_preprocessed_input}
        .caption=${caption}
        .settings=${gridSettings}
        .input=${entitiesValue}
        .type=${entitiesType}
        .ns=${this.ns}
        .selections=${ifDefined(selections)}
        @field-trigger-ended=${(e) => this.runtime.interlink.proxyFieldValueIn(e.detail)}
        @update-mutations-ended=${this._grid__onUpdateMutationsEnded}
        @bulk-action-ended=${this.__grid__onBulkActionEnded}
        @row-action-ended=${this.__grid__onRowActionEnded}
        @pagination-ended=${this.__grid__onPaginationEnded}
        @list-scroll-ended=${this.__grid__onListScrollEnded}
        @export-ended=${this.__grid__onExportEnded}
        @add-row-ended=${this.__grid__onAddRowEnded}
        @selections-changed=${this.__grid__onSelectionChanged}
        @selection-changed=${this.__grid__onSelectionChanged}
        @commit-ended=${this.__grid__onCommitEnded}
      ></vi-grid> `;
    }
    __grid__onSelectionChanged(e) {
      const { value } = e.detail;
      const selectionTaskI = this.runtime.workspace.queryInstance("entities_selected");
      if (selectionTaskI) {
        selectionTaskI.invoke(value);
      }
    }
    __computeSelectionList() {
      return f1.read("/value", void 0, this._injectInstance("entities_selected"));
    }
    __computeActionsList(instanceName) {
      const actionsListSource = f1.read(
        "/type/fields/action_list/source/out/value",
        null,
        this._injectInstance(instanceName)
      );
      if (actionsListSource) {
        return actionsListSource;
      }
      const type = f1.read("/type", {}, this._injectInstance(instanceName));
      const actionsList = this.fieldMutation.invoke(type);
      return f1.map(
        (field) => ({
          value: field.name,
          title: field.title
        }),
        actionsList
      );
    }
    async _grid__onUpdateMutationsEnded(e) {
    }
    __grid__onAddRowEnded(e) {
      const addEntityI = this.runtime.workspace.queryInstance("add_entity");
      if (!addEntityI) {
        return;
      }
      addEntityI.invoke();
    }
    __grid__onCommitEnded(e) {
      const { row, value, field, table } = e.detail;
      const editEntityFieldI = this.runtime.workspace.queryInstance("edit_entity_field");
      if (editEntityFieldI) {
        editEntityFieldI.invoke({
          row,
          value,
          field,
          row_change: {
            [field]: value
          }
        });
      }
      const editEntitiesI = this.runtime.workspace.queryInstance("edit_entities");
      if (editEntitiesI) {
        editEntitiesI.invoke(table);
      }
    }
    async __grid__onBulkActionEnded(e) {
      const actionModel = e.detail.value;
      const { selections, action } = actionModel;
      this.__updateField__gridActions("entities_bulk_actions", selections, action, selections);
      this.__updateField__gridActions("entities_bulk_actions", selections, "action_list", actionModel);
    }
    __grid__onRowActionEnded(e) {
      const actionModel = e.detail.value;
      const { selections, action } = actionModel;
      this.__updateField__gridActions("entities_row_actions", selections, action, selections);
      this.__updateField__gridActions("entities_row_actions", selections, "action_list", actionModel);
    }
    __updateField__gridActions(taskId, selections, action, value) {
      const { type } = this._injectInstance(taskId);
      const field = f1.read(`/fields/${action}`, null, type);
      if (!field) {
        return;
      }
      this.runtime.interlink.proxyFieldValueIn({
        field,
        value
      });
    }
    async __grid__onPaginationEnded(e) {
      for await (const [_, behavior] of this.runtime.behaviors.entries()) {
        const evt = f1.read("/events/grid/pagination-ended", null, behavior);
        if (evt) {
          await evt(this.runtime, e);
        }
      }
    }
    async __grid__onListScrollEnded(e) {
      for await (const [_, behavior] of this.runtime.behaviors.entries()) {
        const evt = f1.read("/events/grid/list-scroll-ended", null, behavior);
        if (evt) {
          await evt(this.runtime, e);
        }
      }
    }
    __grid__onExportEnded(e) {
      const gridEl = HIP.browser.queryOne("grid", this.ns);
      if (!gridEl.settings.allow.remote_export) {
        return;
      }
      const exportEntitiesI = this.runtime.workspace.queryInstance("export_entities");
      if (exportEntitiesI) {
        exportEntitiesI.invoke().then((csv) => {
          if (!B.hasError(csv)) {
            gridEl.generateExport(csv.val);
          }
        });
      }
    }
  };
}
export const BehaviorEntityTable = BaseBehaviorEntityTable;
