
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { BehaviorTemplate, AsLitTemplateHILC } from '../../../aip/element/ai-behavior-template/ai-behavior-template.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import '../../../aip/element/ai-factory-viewer/ai-factory-viewer.js';
import '../../../aip/element/ai-factory-editor/ai-factory-editor.js';
import '../../../eip/element/hi-form/hi-form.js';
import '../../../eip/element/hi-panel/hi-panel.js';
import '../../../eip/element/hi-toggle/hi-toggle.js';
import '../../../eip/element/hi-toggle/hi-toggle-group.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class TemplateEntityDetailsElement extends BehaviorTemplate(AsLitTemplateHILC(LitElement)) {
  constructor() {
    super(...arguments);
    this.__activeState = "edit";
    this.__renderEntityDetailsState = {
      view(entityType, entityValue) {
        const entityGid = `entity-viewer-panel`;
        return html` <hi-panel gid=${entityGid} .ns=${this.ns}> </hi-panel>
        <ai-factory-viewer gid="force-paint" for=${entityGid} .ns=${this.ns} .value=${entityValue} .type=${entityType}>
        </ai-factory-viewer>`;
      },
      edit(entityType, entityValue) {
        const editorEntity = this.editorEntity();
        const entityGid = `entity-editor-form`;
        return html`<hi-form
          layout="left"
          disable-http
          gid=${entityGid}
          .ns=${this.ns}
          form=${editorEntity}
          @submit-ended=${this.__form__onSubmitEnded}
        >
        </hi-form>
        <ai-factory-editor
          include-submit
          gid="force-paint"
          submit-label="Save"
          for=${entityGid}
          .ns=${this.ns}
          form=${editorEntity}
          .value=${entityValue}
          .type=${entityType}
          @field-trigger-ended=${(e) => this.__field__onTriggerEnded(e)}
          @field-state-trigger-ended=${(e) => this.runtime.interlink.proxyFieldStateValueIn(e.detail)}
        ></ai-factory-editor>`;
      }
    };
  }
  static get is() {
    return "ai-template-entity-details";
  }
  _injectInstances() {
    return ["entity", "submit_entity"];
  }
  editorEntity() {
    return this.runtime.id ? `${this.runtime.id}-entity` : `entity-${f1.id()}`;
  }
  _render() {
    const instances = this._instances;
    const entityValue = instances.entity.value();
    const entityType = instances.entity.type();
    const title = f1.read("/title", "", entityValue);
    const ns = this.ns;
    return html`
      <section l-kind="hort" l-justify="right" class="pbsxxx">
        <hi-toggle-group .ns=${ns} l-size="12" l-sml-size="3">
          <hi-toggle
            .ns=${ns}
            .on=${this.__activeState === "view"}
            @toggle-ended=${this.__updateState__onToggleEnded}
          ></hi-toggle>
          <hi-label .ns=${ns}>Edit ${title}</hi-label>
        </hi-toggle-group>
      </section>
      <section class="entity-details">
        ${this.__renderEntityDetailsState[this.__activeState].call(this, entityType, entityValue)}
      </section>
    `;
  }
  __updateState__onToggleEnded(e) {
    this.__activeState = e.detail.value ? "view" : "edit";
    this.requestUpdate();
  }
  __field__onTriggerEnded(e) {
    this.runtime.interlink.proxyFieldValueIn(e.detail);
    const fieldName = f1.read("/detail/field/name", "", e);
    const value = f1.read("/detail/value", void 0, e);
    const fieldValue = f1.is.record(value) ? f1.read("/value", void 0, value) : value;
    f1.write(`/${fieldName}`, fieldValue, this._instances.entity.value());
  }
  __form__onSubmitEnded(e) {
    const instances = this._instances;
    const formEntity = e.detail.value;
    instances.submit_entity.invoke(formEntity);
  }
}
export default HIP.factory.element(TemplateEntityDetailsElement);
