
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { BehaviorTemplate, AsLitTemplateHILC } from '../../../aip/element/ai-behavior-template/ai-behavior-template.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import '../../../aip/element/ai-factory-editor/ai-factory-editor.js';
import '../../../eip/element/hi-button/hi-button.js';
import '../../../eip/element/hi-form/hi-form.js';
import '../../../eip/element/hi-accordion/hi-accordion.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class TemplateEntityEditorListElement extends BehaviorTemplate(AsLitTemplateHILC(LitElement)) {
  static get is() {
    return "ai-template-entity-editor-list";
  }
  _injectInstances() {
    return ["entities", "edit_entity", "add_entity", "delete_entity"];
  }
  static get properties() {
    return {
      _formsValue: {
        type: Array
      }
    };
  }
  editorForm() {
    return this.runtime.id ? `${this.runtime.id}-form-${f1.id()}` : `form-${f1.id()}`;
  }
  __newFormGid() {
    return `entity-editor-form-${f1.id()}`;
  }
  __removeForm(index) {
    this.__submitFormDelete(this._formsValue[index]);
    this._formsValue.splice(index, 1);
    this.requestUpdate();
  }
  __createNewFormObject(formObject) {
    return Object.keys(formObject).reduce((newObject, key) => {
      if (key === "entity_form_title") {
        newObject[key] = formObject[key];
      } else {
        newObject[key] = null;
      }
      return newObject;
    }, {});
  }
  __mergeFormObject(formValue) {
    return { ...this._formsObject, ...formValue };
  }
  _render() {
    const instances = this._instances;
    if (f1.is.nil(this._formsValue)) {
      const formValue = instances.entities.value();
      this._formsValue = f1.is.array(formValue) ? formValue : [formValue];
      this._formsObject = this.__createNewFormObject(this._formsValue[0]);
    }
    const formType = instances.entities.type();
    const ns = this.ns;
    const formElements = this._formsValue.map((form, index) => {
      const editorForm = this.editorForm();
      const formGid = this.__newFormGid();
      return html`
        <hi-bar>${form.entity_form_title ? form.entity_form_title : "Form"}</hi-bar>
        <hi-note>
          <div style="display: flex; justify-content: end">
            <hi-button aesthetic="icon" @trigger-ended=${() => this.__removeForm(index)}>
              <hi-hint placement="left">Delete</hi-hint>
              <hi-icon kind="trash"></hi-icon>
            </hi-button>
          </div>
          <hi-form
            layout="left"
            disable-http
            form=${editorForm}
            gid=${formGid}
            .ns=${ns}
            @submit-ended=${(e) => this.__form__onSubmitEnded(e, index)}
          >
          </hi-form>
          <ai-factory-editor
            include-submit
            gid="force-paint"
            for=${formGid}
            .ns=${ns}
            form=${editorForm}
            .value=${form}
            .type=${formType}
            @field-trigger-ended=${(e) => this.runtime.interlink.proxyFieldValueIn(e.detail)}
            @field-state-trigger-ended=${(e) => this.runtime.interlink.proxyFieldStateValueIn(e.detail)}
          ></ai-factory-editor>
        </hi-note>
      `;
    });
    return html`
      <hi-accordion> ${formElements} </hi-accordion>
      <hi-button aesthetic="secondary" class="mtm" @trigger-ended=${this.__addNewForm}>Add Another</hi-button>
    `;
  }
  __submitFormDelete(formEntity) {
    this._instances.delete_entity.invoke(formEntity);
  }
  __form__onSubmitEnded(e, index) {
    const instances = this._instances;
    const formEntity = this.__mergeFormObject(e.detail.value);
    const submitButtonEl = e.currentTarget.$.submitButton;
    if (instances.edit_entity) {
      this._formsValue.splice(index, 1, formEntity);
      submitButtonEl.waitFor(instances.edit_entity.invoke(this._formsValue));
    }
  }
  __addNewForm() {
    this._formsValue.push(this._formsObject);
    this._instances.add_entity.invoke(this._formsObject);
    this.requestUpdate();
  }
}
export default HIP.factory.element(TemplateEntityEditorListElement);
