
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { BehaviorTemplate, AsLitTemplateHILC } from '../../../aip/element/ai-behavior-template/ai-behavior-template.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import '../../../aip/element/ai-factory-editor/ai-factory-editor.js';
import '../../../eip/element/hi-button/hi-button.js';
import '../../../eip/element/hi-form/hi-form.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class TemplateEntityEditorElement extends BehaviorTemplate(AsLitTemplateHILC(LitElement)) {
  static get is() {
    return "ai-template-entity-editor";
  }
  _injectInstances() {
    return ["entity", "submit_entity", "cancel_entity"];
  }
  editorEntity() {
    return this.runtime.id ? `${this.runtime.id}-entity` : `entity-${f1.id()}`;
  }
  _render() {
    const instances = this._instances;
    const { value: entityValue, type: entityType, settings: entitySettings } = instances.entity.state();
    const cancelButton = f1.read("/form/include_cancel", false, entitySettings);
    const editorEntity = this.editorEntity();
    const entityGid = "entity-editor-form";
    if (!entityType) {
      return this._renderStateError({
        title: `Entity Type is not provided`,
        details: `Please provide type information to render the editor template view`
      });
    }
    return html`<hi-form
        layout="left"
        disable-http
        gid=${entityGid}
        form=${editorEntity}
        .ns=${this.ns}
        @submit-ended=${this.__form__onSubmitEnded}
        @submit-failed=${this.__form__onSubmitFailed}
        @cancel-ended=${this.__form__onCancelEnded}
      >
      </hi-form>
      <ai-factory-editor
        gid="force-paint"
        for=${entityGid}
        form=${editorEntity}
        .ns=${this.ns}
        .value=${entityValue}
        .type=${entityType}
        include-submit
        ?include-cancel=${cancelButton}
        @field-state-trigger-ended=${(e) => this.runtime.interlink.proxyFieldStateValueIn(e.detail)}
        @field-trigger-ended=${(e) => {
      this.runtime.interlink.proxyFieldValueIn(e.detail);
      this.runtime.interlink.proxyTemporaryModel("entity", e.srcElement.model);
    }}
      ></ai-factory-editor>`;
  }
  __form__onSubmitEnded(e) {
    const instances = this._instances;
    const formEntity = e.detail.value;
    const submitButtonEl = e.currentTarget.$.submitButton;
    if (instances.submit_entity) {
      submitButtonEl.waitFor(instances.submit_entity.invoke(formEntity));
    }
  }
  __form__onSubmitFailed(e) {
  }
  __form__onCancelEnded(e) {
    const instances = this._instances;
    if (instances.cancel_entity) {
      instances.cancel_entity.invoke();
    }
  }
}
export default HIP.factory.element(TemplateEntityEditorElement);
