
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { BehaviorTemplate, AsLitTemplateHILC } from '../../../aip/element/ai-behavior-template/ai-behavior-template.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorEntityTableFilter } from './ai-behavior-entity-table-filter.js';
import '../../../aip/element/ai-factory-control/ai-factory-control.js';
import '../../../yip/element/yi-view-header/yi-view-header.js';
import adt from '../../../pip/library/pi-adt/pi-adt.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class TemplateEntityFilterTableSelectionsElement extends BehaviorEntityTableFilter(
  BehaviorTemplate(AsLitTemplateHILC(LitElement))
) {
  constructor() {
    super(...arguments);
    this.fieldMutations = adt.mutation("fields");
  }
  static get is() {
    return "ai-template-entity-filter-table-selections";
  }
  _injectInstances() {
    return [
      "entity_filters",
      "entities",
      "entities_bulk_actions",
      "selected_entities",
      "selected_entities_header",
      "selected_entities_row_actions"
    ];
  }
  _defaultRuntime() {
    return {
      startup: [],
      tasks: {
        view: {
          $is: "task",
          model_default: {
            filter_heading: "Filter Preferences",
            grid_heading: "Search Results"
          },
          settings: {
            workflow: {
              settings: {
                hidden: {
                  visibility_control_left: false,
                  visibility_control_right: false,
                  visibility_control_main: false
                }
              }
            }
          }
        }
      }
    };
  }
  _computeActionsList(instanceName) {
    const type = f1.read("/type", {}, this._injectInstance(instanceName));
    const actionsList = this.fieldMutation.invoke(type);
    return f1.map(
      (field) => ({
        value: field.name,
        title: field.title
      }),
      actionsList
    );
  }
  __list__onRowActionEnded(e) {
    const { selections, action } = e.detail.value;
    const { type } = this._injectInstance("selected_entities_row_actions");
    const field = f1.read(`/fields/${action}`, {}, type);
    const value = selections;
    this.runtime.interlink.proxyFieldValueIn({
      field,
      value
    });
  }
  __renderHeading(subviewHeader) {
    return html`
      <yi-view-header
        title=${subviewHeader.title}
        description=${subviewHeader.description}
        level=${subviewHeader.level}
      >
      </yi-view-header>
    `;
  }
  __renderSelectedEntitiesHeading(selectedEntitiesHeaderInstance, subviewHeader) {
    const { value: selectedEntitiesHeaderValue, type: selectedEntitiesHeaderType } = selectedEntitiesHeaderInstance.state();
    const selectedEntitiesHeaderFields = f1.read("/fields", {}, selectedEntitiesHeaderType);
    const gid = "selected-entities-header";
    const ns = this.ns;
    return html`<yi-view-header
        title=${subviewHeader.title}
        description=${subviewHeader.description}
        level=${subviewHeader.level}
      >
        ${!f1.is.empty(selectedEntitiesHeaderFields) ? html`<hi-panel .gid=${gid} .ns=${ns} slot="toolbar"></hi-panel>` : nothing}
      </yi-view-header>
      <ai-factory-control
        gid="force-paint"
        .for=${gid}
        .ns=${ns}
        .type=${selectedEntitiesHeaderType}
        .value=${selectedEntitiesHeaderValue}
        @field-trigger-ended=${(e) => this.runtime.interlink.proxyFieldValueIn(e.detail)}
      >
      </ai-factory-control>`;
  }
  __renderSelectedList(selectedEntities) {
    const {
      value: selectedEntitiesValue,
      type: selectedEntitiesType,
      settings: selectedEntitiesSettings
    } = selectedEntities.state();
    const mode = f1.read("/mode", void 0, selectedEntitiesSettings);
    const templateURI = f1.read("/template", "", selectedEntitiesSettings);
    const templateResource = this.runtime.templates.get(templateURI);
    const template = templateResource ? templateResource : nothing;
    const rowActions = this._computeActionsList("selected_entities_row_actions");
    return html`<vi-list
      .mode=${mode}
      .template=${template}
      .type=${selectedEntitiesType}
      .input=${selectedEntitiesValue}
      .settings=${selectedEntitiesSettings}
      .rowActions=${rowActions}
      @row-action-ended=${this.__list__onRowActionEnded}
    >
    </vi-list>`;
  }
  _render() {
    const instances = this._instances;
    const viewI = this.runtime.workspace.queryInstance("view");
    const {
      entity_filters: entityFiltersI,
      entities: entitiesI,
      selected_entities: selectedEntitiesI,
      selected_entities_header: selectedEntitiesHeaderI
    } = instances;
    const instanceUndefined = !viewI || !entityFiltersI || !entitiesI || !selectedEntitiesI || !selectedEntitiesHeaderI;
    if (instanceUndefined) {
      return nothing;
    }
    const selectedEntitiesHeaderInstance = instances.selected_entities_header;
    const { value: viewValue, settings: viewSettings } = viewI.state();
    const subViews = f1.merge(
      "over",
      {
        filter_heading: "",
        filter_description: "",
        grid_heading: "",
        grid_description: "",
        list_heading: "",
        list_description: ""
      },
      viewValue
    );
    const workflowSettings = f1.read("/workflow/settings", {}, viewSettings);
    const workflowDisallowBorders = f1.read("/workflow/disallow_borders", false, viewSettings);
    const subHeadingLevel = this.level + 1;
    const filterHeader = {
      title: subViews.filter_heading,
      description: subViews.filter_description,
      level: subHeadingLevel
    };
    const gridHeader = { title: subViews.grid_heading, description: subViews.grid_description, level: subHeadingLevel };
    const listHeader = { title: subViews.list_heading, description: subViews.list_description, level: subHeadingLevel };
    return html`${this.__renderStyles()}
      <yi-layout-workflow
        class="mtlx"
        .settings=${workflowSettings}
        ?disallow-borders=${workflowDisallowBorders}
        layout="left_main_right"
      >
        <section slot="left">${this.__renderHeading(filterHeader)} ${this._renderSearchForm()}</section>
        <section slot="main">${this.__renderHeading(gridHeader)}${this._renderSearchResults(subHeadingLevel)}</section>
        <section slot="right">
          ${this.__renderSelectedEntitiesHeading(selectedEntitiesHeaderInstance, listHeader)}
          ${this.__renderSelectedList(selectedEntitiesI)}
        </section>
      </yi-layout-workflow>`;
  }
  __renderStyles() {
    return html`<style>
      .template-entity-table-filter-message-container {
        display: grid;
        justify-items: center;
        border: var(--hi-px-1) solid var(--hi-clr--gray-4);
        min-height: calc((var(--hi-table--row-height) * 10) + var(--hi-table--column-height));
      }

      yi-layout-workflow {
        --hi-left-panel--width: 25.33%;
        --hi-right-panel--width: 28.33%;
      }
    </style>`;
  }
  async _grid__onUpdateMutationsEnded(e) {
    await this._runSearchFilter(e.detail.value);
  }
}
export default HIP.factory.element(TemplateEntityFilterTableSelectionsElement);
