
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { BehaviorTemplate, AsLitTemplateHILC } from '../../../aip/element/ai-behavior-template/ai-behavior-template.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorEntityListFilter } from './ai-behavior-entity-list-filter.js';
import '../../../aip/element/ai-factory-editor/ai-factory-editor.js';
import '../../../vip/element/vi-list/vi-list.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class TemplateEntityListFilterElement extends BehaviorEntityListFilter(
  BehaviorTemplate(AsLitTemplateHILC(LitElement))
) {
  static get is() {
    return "ai-template-entity-list-filter";
  }
  __renderSearchFormAndResults(viewValue) {
    const filterHeading = f1.read("/filter_heading", "", viewValue);
    const listHeading = f1.read("/list_heading", "", viewValue);
    const subHeadingLevel = this.level + 1;
    const panels = [
      {
        region: "search",
        title: filterHeading,
        component: () => this._renderSearchForm()
      },
      {
        region: "results",
        title: listHeading,
        component: () => this._renderSearchResults(subHeadingLevel)
      }
    ];
    return this.screenDevice === "mobile" ? this.__renderMobileResultsList(subHeadingLevel, panels) : this.__renderDesktopResultsList(subHeadingLevel, filterHeading, listHeading);
  }
  __renderMobileResultsList(subHeadingLevel, panels) {
    return html`<section class="template-entity-list-filter-container">
        ${panels.map(
      (panel) => this.activePanel === panel.region ? html` <hi-title .level=${subHeadingLevel} title="${panel.title}"></hi-title>
                ${panel.component()}` : nothing
    )}
      </section>
      ${this._renderSearchFormAndResultsButtons()}`;
  }
  __renderDesktopResultsList(subHeadingLevel, filterHeading, listHeading) {
    return html`<section class="template-entity-filter-container" slot="left" class="mrl">
        <hi-title .level=${subHeadingLevel} title="${filterHeading}"></hi-title>
        ${this._renderSearchForm()}
      </section>
      <section class="template-entity-list-container">
        <hi-title .level=${subHeadingLevel} title="${listHeading}"></hi-title>
        ${this._renderSearchResults(subHeadingLevel)}
      </section>`;
  }
  _render() {
    const workspace = this.runtime.workspace;
    const formI = workspace.queryInstance("entity_filters");
    if (!formI) {
      return nothing;
    }
    const listI = workspace.queryInstance("entities");
    if (!listI) {
      return nothing;
    }
    const viewI = workspace.queryInstance("view");
    if (!viewI) {
      return nothing;
    }
    const { value: viewValue, settings: viewSettings } = viewI.state();
    const workflowSettings = f1.read("/workflow/settings", {}, viewSettings);
    const workflowDisallowBorders = f1.read("/workflow/disallow_borders", false, viewSettings);
    return html`${this._renderStyles()}
    ${this.screenDevice === "desktop" ? html`<yi-layout-workflow
          class="mtlx"
          .settings=${workflowSettings}
          ?disallow-borders=${workflowDisallowBorders}
          disallow-overflow="disallow-overflow"
          layout="left_main"
        >
          ${this.__renderSearchFormAndResults(viewValue)}
        </yi-layout-workflow>` : this.__renderSearchFormAndResults(viewValue)}`;
  }
  async _list__onUpdateMutationsEnded(e) {
    await this._runSearchFilter(e.detail.value);
  }
}
export default HIP.factory.element(TemplateEntityListFilterElement);
