
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { BehaviorTemplate, AsLitTemplateHILC } from '../../../aip/element/ai-behavior-template/ai-behavior-template.js';
import { BehaviorEntityTableFilter } from './ai-behavior-entity-table-filter.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class TemplateEntityTableFilterDetailsElement extends BehaviorEntityTableFilter(
  BehaviorTemplate(AsLitTemplateHILC(LitElement))
) {
  constructor() {
    super(...arguments);
    this.formState = "results";
    this.__renderDetailsFormState = {
      loading(settings, headingLevel) {
        return html`
        <section class="template-entity-table-filter-details-message-container">
          <div class="mam">
            <hi-title .level=${headingLevel} title="${settings.messages.state_loading.title}"></hi-title>
            <p>${settings.messages.state_loading.description}</p>
          </div>
        </section>
      `;
      },
      initial(settings, headingLevel) {
        return html`
        <section class="template-entity-table-filter-details-message-container">
          <div class="mte">
            <hi-title .level=${headingLevel} title="${settings.messages.state_initial.title}"></hi-title>
            <p>${settings.messages.state_initial.description}</p>
          </div>
        </section>
      `;
      },
      results(settings, headingLevel) {
        return this.__renderDetailsForm();
      }
    };
  }
  static get properties() {
    return {
      formState: {
        type: String
      }
    };
  }
  static get is() {
    return "ai-template-entity-table-filter-details";
  }
  _injectInstances() {
    return ["entity_filters", "entities", "entities_bulk_actions", "entity_details"];
  }
  _defaultRuntime() {
    return f1.merge("over", super._defaultRuntime(), {
      startup: [],
      tasks: {
        view: {
          $is: "task",
          model_default: {
            filter_heading: "Filter Preferences",
            grid_heading: "Search Results",
            details_heading: "Entity Details"
          }
        },
        entity_details: {
          $is: "task",
          settings: {
            form: {
              messages: {
                state_initial: {
                  title: "Select a Row",
                  description: "No row is selected to show details form. Please select a row"
                },
                state_loading: {
                  title: "Fetching the Form",
                  description: "Fecthing details of selected row"
                }
              }
            }
          }
        }
      }
    });
  }
  _render() {
    const workspace = this.runtime.workspace;
    const formI = workspace.queryInstance("entity_filters");
    if (!formI) {
      return nothing;
    }
    const gridI = workspace.queryInstance("entities");
    if (!gridI) {
      return nothing;
    }
    const viewI = workspace.queryInstance("view");
    if (!viewI) {
      return nothing;
    }
    const { value: viewIValue, settings: viewSettings } = viewI.state();
    const filterHeading = f1.read("/filter_heading", "", viewIValue);
    const gridHeading = f1.read("/grid_heading", "", viewIValue);
    const detailsHeading = f1.read("/details_heading", "", viewIValue);
    const subHeadingLevel = this.level + 1;
    const workflowSettings = f1.read("/workflow/settings", {}, viewSettings);
    const workflowDisallowBorders = f1.read("/workflow/disallow_borders", false, viewSettings);
    return html`
      <yi-layout-workflow class="mtlx" .settings=${workflowSettings} ?disallow-borders=${workflowDisallowBorders}>
        <section slot="left">
          <hi-title .level=${subHeadingLevel} title="${filterHeading}"></hi-title>
          ${this._renderSearchForm()}
        </section>
        <section class="pam">
          <hi-title .level=${subHeadingLevel} title="${gridHeading}"></hi-title>
          ${this._renderSearchResults(subHeadingLevel)}
        </section>
        <section slot="right">
          <hi-title .level=${subHeadingLevel} title="${detailsHeading}"></hi-title>
          ${this._renderDetailsForm(subHeadingLevel)}
        </section>
      </yi-layout-workflow>
    `;
  }
  _renderDetailsForm(subHeadingLevel) {
    const detailsI = this.runtime.workspace.queryInstance("entity_details");
    if (!detailsI) {
      return nothing;
    }
    const { settings: detailSettings } = detailsI.state();
    return html`${this.__renderDetailsFormState[this.formState].call(
      this,
      f1.read("/form", {}, detailSettings),
      subHeadingLevel + 1
    )}`;
  }
  __renderDetailsForm() {
    const detailsI = this.runtime.workspace.queryInstance("entity_details");
    if (!detailsI) {
      return nothing;
    }
    const { type: detailsFormType, value: detailsFormValue } = detailsI.state();
    return html`<hi-form
        gid="entity_table_detail_form"
        name="detail-table-form"
        form="detail-table-form"
        layout="flush"
        .ns=${this.ns}
        @submit-ended=${this.__entityTableDetailForm__onSubmitEnded}
      ></hi-form>
      <ai-factory-editor
        gid="force-paint"
        for="entity_table_detail_form"
        form="detail-table-form"
        submit-label="Save"
        include-submit
        .ns=${this.ns}
        .type=${detailsFormType}
        .value=${detailsFormValue}
        @field-trigger-ended=${(e) => this.runtime.interlink.proxyFieldValueIn(e.detail)}
        @field-state-trigger-ended=${(e) => this.runtime.interlink.proxyFieldStateValueIn(e.detail)}
      ></ai-factory-editor>`;
  }
  async __entityTableDetailForm__onSubmitEnded(event) {
    const apiInstance = this.runtime.workspace.queryInstance("detail_api");
    const gridTaskInstance = this.runtime.workspace.queryInstance("entities");
    if (!apiInstance || !gridTaskInstance) {
      return;
    }
    const response = await apiInstance.invoke(event.detail.value);
    this.viewState = "loading";
    this.requestUpdate("viewState");
    const metadata = f1.read("/val/metadata", {}, response);
    const data = f1.read("/val/data", [], response);
    await gridTaskInstance.typeInjection({
      mutations: metadata
    }).invoke(data);
    this.viewState = "results";
    this.requestUpdate("viewState");
  }
}
export default HIP.factory.element(TemplateEntityTableFilterDetailsElement);
