
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { BehaviorTemplate, AsLitTemplateHILC } from '../../../aip/element/ai-behavior-template/ai-behavior-template.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorEntityTableFilter } from './ai-behavior-entity-table-filter.js';
import '../../../aip/element/ai-factory-editor/ai-factory-editor.js';
import '../../../vip/element/vi-grid/vi-grid.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class TemplateEntityTableFilterElement extends BehaviorEntityTableFilter(
  BehaviorTemplate(AsLitTemplateHILC(LitElement))
) {
  static get is() {
    return "ai-template-entity-table-filter";
  }
  __renderSearchFormAndResults(viewValue) {
    const filterHeading = f1.read("/filter_heading", "", viewValue);
    const gridHeading = f1.read("/grid_heading", "", viewValue);
    const subHeadingLevel = this.level + 1;
    return html`<section slot="left" class="mrl">
        <hi-title .level=${subHeadingLevel} title="${filterHeading}"></hi-title>
        ${this._renderSearchForm()}
      </section>
      <section>
        <hi-title .level=${subHeadingLevel} title="${gridHeading}"></hi-title>
        ${this._renderSearchResults(subHeadingLevel)}
      </section>`;
  }
  _render() {
    const workspace = this.runtime.workspace;
    const formI = workspace.queryInstance("entity_filters");
    if (!formI) {
      return nothing;
    }
    const gridI = workspace.queryInstance("entities");
    if (!gridI) {
      return nothing;
    }
    const viewI = workspace.queryInstance("view");
    if (!viewI) {
      return nothing;
    }
    const { value: viewValue, settings: viewSettings } = viewI.state();
    const workflowSettings = f1.read("/workflow/settings", {}, viewSettings);
    const workflowDisallowBorders = f1.read("/workflow/disallow_borders", false, viewSettings);
    return html`${this._renderStyles()}
    ${this.screenDevice === "desktop" ? html`<yi-layout-workflow
          class="mtlx"
          .settings=${workflowSettings}
          ?disallow-borders=${workflowDisallowBorders}
          layout="left_main"
        >
          ${this.__renderSearchFormAndResults(viewValue)}
        </yi-layout-workflow>` : this.__renderSearchFormAndResults(viewValue)}`;
  }
  async _grid__onUpdateMutationsEnded(e) {
    await this._runSearchFilter(e.detail.value);
  }
}
export default HIP.factory.element(TemplateEntityTableFilterElement);
