
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { BehaviorTemplate, AsLitTemplateHILC } from '../../../aip/element/ai-behavior-template/ai-behavior-template.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorEntityTableFilter } from './ai-behavior-entity-table-filter.js';
import '../../../aip/element/ai-factory-editor/ai-factory-editor.js';
import '../../../vip/element/vi-grid/vi-grid.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class TemplateEntityTableSearchFilterElement extends BehaviorEntityTableFilter(
  BehaviorTemplate(AsLitTemplateHILC(LitElement))
) {
  static get is() {
    return "ai-template-entity-table-search-filter";
  }
  _render() {
    const workspace = this.runtime.workspace;
    const formI = workspace.queryInstance("entity_filters");
    if (!formI) {
      return nothing;
    }
    const gridI = workspace.queryInstance("entities");
    if (!gridI) {
      return nothing;
    }
    const viewI = workspace.queryInstance("view");
    if (!viewI) {
      return nothing;
    }
    const { value: viewValue, settings: viewSettings } = viewI.state();
    const filterHeading = f1.read("/filter_heading", "", viewValue);
    const gridHeading = f1.read("/grid_heading", "", viewValue);
    const workflowSettings = f1.read("/workflow/settings", {}, viewSettings);
    const workflowDisallowBorders = f1.read("/workflow/disallow_borders", false, viewSettings);
    const subHeadingLevel = this.level + 1;
    return html`${this._renderStyles()}
      <yi-layout-workflow
        class="mtlx"
        .settings=${workflowSettings}
        ?disallow-borders=${workflowDisallowBorders}
        layout="left_main"
      >
        <section slot="left" class="mrl">
          <hi-title .level=${subHeadingLevel} title="${filterHeading}"></hi-title>
          ${this._renderSearchForm()}
        </section>
        <section>
          <hi-title .level=${subHeadingLevel} title="${gridHeading}"></hi-title>
          ${this._renderSearchResults(subHeadingLevel)}
        </section>
      </yi-layout-workflow>`;
  }
  async _runSearchFilter(filterParams = {}) {
    const searchForm = document.getElementById("filter-table-form");
    const searchFormValues = searchForm.evaluate().valid;
    const searchApiI = this.runtime.workspace.queryInstance("search_api");
    if (!searchApiI) {
      return;
    }
    const searchProperties = f1.merge("over", f1.merge("over", {}, searchFormValues), filterParams);
    return searchApiI.invoke(searchProperties).finally(() => this._submitButtonEl.locked = false);
  }
  async _grid__onUpdateMutationsEnded(e) {
    const mutations = e.detail.value;
    const filterApiI = this.runtime.workspace.queryInstance("filter_api");
    if (!filterApiI) {
      return;
    }
    await filterApiI.invoke(mutations);
  }
}
export default HIP.factory.element(TemplateEntityTableSearchFilterElement);
