
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { BehaviorTemplate, AsLitTemplateHILC } from '../../../aip/element/ai-behavior-template/ai-behavior-template.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import '../../../aip/element/ai-factory-viewer/ai-factory-viewer.js';
import '../../../eip/element/hi-form/hi-form.js';
import '../../../eip/element/hi-tab/hi-tab.js';
import '../../../eip/element/hi-title/hi-title.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class TemplateEntityViewerTabbedElement extends BehaviorTemplate(AsLitTemplateHILC(LitElement)) {
  constructor() {
    super(...arguments);
    this.selectedIndex = 0;
  }
  static get is() {
    return "ai-template-entity-viewer-tabbed";
  }
  _injectInstances() {
    return ["entities"];
  }
  static get properties() {
    return {
      selectedIndex: {
        type: Number
      },
      drilledIn: {
        type: Boolean,
        attribute: "drilled-in",
        reflect: true
      }
    };
  }
  viewerForm() {
    return this.runtime.id ? `${this.runtime.id}-form-${f1.id()}` : `form-${f1.id()}`;
  }
  __newFormGid() {
    return `entity-viewer-form-${f1.id()}`;
  }
  _render() {
    const instances = this._instances;
    const { value: entities, type } = instances.entities.state();
    if (f1.is.nil(entities) || f1.is.empty(entities)) {
      return nothing;
    }
    const entity = entities[this.selectedIndex];
    const ns = this.ns;
    const formGid = this.__newFormGid();
    return html`
      <div>
        <yi-layout-split-2
          .auto=${false}
          .drilledIn=${this.drilledIn}
          @trigger-ended=${this.__layout__onTriggerEnded}
          panel-to-show=${this.drilledIn ? "right" : "left"}
        >
          <div slot="leftPanel" class="width-100">
            <hi-tab-group @selected-index-changed=${this.__tabGroup__onCommitEnded} kind="stacked" stretched>
              ${f1.map((entity2, index) => {
      return html` <hi-tab ?selected=${this.selectedIndex === index}> ${entity2.entity_title} </hi-tab> `;
    }, entities)}
            </hi-tab-group>
          </div>
          <div slot="rightPanel" class="pam">
            <hi-title level=${this.level + 1} title=${entity.entity_title}></hi-title>
            <hi-panel gid=${formGid} .ns=${ns} class="mtm"></hi-panel>
            <ai-factory-viewer
              gid="force-paint"
              for=${formGid}
              .ns=${ns}
              .value=${entity}
              .type=${type}
            ></ai-factory-viewer>
          </div>
        </yi-layout-split-2>
      </div>
    `;
  }
  __tabGroup__onCommitEnded(event) {
    this.selectedIndex = event.detail.value;
    this.drilledIn = true;
  }
  __layout__onTriggerEnded() {
    this.drilledIn = false;
  }
}
export default HIP.factory.element(TemplateEntityViewerTabbedElement);
