
import { ifDefined as ifDef } from '../../../hip/library/lit-html/directives/if-defined.js';
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { unsafeHTML } from '../../../hip/library/lit-html/directives/unsafe-html.js';
import { BehaviorTemplate, AsLitTemplateHILC } from '../../../aip/element/ai-behavior-template/ai-behavior-template.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class TemplateResourcesElement extends BehaviorTemplate(AsLitTemplateHILC(LitElement)) {
  constructor() {
    super(...arguments);
    this.activeTab = "";
    this.filteredResourceGroups = [];
  }
  static get is() {
    return "ai-template-library";
  }
  static get properties() {
    return {
      activeTab: {
        type: String,
        attribute: "active-tab",
        reflect: true
      },
      drilledIn: {
        type: Boolean,
        attribute: "drilled-in",
        reflect: true
      },
      filteredResourceGroups: {
        type: Array,
        attribute: "filtered-resource-groups"
      }
    };
  }
  _injectInstances() {
    return ["resourceGroups", "filter"];
  }
  _render() {
    const instances = this._instances;
    const resourceGroupsValue = f1.safe({}, instances.resourceGroups.value());
    const filterValue = f1.safe({}, instances.filter.value());
    const resourceGroups = !f1.is.empty(filterValue) ? this.filteredResourceGroups : f1.to.array(resourceGroupsValue);
    return html`
      <div>
        <yi-layout-split-2
          .auto=${false}
          @trigger-ended=${this.__layout__onTriggerEnded}
          .drilledIn=${this.drilledIn}
          panel-to-show=${this.drilledIn ? "right" : "left"}
          style="--left-panel-width: 34rem; --right-panel-width: 100%;"
        >
          <div slot="leftPanel">
            ${!f1.is.empty(filterValue) ? html`
                  <div style="display: flex; flex-direction: column;">
                    <div>
                      <hi-input-group class="width-100">
                        <hi-label>
                          ${filterValue.name}
                          <hi-icon kind="info_circle" id="filter-info"></hi-icon>
                          <hi-tooltip heading=${filterValue.tooltip.heading} for="filter-info" placement="right">
                            ${filterValue.tooltip.body}
                          </hi-tooltip>
                        </hi-label>
                        <hi-combobox
                          id="filter-combobox"
                          .disabled=${filterValue.options.length === 0}
                          placeholder=${filterValue.options.length > 0 ? filterValue.placeholder.optionsExist : filterValue.placeholder.optionsDoNotExist}
                          .input=${filterValue.options}
                        >
                        </hi-combobox>
                      </hi-input-group>
                    </div>
                    <div class="mtm" l-kind="hort" l-justify="left">
                      <div class="min-w-240" l-kind="hort" l-size="3" l-align="start">
                        <hi-button
                          stretched
                          class="mrsx"
                          aesthetic="secondary"
                          @trigger-ended=${this.__submitButton__onTriggerEnded}
                        >
                          Submit
                        </hi-button>
                        <hi-button
                          stretched
                          class="mlsx"
                          aesthetic="ghost"
                          @trigger-ended=${this.__clearButton__onTriggerEnded}
                        >
                          Clear
                        </hi-button>
                      </div>
                    </div>
                  </div>
                ` : nothing}
            ${resourceGroups.length > 0 ? html` <hi-tab-group
                  id="resource-groups-tab-group"
                  class="mtm mbm"
                  stretched
                  aesthetic="stacked"
                  @commit-ended=${this.__tabGroup__onCommitEnded}
                >
                  ${resourceGroups.map((resourceGroup) => {
      return html`
                      <hi-tab value=${ifDef(resourceGroup.name)}>
                        ${resourceGroup.name}
                        <hi-badge theme="blue" aesthetic="pill">${resourceGroup.resources.length}</hi-badge>
                      </hi-tab>
                    `;
    }, resourceGroupsValue)}
                </hi-tab-group>` : this.__determineEmptyState()}
          </div>
          <div slot="rightPanel" class="height-100 pam">
            ${this.activeTab !== "" ? html` <hi-accordion id="resources-accordion">
                  ${resourceGroups.map((resourceGroup) => {
      return html`
                      ${this.activeTab === resourceGroup.name ? resourceGroup.resources.map((resource, index) => {
        return html`<hi-bar>
                                <strong>${resource.title}</strong> | ${index + 1} of ${resourceGroup.resources.length}
                                Resources
                              </hi-bar>
                              <hi-note>
                                <div class="mbm">${unsafeHTML(resource.description)}</div>
                                <div>${unsafeHTML(resource.contact)}</div>
                              </hi-note>`;
      }, resourceGroupsValue) : nothing}
                    `;
    })}
                </hi-accordion>` : html`<div
                  style="font-size: 8em; display: flex; flex-direction: column; height: 100%; width: 100%; justify-content: center; align-items: center;"
                >
                  <hi-icon type="material" kind="search_v2" theme="gray"></hi-icon>
                  <div class="text-sz-m clr-gray_80-txt mtm mbm">Results will show up here.</div>
                </div>`}
          </div>
        </yi-layout-split-2>
      </div>
    `;
  }
  __filterResources(value) {
    if (value === void 0) {
      this.filteredResourceGroups = [];
    }
    const instances = this._instances;
    const resourceGroupsValue = f1.safe({}, instances.resourceGroups.value());
    const filterValue = f1.safe({}, instances.filter.value());
    const key = filterValue.key;
    this.filteredResourceGroups = f1.to.array(resourceGroupsValue).filter((resourceGroup) => {
      return resourceGroup[key] === value;
    });
  }
  __submitButton__onTriggerEnded() {
    const resourceGroupsTabGroup = document.getElementById("resource-groups-tab-group");
    const resourcesAccordion = document.getElementById("resources-accordion");
    const filterCombobox = document.getElementById("filter-combobox");
    const value = filterCombobox.value;
    if (resourceGroupsTabGroup) {
      resourceGroupsTabGroup.selectedIndex = void 0;
    }
    if (resourcesAccordion) {
      resourcesAccordion.selectedIndex = void 0;
    }
    this.activeTab = "";
    this.drilledIn = false;
    this.__filterResources(value);
  }
  __clearButton__onTriggerEnded() {
    const resourceGroupsTabGroup = document.getElementById("resource-groups-tab-group");
    const resourcesAccordion = document.getElementById("resources-accordion");
    const filterCombobox = document.getElementById("filter-combobox");
    if (resourceGroupsTabGroup) {
      resourceGroupsTabGroup.selectedIndex = void 0;
    }
    if (resourcesAccordion) {
      resourcesAccordion.selectedIndex = void 0;
    }
    filterCombobox.value = void 0;
    this.activeTab = "";
    this.drilledIn = false;
    this.__filterResources(void 0);
  }
  __tabGroup__onCommitEnded(event) {
    this.activeTab = event.detail.value;
    this.drilledIn = true;
  }
  __layout__onTriggerEnded() {
    this.drilledIn = false;
  }
  __determineEmptyState() {
    const instances = this._instances;
    const resourceGroupsValue = f1.safe({}, instances.resourceGroups.value());
    const filterValue = f1.safe({}, instances.filter.value());
    const resourceGroups = f1.to.array(resourceGroupsValue);
    if (!f1.is.empty(filterValue) && resourceGroups.length > 0) {
      return html``;
    } else {
      return html`<hi-alert class="mbm" theme="warning" mode="presentation">
        No Resources Found
        <span slot="details"> There are currently no resources for you to review. </span>
      </hi-alert>`;
    }
  }
}
HIP.factory.element(TemplateResourcesElement);
