
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { BehaviorTemplate, AsLitTemplateHILC } from '../../../aip/element/ai-behavior-template/ai-behavior-template.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import '../../../aip/element/ai-factory-editor/ai-factory-editor.js';
import '../../../eip/element/hi-button/hi-button.js';
import '../../../eip/element/hi-form/hi-form.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class TemplateLookupTableElement extends BehaviorTemplate(AsLitTemplateHILC(LitElement)) {
  static get is() {
    return "ai-template-lookup-table";
  }
  _injectInstances() {
    return ["entities", "submit_entities", "view"];
  }
  _defaultRuntime() {
    return {
      tasks: {
        view: {
          model_default: {
            submit_button_title: "Submit"
          }
        }
      }
    };
  }
  _render() {
    const instances = this._instances;
    const { value, type, settings } = instances.entities.state();
    const { value: viewValue } = instances.view.state();
    const editorForm = `${this.runtime.id}-form`;
    const ns = this.ns;
    const inputs = f1.to.array(f1.safe([], value));
    return html`
      <style>
        #ai-template-lookup-table-form-${ns} hi-input-group:not(first-child) {
          margin-top: 0;
        }
      </style>
      <hi-form
        id="ai-template-lookup-table-form-${ns}"
        layout="left"
        disable-http
        form=${editorForm}
        .ns=${ns}
        @submit-ended=${this.__form__onSubmitEnded}
      >
        <ul kind="table">
          ${inputs.map((form, index) => {
      const gid = `input-group-${index}`;
      return html`
              <li>
                <hi-panel gid=${gid} .ns=${ns} l-kind="hort" class="pam mts "></hi-panel>
                <ai-factory-editor
                  gid="force-paint"
                  name-prefix="/${index}"
                  form=${editorForm}
                  for=${gid}
                  .ns=${ns}
                  .value=${inputs}
                  .type=${type}
                  @field-trigger-ended=${(e) => this.runtime.interlink.proxyFieldValueIn(e.detail)}
                  @field-state-trigger-ended=${(e) => this.runtime.interlink.proxyFieldStateValueIn(e.detail)}
                ></ai-factory-editor>
              </li>
            `;
    })}
        </ul>
        <hi-button .ns=${ns} slot="controls" type="submit">${viewValue.submit_button_title}</hi-button>
      </hi-form>
    `;
  }
  __form__onSubmitEnded(e) {
    const instances = this._instances;
    const formEntities = e.detail.value;
    const submitButtonEl = e.currentTarget.$.submitButton;
    submitButtonEl.waitFor(instances.submit_entities.invoke(formEntities));
  }
}
export default HIP.factory.element(TemplateLookupTableElement);
