
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { BehaviorTemplate, AsLitTemplateHILC } from '../../../aip/element/ai-behavior-template/ai-behavior-template.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class TemplateNavTocElement extends BehaviorTemplate(AsLitTemplateHILC(LitElement)) {
  static get is() {
    return "ai-template-nav-toc";
  }
  _injectInstances() {
    return ["index"];
  }
  _render() {
    const instances = this._instances;
    const index = f1.safe([], instances.index.value());
    return html`
      <hi-tile-group layout="grid">
        ${index.map((tile) => {
      return html`<article l-cell l-size="12" l-sml-size="6" l-med-size="4" l-lrg-size="3">
            <hi-tile>
              <h5 slot="title">${tile.title}</h5>
              <ul kind="index">
                ${this.__readIndexTiles(tile).map((route) => {
        return html`<li class="mt-4">
                    <hi-link href=${this.__writeLink(tile, route)}>${route.title}</hi-link>
                    <div class="mt-3 toolbar" x-hie>
                      ${this.__readIndexRouteFlags(route).map((badge) => {
          return html`<hi-badge class="mrsx pbsx" theme=${badge.theme} aesthetic="light" size="small">
                          ${badge.title}
                        </hi-badge>`;
        })}
                    </div>
                    ${route.description ? html`<p class="mt-3">${route.description}</p>` : nothing}
                  </li>`;
      })}
              </ul>
            </hi-tile>
          </article>`;
    })}
      </hi-tile-group>
    `;
  }
  __readIndexTiles(tile) {
    return tile.routes || tile.navigation_items || [];
  }
  __readIndexRouteFlags(route) {
    return route.flags || route.badges || [];
  }
  __writeLink(tile, route) {
    const uri = `${route.uri || ""}${tile.uri || ""}`;
    const baseURI = uri.indexOf("http") === 0 ? uri : uri.indexOf("#") === 0 ? uri : `#${uri}`;
    return `${baseURI}${tile.goto_uri || ""}`;
  }
}
export default HIP.factory.element(TemplateNavTocElement);
