
import { dedupingMixin } from '../../../hip/library/polymer/polymer.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import { createRef } from '../../../hip/library/lit-html/directives/ref.js';
import { BehaviorTemplate } from '../../../aip/element/ai-behavior-template/ai-behavior-template.js';
import { BehaviorBehavior } from '../../../eip/element/hi-behavior-behavior/hi-behavior-behavior.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import c11n from '../../../pip/library/pi-c11n/pi-c11n.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export function BaseBehaviorTemplateRegion(Base) {
  class BehaviorTemplateRegion2 extends BehaviorTemplate(BehaviorBehavior(Base)) {
    constructor() {
      super(...arguments);
      this.subRegionRef = createRef();
      this.controlGroupRef = createRef();
    }
    static get properties() {
      return {
        activeRegion: {
          type: String,
          attribute: "active-region",
          reflect: true
        }
      };
    }
    _region_firstUpdated() {
      const controlGroupEl = this.controlGroupRef.value;
      if (!controlGroupEl) {
        return;
      }
      this.activeRegion = this._computeActiveRegion(controlGroupEl);
    }
    _region_updated(activeRegion) {
      const subRegionEl = this.subRegionRef.value;
      if (!subRegionEl) {
        return;
      }
      const controlGroupEl = this.controlGroupRef.value;
      if (!controlGroupEl) {
        return;
      }
      if (activeRegion) {
        return this._renderViewToSubRegion(subRegionEl, activeRegion);
      }
      this._renderViewToSubRegion(subRegionEl, this._computeActiveRegion(controlGroupEl));
    }
    _computeRegionList() {
      return f1.pipe(
        [
          f1.read("/runtime/render/regions", {}),
          f1.to.array,
          f1.remove((region) => c11n.isRuleOff("allowed", c11n.readRefs(region)))
        ],
        this
      );
    }
    _computeControlList(oldInstanceId) {
      const instances = this._instances;
      const controlList = instances.controls ? instances.controls.value() : instances[oldInstanceId] ? instances[oldInstanceId].value() : {};
      return f1.safe({}, controlList);
    }
    _computeActiveRegion(stepGroupEl) {
      const stepGroupInitialValue = stepGroupEl ? stepGroupEl.readActiveState() : null;
      const regionsDB = f1.read("/runtime/render/regions", {}, this);
      const firstRegion = f1.nth(0, f1.keys(regionsDB));
      return regionsDB[this.activeRegion] ? this.activeRegion : stepGroupInitialValue ? stepGroupInitialValue : firstRegion;
    }
    _renderViewToSubRegion(subRegionEl, activeRegion) {
      const regionsDB = f1.read("/runtime/render/regions", {}, this);
      const viewRegion = regionsDB[activeRegion];
      const originalScript = viewRegion ? f1.copy(f1.safe({}, viewRegion.view.script)) : {};
      const c11nRefs = f1.join(",", c11n.readRefs(activeRegion));
      const autoRunScript = f1.clone(originalScript);
      const ns = `${this.ns}-${activeRegion}`;
      const gid = `sub-view-${activeRegion}`;
      const id = `sub-view-${activeRegion}`;
      const level = this.level + 1;
      dom.removeAll(subRegionEl);
      const aiViewEl = dom.create(null, "ai-view", {
        c11n: c11nRefs,
        autoRunScript,
        level,
        gid,
        id,
        ns
      });
      subRegionEl.appendChild(aiViewEl);
    }
    _controlGroup__onCommitEnded(e) {
      this.activeRegion = e.detail.value;
    }
  }
  return HIP.factory.mixin(BehaviorTemplateRegion2);
}
export const BehaviorTemplateRegion = dedupingMixin(BaseBehaviorTemplateRegion);
