
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { styleMap } from '../../../hip/library/lit-html/directives/style-map.js';
import { setAttributes } from '../../../eip/element/hi-lit/hi-lit.js';
import { BehaviorTemplate, AsLitTemplateHILC } from '../../../aip/element/ai-behavior-template/ai-behavior-template.js';
import { BehaviorTemplateRegion } from './ai-behavior-region.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import c11n from '../../../pip/library/pi-c11n/pi-c11n.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
export class TemplateRegionListElement extends BehaviorTemplateRegion(BehaviorTemplate(AsLitTemplateHILC(LitElement))) {
  constructor() {
    super(...arguments);
    this.orientationAlg = f1.strategy({
      vert: (el) => {
        el.classList.remove("display-flex");
        dom.setAttrs(el, {
          "l-kind": "vert",
          "l-width-fixed": false
        });
      },
      hort: (el) => {
        el.classList.add("display-flex");
        dom.setAttrs(el, {
          "l-kind": "hort",
          "l-width-fixed": true
        });
      }
    });
  }
  static get is() {
    return "ai-template-region-list";
  }
  _defaultRuntime() {
    return {
      startup: [],
      tasks: {
        view: {
          $is: "task",
          settings: {
            region: {
              alignment: "vert"
            }
          }
        }
      }
    };
  }
  _injectInstances() {
    return ["view"];
  }
  updated() {
    const viewI = f1.read("/_instances/view", {}, this);
    if (f1.is.empty(viewI)) {
      return;
    }
    const { settings } = viewI.state();
    const alignment = f1.read("/region/alignment", "vert", settings);
    const orientationFn = this.orientationAlg(alignment);
    orientationFn(this);
  }
  _render() {
    const level = this.level + 1;
    const regions = f1.pipe(
      [
        f1.read("/runtime/render/regions", {}),
        f1.to.array,
        f1.remove((region) => c11n.isRuleOff("allowed", c11n.readRefs(region)))
      ],
      this
    );
    return html`${regions.map((region) => {
      const c11nRefs = f1.join(",", c11n.readRefs(region));
      const script = f1.clone(region.view.script);
      const ns = `${this.ns}-${region.id}`;
      return html`<section
        id="sub-region-${region.id}"
        style=${styleMap(f1.read("/style", {}, region))}
        data-attrs=${setAttributes(region.attributes)}
      >
        <ai-view
          id="sub-view-${region.id}"
          gid="sub-view-${region.id}"
          c11n=${c11nRefs}
          .ns=${ns}
          .level=${level}
          .autoRunScript=${script}
        ></ai-view>
      </section>`;
    })}`;
  }
}
export default HIP.factory.element(TemplateRegionListElement);
