
import { ref, createRef } from '../../../hip/library/lit-html/directives/ref.js';
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { BehaviorTemplate, AsLitTemplateHILC } from '../../../aip/element/ai-behavior-template/ai-behavior-template.js';
import { BehaviorTemplateRegion } from './ai-behavior-region.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class TemplateRegionTabbedElement extends BehaviorTemplateRegion(
  BehaviorTemplate(AsLitTemplateHILC(LitElement))
) {
  constructor() {
    super(...arguments);
    this.subRegionRef = createRef();
    this.controlGroupRef = createRef();
  }
  static get is() {
    return "ai-template-region-tabbed";
  }
  _injectInstances() {
    return ["tabs", "controls"];
  }
  _render() {
    const regions = this._computeRegionList();
    const controlList = this._computeControlList("tabs");
    return html` <hi-tab-group
        ${ref(this.controlGroupRef)}
        uri-state="sub-region"
        @commit-ended=${this._controlGroup__onCommitEnded}
      >
        ${regions.map((region) => {
      const controlStep = f1.safe(
        {
          id: region.id,
          hidden: false
        },
        controlList[region.id]
      );
      region.title = f1.safe(f1.word(region.id), region.title);
      return html`<hi-tab
            value=${region.id}
            ?hidden=${controlStep.hidden}
            ?selected=${this.activeRegion && region.id === this.activeRegion}
            >${region.title}</hi-tab
          >`;
    })}
      </hi-tab-group>
      <section ${ref(this.subRegionRef)}></section>`;
  }
  firstUpdated() {
    this._region_firstUpdated();
  }
  updated() {
    this._region_updated();
  }
}
export default HIP.factory.element(TemplateRegionTabbedElement);
