
import { ifDefined as ifDef } from '../../../hip/library/lit-html/directives/if-defined.js';
import { ref } from '../../../hip/library/lit-html/directives/ref.js';
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { BehaviorTemplate, AsLitTemplateHILC } from '../../../aip/element/ai-behavior-template/ai-behavior-template.js';
import { BehaviorTemplateRegion } from './ai-behavior-region.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class TemplateRegionWorkflowElement extends BehaviorTemplateRegion(
  BehaviorTemplate(AsLitTemplateHILC(LitElement))
) {
  static get is() {
    return "ai-template-region-workflow";
  }
  _injectInstances() {
    return ["workflow", "controls", "active_region"];
  }
  getActiveRegion() {
    const controlGroupEl = this.controlGroupRef.value;
    if (!controlGroupEl) {
      return;
    }
    const activeRegion = this._instances.active_region.value();
    if (f1.is.empty(activeRegion)) {
      return this._computeActiveRegion(this.controlGroupRef.value);
    }
    return activeRegion;
  }
  _render() {
    const activeRegion = this.getActiveRegion();
    const regions = this._computeRegionList();
    const controlList = this._computeControlList("workflow");
    return html` <hi-step-group
        ${ref(this.controlGroupRef)}
        manual-furthest-index
        uri-state="sub-region"
        @commit-ended=${this._controlGroup__onCommitEnded}
      >
        ${regions.map((region) => {
      const controlStep = f1.safe(
        {
          id: region.id,
          hidden: false
        },
        controlList[region.id]
      );
      const title = f1.safe(f1.word(region.id), region.title);
      region.title = title;
      return html`<hi-step
            aesthetic=${ifDef(controlStep.aesthetic)}
            state=${ifDef(controlStep.state)}
            value=${region.id}
            ?hidden=${controlStep.hidden}
            ?locked=${controlStep.locked}
            ?selected=${activeRegion && region.id === activeRegion}
            >${region.title}</hi-step
          >`;
    })}
      </hi-step-group>
      <section ${ref(this.subRegionRef)}></section>`;
  }
  firstUpdated() {
    this._region_firstUpdated();
  }
  updated() {
    this._region_updated(this.getActiveRegion());
  }
  _controlGroup__onCommitEnded(e) {
    super._controlGroup__onCommitEnded(e);
    this._instances.active_region.invoke(this.activeRegion);
  }
}
export default HIP.factory.element(TemplateRegionWorkflowElement);
