
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { BehaviorTemplate, AsLitTemplateHILC } from '../../../aip/element/ai-behavior-template/ai-behavior-template.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import adt from '../../../pip/library/pi-adt/pi-adt.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
class TileTemplate {
  constructor() {
    this.id = "template-split-view-default";
  }
  render({ type, viewTemplate }) {
    return (input, index) => {
      const _titleField = f1.read("/fields/title", {}, type);
      const titleField = adt.expression.evaluateAll(_titleField, input);
      const titleHREF = f1.read("/settings/href", "#", titleField);
      const actionType = {
        fields: f1.read("/fields/actions/fields", {}, type)
      };
      return html` <article class="card card--content">
        ${input.full_name ? html`
              <hi-avatar class="card__avatar" theme="primary" aesthetic="condensed" full-name=${input.full_name}>
              </hi-avatar>
            ` : nothing}
        <section class="card__detail">
          <hi-link
            href=${titleHREF}
            value=${input.link}
            .ns=${viewTemplate.ns}
            @trigger-ended=${(e) => {
        viewTemplate.runtime.interlink.proxyFieldValueIn({
          field: titleField,
          value: e.detail.value
        });
      }}
            >${input.title}</hi-link
          >
          ${input.description ? html`<p>${input.description}</p>` : nothing}
          ${input.badges ? html`
                <div>${input.badges.map((title) => html`<hi-badge aesthetic="pill">${title}</hi-badge> `)}</div>
              ` : nothing}
        </section>

        <hi-panel gid="ctr-${index}" .ns=${viewTemplate.ns}></hi-panel>
        <ai-factory-control
          gid="force-paint"
          for="ctr-${index}"
          .ns=${viewTemplate.ns}
          .value=${input}
          .type=${actionType}
          @field-trigger-ended=${(e) => viewTemplate.runtime.interlink.proxyFieldValueIn(e.detail)}
        ></ai-factory-control>
      </article>`;
    };
  }
}
export class TemplateSplitViewElement extends BehaviorTemplate(AsLitTemplateHILC(LitElement)) {
  constructor() {
    super(...arguments);
    this.listRegionViewMode = "list";
    this.defaultTemplate = new TileTemplate();
  }
  static get is() {
    return "ai-template-split-view";
  }
  static get properties() {
    return {
      listRegionViewMode: {
        type: String
      },
      viewModelValue: {
        type: String
      }
    };
  }
  _defaultRuntime() {
    return {
      tasks: {
        show_left: {
          $is: "assignment",
          settings: {
            target: {
              id: "view_mode"
            }
          },
          model: { view_mode: "left" }
        },
        show_right: {
          $is: "assignment",
          settings: {
            target: {
              id: "view_mode"
            }
          },
          model: { view_mode: "right" }
        },
        view_mode: {
          $is: "task"
        }
      }
    };
  }
  _injectInstances() {
    return [
      "view_mode",
      "view",
      "list",
      "detail",
      "detail_header_controls",
      "filter_form",
      "filter_summary",
      "submit_filter"
    ];
  }
  editorFilter() {
    return this.runtime.id ? `${this.runtime.id}-filter` : `filter-${f1.id()}`;
  }
  _connectedViewCallback() {
    this.runtime.subscribe({
      id: Symbol(),
      invoke: (job) => {
        const jobId = job.id;
        switch (jobId) {
          case "view_mode":
            this.viewModelValue = f1.read("/view_mode", "left", job.value);
            break;
        }
      }
    });
  }
  _render() {
    const instances = this._instances;
    const { value: viewValue } = instances.view.state();
    const { value: detailValue } = instances.detail.state();
    const viewModelValue = f1.read("/view_mode", this.viewModelValue, instances.view_mode.value());
    const headerLevel = this.level + 1;
    const subViews = f1.merge(
      "over",
      {
        list_zone: {
          header: "",
          description: ""
        },
        detail_zone: {
          header: "",
          description: ""
        }
      },
      viewValue
    );
    return html`
      ${this._renderStyle()}
      <main x-layout-rule="split-view" x-layout-mode=${viewModelValue}>
        <section x-area="region" class="zone-left">${this._renderList(subViews, headerLevel)}</section>
        <section x-area="region" class="zone-right">
          ${f1.is.empty(detailValue) ? html`<hi-alert mode="presentation" class="ma-4"
                >Select an item from the left panel to get started.</hi-alert
              >` : this._renderDetail(subViews, headerLevel)}
        </section>
      </main>
    `;
  }
  _renderStyle() {
    return html`<style>
      [x-layout-rule='split-view'] {
        --x-micro-region-order: left, right;
        --region-left--width: 1fr;
        --region-right--width: 1fr;

        display: grid;
        grid-template-areas: 'left right';
        grid-template-columns: var(--region-left--width) var(--region-right--width);
      }

      [x-layout-rule='split-view'] > .zone-left,
      [x-layout-rule='split-view'] > .zone-right {
        overflow-x: hidden;
        max-width: var(--hi-media--desktop);
        margin-bottom: var(--hi-px-12);
      }

      [x-layout-rule='split-view'] > .zone-left {
        margin-right: calc(var(--hi-px-32) / 2);
        grid-area: left;
      }

      [x-layout-rule='split-view'] > .zone-right {
        margin-left: calc(var(--hi-px-32) / 2);
        grid-area: right;
      }

      [x-control='detail-panel-mobile-back'] {
        display: none;
      }

      @media only screen and (max-width: 40rem) {
        [x-control='detail-panel-mobile-back'] {
          display: inline-block;
        }

        [x-layout-rule='split-view'] {
          grid-template-areas: 'main';
          grid-template-columns: 1fr;
        }

        [x-layout-rule='split-view'][x-layout-mode='right'] > .zone-right,
        [x-layout-rule='split-view']:not([x-layout-mode='right']) > .zone-left {
          grid-area: main;
          margin: 0 !important;
          max-width: var(--hi-media--desktop);
        }

        [x-layout-rule='split-view'][x-layout-mode='right'] > .zone-left,
        [x-layout-rule='split-view']:not([x-layout-mode='right']) > .zone-right {
          display: none;
        }
      }
    </style>`;
  }
  _renderList(subViews, headerLevel) {
    const instances = this._instances;
    const { value: listValue, type: listType } = instances.list.state();
    const { value: filterValue, type: filterType } = instances.filter_form.state();
    const ns = this.ns;
    const editorGid = "filter-editor-form";
    const viewerGid = "filter-viewer-tags";
    const editorFilter = this.editorFilter();
    const willShowListViewMode = this.listRegionViewMode === "list";
    const context = {
      viewTemplate: this
    };
    return html`
      <yi-view-header
        level=${headerLevel}
        title=${subViews.list_zone.header}
        description=${subViews.list_zone.description}
      >
        <hi-toggle slot="navigation" kind="button" icon="filter" @toggle-ended=${this._filterOnToggleEnded}></hi-toggle>
        ${willShowListViewMode ? html` <hi-panel slot="status" gid=${viewerGid} .ns=${ns}></hi-panel>
              <ai-factory-viewer
                gid="force-paint"
                slot="status"
                for=${viewerGid}
                .ns=${ns}
                .value=${filterValue}
                .type=${filterType}
                @field-trigger-ended=${(e) => this.runtime.interlink.proxyFieldValueIn(e.detail)}
                @model-update-ended=${this._filterList__onModelUpdateEnded}
              ></ai-factory-viewer>` : nothing}
      </yi-view-header>

      ${willShowListViewMode ? html` <vi-list
            gid="grid"
            .context=${context}
            .template=${this.defaultTemplate}
            .input=${listValue}
            .type=${listType}
            .ns=${ns}
          >
          </vi-list>` : html`
            <hi-form
              gid=${editorGid}
              form=${editorFilter}
              disable-http
              .ns=${ns}
              @submit-ended=${this._filterForm__onSubmitEnded}
              @submit-failed=${this._filterForm__onSubmitFailed}
              @reset-ended=${this._filterForm__onResetEnded}
              @cancel-ended=${this._filterForm__onCancelEnded}
            ></hi-form>
            <ai-factory-editor
              gid="force-paint"
              include-submit
              include-reset
              include-cancel
              for=${editorGid}
              form=${editorFilter}
              .ns=${ns}
              .value=${filterValue}
              .type=${filterType}
              @field-trigger-ended=${(e) => this.runtime.interlink.proxyFieldValueIn(e.detail)}
            ></ai-factory-editor>
          `}
    `;
  }
  _renderDetail(subViews, headerLevel) {
    const ns = this.ns;
    const instances = this._instances;
    const { value: detailValue, type: detailType } = instances.detail.state();
    const { value: detailHeaderControlsValue, type: detailHeaderControlsType } = instances.detail_header_controls.state();
    return html`
      <yi-view-header
        level=${headerLevel}
        title=${subViews.detail_zone.header}
        description=${subViews.detail_zone.description}
      >
        <hi-panel slot="navigation" gid="right-header-controls" .ns=${ns}></hi-panel>
        <ai-factory-control
          gid="force-paint"
          slot="navigation"
          for="right-header-controls"
          .value=${detailHeaderControlsValue}
          .type=${detailHeaderControlsType}
          .ns=${ns}
          @field-trigger-ended=${(e) => this.runtime.interlink.proxyFieldValueIn(e.detail)}
        ></ai-factory-control>
      </yi-view-header>
      <hi-panel gid="detail" .ns=${ns}></hi-panel>
      <ai-factory-viewer
        gid="force-paint"
        for="detail"
        .value=${detailValue}
        .type=${detailType}
        .ns=${ns}
        @field-trigger-ended=${(e) => this.runtime.interlink.proxyFieldValueIn(e.detail)}
      ></ai-factory-viewer>
    `;
  }
  _filterOnToggleEnded() {
    this.listRegionViewMode = this.listRegionViewMode === "list" ? "form" : "list";
  }
  _filterList__onModelUpdateEnded(e) {
    const instances = this._instances;
    const filterModel = e.detail.value;
    if (instances.submit_filter) {
      instances.submit_filter.invoke(filterModel).then(() => {
        this.listRegionViewMode = "list";
      });
    } else {
      this.listRegionViewMode = "list";
    }
  }
  _filterForm__onSubmitEnded(e) {
    const submitButtonEl = e.currentTarget.$.submitButton;
    const filterForm = e.detail.value;
    const instances = this._instances;
    if (instances.submit_filter) {
      submitButtonEl.waitFor(instances.submit_filter.invoke(filterForm)).then(() => {
        this.listRegionViewMode = "list";
      });
    } else {
      this.listRegionViewMode = "list";
    }
  }
  _filterForm__onSubmitFailed() {
    this.listRegionViewMode = "list";
  }
  _filterForm__onResetEnded(e) {
    const submitButtonEl = e.currentTarget.$.resetButton;
    const instances = this._instances;
    if (instances.submit_filter) {
      submitButtonEl.waitFor(instances.submit_filter.invoke({})).then(() => {
        this.listRegionViewMode = "list";
      });
    } else {
      this.listRegionViewMode = "list";
    }
  }
  _filterForm__onCancelEnded() {
    this.listRegionViewMode = "list";
  }
}
export default HIP.factory.element(TemplateSplitViewElement);
