
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { BehaviorTemplate, AsLitTemplateHILC } from '../../../aip/element/ai-behavior-template/ai-behavior-template.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import '../../../aip/element/ai-factory-editor/ai-factory-editor.js';
import '../../../eip/element/hi-button/hi-button.js';
import '../../../eip/element/hi-form/hi-form.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import adt from '../../../pip/library/pi-adt/pi-adt.js';
export class TemplateWorkflowQuestionElement extends BehaviorTemplate(AsLitTemplateHILC(LitElement)) {
  constructor() {
    super(...arguments);
    this.fieldMutation = adt.mutation("fields");
    this.__activeQuestion = 0;
    this.__formValues = {};
  }
  static get is() {
    return "ai-template-workflow-question";
  }
  static get properties() {
    return {
      __activeQuestion: {
        type: Number
      },
      __mutatedFields: {
        type: Array
      },
      __formValues: {
        type: Object
      }
    };
  }
  _injectInstances() {
    return ["entity", "submit_entity", "goto_uri"];
  }
  __getWorkflowFormType() {
    if (this.__mutatedFields[this.__activeQuestion]?.hidden) {
      this.__activeQuestion++;
    }
    return {
      fields: {
        [this.__mutatedFields[this.__activeQuestion].name]: this.__mutatedFields[this.__activeQuestion]
      }
    };
  }
  _render() {
    const instances = this._instances;
    const editorGid = "workflow--question--form";
    const editorForm = this.runtime.id ? `${this.runtime.id}-question-form` : `${f1.id()}-question-form`;
    const entityType = instances.entity.type();
    const entityValue = instances.entity.value();
    const isQuestionActive = this.__activeQuestion === 0 || this.__activeQuestion != this.__mutatedFields.length;
    if (!entityType) {
      return this._renderStateError({
        title: `Entity Type is not provided`,
        details: `Please provide type information to render the editor template view`
      });
    }
    this.__mutatedFields = this.fieldMutation.invoke(entityType);
    return isQuestionActive ? html`<hi-form
            layout="center"
            disable-http
            gid=${editorGid}
            form=${editorForm}
            .ns=${this.ns}
            @submit-ended=${this.__form__onSubmitEnded}
          >
          </hi-form>
          <ai-factory-editor
            gid="force-paint"
            include-submit
            for=${editorGid}
            form=${editorForm}
            .ns=${this.ns}
            .value=${entityValue}
            .type=${this.__getWorkflowFormType()}
            @field-trigger-ended=${(e) => this.runtime.interlink.proxyFieldValueIn(e.detail)}
            @field-state-trigger-ended=${(e) => this.runtime.interlink.proxyFieldStateValueIn(e.detail)}
          >
          </ai-factory-editor>` : html`<section style="width:70%; margin:auto; padding:0.5em;">
          <hi-alert mode="presentation">
            <span>Congratulations you got to the end of this form.</span>
            <span slot="action">
              <hi-button
                id="submitWorkflow"
                @trigger-ended=${(e) => this.__submitWorkflow__onTriggerEnded(e)}
              >
                GO TO</hi-button
              >
            </span>
          </hi-alert>
        </section>`;
  }
  __form__onSubmitEnded(e) {
    f1.merge("over", this.__formValues, e.detail.value);
    if (this.__activeQuestion != this.__mutatedFields.length - 1) {
      this.__activeQuestion++;
    } else {
      this.__activeQuestion = this.__mutatedFields.length;
    }
  }
  __submitWorkflow__onTriggerEnded(e) {
    const instances = this._instances;
    const buttonEl = e.currentTarget;
    instances.goto_uri.invoke();
    buttonEl.waitFor(instances.submit_entity.invoke(this.__formValues));
  }
}
export default HIP.factory.element(TemplateWorkflowQuestionElement);
