
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import { BehaviorIO } from '../../../eip/element/hi-behavior-io/hi-behavior-io.js';
import { BehaviorSetGroup } from '../../../eip/element/hi-behavior-group/hi-behavior-set-group.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import './ai-field.js';
export class TypeGroupElement extends BehaviorSetGroup(BehaviorIO(BehaviorElement(AsHIC(PolymerElement)))) {
  constructor() {
    super("type");
  }
  static get is() {
    return "ai-type-group";
  }
  static get properties() {
    return {
      reference: {
        type: Object,
        computed: "__referenceCompute(output)",
        notify: true
      },
      crossmap: {
        type: Object,
        computed: "__crossmapCompute(output, reference)",
        notify: true
      }
    };
  }
  static get valueEvent() {
    return "output-changed";
  }
  static get autoBinding() {
    return {
      signal: "output-changed",
      field: "input"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("hidden", "hidden");
    this.group = "type";
    this.input = {};
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    this.__clearType(this);
  }
  _groupChanged(group, groupOld) {
    this._forceRegisterHooks(group, groupOld);
  }
  _group__itemRemoved(hook, el) {
    super._group__itemRemoved(hook, el);
    this.__clearType(el);
  }
  __clearType(el) {
    if (this.input) {
      delete this.input[el.name];
    }
  }
  _group__itemChanged(hook, el) {
    this.__updateType(el);
  }
  __updateType(el) {
    if (el.output && this.input) {
      this.input[el.name] = el.output;
      this.flush();
    }
  }
  __referenceCompute(output) {
    return this.__referenceBuild(output || {});
  }
  __referenceBuild(typeMetadata, reference = {}) {
    for (const typeName in typeMetadata) {
      const typeDefinition = f1.safe({}, typeMetadata[typeName]);
      const typeId = typeof typeDefinition.id === "string" ? typeDefinition.id : `#/type/${typeName}`;
      reference[typeId] = typeDefinition;
    }
    return reference;
  }
  __crossmapCompute(output, reference) {
    return this.__crossmapBuild(output, reference);
  }
  __crossmapBuild(output, reference, sourceRoot = "#/type", crossmap = []) {
    for (const keyRef in output) {
      const typeDefinition = this.__resolveReferences(reference, output, keyRef);
      if (!typeDefinition.properties) {
        continue;
      }
      const sourceRef = `${sourceRoot}/${keyRef}`;
      if (!typeDefinition["properties"]["$ref"]) {
        this.__crossmapBuild(typeDefinition["properties"], reference, sourceRef, crossmap);
      } else {
        const lookupRef = typeDefinition["properties"]["$ref"];
        const isLookupArray = typeDefinition.type === "array";
        const lookupSource = f1.safe({}, reference[lookupRef]);
        const isLookupSourceCollection = lookupSource && (lookupSource.type === "dictionary" || lookupSource.type === "array");
        if (isLookupArray && lookupRef && isLookupSourceCollection) {
          crossmap.push({
            lookupRef,
            sourceRef
          });
        }
      }
    }
    return crossmap;
  }
  __resolveReferences(reference, output, keyRef) {
    const typeDefinition = f1.safe({}, output[keyRef]);
    return typeDefinition["$ref"] ? this.__resolveReferences(reference, reference, typeDefinition["$ref"]) : typeDefinition;
  }
}
HIP.factory.element(TypeGroupElement);
