
import { html } from '../../../hip/library/lit-element/lit-element.js';
import { Debouncer } from '../../../hip/library/polymer/polymer.js';
import { dedupingMixin } from '../../../hip/library/polymer/polymer.js';
import { afterNextRender } from '../../../hip/library/polymer/polymer.js';
import { idlePeriod } from '../../../hip/library/polymer/polymer.js';
import '../../../aip/element/ai-behavior-template/ai-behavior-template.js';
import '../../../aip/element/ai-behavior-prefab/ai-behavior-prefab.js';
import '../../../aip/element/ai-compiler/ai-compiler.js';
import '../../../aip/element/ai-source/ai-source.js';
import '../../../aip/element/ai-state/ai-state.js';
import '../../../eip/element/hi-binding/hi-binding.js';
import '../../../eip/element/hi-stamp/hi-stamp.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import ess from '../../../pip/library/pi-ess/pi-ess.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import {
  HIP,
  AsLitHIC
} from '../../../eip/element/hi-hip/hi-hif.js';
import { BehaviorBehavior } from '../../../eip/element/hi-behavior-behavior/hi-behavior-behavior.js';
export function BaseBehaviorViewV1(Base) {
  class BehaviorViewV12 extends AsLitHIC(BehaviorBehavior(Base)) {
    constructor() {
      super(...arguments);
      this.ns = `ns-${f1.id()}`;
      this.level = 1;
      this._onSourceOutputChangedBound = this._onSourceOutputChanged.bind(this);
    }
    get defaultScript() {
      return {
        bucket: {
          view_header: {},
          view_footer: {},
          context: {}
        }
      };
    }
    static get properties() {
      return {
        level: {
          type: Number
        },
        autoRunScript: {
          type: Object
        },
        script: {
          type: Object
        },
        pointCutScript: {
          type: Object,
          attribute: false
        },
        environment: {
          type: String
        },
        ns: {
          type: String,
          reflect: true
        },
        context: {
          type: Object
        }
      };
    }
    static get api() {
      return ["startup", "shutdown"];
    }
    _renderH1Tag() {
      const ns = this.ns;
      return html`
        <ai-source id="source" gid="source" .ns=${ns}> </ai-source>
        <ai-compiler id="transpiler" gid="transpiler" .ns=${ns}> </ai-compiler>

        <ai-state id="code" gid="code" kind="logic" .ns=${ns}>
          <hi-stamp id="codeStamp" gid="codeStamp" .ns=${ns}> </hi-stamp>
        </ai-state>

        <hi-binding
          id="screenToCodeBinding"
          source="_screen"
          signal="state-ended"
          sink="code"
          field="refreshBindings"
          disabled
          .ns=${ns}
        >
        </hi-binding>
        <hi-binding source="codeStamp" sink="code" field="refreshBindings" .ns=${ns}> </hi-binding>
      `;
    }
    cleanup() {
      afterNextRender(this, function() {
        this.__disableInstancesDebug();
        this.__removeViewElements();
      });
    }
    startup() {
      return new Promise((resolve) => {
        const initializeEl = HIP.db.gid.queryOne("initialize", this.ns);
        if (initializeEl) {
          initializeEl.addEventListener("invoke-ended", resolve, {
            once: true
          });
        }
        this.$.source.flush();
        this.$.screenToCodeBinding.disabled = false;
        this.$.screenToCodeBinding.relink();
        const invoked = initializeEl ? initializeEl.invoke({}) : Promise.resolve("immediate");
        return invoked.then((rtn) => {
          if (this._templateEl && f1.is.function(this._templateEl.initialize)) {
            this._templateEl.initialize();
          }
          this._dispatchProcess("initialize", {
            detail: rtn
          });
          resolve(rtn);
        });
      });
    }
    uninitialize() {
      return this.shutdown();
    }
    shutdown() {
      return new Promise((resolve) => {
        const prefabEl = this._templateEl;
        if (prefabEl && f1.is.function(prefabEl.uninitialize)) {
          prefabEl.uninitialize();
        } else if (prefabEl && f1.is.function(prefabEl.disconnectedCallback)) {
          prefabEl.disconnectedCallback();
        }
        const uninitializeEl = HIP.db.gid.queryOne("uninitialize", this.ns);
        if (uninitializeEl) {
          uninitializeEl.addEventListener("invoke-ended", resolve, {
            once: true
          });
          uninitializeEl.invoke({});
        }
        this.cleanup();
        this._dispatchProcess("uninitialize", {
          detail: {}
        });
        resolve({});
      });
    }
    _renderViewTemplate(viewTemplateEl) {
    }
    __disableInstancesDebug() {
      f1.each((listenersCB) => {
        ess.off(listenersCB);
      }, this._debugListeners || []);
    }
    __removeViewElements() {
      this._templateEl = void 0;
      HIP.db.gid.removeDatabase(this.ns);
      const builder = f1.read("/$/transpiler/builder", void 0, this);
      if (builder) {
        this.$.transpiler.builder.cleanup();
      }
    }
    transpiler() {
      return this.$.transpiler.transpiler;
    }
    builder() {
      return this.$.transpiler.transpiler.builder;
    }
    _invokeTranspiler(script, ver) {
      this.$.transpiler.unsetFlag("disallow-global");
      this.$.transpiler.namespace = this.ns;
      script = f1.merge("onto", {}, f1.merge("onto", this.defaultScript, script));
      script = f1.is.function(this.pointCutScript) ? this.pointCutScript(script) : script;
      return this.$.transpiler.builder.cleanup().build(script, this._templateEl).then((view) => {
        const prefabEl = this._templateEl;
        const { hilCompiled } = view;
        this.__enableInstancesDebug(hilCompiled);
        this.__updateView__v1(hilCompiled);
        this._renderViewTemplate(prefabEl);
        return { hilCompiled, view };
      }).catch((issues) => this.__reportIssues(issues));
    }
    __updateView__v1(hilCompiled) {
      this.$.codeStamp.input = hilCompiled.markup.children;
      this.$.source.input = hilCompiled.source;
      afterNextRender(this, function() {
        this.__setRelease();
        this.$.transpiler.setFlag("disallow-global");
        this.startup();
        this._refreshNewLogic();
      });
    }
    __setRelease() {
      const envEl = HIP.db.gid.queryOne("environment", this.ns);
      if (envEl && this.environment != null) {
        envEl.setting = this.environment;
      }
    }
    __reportIssues(issues) {
      const ns = this.ns;
      f1.each(
        ({ code, context }) => ess.send({
          ns,
          code,
          site: "ai-transpiler",
          source: "ai-view",
          action: "compile",
          data: context
        }),
        issues
      );
      return issues;
    }
    __enableInstancesDebug(hilCompiled) {
      const debugRequests = f1.read("/source/debug", [], hilCompiled);
      const debugSources = f1.map((declarationName) => {
        return {
          source: declarationName
        };
      }, debugRequests);
      this._debugListeners = f1.map((event) => {
        return ess.on(event, ess.reporters.console);
      }, debugSources);
    }
    _renderInternalViews(prefabEl, hilCompiled) {
      const tags = f1.read("/source/tag", null, hilCompiled);
      if (tags == null) {
        return;
      }
      const internalRegions = f1.read("/$internal/tagged", null, tags);
      if (!internalRegions) {
        return;
      }
      const regionDefinitions = f1.filter(
        (_) => !!_,
        f1.map((_) => _.$def, internalRegions) || []
      );
      return f1.map((internalRegion) => {
        return this._renderInternalView(internalRegion, prefabEl);
      }, regionDefinitions);
    }
    _renderInternalView(subViewRegionEl, regionName) {
      return new Promise(() => {
        afterNextRender(this, function() {
          const source = this.$.source.input;
          const region = source.region[regionName];
          const regionTarget = f1.read("/name", "no-region-name", region);
          const subViewRegionName = `_sub_view__internal__${regionTarget}`;
          dom.setAttrs(subViewRegionEl, {
            id: subViewRegionName,
            "x-sub_view": "true"
          });
          HIP.fn.createSubView(this, {
            subViewRegionEl,
            regionTarget
          });
          this.appendChild(subViewRegionEl);
        });
      });
    }
    _onSourceOutputChanged() {
      this._sourceRefreshDebouncer = Debouncer.debounce(this._sourceRefreshDebouncer, idlePeriod, () => {
        Array.from(this.$.code.children).filter((el) => el.nodeName === "HI-BINDING").forEach((el) => {
          el.safeRelink();
        });
      });
    }
    _refreshNewLogic() {
      const sourceEl = HIP.db.gid.queryOne("source", this.ns);
      if (!sourceEl) {
        setTimeout(() => this._refreshNewLogic, 32);
      } else {
        sourceEl.addEventListener("output-changed", this._onSourceOutputChangedBound);
      }
    }
  }
  return HIP.factory.mixin(BehaviorViewV12);
}
export const BehaviorViewV1 = dedupingMixin(BaseBehaviorViewV1);
