
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { HIP, BehaviorElement, AsLitHIC } from '../../../eip/element/hi-hip/hi-hif.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import { setAttributes } from '../../../eip/element/hi-lit/hi-lit.js';
export const styles = css`
  :host {
    display: block;
  }
`;
export class ViewFootElement extends BehaviorElement(AsLitHIC(LitElement)) {
  constructor() {
    super(...arguments);
    this.__runtimeSubscriber = {
      id: Symbol(),
      load: () => {
        this.requestUpdate();
      },
      invoke: () => {
        this.requestUpdate();
      }
    };
  }
  static get is() {
    return "ai-view-foot";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      level: {
        type: Number
      }
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.runtime.subscribe(this.__runtimeSubscriber);
    this.style.display = "block";
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    this.runtime.unsubscribe(this.__runtimeSubscriber.id);
  }
  createRenderRoot() {
    return this;
  }
  render() {
    return html` ${this.renderToolbar()} ${this.renderStatus()} ${this.renderAlert()} ${this.renderTemplate()}`;
  }
  renderStatus() {
    const headerStatusI = this.runtime.workspace.queryInstance("footer_status");
    if (!headerStatusI) {
      return nothing;
    }
    const ns = this.ns;
    const { value, type } = headerStatusI.state();
    return html`
      <div slot="status">
        <hi-panel gid="footer_status_panel" .ns=${ns}></hi-panel>
        <ai-factory-info
          for="footer_status_panel"
          .ns=${ns}
          .type=${type}
          .value=${value}
          @field-trigger-ended=${(e) => this.runtime.interlink.proxyFieldValueIn(e.detail)}
        ></ai-factory-info>
      </div>
    `;
  }
  renderToolbar() {
    const footerToolbarI = this.runtime.workspace.queryInstance("footer_toolbar");
    if (!footerToolbarI) {
      return nothing;
    }
    const ns = this.ns;
    const { value, type } = footerToolbarI.state();
    return html`
      <hi-panel gid="footer_toolbar_panel" .ns=${ns} x-hie class="toolbar" style="justify-content: center"></hi-panel>
      <ai-factory-control
        for="footer_toolbar_panel"
        .ns=${ns}
        .type=${type}
        .value=${value}
        @field-trigger-ended=${(e) => this.runtime.interlink.proxyFieldValueIn(e.detail)}
      ></ai-factory-control>
    `;
  }
  renderAlert() {
    const footerAlertI = this.runtime.workspace.queryInstance("footer_alert");
    if (!footerAlertI) {
      return nothing;
    }
    const ns = this.ns;
    const { value, settings } = footerAlertI.state();
    return html`
      <hi-alert .ns=${ns} data-attrs=${setAttributes(settings.attributes)}>
        ${value.title}
        <span slot="details"> ${value.details} </span>
      </hi-alert>
    `;
  }
  renderTemplate() {
    const footerTemplateI = this.runtime.workspace.queryInstance("footer_template");
    if (!footerTemplateI) {
      return nothing;
    }
    const { value, type, settings } = footerTemplateI.state();
    const template = this.runtime.templates.get(f1.read("/template", "--no template--", settings));
    if (!template) {
      return nothing;
    }
    return template.render({ type })(value, "footer");
  }
}
HIP.factory.element(ViewFootElement);
export default ViewFootElement.is;
