
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { unsafeHTML } from '../../../hip/library/lit-html/directives/unsafe-html.js';
import { setAttributes } from '../../../eip/element/hi-lit/hi-lit.js';
import { HIP, BehaviorElement, AsLitHIC } from '../../../eip/element/hi-hip/hi-hif.js';
import '../../../yip/element/yi-view-header/yi-view-header.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class ViewHeadElement extends BehaviorElement(AsLitHIC(LitElement)) {
  constructor() {
    super(...arguments);
    this.__runtimeSubscriber = {
      id: Symbol(),
      load: () => {
        this.requestUpdate();
      },
      invoke: () => {
        this.requestUpdate();
      }
    };
  }
  static get is() {
    return "ai-view-head";
  }
  static get properties() {
    return {
      level: {
        type: Number
      }
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.runtime.subscribe(this.__runtimeSubscriber);
    this.style.display = "block";
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    this.runtime.unsubscribe(this.__runtimeSubscriber.id);
  }
  createRenderRoot() {
    return this;
  }
  render() {
    const viewI = this.runtime.workspace.queryInstance("view");
    const { description, title, subtitle, super_title } = f1.merge(
      "over",
      {
        description: "",
        super_title: "",
        subtitle: "",
        title: ""
      },
      f1.safe({}, viewI?.value())
    );
    return html`<yi-view-header>
      <p slot="super_title">${subtitle}${super_title}</p>
      <hi-title class="view-title" slot="title" level=${this.level} title=${title} .ns=${this.ns}></hi-title>
      <hi-reveal class="view-description" slot="description" max-height="2" max-content="200" .ns=${this.ns}
        >${unsafeHTML(description)}</hi-reveal
      >
      ${this.renderStatus()} ${this.renderAlert()} ${this.renderToolbar()} ${this.renderTemplate()}
    </yi-view-header>`;
  }
  renderAlert() {
    const headerAlertI = this.runtime.workspace.queryInstance("header_alert");
    if (!headerAlertI) {
      return nothing;
    }
    const ns = this.ns;
    const { value, settings } = headerAlertI.state();
    return html`
      <div slot="alert">
        <hi-alert .ns=${ns} data-attrs=${setAttributes(settings.attributes)}>
          ${value.title}
          <span slot="details"> ${value.details} </span>
        </hi-alert>
      </div>
    `;
  }
  renderToolbar() {
    const headerToolbarI = this.runtime.workspace.queryInstance("header_toolbar");
    if (!headerToolbarI) {
      return nothing;
    }
    const ns = this.ns;
    const { value, type } = headerToolbarI.state();
    return html`
      <div slot="toolbar">
        <hi-panel gid="header_toolbar_panel" .ns=${ns} x-hie class="toolbar"></hi-panel>
        <ai-factory-control
          for="header_toolbar_panel"
          .ns=${ns}
          .type=${type}
          .value=${value}
          @field-trigger-ended=${(e) => this.runtime.interlink.proxyFieldValueIn(e.detail)}
        ></ai-factory-control>
      </div>
    `;
  }
  renderStatus() {
    const headerStatusI = this.runtime.workspace.queryInstance("header_status");
    if (!headerStatusI) {
      return nothing;
    }
    const ns = this.ns;
    const { value, type } = headerStatusI.state();
    return html`
      <div slot="status">
        <hi-panel gid="header_status_panel" .ns=${ns}></hi-panel>
        <ai-factory-info
          for="header_status_panel"
          .ns=${ns}
          .type=${type}
          .value=${value}
          @field-trigger-ended=${(e) => this.runtime.interlink.proxyFieldValueIn(e.detail)}
        ></ai-factory-info>
      </div>
    `;
  }
  renderTemplate() {
    const headerTemplateI = this.runtime.workspace.queryInstance("header_template");
    if (!headerTemplateI) {
      return nothing;
    }
    const { value, type, settings } = headerTemplateI.state();
    const template = this.runtime.templates.get(f1.read("/template", "--no template--", settings));
    if (!template) {
      return nothing;
    }
    return html`<div slot="template">${template.render({ type })(value, "header")}</div>`;
  }
}
export default HIP.factory.element(ViewHeadElement);
