
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import '../../../yip/element/yi-view-header/yi-view-header.js';
export class ViewHeaderElement extends BehaviorElement(AsHIC(PolymerElement)) {
  static get is() {
    return "ai-view-header";
  }
  static get template() {
    return html`
      <header class="ai-view-header-block ptl pbm mbm baz bbs bdr-solid clr-gray_40-bdr">
        <hi-title
          id="title"
          class="view-title ai-view-title mbsx"
          title="[[ context.title ]]"
          level="[[ level ]]"
        ></hi-title>
        <section>
          <hi-reveal
            id="description"
            max-height="2"
            max-content="200"
            class="clr-gray_80-txt view-description"
            inner-h-t-m-l="[[ context.description ]]"
          ></hi-reveal>
        </section>
      </header>

      <!-- header widget -->
      <ai-block gid="_view_header__block" region="view_header__widget_group__builder" ns$="[[ ns ]]" class="mll mrl">
        <div id="_view_header_none_task__widget_group__builder" l-kind="hort" l-gutter l-width-fixed l-align="center">
          <ai-builder-widget gid="_view_header_none_task__widget_group__builder" ns$="[[ ns ]]"> </ai-builder-widget>
        </div>
      </ai-block>
      <hi-binding source="source" sink="_view_header_none_task__widget_group__builder" ns$="[[ ns ]]">
        <di-fn-get path="region.view_header.tasks.none"> </di-fn-get>
      </hi-binding>

      <!-- header message -->
      <hi-panel gid="_view_header__panel" region="view_header" ns$="[[ ns ]]">
        <ai-builder-message
          gid="_view_header__message_group__builder"
          id="_view_header__message_group__builder"
          ns$="[[ ns ]]"
        >
        </ai-builder-message>
      </hi-panel>
      <hi-binding source="source" sink="_view_header__message_group__builder" ns$="[[ ns ]]">
        <di-fn-get path="region.view_header.messages"> </di-fn-get>
      </hi-binding>

      <!-- header template -->
      <section>
        <hi-panel id="_view_header__target" gid="_view_header__target" ns$="[[ ns ]]"> </hi-panel>
      </section>
      <hi-render id="_view_header__render" gid="_view_header__render" for="_view_header__target" ns$="[[ ns ]]">
      </hi-render>
      <ai-builder-template
        id="_view_header__template_builder"
        gid="_view_header__template_builder"
        for="_view_header__render"
        mode="template"
        ns$="[[ ns ]]"
      >
      </ai-builder-template>
      <hi-binding source="context" field="template" sink="_view_header__template_builder" ns$="[[ ns ]]">
        <di-fn-get path="template_header"></di-fn-get>
      </hi-binding>
      <hi-binding source="view_header_type" sink="_view_header__template_builder" ns$="[[ ns ]]"> </hi-binding>
      <hi-binding source="view_header" sink="_view_header__render" ns$="[[ ns ]]"> </hi-binding>
    `;
  }
  static get properties() {
    return {
      context: {
        type: Object
      },
      level: {
        type: Number
      }
    };
  }
  static get observers() {
    return ["__contextUpdates(context.title, level)"];
  }
  _attachDom(dom) {
    this.classList.add("display-block");
    this.appendChild(dom);
    return null;
  }
  __contextUpdates(contextUpdate) {
    const headerTag = this.querySelector("#title");
    headerTag.title = contextUpdate;
    headerTag.level = this.level;
  }
}
export default HIP.factory.element(ViewHeaderElement);
