
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { BehaviorViewV1 } from './ai-behavior-view-v1.js';
import { AsLitHIC, BehaviorElement, HIP, BehaviorCustomizable } from '../../../eip/element/hi-hip/hi-hif.js';
import { Ignition } from '../../../pip/library/pi-interop/pi-interop.js';
import lucid from '../../../aip/library/ai-lucid/ai-lucid.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import ess from '../../../pip/library/pi-ess/pi-ess.js';
import $ip from '../../../pip/library/pi-ip/pi-ip.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
export const styles = css`
  :host {
    /* this is a problem because not all of our view templates use a white background */
    background: var(--hi-clr--white);
    display: block;
    width: 100%;
  }

  :host([view-template='none']) {
    display: none;
  }
`;
export class ViewElement extends BehaviorViewV1(BehaviorCustomizable(BehaviorElement(AsLitHIC(LitElement)))) {
  constructor() {
    super(...arguments);
    this.findFilenameInHREF = /.*?\/([^\/]*?)\.js.*?$/;
  }
  static get is() {
    return "ai-view";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {};
  }
  static get api() {
    return ["invoke"];
  }
  static get autoBinding() {
    return {
      signal: "invoke-ended",
      field: "invoke"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    dom.write(() => {
      this.__initializeView();
    });
  }
  __initializeView() {
    if (!$ip.flags.is("/aip/ai-view/no-padding")) {
      this.classList.add("pll", "prl");
    }
    dom.setAttrs(this, {
      role: "region",
      gid: "view"
    });
    const regionEl = this.parentElement;
    if (regionEl) {
      dom.setAttrs(regionEl, {
        gid: f1.safe("view-region", regionEl.gid),
        ns: this.ns
      });
    }
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    this.cleanup();
    const sourceEl = HIP.db.gid.queryOne("source", this.ns);
    if (sourceEl) {
      sourceEl.removeEventListener("output-changed", this._onSourceOutputChangedBound);
    }
  }
  update(props) {
    super.update(props);
    this._watchChanges(props, [
      {
        properties: ["script", "autoRunScript"],
        method: "__scriptWatcher"
      }
    ]);
  }
  render() {
    const ver = f1.to.integer(f1.read("/script/hil", "1.0", this));
    const requiresAIElements = ver <= 2;
    return html`
      ${requiresAIElements ? this._renderH1Tag() : nothing}
      <slot name="header"></slot>
      <slot></slot>
      <slot name="footer"></slot>
    `;
  }
  firstUpdated() {
    const shadowRoot = this.shadowRoot;
    this.$ = f1.reduce(
      (rec, id) => {
        rec[id] = shadowRoot.querySelector(`#${id}`);
        return rec;
      },
      {},
      ["screenToCodeBinding", "codeStamp", "transpiler", "source", "error", "code"]
    );
  }
  running() {
    return !f1.is.nil(this._templateEl);
  }
  stateChange(locationEl) {
    return new Promise(async (resolve) => {
      if (this._templateEl && f1.is.function(this._templateEl.stateChange)) {
        await this._templateEl.stateChange();
      }
      const runtime = this.runtime();
      if (runtime) {
        const locationI = runtime.workspace.queryInstance("location");
        if (locationI) {
          locationI.invoke(null);
        }
      }
      const stateChangeEl = HIP.db.gid.queryOne("state_change", this.ns);
      if (stateChangeEl) {
        stateChangeEl.addEventListener("invoke-ended", resolve, {
          once: true
        });
        stateChangeEl.invoke(locationEl);
      }
      resolve({});
    });
  }
  get prefabEl() {
    return this._templateEl;
  }
  get templateEl() {
    return this._templateEl;
  }
  runtime() {
    return this._runtime;
  }
  queryOne(query) {
    return HIP.browser.queryOne(query, this.ns, this);
  }
  invoke(script) {
    const ver = f1.to.integer(f1.read("/hil", "1.0", script));
    const runtimeScript = f1.read("/runtime", {}, script);
    const renderScript = f1.read("/render", {}, script);
    this.activeScript = script;
    const loadedViewTemplate = this.__loadViewTemplate(script);
    const runRuntime = loadedViewTemplate.then(async () => {
      return ver === 2 || ver === 3 ? await this.__loadRuntime(runtimeScript, renderScript, ver) : script;
    });
    const runTranspiler = runRuntime.then(async () => {
      script = f1.write("/runtime", null, script);
      script = f1.write("/render", null, script);
      ver === 1 || ver === 2 ? await this._invokeTranspiler(script, ver) : script;
      script = f1.write("/runtime", runtimeScript, script);
      script = f1.write("/render", renderScript, script);
      return script;
    });
    const renderViewTemplate = runTranspiler.then(({ hilCompiled, view }) => {
      if (ver < 3) {
        return { hilCompiled, view };
      }
      this._renderViewTemplate(this._templateEl);
      return { hilCompiled, view };
    });
    const renderView = renderViewTemplate.then(({ hilCompiled }) => {
      const templateEl = this._templateEl;
      this._renderHeader(templateEl, hilCompiled, ver);
      this._renderFooter(templateEl, hilCompiled, ver);
      this._renderInternalViews(templateEl, hilCompiled);
    });
    return renderView.then(() => script).catch((issueReasons) => {
      const hasIssues = Array.isArray(issueReasons) && issueReasons.length > 0;
      if (!hasIssues) {
        return;
      }
      const groupByErrorCode = issueReasons.reduce((all, item) => {
        all[item.code] = item;
        return all;
      }, {});
      const issues = f1.values(groupByErrorCode);
      if (!f1.is.empty(issues)) {
        ess.send(
          {
            site: "ai-view",
            source: this.id,
            action: "invoke",
            data: issues
          },
          "failure"
        );
      }
    });
  }
  showDialog(regionName) {
    const subViewRegionEl = dom.create(null, "hi-dialog", {
      gid: "view-dialog,view-overlay",
      aesthetic: "minimal",
      id: "view-dialog",
      opened: "opened",
      modal: "modal",
      ns: this.ns
    });
    this._renderInternalView(subViewRegionEl, regionName);
  }
  showDrawer(regionName) {
    const subViewRegionEl = dom.create(null, "hi-drawer", {
      gid: "view-drawer,view-overlay",
      aesthetic: "minimal",
      class: "max-w-960",
      direction: "right",
      id: "view-drawer",
      opened: "opened",
      modal: "modal",
      ns: this.ns
    });
    this._renderInternalView(subViewRegionEl, regionName);
  }
  cleanup() {
    super.cleanup();
    this.__unloadRuntime();
  }
  __scriptWatcher(script, autoRunScript) {
    if (autoRunScript) {
      script = autoRunScript;
    }
    const whenInStatis = this.running() ? this.shutdown() : Promise.resolve();
    whenInStatis.then(() => {
      this.invoke(script);
    });
  }
  __loadRuntime(runtimeScript, renderScript, ver) {
    const runtimeId = f1.read("/id", void 0, runtimeScript);
    const runtimeNs = f1.read("/ns", this.ns, runtimeScript);
    const runtime = this._runtime = lucid.createRuntime(runtimeNs, runtimeId);
    const defaultRuntime = f1.read("/templateEl/defaultRuntime", {}, this);
    const runtimeHIL = f1.merge("onto", f1.merge("onto", {}, defaultRuntime), runtimeScript);
    return runtime.renderWith(renderScript).computeWith(runtimeHIL);
  }
  async __unloadRuntime() {
    const runtime = this.runtime();
    if (!runtime) {
      return;
    }
    await runtime.deactivate();
    lucid.destroyRuntime(runtime);
    this._runtime = void 0;
  }
  _renderHeader(templateEl, hilCompiled, ver) {
    if (templateEl.requestNoHeader) {
      return;
    }
    if (f1.read("/source/behavior/request-no-header", null, hilCompiled) != null) {
      return;
    }
    const headerEl = ver <= 1 ? this._renderHeader__v1(templateEl, hilCompiled) : this._renderHeader__v3(templateEl, hilCompiled);
    if (templateEl.requestCenterAlign) {
      this.__enableCenterAlign(headerEl);
    }
    if (this.$.header) {
      this.$.header.remove();
    }
    this.$.header = headerEl;
    this.prepend(headerEl);
    return headerEl;
  }
  __enableCenterAlign(element) {
    dom.setAttrs(element, {
      "l-justify": "center",
      "l-align": "center",
      "l-kind": "hort"
    });
  }
  _renderHeader__v3(templateEl, hilCompiled) {
    const headerEl = dom.create(null, "ai-view-head", {
      gid: "view_head",
      id: "view_head",
      level: this.level,
      slot: "header",
      ns: this.ns
    });
    headerEl.gid = "view_head";
    headerEl.id = "view_head";
    headerEl.ns = this.ns;
    headerEl.runtime = this.runtime();
    return headerEl;
  }
  _renderHeader__v1(templateEl, hilCompiled) {
    return dom.create(null, "ai-view-header", {
      gid: "_view_header",
      id: "_view_header",
      level: this.level,
      slot: "header",
      ns: this.ns
    });
  }
  _renderFooter(templateEl, hilCompiled, ver) {
    if (templateEl.requestNoFooter) {
      return;
    }
    if (f1.read("/source/behavior/request-no-footer", null, hilCompiled) != null) {
      return;
    }
    const footerEl = ver <= 1 ? this._renderFooter__v1(templateEl, hilCompiled) : this._renderFooter__v3(templateEl, hilCompiled);
    if (templateEl.requestCenterAlign) {
      this.__enableCenterAlign(footerEl);
    }
    if (this.$.footer) {
      this.$.footer.remove();
    }
    this.$.footer = footerEl;
    this.appendChild(footerEl);
    return footerEl;
  }
  _renderFooter__v3(templateEl, hilCompiled) {
    const footerEl = dom.create(null, "ai-view-foot", {
      level: this.level,
      gid: "view_foot",
      id: "view_foot",
      slot: "footer",
      ns: this.ns
    });
    footerEl.gid = "view_foot";
    footerEl.id = "view_foot";
    footerEl.ns = this.ns;
    footerEl.runtime = this.runtime();
    return footerEl;
  }
  _renderFooter__v1(templateEl, hilCompiled) {
    return dom.create(null, "ai-view-footer", {
      gid: "_view_footer",
      id: "_view_footer",
      level: this.level,
      slot: "footer",
      ns: this.ns
    });
  }
  __loadViewTemplate(hil) {
    const prefabRequest = this.__readViewTemplateRequestId(hil);
    const updatedPrefabRequest = this.__createPrefabResource(hil, prefabRequest);
    return updatedPrefabRequest.then(() => this.__loadTemplate(prefabRequest)).then((templateEl) => Ignition.create().startup(templateEl, {})).then((_) => {
      dom.write(() => {
        dom.setAttr(this, `view-template`, prefabRequest);
      });
      return _;
    });
  }
  __readViewTemplateRequestId(userScriptHIL) {
    if (userScriptHIL.render && userScriptHIL.render.view_template) {
      return userScriptHIL.render.view_template;
    } else if (userScriptHIL.system && userScriptHIL.system.prefab) {
      return userScriptHIL.system.prefab;
    } else if (userScriptHIL.prefab) {
      return userScriptHIL.prefab;
    } else {
      return "none";
    }
  }
  async __createPrefabResource(hil, requestId) {
    if (!requestId) {
      return Promise.reject("invoke could not find prefab");
    }
    if (typeof requestId !== "string") {
      return Promise.reject(`a prefab name must be a string`);
    }
    if (requestId === "none") {
      return Promise.resolve(requestId);
    }
    const prefabRequestExpanded = this.normalizeViewTemplateRequestId(requestId);
    const prefabResource = await $ip.require(prefabRequestExpanded);
    return requestId;
  }
  normalizeViewTemplateRequestId(requestId) {
    const isOldFormat = f1.search("@hip", 0, requestId) && !f1.search("-prefab-", 0, requestId);
    const isBareResourceFormat = !isOldFormat;
    requestId = isBareResourceFormat ? requestId : f1.pipe([f1.split("/"), f1.nth(1), f1.prefix("@eab-aip/ai-prefab-")], requestId);
    return f1.search("?resource_type", 0, requestId) ? requestId : `${requestId}?resource_type=module`;
  }
  __loadTemplate(requestId) {
    const prefabRequestPath = $ip.path(requestId);
    const prefabId = this.__getFilenameInHREF(prefabRequestPath);
    const prefabName = prefabId.includes("ai-") || prefabId.includes("eab-") ? prefabId : `ai-prefab-${prefabId}`;
    return this._templateEl = document.createElement(prefabName);
  }
  __getFilenameInHREF(uri) {
    const matched = uri.match(this.findFilenameInHREF);
    const filename = f1.read("/1", "", f1.safe([], matched));
    return filename.replace(".js", "");
  }
  _renderViewTemplate(viewTemplateEl) {
    if (!viewTemplateEl) {
      return;
    }
    Array.from(this.children).forEach((el) => {
      if (el.getAttribute("id") === "prefab") {
        el.remove();
      }
    });
    viewTemplateEl.classList.add("display-block");
    dom.setAttrs(viewTemplateEl, {
      level: this.level,
      gid: "prefab",
      id: "prefab",
      ns: this.ns
    });
    viewTemplateEl.gid = "prefab";
    viewTemplateEl.id = "prefab";
    viewTemplateEl.ns = this.ns;
    viewTemplateEl["runtime"] = this.runtime();
    this.appendChild(viewTemplateEl);
    if (f1.is.in("connectedViewCallback", viewTemplateEl) && f1.is.function(viewTemplateEl.connectedViewCallback)) {
      viewTemplateEl.connectedViewCallback();
    }
  }
}
export default HIP.factory.element(ViewElement);
