
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import { BehaviorModifier } from '../../../eip/element/hi-behavior-modifier/hi-behavior-modifier.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class FnFilterElement extends BehaviorModifier(BehaviorElement(AsHIC(PolymerElement))) {
  static get is() {
    return "di-fn-filter";
  }
  static get properties() {
    return {
      kind: {
        type: String,
        reflectToAttribute: true
      },
      field: {
        type: String,
        reflectToAttribute: true
      },
      comparator: {
        type: String,
        reflectToAttribute: true,
        value: "like"
      },
      not: {
        type: Boolean,
        reflectToAttribute: true
      },
      value: {
        type: Object,
        notify: true
      }
    };
  }
  static get observers() {
    return ["_settingsWatcher(field, comparator, not, value)"];
  }
  get purpose() {
    return "filter";
  }
  _modify(input) {
    const field = this.field;
    const comparator = this.not ? `not ${this.comparator}` : this.comparator;
    const valuePeek = HIP.kernel.attributeToValue(this.value);
    if (valuePeek == null) {
      return input;
    }
    const value = HIP.kernel.isReference(valuePeek) ? HIP.kernel.resolveReference(valuePeek, this) : valuePeek;
    const isValueComplex = f1.is.object(value);
    const isComparatorComplex = field == null;
    let filterRule = "pass-thru";
    if (isValueComplex && isComparatorComplex) {
      filterRule = "deep-object-match";
    } else if (isValueComplex && !isComparatorComplex) {
      filterRule = "object-match";
    } else if (!isValueComplex && !isComparatorComplex) {
      filterRule = "field-match";
    }
    if (this.kind != null) {
      filterRule = this.kind;
    }
    this.kind = filterRule;
    switch (filterRule) {
      case "complex":
      case "deep-object-match":
        return f1.filter((o) => {
          return f1.reduce(
            (isMatch, _value, _field) => {
              return isMatch && HIP.kernel.runComparator(comparator, this.__lookupValue(_field, o), _value);
            },
            true,
            this.__objectToFilter(value)
          );
        }, input);
      case "object-match":
        return f1.filter(
          (o) => HIP.kernel.runComparator(
            comparator,
            this.__lookupValue(field, o),
            this.__lookupValue(field, value)
          ),
          input
        );
      case "field-match":
        return f1.filter(
          (o) => HIP.kernel.runComparator(comparator, this.__lookupValue(field, o), value),
          input
        );
      case "pass-thru":
      default:
        return input;
    }
  }
  __lookupValue(field, o) {
    return field ? o[field] : o;
  }
  __objectToFilter(value) {
    return Object.keys(value).reduce((acc, key) => {
      const isNotNil = value[key] != null;
      const isNotEmptyString = typeof value !== "string" || typeof value === "string" && value[key] != "";
      if (isNotNil && isNotEmptyString) {
        acc[key] = value[key];
      }
      return acc;
    }, {});
  }
}
HIP.factory.element(FnFilterElement);
