
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import { BehaviorModifier } from '../../../eip/element/hi-behavior-modifier/hi-behavior-modifier.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class FnMapElement extends BehaviorModifier(BehaviorElement(AsHIC(PolymerElement))) {
  static get is() {
    return "di-fn-map";
  }
  static get properties() {
    return {
      kind: {
        type: String,
        value: "collection",
        reflectToAttribute: true
      },
      host: {
        type: String,
        reflectToAttribute: true
      },
      method: {
        type: String,
        reflectToAttribute: true
      }
    };
  }
  static get observers() {
    return ["__settingsWatcher(host, method)"];
  }
  get purpose() {
    return "map";
  }
  _modify(input) {
    const _hostEl = HIP.browser.queryOne(this.host, this.ns);
    const method = this.method;
    let hostEl = void 0;
    if (!_hostEl) {
      return input;
    }
    if (_hostEl[method] != null) {
      hostEl = _hostEl;
    } else if ((_hostEl["output"] || {})[method] != null) {
      hostEl = _hostEl["output"];
    }
    if (!hostEl) {
      return input;
    }
    const _method = f1.read(f1.path(method), (hostEl2, o, index) => o, hostEl);
    switch (this.kind) {
      case "collection":
        return f1.map((o, index) => _method.call(hostEl, o, index), input);
      case "object":
        return _method.call(hostEl, input, -1);
      default:
        return input;
    }
  }
}
HIP.factory.element(FnMapElement);
