
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import { BehaviorModifier } from '../../../eip/element/hi-behavior-modifier/hi-behavior-modifier.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
export class FnOrderElement extends BehaviorModifier(BehaviorElement(AsHIC(PolymerElement))) {
  static get is() {
    return "di-fn-order";
  }
  static get properties() {
    return {
      field: {
        type: String,
        reflectToAttribute: true,
        value: "order"
      },
      initialOrder: {
        type: Number,
        value: 1
      }
    };
  }
  static get observers() {
    return ["__settingsWatcher(field)"];
  }
  static get autoBinding() {
    return {
      signal: "modify-ended",
      field: "field"
    };
  }
  get mutator() {
    return true;
  }
  get purpose() {
    return "order";
  }
  _modify(input) {
    if (!Array.isArray(input)) {
      return input;
    }
    const initialOrder = this.initialOrder | 0;
    const field = this.field;
    if (input.length > 0 && input[0][field] == null) {
      input.forEach((o, i) => {
        o[field] = i + initialOrder;
      });
    }
    input.forEach((o, i) => {
      o[field] = o[field] | 0;
    });
    const sorted = field && input.length > 0 ? input.sort((a, b) => HIP.data.sortComparator(a, b, field, "asc")) : input;
    for (let i = 0, n = sorted.length - 1; i < n; i++) {
      if (sorted[i][field] === sorted[i + 1][field]) {
        const t = sorted[i];
        sorted[i] = sorted[i + 1];
        sorted[i + 1] = t;
        break;
      }
    }
    sorted.forEach((o, i) => {
      o[field] = i + initialOrder;
    });
    return sorted;
  }
}
HIP.factory.element(FnOrderElement);
