
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import { BehaviorModifier } from '../../../eip/element/hi-behavior-modifier/hi-behavior-modifier.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class FnParserElement extends BehaviorModifier(BehaviorElement(AsHIC(PolymerElement))) {
  static get is() {
    return "di-fn-parser";
  }
  static get properties() {
    return {
      kind: {
        type: String,
        value: "collection",
        reflectToAttribute: true
      },
      $type: {
        type: Object
      }
    };
  }
  static get observers() {
    return ["__settingsWatcher()"];
  }
  get purpose() {
    return "parser";
  }
  _modify(input, owner) {
    if (!input) {
      return input;
    }
    this._ensureTypeGroup();
    this._ensureSource();
    const typeGroupReference = this.typeGroup ? this.typeGroup["reference"] : {};
    const sourceReference = this.source ? this.source["reference"] : {};
    const ownerType = sourceReference[owner.id];
    const refRealType = typeGroupReference[f1.read("/$type/$ref", "", ownerType) || f1.read("/$type/$ref", "", this)];
    const realType = f1.read("/$type/properties", void 0, ownerType);
    const instanceType = realType || refRealType || {};
    const instanceTypeProperties = f1.read("/properties", {}, instanceType);
    switch (this.kind) {
      case "collection":
        return f1.map((o) => this.__applyParsers(instanceTypeProperties, o), input);
      case "object":
        return this.__applyParsers(instanceTypeProperties, input);
      default:
        return input;
    }
  }
  __applyParsers(instanceTypeProperties, o) {
    for (const fieldName in o) {
      const fieldType = instanceTypeProperties[fieldName];
      if (!fieldType) {
        continue;
      }
      if (fieldType["properties"]) {
        const typeProperties = ((instanceTypeProperties || {})[fieldName] || {})["properties"];
        if (Array.isArray(o[fieldName])) {
          const list = o[fieldName];
          f1.map((prop) => this.__applyParsers(typeProperties, prop), list);
        } else {
          this.__applyParsers(typeProperties, o[fieldName]);
        }
      }
      const visitor_type = fieldType["parse_type"] || fieldType["type"];
      if (!visitor_type) {
        continue;
      }
      const parseTypeVisitor = HIP.visitor.query({ visitor_type }, "parse");
      if (!parseTypeVisitor) {
        continue;
      }
      f1.write(f1.path(fieldName), parseTypeVisitor.build(f1.read(f1.path(fieldName), void 0, o)), o);
    }
    return o;
  }
}
HIP.factory.element(FnParserElement);
