
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import { BehaviorModifier } from '../../../eip/element/hi-behavior-modifier/hi-behavior-modifier.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';
export class FnRelationshipElement extends BehaviorModifier(BehaviorElement(AsHIC(PolymerElement))) {
  static get is() {
    return "di-fn-relationship";
  }
  static get properties() {
    return {
      kind: {
        type: String,
        value: "auto",
        reflectToAttribute: true
      }
    };
  }
  get purpose() {
    return "relationship";
  }
  _modify(input) {
    const instanceTop = this.parentElement;
    const instanceName = instanceTop.id;
    const typeInstance = HIP.db.gid.queryOne(`${instanceName}_type`, this.ns);
    const typeFields = f1.read("/output/properties", {}, typeInstance);
    this.__parseTypeRelationshipFields(typeFields, input);
    return input;
  }
  __parseTypeRelationshipFields(typeFields, input, depth = { __last_field_name: "" }) {
    const ns = this.ns;
    const relationships = f1.reduce(
      (relationships2, field) => {
        const configuration = f1.read("/$configure/relationship", void 0, field);
        if (configuration) {
          const instance = configuration.instance;
          const instanceEl = HIP.browser.queryOne(instance, ns);
          if (instanceEl == null) {
            return relationships2;
          }
          relationships2.push({
            configuration,
            instanceEl,
            instance,
            field
          });
        }
        return relationships2;
      },
      [],
      typeFields
    );
    const relationshipPromises = f1.map((relationship) => {
      return this.__parseRelationship(relationship, input, depth);
    }, relationships);
    Promise.all(relationshipPromises).then(() => {
      const hasDoneAll = f1.reduce(
        (has, relationshipPromise) => {
          return has && relationshipPromise["__done__"];
        },
        true,
        relationshipPromises
      );
      if (!hasDoneAll) {
        this.parentElement.wake();
      }
    });
  }
  __parseRelationship(relationship, input, depth) {
    const { instanceEl, field } = relationship;
    const isArrayOfObjects = f1.is.array(input) && f1.is.record(ifn.first(input));
    const kind = this.kind === "auto" ? isArrayOfObjects ? "collection" : "object" : this.kind;
    const data = kind === "collection" ? f1.to.array(input) : [input];
    const fieldName = field.name;
    const fieldPromise = ifn.reduce(
      (p, o) => {
        return new Promise((resolve, reject) => {
          p.then(() => {
            const _depth = Object.assign(
              {},
              {
                [`__${depth["__last_field_name"]}`]: depth,
                ...o
              },
              {
                __last_field_name: fieldName
              }
            );
            instanceEl.parameters = _depth;
            const instancePromise = instanceEl.invoke();
            instancePromise["__done__"] = false;
            return instancePromise.then((crossLinkedAPI) => {
              ifn.set(fieldName, o, crossLinkedAPI);
              return field.properties != null ? this.__parseTypeRelationshipFields(field.properties, crossLinkedAPI, _depth) : crossLinkedAPI;
            }).then((v) => {
              this["__done__"] = true;
              return resolve(v);
            }).catch(reject);
          });
        });
      },
      Promise.resolve(),
      data
    );
    return fieldPromise;
  }
}
HIP.factory.element(FnRelationshipElement);
