
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import { BehaviorModifier } from '../../../eip/element/hi-behavior-modifier/hi-behavior-modifier.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';
export class FnRewriteElement extends BehaviorModifier(BehaviorElement(AsHIC(PolymerElement))) {
  static get is() {
    return "di-fn-rewrite";
  }
  static get properties() {
    return {
      kind: {
        type: String,
        value: "auto",
        reflectToAttribute: true
      },
      writeMode: {
        type: String,
        value: "update",
        reflectToAttribute: true
      },
      path: {
        type: String,
        reflectToAttribute: true,
        value: ""
      },
      pattern: {
        type: Object
      }
    };
  }
  static get splitSelfReferencing() {
    return new RegExp(`\\.([^\\s]*)`, "ig");
  }
  static get fieldIsPrivate() {
    return /^\.?_/;
  }
  static get observers() {
    return ["__settingsWatcher(path, pattern)"];
  }
  get purpose() {
    return "rewrite";
  }
  _modify(input) {
    if (!input) {
      return input;
    }
    const pattern = this.pattern;
    if (!ifn.is(Object, pattern)) {
      return input;
    }
    const to = this.__get(this.path, input);
    const from = this.__get(this.path, input);
    const isArrayOfObjects = f1.is.array(input) && f1.is.record(ifn.first(input));
    const kind = this.kind === "auto" ? isArrayOfObjects ? "collection" : "object" : this.kind;
    switch (kind) {
      case "collection":
        return this.__set(
          this.path,
          input,
          f1.map((_fromTo) => this.__rewriteObject(_fromTo, _fromTo, pattern), from)
        );
      case "object":
      default:
        return this.__set(this.path, input, this.__rewriteObject(from, to, pattern));
    }
  }
  __rewriteObject(from, to, pattern) {
    return this.__rewritePattern(from, this.__createObject(to), pattern);
  }
  __createObject(o) {
    return this.writeMode === "create" ? {} : o;
  }
  __rewritePattern(from, to, pattern, path = "") {
    let value;
    for (const k in pattern) {
      const fromRef = pattern[k];
      const toRef = `${path}.${k}`;
      const isDescendingObject = ifn.is(Object, fromRef);
      if (isDescendingObject) {
        this.__rewritePattern(from, to, fromRef, toRef);
        continue;
      }
      value = void 0;
      if (!HIP.kernel.isInstruction(fromRef)) {
        value = fromRef;
      } else if (HIP.kernel.isExpression(fromRef)) {
        value = HIP.kernel.resolveExpression(fromRef, this, from);
      } else if (HIP.kernel.isReference(fromRef)) {
        const [instance, field, property] = fromRef.split(HIP.kernel.symbolTuple);
        const hasOneColon = property === void 0;
        if (hasOneColon) {
          this.__assign(toRef, to, fromRef);
          continue;
        }
        const instanceEl = instance ? HIP.browser.queryOne(instance, this.ns) : this.parentElement;
        const isValueWithColon = !instanceEl;
        if (isValueWithColon) {
          this.__assign(toRef, to, void 0, fromRef);
          continue;
        }
        const eventValue = HIP.kernel.eventAsValueOf(field, instanceEl);
        if (eventValue != null) {
          this.__assign(toRef, to, eventValue);
          continue;
        }
        const eventMethod = HIP.kernel.eventAsMethodOf(field, instanceEl);
        if (eventMethod != null) {
          this.__assign(toRef, to, eventMethod);
          continue;
        }
        const object = f1.read(f1.path(field || "output"), null, instanceEl);
        const _object = object == null ? (
          {}
        ) : object;
        value = property ? ifn.get(property, _object) : _object;
      } else if (HIP.kernel.isPath(fromRef)) {
        value = fromRef.replace(FnRewriteElement.splitSelfReferencing, (_0, path2) => {
          const _value = ifn.get(path2, from);
          return _value != null ? _value : "";
        });
      } else if (fromRef !== void 0) {
        value = fromRef;
      }
      if (value !== void 0) {
        this.__assign(toRef, to, value);
      }
    }
    return to;
  }
  __assign(toRef, to, value, finalValue) {
    if (toRef.match(FnRewriteElement.fieldIsPrivate) != null) {
      return;
    }
    value = value != null ? value : finalValue || value;
    ifn.set(toRef, to, value);
  }
  __get(path, object) {
    return path ? ifn.get(path, object) : object;
  }
  __set(path, object, value) {
    return path ? ifn.set(path, object, value) : value;
  }
}
HIP.factory.element(FnRewriteElement);
