
import { AsLitHIC, HIP, BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorContainer } from '../../../eip/element/hi-behavior-container/hi-behavior-container.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import './hi-note.js';
import './hi-bar.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export const styles = css`
  :host {
    display: block;
    background-color: var(--hi-clr--white);
    min-height: 100%;
    max-height: 100%;
    height: inherit;
  }

  #container {
    height: 100%;
    display: flex;
    flex-direction: column;
  }

  #container ::slotted(hi-bar) {
    border-top-width: 0;
  }

  :host(:not([aesthetic='invisible'])) #container ::slotted(hi-bar:first-of-type) {
    border-top-width: var(--hi-px-1);
  }

  :host([cap-height]) {
    min-height: auto;
  }

  :host([aesthetic='invisible']) {
    border-width: 0;
  }

  :host([aesthetic='invisible']) #container ::slotted(hi-note) {
    background-color: var(--hi-clr--transparent);
    border-width: 0;
  }

  :host([aesthetic='invisible']) #container ::slotted(hi-bar) {
    border-top-width: 0;
    border-left-width: 0;
    border-right-width: 0;
  }

  :host([aesthetic='invisible']) #container ::slotted(hi-bar:not([opened])) {
    border-bottom-color: var(--hi-clr--transparent);
  }
`;
export class AccordionElement extends BehaviorContainer(BehaviorElement(AsLitHIC(LitElement))) {
  constructor() {
    super(...arguments);
    this.fitHeight = false;
    this.mode = "allow-any-opened";
  }
  static get is() {
    return "hi-accordion";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      mode: {
        type: String
      },
      selectedIndex: {
        type: Number,
        attribute: "selected-index",
        reflect: true,
        notify: true
      },
      fitHeight: {
        type: Boolean,
        attribute: "fit-height"
      }
    };
  }
  static get autoBinding() {
    return {
      signal: "selected-index-changed",
      field: "selectedIndex"
    };
  }
  render() {
    return html`<div id="container">
      <slot id="items"></slot>
    </div>`;
  }
  connectedCallback() {
    super.connectedCallback();
    this.addEventListener("__bar-opened-changed", this.__onBarOpenedChanged);
  }
  update(props) {
    super.update(props);
    if (props.has("fitHeight")) {
      this.__applyFitHeight();
    }
    if (props.has("selectedIndex")) {
      this.__applySelectedIndex();
    }
  }
  __applyFitHeight() {
    const fitHeight = this.fitHeight;
    this.noteList().forEach((el) => {
      el.fitHeight = fitHeight;
    });
  }
  __applySelectedIndex() {
    if (f1.is.empty(this.selectedIndex)) {
      return;
    }
    const barList = this.barList();
    const barEl = barList[this.selectedIndex];
    if (barEl) {
      barEl.open();
    }
  }
  _onSlotChanged() {
    this.refresh();
    this.__applySelectedIndex();
    this.barList().forEach((bar) => {
      const note = bar.nextElementSibling;
      const id = f1.id();
      const noteId = `hi-note-${id}`;
      const barId = `hi-bar-${id}`;
      if (bar && note) {
        bar.setAttribute("aria-controls", noteId);
        bar.setAttribute("id", barId);
        note.setAttribute("aria-labelledby", barId);
        note.setAttribute("id", noteId);
      }
    });
  }
  __onBarOpenedChanged(e) {
    e.stopPropagation();
    if (this.mode === "allow-any-opened") {
      return;
    }
    const selectedNode = e.detail.node;
    this.barList().forEach((el, index) => {
      if (selectedNode !== el) {
        el.close();
      } else {
        this.selectedIndex = index;
      }
    });
  }
  noteList() {
    return this.subList("hi-note");
  }
  barList() {
    return this.subList("hi-bar");
  }
  subList(type) {
    return this.slotChildren.filter((el) => dom.is.tag(type, el));
  }
}
export default HIP.factory.element(AccordionElement);
