
import { beforeNextRender } from '../../../hip/library/polymer/polymer.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import {
  HIP,
  AsLitHIC,
  BehaviorA11yKeyboard,
  BehaviorBrowserEvents,
  BehaviorElement,
  BehaviorInteractiveElement
} from '../../../eip/element/hi-component/hi-component.js';
import '../../../eip/element/hi-icon/hi-icon.js';
export const styles = css`
  :host {
    display: flex;
    cursor: default;
    justify-content: start;
    align-content: center;
    font-weight: var(--hi-font--weight-light);
    padding: var(--hi-px-16);
    letter-spacing: calc(var(--hi-px-1) / 2);
    font-size: var(--hi-font-size--text);
    line-height: var(--hi-font-lh--large);
    font-weight: var(--hi-font--weight-light);
    border: var(--hi-px-1) solid var(--hi-clr--gray-6);
    background-color: var(--hi-clr--gray-1);
  }

  :host(:focus) {
    z-index: 1;
  }

  :host([aesthetic='center']) {
    justify-content: center;
  }

  :host([opened]) {
    z-index: 1;
    font-weight: var(--hi-font--weight-bold);
    color: var(--hi-clr--theme-5);
    background-color: var(--hi-clr--white);
    border-bottom: var(--hi-px-3) solid var(--hi-clr--theme-3);
  }

  #container {
    /* height of the icon */
    line-height: calc(var(--hi-px-16) + var(--hi-px-2));
    font-size: var(--hi-font-size--text);
    margin-left: var(--hi-px-8);
    text-overflow: ellipsis;
    white-space: nowrap;
    overflow: hidden;
  }

  @media (hover: hover) {
    :host(:hover) {
      color: var(--hi-clr--text);
      background-color: var(--hi-clr--gray-2);
      text-shadow: var(--hi-clr--text-shadow-1);
    }

    :host([opened]:hover) {
      background-color: var(--hi-clr--gray-3);
      text-shadow: none;
    }

    :host([opened]:hover) #icon {
      fill: var(--hi-clr--theme-5);
    }
  }

  :host([disabled]),
  :host([disabled]) #container {
    opacity: 0.8;
    color: var(--hi-clr--disabled-text);
  }
`;
export class BarElement extends BehaviorA11yKeyboard(
  BehaviorBrowserEvents(BehaviorInteractiveElement(BehaviorElement(AsLitHIC(LitElement))))
) {
  constructor() {
    super(...arguments);
    this.opened = false;
    this.showText = "Show Content";
    this.hideText = "Hide Content";
  }
  static get is() {
    return "hi-bar";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      index: {
        type: Number
      },
      opened: {
        type: Boolean,
        reflect: true,
        notify: true
      },
      aesthetic: {
        type: String,
        reflect: true
      },
      showText: {
        type: String,
        attribute: "show-text"
      },
      hideText: {
        type: String,
        attribute: "hide-text"
      }
    };
  }
  static get api() {
    return ["moveDown", "moveUp", "moveTo", "toggle", "close", "open"];
  }
  static get keyBindings() {
    return {
      "down:keydown": "_onKeyAny__preventDefault",
      "up:keydown": "_onKeyAny__preventDefault",
      "space:keydown": "_onKeyAny__preventDefault",
      "enter:keydown": "_onKeyAny__preventDefault",
      "enter:keyup": "__onKeyUp_enter",
      "space:keyup": "__onKeyUp_space",
      "right:keyup": "__onKeyUp_right",
      "left:keyup": "__onKeyUp_left",
      "down:keyup": "__onKeyUp_down",
      "up:keyup": "__onKeyUp_up"
    };
  }
  static get browserBindings() {
    return {
      click: "__onClick"
    };
  }
  static get autoBinding() {
    return {
      signal: "opened-changed",
      field: "opened"
    };
  }
  render() {
    const iconRotation = this.opened ? 0 : 180;
    const textContent = this.__updateTextCompute();
    return html`<div id="icon-wrapper"></div>
      <hi-icon id="icon" mode="presentation" kind="chevron_up" rotation=${iconRotation} theme=${this.theme}> </hi-icon>
      <div id="container">
        <slot id="items">
          <span>${textContent}</span>
        </slot>
      </div> `;
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "button");
  }
  updated(props) {
    super.updated(props);
    if (props.has("opened")) {
      this.__openedWatcher(this.opened);
    }
    if (props.has("disabled")) {
      this.__reflectA11y(this.disabled);
    }
  }
  __reflectA11y(disabled) {
    const note = this.note;
    if (note) {
      dom.flipAttr(note, "disabled", disabled, "disabled");
    }
  }
  __openedWatcher(_opened) {
    if (_opened) {
      this.__reflectOpen(_opened);
    } else {
      this.__reflectClose(_opened);
    }
  }
  __reflectClose(_opened) {
    this.setAttribute("aria-expanded", "false");
    const note = this.note;
    if (note) {
      note.opened = _opened;
    }
  }
  __reflectOpen(_opened) {
    this.setAttribute("aria-expanded", "true");
    beforeNextRender(this, function() {
      const note = this.note;
      if (note) {
        note.opened = _opened;
        this._dispatchBubblingEvent("__bar-opened-changed", {
          open: _opened,
          node: this
        });
      }
    });
  }
  __onFocus() {
    this.open();
  }
  __onKeyUp_space() {
    this.toggle();
  }
  __onKeyUp_enter(e) {
    e.stopPropagation();
    this.toggle();
  }
  __onKeyUp_left() {
    this.moveUp();
  }
  __onKeyUp_up() {
    this.moveUp();
  }
  __onKeyUp_right() {
    this.moveDown();
  }
  __onKeyUp_down() {
    this.moveDown();
  }
  __onClick() {
    this.toggle();
  }
  __getParentChildren() {
    return (this.parentElement || {}).slotChildren || [];
  }
  __updateTextCompute() {
    return this.opened ? this.hideText : this.showText;
  }
  get note() {
    return this.nextElementSibling;
  }
  moveUp() {
    return this.moveTo("up");
  }
  moveDown() {
    return this.moveTo("down");
  }
  moveTo(dir) {
    const childrenEls = this.__getParentChildren();
    if (childrenEls.length <= 0) {
      return;
    }
    this.index = childrenEls.indexOf(this);
    const isUp = dir === "up";
    const cycleIndex = isUp ? childrenEls.length - 2 : 0;
    const moveToIndex = this.index + (isUp ? -2 : 2);
    const hasBarBelow = isUp ? moveToIndex >= 0 : moveToIndex < childrenEls.length;
    const nextIndex = hasBarBelow ? moveToIndex : cycleIndex;
    childrenEls[nextIndex].focus();
  }
  open() {
    this.opened = true;
  }
  close() {
    this.opened = false;
  }
  toggle() {
    this.opened = !this.opened;
  }
}
export default HIP.factory.element(BarElement);
