
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { BehaviorScrollable } from '../../../eip/element/hi-behavior-scrollable/hi-behavior-scrollable.js';
import { HIP, BehaviorElement, AsLitHIC } from '../../../eip/element/hi-component/hi-component.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
export const styles = css`
  :host {
    flex: 1;
    display: block;
    overflow-y: auto;
    font-size: var(--hi-font-size--text);
    line-height: var(--hi-font-lh--text);
    color: var(--hi-clr--text);
    background-color: var(--hi-clr--white);
    border: var(--hi-px-1) solid var(--hi-clr--gray-6);
    border-top: 0;
    padding: var(--hi-px-16);
  }

  :host([allow-overflow]) {
    overflow-y: initial;
  }

  :host([fit-height]) {
    max-height: initial;
    min-height: 100%;
  }

  :host([scrollable]) #container:focus-within {
    outline: var(--hi-outline--width) var(--hi-outline--line) var(--hi-clr--outline);
    outline-offset: -1px;
  }
`;
export class NoteElement extends BehaviorScrollable(BehaviorElement(AsLitHIC(LitElement))) {
  constructor() {
    super(...arguments);
    this.opened = false;
    this.fitHeight = false;
  }
  static get is() {
    return "hi-note";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      opened: {
        type: Boolean,
        reflect: true,
        notify: true
      },
      fitHeight: {
        type: Boolean,
        attribute: "fit-height"
      }
    };
  }
  static get autoBinding() {
    return {
      signal: "opened-changed",
      field: "opened"
    };
  }
  render() {
    return html`<div id="container" mixin="text-natural">
      <slot></slot>
    </div>`;
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "region");
    this._assignScrollable(this);
    this.__openedChanged(false);
  }
  update(props) {
    super.update(props);
    if (props.has("opened")) {
      this.__openedChanged(this.opened);
    }
  }
  __openedChanged(_opened) {
    const closed = !_opened;
    dom.flipAttr(this, "hidden", closed, "hidden");
    this._updateScrollable();
  }
  get bar() {
    return this.previousElementSibling;
  }
}
export default HIP.factory.element(NoteElement);
