
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { ifDefined as ifDef } from '../../../hip/library/lit-html/directives/if-defined.js';
import { styleMap } from '../../../hip/library/lit-html/directives/style-map.js';
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import { HIP, AsLitHIC } from '../../../eip/element/hi-component/hi-component.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export const styles = css`
  :host {
    display: inline-flex;
    align-content: space-around;
    flex-direction: column;
    width: var(--hi-avatar--size);
    border-radius: var(--hi-avatar--size);
  }

  :host([size]) {
    font-size: 1rem !important;
  }

  :host([size='small']) {
    --hi-avatar--size: 2em;
    --hi-avatar--font-size: 0.75rem;
  }

  :host,
  :host([size='medium']) {
    --hi-avatar--size: 3em;
    --hi-avatar--font-size: 1em;
  }

  :host([size='large']) {
    --hi-avatar--size: 4em;
    --hi-avatar--font-size: 1.25rem;
  }

  :host([size='large-x']) {
    --hi-avatar--size: 5em;
    --hi-avatar--font-size: 1.5rem;
  }

  :host([size='large-xx']) {
    --hi-avatar--size: 6em;
    --hi-avatar--font-size: 1.75rem;
  }

  :host([size='large-xxx']) {
    --hi-avatar--size: 8em;
    --hi-avatar--font-size: 2.25rem;
  }

  :host([layout-inline]) {
    flex-direction: row;
    width: auto;
  }

  #backdrop {
    position: relative;
    box-sizing: border-box;
    height: var(--hi-avatar--size);
    width: var(--hi-avatar--size);
    border: var(--hi-px-4) solid var(--hi-clr--white);
    background-color: var(--hi-clr--theme-1);
    border-radius: var(--hi-avatar--size);
  }

  :host([interactive]) #backdrop:hover {
    filter: brightness(0.97);
  }

  :host([image]) #backdrop {
    background-size: cover;
    box-shadow: inset 0 0 0 var(--hi-px-1) rgba(0, 0, 0, 0.2);
    filter: saturate(1);
  }

  :host([image][allow-border-color]) #backdrop {
    box-shadow: none;
  }

  :host([image][interactive]) #backdrop:hover {
    filter: brightness(0.85) saturate(1);
  }

  :host([kind='square']) #backdrop {
    border-radius: var(--hi-shape--radius);
  }

  :host([theme='gray']) #backdrop,
  :host([theme='secondary']) #backdrop {
    background-color: var(--hi-clr--theme-1);
  }

  :host([allow-border-color]) #backdrop {
    border-color: var(--hi-clr--theme-3);
  }

  #backdrop:after {
    content: attr(acronym);
    position: absolute;
    top: 50%;
    left: 50%;
    transform: translate(-50%, -50%);
    font-weight: var(--hi-font--weight-bold);
    font-size: var(--hi-avatar--font-size);
    color: var(--hi-clr--theme-5);
    text-align: center;
  }

  #backdrop:hover #backdrop:after {
    filter: brightness(1.1);
  }

  :host([image]:not([image=''])) #backdrop:after {
    display: none;
  }

  #backdrop ::slotted([slot='badge']) {
    /* (√2h -h)/2 - h/2 = 0.20710678118654757h : h is height of circle*/
    --hi-avatar-badge--distance: calc((0.20710678118654757 * var(--hi-avatar--size)) - var(--hi-px-2));
    position: absolute;
    z-index: 1;
    box-shadow: 0 0 0 var(--hi-px-2) var(--hi-clr--white);
    border-radius: var(--hi-shape--rounded);
  }

  #backdrop ::slotted(hi-badge[kind='indicator']) {
    height: var(--hi-px-12) !important;
    width: var(--hi-px-12) !important;
  }

  #backdrop ::slotted([slot='badge'][dock='top-right']),
  #backdrop ::slotted([slot='badge']:not([dock])) {
    top: var(--hi-avatar-badge--distance);
    right: var(--hi-avatar-badge--distance);
    transform: translate(50%, -50%) scale(1.0001);
  }

  #backdrop ::slotted([slot='badge'][dock='bottom-right']) {
    bottom: var(--hi-avatar-badge--distance);
    right: var(--hi-avatar-badge--distance);
    transform: translate(50%, 50%) scale(1.0001);
  }

  #backdrop ::slotted([slot='badge'][dock='bottom-left']) {
    bottom: var(--hi-avatar-badge--distance);
    left: var(--hi-avatar-badge--distance);
    transform: translate(-50%, 50%) scale(1.0001);
  }

  #backdrop ::slotted([slot='badge'][dock='top-left']) {
    top: var(--hi-avatar-badge--distance);
    left: var(--hi-avatar-badge--distance);
    transform: translate(-50%, -50%) scale(1.0001);
  }

  #fullName {
    align-self: center;
    text-align: center;
    font-size: var(--hi-font-size--text);
    font-weight: var(--hi-font--weight-heavy);
    margin-top: var(--hi-px-2);
    color: var(--hi-clr--theme-5);
  }

  :host(:not([full-name])) #fullName,
  :host([aesthetic='condensed']) #fullName {
    display: none;
  }

  :host([layout-inline]) #fullName {
    padding-left: var(--hi-px-8);
  }

  :host([full-color]) #backdrop {
    background-color: var(--hi-clr--theme-3);
  }

  :host([full-color]) #backdrop:after {
    color: var(--hi-clr--theme-text-3);
  }
`;
export class AvatarElement extends BehaviorElement(AsLitHIC(LitElement)) {
  constructor() {
    super(...arguments);
    this.kind = "circle";
  }
  static get is() {
    return "hi-avatar";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      kind: {
        type: String
      },
      theme: {
        type: String,
        reflect: true
      },
      fullName: {
        type: String,
        attribute: "full-name"
      },
      initials: {
        type: String
      },
      acronym: {
        type: String
      },
      allowBorderColor: {
        type: Boolean,
        attribute: "allow-border-color",
        reflect: true
      },
      interactive: {
        type: Boolean,
        reflect: true
      },
      image: {
        type: String
      },
      aesthetic: {
        type: String,
        reflect: true
      }
    };
  }
  render() {
    const image = this.image;
    const backdropStyle = {
      "background-image": image ? `url("${image}")` : "none"
    };
    return html`
      <div id="backdrop" acronym=${ifDef(this.acronym)} style=${styleMap(backdropStyle)}>
        <slot name="badge"> </slot>
      </div>
      <p id="fullName">${this.fullName}</p>
    `;
  }
  static get autoBinding() {
    return {
      signal: "render-ended",
      field: "fullName"
    };
  }
  static get __themeList() {
    return ["red", "blue", "gray-dark", "orange", "green", "cyan", "purple", "magenta", "teal", "yellow"];
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "img");
  }
  update(props) {
    super.update(props);
    if (props.has("initials") || props.has("fullName")) {
      this.acronym = this.computeAcronym();
    }
    if (props.has("acronym") && !this.hasAttribute("theme")) {
      this.theme = this.computeTheme();
    }
  }
  updated(props) {
    super.updated(props);
    if (props.has("fullName") || props.has("initials") || props.has("image")) {
      this.__computeAriaMarkers();
      this.__dispatchRenderEvent();
    }
  }
  __computeAriaMarkers() {
    if (this.hasAttribute("aria-label")) {
      return;
    }
    this.setAttribute("aria-label", `Profile graphic for ${this.computeName()}`);
  }
  __dispatchRenderEvent() {
    this._dispatchProcess("render-ended", {
      fullName: this.fullName,
      initials: this.initials,
      image: this.image
    });
  }
  computeName() {
    const anonymous = "Anonymous";
    return this.image ? anonymous : this.fullName ? this.fullName : this.initials ? this.initials : anonymous;
  }
  computeTheme() {
    const themes = AvatarElement.__themeList;
    const themeCount = themes.length;
    const acronym = this.acronym;
    const value = f1.reduce((sum, letter) => sum + letter.charCodeAt(0), 0, f1.split("", acronym));
    const clampedValue = f1.clamp(0, themeCount, value % themeCount + 1);
    return themes[clampedValue];
  }
  computeAcronym() {
    const initials = this.initials;
    return f1.is.empty(this.initials) ? this.__computeAcronym((this.fullName || "").split(" ")) : this.__computeAcronym((initials || "").split(""));
  }
  __computeAcronym(word) {
    return word.map((_) => (_[0] || "").toUpperCase()).join("").trim();
  }
}
export default HIP.factory.element(AvatarElement);
