
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorBrowserEvents, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
export class BackdropElement extends BehaviorBrowserEvents(BehaviorElement(AsHIC(PolymerElement))) {
  static get is() {
    return "hi-backdrop";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          top: 0;
          left: 0;
          cursor: default;
          position: absolute;
          background: var(--hi-clr--blue-1);
          transition: opacity 150ms linear 0ms, visibility 150ms linear 0ms;
          z-index: calc(var(--hi-depth--modal) - 10);
          -webkit-tap-highlight-color: var(--hi-clr--transparent);
        }

        :host(:not([opened])) {
          visibility: hidden;
          top: -1000px;
          left: -1000px;
          width: 0;
          height: 0;
          opacity: 0;
        }

        :host([opened]) {
          visibility: visible;
          top: 0;
          left: 0;
          opacity: 0.5;
          /* position absolute does the right thing to keep this under dialog and over the content, but we need to make it really long because it is docked to the top of the screen */
          height: 10000vh;
          width: 100vw;
        }
      </style>
    `;
  }
  static get properties() {
    return {
      opened: {
        type: Boolean,
        reflectToAttribute: true,
        notify: true
      },
      disallowClose: {
        type: Boolean
      }
    };
  }
  static get observers() {
    return ["__openedWatcher(opened)"];
  }
  static get api() {
    return ["open", "close"];
  }
  static get browserBindings() {
    return {
      click: "__onClick"
    };
  }
  static get autoBinding() {
    return {
      signal: "opened-changed",
      field: "opened"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "presentation");
  }
  __openedWatcher(opened) {
    if (opened) {
      return;
    }
    const paneEl = HIP.browser.queryOne("hi-menu") || HIP.browser.queryOne("hi-dialog") || HIP.browser.queryOne("hi-drawer");
    if (!paneEl) {
      return;
    }
    if (paneEl.activePane) {
      paneEl.activePane.close();
    }
  }
  __onClick() {
    if (!this.disallowClose) {
      this.opened = false;
    }
  }
  open() {
    if (!this.disabled) {
      this.opened = true;
    }
  }
  close() {
    if (!this.disabled) {
      this.opened = false;
    }
  }
  static get instance() {
    return HIP.fn.instance(BackdropElement);
  }
}
HIP.factory.element(BackdropElement);
export default "hi-backdrop";
