
import { stylesTagBadge } from '../../../eip/element/hi-behavior-style/hi-behavior-style.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { styles as stylesDock } from '../../../eip/element/hi-behavior-dock/hi-behavior-dock.js';
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { AsLitHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export const styles = css`
  :host([kind='indicator']) {
    color: var(--hi-clr--theme-text-3);
    border-color: var(--hi-clr--theme-5);
    background-color: var(--hi-clr--theme-3);
    border-radius: var(--hi-shape--rounded);
    min-height: var(--hi-px-8);
    min-width: var(--hi-px-8);
    height: var(--hi-px-8);
    width: var(--hi-px-8);
    vertical-align: super;
    padding: 0;
  }
`;
export class BadgeElement extends BehaviorElement(AsLitHIC(LitElement)) {
  constructor() {
    super(...arguments);
    this.max = 998;
  }
  static get is() {
    return "hi-badge";
  }
  static get styles() {
    return [stylesTheme, stylesDock, stylesTagBadge, styles];
  }
  static get properties() {
    return {
      kind: {
        type: String,
        reflect: true
      },
      aesthetic: {
        type: String,
        reflect: true
      },
      value: {
        type: String,
        reflect: true,
        notify: true
      },
      max: {
        type: Number
      }
    };
  }
  static get autoBinding() {
    return {
      signal: "value-changed",
      field: "value"
    };
  }
  render() {
    return html`
      <slot id="content">
        <span>${this.__valueDisplayCompute(this.value, this.max)}</span>
      </slot>
    `;
  }
  updated(changedProperties) {
    super.updated(changedProperties);
    if (changedProperties.has("value")) {
      this.__addAriaLabel();
    }
  }
  __addAriaLabel() {
    if (!this.hasAttribute("aria-label")) {
      this.setAttribute("aria-label", `${this.value || ""}`);
    }
  }
  __valueDisplayCompute(value, max) {
    const _value = f1.cast.number(value);
    if (f1.is.nan(_value)) {
      return value;
    }
    const _max = f1.cast.number(max);
    if (f1.is.nan(_max)) {
      return _value;
    }
    return _value <= _max ? _value : `${_max}+`;
  }
}
export default HIP.factory.element(BadgeElement);
