
import { Debouncer } from '../../../hip/library/polymer/polymer.js';
import { animationFrame } from '../../../hip/library/polymer/polymer.js';
import { dedupingMixin } from '../../../hip/library/polymer/polymer.js';
import { HIP, BehaviorElement, a11yLabel } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorBehavior } from '../../../eip/element/hi-behavior-behavior/hi-behavior-behavior.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
export function BaseBehaviorAutoComplete(Base) {
  class BehaviorAutoComplete2 extends BehaviorBehavior(BehaviorElement(Base)) {
    constructor() {
      super(...arguments);
      this.input = [];
      this.maxResults = 500;
      this._listResults = [];
      this.__simulatedInput = false;
      this._searchActionMutex = -1;
    }
    static get properties() {
      return {
        kind: {
          type: String,
          value: "static"
        },
        multiple: {
          type: Boolean
        },
        opened: {
          type: Boolean,
          notify: true,
          reflectToAttribute: true
        },
        placeholder: {
          type: String
        },
        value: {
          type: String,
          reflectToAttribute: true,
          notify: true
        },
        input: {
          type: Array
        },
        maxResults: {
          type: Number,
          value: 500
        },
        ariaLabel: {
          type: String
        },
        ariaLabelledby: {
          type: String
        },
        ariaDescribedby: {
          type: String
        }
      };
    }
    static get api() {
      return ["toggle", "close", "open"];
    }
    static get autoBinding() {
      return {
        signal: "value-changed",
        field: "value"
      };
    }
    static get observers() {
      return [
        "__a11yLabelWatcher(ariaLabel, ariaLabelledby, ariaDescribedby)",
        "__openedWatcher(opened)",
        "_inputWatcher(input)"
      ];
    }
    connectedCallback() {
      super.connectedCallback();
      this._registerBehavior("BehaviorAutoComplete");
      dom.setAttrs(this, {
        focusable: "true",
        role: this.readonly ? null : "group"
      });
      this._initializeAuto__blur();
      this._initializeAuto__input();
      this._initializeAuto__dropdown();
    }
    _initializeAuto__blur() {
      this.$.searchInput.addEventListener("blur", () => {
        if (!this["multiple"] && this.value) {
          this._assignSearchInput();
        }
      });
      this.addEventListener("blur", (evt) => {
        const detail = evt.detail;
        if (detail instanceof FocusEvent && detail.relatedTarget) {
          this.close();
        }
      });
    }
    _initializeAuto__input() {
      this.$.searchInput.addEventListener("input", () => {
        this._searchActionMutex = this.peekSearchInput().length;
        this.__searchValueChangedDebouncer = Debouncer.debounce(
          this.__searchValueChangedDebouncer,
          this._generateTimeout(),
          () => {
            this.__onValueChanged__input();
          }
        );
      });
      this.$.searchInput.addEventListener("keyup", (evt) => {
        const keyCode = evt.key || evt.code;
        switch (keyCode) {
          case "ArrowUp":
            evt.preventDefault();
            this.__onKeyUp_upKey__input();
            break;
          case "ArrowDown":
            evt.preventDefault();
            this.__onKeyUp_downKey__input();
            break;
          case "Enter":
            this._onKeyUp_enterKey__input();
            break;
          case "Backspace":
            this._onKeyUp_backspaceKey__input();
            break;
          case "Escape":
            this._onKeyUp_escKey__input();
            break;
        }
      });
      this.$.searchInput.addEventListener("keydown", (event) => {
        const keyCode = event.key || event.code;
        const isUpOrDown = keyCode === "ArrowUp" || keyCode === "ArrowDown";
        if (isUpOrDown) {
          event.preventDefault();
        }
      });
    }
    _initializeAuto__dropdown() {
      this.$.dropdown.addEventListener("opened-changed", this.__onOpenChanged__dropdown.bind(this));
      this.$.dropdown.addEventListener("click-ended", this.__onClickEnded__dropdown.bind(this));
    }
    __openedWatcher(opened) {
      this.$.searchInput.setAttribute("aria-expanded", `${!!opened}`);
      this.resetFocusToFirst();
    }
    _inputWatcher(input) {
      this._computeResultsList(input, this.peekSearchInput());
    }
    __a11yLabelWatcher(label, labelledBy, describedBy) {
      a11yLabel.writeA11yLabel(this.$.searchInput, label, labelledBy, describedBy, { ns: this.ns });
    }
    __onKeyUp_downKey__input() {
      if (this.opened) {
        this.$.popupElement.scrollToItem(this.$.popupElement.shiftFocusDown());
      } else {
        this.open();
      }
    }
    __onKeyUp_upKey__input() {
      if (this.opened) {
        this.$.popupElement.scrollToItem(this.$.popupElement.shiftFocusUp());
      } else {
        this.open();
      }
    }
    _onKeyUp_enterKey__input() {
      if (this.opened) {
        this.$.popupElement._setFocus__onKeyUp_enter();
      }
    }
    _onKeyUp_escKey__input() {
    }
    _onKeyUp_backspaceKey__input() {
    }
    _onFocusIndexEnded__popupElement(e) {
      const selectedOptionEl = e.detail.value;
      const hasActiveDescendant = !!selectedOptionEl;
      const activeDescendantId = hasActiveDescendant ? selectedOptionEl.getAttribute("id") : "";
      dom.flipAttr(this.$.searchInput, "aria-activedescendant", hasActiveDescendant, activeDescendantId);
    }
    __onOpenChanged__dropdown(e) {
      this.opened = e.detail.value;
    }
    __onClickEnded__dropdown(e) {
      const mousePositionX = f1.read("/detail/args/0/offsetX", -1, e);
      const fullWidth = e.target["scrollWidth"];
      const widthOfIconInAnchor = 30;
      const isMouseNearRight = fullWidth - mousePositionX <= widthOfIconInAnchor;
      const allowMultiSelect = this["multiple"] && this.opened;
      if (isMouseNearRight || !allowMultiSelect) {
        this.toggle();
      }
      if (this.opened) {
        this.$.searchInput.focus();
      }
    }
    _reflectSearchInput() {
      this._clearSearchInput();
    }
    _assignSearchInput() {
      const title = f1.read("/selections/0/title", "", this);
      this.assignInput(title);
    }
    _clearSearchInput() {
      this.__clearSearchInputDebouncer = Debouncer.debounce(
        this.__clearSearchInputDebouncer,
        this._generateTimeout(),
        () => {
          this.clearInput();
        }
      );
    }
    __onValueChanged__input() {
      const searchString = this.peekSearchInput();
      this._searchActionMutex = searchString.length;
      this._computeResultsList(this.input, searchString);
      this.$.clear.hidden = f1.is.empty(searchString);
      if (this.__simulatedInput) {
        this.__simulatedInput = false;
        return;
      }
      this._dispatchEvent("search-value-changed", {
        value: searchString
      });
      if (!this.opened && !f1.is.empty(searchString)) {
        this.open();
      }
    }
    _getNormalizedInput(input) {
      return f1.compact(f1.to.array(f1.read("/properties", input, input)));
    }
    _hasEmptyInput(input = []) {
      return f1.is.empty(input);
    }
    _computeResultsList(_input, searchString) {
      const sourcedInput = this.__sortByOrder(this._getNormalizedInput(_input));
      const listResults = this._shouldPassThruResultList(searchString) ? sourcedInput : sourcedInput.filter(
        (op) => op.heading === true || this.__toComparisonFormat(op.title).includes(searchString)
      );
      const filteredCount = listResults.filter((op) => !op.heading).length;
      const totalCount = f1.read(
        "/metadata/count",
        f1.safe(0, sourcedInput.filter((op) => !op.heading).length),
        _input
      );
      this.__updateResultsHeaderText(searchString, filteredCount, totalCount);
      this.resetFocusToFirst();
      return this._listResults = listResults.slice(0, this.maxResults);
    }
    _shouldPassThruResultList(searchString) {
      return this.kind === "dynamic" || searchString === "";
    }
    __updateResultsHeaderText(searchString, matchedResults, totalCount) {
      const hasSearch = !!searchString;
      const totalCountSafe = f1.safe(0, totalCount);
      this.$.resultMessage.textContent = hasSearch ? totalCountSafe >= 0 ? `Showing ${matchedResults} matched option(s)` : `Showing ${matchedResults} of ${totalCountSafe} matched option(s)` : "";
    }
    peekSearchInput() {
      return this.__toComparisonFormat(this.$.searchInput.value);
    }
    __toComparisonFormat(s) {
      return `${s || ""}`.trim().toLowerCase();
    }
    __sortByOrder(sourcedInput) {
      return f1.is.array(sourcedInput) && f1.is.in("order", f1.nth(0, sourcedInput)) ? f1.sort(
        (a, b) => a.order - b.order,
        sourcedInput
      ) : sourcedInput;
    }
    _generateTimeout() {
      return animationFrame;
    }
    type(search) {
      this.$.searchInput.value = search;
      this.$.searchInput.dispatchEvent(
        new Event("input", {
          bubbles: true,
          cancelable: true
        })
      );
    }
    clearInput() {
      const isAlreadyCleared = "" === this.$.searchInput.value;
      this.$.searchInput.value = "";
      if (isAlreadyCleared) {
        return;
      }
      this.evaluate();
    }
    assignInput(s) {
      this.$.searchInput.value = s;
    }
    evaluate() {
      this.__simulatedInput = true;
      this.$.searchInput.dispatchEvent(new Event("input"));
    }
    resetFocusToFirst() {
      if (!this.opened) {
        return;
      }
      const popupEl = this.$.popupElement;
      if (!popupEl) {
        return;
      }
      const focusedIndex = popupEl.focusedIndex;
      if (f1.is.undefined(focusedIndex) || focusedIndex > 0) {
        popupEl.clearFocusIndex();
        popupEl.shiftFocusFirst();
      }
    }
    focus() {
      this.$.searchInput.focus();
    }
    open() {
      this.opened = true;
      this.$.dropdown.open();
    }
    close() {
      this.opened = false;
      this.$.dropdown.close();
    }
    toggle() {
      if (this.opened) {
        this.close();
      } else {
        this.$.searchInput.focus();
        this.open();
      }
    }
  }
  return HIP.factory.mixin(BehaviorAutoComplete2);
}
export const BehaviorAutoComplete = dedupingMixin(BaseBehaviorAutoComplete);
HIP["BehaviorAutoComplete"] = BehaviorAutoComplete;
