
import { Debouncer } from '../../../hip/library/polymer/polymer.js';
import { idlePeriod } from '../../../hip/library/polymer/polymer.js';
import { dedupingMixin } from '../../../hip/library/polymer/polymer.js';
import { beforeNextRender } from '../../../hip/library/polymer/polymer.js';
import { BehaviorContainer } from '../../../eip/element/hi-behavior-container/hi-behavior-container.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorBehavior } from '../../../eip/element/hi-behavior-behavior/hi-behavior-behavior.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import $ip from '../../../pip/library/pi-ip/pi-ip.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export function BaseBehaviorFlyout(Base) {
  class BehaviorFlyout2 extends BehaviorContainer(BehaviorBehavior(Base)) {
    constructor() {
      super(...arguments);
      this.__hasChangedValue = false;
      this._flyout__popper_modifiers_defaults = [
        {
          name: "flip",
          options: {
            flipVariations: true,
            allowedAutoPlacements: ["top", "bottom"]
          }
        },
        {
          name: "computeStyles",
          options: {
            gpuAcceleration: true
          }
        },
        {
          name: "preventOverflow",
          options: {
            padding: {
              top: 2
            },
            altBoundary: true
          }
        }
      ];
      this.childRoleAlg = f1.algorithm(
        f1.classify({
          group: (child) => !f1.is.nil(child) && "group" === child.getAttribute("role"),
          child: (child) => !f1.is.nil(child),
          fallback: () => true
        }),
        f1.strategy({
          group: () => "true",
          child: (child) => child.getAttribute("role") || "true",
          fallback: () => "true"
        })
      );
    }
    static get properties() {
      return {
        _kindFlyout: {
          type: String,
          value: "value"
        },
        opened: {
          type: Boolean,
          observer: "_flyout__openedChanged",
          reflectToAttribute: true,
          notify: true
        },
        manual: {
          type: Boolean
        },
        alignment: {
          type: String,
          reflectToAttribute: true,
          value: "left"
        },
        options: {
          type: Object,
          value: () => {
            return { modifiers: [] };
          }
        }
      };
    }
    connectedCallback() {
      super.connectedCallback();
      this._registerBehavior("BehaviorFlyout");
      this.setAttribute("touch-action", "none");
      this._window__onFocusOutBound = this._window__onFocusOut.bind(this);
    }
    disconnectedCallback() {
      super.disconnectedCallback();
      this.__removeGlobalListeners();
      this.__removePopperInstance();
    }
    __removePopperInstance() {
      if (this._flyout__popper) {
        this._flyout__popper.destroy();
        this._flyout__popper = void 0;
      }
    }
    get __alignPopper() {
      const alignmentStrategy = {
        left: "bottom-start",
        right: "bottom-end",
        fallback: this.alignment
      };
      return f1.strategy(alignmentStrategy, this.alignment);
    }
    _popperConfig() {
      return {
        placement: this.options.placement || this.__alignPopper,
        strategy: this.options.strategy || "absolute",
        modifiers: [...this._flyout__popper_modifiers_defaults, ...this.options.modifiers || []]
      };
    }
    _flyout___openedWatcher(opened) {
      if (opened) {
        this._flyout__popper = Popper.createPopper(this.$.flyoutValue, this.$.flyoutContainer, this._popperConfig());
      } else {
        this.__removePopperInstance();
      }
    }
    _flyout___onValueNotified(event) {
      super._onValueNotified(event);
      if ($ip.flags.is("/eip/hi-behavior-flyout/edify-stop-dropdown-value")) {
        if (this.__slotChildValue != null && this.__slotChildValue !== "") {
          this.value = this.__slotChildValue;
          this.__hasChangedValue = true;
        }
      } else {
        if (this.__slotChildValue != null) {
          this.value = this.__slotChildValue;
          this.__hasChangedValue = true;
        }
      }
      const sourceEl = event.srcElement;
      if (sourceEl) {
        const keepFlyoutOpened = sourceEl.hasAttribute("multiple") || sourceEl.hasAttribute("allow-key-search");
        if (keepFlyoutOpened) {
          return;
        }
      }
      beforeNextRender(this, function() {
        if (this.kind === "value" || this.kind === "menu") {
          this.close();
        }
      });
    }
    _flyout__toggleUI(event) {
      if (this.manual) {
        return;
      }
      const targeted = this.__getAppropriateTarget(event);
      const isFocusOnElement = targeted === this || targeted.parentElement === this;
      const parentEls = this._getParentsOnEvent(event);
      const isFocusOnChildElement = parentEls.find((el) => el === this);
      if (isFocusOnElement || isFocusOnChildElement) {
        this._flyout__toggle();
      }
    }
    _flyout__toggle() {
      if (!this.disabled) {
        if (!this.opened) {
          this.open();
          HIP.browser.setFocusOnFirstOfRoot(this);
        } else {
          this.close();
          this.__flyout__focusOnTarget();
        }
      }
    }
    _flyout__openedChanged(_opened) {
      if (_opened) {
        this.__addGlobalListeners();
      } else {
        this.__removeGlobalListeners();
      }
      if (this.disabled) {
        this.opened = _opened = false;
      }
      this.$.flyoutValue.setAttribute("aria-expanded", _opened + "");
      if (_opened === false) {
        this.__hasChangedValue = false;
      }
    }
    __addGlobalListeners() {
      window.addEventListener("focusout", this._window__onFocusOutBound, true);
      window.addEventListener("mouseup", this._window__onFocusOutBound, true);
    }
    __removeGlobalListeners() {
      window.removeEventListener("focusout", this._window__onFocusOutBound, true);
      window.removeEventListener("mouseup", this._window__onFocusOutBound, true);
    }
    _getParentsOfActiveEl() {
      const activeEl = dom.activeElement();
      if (!activeEl) {
        return [];
      }
      if (activeEl.hasAttribute("disallowed-shadow-active-element")) {
        return [];
      }
      return dom.parentsOf(activeEl);
    }
    _window__onFocusOut(event) {
      this._window__onFocusOutDebouncer = Debouncer.debounce(this._window__onFocusOutDebouncer, idlePeriod, () => {
        const elementFoundInEventPath = this._findElementsInEventPath(event);
        if (elementFoundInEventPath != null) {
          return;
        }
        const relatedTarget = event.relatedTarget;
        const target = event.target;
        const preParentsEl = event.path || [];
        const parentsEl = dom.parentsOf(relatedTarget || target);
        const parentsOfActiveEl = this._getParentsOfActiveEl();
        const widgetEl = [...parentsEl, ...preParentsEl, ...parentsOfActiveEl].find((_el) => _el === this);
        if (!widgetEl) {
          this.close();
        }
      });
    }
    _findElementsInEventPath(event) {
      const parentEls = [
        ...f1.safe([], event.target.children),
        ...f1.safe([], event.target.shadowRoot?.children),
        ...this.createParentsOnEvent(event.target).filter((el) => el instanceof Element)
      ];
      return parentEls.find((el) => el.isSameNode(this));
    }
    __getAppropriateTarget(event) {
      return event.detail && event.detail.keyboardEvent ? event.detail.keyboardEvent.target : event.target;
    }
    _flyout__onSlotChanged() {
      this._readSlotChildren();
      this.__reflectListboxControl();
    }
    __reflectListboxControl() {
      this.$.flyoutValue.setAttribute("aria-haspopup", this.childRoleAlg(this.firstSlotChild));
    }
    _flyout__onKeyUp_escape() {
      if (!this.manual) {
        this.close();
        this.__flyout__focusOnTarget();
      }
    }
    _flyout__onTap(event) {
      this._flyout__onKeyUp_spaceOrEnterOrTap(event);
    }
    _flyout__onKeyUp_enter(event) {
      if (this.opened) {
        this._flyout__onKeyUp_spaceOrEnterOrTap(event);
        event.detail.keyboardEvent.preventDefault();
        event.detail.keyboardEvent.stopPropagation();
      }
    }
    _flyout__onKeyUp_space(event) {
      this._flyout__onKeyUp_spaceOrEnterOrTap(event);
    }
    _flyout__onKeyUp_spaceOrEnterOrTap(event) {
      if (this.manual) {
        return;
      } else if (this.opened && this.__hasChangedValue === false) {
        this.close();
        this.__flyout__focusOnTarget();
      } else if (this.opened && this.__flyout__isKindWaitForUser()) {
        return;
      } else {
        this._flyout__toggleUI(event);
      }
    }
    __flyout__isKindWaitForUser() {
      if (this._kindFlyout === "sticky") {
        return true;
      }
      const firstElement = this.slotChildren[0];
      return this._kindFlyout === "value" && firstElement != null && firstElement.hasAttribute("multiple");
    }
    __flyout__focusOnTarget() {
      this.$.flyoutValue.focus();
    }
    open() {
      this.opened = true;
    }
    close() {
      this.opened = false;
    }
    alignFlyout() {
      if (this._flyout__popper) {
        this._flyout__popper.update();
      }
    }
  }
  return HIP.factory.mixin(BehaviorFlyout2);
}
export const BehaviorFlyout = dedupingMixin(BaseBehaviorFlyout);
HIP["BehaviorFlyout"] = BehaviorFlyout;
