
import { dedupingMixin } from '../../../hip/library/polymer/polymer.js';
import { BehaviorBehavior } from '../../../eip/element/hi-behavior-behavior/hi-behavior-behavior.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorInput } from './hi-behavior-input.js';
import { BehaviorFormat } from '../../../eip/element/hi-behavior-format/hi-behavior-format.js';
import { BehaviorParse } from '../../../eip/element/hi-behavior-parse/hi-behavior-parse.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export function BaseBehaviorInputEditable(Base) {
  class BehaviorInputEditable2 extends BehaviorParse(BehaviorFormat(BehaviorInput(BehaviorBehavior(Base)))) {
    static get properties() {
      return {
        kind: {
          type: String
        },
        placeholder: {
          type: String,
          observer: "_placeholderChanged"
        },
        _value: {
          type: Object
        },
        autocomplete: {
          type: String,
          value: "off"
        },
        maxlength: {
          type: Number,
          observer: "_maxlengthChanged"
        },
        minlength: {
          type: Number,
          value: 0
        },
        count: {
          type: Number,
          computed: "_countCompute(value)",
          reflectToAttribute: true,
          notify: true
        },
        focused: {
          type: Boolean,
          reflectToAttribute: true
        }
      };
    }
    connectedCallback() {
      super.connectedCallback();
      this._correctValue();
      this._registerBehavior("BehaviorInputEditable");
    }
    _correctValue(value = this.value) {
      this.value = HIP.fn.format(this._coerceValue(value), this.formatter, this.formatterRule);
    }
    _placeholderChanged() {
      if (this.mask) {
        this.placeholder = this.mask;
      }
    }
    _maxlengthChanged(maxlength) {
      if (this.mask) {
        this.maxlength = this.mask.length;
      }
    }
    _coerceValue(value) {
      return value;
    }
    get willValidate() {
      return this.novalidate !== true && this.$.input.willValidate;
    }
    get validity() {
      return this._computeValidity(
        f1.merge("over", this._spreadValidity(this.$.input.validity), {
          tooShort: this.__html5Validity_isTooShort(),
          tooLong: this.__html5Validity_isTooLong()
        })
      );
    }
    __html5Validity_isTooShort() {
      const len = `${this._value || ""}`.length;
      return (this.required || len > 0) && this.minlength != null && len < this.minlength;
    }
    __html5Validity_isTooLong() {
      const len = `${this._value || ""}`.length;
      return (this.required || len > 0) && this.maxlength != null && len > this.maxlength;
    }
    setCustomValidity(validationMessage) {
      if (validationMessage) {
        super.setCustomValidity(validationMessage);
        this.$.input.setCustomValidity(validationMessage);
      }
    }
    focus() {
      this.$.input.focus();
    }
    _countCompute(value) {
      return (value == null ? "" : value).length;
    }
    _inputEditable_valueChanged(value) {
      value = HIP.fn.format(this._coerceValue(value), this.formatter, this.formatterRule);
      if (this.$.input.value !== value) {
        this.$.input.value = value;
      }
      this._value = value;
      if (this.kind !== "edit") {
        this.value = HIP.fn.parse(value, this.parser, this.parserRule);
      }
    }
    _inputEditable_onInput_input(event) {
      event.preventDefault();
      this._value = this._coerceValue(event.target.value);
      if (this.kind !== "edit") {
        this.value = this._value;
        this._dispatchEvent("input", {
          value: this.value
        });
      }
    }
    _inputEditable_onKeyUp_enter(event) {
      if (this.kind === "edit") {
        this.value = this._coerceValue(this._value);
        this.__commitValid(event);
      }
    }
    _inputEditable_onChange_input(event) {
      event.preventDefault();
      if (this.kind === "edit") {
        return;
      }
      this._dispatchEvent("change", {
        value: this.value
      });
    }
    _inputEditable_onFocus_input() {
    }
    _inputEditable_onBlur_input(event) {
      this._correctValue(this._value);
      this._dispatchEvent(`blur`, event);
      this.focused = false;
      this.__commitValid(event);
    }
    __commitValid(event) {
      const relatedTarget = event.relatedTarget;
      const inputClearButtonId = "inputClear";
      if (relatedTarget && relatedTarget.id === inputClearButtonId) {
        event.stopPropagation();
        return;
      }
      if (this.validate()) {
        this.commit();
      }
    }
  }
  return HIP.factory.mixin(BehaviorInputEditable2);
}
export const BehaviorInputEditable = dedupingMixin(BaseBehaviorInputEditable);
HIP["BehaviorInputEditable"] = BehaviorInputEditable;
