
import { dedupingMixin } from '../../../hip/library/polymer/polymer.js';
import {
  HIP,
  BehaviorElement
} from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorBehavior } from '../../../eip/element/hi-behavior-behavior/hi-behavior-behavior.js';
import { BehaviorValidated } from '../../../eip/element/hi-behavior-validation/hi-behavior-validated.js';
import { ServiceNamespace } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorForm } from '../../../eip/element/hi-behavior-form/hi-behavior-form.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export function BaseBehaviorInput(Base) {
  class BehaviorInput2 extends BehaviorForm(BehaviorValidated(BehaviorBehavior(BehaviorElement(Base)))) {
    get valueEventChannel() {
      return "form";
    }
    static get properties() {
      return {
        form: {
          type: String,
          reflectToAttribute: true,
          reflect: true,
          observer: "_formChanged"
        },
        required: {
          type: Boolean,
          reflectToAttribute: true,
          reflect: true
        },
        locked: {
          type: Boolean,
          reflectToAttribute: true,
          reflect: true
        },
        excluded: {
          type: Boolean,
          reflectToAttribute: true,
          reflect: true
        },
        readonly: {
          type: Boolean,
          reflectToAttribute: true,
          reflect: true,
          observer: "__input__readonlyChanged"
        },
        silentValidate: {
          type: Boolean,
          attribute: "silent-validate"
        },
        hiddenValidate: {
          type: Boolean,
          attribute: "hidden-validate"
        },
        validationNotice: {
          type: String,
          notify: true,
          attribute: "validation-notice"
        },
        dirty: {
          type: Boolean,
          reflectToAttribute: true,
          reflect: true
        },
        disallowEnter: {
          type: Boolean,
          attribute: "disallow-enter"
        },
        name: {
          type: String,
          reflectToAttribute: true,
          reflect: true
        },
        modelPath: {
          type: String,
          attribute: "model-path"
        },
        nullable: {
          type: Boolean,
          reflectToAttribute: true
        },
        blankable: {
          type: Boolean,
          reflectToAttribute: true
        }
      };
    }
    static get api() {
      return ["clear", "reset", "commit"];
    }
    static get observers() {
      return ["__invalidWatcher(invalid, dirty)", "__formNameWatcher(form, name)", "__formValueWatcher(value)"];
    }
    connectedCallback() {
      super.connectedCallback();
      this._registerBehavior("BehaviorInput");
      this.__resetForm(this.form, this.form);
    }
    disconnectedCallback() {
      super.disconnectedCallback();
      this.__removeForm(this.form);
    }
    update(props) {
      super.update(props);
      this._watchChanges(props, [
        {
          properties: ["invalid", "dirty"],
          method: "__invalidWatcher"
        },
        {
          properties: ["form", "name"],
          method: "__formNameWatcher"
        },
        {
          properties: ["value"],
          method: "__formValueWatcher"
        }
      ]);
    }
    __input__readonlyChanged(_readonly) {
      dom.flipAttr(this, `aria-readonly`, !!_readonly, `true`);
    }
    _nsChanged(ns, nsOld) {
      super._nsChanged(ns, nsOld);
      const hasInvalidOldNS = f1.is.nil(nsOld);
      if (hasInvalidOldNS) {
        nsOld = ServiceNamespace["defaultNS"];
      }
      this.__removeForm(this.form, nsOld);
      this.__addForm(this.form, ns);
    }
    _formChanged(_form, _formOld) {
      this.__resetForm(_form, _formOld);
    }
    __resetForm(_form, _formOld) {
      if (_formOld) {
        this.__removeForm(_formOld);
      }
      if (_form) {
        this.__addForm(_form);
      }
    }
    __removeForm(_formOld, ns) {
      if (_formOld) {
        HIP.db.form.removeKey(_formOld, this, ns);
      }
    }
    __addForm(_form, ns) {
      if (_form) {
        HIP.db.form.addKey(_form, this, ns);
      }
    }
    __invalidWatcher(invalid, dirty) {
      if (dirty) {
        this.removeAttribute("aria-invalid");
      } else {
        dom.flipAttr(this, `aria-invalid`, invalid, `true`);
      }
    }
    __formNameWatcher(_form, _name) {
      if (_form && _name) {
        this.setAttribute("data-qa", `${_form}.${_name}`);
      }
    }
    __formValueWatcher(value) {
      if (value === "" && this.nullable && !this.blankable) {
        this.value = null;
      }
    }
    clear() {
      this.reset();
    }
    reset() {
      this.resetCustomValidity();
      this.removeAttribute("validated");
      this.dirty = false;
      if (this.values) {
        this.values = [];
      }
      this.value = f1.is.array(this.value) ? [] : null;
      this.commit();
    }
    validate(forceValidStatus) {
      if (!this.willValidate) {
        return true;
      }
      const isValid = f1.is.boolean(forceValidStatus) ? forceValidStatus : this.checkValidity();
      this._applyValidity(isValid);
      this.validationNotice = !this.hiddenValidate && !isValid ? this.validationMessage : "";
      this._dispatchNotification("validate", {
        value: isValid
      });
      return this.toggleValidation(isValid);
    }
    toggleValidation(isValid) {
      if (isValid) {
        this.hideValidation();
      } else {
        this.showValidation();
      }
      return isValid;
    }
    hideValidation() {
      if (this.__validationNoticeEl) {
        this.__validationNoticeEl.setAttribute("hidden", "hidden");
        this.__validationNoticeEl.validationNotice = "";
      }
    }
    showValidation() {
      if (this.__validationNoticeEl) {
        if ((this.validationNotice || "").length > 0) {
          this.__validationNoticeEl.removeAttribute("hidden");
          this.__validationNoticeEl.trigger(this.validationNotice);
        }
      }
    }
    _internalInput() {
      const hasInternalInput = this.$ && this.$.input;
      if (!hasInternalInput) {
        return;
      }
      return this.$.input;
    }
    commit(value = this.value) {
      return {
        value
      };
    }
  }
  return HIP.factory.mixin(BehaviorInput2);
}
export const BehaviorInput = dedupingMixin(BaseBehaviorInput);
HIP["BehaviorInput"] = BehaviorInput;
