
import '../../../hip/library/polymer/polymer.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
export class InputService {
  tetherInputSet(set = {}, parentEl) {
    const novalidate = parentEl.novalidate;
    const ns = parentEl.ns;
    f1.each(
      (el) => {
        if (novalidate != null) {
          el.novalidate = novalidate;
        }
        if (ns != null) {
          el.ns = ns;
        }
      },
      f1.filter((o) => !!o, set)
    );
    if (set.label) {
      this.autoAssignGid(set.label);
    }
    if (set.input) {
      this.autoAssignGid(set.input);
      this.tetherInput(set.input, parentEl);
    }
    if (set.input && !set.validation) {
      set.validation = this.addValidationNotice(set.input, parentEl);
    }
    if (set.input && set.label) {
      this.tetherLabelToInput(set.input, set.label);
    }
  }
  initializeInputs(el) {
    this.tetherInputs(el);
    this.tetherLabels(el);
    this.applyValidation(el);
    this.addValidationNotices(el);
  }
  tetherLabels(parent) {
    let labelEl;
    let inputEl;
    parent.slotChildren.forEach((el) => {
      if (this.isLabel(el)) {
        labelEl = el;
      } else if (this.isInput(el)) {
        inputEl = el;
      }
      if (inputEl && labelEl) {
        this.autoAssignGid(inputEl);
        this.autoAssignGid(labelEl);
        this.tetherLabelToInput(inputEl, labelEl);
        labelEl = null;
        inputEl = null;
      }
    });
  }
  autoAssignGid(el) {
    if (!el) {
      return;
    }
    el = dom.ensureAttr(el, "id");
    el = dom.setAttr(el, "gid", el.id);
    return el;
  }
  tetherLabelToInput(inputEl, labelEl) {
    labelEl["inputEl"] = inputEl;
    if (!labelEl.hasAttribute("for")) {
      dom.setAttr(labelEl, "for", inputEl.id);
    }
  }
  isLabel(el) {
    return el.localName === "hi-label" || el.localName === "label";
  }
  isInput(el) {
    return HIP.util.doesExtend(el, "BehaviorForm");
  }
  isFormElement(el) {
    return HIP.util.doesExtend(el, "BehaviorForm") || HIP.util.doesExtend(el, "BehaviorInputContainer");
  }
  isValidationNotice(el) {
    return el.localName === "hi-validation-notice";
  }
  tetherInputs(parentEl) {
    parentEl.slotChildren.filter(HIP.util.isCustomElement).filter((el) => !el.hasAttribute("form") && this.isFormElement(el)).forEach((el) => this.tetherInput(el, parentEl));
  }
  tetherInput(el, parentEl) {
    const form = parentEl.form || parentEl.getAttribute("form");
    if (form) {
      el.setAttribute("form", form);
    }
  }
  applyValidation(formEl, novalidate = formEl.novalidate) {
    f1.each((el) => {
      el.novalidate = novalidate;
    }, f1.filter(this.isInput, formEl.slotChildren));
  }
  addValidationNotices(formEl) {
    f1.each((el) => {
      this.addValidationNotice(el, formEl);
    }, f1.filter(this.isInput, formEl.slotChildren));
  }
  addValidationNotice(el, formEl) {
    const hasValidationAssociation = el.__validationNoticeEl;
    const hasNoFormAssociation = !el.form;
    if (hasNoFormAssociation || hasValidationAssociation) {
      return;
    }
    const validationNoticeEl = document.createElement("hi-validation-notice");
    validationNoticeEl.for = el.id;
    formEl.insertBefore(validationNoticeEl, el.nextSibling);
    el.__validationNoticeEl = validationNoticeEl;
    return validationNoticeEl;
  }
}
export const inputService = HIP["input"] = HIP["input"] || new InputService();
