
import { Debouncer } from '../../../hip/library/polymer/polymer.js';
import { microTask, idlePeriod } from '../../../hip/library/polymer/polymer.js';
import { afterNextRender } from '../../../hip/library/polymer/polymer.js';
import { dedupingMixin } from '../../../hip/library/polymer/polymer.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorBehavior } from '../../../eip/element/hi-behavior-behavior/hi-behavior-behavior.js';
import '../../../eip/element/hi-backdrop/hi-backdrop.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export function BaseBehaviorPane(Base) {
  function __disableScrolling() {
    const x = window.scrollX;
    const y = window.scrollY;
    window.onscroll = function() {
      window.scrollTo(x, y);
    };
    document.body.setAttribute("scroll-y", "never");
  }
  function __enableScrolling() {
    window.onscroll = function() {
    };
    document.body.removeAttribute("scroll-y");
  }
  const _BehaviorPane = class extends BehaviorBehavior(Base) {
    static get properties() {
      return {
        heading: {
          type: String,
          reflectToAttribute: true
        },
        subHeading: {
          type: String,
          reflectToAttribute: true
        },
        opened: {
          type: Boolean,
          observer: "_openedChanged",
          reflectToAttribute: true,
          notify: true
        },
        modal: {
          type: Boolean,
          reflectToAttribute: true,
          notify: true
        },
        mode: {
          type: String
        },
        disallowClose: {
          type: Boolean,
          reflectToAttribute: true
        }
      };
    }
    static get api() {
      return ["close", "open"];
    }
    static get autoBinding() {
      return {
        signal: "opened-changed",
        field: "opened"
      };
    }
    static get observers() {
      return ["__headingAttributeWatcher(heading)"];
    }
    connectedCallback() {
      super.connectedCallback();
      this._registerBehavior("BehaviorPane");
      this.__assignPaneAttributes();
      this.__includeInSlotHeading();
      this.__includeInSlotClose();
    }
    __assignPaneAttributes() {
      const labelId = "headerWrap";
      const descriptionId = f1.strategy(
        {
          "hi-dialog": "dialogBody",
          "hi-drawer": "itemsContainer"
        },
        this.localName
      );
      dom.setAttrs(this, {
        "aria-labelledby": labelId,
        "aria-describedby": descriptionId,
        "aria-modal": "true",
        tabindex: "-1",
        role: "dialog"
      });
    }
    __headingAttributeWatcher(heading) {
      if (f1.is.exists(this.__heading)) {
        dom.setText(this.__heading, heading);
      }
    }
    __includeInSlotHeading() {
      const headingTag = "h2";
      const slot = "heading";
      this.__heading = this.__heading || this.querySelector(`${headingTag}[slot="${slot}"]`);
      if (this.__heading) {
        return this.__headingAttributeWatcher(this.heading);
      }
      const headingEl = dom.setAttrs(document.createElement(headingTag), {
        tabindex: -1,
        id: "heading",
        class: "heading"
      });
      headingEl.slot = slot;
      this.__heading = headingEl;
      afterNextRender(this, () => {
        if (this.__heading) {
          this.prepend(this.__heading);
        }
        this.__headingAttributeWatcher(this.heading);
        this.__openPane__focusHeader();
      });
    }
    __removeSlotHeading() {
      if (this.__heading) {
        this.__heading.remove();
      }
      this.__heading = null;
    }
    __includeInSlotClose() {
      if (this.disallowClose) {
        return;
      }
      const closeButton = dom.create(null, "hi-close", {
        slot: "closeButton",
        device: "mobile",
        size: "small"
      });
      this.__closeButton = closeButton;
      this.appendChild(closeButton);
    }
    disconnectedCallback() {
      super.disconnectedCallback();
      this.__closePane();
      this.__removeSlotHeading();
      if (this.__closeButton) {
        this.__closeButton.remove();
      }
      this.__closeButton = void 0;
    }
    static get backdrop() {
      return HIP.factory.queryOne("hi-backdrop").instance;
    }
    open() {
      if (!this.disabled) {
        this.opened = true;
      }
    }
    close() {
      if (!this.disabled) {
        this.opened = false;
      }
    }
    _openedChanged(opened, oldOpened) {
      if (this.disabled) {
        return oldOpened;
      }
      this.setAttribute("aria-hidden", !opened + "");
      if (opened) {
        this.__openPane();
      } else {
        this.__closePane();
      }
    }
    __openPane() {
      if (this.currentPane == null) {
        window.addEventListener("keydown", _BehaviorPane._pane_keyDown_window, true);
      }
      this.__setPreviousItemFocusable();
      if (this.mode === "sub-window") {
        this.__openPane__subWindow();
      } else {
        this.__openPane__main();
      }
      this.__openPane__focusHeader();
      this.__openPane__backdrop();
      this.__openPane__events();
    }
    __closePane() {
      if (this.mode === "sub-window") {
        this.__closePane__subWindow();
      } else {
        this.__closePane__main();
      }
      this.__closePane__backdrop();
      this.__closePane__events();
      if (this.currentPane == null) {
        window.removeEventListener("keydown", _BehaviorPane._pane_keyDown_window, true);
      }
    }
    __setPreviousItemFocusable() {
      this.__lastActiveItem = dom.activeElement() || this._focusedChild();
    }
    __openPane__subWindow() {
      const sub = _BehaviorPane.__sub__;
      const paneIndex = sub.length;
      this.paneIndex = paneIndex;
      sub[paneIndex] = this;
      sub.forEach((el, index) => {
        el.style.zIndex = `calc(var(--hi-depth--modal) + ${index * 100 + 100})`;
        return el;
      });
    }
    __openPane__main() {
      const pane = _BehaviorPane.__main__;
      if (pane) {
        pane.close();
      }
      _BehaviorPane.__main__ = this;
      _BehaviorPane.__main__.style.zIndex = `var(--hi-depth--modal)`;
    }
    __closePane__subWindow() {
      const sub = _BehaviorPane.__sub__;
      sub.splice(this.paneIndex, 1);
      sub.forEach((el, index) => {
        el.paneIndex = index;
        return el;
      });
    }
    __closePane__main() {
      _BehaviorPane.__main__ = void 0;
    }
    __openPane__focusHeader() {
      if (this.__heading) {
        this.__heading.focus();
      }
    }
    __openPane__backdrop() {
      const sub = _BehaviorPane.__sub__;
      const backdrop = _BehaviorPane.backdrop;
      this.setAttr_disallowClosing(backdrop);
      backdrop.style.zIndex = this.__writeStyleBackdropZIndex(sub);
      backdrop.opened = true;
    }
    __closePane__backdrop() {
      const sub = _BehaviorPane.__sub__;
      const backdrop = _BehaviorPane.backdrop;
      const anyOpenPanes = sub.length > 0 || _BehaviorPane.__main__;
      if (anyOpenPanes) {
        backdrop.style.zIndex = this.__writeStyleBackdropZIndex(sub);
      }
      this.setAttr_disallowClosing(backdrop);
      backdrop.opened = anyOpenPanes;
    }
    setAttr_disallowClosing(backdropElement) {
      dom.flipAttr(backdropElement, "disallow-close", this.disallowClose);
    }
    __writeStyleBackdropZIndex(sub) {
      return `calc(var(--hi-depth--modal) + ${sub.length > 0 ? sub.length * 100 : 0} - 10)`;
    }
    __openPane__events() {
      this.__disableScrollingDebouncer = Debouncer.debounce(this.__disableScrollingDebouncer, microTask, () => {
        afterNextRender(this, __disableScrolling);
      });
    }
    __closePane__events() {
      this.__enableScrollingDebouncer = Debouncer.debounce(this.__enableScrollingDebouncer, microTask, () => {
        afterNextRender(this, __enableScrolling);
      });
      const activePane = _BehaviorPane.activePane;
      if (!this.__lastActiveItem) {
        return;
      }
      const isInParent = dom.existsIn(activePane, this.__lastActiveItem);
      if (this.__lastActiveItem.hasAttribute("for")) {
        const __for = this.__lastActiveItem.getAttribute("for");
        this.__lastActiveItem = HIP.browser.queryOne(__for);
      }
      const nextFocusEl = activePane ? isInParent ? this.__lastActiveItem : activePane._focusableChildren()[0] : this.__lastActiveItem;
      if (nextFocusEl) {
        nextFocusEl.focus();
      }
    }
    static get currentPane() {
      return this.activePane;
    }
    static get activePane() {
      const sub = _BehaviorPane.__sub__;
      return sub[sub.length - 1] || _BehaviorPane.__main__;
    }
    get activePane() {
      return _BehaviorPane.activePane;
    }
    __isButton(el) {
      return el && el.nodeName.indexOf("BUTTON") >= 0;
    }
    _pane_focusFirstFocusable() {
      this.__autoFocusDebouncer = Debouncer.debounce(this.__autoFocusDebouncer, idlePeriod, () => {
        const focusableChildren = this._focusableChildren();
        const firstFocusableChild = focusableChildren && focusableChildren.length > 0 ? focusableChildren[0] : !this.disallowClose && this.__closeButton ? this.__closeButton : void 0;
        if (firstFocusableChild) {
          firstFocusableChild.focus();
        }
      });
    }
    _focusableChildren() {
      const children = dom.queryFocusable(_BehaviorPane.activePane);
      const content = children.filter((el) => !this.__isClose(el));
      const close = children.filter(this.__isClose);
      return content.concat(close).filter(this.__isTabbable);
    }
    __isClose(el) {
      return el.nodeName.indexOf("CLOSE") >= 0;
    }
    __isTabbable(el) {
      return el.getAttribute("tabindex") !== "-1";
    }
    _focusedChild() {
      const activePane = _BehaviorPane.activePane;
      if (!activePane) {
        return null;
      }
      const focusableElements = dom.queryFocusable(activePane);
      const firstFocusableElement = focusableElements[0];
      const activeElement = document.activeElement;
      if (!activeElement) {
        return firstFocusableElement;
      }
      const isActiveElementInsidePane = focusableElements.includes(activeElement);
      if (isActiveElementInsidePane) {
        return activeElement;
      }
      return firstFocusableElement;
    }
    _focusedChildIndex(focusableChildren, focusedChild) {
      return focusedChild ? focusableChildren.indexOf(focusedChild) : -1;
    }
    static _pane_keyDown_window(e) {
      const keyCode = e.which || e.keyCode;
      const ESCAPE = 27;
      const TAB = 9;
      const activePane = _BehaviorPane.activePane;
      if (_BehaviorPane._isOpenedFlyout(e.target)) {
        return;
      }
      switch (keyCode) {
        case ESCAPE:
          _BehaviorPane._pane_onKeyUp_escape_a11y(e, activePane);
          break;
        case TAB:
          _BehaviorPane._pane_onKeyUp_tab_a11y(e, activePane);
          break;
        default:
          break;
      }
    }
    static _isOpenedFlyout(el) {
      const isNotAPane = el && !(el instanceof _BehaviorPane);
      return isNotAPane ? dom.parentsOf(el).findIndex((el2) => f1.read("/behaviors/BehaviorFlyout", false, el2) && f1.read("/opened", false, el2)) >= 0 : false;
    }
    static _pane_onKeyUp_escape_a11y(e, pane) {
      if (pane.disallowClose) {
        return;
      }
      const escapeEl = e.target;
      if (!f1.read("/behaviors/BehaviorPane", false, escapeEl) && escapeEl?.opened) {
        return;
      }
      if (pane.opened) {
        pane.close();
      }
    }
    static _pane_onKeyUp_tab_a11y(e, pane) {
      if (!pane.opened) {
        return;
      }
      const focusableChildren = pane._focusableChildren();
      const focusedChild = pane._focusedChild() ?? focusableChildren[0];
      const focusedIndex = pane._focusedChildIndex(focusableChildren, focusedChild);
      if (focusedIndex < 0) {
        return;
      }
      let focusIndex = -1;
      if (e.shiftKey && focusedIndex === 0) {
        focusIndex = focusableChildren.length - 1;
      } else if (!e.shiftKey && focusedIndex >= focusableChildren.length - 1) {
        focusIndex = 0;
      }
      if (focusIndex >= 0) {
        focusableChildren[focusIndex].focus();
        e.preventDefault();
      }
    }
  };
  let BehaviorPane2 = _BehaviorPane;
  BehaviorPane2.__sub__ = [];
  BehaviorPane2.__main__;
  BehaviorPane2.__sub__ = [];
  return HIP.factory.mixin(BehaviorPane2);
}
export const BehaviorPane = dedupingMixin(BaseBehaviorPane);
HIP["BehaviorPane"] = BehaviorPane;
