
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import { dedupingMixin } from '../../../hip/library/polymer/polymer.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorBehavior } from '../../../eip/element/hi-behavior-behavior/hi-behavior-behavior.js';
export function BaseBehaviorScrollable(Base) {
  class BehaviorScrollable2 extends BehaviorBehavior(Base) {
    static get properties() {
      return {
        scrollable: {
          type: Boolean,
          reflectToAttribute: true,
          reflect: true
        }
      };
    }
    connectedCallback() {
      super.connectedCallback();
      this._registerBehavior("BehaviorScrollable");
      this.__scrollToTopBound = this._scrollToTop.bind(this);
      this.__scrollButtonEl = this.__createScrollButton();
      this.__hasScrollButton = false;
    }
    disconnectedCallback() {
      super.disconnectedCallback();
      if (this.__scrollButtonEl) {
        this.__scrollButtonEl.removeEventListener("trigger-ended", this.__scrollToTopBound);
      }
    }
    __createScrollButton() {
      if (this.__scrollButtonEl) {
        return this.__scrollButtonEl;
      }
      const scrollButtonEl = document.createElement("hi-button");
      scrollButtonEl.addEventListener("trigger-ended", this.__scrollToTopBound);
      scrollButtonEl.innerText = "Scroll to Top";
      return dom.setAttrs(scrollButtonEl, {
        aesthetic: "secondary",
        class: "mtm mbm",
        stretched: true
      });
    }
    _scrollToTop() {
      this.__scrollContainerEl.scrollTo({ top: 0 });
    }
    _assignScrollable(scrollableContainerEl) {
      this.__scrollContainerEl = scrollableContainerEl;
    }
    __indexOfChildHiClose() {
      const notFound__hiClose = -1;
      return f1.safe(
        notFound__hiClose,
        f1.pipe(
          [
            f1.values,
            f1.map((child) => child.tagName),
            f1.findKey((tagName) => tagName === "HI-CLOSE")
          ],
          this.children
        )
      );
    }
    __insertScrollButton() {
      const indexOfChildHiClose = this.__indexOfChildHiClose();
      if (indexOfChildHiClose > 0) {
        this.insertBefore(this.__scrollButtonEl, this.children[indexOfChildHiClose]);
      } else {
        this.appendChild(this.__scrollButtonEl);
      }
      this.__hasScrollButton = true;
    }
    _updateScrollable() {
      if (!this.scrollable) {
        return;
      }
      const hasScrollButton = this.__hasScrollButton;
      const scrollContainerEl = this.__scrollContainerEl;
      if (!scrollContainerEl) {
        return;
      }
      const hasMoreContent = scrollContainerEl.scrollHeight > scrollContainerEl.clientHeight;
      if (hasMoreContent && !hasScrollButton) {
        this.__insertScrollButton();
      } else if (!hasMoreContent && hasScrollButton) {
        this.removeChild(this.__scrollButtonEl);
        this.__hasScrollButton = false;
      }
    }
  }
  return HIP.factory.mixin(BehaviorScrollable2);
}
export const BehaviorScrollable = dedupingMixin(BaseBehaviorScrollable);
HIP["BehaviorScrollable"] = BehaviorScrollable;
