
import { dedupingMixin } from '../../../hip/library/polymer/polymer.js';
import { HIP, BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorBehavior } from '../../../eip/element/hi-behavior-behavior/hi-behavior-behavior.js';
import { BehaviorContainer } from '../../../eip/element/hi-behavior-container/hi-behavior-container.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export function BaseBehaviorSetFocus(Base) {
  class BehaviorSetFocus2 extends BehaviorContainer(BehaviorElement(BehaviorBehavior(Base))) {
    constructor() {
      super();
      this.canBeInFlyout = false;
      this.__mutex__focusIndex = false;
      this.__keyPressHistory = [];
    }
    static get properties() {
      return {
        focusedIndex: {
          type: Number,
          observer: "__setFocus__focusedIndexChanged",
          reflectToAttribute: true,
          notify: true
        },
        focusedValue: {
          type: Object,
          computed: "__setFocus__focusedValueComputed(focusedIndex, __slotChildren)",
          notify: true
        },
        focusedValueFormatted: {
          type: String
        },
        valueAttr: {
          type: String,
          value: "value"
        },
        allowKeySearch: {
          type: Boolean,
          observer: "_setFocus__onKeyPress"
        }
      };
    }
    connectedCallback() {
      super.connectedCallback();
      this.addEventListener("set-focus-item-focused", this._setFocus__onItemFocused);
      this.addEventListener("set-focus-item-tapped", this._setFocus__onItemTapped);
      this._registerBehavior("BehaviorSetFocus");
    }
    _setFocus__onSlotChanged() {
      this._readSlotChildren();
      if (this.focusedIndex != null) {
        this.focusIndex(this.focusedIndex);
      } else {
        this.clearFocusIndex();
      }
    }
    __setFocus__focusedIndexChanged(focusedIndex, oldValue) {
      this.focusIndex(focusedIndex, oldValue);
    }
    __setFocus__focusedValueComputed(_focusedIndex) {
      return this._indexToItemValue(_focusedIndex);
    }
    _setFocus__onItemFocused(event) {
      const detail = event.detail;
      event.stopPropagation();
      if (this.tagName) {
        this.focusIndex(detail.index, this.focusedIndex);
      }
    }
    _setFocus__onKeyPress() {
      if (this.__onKeyPressBounded) {
        this.removeEventListener("keypress", this.__onKeyPressBounded);
      }
      this.__onKeyPressBounded = this.__onKeyPress.bind(this);
      this.addEventListener("keypress", this.__onKeyPressBounded);
    }
    __onKeyPress(event) {
      const pressedKeyCode = typeof event.which == "number" ? event.which : event.keyCode;
      const slotChildrenArray = this._readSlotChildren();
      const childrenInnerText = slotChildrenArray.map((x) => x.innerText.toLowerCase());
      const pressedKey = String.fromCharCode(pressedKeyCode);
      const matchingFirstLetter = [];
      const matchingRestOfTheLetters = [];
      if (pressedKeyCode) {
        this.__keyPressHistory.push(pressedKey);
      }
      const keyPressRegex = this.__buildRegex(this.__keyPressHistory);
      childrenInnerText.map((el, index) => {
        if (el.charAt(0) == pressedKey) {
          matchingFirstLetter.push(index);
        } else if (keyPressRegex.test(el) && el.match(keyPressRegex) !== null) {
          matchingRestOfTheLetters.push(index);
        }
      });
      const filteredArray = [...matchingFirstLetter, ...matchingRestOfTheLetters];
      if (!f1.is.empty(filteredArray)) {
        const focusedIndex = f1.nth(0, filteredArray);
        this.scrollToItem(this.focusIndex(focusedIndex));
        this.focusIndex(focusedIndex);
      }
    }
    __buildRegex(keyPressArray) {
      const lettersReg = keyPressArray.map((letter) => `(^[^${letter}]*${letter})`);
      const joinLettersReg = lettersReg.join("?");
      const keyPressReg = new RegExp(`${joinLettersReg}`, "gi");
      return keyPressReg;
    }
    _setFocus__onTap(event) {
      const srcElement = event.target;
      event.stopPropagation();
      if (this.tagName) {
        this.focusIndex(srcElement.index, this.focusedIndex);
        this.chooseFocused();
      }
    }
    _setFocus__onItemTapped(event) {
      const detail = event.detail;
      event.stopPropagation();
      if (this.tagName) {
        this.focusIndex(detail.index, this.focusedIndex);
        this.chooseFocused();
      }
    }
    _setFocus__onFocus() {
      this.focusIndex(this.focusedIndex || 0);
    }
    shiftFocusUp() {
      return this.focusIndex((this.focusedIndex || 0) - 1, this.focusedIndex);
    }
    shiftFocusDown() {
      const focusedIndex = this.focusedIndex == null ? 0 : this.focusedIndex + 1;
      return this.focusIndex(focusedIndex, this.focusedIndex);
    }
    shiftFocusFirst() {
      return this.focusIndex(0);
    }
    shiftFocusLast() {
      return this.focusIndex(this.slotChildren.length - 1);
    }
    _setFocus__onKeyUp_end(event) {
      event.preventDefault();
      return this.scrollToItem(this.shiftFocusLast());
    }
    _setFocus__onKeyUp_home(event) {
      event.preventDefault();
      return this.scrollToItem(this.shiftFocusFirst());
    }
    _setFocus__onKeyUp_up(event) {
      event.preventDefault();
      return this.scrollToItem(this.shiftFocusUp());
    }
    _setFocus__onKeyUp_left(event) {
      event.preventDefault();
      return this.scrollToItem(this.shiftFocusUp());
    }
    _setFocus__onKeyUp_down(event) {
      event.preventDefault();
      return this.scrollToItem(this.shiftFocusDown());
    }
    _setFocus__onKeyUp_right(event) {
      event.preventDefault();
      return this.scrollToItem(this.shiftFocusDown());
    }
    scrollToItem(item) {
      const isInsideAFlyout = HIP.util.doesExtend(this.parentElement, "BehaviorFlyout");
      const isScrollable = this.$.container && dom.is.scrollable(this.$.container);
      const rtn = item ? item.index : "-1";
      if (item && (isInsideAFlyout || isScrollable)) {
        try {
          item.scrollIntoView({
            behavior: "auto",
            inline: "nearest",
            block: "nearest"
          });
        } catch (e) {
          item.scrollIntoView({
            behavior: "auto",
            inline: "start",
            block: "start"
          });
        } finally {
          this._dispatchNotification("scroll-to-item", {
            item: rtn
          });
        }
      }
      return rtn;
    }
    _setFocus__onKeyUp_space() {
      this.chooseFocused();
    }
    _setFocus__onKeyUp_enter() {
      this.chooseFocused();
    }
    _setFocus__onKeyUp__space() {
      this._setFocus__onKeyUp_space();
    }
    _setFocus__onKeyUp__enter() {
      this._setFocus__onKeyUp_enter();
    }
    chooseFocused() {
      const item = this._indexToItem(this.focusedIndex);
      if (!item) {
        return;
      }
      const focusedValue = this.focusedValue;
      const focusedValueFormatted = this._readValueFormatted(item);
      this.focusedValueFormatted = focusedValueFormatted;
      this.__keyPressHistory = [];
      this._dispatchProcess(`chosen`, {
        valueFormatted: focusedValueFormatted,
        index: this.focusedIndex,
        value: focusedValue,
        item
      });
      item._dispatchProcess(`trigger`, {
        valueFormatted: focusedValueFormatted,
        index: this.focusedIndex,
        value: focusedValue,
        item
      });
    }
    focusItem(_item) {
      return this.focusIndex(this._itemToIndex(_item));
    }
    _readValueFormatted(item) {
      return item.getAttribute("label") || item["valueFormatted"] || item.textContent?.trim();
    }
    focusIndex(_index, _indexOld) {
      const indexIsNumber = typeof _index === "number" && _index === _index;
      if (!indexIsNumber) {
        return;
      }
      if (_index === _indexOld) {
        return;
      }
      if (this.__mutex__focusIndex) {
        return;
      }
      this.__mutex__focusIndex = true;
      const item = this._focusIndex(_index, _indexOld);
      this.__mutex__focusIndex = false;
      if (item) {
        const isItemSelected = item.selected;
        this.setAttribute("aria-activedescendant", item.getAttribute("active-id") || item.getAttribute("id") || "");
        if (isItemSelected) {
          HIP.fn.announce(`${this._readValueFormatted(item)} selected`, "assertive");
        }
        return item;
      } else {
        this.removeAttribute("aria-activedescendant");
        return void 0;
      }
    }
    _focusIndex(_index, _indexOld) {
      const boundedIndex = this.__boundIndex(_index, _indexOld);
      const children = this.slotChildren;
      children.forEach((child, index) => {
        child.focused = index === boundedIndex;
      });
      this.focusedIndex = boundedIndex;
      this.notifyPath("focusedIndex");
      return children[boundedIndex];
    }
    clearFocusIndex() {
      const focusedChild = this.getFocusedChild();
      if (focusedChild) {
        focusedChild.focused = false;
      }
      this.focusedIndex = void 0;
      this.notifyPath("focusedIndex");
      this.removeAttribute("aria-activedescendant");
    }
    getFocusedChild() {
      return this.focusedIndex == null ? null : this.slotChildren[this.focusedIndex];
    }
    _setFocus__onMouseover(event) {
      const requestIndex = event.target.index;
      if (requestIndex >= 0) {
        this.focusedIndex = requestIndex;
      }
    }
    __boundIndex(index, oldIndex = 0) {
      const children = this.slotChildren;
      if (children.length <= 0) {
        return 0;
      }
      const isNumber = typeof index === "number" && index === index;
      if (!isNumber) {
        return 0;
      }
      const normalizedVector = this.__normalizeVector(index, oldIndex);
      let goToIndex = this.__clampIndex(index);
      while (children[goToIndex] && children[goToIndex].hasAttribute("disabled")) {
        goToIndex = goToIndex + normalizedVector;
      }
      const item = children[goToIndex];
      return !item ? oldIndex : goToIndex;
    }
    __normalizeVector(v1, v2) {
      return v1 > v2 ? 1 : -1;
    }
    __getCompareValue(val) {
      return f1.is.object(val) ? f1.read("/value", `${val}`, val) : `${val}`;
    }
    __getElValue(el) {
      const value = el[this.valueAttr];
      return f1.is.object(value) ? value : el.getAttribute(this.valueAttr);
    }
    _itemValueToIndex(value) {
      const matchValue = this.__getCompareValue(value);
      const elements = this.slotChildren.filter((el) => this.__getCompareValue(this.__getElValue(el)) === matchValue);
      return elements.length > 0 ? elements[0].index : void 0;
    }
    _itemToIndex(item) {
      return this.slotChildren.indexOf(item);
    }
    _indexToItem(index) {
      const boundedIndex = this.__clampIndex(index);
      return this.slotChildren[boundedIndex];
    }
    _indexToItemValue(index) {
      const dom2 = this._indexToItem(index);
      return dom2 ? dom2.getAttribute(this.valueAttr) || dom2.index : null;
    }
    __clampIndex(index) {
      const n = this.slotChildren.length;
      index = index >= n ? n - 1 : index;
      index = index < 0 ? 0 : index;
      return index;
    }
  }
  return HIP.factory.mixin(BehaviorSetFocus2);
}
export const BehaviorSetFocus = dedupingMixin(BaseBehaviorSetFocus);
HIP["BehaviorSetFocus"] = BehaviorSetFocus;
