
import { dedupingMixin, afterNextRender } from '../../../hip/library/polymer/polymer.js';
import { BehaviorSetFocus } from '../../../eip/element/hi-behavior-set-focus/hi-behavior-set-focus.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export function BaseBehaviorSetSelect(Base) {
  class BehaviorSetSelect2 extends BehaviorSetFocus(Base) {
    static get properties() {
      return {
        multiple: {
          type: Boolean
        },
        max: {
          type: Number
        },
        selectedIndices: {
          type: Array,
          observer: "__setSelect__selectedIndicesChanged",
          notify: true,
          value: () => []
        },
        values: {
          type: Array,
          observer: "__setSelect__valuesChanged",
          notify: true,
          value: () => []
        },
        selectedIndex: {
          type: Number,
          observer: "__setSelect__selectedIndexChanged",
          reflectToAttribute: true,
          notify: true
        },
        value: {
          type: Object,
          observer: "__setSelect__valueChanged",
          reflectToAttribute: true,
          notify: true
        },
        valueFormatted: {
          type: String,
          computed: "__valueFormattedCompute(selectedIndices, selectedIndex, value, values)"
        },
        onlyValue: {
          type: Boolean
        },
        mode: {
          type: String
        }
      };
    }
    static get api() {
      return ["select", "deselect", "commit"];
    }
    static get observers() {
      return ["_valuesWatcher(value, values)", "_indicesWatcher(selectedIndex, selectedIndices)"];
    }
    connectedCallback() {
      super.connectedCallback();
      this.addEventListener("set-item-selected", this.__setSelect__onItemSelected);
      this._registerBehavior("BehaviorSetSelect");
    }
    deselect() {
      this.deselectSelected();
    }
    commit() {
      return this.multiple ? this.values : this.value;
    }
    _valuesWatcher(value, values) {
      const hasRunningOperation = !!this.__mutex__;
      const isNotValueMode = this.mode !== "value";
      if (hasRunningOperation || isNotValueMode) {
        return;
      }
      const selectedValues = this.__computedSelectedValues(value, values);
      this.__resetSelectedItems();
      this.selectedIndices = [];
      this.selectIndices(selectedValues.map((val) => this._itemValueToIndex(val)), true);
    }
    _indicesWatcher(selectedIndex, selectedIndices) {
      const hasRunningOperation = !!this.__mutex__;
      const isNotIndexMode = this.mode !== "index";
      if (hasRunningOperation || isNotIndexMode) {
        return;
      }
      const indices = [...selectedIndices, selectedIndex].map((value) => Number.parseInt(value, 10));
      this.__resetSelectedItems();
      this.selectIndices(indices, true);
    }
    _setSelect__onSlotChanged() {
      this._setFocus__onSlotChanged();
      this.__reconcileState();
      this.__notifyValue();
    }
    __reconcileState() {
      switch (this.mode) {
        case "value":
          this._valuesWatcher(this.value, this.values);
          break;
        case "index":
          this._indicesWatcher(this.selectedIndex, this.selectedIndices);
          break;
        default:
          this.__reconcileStateAuto();
          break;
      }
    }
    __notifyValue() {
      afterNextRender(this, () => {
        this._dispatchBubblingEvent("value-notified", {
          value: this.value
        });
      });
    }
    _readSlotChildren(elSlot, nodeFilter = HIP.util.isNode) {
      const rtn = super._readSlotChildren(elSlot, nodeFilter);
      this.__notifyValue();
      return rtn;
    }
    __reconcileStateAuto() {
      const selectedChildren = this.slotChildren.filter(
        (option) => option.selected
      );
      const selectedValues = [...this.values, this.value].filter((v) => v !== void 0);
      if (selectedValues.length > 0) {
        this.selectIndices(
          selectedValues.map((value) => this._itemValueToIndex(value)),
          true
        );
      } else if (f1.is.number(this.selectedIndex)) {
        this.selectIndex(this.selectedIndex, true);
      } else if (this.value != null) {
        this.selectValue(this.value, true);
      } else if (selectedChildren.length > 0) {
        this.selectIndices(
          selectedChildren.map((el) => el.index),
          true
        );
      } else {
        this.reset();
      }
    }
    __setSelect__selectedValueCompute(_selectedIndex) {
      return this._indexToItemValue(_selectedIndex);
    }
    __valueFormattedCompute(selectedIndices, selectedIndex, value, values) {
      return this.computedValueFormatted(selectedIndices, selectedIndex, value, values);
    }
    computedValueFormatted(selectedIndices = this.selectedIndices, selectedIndex = this.selectedIndex, value = this.value, values = this.values) {
      let valueIndices = [];
      switch (this.mode) {
        case "value":
          valueIndices = this.__computedSelectedValues(value, values).map(
            (val) => this._itemValueToIndex(val)
          );
          break;
        case "index":
          valueIndices = selectedIndices && selectedIndices.length > 0 ? selectedIndices : selectedIndex ? [selectedIndex] : [];
          break;
        default:
          valueIndices = selectedIndices && selectedIndices.length > 0 ? selectedIndices : selectedIndex ? [selectedIndex] : [];
          break;
      }
      return valueIndices.map((index) => {
        const item = this._indexToItem(index);
        if (!item) {
          return "";
        } else if (item.hasAttribute("value-formatted")) {
          return item.getAttribute("value-formatted");
        } else if (item.valueFormatted != null) {
          return item.valueFormatted;
        } else {
          return item.textContent?.trim();
        }
      }).join(", ");
    }
    __computedSelectedValues(value = this.value, values = this.values) {
      const selectedValues = !this.multiple && value != null ? [value] : this.multiple && this.onlyValue && value ? value : this.multiple && values ? values.filter((v) => v !== void 0) : [];
      return selectedValues;
    }
    __setSelect__selectedIndicesChanged(_indices, _indicesOld) {
      if (this.mode || this.__mutex__) {
        return;
      }
      if (!_indices || _indices.length <= 0) {
        return this.reset();
      }
      const requestedIndices = HIP.util.clone(_indices);
      (_indicesOld || []).forEach((_index) => {
        this.deselectIndex(_index, true);
      });
      requestedIndices.forEach((_index) => {
        this.selectIndex(_index, true);
      });
    }
    __setSelect__valuesChanged(_values) {
      if (this.mode || this.__mutex__) {
        return;
      } else if (!_values || _values.length <= 0) {
        return this.reset();
      }
      this.reset();
      _values.forEach((_value) => {
        this.selectValue(_value, true);
      });
    }
    __setSelect__selectedIndexChanged(_index, _oldIndex) {
      if (this.mode || this.__mutex__) {
        return;
      }
      this.reset();
      this.selectIndex(_index, true, _oldIndex);
    }
    __setSelect__valueChanged(_value) {
      if (this.mode || this.__mutex__) {
        return;
      }
      this.reset();
      this.selectValue(_value, true);
    }
    _setSelect__onTap(evt) {
      if (this.disabled) {
        return;
      }
      if (!evt.target) {
        return;
      }
      const target = this.__closestHIParent(evt.target);
      const currentActiveChild = evt.currentTarget?.children[target.index];
      const hasDisabledChildren = currentActiveChild && currentActiveChild.hasAttribute("disabled");
      if (hasDisabledChildren != false) {
        return;
      }
      if (target.index != null) {
        this.toggleIndex(target.index);
      }
    }
    __closestHIParent(start = this) {
      const parents = this._getParentsOnEvent({
        target: start
      });
      for (let i = 0; i < parents.length; i++) {
        if (parents[i].hasAttribute("x-hip")) {
          return parents[i];
        }
      }
      return void 0;
    }
    __setSelect__onItemSelected(event) {
      event.stopPropagation();
      if (this.__mutex__) {
        return;
      }
      this.selectIndex(event.detail.index);
    }
    _setSelect__onKeyUp_space() {
      if (this.disabled) {
        return;
      }
      super._setFocus__onKeyUp_space();
      this.toggleIndex(this.focusedIndex);
    }
    _setSelect__onKeyUp_enter() {
      if (this.disabled) {
        return;
      }
      const isInsideAFlyout = HIP.util.doesExtend(this.parentElement, "BehaviorFlyout");
      if (this.canBeInFlyout && !isInsideAFlyout) {
        return;
      }
      super._setFocus__onKeyUp_space();
      this.toggleIndex(this.focusedIndex);
    }
    reset() {
      this.__mutex__ = true;
      this.setProperties({
        selectedIndex: this.onlyValue && this.multiple ? [] : void 0,
        value: this.onlyValue && this.multiple ? [] : void 0,
        selectedIndices: [],
        values: []
      });
      const selectedChildren = this.slotChildren.filter(
        (option) => option.selected
      );
      selectedChildren.forEach((item) => {
        this._alterItemState(item, (item2) => {
          item2.selected = false;
        });
      });
      this.__mutex__ = false;
    }
    findIndicesOf(_values) {
      const indices = [];
      const children = this.slotChildren;
      const valueAttr = this.valueAttr;
      const n = children.length;
      let idx = 0;
      for (; idx < n; idx++) {
        const valStr = children[idx].getAttribute(valueAttr);
        if (_values.findIndex((_v) => `${_v}` === valStr) >= 0) {
          indices.push(idx);
        }
      }
      return indices;
    }
    toggleFocusedIndex(suppressSelectEvent = false) {
      const focusedIndex = this.focusedIndex;
      if (focusedIndex != null) {
        this.toggleIndex(focusedIndex, suppressSelectEvent);
      }
    }
    toggleIndex(_index, suppressSelectEvent = false) {
      const isSelected = this.selectedIndices.indexOf(_index) >= 0;
      if (this.multiple && isSelected) {
        this.deselectIndex(_index, suppressSelectEvent);
      } else {
        this.selectIndex(_index, suppressSelectEvent);
      }
      this.commit();
    }
    selectItem(_item, suppressSelectEvent = false) {
      return this.selectIndex(this._itemToIndex(_item), suppressSelectEvent);
    }
    selectValues(_values, suppressSelectEvent = false) {
      return this.selectIndices(this.findIndicesOf(_values), suppressSelectEvent);
    }
    selectValue(_value, suppressSelectEvent = false) {
      return this.selectIndices(this.findIndicesOf([_value]), suppressSelectEvent);
    }
    selectIndices(_indices, suppressSelectEvent = false) {
      return this.selectIndex(_indices, suppressSelectEvent);
    }
    selectIndex(_index, suppressSelectEvent = false, oldIndex = this.selectedIndex) {
      const indices = f1.is.array(_index) ? _index : [_index];
      this.__mutex__ = true;
      indices.forEach((idx) => {
        this._selectIndex(idx, suppressSelectEvent, oldIndex);
      });
      this.__mutex__ = false;
      return this;
    }
    _selectIndex(_index, suppressSelectEvent = false, oldIndex = this.selectedIndex) {
      const item = this.slotChildren[_index];
      if (!item) {
        return;
      }
      const selected = item.selected;
      const _valueAttr = item.getAttribute(this.valueAttr);
      const _valueProp = item[this.valueAttr];
      const value = _valueProp !== void 0 ? _valueProp : _valueAttr;
      const internalIndex = this.selectedIndices.indexOf(_index);
      super.focusIndex(_index);
      if (!this.multiple) {
        this.pop("selectedIndices");
        this.pop("values");
        if (typeof oldIndex === "number" && oldIndex === oldIndex) {
          const oldItem = this.slotChildren[oldIndex];
          if (oldItem) {
            this.deactivateItem(oldItem);
          }
        }
      }
      if (internalIndex < 0) {
        this.selectedIndices = f1.copy(f1.push(_index, this.selectedIndices));
      }
      if (this.values.indexOf(value) < 0) {
        this.values = f1.copy(f1.push(value, this.values));
      }
      const keepSameValue = this.multiple && this.onlyValue;
      const selectedIndexOut = _index;
      const valueOut = value;
      this.selectedIndex = selectedIndexOut;
      this.value = keepSameValue ? this.values : valueOut;
      this.activateItem(item);
      const hasValueChanged = item.selected !== selected;
      if (hasValueChanged && !suppressSelectEvent) {
        HIP.fn.announce(`${this._readValueFormatted(item)} selected`, "assertive");
        this._dispatchProcess(`selected`, {
          item: this.slotChildren[this.selectedIndex],
          index: this.selectedIndex,
          value: this.value
        });
      }
    }
    deselectIndices(_indices, suppressSelectEvent = false) {
      return this.deselectIndex(_indices, suppressSelectEvent);
    }
    deselectIndex(_index, suppressSelectEvent = false) {
      const indices = f1.is.array(_index) ? _index : [_index];
      this.__mutex__ = true;
      indices.forEach((idx) => {
        this._deselectIndex(idx, suppressSelectEvent);
      });
      this.__mutex__ = false;
      return this;
    }
    _deselectIndex(_index, suppressSelectEvent = false) {
      const item = this.slotChildren[_index];
      if (!item) {
        return;
      }
      const deselected__valueFormatted = this._readValueFormatted(item);
      const deselected__details = {
        item: this.slotChildren[item.selectedIndex],
        index: item.selectedIndex,
        value: item.value
      };
      const internalIndex = this.selectedIndices.indexOf(_index);
      this.selectedIndices.splice(internalIndex, 1);
      const internalValueIndex = this.values.findIndex((value) => `${item.value}` === `${value}`);
      if (internalValueIndex >= 0) {
        this.values.splice(internalValueIndex, 1);
      }
      this.selectedIndices = f1.copy(this.selectedIndices);
      this.values = f1.copy(this.values);
      const keepSameValue = this.multiple && this.onlyValue;
      const selectedIndexOut = this.selectedIndices.length > 0 ? new Number(internalIndex) : void 0;
      const valueOut = this.values.length > 0 ? this.values[0] : void 0;
      this.set("selectedIndex", keepSameValue ? this.selectIndices : selectedIndexOut);
      this.set("value", keepSameValue ? this.values : valueOut);
      this.deactivateItem(item);
      if (!suppressSelectEvent) {
        HIP.fn.announce(`${deselected__valueFormatted} deselected`, "assertive");
        this._dispatchProcess(`deselected`, deselected__details);
      }
    }
    activateItem(item) {
      this._alterItemState(item, (item2) => {
        if (item2.selected) {
          item2.activate();
        } else {
          item2.selected = true;
        }
      });
    }
    deactivateItem(item) {
      this._alterItemState(item, (item2) => {
        if (!item2.selected) {
          item2.deactivate();
        } else {
          item2.selected = false;
        }
      });
    }
    deselectSelected() {
      if (f1.is.exists(this.selectedIndex) && this.selectedIndex >= 0) {
        this.clearFocusIndex();
        this.deselectIndex(this.selectedIndex, true);
        this.reset();
      }
    }
    __resetSelectedItems() {
      const selectedChildren = this.slotChildren.filter((option) => option.selected);
      selectedChildren.forEach((item) => {
        this._alterItemState(item, (item2) => {
          item2.selected = false;
        });
      });
    }
    _alterItemState(item, fn) {
      item.directMode = true;
      fn(item);
      item.directMode = false;
    }
  }
  return HIP.factory.mixin(BehaviorSetSelect2);
}
export const BehaviorSetSelect = dedupingMixin(BaseBehaviorSetSelect);
HIP["BehaviorSetSelect"] = BehaviorSetSelect;
