
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
export class TreeService {
  static notifyOfSelection(tree, item) {
    if (!item) {
      const els = tree.querySelectorAll(`#${tree.getAttribute("aria-activedescendant")}`);
      if (els.length > 0) {
        item = els[0];
      }
    }
    if (!item) {
      return tree;
    }
    tree._dispatchProcess("selected", {
      id: item.getAttribute("id"),
      value: item.value,
      item
    });
    return tree;
  }
  log(root) {
    TreeService.traverseNode(
      (node) => window.console.log("leaf", node.getAttribute("id"), node.innerText),
      (node) => window.console.log("node", node.getAttribute("id"), node.innerText),
      root
    );
  }
  static traverseNode(leafFn, nodeFn = () => {
  }, root, packet = {
    values: [],
    db: {}
  }) {
    root.slotChildren.forEach((node) => {
      const isNode = TreeService.nodeFilter(node);
      if (!packet.db[node.id]) {
        const value = isNode ? nodeFn(node) : leafFn(node);
        if (value) {
          packet.values.push(value);
        }
      }
      packet.db[node.id] = true;
      if (isNode) {
        packet = TreeService.traverseNode(leafFn, nodeFn, node, packet);
      }
    });
    return packet;
  }
  static traverseParentChain(item, nodeFn = (node) => node, packet = {
    values: [],
    db: {}
  }) {
    const parent = item.parentElement;
    if (TreeService.treeFilter(parent)) {
      return packet;
    }
    const value = nodeFn(parent);
    if (value) {
      packet.values.push(value);
    }
    return TreeService.traverseParentChain(parent, nodeFn, packet);
  }
  static leafFilter(o) {
    return o.of === "leaf";
  }
  static nodeFilter(o) {
    return o.of === "node";
  }
  static treeFilter(o) {
    return o.of === "tree";
  }
  nodeFilter(o) {
    return TreeService.nodeFilter(o);
  }
  leafFilter(o) {
    return TreeService.leafFilter(o);
  }
  notifyOfSelection(tree, item) {
    return TreeService.notifyOfSelection(tree, item);
  }
  traverseNode(leafFn, nodeFn = () => {
  }, root, packet = {
    values: [],
    db: {}
  }) {
    return TreeService.traverseNode(leafFn, nodeFn, root, packet);
  }
}
const treeService = new TreeService();
HIP["tree"] = treeService;
export default treeService;
