
import { dedupingMixin } from '../../../hip/library/polymer/lib/utils/mixin.js';
import { BehaviorBehavior } from '../../../eip/element/hi-behavior-behavior/hi-behavior-behavior.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import '../../../eip/element/hi-component/hi-component.js';
import state from '../../../fip/library/fi-state/fi-state.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export function BaseBehaviorState(Base) {
  class BehaviorState2 extends BehaviorBehavior(Base) {
    constructor() {
      super(...arguments);
      this.mementoChangeProperties = [];
      this.mementoCacheProperties = [];
      this.mementoVersion = 1;
      this.__mementoRecovered = false;
      this.disallowAutoState = false;
    }
    static get properties() {
      return {
        stateStoreKey: {
          type: String,
          attribute: "state-store-key"
        },
        stateAlgorithm: {
          type: String,
          attribute: "state-algorithm"
        },
        disallowAutoState: {
          type: Boolean,
          attribute: "disallow-auto-state"
        }
      };
    }
    connectedCallback() {
      super.connectedCallback();
      this._registerBehavior("BehaviorState");
    }
    disconnectedCallback() {
      super.disconnectedCallback();
      if (!this.disallowAutoState) {
        this.backup();
      }
    }
    updated(props) {
      super.updated(props);
      if (this.disallowAutoState) {
        return;
      }
      if (!this.stateAlgorithm) {
        return;
      }
      const hasChangePropertiesChanged = this.mementoChangeProperties.some((prop) => props.has(prop));
      const hasRecovered = this.__mementoRecovered;
      if (!hasRecovered && hasChangePropertiesChanged) {
        this.recover().finally(() => {
          this.__mementoRecovered = true;
        });
      }
      const hasCachePropertiesChanged = this.mementoCacheProperties.some((prop) => props.has(prop));
      if (hasRecovered && (hasChangePropertiesChanged || hasCachePropertiesChanged)) {
        this.backup();
      }
    }
    computeDigestState(changeSet) {
      return this.stateStoreKey ?? this._computeDigestState(changeSet);
    }
    _computeDigestState(changeSet) {
      return changeSet;
    }
    save() {
      return {
        owner: this.localName,
        state: this._save()
      };
    }
    _save() {
      throw new Error(`BehaviorState#_save is abstract`);
    }
    restore(memento) {
      const mementoState = f1.read("/state", {}, memento);
      if (f1.is.empty(mementoState)) {
        return this;
      }
      this._restore(memento);
      this.mementoChangeProperties.forEach((key) => {
        this.requestUpdate(key);
      });
      return this;
    }
    _restore(memento) {
      throw new Error(`BehaviorState#_restore is abstract`);
    }
    async backup() {
      return state.backup(this, this.stateAlgorithm);
    }
    async recover() {
      return state.recover(this, this.stateAlgorithm);
    }
    async forget() {
      return state.forget(this, this.stateAlgorithm);
    }
  }
  return HIP.factory.mixin(BehaviorState2);
}
export const BehaviorState = HIP["BehaviorState"] = dedupingMixin(BaseBehaviorState);
