
import { dedupingMixin } from '../../../hip/library/polymer/polymer.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorBehavior } from '../../../eip/element/hi-behavior-behavior/hi-behavior-behavior.js';
import '../../../hip/library/polymer/polymer.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export function BaseBehaviorValidated(Base) {
  const _BehaviorValidated = class extends BehaviorBehavior(Base) {
    constructor() {
      super(...arguments);
      this.customValidations = {};
    }
    static get properties() {
      return {
        novalidate: {
          type: Boolean,
          reflectToAttribute: true,
          reflect: true
        },
        invalid: {
          type: Boolean,
          reflectToAttribute: true,
          reflect: true
        }
      };
    }
    connectedCallback() {
      super.connectedCallback();
      this._registerBehavior("BehaviorValidated");
    }
    reportValidity() {
      return this.checkValidity();
    }
    resetCustomValidity() {
      this.__customValidityMessage = void 0;
    }
    setCustomValidity(message) {
      this.__customValidityMessage = message;
    }
    get validationMessage() {
      if (this.__customValidityMessage) {
        return this.__customValidityMessage;
      } else if (this.validationKey) {
        return this.validityReasons[this.validationKey];
      } else {
        return "";
      }
    }
    get validationKey() {
      const failingValidation = this.validityReasons || {};
      for (const index in failingValidation) {
        return index;
      }
      return "";
    }
    checkValidity() {
      const validity = this.validity;
      const isValid = validity.valid === true;
      this._applyValidity(isValid);
      return isValid;
    }
    _applyValidity(isValid) {
      this.invalid = !isValid;
      if (!isValid) {
        this._dispatchEvent(`invalid`, {
          input: this.value,
          value: isValid
        });
      }
    }
    get validity() {
      return this._computeValidity({});
    }
    _spreadValidity(validity) {
      const validityRec = {};
      for (const k in validity) {
        validityRec[k] = validity[k];
      }
      return validityRec;
    }
    _computeValidity(validity) {
      const validityFinal = f1.merge(
        "over",
        {
          typeMismatch: false,
          patternMismatch: false,
          rangeUnderflow: false,
          rangeOverflow: false,
          valueMissing: false,
          stepMismatch: false,
          customError: false,
          badInput: false,
          tooShort: false,
          tooLong: false,
          valid: true
        },
        validity
      );
      validityFinal.customError = this._hasFailingValidation(this._getFailingCustomValidations(this.value));
      validityFinal.valid = this.__isValidityValid(validityFinal);
      return validityFinal;
    }
    __isValidityValid(validity) {
      return !f1.some(
        f1.identity,
        f1.filter((_, key) => key !== "valid", validity)
      );
    }
    get willValidate() {
      return this.novalidate !== true;
    }
    get validityReasons() {
      return this._getFailingValidations(this.value);
    }
    registerValidation(validation) {
      this.customValidations[validation.name] = validation;
    }
    unregisterValidation(validation) {
      delete this.customValidations[validation.name];
    }
    _hasFailingValidation(validations) {
      for (const index in validations) {
        return true;
      }
      return false;
    }
    _getFailingValidations(value) {
      return this._getFailingInputValidations(value, this._getFailingCustomValidations(value));
    }
    _getFailingCustomValidations(value, failingValidations = {}) {
      const validValidations = f1.filter(
        (validation) => !validation.checkValidity(value),
        this.customValidations
      );
      return f1.reduce(
        (failingValidations2, validation) => {
          failingValidations2[validation.name] = validation.validationMessage(value);
          return failingValidations2;
        },
        failingValidations,
        validValidations
      );
    }
    _getFailingInputValidations(value, failingValidations = {}) {
      const validityMessages = _BehaviorValidated.validityMessages;
      const validValidations = f1.filter(
        (validation, index) => validation && validityMessages[index],
        this.validity
      );
      let el = this;
      if (this.localName === "hi-dropdown") {
        el = f1.read("/firstSlotChild", this, this);
        el.labelEl = f1.safe(this.labelEl, el.labelEl);
      }
      return f1.reduce(
        (failingValidations2, validation, index) => {
          failingValidations2[index] = validityMessages[index](el, value);
          return failingValidations2;
        },
        failingValidations,
        validValidations
      );
    }
    _applyValidationAlterations(value) {
      return f1.reduce(
        (value2, validation) => {
          return validation.value(value2);
        },
        value,
        this.customValidations
      );
    }
    static __getLabelText(el) {
      return f1.read("/labelEl/innerText", "Value", el);
    }
  };
  let BehaviorValidated2 = _BehaviorValidated;
  BehaviorValidated2.validityMessages = {
    stepMismatch: (el, value) => `${_BehaviorValidated.__getLabelText(el)} must be at an interval of ${el.step}`,
    typeMismatch: (el, value) => {
      const mismatchStrategy = f1.strategy({
        url: `${_BehaviorValidated.__getLabelText(el)} is not a valid URL`,
        email: `Please enter your email address in this format: email@example.com`,
        fallback: `${_BehaviorValidated.__getLabelText(el)} is not the correct kind of information`
      });
      return mismatchStrategy(el.type);
    },
    patternMismatch: (el, value) => `${_BehaviorValidated.__getLabelText(el)} does not follow the required pattern with "${value}"`,
    badInput: (el) => `${_BehaviorValidated.__getLabelText(el)} is missing information`,
    rangeOverflow: (el, value) => `${_BehaviorValidated.__getLabelText(el)} must be less than or equal to ${el.formatType ? f1.format("iso_date", { format_type: el.formatType }, el.max) : el.max}`,
    rangeUnderflow: (el, value) => `${_BehaviorValidated.__getLabelText(el)} must be greater than or equal to ${el.formatType ? f1.format("iso_date", { format_type: el.formatType }, el.min) : el.min}`,
    valueMissing: (el) => `${_BehaviorValidated.__getLabelText(el)} is required`,
    tooShort: (el) => `${_BehaviorValidated.__getLabelText(el)} must contain at least ${el.minlength} characters`,
    tooLong: (el) => `${_BehaviorValidated.__getLabelText(el)} may contain at most ${el.maxlength} characters`
  };
  return HIP.factory.mixin(BehaviorValidated2);
}
export const BehaviorValidated = dedupingMixin(BaseBehaviorValidated);
HIP["BehaviorValidated"] = BehaviorValidated;
