
import { dedupingMixin } from '../../../hip/library/polymer/polymer.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorBehavior } from '../../../eip/element/hi-behavior-behavior/hi-behavior-behavior.js';
import { BehaviorValidation } from './hi-behavior-validation.js';
import '../../../hip/library/polymer/polymer.js';
import './hi-behavior-validation.js';
export function BaseBehaviorValidationMask(Base) {
  const STRIP_PADDED_CHARACTERS = /[^a-z0-9]$/i;
  const STRIP_FORMATTING_INPUT = /[^a-z\d]+/gi;
  const STRIP_FORMATTING_RULE = /[^*?^&@]+/g;
  const HAS_NOT_STRICT_RULE = /[*^&@]+/g;
  const MASK_RULES = {
    "?": /[a-z0-9]?/i,
    "*": /[a-z0-9]/i,
    "^": /[a-f0-9]/i,
    "@": /[a-z]/i,
    "&": /\d/
  };
  class Mask {
    static maskValue(input = "", rule = "") {
      const characterRules = rule.replace(STRIP_FORMATTING_RULE, "").split("");
      const inputCharacters = input.replace(STRIP_FORMATTING_INPUT, "").split("");
      const ruleToExamine = inputCharacters.length;
      let allowedCharacters = "";
      for (let i = 0; i < ruleToExamine; i++) {
        const _input = inputCharacters[i];
        const _rule = MASK_RULES[characterRules[i]];
        if (!!_rule && !!_input.match(_rule)) {
          allowedCharacters += _input;
        } else {
          return allowedCharacters;
        }
      }
      return allowedCharacters;
    }
    static maskFormat(value = "", rule = "") {
      const valueCharacters = value.split("");
      const ruleCharacters = rule.split("");
      const rulesLength = ruleCharacters.length;
      let formattedString = "";
      let j = 0;
      for (let i = 0; i < rulesLength; i++) {
        const _rule = ruleCharacters[i];
        const isKnownRule = !!MASK_RULES[_rule];
        if (isKnownRule) {
          const _value = valueCharacters[j++];
          if (!!_value && !!_value.match(MASK_RULES[_rule])) {
            formattedString += _value;
          } else {
            return formattedString.replace(STRIP_PADDED_CHARACTERS, "");
          }
        } else {
          formattedString += _rule;
        }
      }
      return formattedString.substring(0, rule.length);
    }
    static maskValidate(input = "", rule = "") {
      const inputCharacters = input.split("");
      const ruleCharacters = rule.split("");
      const rulesLength = ruleCharacters.length;
      for (let i = 0; i < rulesLength; i++) {
        const _input = inputCharacters[i];
        const _rule = ruleCharacters[i];
        const isKnownRule = !!MASK_RULES[_rule];
        if (_input === void 0 && !rule.substr(i).match(HAS_NOT_STRICT_RULE)) {
          return true;
        } else if (isKnownRule) {
          const _doesMatch = !!_input && !!_input.match(MASK_RULES[_rule]);
          if (!_doesMatch) {
            return false;
          }
        } else if (_input !== _rule) {
          return false;
        }
      }
      return true;
    }
  }
  class BehaviorValidationMask2 extends BehaviorValidation(BehaviorBehavior(Base)) {
    connectedCallback() {
      super.connectedCallback();
      this._registerBehavior("BehaviorValidationMask");
    }
    value(input) {
      return this._mask(input, this.rule);
    }
    _mask(input, rule) {
      return Mask.maskFormat(Mask.maskValue(input, rule), rule);
    }
    _maskValidate(input = "", rule = "") {
      return Mask.maskValidate(input, rule);
    }
    _maskValue(input, rule) {
      return Mask.maskValue(input, rule);
    }
    _maskFormat(input, rule) {
      return Mask.maskFormat(input, rule);
    }
  }
  return HIP.factory.mixin(BehaviorValidationMask2);
}
export const BehaviorValidationMask = dedupingMixin(BaseBehaviorValidationMask);
HIP["BehaviorValidationMask"] = BehaviorValidationMask;
