
import { microTask, idlePeriod } from '../../../hip/library/polymer/polymer.js';
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import { Debouncer } from '../../../hip/library/polymer/polymer.js';
import { HIP, AsHIC, BehaviorElement } from '../../../eip/element/hi-hip/hi-hif.js';
import { BehaviorAssignment } from '../../../eip/element/hi-behavior-assignment/hi-behavior-assignment.js';
import { BehaviorImperativeElement } from '../../../eip/element/hi-behavior-imperative/hi-behavior-imperative.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import ess from '../../../pip/library/pi-ess/pi-ess.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class BindingElement extends BehaviorAssignment(
  BehaviorImperativeElement(BehaviorElement(AsHIC(PolymerElement)))
) {
  static get is() {
    return "hi-binding";
  }
  static get properties() {
    return {
      source: {
        type: Object,
        reflectToAttribute: true
      },
      signal: {
        type: String,
        reflectToAttribute: true
      },
      bound: {
        type: Boolean,
        notify: true
      },
      delayed: {
        type: Boolean
      },
      slot: {
        type: String,
        observer: "__slotChanged"
      },
      retries: {
        type: Number,
        readOnly: true,
        value: 5
      },
      injectSource: {
        type: String,
        observer: "__injectSourceChanged"
      },
      injectProperty: {
        type: String,
        observer: "__injectPropertyChanged"
      }
    };
  }
  __slotChanged(_slot) {
    this.setAttribute("field", _slot);
    this.removeAttribute("slot");
  }
  __injectSourceChanged(injectSource) {
    this.removeAttribute("injectSource");
    this.pullSource = injectSource;
  }
  __injectPropertyChanged(injectProperty) {
    this.removeAttribute("injectProperty");
    this.pullField = injectProperty;
  }
  static get observers() {
    return ["__makeBindingWatcher(source, signal)"];
  }
  static get api() {
    return ["refresh", "relink", "unlink", "link"];
  }
  connectedCallback() {
    super.connectedCallback();
    this.__retries = this.retries;
    this.setAttribute("exclude", "exclude");
    this.setAttribute("hidden", "hidden");
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    this.unlink();
    this._unlinkReferences();
  }
  _disabledChanged(disabled) {
    if (this.bound && disabled) {
      this.unlink();
    } else if (this.bound && !disabled) {
      this.safeRelink();
    } else if (!this.bound && !disabled) {
      this.link();
    }
  }
  __makeBindingWatcher(_source) {
    if (!this.delayed && !this.linking && _source) {
      this.relink();
    }
  }
  refresh() {
    if (!this.bound) {
      this._unlink();
      this._link();
    }
  }
  safeRelink() {
    if (!this.bound || !this.isValid()) {
      this.relink();
    }
  }
  isValid() {
    return this._sourceEl && this._sourceEl.parentElement != null;
  }
  relink() {
    const hiReady = window["hiReady"] || function(fn) {
      fn();
    };
    hiReady(() => {
      this._unlink();
      this._link();
    });
  }
  link() {
    this.linking = true;
    return this._link();
  }
  _link() {
    if (this.disabled) {
      this.linking = false;
      return;
    }
    if (!BindingElement.__isBindingEntitiesSpecified(this.source)) {
      this.linking = false;
      return;
    }
    if (this.bound) {
      this.__unlink();
    }
    this.delayed = false;
    this.__debounceLink = Debouncer.debounce(this.__debounceLink, microTask, () => {
      this.__link();
    });
  }
  __link() {
    this._sourceEl = this._lookupNode(this.source);
    if (!this._sourceEl) {
      return this.__attemptRelink(`binding source invalid: ${this.toString()}`);
    }
    const autoBindingSource = this._sourceEl.autoBinding || {};
    this.signal = this.signal || this._sourceEl.getAttribute("override-signal") || autoBindingSource.signal;
    if (!this.signal) {
      return this.__attemptRelink(`binding signal invalid: ${this.toString()}`);
    }
    this._rebind();
    this.linking = false;
    return this;
  }
  __attemptRelink(reason) {
    this.__retries -= 1;
    if (this.__retries > 0) {
      this.__retryRelink = Debouncer.debounce(this.__retryRelink, idlePeriod, () => {
        this.relink();
      });
    } else {
      this.linking = false;
      ess.ele(this, {
        status: "failure",
        action: "relink",
        data: {
          memento: this.memento(),
          reason
        }
      });
    }
    return this;
  }
  __autoWireTarget(element, path) {
    return this[path] || element.autoBinding[path];
  }
  _rebind() {
    this.__unbind();
    this.__bind();
    return this;
  }
  __bind() {
    this._fieldBound = BindingElement.__createSyncBinding(this);
    this._sourceEl.addEventListener(this.signal, this._fieldBound);
    const detailValue = HIP.data.eventAsValueOf(this.signal, this._sourceEl);
    if (detailValue != null) {
      this._fieldBound({
        detail: {
          value: detailValue
        }
      });
    }
    this.bound = true;
    return this;
  }
  __unbind() {
    if (this._sourceEl && this._fieldBound) {
      this._sourceEl.removeEventListener(this.signal, this._fieldBound);
    }
    this._fieldBound = void 0;
    this.bound = false;
    return this;
  }
  static __createSyncBinding(binding) {
    return function(event) {
      const _value = event.detail && event.detail.hasOwnProperty("value") ? event.detail.value : event.detail;
      binding.invoke(_value, event);
    };
  }
  unlink() {
    this.__retries = this.retries;
    return this._unlink();
  }
  _unlink() {
    return this.__unlink();
  }
  __unlink() {
    return this.__unbind();
  }
  static __isBindingComplete(source, signal) {
    return this.__isBindingEntitiesSpecified(source) && signal && signal.length > 0;
  }
  static __isBindingEntitiesSpecified(source) {
    return source && (source instanceof HTMLElement || source.length > 0);
  }
  invoke(value, event = document.createEvent("CustomEvent")) {
    if (this.disabled) {
      return;
    } else if (this.__isInternalPolymerEvent(event)) {
      return;
    }
    this.__debounceInvoke = Debouncer.debounce(this.__debounceInvoke, microTask, () => {
      const promiseRtn = this._invoke(value, event);
      const promise = f1.is.array(promiseRtn) ? Promise.all(promiseRtn) : promiseRtn;
      promise.then((rtn) => {
        this._dispatchProcess("invoke", {
          value,
          event,
          rtn
        });
      }).catch(() => Promise.resolve(void 0));
    });
  }
  __isInternalPolymerEvent(event) {
    return event.detail && (event.detail.path || "").length > 0;
  }
  memento() {
    const _memento = super.memento();
    _memento.signal = this.signal;
    _memento.source = this.source;
    _memento.bound = this.bound;
    return _memento;
  }
}
HIP.fn.bind = function(el, source, signal, sink, field) {
  const binding = document.createElement("hi-binding");
  dom.ensureAttr(el, "gid");
  const selfReference = `#${el.getAttribute("gid")}`;
  binding.source = !source ? selfReference : source;
  binding.signal = signal;
  binding.sink = !sink ? selfReference : sink;
  binding.field = field;
  if (el.ns) {
    binding.setAttribute("ns", el.ns);
  }
  document.body.appendChild(binding);
  return binding;
};
HIP.fn.unbind = function(bindingEl) {
  if (bindingEl) {
    bindingEl.remove();
  }
};
HIP.factory.element(BindingElement);
