
import { afterNextRender } from '../../../hip/library/polymer/polymer.js';
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, HIP, BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
export class CrumbElement extends BehaviorElement(AsHIC(PolymerElement)) {
  static get is() {
    return "hi-crumb";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          display: inline-flex;
          align-items: center;
        }

        :host ::slotted(*) {
          line-height: 1;
        }

        :host(:not([index-last])):after {
          content: '›';
          color: var(--hi-clr--gray-6);
          pointer-events: none !important;
          user-select: none !important;
          font-size: var(--hi-px-24);
          margin: 0 var(--hi-px-8);
          position: relative;
          top: -2px;
        }
      </style>
      <slot id="items"></slot>
    `;
  }
  static get properties() {
    return {
      selected: {
        type: Boolean,
        observer: "__selectedChanged",
        reflectToAttribute: true,
        notify: true
      }
    };
  }
  static get autoBinding() {
    return {
      signal: "selected-changed",
      field: "selected"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "listitem");
  }
  __selectedChanged(selected) {
    afterNextRender(this, function() {
      const aEl = this.children[0];
      if (aEl != null) {
        dom.flipAttr(aEl, "aria-current", selected, "page");
        dom.flipAttr(aEl, "disabled", selected, "disabled");
      }
    });
  }
}
HIP.factory.element(CrumbElement);
export default "hi-crumb";
