
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { BehaviorContainer } from '../../../eip/element/hi-behavior-container/hi-behavior-container.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export const styles = css`
  #container {
    display: flex;
    flex-direction: row;
    align-items: center;
  }

  :host(:where([layout='icon'], [layout='tight'])) ::slotted(*) {
    border-radius: 0;
    margin-right: calc(-1 * var(--hi-px-1));
  }

  :host(:where([layout='icon'], [layout='tight'])) #container ::slotted(hi-button:first-of-type) {
    border-top-left-radius: var(--hi-shape--radius);
    border-bottom-left-radius: var(--hi-shape--radius);
  }

  :host(:where([layout='icon'], [layout='tight'])) #container ::slotted(hi-button:last-of-type) {
    border-top-right-radius: var(--hi-shape--radius);
    border-bottom-right-radius: var(--hi-shape--radius);
    margin-right: 0;
  }

  :host([layout='stacked']) #container,
  :host([layout='inline']) #container {
    gap: var(--hi-px-4);
  }

  :host([layout='stacked']) #container {
    flex-direction: column;
  }
`;
export class ButtonGroupElement extends BehaviorContainer(BehaviorElement(AsHIC(LitElement))) {
  constructor() {
    super();
    this.aesthetic = "secondary";
    this.layout = "tight";
    f1.write("/features/container-index", false, this);
  }
  static get is() {
    return "hi-button-group";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      aesthetic: {
        type: String,
        reflect: true
      },
      layout: {
        type: String,
        reflect: true
      }
    };
  }
  render() {
    return html`
      <div id="container">
        <slot id="items"></slot>
      </div>
    `;
  }
  _onSlotChanged() {
    const isLayoutIcon = this.getAttribute("layout") === "icon";
    const aesthetic = this.aesthetic;
    dom.write(() => {
      this._readSlotChildren().forEach((node) => {
        if (aesthetic !== "none") {
          node.setAttribute("aesthetic", aesthetic);
        }
        if (isLayoutIcon) {
          node.setAttribute("layout", "icon");
        }
      });
    });
  }
}
export default HIP.factory.element(ButtonGroupElement);
