
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import {
  BehaviorA11yKeyboard,
  BehaviorInteractiveElement,
  BehaviorBrowserEvents,
  BehaviorElement,
  HIP,
  AsLitHIC
} from '../../../eip/element/hi-component/hi-component.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export const styles = css`
  :host {
    will-change: color;
    display: inline-flex;
    align-items: center;
    justify-content: center;
    color: var(--hi-clr--theme-5);
    transition: color var(--hi-animation--speed) var(--hi-animation--accel);
    height: var(--hi-close--size, var(--hi-px-32));
    width: var(--hi-close--size, var(--hi-px-32));
    border: var(--hi-px-1) solid var(--hi-clr--transparent);
    border-radius: var(--hi-shape--pill);
  }

  :host #closeSVG {
    height: 90%;
    width: 90%;
  }

  @media screen and (-ms-high-contrast: active) {
    #closeSVG {
      filter: brightness(10) contrast(5);
    }
  }

  /* TODO: FIx cursor not allowed */
  :host([disabled]) {
    pointer-events: none;
    cursor: not-allowed;
    color: var(--hi-clr--disabled);
  }

  @media (hover: hover) {
    :host(:hover) {
      border-color: var(--hi-clr--theme-5);
      box-shadow: var(--hi-clr--halo-1);
      color: var(--hi-clr--theme-3);
    }
  }

  :host(:focus) {
    color: var(--hi-clr--theme-5);
  }

  :host(:hover:active),
  :host(:active),
  :host(:hover[active]),
  :host(:focus:active),
  :host([active]) {
    box-shadow: var(--hi-clr--shadow-inset-2);
  }

  :host([size='small']) {
    --hi-close--size: var(--hi-px-24);
  }
  :host([size='large']) {
    --hi-close--size: var(--hi-px-48);
  }

  /* increase size of close on mobile for accessiblity */
  @media only screen and (max-width: 40rem) {
    :host([device='mobile']) {
      min-height: var(--hi-el--height-mobile) !important;
      min-width: var(--hi-el--width-mobile) !important;
      padding: var(--hi-px-8);
    }

    :host([device='mobile'][size='small']) {
      padding: var(--hi-px-12) !important;
    }

    :host([device='mobile'][size='large']) {
      padding: 0 !important;
    }
  }
`;
export class CloseElement extends BehaviorA11yKeyboard(
  BehaviorBrowserEvents(BehaviorInteractiveElement(BehaviorElement(AsLitHIC(LitElement))))
) {
  static get is() {
    return "hi-close";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      value: {
        type: Object,
        notify: true
      }
    };
  }
  static get api() {
    return ["trigger"];
  }
  static get keyBindings() {
    return {
      "space:keydown": "_onKeyDown__any",
      "enter:keydown": "_onKeyDown__any",
      "space:keyup": "__onKeyUp_space",
      "enter:keyup": "__onKeyUp_enter"
    };
  }
  static get browserBindings() {
    return {
      click: "__onClick"
    };
  }
  static get autoBinding() {
    return {
      signal: "trigger-ended",
      field: "trigger"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "button");
    this.__includeAXLabel();
  }
  render() {
    return html`
      <svg
        id="closeSVG"
        preserveAspectRatio="xMidYMid meet"
        xmlns="http://www.w3.org/2000/svg"
        fill="currentColor"
        aria-hidden="true"
        focusable="false"
        viewBox="0 0 32 32"
        role="img"
      >
        <path
          d="M24 9.4L22.6 8 16 14.6 9.4 8 8 9.4 14.6 16 8 22.6 9.4 24 16 17.4 22.6 24 24 22.6 17.4 16 24 9.4z"
        ></path>
      </svg>
    `;
  }
  trigger() {
    const oneParentEl = f1.compact(
      f1.head(
        this._getParentsOnEvent({
          target: this
        }).filter(this.isCloseable)
      )
    );
    oneParentEl.forEach((closeableParent) => {
      closeableParent.close();
    });
    return f1.is.exists(oneParentEl);
  }
  isCloseable(el) {
    return f1.is.in("close", el) && f1.is.function(el.close);
  }
  __includeAXLabel() {
    const parentEl = this.parentElement;
    const parentDesignation = parentEl && parentEl.localName.includes(`hi-`) ? f1.nth(0, parentEl.localName.split("-").reverse()) : "";
    if (!this.hasAttribute("aria-label")) {
      this.setAttribute("aria-label", parentDesignation ? `Close ${parentDesignation}.` : `Close`);
    }
  }
  _onKeyDown__any(event) {
    event.preventDefault();
    this.setAttribute("active", "active");
  }
  __onKeyUp_enter(event) {
    event.preventDefault();
    this.removeAttribute("active");
    this.trigger();
  }
  __onKeyUp_space(event) {
    event.preventDefault();
    this.removeAttribute("active");
    this.trigger();
  }
  __onClick(event) {
    event.stopPropagation();
    event.preventDefault();
    this.trigger();
  }
}
export default HIP.factory.element(CloseElement);
