
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorContainer } from '../../../eip/element/hi-behavior-container/hi-behavior-container.js';
import { BehaviorCheckedSetGroup } from '../../../eip/element/hi-behavior-group/hi-behavior-checked-set-group.js';
import { BehaviorInput } from '../../../eip/element/hi-behavior-input/hi-behavior-input.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
export class CheckboxGroupElement extends BehaviorCheckedSetGroup(
  BehaviorContainer(BehaviorInput(BehaviorElement(AsHIC(PolymerElement))))
) {
  constructor() {
    super();
    this.multiple = true;
    this.groupName = "checkbox";
    if (this.hasAttribute("values")) {
      this.__initializeValues = true;
    }
    if (this.hasAttribute("selected-indices")) {
      this.__initializeSelectedIndices = true;
    }
  }
  static get is() {
    return "hi-checkbox-group";
  }
  static get properties() {
    return {
      state: {
        type: String,
        reflectToAttribute: true,
        notify: true
      },
      max: {
        type: Number,
        value: -1
      },
      values: {
        type: Array,
        notify: true,
        observer: "__valuesChanged"
      },
      selectedIndices: {
        type: Array,
        notify: true,
        observer: "__selectedIndicesChanged"
      },
      selectedCount: {
        type: Number,
        reflectToAttribute: true,
        notify: true,
        computed: "__selectedCountCompute(selectedIndices)"
      },
      stateIndices: {
        type: Array,
        notify: true,
        computed: "__stateIndicesCompute(selectedIndices)"
      },
      checkedCount: {
        type: Number,
        reflectToAttribute: true,
        notify: true,
        computed: "__checkedCountCompute(stateIndices)"
      },
      name: {
        type: String
      },
      multiple: {
        type: Boolean,
        readOnly: true,
        value: true
      }
    };
  }
  static get api() {
    return ["disableUncheckedItems", "enableItems", "uncheckItems", "checkItems", "toggle"];
  }
  static get autoBinding() {
    return {
      signal: "values-changed",
      field: "values"
    };
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          --hi-checkbox--box-size: calc(var(--hi-px-16) + var(--hi-px-4));
          --hi-checkbox-group--padding: var(--hi-px-12);

          display: block;
        }

        #container ::slotted([x-hip]) {
          margin-bottom: var(--hi-px-4);
          margin-top: var(--hi-px-4);
          vertical-align: top;
        }

        #container ::slotted(hi-label) {
          width: calc(100% - var(--hi-checkbox--box-size) - var(--hi-checkbox-group--padding));
          line-height: var(--hi-checkbox--box-size);
        }

        #container ::slotted(hi-checkbox) {
          width: var(--hi-checkbox--box-size);
        }

        :host([aesthetic='horizontal']) #container {
          display: flex;
        }

        :host([aesthetic='horizontal']) #container ::slotted(hi-label) {
          width: auto;
          margin-right: var(--hi-px-16);
        }
      </style>
      <div id="container">
        <slot id="items"></slot>
      </div>
    `;
  }
  static get observers() {
    return [
      "__stateWatcher(selectedCount, checkedCount, totalCount)",
      "__identificationWatcher(name, group)",
      "__stateReflectionWatcher(state)"
    ];
  }
  get validity() {
    return this._computeValidity({
      valueMissing: this.required && f1.is.empty(this.values)
    });
  }
  disableUncheckedItems() {
    this._group__groupSet().forEach((item) => {
      item.disabled = !item.checked;
    });
  }
  enableItems() {
    this._group__groupSet().forEach((item) => {
      item.disabled = false;
    });
  }
  activate() {
    const children = this._group__groupSet();
    const firstChild = f1.nth(0, children);
    if (firstChild) {
      firstChild.focus();
    }
  }
  toggle(checkedStatus) {
    dom.cs(this, "__stateMutex__", () => {
      const areNoneChecked = this.selectedCount <= 0;
      const isStatusChecked = checkedStatus != null ? !!checkedStatus : areNoneChecked;
      if (isStatusChecked) {
        this.__checkItems();
      } else {
        this.__uncheckItems();
      }
      this.__proxyValuesToGroup();
    });
  }
  checkItems() {
    this.__checkItems();
    this.__proxyValuesToGroup();
  }
  uncheckItems() {
    this.__uncheckItems();
    this.__proxyValuesToGroup();
  }
  _groupChanged(group, groupOld) {
    super._groupChanged(group, groupOld);
    if (this.selectedCount === void 0) {
      this.__proxyValuesToGroup();
      this.__applyEnabledGroupState();
    }
  }
  _group__itemRemoved(...args) {
    super._group__itemRemoved(...args);
    this.__proxyValuesToGroup();
  }
  _group__itemAdded(...args) {
    super._group__itemAdded(...args);
    this.__proxyValuesToGroup();
  }
  _group__itemChanged(hook, el) {
    super._group__itemChanged(hook, el);
    if (this.__initializeValues) {
      return;
    } else if (this.__initializeSelectedIndices) {
      return;
    }
    this.__applyLimitsOnChange(el);
    this.__proxyValuesToGroup();
    this.__applyEnabledGroupState();
    this._dispatchEvent("value-changed", {
      valueFormatted: el.getAttribute("value"),
      value: el.checked,
      srcElement: el
    });
  }
  __stateIndicesCompute() {
    return this._group__groupSet().map((item) => item.state || "unchecked");
  }
  __checkedCountCompute(stateIndices) {
    return stateIndices.filter((state) => state === "checked").length;
  }
  __selectedCountCompute(selectedIndices = []) {
    return selectedIndices.length;
  }
  __stateWatcher(selectedCount, checkedCount, totalCount) {
    if (totalCount === 0) {
    } else if (checkedCount === totalCount) {
      this.state = "checked";
    } else if (selectedCount === 0) {
      this.state = "unchecked";
    } else {
      this.state = "partial";
    }
  }
  __stateReflectionWatcher(state) {
    switch (state) {
      case "checked":
        this.toggle(true);
        break;
      case "unchecked":
        this.toggle(false);
        break;
      default:
        break;
    }
  }
  __identificationWatcher(_name, _group) {
    if ((_name || "").length > 0 && !_group) {
      const formId = this.parentElement ? this.parentElement.getAttribute("id") : `group-${f1.id()}`;
      const checkboxId = _name;
      this.group = `${formId}-${checkboxId}`;
    }
  }
  __onSlotChanged() {
    this._group__slotChange();
  }
  __selectedIndicesChanged(_selectedIndices) {
    if (!CheckboxGroupElement.__mutex__) {
      this.__applySelectedIndices(_selectedIndices);
    }
  }
  __applySelectedIndices(_selectedIndices) {
    this._group__groupSet().forEach((item, index) => {
      item.checked = _selectedIndices.indexOf(index) >= 0;
    });
    this.__applyEnabledGroupState();
  }
  __valuesChanged(_values) {
    if (!CheckboxGroupElement.__mutex__) {
      this.__applyValues(_values);
    }
  }
  __applyValues(_values) {
    this._group__groupSet().forEach((item) => {
      item.checked = _values.indexOf(item.getAttribute(this.valueAttr)) >= 0;
    });
    this.__applyEnabledGroupState();
  }
  __applyLimitsOnChange(el) {
    if (this.max < 0) {
      return;
    }
    const decrementBy = el.checked ? 0 : 1;
    const limitSelection = this.selectedCount - decrementBy >= this.max;
    if (limitSelection) {
      el.checked = false;
    }
  }
  __applyEnabledGroupState() {
    if (this.max < 0) {
      return;
    }
    const limitStatus = this.selectedCount >= this.max;
    if (limitStatus) {
      this.disableUncheckedItems();
    } else {
      this.enableItems();
    }
  }
  __checkItems() {
    this._group__groupSet().forEach((item) => {
      item.checked = true;
    });
  }
  __uncheckItems() {
    this._group__groupSet().forEach((item) => {
      item.checked = false;
    });
  }
  __proxyValuesToGroup() {
    const set = this._group__groupSet();
    const hasNoItems = set.length <= 0;
    if (hasNoItems) {
      return;
    }
    const children = [...this.children].filter((el) => el.localName === "hi-checkbox");
    const hasNotReadAll = children.length > set.length;
    if (hasNotReadAll) {
      return;
    }
    CheckboxGroupElement.__mutex__ = true;
    if (this.__initializeSelectedIndices) {
      this.__applySelectedIndices(this.selectedIndices);
      this.__initializeSelectedIndices = false;
    } else {
      this.selectedIndices = set.map((el, index) => el.hasAttribute("checked") ? index : -1).filter((index) => index >= 0);
    }
    if (this.__initializeValues) {
      this.__applyValues(this.values);
      this.__initializeValues = false;
    } else {
      this.values = set.filter((el) => el.hasAttribute("checked")).map((el) => el.getAttribute(this.valueAttr));
    }
    CheckboxGroupElement.__mutex__ = false;
  }
}
CheckboxGroupElement.__mutex__ = false;
export default HIP.factory.element(CheckboxGroupElement);
