
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { BehaviorInput } from '../../../eip/element/hi-behavior-input/hi-behavior-input.js';
import { BehaviorItemGroup } from '../../../eip/element/hi-behavior-group/hi-behavior-item-group.js';
import { BehaviorMirror } from '../../../eip/element/hi-behavior-mirror/hi-behavior-mirror.js';
import {
  AsHIC,
  BehaviorA11yKeyboard,
  BehaviorInteractiveElement,
  BehaviorBrowserEvents,
  BehaviorCustomizable,
  BehaviorElement,
  HIP
} from '../../../eip/element/hi-component/hi-component.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
export class CheckboxElement extends BehaviorCustomizable(
  BehaviorItemGroup(
    BehaviorA11yKeyboard(
      BehaviorBrowserEvents(
        BehaviorInput(BehaviorMirror(BehaviorInteractiveElement(BehaviorElement(AsHIC(PolymerElement)))))
      )
    )
  )
) {
  constructor() {
    super();
    this.kind = "bi-state";
    this.groupName = "checkbox";
    Object.defineProperty(this, "value", {
      get() {
        return this.indeterminate ? void 0 : this.checked;
      },
      set(_value) {
        if ("bi-state" !== this.kind) {
          this.indeterminate = null == _value;
        }
        if (null == _value) {
          if (this.kind === "bi-state") {
            this.checked = false;
          }
        } else if (f1.is.boolean(_value)) {
          this.checked = _value;
        } else if (f1.is.string(_value)) {
          this.checked = _value.toLowerCase() === "true";
        } else if (f1.is.number(_value)) {
          this.checked = _value > 0;
        } else {
          this.checked = !!_value;
        }
        this.__mutex_property__ = true;
        this.setAttribute("value", _value + "");
        this.__mutex_property__ = false;
        this._dispatchEvent("value-changed", {
          valueFormatted: this.getAttribute("value"),
          value: this.value
        });
      }
    });
  }
  static get is() {
    return "hi-checkbox";
  }
  static get properties() {
    return {
      kind: {
        type: String,
        reflectToAttribute: true
      },
      state: {
        type: String,
        reflectToAttribute: true,
        notify: true
      },
      checked: {
        type: Boolean,
        reflectToAttribute: true,
        notify: true
      },
      indeterminate: {
        type: Boolean,
        reflectToAttribute: true,
        notify: true
      },
      value: {
        type: Object,
        reflectToAttribute: true
      },
      forLabel: {
        type: String
      }
    };
  }
  static get api() {
    return ["toggle", "commit"];
  }
  static get keyBindings() {
    return {
      "space:keypress": "__onKeyPress_space"
    };
  }
  static get browserBindings() {
    return {
      click: "__onClick"
    };
  }
  static get autoBinding() {
    return {
      signal: "checked-changed",
      field: "checked"
    };
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          --hi-checkbox--box-size: calc(var(--hi-px-16) + var(--hi-px-4));
          --hi-checkbox--border-color: var(--hi-clr--gray-7);
        }

        :host {
          display: inline-flex;
          vertical-align: middle;
          cursor: pointer;
          min-width: var(--hi-px-16);
          width: var(--hi-checkbox--box-size);
          height: var(--hi-checkbox--box-size);
          margin-right: var(--hi-px-8);
          border-radius: var(--hi-shape--radius-2);
        }

        :host([size='small']),
        :host([size='large']),
        :host([size='large-x']),
        :host([size='large-xx']) {
          font-size: var(--hi-font-size--large) !important;
        }

        #checkbox {
          background: var(--hi-clr--white);
          border-radius: var(--hi-shape--radius-2);
          border: var(--hi-px-2) solid var(--hi-checkbox--border-color);
          height: var(--hi-checkbox--box-size);
          will-change: box-shadow;
          transition: box-shadow var(--hi-animation--speed) var(--hi-animation--accel);
          width: var(--hi-checkbox--box-size);
        }

        #checkbox,
        #checkmark {
          display: flex;
          align-items: center;
          justify-content: center;
        }

        #checkmark {
          width: 100%;
          height: 100%;
        }

        #checkmark svg {
          width: var(--hi-checkbox--box-size);
          height: var(--hi-checkbox--box-size);
          opacity: 0;
          color: var(--hi-clr--white);
          fill: var(--hi-clr--white);
          transition: opacity var(--hi-animation--speed) var(--hi-animation--accel);
        }

        :host([checked]) #checkmark svg,
        :host([indeterminate]) #checkmark svg {
          opacity: 1;
        }

        :host([theme='white']) #checkbox {
          --hi-checkbox--border-color: var(--hi-clr--gray-7);
        }

        :host([theme='white']) #checkmark svg {
          color: var(--hi-clr--gray-7);
          fill: var(--hi-clr--gray-7);
        }

        :host([checked]) #checkbox {
          background: var(--hi-clr--theme-3);
          --hi-checkbox--border-color: var(--hi-clr--theme-3);
        }

        :host([indeterminate]) #checkbox {
          background: var(--hi-clr--white);
          --hi-checkbox--border-color: var(--hi-clr--theme-3);
        }

        :host(:hover:not([disabled])) #checkbox {
          box-shadow: var(--hi-clr--shadow-down-2);
          --hi-checkbox--border-color: var(--hi-clr--theme-3);
        }

        :host(:active) #toggleThumb {
          box-shadow: none;
        }

        :host([disabled]) {
          cursor: default;
          pointer-events: none !important;
          color: var(--hi-clr--disabled);
          --hi-checkbox--border-color: var(--hi-clr--disabled);
        }

        :host([disabled]) * {
          pointer-events: none !important;
        }

        :host([disabled]) #checkbox {
          color: var(--hi-clr--disabled);
          box-shadow: none;
        }

        :host([disabled]) #checkbox svg {
          color: var(--hi-clr--gray-1);
        }

        #unknown {
          width: 100%;
          height: 100%;
          border-radius: calc(0.5 * var(--hi-px-4));
          background: linear-gradient(
            to bottom left,
            var(--hi-clr--white) 0%,
            var(--hi-clr--white) 50%,
            var(--hi-checkbox--border-color) 50%,
            var(--hi-checkbox--border-color) 100%
          );
        }

        #partial {
          width: calc(0.5 * var(--hi-checkbox--box-size));
          height: calc(0.5 * var(--hi-checkbox--box-size));
          background-color: var(--hi-clr--theme-3);
          border-radius: calc(0.5 * var(--hi-px-4));
        }

        #unknown,
        #partial,
        #determinate {
          display: none;
        }

        :host(:not([indeterminate])) #determinate {
          display: block;
        }

        :host([indeterminate][kind='group']) #partial {
          display: block;
        }

        :host([indeterminate][kind='tri-state']) #checkbox {
          background: var(--hi-clr--theme-1);
        }

        :host([indeterminate][kind='tri-state']) #unknown {
          display: block;
        }
      </style>
      <div id="checkbox">
        <div id="checkmark">
          <svg id="determinate" width="512" height="512" viewBox="0 0 512 512" aria-hidden="true">
            <path d="m186 340l-90-91-32 31 122 122 262-261-32-31z"></path>
          </svg>
          <div id="partial"></div>
          <div id="unknown"></div>
        </div>
      </div>
    `;
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "checkbox");
  }
  static get observedAttributes() {
    return [...super["observedAttributes"], "value"];
  }
  static get observers() {
    return [
      "__stateReflectionWatcher(state, kind)",
      "__stateWatcher(checked, indeterminate)",
      "__identificationWatcher(name, group)",
      "__groupWatcher(checked, group)"
    ];
  }
  static get valueEvent() {
    return "checked-changed";
  }
  get inputMirrorType() {
    return "checkbox";
  }
  get validity() {
    return this._computeValidity({
      valueMissing: this.kind !== "tri-state" ? false : this.required && this.value === void 0
    });
  }
  attributeChangedCallback(name, _0, value) {
    if (name !== "value") {
      super.attributeChangedCallback(name, _0, value, null);
    } else if (name === "value" && !this.__mutex_property__ && _0 !== value) {
      this.value = value;
    }
  }
  clear() {
    super.clear();
    if (this.kind === "bi-state") {
      this.toggle();
    }
  }
  toggle(toState) {
    dom.cs(this, "__stateMutex__", () => {
      if (!this.disabled) {
        if (toState == null) {
          toState = this.indeterminate === true ? false : !this.checked;
        }
        if (this.kind === "tri-state") {
          this.indeterminate = this.checked && !this.indeterminate === true ? true : false;
        } else {
          this.indeterminate = false;
        }
        this.checked = toState;
      }
    });
    this.validate();
  }
  toString() {
    return `${super.toString()} ${this.getAttribute("value") || ""}`;
  }
  __identificationWatcher(_name, _group) {
    if ((_name || "").length > 0 && !_group) {
      if (this.parentElement && this.parentElement.nodeName === this.nodeName + "-GROUP") {
        const parent = this.parentElement;
        this.group = parent.group;
      } else {
        const checkboxId = _name;
        const formId = this.parentElement ? this.parentElement.getAttribute("id") : `group-${f1.id()}`;
        this.group = `${formId}-${checkboxId}`;
      }
    }
  }
  __onKeyPress_space(event) {
    event.preventDefault();
    this.__humanToggle();
  }
  __onClick(event) {
    event.stopPropagation();
    this.__humanToggle();
  }
  __humanToggle(toState) {
    if (this.readonly) {
      return;
    }
    this.toggle(toState);
    this.commit();
  }
  __stateReflectionWatcher(state, kind) {
    dom.cs(this, "__stateMutex__", () => {
      if (kind === "bi-state") {
        this.indeterminate = false;
      }
      switch (state) {
        case "checked":
          this.indeterminate = false;
          this.checked = true;
          break;
        case "unchecked":
          this.indeterminate = false;
          this.checked = false;
          break;
        case "unknown":
        case "partial":
          this.indeterminate = kind !== "bi-state";
          this.checked = kind !== "bi-state";
          break;
        default:
          break;
      }
    });
  }
  __stateWatcher(_checked, _indeterminate) {
    const ariaCheckedEnum = _indeterminate ? "mixed" : !!_checked + "";
    const _kind = this.kind;
    this.state = CheckboxElement.__toState(_kind, _checked, _indeterminate);
    this.setAttribute("aria-checked", ariaCheckedEnum);
    this.__writeToLabel();
  }
  __writeToLabel() {
    if (this.forLabel) {
      const forLabelEl = HIP.browser.queryOne(this.forLabel, this.ns);
      if (forLabelEl) {
        forLabelEl.textContent = CheckboxElement.__toLabel(this.state);
      }
    }
  }
  static __toState(_kind, _checked, _indeterminate) {
    switch (_kind) {
      case "bi-state":
        return _checked ? "checked" : "unchecked";
      case "group":
      case "tri-state":
        if (_indeterminate && _checked) {
          return _kind === "group" ? "partial" : "unknown";
        } else if (!_indeterminate && _checked) {
          return "checked";
        } else if (!_indeterminate && !_checked) {
          return "unchecked";
        }
    }
    return "unknown";
  }
  static __toLabel(state) {
    switch (state) {
      case "unchecked":
        return "Check all";
      default:
        return "Uncheck selected";
    }
  }
  __groupWatcher() {
    this._group__stateChanged();
  }
}
export default HIP.factory.element(CheckboxElement);
