
import { stylesTagBadge } from '../../../eip/element/hi-behavior-style/hi-behavior-style.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { AsLitHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import { LitElement, html, css } from '../../../hip/library/lit-element/lit-element.js';
import '../../../eip/element/hi-badge/hi-badge.js';
import '../../../eip/element/hi-icon/hi-icon.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export const styles = css`
  :host {
    border-radius: var(--hi-shape--radius-2);
  }

  :host([allow-ellipsis]) {
    overflow: hidden;
    text-overflow: ellipsis;
    white-space: nowrap;
  }
`;
const _CheckmarkElement = class extends BehaviorElement(AsLitHIC(LitElement)) {
  constructor() {
    super(...arguments);
    this.kind = "bi-state";
  }
  static get is() {
    return "hi-checkmark";
  }
  static get styles() {
    return [stylesTheme, stylesTagBadge, styles];
  }
  static get properties() {
    return {
      kind: {
        type: String
      },
      value: {
        type: Object
      },
      caseTrue: {
        type: String,
        attribute: "case-true"
      },
      caseFalse: {
        type: String,
        attribute: "case-false"
      },
      caseDefault: {
        type: String,
        attribute: "case-default"
      },
      indeterminate: {
        type: Boolean,
        reflect: true
      },
      checked: {
        type: Boolean,
        reflect: true
      },
      disallowHint: {
        type: Boolean,
        attribute: "disallow-hint"
      }
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "presentation");
  }
  update(props) {
    const hasAssignedValue = props.has("value");
    if (!hasAssignedValue && !props.has("checked")) {
      this.value = null;
    }
    if (hasAssignedValue || props.has("kind")) {
      this.__applyValueToChecked(this.value, this.kind);
    }
    return super.update(props);
  }
  render() {
    const ns = this.ns;
    const nullable = this.kind === "tri-state";
    const val = nullable && this.indeterminate ? "null" : !!this.checked;
    const rule = {
      nullable,
      case_true: this.caseTrue,
      case_false: this.caseFalse,
      case_default: this.caseDefault
    };
    return html`<hi-text
      allow-ellipsis
      size="small"
      formatter="boolean"
      .formatterRule=${rule}
      .value=${val}
      .ns=${ns}
      ?disallow-hint=${this.disallowHint}
    ></hi-text>`;
  }
  __applyValueToChecked(value, kind) {
    if ("bi-state" !== kind) {
      this.indeterminate = null == value;
    }
    if (null == value) {
      if (kind === "bi-state") {
        this.checked = false;
      }
    } else if (f1.is.boolean(value)) {
      this.checked = value;
    } else if (f1.is.string(value)) {
      this.checked = value.toLowerCase() === "true";
    } else if (f1.is.number(value)) {
      this.checked = value > 0;
    } else {
      this.checked = !!value;
    }
  }
  __computeState(checked, indeterminate) {
    const value = this.__computeValue(checked, indeterminate) + "";
    const userStrategy = _CheckmarkElement.defaultValues.user;
    const userValue = userStrategy(value);
    const userState = this[userValue];
    if (userState) {
      return userState;
    }
    const valueStrategy = _CheckmarkElement.defaultValues.text;
    const valueState = valueStrategy(value);
    if (valueState) {
      return valueState;
    }
    return value;
  }
  __computeValue(checked, indeterminate) {
    return indeterminate ? null : !!checked;
  }
  __computeAriaLabel(checked, indeterminate) {
    return f1.is.boolean(checked) && !indeterminate ? `value is ${checked}` : "value is unknown";
  }
};
export let CheckmarkElement = _CheckmarkElement;
CheckmarkElement.defaultValues = {
  text: f1.strategy({
    true: "Yes",
    false: "No",
    fallback: "- -"
  }),
  user: f1.strategy({
    true: "caseTrue",
    false: "caseFalse",
    fallback: "caseDefault"
  })
};
export default HIP.factory.element(CheckmarkElement);
