
import { LitElement } from '../../../hip/library/lit-element/lit-element.js';
import { AsLitHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorInput } from '../../../eip/element/hi-behavior-input/hi-behavior-input.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import $ip from '../../../pip/library/pi-ip/pi-ip.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
const loadCodeMirror = f1.once(() => {
  return $ip.requires(["css", "js"].map((ext) => `${$ip.configuration.host}/hip/library/codemirror/codemirror.${ext}`));
});
export class CodeElement extends BehaviorInput(BehaviorElement(AsLitHIC(LitElement))) {
  constructor() {
    super(...arguments);
    this.__valueMutex = false;
  }
  static get is() {
    return "hi-code";
  }
  static get properties() {
    return {
      language: {
        type: String
      },
      value: {
        type: String,
        notify: true
      }
    };
  }
  static get valueEvent() {
    return "value-changed";
  }
  static get autoBinding() {
    return {
      signal: "commit-ended",
      field: "value"
    };
  }
  createRenderRoot() {
    return this;
  }
  connectedCallback() {
    super.connectedCallback();
    dom.setAttrs(this, {
      role: "presentation"
    });
    this.style.overflow = "auto";
    this.style.display = "block";
    this.style.padding = "1px";
    this.style.borderRadius = "var(--hi-shape--radius)";
    this.style.border = "var(--hi-px-1) solid var(--hi-clr--theme-3)";
    this.__loadCodeMirror();
  }
  update(props) {
    super.update(props);
    this._watchChanges(props, [
      {
        properties: ["value"],
        method: "__valueWatcher"
      }
    ]);
  }
  __loadCodeMirror() {
    loadCodeMirror().then(() => {
      const codeMirror = this.__create__codeMirror();
      setTimeout(() => {
        this.__setup__codeMirror();
        codeMirror.refresh();
        this.writeModel();
      });
      codeMirror.on(
        "changes",
        (instance, changes) => this.__codeMirror__onChanges(instance, changes)
      );
      codeMirror.on(
        "keyHandled",
        (instance, name, event) => this.__codeMirror__onKey(instance, name, event)
      );
      codeMirror.on("blur", (e) => this.__codeMirror__onBlur(e));
    });
  }
  __create__codeMirror() {
    return this.instance = window.CodeMirror(this, {
      mode: f1.safe("json", this.language),
      value: f1.safe("", this.value),
      readOnly: this.disabled,
      lineNumbers: true
    });
  }
  __setup__codeMirror() {
    const codeMirror = this.instance;
    const codeMirrorDOM = this.children[0];
    codeMirror.setSize(null, "auto");
    codeMirrorDOM.style.height = "100%";
  }
  __reflectValue() {
    this.__valueMutex = true;
    this.value = this.readModel();
  }
  __codeMirror__onBlur(e) {
    this.__reflectValue();
    this.commit();
  }
  __codeMirror__onKey(instance, name, event) {
    this.__reflectValue();
    this._dispatchProcess(`key-${name.toLowerCase()}`, {
      instance,
      event
    });
  }
  __codeMirror__onChanges(instance, changes) {
    this._dispatchProcess(`changes`, {
      instance,
      changes
    });
  }
  __valueWatcher() {
    this.writeModel();
  }
  focus() {
    this.instance.focus();
  }
  redo() {
    this.instance.redo();
  }
  undo() {
    this.instance.undo();
  }
  readModel() {
    return this.instance.getValue();
  }
  writeModel(value = this.value) {
    if (this.__valueMutex) {
      this.__valueMutex = false;
      return;
    }
    value = f1.cast.string(f1.safe("", value));
    if (this.instance) {
      this.instance.setValue(value);
    }
  }
}
export default HIP.factory.element(CodeElement);
