
import { css, LitElement } from '../../../hip/library/lit-element/lit-element.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { AsLitHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
export const styles = css`
  :host {
    --color: currentColor;
  }

  :host {
    display: inline-block;
    line-height: 1;
    font-size: 1em;
    height: 1em;
    width: 1em;
    color: var(--color);
    background-color: var(--color);
    border: var(--hi-px-1) solid var(--color);
    border-radius: var(--hi-shape--radius-2);
  }

  :host([size='small']) {
    font-size: var(--hi-px-13) !important;
  }

  :host([size='large-x']) {
    font-size: var(--hi-px-24) !important;
  }
`;
const _ColorElement = class extends BehaviorElement(AsLitHIC(LitElement)) {
  static get is() {
    return "hi-color";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      value: {
        type: String
      }
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "img");
  }
  update(props) {
    super.update(props);
    if (props.has("value")) {
      const value = this.convertColor(this.value);
      this.style.setProperty("--color", value);
    }
  }
  convertColor(value) {
    return _ColorElement.colors.includes(value) ? `var(--hi-clr--${value}-3)` : value;
  }
};
export let ColorElement = _ColorElement;
ColorElement.colors = ["red", "orange", "yellow", "green", "teal", "blue", "cyan", "purple", "magenta", "gray", "black"];
export default HIP.factory.element(ColorElement);
