
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorA11yKeyboard, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorContainer } from '../../../eip/element/hi-behavior-container/hi-behavior-container.js';
import { BehaviorValidated } from '../../../eip/element/hi-behavior-validation/hi-behavior-validated.js';
import { BehaviorMirror } from '../../../eip/element/hi-behavior-mirror/hi-behavior-mirror.js';
import { BehaviorInput } from '../../../eip/element/hi-behavior-input/hi-behavior-input.js';
import { BehaviorForm } from '../../../eip/element/hi-behavior-form/hi-behavior-form.js';
import {
  BehaviorAutoComplete
} from '../../../eip/element/hi-behavior-auto-complete/hi-behavior-auto-complete.js';
import '../../../eip/element/hi-dropdown/hi-dropdown.js';
import '../../../eip/element/hi-select/hi-select.js';
import '../../../eip/element/hi-select/hi-option.js';
import '../../../eip/element/hi-tag/hi-tag-group.js';
import '../../../eip/element/hi-button/hi-close.js';
import '../../../eip/element/hi-input/hi-input.js';
import '../../../eip/element/hi-tag/hi-tag.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class ComboboxElement extends BehaviorValidated(
  BehaviorAutoComplete(
    BehaviorInput(
      BehaviorMirror(BehaviorContainer(BehaviorA11yKeyboard(BehaviorForm(BehaviorElement(AsHIC(PolymerElement))))))
    )
  )
) {
  constructor() {
    super(...arguments);
    this.allowTileWrap = false;
    this.persistSearch = false;
    this.selections = [];
    this.selectedCount = 0;
    this.__db =  new Map();
    this.__updateValue__mutex = false;
    this.__initialCount = 100;
  }
  static get is() {
    return "hi-combobox";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          --hi-combobox-select-height: 15rem;
          --hi-combobox-popup-select-height: var(--hi-combobox-select-height);
          --hi-combobox-input--height: calc(var(--hi-px-32) + var(--hi-px-8));
          --hi-combobox--placeholder-color: var(--hi-clr--gray-7);

          min-width: calc(var(--hi-px-32) * 4);
          display: block;
        }

        #valueContainer {
          display: flex;
          flex-wrap: nowrap;
          align-items: center;
          flex-direction: row;
        }

        [x-style='combobox-tag'] {
          margin: var(--hi-px-4);
        }

        #dropdown {
          --hi-dropdown--height: auto;
          --hi-dropdown--color-bkg-focus: var(--hi-clr--gray-1);
          width: auto;
          height: auto;
        }

        #popupElement {
          max-height: var(--hi-combobox-select-height);
        }

        #tagGroup {
          overflow-x: auto;
          flex: 0 1 auto;
          max-width: 100%;
          width: auto;
        }

        #searchInput {
          font-family: inherit;
          font-weight: var(--hi-font--weight-light);
          font-size: var(--hi-font-size--text);
          border: 0;
          background-color: var(--hi-clr--transparent);
          color: var(--hi-clr--gray-9);
          padding: var(--hi-px-4);
          margin-left: var(--hi-px-4);
          transition: width 0.25s ease-out 0.05s;
          will-change: width;
          flex-grow: 1;
          width: 8ch;
          font-weight: var(--hi-font--weight-light);
          text-rendering: optimizeLegibility;
          font-family: var(--hi-font--family-content);
          font-size: var(--hi-font-size--text);
          line-height: var(--hi-font-lh--text);
        }

        :host([multiple]) #searchInput {
          border: var(--hi-px-1) dashed var(--hi-clr--gray-8);
        }

        #searchInput:focus {
          outline: none;
        }

        :host([opened]) #searchInput:focus {
          width: 8ch;
        }

        :host([selected-count='0']) #searchInput,
        :host([selected-count='0']) #searchInput:focus {
          width: 100%;
        }

        #statusMessage,
        #resultMessage {
          font-size: var(--hi-font-size--text);
          font-weight: var(--hi-font--weight-heavy);
          background: var(--hi-clr--white);
          color: var(--hi-clr--gray-9);
          padding: var(--hi-px-8);
          width: 100%;
          z-index: 3;
        }

        #resultMessage {
          border-bottom: var(--hi-px-1) solid var(--hi-clr--gray-7);
        }

        #statusMessage:empty,
        #resultMessage:empty {
          display: none;
        }

        :host([hidden-result-message]) #resultMessage {
          display: none;
        }

        :host([hidden-status-message]) #statusMessage {
          display: none;
        }

        :host([disallow-quick-selections]) #tagGroup {
          display: none;
        }

        .option-separator-heading {
          font-size: var(--hi-font-size--text);
          font-weight: var(--hi-font--weight-bold);
          color: var(--hi-clr--theme-3);
          border-top: var(--hi-px-1) solid var(--hi-clr--theme-1);
          padding: var(--hi-px-8);
        }

        .option-separator-heading:not([hidden]) ~ hi-option:not([hidden]) {
          padding-left: var(--hi-px-16);
        }

        :host([readonly]),
        :host([readonly]) * {
          opacity: 1;
        }

        #clear {
          flex-shrink: 0;
        }

        :host([readonly]) #clear,
        :host([disabled]) #clear {
          display: none;
        }

        :host([disabled]) #searchInput {
          border: var(--hi-px-1) solid var(--hi-clr--gray-3);
        }

        :host([multiple][allow-tile-wrap]) #searchInput,
        :host([multiple][allow-tile-wrap]) #searchInput:focus,
        :host([multiple][allow-tile-wrap]) #searchInput {
          border: var(--hi-px-1) dashed var(--hi-clr--gray-8);
          width: 100%;
        }

        :host([multiple][allow-tile-wrap]:not([selected-count='0'])) #searchInput,
        :host([multiple][allow-tile-wrap]:not([selected-count='0'])) #searchInput:focus,
        :host([multiple][allow-tile-wrap]:not([selected-count='0'])) #searchInput {
          margin-bottom: var(--hi-px-4);
        }

        :host([multiple][allow-tile-wrap]) #valueContainer {
          align-items: flex-start;
          flex-direction: column;
        }

        :host([multiple][allow-tile-wrap]) #clear {
          display: none;
        }

        :host([multiple][allow-tile-wrap]) #tagGroup {
          align-self: flex-start;
        }
      </style>

      <hi-dropdown
        id="dropdown"
        mode="input"
        kind="sticky"
        manual
        cap-width
        flush-width
        disabled-focusable
        disallow-aria-role
        disallow-responsiveness
        theme$="[[ theme ]]"
        aesthetic$="[[ aesthetic ]]"
        disabled$="[[ disabled ]]"
        readonly$="[[ readonly ]]"
      >
        <div id="valueContainer" slot="anchor">
          <hi-tag-group id="tagGroup" disabled$="[[ disabled ]]" readonly$="[[ readonly ]]" hidden>
            <dom-repeat items="{{ selections }}" initial-count="[[ __initialCount ]]">
              <template>
                <hi-tag
                  x-style="combobox-tag"
                  disallow-text-wrap$="[[ !allowTileWrap ]]"
                  ns$="[[ ns ]]"
                  theme$="[[ theme ]]"
                  value="[[ item.value ]]"
                  disabled$="[[ disabled ]]"
                  readonly$="[[ readonly ]]"
                  allow-close="true"
                  on-close-ended="__onCloseEnded__tag"
                  data-qa$="[[ item.title ]]"
                >
                  [[ item.title ]]
                </hi-tag>
              </template>
            </dom-repeat>
          </hi-tag-group>
          <input
            id="searchInput"
            mixin="input-placeholder"
            type="text"
            role="combobox"
            autocomplete="off"
            aesthetic="minimal"
            aria-autocomplete="none"
            aria-expanded="false"
            aria-haspopup="listbox"
            aria-controls="popupElement"
            disabled$="[[ disabled ]]"
            readonly$="[[ readonly ]]"
            nullable$="[[ nullable ]]"
            blankable$="[[ blankable ]]"
            placeholder$="[[ placeholder ]]"
            aria-activedescendant$="[[ activeDescendant ]]"
            disallowed-shadow-active-element
          />
          <hi-close
            id="clear"
            size="small"
            theme$="[[ theme ]]"
            disabled$="[[ disabled ]]"
            readonly$="[[ readonly ]]"
            aria-label="Clear all selections"
            on-trigger-ended="__onTriggerEnded__clearAll"
          ></hi-close>
        </div>
        <p id="resultMessage" slot="status" aria-live="polite" aria-atomic="true">[[ __resultsHeaderText ]]</p>
        <hi-select
          id="popupElement"
          mode="value"
          tabindex="-1"
          role="listbox"
          allow-remote-focus
          ns$="[[ ns ]]"
          value="[[ value ]]"
          values="[[ values ]]"
          theme$="[[ theme ]]"
          readonly$="[[readonly]]"
          required$="[[ required ]]"
          disabled$="[[ disabled ]]"
          multiple$="[[ multiple ]]"
          hidden$="[[ _hasEmptyInput(_listResults) ]]"
          aria-activedescendant$="[[ activeDescendant ]]"
          on-focus-index-ended="_onFocusIndexEnded__popupElement"
          on-deselected-ended="__onDeselectedEnded__popupElement"
          on-selected-ended="__onSelectedEnded__popupElement"
          on-commit-ended="__onCommitEnded__popupElement"
        >
          <dom-repeat items="{{ _listResults }}" initial-count="[[ __initialCount ]]">
            <template>
              <hi-option
                ns$="[[ ns ]]"
                theme$="[[ theme ]]"
                exclude$="[[  item.heading ]]"
                hidden$="[[ item.heading ]]"
                readonly$="[[readonly]]"
                disabled$="[[ disabled ]]"
                value$="[[ item.value ]]"
                selected="[[ item.selected ]]"
                aria-label="[[ item.title ]]"
                disallow-responsiveness
              >
                [[ item.title ]]
              </hi-option>
              <div exclude hidden$="[[ !item.heading ]]" role="separator" class="option-separator-heading">
                [[ item.title ]]
              </div>
            </template>
          </dom-repeat>
        </hi-select>
        <div id="statusMessage" aria-live="polite" aria-atomic="true" hidden$="[[ !_hasEmptyInput(_listResults) ]]">
          <slot name="empty-message">
            <p>No results available.</p>
          </slot>
        </div>
      </hi-dropdown>
    `;
  }
  static get properties() {
    return {
      aesthetic: {
        type: String
      },
      multiple: {
        type: Boolean,
        reflectToAttribute: true
      },
      values: {
        type: Array,
        notify: true
      },
      max: {
        type: Number
      },
      persistSearch: {
        type: Boolean
      },
      selectedCount: {
        type: Number,
        reflectToAttribute: true
      },
      selections: {
        type: Array
      },
      initialSelections: {
        type: Object
      },
      disallowQuickSelections: {
        type: Boolean,
        reflectToAttribute: true
      },
      allowTileWrap: {
        type: Boolean,
        reflectToAttribute: true
      }
    };
  }
  static get api() {
    return ["clearAll", "commit"];
  }
  static get observers() {
    return [
      "__valueSingleWatcher(value, multiple)",
      "__valueWatcher(values, value, input)",
      "__initialSelectionsWatcher(initialSelections)"
    ];
  }
  get valueProperty() {
    return this.multiple ? "values" : "value";
  }
  get valueEvent() {
    return this.multiple ? "values-changed" : "value-changed";
  }
  get autoBinding() {
    return {
      field: this.valueProperty,
      signal: this.valueEvent
    };
  }
  get inputMirrorType() {
    return "select";
  }
  connectedCallback() {
    super.connectedCallback();
    this.__reflectSelections();
    this.disallowEnter = true;
    if (!this.multiple) {
      this._assignSearchInput();
    }
    this.$.tagGroup.addEventListener("values-changed", () => this.$.dropdown.alignFlyout());
  }
  __valueSingleWatcher(value, multiple) {
    if (!multiple && value) {
      this.evaluate();
    }
  }
  __valueWatcher(_values, _value, input) {
    if (f1.is.nil(_values) && f1.is.nil(_value)) {
      this.__reflectEmptyValues(_values, _value, input);
    } else {
      this.__reflectValues(_values, _value, input);
    }
    if (this.multiple && f1.is.array(_values)) {
      this.value = f1.nth(0, f1.last(_values));
    }
  }
  __initialSelectionsWatcher(_initialSelections) {
    const initialSelections = f1.is.array(_initialSelections) ? _initialSelections : [_initialSelections];
    this.__stashItems(initialSelections);
    this.selections = initialSelections;
    this.values = f1.safe([], initialSelections).map((_) => _.value);
    this.value = f1.nth(0, this.values);
  }
  _inputWatcher(input) {
    super._inputWatcher(input);
    this.__stashItems(input);
  }
  __stashItems(input) {
    const db = this.__db;
    f1.each(
      (v) => {
        const key = f1.read("/value", v.value, v.value);
        db.set(`${key}`, v);
      },
      f1.remove((_) => !!_.heading, this._getNormalizedInput(input))
    );
  }
  __reflectEmptyValues(_values, _value, input) {
    this.$.popupElement.reset();
    this.__reflectSelections([]);
  }
  __reflectValues(_values, _value, input) {
    if ((!_values || this.multiple) && Array.isArray(_value)) {
      this.values = _values = _value;
    }
    this.__stashItems(input);
    this.__reflectSelections(this.__computeSelections(_values, _value));
    this.$.dropdown.alignFlyout();
  }
  __reflectSelections(selections = this.selections) {
    const hasNoSelectedResults = !selections || selections && selections.length <= 0;
    this.$.clear.hidden = hasNoSelectedResults;
    this.$.tagGroup.hidden = this.multiple ? hasNoSelectedResults : true;
    this.selectedCount = selections.length || 0;
    this.selections = selections;
    if (!f1.is.empty(this.selections)) {
      this._reflectSearchInput();
    }
  }
  _reflectSearchInput() {
    if (this.shouldPersistSearch()) {
      return;
    }
    if (!this.multiple) {
      this._assignSearchInput();
    }
  }
  _shouldPassThruResultList(searchString) {
    return super._shouldPassThruResultList(searchString) || !this.multiple && f1.is.similar(searchString, `${this.value}`);
  }
  __compareValues(left, right) {
    const _right = this.__getCompareValue(right);
    const _left = this.__getCompareValue(left);
    return f1.is.similar(_left, _right);
  }
  __getCompareValue(val) {
    return f1.is.object(val) ? f1.read("/value", `${val}`, val) : `${val}`;
  }
  _onKeyUp_enterKey__input() {
    super._onKeyUp_enterKey__input();
    this.$.popupElement.toggleFocusedIndex();
    if (!this.multiple) {
      this.close();
    }
    if (this.multiple && !this.persistSearch) {
      this._clearSearchInput();
    }
  }
  _onKeyUp_escKey__input() {
    super._onKeyUp_escKey__input();
    if (this["multiple"]) {
      this._clearSearchInput();
    }
    this.close();
  }
  _onKeyUp_backspaceKey__input() {
    super._onKeyUp_backspaceKey__input();
    const lastTagEl = this.__lastTag();
    if (this._searchActionMutex <= 0 && !this.multiple) {
      this.clearAll();
    }
    if (lastTagEl == null) {
      return;
    }
    if (this._searchActionMutex < 0) {
      lastTagEl.close();
      return;
    }
    if (this._searchActionMutex <= 0) {
      this._searchActionMutex -= 1;
    }
  }
  __lastTag() {
    const lastTagsEl = this.$.tagGroup.slotChildren;
    return lastTagsEl[lastTagsEl.length - 1] || null;
  }
  __onCloseEnded__tag(e) {
    if (this.__updateValue__mutex) {
      return;
    }
    const value = e.currentTarget["value"];
    this.__updateAppropriateValue(this.__excludeValue(value));
    this.commit();
    this.$.searchInput.focus();
    HIP.fn.announce(`${e.currentTarget.textContent} deselected`, "assertive");
  }
  __onSelectedEnded__popupElement(e) {
    if (this.__updateValue__mutex) {
      return;
    }
    this.__updateAppropriateValue(this.__includeValue(e.detail.value));
  }
  __onDeselectedEnded__popupElement(e) {
    if (this.__updateValue__mutex) {
      return;
    }
    this.__updateAppropriateValue(this.__excludeValue(e.detail.value));
  }
  __onTriggerEnded__clearAll(e) {
    e.stopPropagation();
    this.clearInput();
    this.clearAll();
    this.focus();
  }
  __includeValue(valueToInclude) {
    const removeItemInValues = this.__cleanValues(valueToInclude);
    const values = f1.push(valueToInclude, removeItemInValues);
    const value = valueToInclude;
    return {
      values,
      value
    };
  }
  __excludeValue(valueToExclude) {
    const removeItemInValues = this.__cleanValues(valueToExclude);
    const values = removeItemInValues;
    const value = f1.nth(0, values);
    return {
      values,
      value
    };
  }
  __cleanValues(targetValue) {
    return f1.remove((value) => this.__compareValues(value, targetValue), f1.copy(f1.safe([], this.values)));
  }
  __updateAppropriateValue({ values, value }) {
    this.__updateValue__mutex = true;
    this.value = value;
    if (this.multiple) {
      this.values = values;
    }
    if (this.multiple && !this.persistSearch) {
      this._clearSearchInput();
    }
    this.__updateValue__mutex = false;
  }
  __computeSelections(_values, _value) {
    const __values = this.multiple ? f1.is.array(_values) ? _values : f1.compact([_values]) : f1.compact([_value]);
    const values = __values.map((_) => f1.read("/value", _, _));
    const searchSpace = this.__db;
    const valuesInList = f1.unique(f1.compact(values.map((value) => searchSpace.get(`${value}`))));
    return this.multiple ? valuesInList : valuesInList.slice(0, 1);
  }
  __onCommitEnded__popupElement(e) {
    e.stopPropagation();
    this.commit();
  }
  clearAll() {
    this._computeResultsList(this.input, "");
    this.$.popupElement.deselectSelected();
    this.value = void 0;
    if (this.multiple) {
      this.values = [];
    }
    this.__reflectSelections([]);
    this.commit();
  }
  commit() {
    return {
      value: this.multiple ? this.values : this.value
    };
  }
  shouldPersistSearch() {
    return this.multiple && (this.kind === "dynamic" || this.persistSearch);
  }
  get validity() {
    const val = this.multiple ? this.values : this.value;
    const valueMissing = this.required && f1.is.empty(val);
    return this._computeValidity({
      valueMissing
    });
  }
}
export default HIP.factory.element(ComboboxElement);
