
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { unsafeHTML } from '../../../hip/library/lit-html/directives/unsafe-html.js';
import { HIP, AsLitHIC, BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
const R = {
  detectLinkTag: /<a(.*?)>(.*?)<\/a>/gi
};
export const reset = css`
  :host {
    all: initial;
  }
`;
export const styles = css`
  :host {
    --sz-3: 1em;
    --sz-4: 1.2em;
    --sz-5: 1.33em;
    --sz-6: 1.77em;
    --sz-7: 2.36em;
    --sz-8: 3.15em;
    --sz-9: 4.2em;

    line-height: 1.4;
  }

  a:not([class]) {
    color: currentColor;
  }

  a {
    outline: 0 !important;
    cursor: pointer !important;
    text-decoration: underline !important;
    color: var(--hi-clr--link-text) !important;
  }

  a:hover {
    text-decoration: none !important;
  }

  :focus {
    outline: var(--hi-outline--width) var(--hi-outline--line) var(--hi-clr--outline) !important;
    outline-offset: var(--hi-outline--offset) !important;
  }

  h1,
  h2,
  h3,
  h4 {
    line-height: 1.1;
    font-weight: 900;
    font-family: -apple-system, avenir, helvetica, arial, sans-serif;
  }

  h1 {
    font-size: var(--sz-8);
  }

  h2 {
    font-size: var(--sz-7);
  }

  h3 {
    font-size: var(--sz-6);
  }

  h4 {
    font-size: var(--sz-5);
  }

  h5 {
    font-size: var(--sz-4);
  }

  h6 {
    font-size: var(--sz-3);
  }

  img {
    max-width: 50em;
  }

  a,
  p,
  li,
  dd,
  dt {
    font-weight: var(--hi-font--weight-normal);
    font-size: var(--sz-3);
    max-width: 60ch;
  }

  figcaption {
    font-size: var(--sz-4);
    font-style: italic;
    margin-top: 1em;
  }

  blockquote {
    padding-left: var(--sz-5);
    border-left: var(--hi-px-4) solid;
    font-style: italic;
    font-size: var(--sz-4);
    max-width: 40ch;
  }

  ::selection {
    color: #fff;
    background: rgba(0, 0, 0, 0.99);
  }

  .flow > * + * {
    margin-top: var(--flow-space, 1em);
  }

  article {
    --flow-space: var(--sz-8);

    width: 90vw;
    width: clamp(16em, 90vw, 70em);
    margin-left: auto;
    margin-right: auto;
    padding-left: 1.5em;
    padding-right: 1.5em;
  }

  article time {
    --flow-space: var(--sz-4);
    display: block;
  }

  article :is(h2, h3) {
    --flow-space: var(--sz-8);
  }

  article :is(h2, h3) + * {
    --flow-space: var(--sz-4);
  }

  article figure,
  article figure + * {
    --flow-space: var(--sz-8);
  }

  article time {
    font-size: var(--sz-6);
  }

  @media (max-width: 40rem) {
    :host {
      font-size: 13px;
    }
  }
`;
export class ContentElement extends BehaviorElement(AsLitHIC(LitElement)) {
  static get is() {
    return "hi-content";
  }
  static get styles() {
    return [reset, stylesTheme, styles];
  }
  static get properties() {
    return {
      html: {
        type: String
      },
      __allyHtml: {
        type: String,
        attribute: false
      }
    };
  }
  update(props) {
    super.update(props);
    if (props.has("html")) {
      this.__allyHtml = this.__fixLinkA11y(this.html);
    }
  }
  __fixLinkA11y(html2) {
    return html2.replace(R.detectLinkTag, (_, tagAttrs, tagContent) => {
      if (tagAttrs.includes("target")) {
        return `<a ${tagAttrs}>${tagContent}</a>`;
      }
      const icon = `<hi-icon type="material" kind="north_east" aria-label="Leave site to visit ${tagContent}"></hi-icon>`;
      const rel = tagAttrs.includes("rel") ? "" : 'rel="external nofollow noopener noreferrer"';
      return `<a target="_blank" ${rel} ${tagAttrs}>${tagContent}</a>${icon}`;
    });
  }
  render() {
    return html` ${unsafeHTML(this.__allyHtml)} `;
  }
}
export default HIP.factory.element(ContentElement);
