
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import {
  BehaviorA11yKeyboard,
  AsHIC,
  BehaviorBrowserEvents,
  BehaviorElement,
  BehaviorInteractiveElement,
  HIP
} from '../../../eip/element/hi-component/hi-component.js';
import dayjs from '../../../lip/library/li-dayjs/li-dayjs.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class CalendarMonthElement extends BehaviorA11yKeyboard(
  BehaviorBrowserEvents(BehaviorInteractiveElement(BehaviorElement(AsHIC(PolymerElement))))
) {
  constructor() {
    super();
    this.inputDate = void 0;
  }
  static get is() {
    return "hi-calendar-month";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          display: block;
          font-size: var(--hi-px-14);
          --calendar--block-size: calc(var(--hi-px-32) + var(--hi-px-8));
        }

        :host(:focus) {
          outline: none;
        }

        [x-style='table'] {
          border-collapse: separate !important;
          border-spacing: var(--hi-px-1);
        }

        tr#weekDays {
          background: var(--hi-clr--gray-1);
        }

        thead [x-style='dotw'],
        tbody#days [x-style='day'] {
          text-align: center;
          padding: var(--hi-px-4);
          width: var(--calendar--block-size);
        }

        thead [x-style='dotw'] {
          font-weight: var(--hi-font--weight-light);
          color: var(--hi-clr--gray-9);
          font-size: var(--hi-px-13);
          height: var(--hi-px-24);
        }

        tbody#days [x-style='day'] {
          box-shadow: none;
          color: var(--hi-clr--theme-5);
          height: var(--calendar--block-size);
          border-radius: var(--hi-shape--rounded);
          border: var(--hi-px-1) solid var(--hi-clr--transparent);
          cursor: pointer;
        }

        tbody#days tr,
        tbody#days [x-style='day'] {
          height: var(--calendar--block-size);
        }

        tbody#days [disabled] {
          color: var(--hi-color--disabled-text);
          pointer-events: none !important;
          opacity: 0.9;
        }

        @media screen and (-ms-high-contrast: active) {
          tbody#days [disabled] {
            opacity: 0.4;
          }
        }

        tbody#days [highlighted] {
          position: relative;
        }

        tbody#days [highlighted]::after {
          content: '';
          display: block;
          position: absolute;
          width: var(--hi-px-4);
          height: var(--hi-px-4);
          border-radius: 50%;
          background: var(--hi-clr--gray-8);
          left: 50%;
          transform: translateX(-50%);
          margin-top: var(--hi-px-3);
        }

        @media screen and (-ms-high-contrast: active) {
          tbody#days [highlighted]::after {
            border: var(--hi-px-1) solid var(--hi-clr--transparent);
          }
        }

        [x-style='day'][x-style='today'] {
          border: var(--hi-px-1) solid var(--hi-clr--theme-5);
        }

        @media (hover: hover) {
          tbody#days [x-style='day'][id]:hover {
            background-color: var(--hi-clr--theme-1);
            color: var(--hi-clr--theme-5);
            text-shadow: var(--hi-px-0) var(--hi-px-0) var(--hi-px-1) var(--hi-clr--theme-5);
          }
        }

        :host(:focus) tbody#days [x-style='day'][id][focused] {
          background-color: var(--hi-clr--theme-1);
          color: var(--hi-clr--theme-5);
          text-shadow: var(--hi-px-0) var(--hi-px-0) var(--hi-px-1) var(--hi-clr--theme-5);
          outline: var(--hi-outline--width) var(--hi-outline--line) var(--hi-clr--outline);
          outline-offset: var(--hi-outline--offset);
          perspective: 1000px;
        }

        :host(:focus) tbody#days [x-style='day'][id][selected],
        tbody#days [x-style='day'][id][selected] {
          background-color: var(--hi-clr--theme-3);
          color: var(--hi-clr--white);
          font-weight: var(--hi-font--weight-heavy);
        }

        :host(:focus) tbody#days [x-style='day'][id][focused][selected] {
          outline-width: var(--hi-outline--width);
          outline-offset: var(--hi-px-4);
        }

        :host(:focus) tbody#days [highlighted][x-style='day'][id][selected]::after,
        tbody#days [highlighted][x-style='day'][id][selected]::after {
          background: var(--hi-clr--white);
        }
        :host([readonly]) tbody#days [x-style='day']:not([selected]) {
          color: var(--hi-clr--gray-7);
        }
      </style>

      <table
        id="monthTable"
        role="grid"
        aria-readonly="true"
        aria-label="Use down arrow to navigate, and press enter to select a day"
        x-style="table"
      >
        <thead>
          <tr id="weekDays">
            <th id="day-index-sun" scope="col" x-style="dotw" aria-label="Sunday">S</th>
            <th id="day-index-mon" scope="col" x-style="dotw" aria-label="Monday">M</th>
            <th id="day-index-tue" scope="col" x-style="dotw" aria-label="Tuesday">T</th>
            <th id="day-index-wed" scope="col" x-style="dotw" aria-label="Wednesday">W</th>
            <th id="day-index-thu" scope="col" x-style="dotw" aria-label="Thursday">T</th>
            <th id="day-index-fri" scope="col" x-style="dotw" aria-label="Friday">F</th>
            <th id="day-index-sat" scope="col" x-style="dotw" aria-label="Saturday">S</th>
          </tr>
        </thead>
        <tbody id="days">
          <tr>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
          </tr>
          <tr>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
          </tr>
          <tr>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
          </tr>
          <tr>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
          </tr>
          <tr>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
          </tr>
          <tr>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
            <td x-style="day"></td>
          </tr>
        </tbody>
      </table>
    `;
  }
  static get properties() {
    return {
      value: {
        type: String,
        reflectToAttribute: true,
        notify: true,
        observer: "__valueChanged"
      },
      focusedValue: {
        type: String,
        reflectToAttribute: true,
        notify: true,
        observer: "__focusedValueChanged"
      },
      focusedModel: {
        type: Object,
        notify: true,
        observer: "__focusedModelChanged"
      },
      min: {
        type: String,
        notify: true
      },
      max: {
        type: String,
        notify: true
      },
      disabledBelowMin: {
        type: Boolean
      },
      disabledAboveMax: {
        type: Boolean
      },
      highlightDates: {
        type: Array
      },
      weekDays: {
        type: Array,
        readOnly: true,
        value: function() {
          return [
            {
              fullDayName: "Sunday",
              shortDayName: "Sun",
              dayNameId: "day-index-sun"
            },
            {
              fullDayName: "Monday",
              shortDayName: "Mon",
              dayNameId: "day-index-mon"
            },
            {
              fullDayName: "Tuesday",
              shortDayName: "Tue",
              dayNameId: "day-index-tue"
            },
            {
              fullDayName: "Wednesday",
              shortDayName: "Wed",
              dayNameId: "day-index-wed"
            },
            {
              fullDayName: "Thursday",
              shortDayName: "Thu",
              dayNameId: "day-index-thu"
            },
            {
              fullDayName: "Friday",
              shortDayName: "Fri",
              dayNameId: "day-index-fri"
            },
            {
              fullDayName: "Saturday",
              shortDayName: "Sat",
              dayNameId: "day-index-sat"
            }
          ];
        }
      }
    };
  }
  static get api() {
    return ["commit"];
  }
  static get keyBindings() {
    return {
      "space:keyup": "__onKeyUp_space",
      "enter:keyup": "__onKeyUp_enter",
      "up:keydown": "__onKeyDown",
      "down:keydown": "__onKeyDown",
      "up:keyup": "__onKeyUp_up",
      "down:keyup": "__onKeyUp_down",
      "left:keyup": "__onKeyUp_left",
      "right:keyup": "__onKeyUp_right"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "group");
    this.__onClick__dayBound = this.__onClick__day.bind(this);
    this.__renderMonthView();
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    const cellsEl = this.$.days.querySelectorAll("td");
    cellsEl.forEach((dayEl) => {
      dayEl.removeEventListener("click", this.__onClick__dayBound);
    });
  }
  __renderMonthView() {
    const cellsEl = this.$.days.querySelectorAll("td");
    const rowsEl = this.$.days.querySelectorAll("tr");
    const numberOfColumns = 7;
    rowsEl.forEach((rowEl, index) => {
      rowEl.setAttribute("id", "row-" + index + 1);
    });
    cellsEl.forEach((dayEl, index) => {
      const colIndex = index % numberOfColumns;
      dayEl.addEventListener("click", this.__onClick__dayBound);
      dayEl.setAttribute("data-row", "row-" + index);
      dayEl.setAttribute("data-col-index", `${colIndex}`);
      dayEl.setAttribute("x-style", "day");
    });
  }
  updateMonthView(date = this.focusedModel) {
    const cellsEl = this.$.days.querySelectorAll("td");
    const currentDay = f1.format("iso_date", { format_type: "european" }, dayjs());
    const yearMonthFormat = f1.format("iso_date", { raw_format: "YYYY-MM" }, date);
    let firstDayOfMonth = f1.format("iso_date", { format_type: "european" }, dayjs(yearMonthFormat + "-01"));
    const startDayIndex = dayjs(firstDayOfMonth).day();
    const endDayIndex = startDayIndex + dayjs(this.focusedModel).daysInMonth();
    let dayMonthIndex = 1;
    cellsEl.forEach((dayEl, index) => {
      const isDayInMonth = index >= startDayIndex && index < endDayIndex;
      const isDayValue = f1.format("iso_date", { format_type: "european" }, firstDayOfMonth) === currentDay;
      dom.flipAttr(dayEl, "x-style", isDayValue, "today");
      dayEl.setAttribute("x-style", "day");
      dayEl.removeAttribute("disabled");
      if (isDayInMonth) {
        const dayString = dayMonthIndex < 10 ? "0" + dayMonthIndex : dayMonthIndex;
        const dateString = yearMonthFormat + "-" + dayString;
        const getColIndex = dom.getAttr(dayEl, "data-col-index", "");
        this.__setValidRange(dayEl, dateString);
        if (dayEl.hasAttribute("data-col-index")) {
          dayEl.setAttribute("headers", this.weekDays[getColIndex].dayNameId);
        }
        dayEl.setAttribute("aria-label", f1.format("iso_date", { format_type: "european" }, dateString));
        dayEl.setAttribute("id", "day-index-" + dateString);
        dayEl.setAttribute("data-value", dateString);
        dayEl.setAttribute("aria-selected", "false");
        dayEl.innerText = `${dayMonthIndex}`;
        firstDayOfMonth = dayjs(firstDayOfMonth).add(1, "day");
        dayMonthIndex += 1;
      } else {
        dayEl.removeAttribute("aria-selected");
        dayEl.removeAttribute("aria-label");
        dayEl.removeAttribute("data-value");
        dayEl.removeAttribute("headers");
        dayEl.removeAttribute("id");
        dayEl.removeAttribute("highlighted");
        dayEl.innerText = "";
      }
    });
  }
  __setValidRange(dayEl, dateString) {
    let isDayInHighlighted = false;
    const currentDay = f1.format("iso_date", { format_type: "european" }, dayjs());
    if (this.highlightDates != null) {
      isDayInHighlighted = this.highlightDates.includes(dateString);
    }
    isDayInHighlighted ? dayEl.setAttribute("highlighted", "true") : dayEl.removeAttribute("highlighted");
    if (this.disabledBelowMin) {
      this.min = f1.safe(currentDay, this.min);
    }
    if (this.disabledAboveMax) {
      this.max = f1.safe(currentDay, this.max);
    }
    if (this.min) {
      dayjs(dateString).isBefore(dayjs(this.min)) ? dayEl.setAttribute("disabled", "true") : dayEl.removeAttribute("disabled");
    }
    if (this.max) {
      dayjs(dateString).isAfter(dayjs(this.max)) ? dayEl.setAttribute("disabled", "true") : "";
    }
  }
  _hasMinDate(date = this.focusedModel) {
    if (!this.min) {
      return false;
    }
    const cellsEl = this.$.days.querySelectorAll("td");
    const yearMonthFormat = f1.format("iso_date", { raw_format: "YYYY-MM" }, date);
    let firstDayOfMonth = f1.format("iso_date", { format_type: "european" }, yearMonthFormat + "-01");
    const startDayIndex = dayjs(firstDayOfMonth).day();
    const endDayIndex = startDayIndex + dayjs(this.focusedModel).daysInMonth();
    let dayMonthIndex = 1;
    for (let index = 0; index < cellsEl.length; index++) {
      const isDayInMonth = index >= startDayIndex && index < endDayIndex;
      if (isDayInMonth) {
        const dayString = dayMonthIndex < 10 ? "0" + dayMonthIndex : dayMonthIndex;
        const dateString = yearMonthFormat + "-" + dayString;
        if (dayjs(dateString).isSameOrBefore(dayjs(this.min))) {
          return true;
        }
        firstDayOfMonth = dayjs(firstDayOfMonth).add(1, "day");
        dayMonthIndex += 1;
      }
    }
    return false;
  }
  _hasMaxDate(date = this.focusedModel) {
    if (!this.max) {
      return false;
    }
    const cellsEl = this.$.days.querySelectorAll("td");
    const yearMonthFormat = f1.format("iso_date", { raw_format: "YYYY-MM" }, date);
    let firstDayOfMonth = f1.format("iso_date", { format_type: "european" }, yearMonthFormat + "-01");
    const startDayIndex = dayjs(firstDayOfMonth).day();
    const endDayIndex = startDayIndex + dayjs(this.focusedModel).daysInMonth();
    let dayMonthIndex = 1;
    for (let index = 0; index < cellsEl.length; index++) {
      const isDayInMonth = index >= startDayIndex && index < endDayIndex;
      if (isDayInMonth) {
        const dayString = dayMonthIndex < 10 ? "0" + dayMonthIndex : dayMonthIndex;
        const dateString = yearMonthFormat + "-" + dayString;
        if (dayjs(dateString).isSameOrAfter(dayjs(this.max))) {
          return true;
        }
        firstDayOfMonth = dayjs(firstDayOfMonth).add(1, "day");
        dayMonthIndex += 1;
      }
    }
    return false;
  }
  __focusedModelChanged(model) {
    this.updateMonthView(model);
    this.__flagElement(this.focusedValue, "focused");
    this.__flagElement(this.value, "selected");
  }
  __focusedValueChanged(focusedValue) {
    this.__flagElement(focusedValue, "focused");
  }
  __valueChanged(value, oldValue) {
    if (!this.__inValidRange(value)) {
      return;
    }
    this.__flagElement(value, "selected");
    this.$.monthTable.setAttribute("aria-activedescendant", `day-index-${value}`);
  }
  __onClick__day(event) {
    const disabledState = event.currentTarget.hasAttribute("disabled");
    if (disabledState) {
      return;
    }
    const value = event.currentTarget.getAttribute("data-value");
    if (value && value.length > 0) {
      this.value = value;
      this.commit();
    }
  }
  __flagElement(value, flag) {
    const oldSelectedEl = this.$.days.querySelectorAll("td[" + flag + "]");
    const newSelectedEl = this.$.days.querySelectorAll('td[data-value="' + value + '"]');
    if (oldSelectedEl && oldSelectedEl.length === 1) {
      oldSelectedEl[0].removeAttribute(flag);
      if (flag === "selected") {
        oldSelectedEl[0].setAttribute("aria-selected", "false");
      }
    }
    if (newSelectedEl && newSelectedEl.length === 1) {
      newSelectedEl[0].setAttribute(flag, flag);
      if (flag === "selected") {
        newSelectedEl[0].setAttribute("aria-selected", "true");
      }
    }
  }
  selectDay(value = this.focusedValue) {
    this.value = value;
    this.commit();
  }
  __onKeyDown(event) {
    event.preventDefault();
  }
  __onKeyUp_up() {
    if (this._isPrevWeekSelectable(this.focusedModel)) {
      this.inputDate?.goToPreviousWeek();
    }
  }
  __onKeyUp_down() {
    if (this._isNextWeekSelectable(this.focusedModel)) {
      this.inputDate?.goToNextWeek();
    }
  }
  __onKeyUp_left() {
    if (this._isYesterdaySelectable(this.focusedModel)) {
      this.inputDate?.goToYesterday();
    }
  }
  __onKeyUp_right() {
    if (!this._isSameOrAfterMin(f1.format("iso_date", { format_type: "european" }, dayjs(this.focusedModel)))) {
      this.inputDate?.shiftToMin();
    } else if (this._isTomorrowSelectable(this.focusedModel)) {
      this.inputDate?.goToTomorrow();
    }
  }
  _isYesterdaySelectable(focusedModel) {
    return this._isSameOrAfterMin(
      f1.format("iso_date", { format_type: "european" }, dayjs(focusedModel).day(dayjs(focusedModel).day() - 1))
    );
  }
  _isTomorrowSelectable(focusedModel) {
    return this._isSameOrBeforeMax(
      f1.format("iso_date", { format_type: "european" }, dayjs(focusedModel).day(dayjs(focusedModel).day() + 1))
    );
  }
  _isPrevWeekSelectable(focusedModel) {
    return this._isSameOrAfterMin(
      f1.format("iso_date", { format_type: "european" }, dayjs(focusedModel).day(dayjs(focusedModel).day() - 7))
    );
  }
  _isNextWeekSelectable(focusedModel) {
    return this._isSameOrBeforeMax(
      f1.format("iso_date", { format_type: "european" }, dayjs(focusedModel).day(dayjs(focusedModel).day() + 7))
    );
  }
  _isSameOrBeforeMax(dateString) {
    return this.max ? dayjs(dateString).isSameOrBefore(dayjs(this.max)) : true;
  }
  _isSameOrAfterMin(dateString) {
    return this.min ? dayjs(dateString).isSameOrAfter(dayjs(this.min)) : true;
  }
  __inValidRange(value) {
    return this._isSameOrBeforeMax(value) && this._isSameOrAfterMin(value);
  }
  __onKeyUp_enter() {
    this.selectDay(this.focusedValue);
  }
  __onKeyUp_space(event) {
    event.preventDefault();
    this.selectDay(this.focusedValue);
  }
  commit() {
    return {
      value: this.value
    };
  }
}
export default HIP.factory.element(CalendarMonthElement);
