
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorBrowserEvents, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorInputTime } from '../../../eip/element/hi-behavior-input/hi-behavior-input-time.js';
import { BehaviorMirror } from '../../../eip/element/hi-behavior-mirror/hi-behavior-mirror.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import $ip from '../../../pip/library/pi-ip/pi-ip.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import dayjs from '../../../lip/library/li-dayjs/li-dayjs.js';
import '../../../eip/element/hi-button/hi-button.js';
import '../../../eip/element/hi-icon/hi-icon.js';
import '../../../eip/element/hi-label/hi-label.js';
import './hi-calendar-month.js';
import './hi-datepicker-generator-list.js';
export class DatepickerElement extends BehaviorInputTime(
  BehaviorMirror(BehaviorBrowserEvents(BehaviorElement(AsHIC(PolymerElement))))
) {
  constructor() {
    super(...arguments);
    this.disallowInitialValue = false;
    this.formatType = "medium";
  }
  static get is() {
    return "hi-datepicker";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          display: table;
          border: var(--hi-px-1) solid var(--hi-clr--gray-3);
          min-width: calc(7 * var(--hi-px-40));
        }

        #headerId {
          display: flex;
          align-items: center;
          justify-content: space-around;
        }

        #dateHeading {
          display: inline-flex;
          gap: var(--hi-px-4);
        }

        #monthSelectEl,
        #yearSelectEl {
          font-size: var(--hi-font-size--text);
          color: var(--hi-clr--theme-5);
          cursor: pointer;
          padding: var(--hi-px-6);
          border: none;
        }

        #tapToday {
          border: var(--hi-px-1) solid var(--hi-clr--theme-3);
          border-top-width: var(--hi-px-4);
          box-shadow: none;
          color: var(--hi-clr--theme-5);
          padding: var(--hi-px-1);
          cursor: pointer;
          min-width: var(--hi-px-24);
          max-height: calc(2 * var(--hi-px-13));
        }

        #tapToday:hover {
          background: var(--hi-clr--theme-1);
          border-color: var(--hi-clr--theme-5);
        }

        #__todayDate {
          font-weight: var(--hi-font--weight-normal);
          font-size: var(--hi-px-13);
        }

        #prevButton[disabled],
        #nextButton[disabled] {
          opacity: 0.8;
        }

        :host([disabled]),
        :host([readonly]) {
          pointer-events: all !important;
          opacity: 0.9;
        }

        :host([disabled]) *,
        :host([readonly]) * {
          pointer-events: none;
        }
      </style>
      <section id="headerId">
        <h4 mixin="show-ax" aria-live="polite" aria-atomic="true">[[ __focusedMonthName ]] [[ __focusedYear ]]</h4>
        <div id="dateHeading">
          <hi-label id="monthSelectLabel" for="monthSelectEl" mixin="show-ax">Month</hi-label>
          <select id="monthSelectEl" aria-labelledby="monthSelectLabel"></select>
          <hi-label id="yearSelectLabel" for="yearSelectEl" mixin="show-ax">Year</hi-label>
          <select id="yearSelectEl" aria-labelledby="yearSelectLabel"></select>
        </div>
        <div x-style="dateControls">
          <hi-button
            id="prevButton"
            disabled$="[[disabled]]"
            enable-isolated-events
            aesthetic="icon"
            layout-icon
            on-trigger-ended="goToPreviousMonth"
            theme$="[[ theme ]]"
          >
            <span mixin="show-ax"> Go To Previous Month </span>
            <hi-icon kind="chevron_left" theme$="[[ theme ]]"></hi-icon>
          </hi-button>
        </div>
        <div>
          <hi-button
            aesthetic="ghost"
            id="tapToday"
            disabled$="[[disabled]]"
            on-trigger-ended="goToToday"
            enable-isolated-events
            role="contentinfo"
            aria-atomic="true"
            aria-live="polite"
            aria-label="Go to today"
            theme$="[[ theme ]]"
          >
            <span id="__todayDate">[[ __todayDate ]]</span>
          </hi-button>
        </div>
        <div x-style="dateControls">
          <hi-button
            id="nextButton"
            disabled$="[[disabled]]"
            enable-isolated-events
            aesthetic="icon"
            layout-icon
            on-trigger-ended="goToNextMonth"
            theme$="[[ theme ]]"
          >
            <span mixin="show-ax"> Go To Next Month </span>
            <hi-icon kind="chevron_right" theme$="[[ theme ]]"></hi-icon>
          </hi-button>
        </div>
      </section>
      <hi-calendar-month
        id="calendarMonth"
        theme$="[[ theme ]]"
        disabled$="[[disabled]]"
        readonly$="[[readonly]]"
        value="{{ value }}"
        focused-value="{{ focusedValue }}"
        focused-model="{{ focusedModel }}"
        format-type="{{ formatType }}"
        min="{{ min }}"
        max="{{ max }}"
        disabled-below-min="[[ disabledBelowMin ]]"
        disabled-above-max="[[ disabledAboveMax ]]"
        highlight-dates="[[ highlightDates ]]"
        on-commit-ended="__calendarMonth__onCommitEnded"
      >
      </hi-calendar-month>
    `;
  }
  static get properties() {
    return {
      value: {
        type: Object,
        reflectToAttribute: true,
        notify: true,
        observer: "__valueChanged"
      },
      valueFormatted: {
        type: String,
        computed: "__valueFormattedCompute(value)"
      },
      formatType: {
        type: String
      },
      focusedValue: {
        type: String,
        reflectToAttribute: true,
        notify: true
      },
      disabledBelowMin: {
        type: Boolean
      },
      disabledAboveMax: {
        type: Boolean
      },
      focusedModel: {
        type: Object
      },
      min: {
        type: String
      },
      max: {
        type: String
      },
      highlightDates: {
        type: Array
      },
      today: {
        type: Object,
        value: () => dayjs()
      },
      disallowInitialValue: {
        type: Boolean
      },
      __todayDate: {
        type: String,
        computed: "__goToTodayDate(today)"
      },
      timeFormat: {
        type: String,
        readOnly: true,
        value: () => DatepickerElement.timeFormat
      }
    };
  }
  static get timeFormat() {
    return "YYYY-MM-DD";
  }
  static get api() {
    return [
      "goToToday",
      "goToYesterday",
      "goToTomorrow",
      "goToPreviousWeek",
      "goToNextWeek",
      "goToPreviousMonth",
      "goToNextMonth",
      "commit"
    ];
  }
  static get observers() {
    return ["__reflectValueToDayjs(focusedValue)"];
  }
  static get valueEvent() {
    return "value-changed";
  }
  static get autoBinding() {
    return {
      signal: "value-changed",
      field: "value"
    };
  }
  get inputMirrorType() {
    return "date";
  }
  connectedCallback() {
    super.connectedCallback();
    this.__initializeValue();
    this.__includeEventHandlers();
    this.$.calendarMonth.inputDate = this;
    dom.setAttr(this, "focusable", true);
    this.__html5Select__onClickEndedProxy(this.$.monthSelectEl);
    this.__html5Select__onClickEndedProxy(this.$.yearSelectEl);
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    this.$.calendarMonth.inputDate = void 0;
  }
  __html5Select__onClickEndedProxy(el) {
    el.addEventListener("click", (evt) => {
      evt.stopPropagation();
    });
  }
  __includeEventHandlers() {
    this.addEventListener("value-notified", (evt) => {
      if (evt.detail && evt.detail.srcElement.localName == "hi-button") {
        evt.stopPropagation();
      }
    });
    this.$.yearSelectEl.addEventListener("change", () => {
      this.__focusedYear = this.$.yearSelectEl.value;
      if (this.__focusedYear) {
        this.generateYearOptions(this.__focusedYear);
        const focusedDate = dayjs(this.focusedValue).year(this.__focusedYear);
        this.set("focusedValue", `${dayjs(focusedDate).format("YYYY-MM")}-01`);
      }
    });
    this.$.monthSelectEl.addEventListener("change", () => {
      this.__focusedMonth = this.$.monthSelectEl.value;
      if (this.__focusedMonth) {
        this.generateMonthOptions(this.__focusedMonth);
        const focusedDate = dayjs(this.focusedValue).month(this.__focusedMonth);
        this.set("focusedValue", `${dayjs(focusedDate).format("YYYY-MM")}-01`);
      }
    });
  }
  __initializeValue() {
    if (this.value !== void 0) {
      return;
    }
    const closestDefaultValue = this.__closestDefaultValue();
    const initializeToToday = !this.disallowInitialValue;
    if (initializeToToday) {
      this.value = closestDefaultValue;
    } else {
      this.focusedValue = closestDefaultValue;
    }
  }
  __valueChanged(value) {
    const _value = f1.format("iso_date", { format_type: "european" }, value);
    this.focusedValue = _value ? _value : this.__closestDefaultValue();
    this.value = _value;
    HIP.fn.announce(`${_value} selected`);
  }
  __valueFormattedCompute(_value) {
    return f1.format("iso_date", { format_type: this.formatType }, _value);
  }
  __reflectValueToDayjs(focusedValue) {
    if (f1.is.null(focusedValue)) {
      this.focusedModel = this.__closestDefaultValue();
    } else {
      this.focusedModel = dayjs(focusedValue);
      const todayDate = f1.format("iso_date", { format_type: "european" }, this.today);
      dom.flipAttr(this.$.prevButton, "disabled", this.$.calendarMonth._hasMinDate(this.focusedModel));
      dom.flipAttr(this.$.nextButton, "disabled", this.$.calendarMonth._hasMaxDate(this.focusedModel));
      const [year, month, date] = focusedValue.split("-");
      this.focusedModel.month(parseInt(month) - 1);
      this.focusedModel.year(parseInt(year));
      this.focusedModel.date(parseInt(date));
    }
    this.set("focusedModel", dayjs(this.focusedModel));
    this.set("focusedIndex", this.focusedModel.day());
    this.__focusedYear = dayjs(this.focusedModel).year();
    this.__focusedMonth = dayjs(this.focusedModel).month();
    this.generateYearOptions(this.__focusedYear);
    this.generateMonthOptions(this.__focusedMonth);
  }
  __closestDefaultValue() {
    const currentDay = dayjs();
    const isCurrentDaySelectable = (!this.min || dayjs(currentDay).isSameOrAfter(this.min, "day")) && (!this.max || dayjs(currentDay).isSameOrBefore(this.max, "day"));
    if (isCurrentDaySelectable) {
      return f1.format("iso_date", { format_type: "european" }, currentDay);
    } else {
      const isCurrentDayBeforeMin = this.min ? dayjs(currentDay).isBefore(dayjs(this.min)) : false;
      return isCurrentDayBeforeMin ? this.min : this.max;
    }
  }
  __calendarMonth__onCommitEnded() {
    this.commit();
  }
  __goToTodayDate(today) {
    return f1.is.in("date", today) ? today.date() : "";
  }
  generateMonthOptions(selectedMonth) {
    const selectMonthEl = this.$.monthSelectEl;
    const generatorMonth = $ip.di.inject("generator", "month");
    selectMonthEl.innerHTML = "";
    generatorMonth.create(selectedMonth).forEach(({ value, title, selected }) => {
      const optionEl = dom.create(selectMonthEl, "option", {
        selected,
        value
      });
      if (selected) {
        this.__focusedMonthName = title;
      }
      optionEl.textContent = title;
    });
  }
  generateYearOptions(selectedYear) {
    const selectYearEl = this.$.yearSelectEl;
    const generatorYear = $ip.di.inject("generator", "year");
    selectYearEl.innerHTML = "";
    generatorYear.create(selectedYear).forEach(({ value, title, selected }) => {
      const optionEl = dom.create(selectYearEl, "option", {
        selected,
        value
      });
      optionEl.textContent = title;
    });
  }
  selectValue() {
    this.selectedIndex = this.focusedIndex;
  }
  _shiftDay(day, shift) {
    day = dayjs(day).day(dayjs(day).day() + shift);
    this.set("focusedValue", f1.format("iso_date", { format_type: "european" }, day));
    HIP.fn.announce(`current date ${f1.format("iso_date", { format_type: "medium" }, day)}`);
  }
  _shiftMonth(day, shift) {
    day = dayjs(day).month(dayjs(day).month() + shift);
    this.set("focusedValue", dayjs(day).format("YYYY-MM") + "-01");
  }
  goToToday() {
    const value = f1.format("iso_date", { format_type: "european" }, this.today);
    this.set("focusedValue", value);
    this.set("value", value);
    this.commit();
  }
  shiftToMin() {
    if (this.min) {
      this.focusedModel = dayjs(this.min);
      this.set("focusedValue", f1.format("iso_date", { format_type: "european" }, this.min));
      HIP.fn.announce(`current date ${f1.format("iso_date", { format_type: "medium" }, this.focusedModel)}`);
    }
  }
  goToYesterday() {
    return this._shiftDay(this.focusedModel, -1);
  }
  goToTomorrow() {
    return this._shiftDay(this.focusedModel, 1);
  }
  goToPreviousWeek() {
    return this._shiftDay(this.focusedModel, -7);
  }
  goToNextWeek() {
    return this._shiftDay(this.focusedModel, 7);
  }
  goToPreviousMonth() {
    return this._shiftMonth(this.focusedModel, -1);
  }
  goToNextMonth() {
    return this._shiftMonth(this.focusedModel, 1);
  }
  focus() {
    this.$.calendarMonth.focus();
  }
}
HIP.factory.element(DatepickerElement);
