
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { beforeNextRender } from '../../../hip/library/polymer/polymer.js';
import { BehaviorContainer } from '../../../eip/element/hi-behavior-container/hi-behavior-container.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorScrollable } from '../../../eip/element/hi-behavior-scrollable/hi-behavior-scrollable.js';
import { BehaviorPane } from '../../../eip/element/hi-behavior-pane/hi-behavior-pane.js';
import '../../../xip/element/xi-a11y-announcer/xi-a11y-announcer.js';
import '../../../eip/element/hi-behavior-pane/hi-behavior-pane.js';
import '../../../eip/element/hi-button/hi-close.js';
export class DialogElement extends BehaviorScrollable(
  BehaviorPane(BehaviorContainer(BehaviorElement(AsHIC(PolymerElement))))
) {
  static get is() {
    return "hi-dialog";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          --hi-dialog--min-height: 12rem;
          --hi-dialog--max-height: 72vh;
          --hi-dialog--max-width: 54vw;
          --hi-dialog--min-width: 20rem;
          --hi-dialog--max-width-full: 100vw;
          --hi-dialog--max-height-full: 100vh;
          --hi-dialog--max-height-prompt: 24rem;
          --hi-dialog--max-height-mobile: 50vh;
          --hi-dialog--frame-margin: 2rem;
        }

        :host {
          position: fixed;
          top: 0;
          left: 0;
          z-index: var(--hi-depth--modal);
          display: none;
        }

        #dialogContainer {
          height: 100vh;
          width: 100vw;
          display: flex;
          justify-content: center;
          align-items: center;
          background: rgba(0.2, 0.2, 0.2, 0.2);
        }

        #dialogBox {
          position: relative;
          display: flex;
          flex-direction: column;
          width: 100%;
          height: 100%;
          background: var(--hi-clr--white);
          overflow-y: auto;
          box-shadow: var(--hi-clr--shadow-down-3);
          padding-bottom: var(--hi-px-24);
          border-radius: var(--hi-shape--radius-3);
          border: var(--hi-px-2) solid var(--hi-clr--gray-6);
          min-height: var(--hi-dialog--min-height);
          max-height: var(--hi-dialog--max-height);
          max-width: var(--hi-dialog--max-width);
          min-width: var(--hi-dialog--min-width);
        }

        #headerWrap {
          display: flex;
          flex-shrink: 0;
          align-items: center;
          background: var(--hi-clr--white);
          margin-bottom: var(--hi-px-16);
          padding: var(--hi-px-24) var(--hi-px-28) 0;
        }

        #headerIconContainer {
          margin-right: var(--hi-px-8);
        }

        #dialogHeader {
          align-items: flex-start;
          flex-direction: column;
          width: 100%;
          margin-right: var(--hi-spacing--medium-xx);
        }

        #dialogHeader > * {
          padding: 0;
          margin: 0;
        }

        #headerWrap #subHeading {
          color: var(--hi-clr--gray-8);
          margin-bottom: var(--hi-px-4);
        }

        #closeButton {
          z-index: var(--hi-depth--above-all);
          background: var(--hi-clr--white);
        }

        #closeButton ::slotted(hi-close) {
          position: absolute;
          top: var(--hi-px-16);
          right: var(--hi-px-24);
          z-index: var(--hi-depth--above-all);
          overflow: hidden;
          color: var(--hi-clr--text);
          min-width: var(--hi-close--size);
          border-radius: var(--hi-px-2);
        }

        #dialogBody {
          display: block;
          flex: 1 1 auto;
          position: relative;
          overflow-y: auto;
          align-self: center;
          padding-top: var(--hi-px-24);
          padding-left: var(--hi-px-24);
          padding-right: var(--hi-px-24);
          width: 99%;
          border-radius: var(--hi-shape--radius-3);
        }

        #dialogFooter {
          margin-top: var(--hi-px-24);
        }

        #controls {
          flex-wrap: nowrap;
          flex: 0 1 auto;
          display: flex;
          flex-direction: row;
          align-items: center;
          justify-content: flex-end;
        }

        #dialogFooter ::slotted(*) {
          margin-left: var(--hi-px-8);
        }

        #dialogFooter {
          padding-left: var(--hi-px-24);
          padding-right: var(--hi-px-24);
        }

        :host([tight-screen]) {
          --hi-dialog--max-width: 27vw;
          --hi-dialog--max-height: 54vh;
        }

        :host([tight-screen]) #dialogBody {
          padding-top: var(--hi-px-12);
        }

        :host([wide-screen]) {
          --hi-dialog--min-height: 80vh;
          --hi-dialog--max-height: 90vh;
          --hi-dialog--max-width: 80vw;
          --hi-dialog--min-width: 90vw;
        }

        :host([frame-screen]) {
          --hi-dialog--max-height: 100vh;
          --hi-dialog--max-width: 100vw;
          --hi-dialog--min-height: 0;
          --hi-dialog--min-width: 0;

          left: var(--hi-screen--left);
          top: var(--hi-screen--top);
          bottom: 0;
          right: 0;
        }

        :host([frame-screen]) #dialogContainer {
          height: calc(100% - var(--hi-screen--top));
          width: 100%;
        }

        :host([frame-screen]) #dialogBox {
          height: calc(100% - var(--hi-dialog--frame-margin));
          width: calc(100% - var(--hi-dialog--frame-margin));
        }

        :host([full-screen]) #dialogBox {
          top: 0;
          left: 0;
          right: 0;
          bottom: 0;
          position: fixed;
          transform: translate(0, 0);
          max-height: var(--hi-dialog--max-height-full);
          max-width: var(--hi-dialog--max-width-full);
          height: var(--hi-dialog--max-height-full);
          width: var(--hi-dialog--max-width-full);
          background: var(--hi-clr--white);
          border-radius: 0;
        }

        :host([full-screen]) #dialogBox {
          z-index: var(--hi-plane--modal);
          min-height: var(--hi-dialog--min-height);
          box-shadow: none;
        }

        :host([opened]) {
          display: initial;
        }

        :host(:not([heading])) #dialogHeader {
          display: none;
        }

        :host([scrollable]) #dialogBody:focus-within {
          outline: var(--hi-outline--width) var(--hi-outline--line) var(--hi-clr--outline);
          outline-offset: -1px;
          border: none;
        }

        #heading:focus,
        #closeButton ::slotted(hi-close:focus) {
          outline: var(--hi-outline--width) var(--hi-outline--line) var(--hi-clr--outline);
          outline-offset: var(--hi-outline--offset);
        }

        :host([aesthetic='prompt']) #dialogBox {
          width: auto;
        }

        :host([aesthetic='prompt']) #dialogBody {
          margin-bottom: var(--hi-px-16);
        }

        :host([aesthetic='prompt']) #dialogHeader {
          margin-bottom: 0;
        }

        :host([aesthetic='prompt']) #dialogFooter {
          margin-top: 0;
        }

        :host([aesthetic='prompt']) #dialogBox {
          max-height: var(--hi-dialog--max-height-prompt);
          height: 60vh;
        }

        :host([aesthetic='minimal']) #dialogBody,
        :host([aesthetic='minimal']) #dialogHeader,
        :host([aesthetic='minimal']) #dialogFooter {
          padding-left: 0;
          padding-right: 0;
          margin: 0;
          border: none;
        }

        :host([disallow-close]) #closeButton {
          display: none;
        }

        @media only screen and (max-width: 40rem) {
          #dialogBox {
            left: 0;
            right: 0;
            bottom: 0;
            position: fixed;
            max-height: var(--hi-dialog--max-height-full);
            max-width: var(--hi-dialog--max-width-full);
            height: var(--hi-dialog--max-height-full);
            width: 100%;
            background: var(--hi-clr--white);
            border-radius: 0;
          }
          #dialogHeader {
            margin-right: var(--hi-px-44);
          }
          :host([aesthetic='prompt']) #dialogBox {
            max-height: var(--hi-dialog--max-height-mobile);
          }
        }
      </style>
      <div id="dialogContainer">
        <div id="dialogBox">
          <div id="headerWrap">
            <div id="headerIconContainer" aria-hidden="true">
              <slot id="headerIcon" name="headerIcon"></slot>
            </div>
            <header id="dialogHeader">
              <p id="subHeading">[[ subHeading ]]</p>
              <slot name="heading"></slot>
            </header>
          </div>
          <section id="dialogBody" role="document">
            <slot id="content"></slot>
          </section>
          <footer id="dialogFooter">
            <slot id="controls" name="controls"></slot>
          </footer>
          <div id="closeButton">
            <slot name="closeButton"></slot>
          </div>
        </div>
      </div>
    `;
  }
  static get properties() {
    return {
      fullScreen: {
        type: Boolean,
        reflectToAttribute: true
      },
      tightScreen: {
        type: Boolean,
        reflectToAttribute: true
      },
      __hasControls: {
        type: Boolean,
        notify: true
      }
    };
  }
  static get observers() {
    return ["__visualControlsObserver(__hasControls, opened)"];
  }
  connectedCallback() {
    super.connectedCallback();
    this.__onSlotChanged_controlsBound = this.__onSlotChanged_controls.bind(this);
    this.__onSlotChanged_contentBound = this.__onSlotChanged_content.bind(this);
    this.__onSlotChanged_headerIconBound = this.__onSlotChanged_headerIcon.bind(this);
    this.$.controls.addEventListener("slotchange", this.__onSlotChanged_controlsBound);
    this.$.content.addEventListener("slotchange", this.__onSlotChanged_contentBound);
    this.$.headerIcon.addEventListener("slotchange", this.__onSlotChanged_headerIconBound);
    beforeNextRender(this, this.__onSlotChanged_headerIconBound);
    beforeNextRender(this, this.__onSlotChanged_controlsBound);
    beforeNextRender(this, this.__onSlotChanged_contentBound);
    this._assignScrollable(this.$.dialogBody);
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    this.$.headerIcon.removeEventListener("slotchange", this.__onSlotChanged_headerIconBound);
    this.$.controls.removeEventListener("slotchange", this.__onSlotChanged_controlsBound);
    this.$.content.removeEventListener("slotchange", this.__onSlotChanged_contentBound);
  }
  _openedChanged(opened, openedOld) {
    const rtn = super._openedChanged(opened, openedOld);
    this._updateScrollable();
    this.__reflectContentFocus();
    if (opened) {
      HIP.fn.announce(`${this.heading || ""} opened`);
    }
    return rtn;
  }
  __onSlotChanged_content() {
    this._updateScrollable();
    this.__reflectContentFocus();
  }
  __onSlotChanged_controls() {
    this.__reflectContentFocus();
  }
  __onSlotChanged_headerIcon() {
    const itemsInHeaderIcon = this._importChildren(this.$.headerIcon, HIP.util.isNode).length;
    this.$.headerIconContainer.style.display = itemsInHeaderIcon <= 0 ? "none" : "initial";
  }
  __reflectContentFocus() {
    if (this.opened) {
      const controlsChildren = this.__getInteractiveControlsChildren();
      this.__updateControlFooter(controlsChildren);
    }
  }
  __getInteractiveContentChildren() {
    return this._importChildren(this.$.content, HIP.util.isNode).filter((el) => {
      return this.__isButton(el) || el.hasAttribute("tabindex");
    });
  }
  __getInteractiveControlsChildren() {
    return this._importChildren(this.$.controls, HIP.util.isNode).filter((el) => {
      return this.__isButton(el) || el.hasAttribute("tabindex");
    });
  }
  __updateControlFooter(controlChildren) {
    this.__hasControls = controlChildren && controlChildren.length > 0;
    if (!this.__hasControls) {
      this.removeAttribute("has-footer");
    }
  }
  __visualControlsObserver(__hasControls) {
    this.$.dialogFooter.setAttribute("show", !!__hasControls + "");
  }
  __openPane__backdrop() {
  }
  __closePane__backdrop() {
  }
}
export default HIP.factory.element(DialogElement);
