
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { BehaviorContainer } from '../../../eip/element/hi-behavior-container/hi-behavior-container.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorScrollable } from '../../../eip/element/hi-behavior-scrollable/hi-behavior-scrollable.js';
import { BehaviorPane } from '../../../eip/element/hi-behavior-pane/hi-behavior-pane.js';
import '../../../eip/element/hi-backdrop/hi-backdrop.js';
import '../../../eip/element/hi-button/hi-close.js';
export class DrawerElement extends BehaviorScrollable(
  BehaviorPane(BehaviorContainer(BehaviorElement(AsHIC(PolymerElement))))
) {
  constructor() {
    super(...arguments);
    this.direction = "right";
    this.transformLookup = {
      top: "translateY(-100px)",
      bottom: "translateY(100px)",
      left: "translateX(-100px)",
      right: "translateX(100px)"
    };
  }
  static get is() {
    return "hi-drawer";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          --drawer-box-height: calc(var(--hi-px-32) * 10);
          --drawer-box-width: 40rem;
        }

        :host {
          position: fixed;
          z-index: var(--hi-depth--modal-hover);
          display: none;
          left: 0;
          top: 0;
        }

        #drawerContainer {
          background-color: rgba(0.2, 0.2, 0.2, 0.2);
          display: flex;
          height: 100vh;
          width: 100vw;
        }

        #drawerBox {
          transition: transform 300ms cubic-bezier(0.4, 0, 0.2, 1) 0ms, opacity 300ms cubic-bezier(0.4, 0, 0.2, 1) 0ms,
            width 300ms linear 0ms;
          background-color: var(--hi-clr--white);
          flex-direction: column;
          overflow: hidden auto;
          position: relative;
          display: flex;
        }

        #headerWrap {
          display: flex;
          align-items: center;
          flex-shrink: 0;
          background: var(--hi-clr--white);
          padding: var(--hi-px-16) var(--hi-px-28) 0;
          margin-bottom: var(--hi-px-16);
        }

        :host([heading]) #drawerHeader {
          flex-grow: 1;
          margin-right: var(--hi-px-28);
        }

        :host(:not([heading])) #drawerHeader {
          display: none;
        }

        #drawerHeader #subHeading {
          color: var(--hi-clr--gray-8);
          margin-bottom: var(--hi-px-4);
        }

        #closeButton ::slotted(hi-close) {
          position: absolute;
          top: var(--hi-px-16);
          right: var(--hi-px-24);
          z-index: var(--hi-depth--above-all);
          overflow: hidden;
          color: var(--hi-clr--text);
          min-width: var(--hi-close--size);
          border-radius: var(--hi-px-2);
        }

        #heading:focus {
          outline: var(--hi-outline--width) var(--hi-outline--line) var(--hi-clr--outline);
          outline-offset: var(--hi-outline--offset);
        }

        #itemsContainer {
          display: block;
          flex: 1 1 auto;
          position: relative;
          overflow-y: auto;
          align-self: center;
          padding: var(--hi-px-24) var(--hi-px-28) 0;
          border-radius: var(--hi-px-8);
        }

        :host([direction='left']) #itemsContainer,
        :host([direction='right']) #itemsContainer,
        :host([direction='top']) #itemsContainer,
        :host([direction='bottom']) #itemsContainer {
          width: 100%;
        }

        :host([opened]) #drawerBox {
          box-shadow: 0 0 0 1px transparent, 0 1px 2px rgba(0, 0, 0, 0.1), 0 2px 6px 0 rgba(0, 0, 0, 0.2);
        }

        :host([direction='top']) #drawerBox {
          border-bottom: var(--hi-px-2) solid var(--hi-clr--gray-6);
          height: var(--drawer-box-height);
        }

        :host([direction='right']) #drawerBox {
          border-left: var(--hi-px-2) solid var(--hi-clr--gray-6);
          width: var(--drawer-box-width);
        }

        :host([direction='bottom']) #drawerBox {
          border-top: var(--hi-px-2) solid var(--hi-clr--gray-6);
          height: var(--drawer-box-height);
        }

        :host([direction='left']) #drawerBox {
          border-right: var(--hi-px-2) solid var(--hi-clr--gray-6);
          width: var(--drawer-box-width);
        }

        :host([direction='right']) #drawerContainer {
          flex-direction: row-reverse;
        }

        :host([direction='top']) #drawerContainer {
          flex-direction: column;
        }

        :host([direction='bottom']) #drawerContainer {
          flex-direction: column-reverse;
        }

        :host([opened]) {
          display: initial;
        }

        :host(:not([opened])) #closeButton {
          display: none;
        }

        :host([scrollable]) #itemsContainer:focus-within {
          outline: var(--hi-outline--width) var(--hi-outline--line) var(--hi-clr--outline);
          outline-offset: -1px;
          border: none;
        }

        :host([aesthetic='minimal']) #drawerHeader,
        :host([aesthetic='minimal']) #itemsContainer {
          padding-left: 0;
          padding-right: 0;
          margin: 0;
          border: none;
        }

        :host([disallow-close]) #closeButton {
          display: none;
        }

        @media only screen and (max-width: 40rem) {
          :host([direction='top']) #drawerBox,
          :host([direction='right']) #drawerBox,
          :host([direction='bottom']) #drawerBox,
          :host([direction='left']) #drawerBox {
            top: 0;
            left: 0;
            right: 0;
            bottom: 0;
            position: fixed;
            border-radius: 0;
            max-height: 100vh;
            max-width: 100vw;
          }

          #drawerHeader {
            margin-right: var(--hi-px-44) !important;
          }

          #itemsContainer {
            border-radius: 0;
          }
        }
      </style>
      <div id="drawerContainer">
        <div id="drawerBox">
          <div id="headerWrap">
            <header id="drawerHeader">
              <p id="subHeading">[[ subHeading ]]</p>
              <slot name="heading"></slot>
            </header>
          </div>
          <div id="itemsContainer">
            <slot id="items"></slot>
          </div>
          <div id="closeButton">
            <slot name="closeButton"></slot>
          </div>
        </div>
      </div>
    `;
  }
  static get properties() {
    return {
      direction: {
        type: String,
        reflectToAttribute: true,
        notify: false
      }
    };
  }
  static get observers() {
    return ["__openWatcher(opened, disabled)", "__directionWatcher(direction)"];
  }
  connectedCallback() {
    super.connectedCallback();
    this.__openWatcher(this.opened, this.disabled);
    this._assignScrollable(this.$.itemsContainer);
  }
  __openWatcher(opened, disabled) {
    if (disabled) {
      return;
    }
    if (opened) {
      this.__setDrawerStyleOpen();
      this.__openPane__focusHeader();
    } else {
      this.__setDrawerStyleClosed();
    }
  }
  __directionWatcher(direction) {
    this.__drawerClosed = {
      transform: this.transformLookup[direction],
      opacity: 0
    };
    this.__drawerOpened = {
      transform: "none",
      opacity: 1
    };
  }
  __setDrawerStyleOpen() {
    setTimeout(() => {
      Object.assign(this.$.drawerBox.style, this.__drawerOpened);
      this._updateScrollable();
    }, 64);
  }
  __setDrawerStyleClosed() {
    Object.assign(this.$.drawerBox.style, this.__drawerClosed);
  }
  __openPane__backdrop() {
  }
  __closePane__backdrop() {
  }
}
export default HIP.factory.element(DrawerElement);
