
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { BehaviorContainer } from '../../../eip/element/hi-behavior-container/hi-behavior-container.js';
import { BehaviorFlyout } from '../../../eip/element/hi-behavior-flyout/hi-behavior-flyout.js';
import { BehaviorInput } from '../../../eip/element/hi-behavior-input/hi-behavior-input.js';
import { BehaviorForm } from '../../../eip/element/hi-behavior-form/hi-behavior-form.js';
import { LayoutScreenQueryElement as LSQ } from '../../../yip/element/yi-layout-screen-query/yi-layout-screen-query.js';
import {
  AsHIC,
  BehaviorA11yKeyboard,
  BehaviorBrowserEvents,
  BehaviorCustomizable,
  a11yLabel,
  BehaviorElement,
  HIP
} from '../../../eip/element/hi-component/hi-component.js';
import '../../../eip/element/hi-icon/hi-icon.js';
import '../../../eip/element/hi-button/hi-close.js';
import '../../../eip/element/hi-title/hi-subtitle.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class DropdownElement extends BehaviorCustomizable(
  BehaviorInput(
    BehaviorFlyout(
      BehaviorContainer(
        BehaviorA11yKeyboard(BehaviorBrowserEvents(BehaviorForm(BehaviorElement(AsHIC(PolymerElement)))))
      )
    )
  )
) {
  constructor() {
    super(...arguments);
    this.__flyoutAriaLabel = "value of dropdown";
    this._flyout__popper_modifiers_defaults = [
      {
        name: "flip",
        enabled: false,
        options: {}
      },
      {
        name: "computeStyles",
        options: {
          gpuAcceleration: true
        }
      },
      {
        name: "preventOverflow",
        options: {
          padding: {
            top: 2
          },
          altBoundary: true
        }
      }
    ];
  }
  static get is() {
    return "hi-dropdown";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          --hi-dropdown--height: var(--hi-el--height);
          --hi-dropdown--min-width: calc(var(--hi-px-32) * 2);
          --hi-dropdown--border-color: var(--hi-clr--theme-3);
          --hi-dropdown--color: var(--hi-clr--gray-9);
          --hi-dropdown--placeholder-color: var(--hi-clr--gray-7);
          --hi-dropdown--anchor-height: 100%;
          --hi-dropdown--container-min-units: 1;
          --hi-dropdown--container-max-units: 5;
        }

        :host {
          position: relative;
          user-select: none;
          text-align: left;
          display: block;
          outline: none;
          cursor: pointer;
          width: auto;
          will-change: box-shadow;
          transition: box-shadow 0.2s cubic-bezier(0.4, 0, 1, 1);
          min-width: var(--hi-dropdown--min-width);
          height: var(--hi-dropdown--height);
        }

        :host([hug-width]) {
          display: inline-flex;
        }

        :host(:focus),
        :host(:focus-within) {
          outline-offset: var(--hi-outline--offset);
        }

        #flyoutValue {
          position: relative;
          height: var(--hi-dropdown--height);
          background-color: var(--hi-clr--white);
          border-radius: var(--hi-shape--radius);
          border: var(--hi-px-1) solid var(--hi-dropdown--border-color);
          padding: 0;

          transition: background-color var(--hi-animation--speed) var(--hi-animation--accel);
        }

        :host(:not([opened])) #flyoutValue:focus,
        :host(:not([opened])) #flyoutValue:focus-within {
          outline: var(--hi-outline--width) var(--hi-outline--line) var(--hi-clr--outline);
          outline-offset: var(--hi-outline--offset);
        }

        #flyoutAnchorContainer {
          flex-grow: 1;
          height: 100%;
          width: calc(100% - var(--hi-px-32));
        }

        #anchorContainer {
          width: 100%;
          overflow: hidden;
          display: flex;
          align-items: center;
          justify-content: start;
          font-size: var(--hi-font-size--text);
          line-height: var(--hi-font-lh--medium);
        }

        #flyoutValue,
        #flyoutPlaceholder {
          box-sizing: border-box;
          width: 100%;
          display: flex;
          flex-wrap: nowrap;
          justify-content: space-between;
          align-items: center;
          height: 100%;
        }

        #flyoutPlaceholder {
          padding: var(--hi-px-4) 0 var(--hi-px-4) var(--hi-px-8);
        }

        #flyoutValue ::slotted([slot='anchor']) {
          height: var(--hi-dropdown--anchor-height);
          font-size: var(--hi-font-size--text);
          font-weight: var(--hi-font--weight-light);
          line-height: var(--hi-font-lh--medium);
          flex-grow: 1;
        }

        #anchorToggle {
          width: var(--hi-px-32);
          height: var(--hi-px-32);
          padding: var(--hi-px-8);
          box-sizing: border-box;
          align-self: center;
          transform-origin: center center;
          cursor: pointer; /* controversial choice */
        }

        #flyoutContainer {
          border: var(--hi-px-1) solid var(--hi-dropdown--border-color);
          border-radius: var(--hi-shape--radius);

          border-top-right-radius: 0;
          border-top-left-radius: 0;
          border-top: 0;

          box-sizing: border-box;
          margin: 0;
          display: none;
          position: absolute;
          background-color: var(--hi-clr--white);
          min-height: calc(var(--hi-el--height) * var(--hi-dropdown--container-min-units));

          min-width: 100%;
          width: auto;
          outline: none;
        }

        #dropdownHeader {
          display: none;
          position: sticky;
          top: 0;
          left: 0;
          z-index: var(--hi-depth--above-all);
        }

        #valueContainer {
          overflow-x: hidden;
          overflow-y: auto;
        }

        :host([opened]) #flyoutValue {
          box-shadow: var(--hi-clr--shadow-down-4);
          border-bottom-right-radius: 0;
          border-bottom-left-radius: 0;
        }

        :host([opened]) #flyoutContainer {
          box-shadow: var(--hi-clr--shadow-down-4);
          z-index: calc(2 * var(--hi-depth--above-all));
          display: block;
        }

        :host([placeholder]) #placeholder {
          color: var(--hi-dropdown--placeholder-color);
        }

        :host(:not([placeholder-shown])[placeholder]),
        :host(:not([placeholder-shown])) #placeholder {
          font-style: normal;
          color: var(--hi-dropdown--color);
        }

        :host([cap-height]) #valueContainer {
          max-height: calc(calc(var(--hi-px-32) + var(--hi-px-8)) * var(--hi-dropdown--container-max-units));
        }

        :host([cap-width]) #flyoutContainer {
          max-width: 100%;
        }

        :host([flush-width]) #flyoutContainer {
          width: 100%;
        }

        :host([hug-width]) #flyoutContainer {
          min-width: auto;
          width: auto !important;
          border: 0;
        }

        :host([aesthetic='minimal']) #flyoutValue {
          background-color: var(--hi-clr--transparent);
          border: var(--hi-px-1) solid var(--hi-clr--transparent);
          border-radius: var(--hi-shape--radius);
          box-shadow: none;
        }

        :host([aesthetic='minimal']) #flyoutContainer {
          border: var(--hi-px-1) solid var(--hi-dropdown--border-color);
          border-radius: var(--hi-shape--radius);
        }

        @media (hover: hover) {
          :host(:hover:not([disabled]):not([opened])) #flyoutValue {
            box-shadow: var(--hi-clr--halo-2);
          }

          :host([aesthetic='minimal']:hover:not([disabled]):not([opened])) #flyoutValue {
            border: var(--hi-px-1) solid var(--hi-dropdown--border-color);
          }
        }

        :host([validated][invalid]):not([silent-validate]) #flyoutValue,
        :host([invalid]:not([focused]):not([silent-validate])) #flyoutValue {
          border-color: var(--hi-clr--red-5);
        }

        :host([readonly]),
        :host([disabled]) {
          pointer-events: all !important;
        }

        :host([readonly]) #flyoutValue,
        :host([disabled]) #flyoutValue,
        :host([readonly]) #anchorToggle,
        :host([disabled]) #anchorToggle {
          pointer-events: none !important;
          -moz-user-focus: ignore !important;
          user-select: none !important;
          border-color: var(--hi-clr--theme-3);
        }

        :host([disabled]) #anchorToggle,
        :host([readonly]) #anchorToggle,
        :host([disabled]) #flyoutValue {
          --hi-clr--theme-3: var(--hi-clr--disabled);
        }

        @media only screen and (max-width: 40rem) {
          :host(:not([disallow-responsiveness])) {
            --hi-dropdown--height: auto;
            --hi-dropdown--min-height: auto;
            --hi-dropdown--max-height: 360px;
            --hi-dropdown--max-width: 270px;
            --hi-dropdown--min-width: 250px;
          }

          :host(:not([disallow-responsiveness])) #mobileFlyoutContainer {
            position: fixed;
            top: 0;
            bottom: 0;
            left: 0;
            right: 0;
            z-index: var(--hi-depth--above-all);
            width: 100vw;
            justify-content: center;
            align-items: center;
            background: rgba(0.2, 0.2, 0.2, 0.2);
            display: none;
          }

          :host(:not([disallow-responsiveness])) #flyoutContainer {
            min-width: var(--hi-dropdown--min-width);
            max-width: var(--hi-dropdown--max-width);
            width: auto;
            background: var(--hi-clr--white);
            border-radius: var(--hi-px-12);
            display: none;
            padding-bottom: var(--hi-px-12);
          }

          :host(:not([disallow-responsiveness])) #dropdownHeader {
            display: block;
          }

          :host(:not([disallow-responsiveness])) #valueContainer {
            height: var(--hi-dropdown--height);
            max-height: var(--hi-dropdown--max-height);
            min-height: var(--hi-dropdown--min-height);
          }

          :host([opened]:not([disallow-responsiveness])) #mobileFlyoutContainer {
            display: flex;
          }

          :host([opened]:not([disallow-responsiveness])) #flyoutContainer {
            border-top: var(--hi-px-1) solid var(--hi-dropdown--border-color);
            box-shadow: var(--hi-clr--shadow-down-4);
            z-index: calc(2 * var(--hi-depth--above-all));
            display: block;
          }
        }
      </style>

      <div
        id="flyoutValue"
        aria-label$="[[ __flyoutAriaLabel ]]"
        aria-controls="flyoutContainer"
        aria-haspopup="listbox"
        aria-expanded="false"
        role="combobox"
      >
        <div id="flyoutAnchorContainer">
          <slot id="anchor" name="anchor">
            <div id="flyoutPlaceholder" mixin="ellipsis">
              <span id="anchorContainer" mixin="ellipsis">
                <slot id="placeholder" mixin="ellipsis" name="placeholder"> [[ _placeholder ]] </slot>
              </span>
            </div>
          </slot>
        </div>
        <hi-icon
          id="anchorToggle"
          kind="chevron_down"
          theme="[[ theme ]]"
          size="medium"
          direction$="[[ _iconDirection ]]"
        >
        </hi-icon>
      </div>
      <div id="mobileFlyoutContainer">
        <div id="flyoutContainer">
          <div id="dropdownHeader">
            <hi-subtitle
              title="[[subtitle]]"
              disallow-responsiveness$="[[disallowResponsiveness]]"
              on-close-ended="__subtitle__onCloseEnded"
            ></hi-subtitle>
          </div>
          <div id="statusContainer">
            <!-- flyup is for backwards compatibility -->
            <slot id="flyup" name="flyup"> </slot>
            <slot id="status" name="status"> </slot>
          </div>
          <div id="valueContainer">
            <slot id="items"></slot>
          </div>
        </div>
      </div>
    `;
  }
  static get properties() {
    return {
      kind: {
        type: String,
        observer: "__kindChanged",
        value: "selection"
      },
      aesthetic: {
        type: String
      },
      mode: {
        type: String
      },
      placeholder: {
        type: String,
        observer: "__placeholderChanged"
      },
      _placeholder: {
        type: String
      },
      manualPlaceholder: {
        type: Boolean
      },
      hugWidth: {
        type: Boolean
      },
      flushWidth: {
        type: Boolean
      },
      capWidth: {
        type: Boolean
      },
      capHeight: {
        type: Boolean
      },
      value: {
        type: Object,
        notify: true
      },
      values: {
        type: Object,
        notify: true,
        reflectToAttribute: true
      },
      _iconDirection: {
        type: String,
        computed: "__iconDirectionComputed(opened)"
      },
      placeholderShown: {
        type: Boolean,
        reflectToAttribute: true,
        computed: "__placeholderShownCompute(_placeholder)"
      },
      __flyoutAriaLabel: {
        type: String
      },
      disallowAriaRole: {
        type: Boolean
      },
      subtitle: {
        type: String
      },
      ariaLabel: {
        type: String
      },
      ariaLabelledby: {
        type: String
      },
      ariaDescribedby: {
        type: String
      }
    };
  }
  static get keyBindings() {
    return {
      "enter:keydown": "_onKeyAny__preventDefault",
      "enter:keyup": "_dropdown__onKeyUp_enter",
      "space:keyup": "_dropdown__onKeyUp_space",
      "space:keydown": "_dropdown__onKeyDown_space",
      "down:keyup": "_dropdown__onKeyDown_up_down_arrow",
      "up:keyup": "_dropdown__onKeyDown_up_down_arrow",
      "esc:keyup": "_flyout__onKeyUp_escape"
    };
  }
  static get browserBindings() {
    return {
      click: "_flyout__onTap",
      focusout: "_flyout__onFocusOut"
    };
  }
  static get api() {
    return ["refresh", "toggle", "close", "open"];
  }
  static get observers() {
    return [
      "__openedWatcher(opened)",
      "__reflectValueToChildWatcher(value)",
      "__reflectValuesToChildWatcher(values)",
      "_ariaLabelTextWatcher(ariaLabel, ariaLabelledby, ariaDescribedby, placeholder)"
    ];
  }
  static get autoBinding() {
    return {
      signal: "opened-changed",
      field: "opened"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    dom.setAttr(this, "focusable", true);
    const allowAriaRole = !this.disallowAriaRole;
    if (allowAriaRole) {
      dom.setAttr(this, "role", "group");
    }
    this.$.items.addEventListener("validate-notified", (event) => {
      this.validate(event.detail.value);
    });
    this.__configureByMode();
  }
  __formNameWatcher(_form, _name) {
  }
  __configureByMode() {
    dom.write(() => {
      switch (this.mode) {
        case "input":
          break;
        default:
          dom.setAttrs(this.$.flyoutValue, {
            "aria-expanded": `${!!this.opened}`,
            tabindex: "0"
          });
      }
    });
  }
  _popperConfig() {
    if (LSQ.instance.screenDevice === "mobile") {
      return {};
    }
    return {
      ...super._popperConfig(),
      onFirstUpdate: (state) => {
        state.elements.popper.style.transform = state.elements.popper.style.transform.replace(/\(.*?,/, "(0,");
      }
    };
  }
  __subtitle__onCloseEnded() {
    this.validate(true);
    this.close();
  }
  _flyout__onFocusOut(event) {
    let hasOccurredFromCombobox;
    if (event.srcElement && event.srcElement.__dataHost) {
      hasOccurredFromCombobox = event.srcElement.__dataHost.localName === "hi-combobox";
    }
    const getRelatedTargetEl = event.relatedTarget;
    if (!getRelatedTargetEl) {
      return;
    }
    const parentEl = this.parentElement;
    const hasOccurredFromChild = this.contains(getRelatedTargetEl);
    if (hasOccurredFromChild || getRelatedTargetEl.contains(parentEl) || hasOccurredFromCombobox) {
      return;
    } else {
      this.validate(true);
      this.close();
    }
  }
  _dropdown__onKeyUp_space(event) {
    if (this.mode !== "input") {
      event.preventDefault();
      this._flyout__onKeyUp_space(event);
    }
  }
  _dropdown__onKeyDown_space(event) {
    if (this.mode !== "input") {
      event.preventDefault();
    }
  }
  _dropdown__onKeyUp_enter(event) {
    this._flyout__onKeyUp_enter(event);
    event.preventDefault();
  }
  _dropdown__onKeyDown_up_down_arrow(event) {
    event.preventDefault();
    if (!this.opened) {
      this._flyout__onKeyUp_space(event);
    }
  }
  __openedWatcher(opened) {
    if (LSQ.instance.screenDevice === "mobile") {
      return;
    }
    super._flyout___openedWatcher(opened);
    if (opened) {
      this.$.flyoutContainer.style.width = `${this.offsetWidth}px`;
    }
  }
  commit() {
    return f1.is.in("commit", this.firstSlotChild) && f1.is.function(this.firstSlotChild.commit) ? this.firstSlotChild.commit() : { value: this.value };
  }
  __reflectValueToChildWatcher(value) {
    if (this.__valueUpdate__) {
      return;
    }
    const firstEl = this.firstSlotChild;
    if (null == firstEl) {
      this.__initialValue = value;
      return;
    }
    if (firstEl.value !== value) {
      firstEl.value = value;
    }
  }
  __reflectValuesToChildWatcher(values) {
    if (this.__valueUpdate__) {
      return;
    }
    const firstEl = this.firstSlotChild;
    if (null == firstEl) {
      this.__initialValue = values;
      return;
    }
    if (firstEl.values !== values) {
      firstEl.values = values;
    }
  }
  __onSlotChanged() {
    this._flyout__onSlotChanged();
    this.__reflectFirstChildValue();
  }
  __reflectFirstChildValue() {
    const firstEl = this.firstSlotChild;
    if (null == firstEl) {
      return;
    }
    if (null == firstEl.value && this.__initialValue != firstEl.value) {
      firstEl.value = this.__initialValue;
    }
    this.value = firstEl.value;
    if (firstEl.values) {
      this.values = firstEl.values;
    }
    if (!this.__slotChildValueFormatted && this.value != null) {
      this.__slotChildValueFormatted = f1.is.in("computedValueFormatted", firstEl) && f1.is.function(firstEl.computedValueFormatted) ? firstEl.computedValueFormatted() : firstEl.valueFormatted || `${this.value}`;
    }
    this.__updatePlaceholder();
  }
  __placeholderShownCompute(_placeholder) {
    return this.placeholder == _placeholder;
  }
  __placeholderChanged(_placeholder) {
    this._placeholder = _placeholder;
  }
  __kindChanged(_kind) {
    this._kindFlyout = _kind;
  }
  __iconDirectionComputed(opened) {
    return opened ? "down" : "up";
  }
  _onValueNotified(event) {
    super._flyout___onValueNotified(event);
    if (this.kind === "menu") {
      return;
    }
    this.__updatePlaceholder();
  }
  __updatePlaceholder() {
    if (this.manualPlaceholder) {
      return;
    }
    this._placeholder = this.value == null ? this.placeholder : this.__slotChildValueFormatted || this.placeholder;
  }
  _ariaLabelTextWatcher(label, labelledBy, describedBy) {
    this.__flyoutAriaLabel = `value for ${f1.safe("dropdown", f1.read("/labelEl/innerText", label, this))}`;
    a11yLabel.writeA11yLabel(this.$.flyoutValue, label, labelledBy, describedBy, {
      placeholder: this._placeholder,
      ns: this.ns
    });
    const ariaLabelledBy = f1.safe("", labelledBy).trim();
    const textFromAriaLabelledBy = a11yLabel.getTextContentByIds(ariaLabelledBy);
    this.subtitle = f1.safe(this.subtitle, textFromAriaLabelledBy);
  }
  refresh() {
    const rtn = super.refresh();
    this.__reflectFirstChildValue();
    return rtn;
  }
  focus() {
    this.$.flyoutValue.focus();
  }
  toggle() {
    this._flyout__toggle();
  }
  checkValidity() {
    const firstSlotChild = this.firstSlotChild;
    if (!firstSlotChild) {
      return true;
    }
    return firstSlotChild.checkValidity();
  }
  get validity() {
    const firstSlotChild = this.firstSlotChild;
    if (!firstSlotChild) {
      return this._computeValidity({});
    }
    return firstSlotChild.validity;
  }
}
export default HIP.factory.element(DropdownElement);
