
import { html, css, LitElement } from '../../../hip/library/lit-element/lit-element.js';
import { BehaviorInput } from '../../../eip/element/hi-behavior-input/hi-behavior-input.js';
import { HIP, AsLitHIC, BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorForm } from '../../../eip/element/hi-behavior-form/hi-behavior-form.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import '../../../eip/element/hi-button/hi-button.js';
import '../../../eip/element/hi-icon/hi-icon.js';
import '../../../eip/element/hi-hint/hi-hint.js';
import './hi-picklist.js';
const styles = css`
  :host {
    --hi-dual-picklist--button-group-width: calc(var(--hi-px-32) * 2);
    --hi-dual-picklist--list-height: calc(2 * calc(var(--hi-px-32) * 5));
  }
  :host {
    display: block;
    width: 100%;
  }

  :host * {
    box-sizing: border-box;
  }
  .container {
    display: flex;
    align-items: stretch;
    min-height: calc(var(--hi-px-32) * 5);
  }

  .list-section {
    width: calc(0.5 * calc(100% - var(--hi-dual-picklist--button-group-width)));
    min-width: calc(var(--hi-px-32) * 5);
  }

  .move-buttons {
    padding: var(--hi-px-16);
    width: var(--hi-dual-picklist--button-group-width);
    display: flex;
    flex-direction: column;
    place-items: center;
    justify-content: center;
  }

  .move-buttons hi-button:first-of-type {
    margin-block-end: var(--hi-px-16);
  }
`;
export class HiDualPicklist extends BehaviorForm(BehaviorInput(BehaviorForm(BehaviorElement(AsLitHIC(LitElement))))) {
  constructor() {
    super(...arguments);
    this.__list = [];
    this.__availableList = [];
    this.__selectedList = [];
    this.__uniqueID = f1.uuid();
    this.input = [];
    this.value = void 0;
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get is() {
    return "hi-dual-picklist";
  }
  static get properties() {
    return {
      input: {
        type: Object
      },
      value: {
        type: Array,
        notify: true
      }
    };
  }
  static get valueEvent() {
    return "value-changed";
  }
  static get api() {
    return ["clear", "reset", "selectItem"];
  }
  static get autoBinding() {
    return {
      signal: "value-changed",
      field: "value"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "group");
  }
  render() {
    return html` <div role="presentation">
      <div class="container">
        <div class="list-section">${this.__renderList("available")}</div>
        <div class="move-buttons">
          <hi-button
            ?disabled="${!this.__hasAvailableOptions}"
            aria-labelledby="moveSecondLabel"
            layout="icon"
            @click="${this.__selectListItems}"
            @keyup="${this.__moveSelectedToSecondList}"
            theme="${this.theme}"
          >
            <hi-icon kind="chevron_right" inherit-color ?disabled="${!this.__hasAvailableOptions}"></hi-icon>
            <hi-hint position="top" id="moveSecondLabel">Move selection to second list</hi-hint>
          </hi-button>
          <hi-button
            ?disabled="${!this.__hasSelectedOptions}"
            aria-labelledby="moveFirstLabel"
            layout="icon"
            @click="${this.__unselectListItems}"
            @keyup="${this.__moveSelectedToFirstList}"
            theme="${this.theme}"
          >
            <hi-icon kind="chevron_left" inherit-color ?disabled="${!this.__hasSelectedOptions}"></hi-icon>
            <hi-hint position="bottom" id="moveFirstLabel">Move selection to first list</hi-hint>
          </hi-button>
        </div>
        <div class="list-section">${this.__renderList("selected")}</div>
      </div>
    </div>`;
  }
  updated(props) {
    super.updated(props);
    if (props.has("input")) {
      this.__transformInput();
      this.__initializeLists();
    }
  }
  __renderList(listKind) {
    const list = this[`__${listKind}List`];
    return html`<hi-picklist
      kind="${listKind}"
      .list=${list}
      .uniqueId=${this.__uniqueID}
      @picklist-push-selected=${this.__handlePushSelectedEvent(listKind)}
      @picklist-pull-selected=${this.__handlePullSelectedEvent(listKind)}
      theme="${this.theme}"
      ?disabled="${this.disabled}"
    ></hi-picklist>`;
  }
  __transformInput() {
    const _input = f1.to.array(f1.read("/properties", this.input, this.input));
    const mapFn = (item) => ({
      title: item.title || item.label || "",
      value: item.value || item.id || "",
      selected: false
    });
    this.__list = f1.map(mapFn, _input);
    return this.__list;
  }
  __updateValue() {
    this.value = f1.map((item) => item.value, this.__selectedList);
  }
  __isValidKeyPress(event) {
    const { key, code } = event;
    const keyCode = key || code;
    const SPACEKEY = " ";
    const isValidKeyPress = keyCode === "Enter" || keyCode === SPACEKEY;
    return isValidKeyPress;
  }
  __handlePushSelectedEvent(listKind) {
    return () => {
      if (listKind === "available") {
        this.__selectListItems();
      } else {
        this.__unselectListItems();
      }
    };
  }
  __handlePullSelectedEvent(listKind) {
    return () => {
      if (listKind === "available") {
        this.__unselectListItems();
      } else {
        this.__selectListItems();
      }
    };
  }
  __moveSelectedToSecondList(event) {
    if (this.__isValidKeyPress(event)) {
      event.preventDefault();
      this.__selectListItems();
    }
  }
  __moveSelectedToFirstList(event) {
    if (this.__isValidKeyPress(event)) {
      event.preventDefault();
      this.__unselectListItems();
    }
  }
  __initializeLists() {
    const [picked, unpicked] = f1.partition((item) => (this.value || []).indexOf(item.value) > -1, this.__list);
    this.__availableList = unpicked;
    this.__selectedList = picked;
    this.__sortListAndRender();
  }
  __sortListAndRender() {
    this.__availableList.sort();
    this.__selectedList.sort();
    this.requestUpdate();
    this.__updateValue();
  }
  __selectListItems() {
    const [unpicked, picked] = this.__moveSelectedItemsBetweenLists(this.__availableList, this.__selectedList);
    this.__selectedList = picked;
    this.__availableList = unpicked;
    this.__sortListAndRender();
  }
  __unselectListItems() {
    const [picked, unpicked] = this.__moveSelectedItemsBetweenLists(this.__selectedList, this.__availableList);
    this.__selectedList = picked;
    this.__availableList = unpicked;
    this.__sortListAndRender();
  }
  __moveSelectedItemsBetweenLists(fromList, toList) {
    const [selected, rest] = f1.partition((item) => item.selected, fromList);
    toList = toList.concat(selected);
    toList.forEach((item) => item.selected = false);
    fromList = rest;
    return [fromList, toList];
  }
  get __hasAvailableOptions() {
    return this.disabled ? true : this.__availableList.length > 0;
  }
  get __hasSelectedOptions() {
    return this.disabled ? true : this.__selectedList.length > 0;
  }
  reset() {
    this.__transformInput();
    this.__sortListAndRender();
    return this.value;
  }
  clear() {
    this.value = void 0;
    return this.value;
  }
  get validity() {
    return this._computeValidity({
      valueMissing: !this.required || this.required && f1.is.nil(this.value)
    });
  }
}
HIP.factory.element(HiDualPicklist);
export default "hi-dual-picklist";
