
import { html, css, LitElement } from '../../../hip/library/lit-element/lit-element.js';
import { repeat } from '../../../hip/library/lit-html/directives/repeat.js';
import { ifDefined } from '../../../hip/library/lit-html/directives/if-defined.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import {
  HIP,
  AsLitHIC,
  BehaviorA11yKeyboard,
  BehaviorElement,
  BehaviorInteractiveElement
} from '../../../eip/element/hi-component/hi-component.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import '../../../eip/element/hi-button/hi-button.js';
import '../../../eip/element/hi-icon/hi-icon.js';
import '../../../eip/element/hi-hint/hi-hint.js';
const styles = css`
  :host * {
    box-sizing: border-box;
  }

  .list {
    list-style: none;
    padding: 0;
    margin: 0;
    height: var(--hi-dual-picklist--list-height);
    overflow: auto;
    border: var(--hi-px-1) solid var(--hi-clr--gray-4);
  }

  .list-item {
    display: flex;
    align-items: center;
    color: var(--hi-clr--text);
    line-height: var(--hi-font-lh--medium);
    font-size: var(--hi-font-size--text);
    border-bottom: var(--hi-px-1) solid var(--hi-clr--gray-4);
  }

  .list-item:hover {
    background: var(--hi-clr--gray-3);
  }

  .list-item[selected] {
    font-weight: var(--hi-font--weight-bold);
    position: relative;
  }

  .list-item[focused] {
    background: var(--hi-clr--theme-1);
    outline: var(--hi-outline--width) var(--hi-outline--line) var(--hi-clr--outline);
    outline-offset: var(--hi-outline--offset);
  }

  .list-item [drag-handle] {
    cursor: move;
    padding: var(--hi-px-8);
    align-self: stretch;
    display: flex;
    align-items: center;
    justify-content: center;
    color: var(--hi-clr--theme-3);
  }

  [drag-handle] hi-icon {
    cursor: move;
  }

  .list-item-body {
    padding: var(--hi-px-8);
    padding-left: 0;
    width: calc(100% - var(--hi-px-32));
    flex-grow: 1;
  }

  .list-item .checkmark {
    margin: var(--hi-px-8);
  }

  .drag-item {
    background: var(--hi-clr--theme-1);
  }

  .list.drag-hover {
    border: 2px dashed var(--hi-clr--theme-3) !important;
  }

  .empty-placeholder {
    padding: var(--hi-px-32);
    height: 100%;
    display: flex;
    place-items: center;
    justify-content: center;
    background: var(--hi-clr--gray-1);
  }

  .placeholderText {
    text-align: center;
    color: var(--hi-clr--gray-8);
  }
`;
export class HiPicklist extends BehaviorA11yKeyboard(
  BehaviorInteractiveElement(BehaviorElement(AsLitHIC(LitElement)))
) {
  constructor() {
    super(...arguments);
    this.kind = "available";
    this.selected = void 0;
    this.list = [];
  }
  static get properties() {
    return {
      kind: {
        type: String,
        reflect: true
      },
      selected: {
        type: Array
      },
      focusedIndex: {
        type: Number,
        reflect: true,
        attribute: "focused-index"
      },
      list: {
        type: Array
      }
    };
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get is() {
    return "hi-picklist";
  }
  static get api() {
    return ["clear", "reset", "selectItem"];
  }
  static get keyBindings() {
    return {
      "home:keyup": "_pickList__onKeyUp_home",
      "home:keydown": "_onKeyAny__preventDefault",
      "end:keyup": "_pickList__onKeyUp_end",
      "end:keydown": "_onKeyAny__preventDefault",
      "space:keyup": "_pickList__onKeyUp_space",
      "space:keydown": "_onKeyAny__preventDefault",
      "enter:keyup": "_pickList__onKeyUp_enter",
      "up:keyup": "_pickList__onKeyUp_up",
      "up:keydown": "_onKeyAny__preventDefault",
      "down:keyup": "_pickList__onKeyUp_down",
      "down:keydown": "_onKeyAny__preventDefault",
      "right:keyup": "_pickList__onKeyUp_right",
      "right:keydown": "_onKeyAny__preventDefault",
      "left:keyup": "_pickList__onKeyUp_left",
      "left:keydown": "_onKeyAny__preventDefault",
      "delete:keyup": "_pickList__onKeyUp_delete",
      "delete:keydown": "_onKeyAny__preventDefault",
      "backspace:keyup": "_pickList__onKeyUp_backspace",
      "backspace:keydown": "_onKeyAny__preventDefault"
    };
  }
  render() {
    const focusIndex = this.focusedIndex;
    return html`<div>
      <p id="instructions" class="show-ax" mixin="show-ax">
        Press spacebar to toggle drag drop mode, use arrow keys to move selected elements.
      </p>
      <ul
        class="list"
        id="${this.kind}List"
        tabindex="0"
        role="listbox"
        aria-multiselectable="true"
        aria-activedescendant="listItem_${this.kind}_${ifDefined(focusIndex)}"
        aria-labelledby="instructions"
        @focus="${() => this.__initFocusIndex()}"
        @dragover="${(e) => this.__dndOver(e, this.uniqueId)}"
        @dragenter="${(e) => this.__dndEnter(e, this.uniqueId)}"
        @dragleave="${(e) => this.__dndLeave(e, this.uniqueId)}"
        @drop="${(e) => this.__dndDrop(e, this.uniqueId)}"
        ?disabled="${this.disabled}"
      >
        ${repeat(
      this.list,
      (item) => item.value,
      (item, index) => html`
            <li
              class="list-item"
              id="listItem_${this.kind}_${index}"
              ?focused=${index === focusIndex}
              ?selected=${item.selected}
              aria-selected="${item.selected}"
              role="option"
              draggable="true"
              @click="${() => this.selectItem(index)}"
              @dragstart="${(e) => this.__dndStart(index, e)}"
              @dragend="${(e) => this.__dndEnd(e, this.uniqueId)}"
            >
              <div drag-handle>
                <hi-icon kind="draggable" size="small" inherit-color></hi-icon>
              </div>
              <span class="list-item-body">${item.title}</span>
              ${item.selected ? html`<hi-icon class="checkmark" kind="check" size="small" inherit-color></hi-icon>` : nothing}
            </li>
          `
    )}
        ${!this.__hasOptions ? html`
              <div class="empty-placeholder">
                <span class="placeholderText"> Drag options here to move</span>
              </div>
            ` : nothing}
      </ul>
    </div>`;
  }
  __updateSelected() {
    this.selected = f1.map((item) => item.value, this.list);
  }
  _pickList__onKeyUp_enter(event) {
    if (this.kind === "available") {
      event.preventDefault();
      this.__pushSelected();
    }
  }
  _pickList__onKeyUp_space() {
    this.__toggleFocusedItem(false);
  }
  _pickList__onKeyUp_up(event) {
    event.preventDefault();
    this.__shiftFocusUp();
  }
  _pickList__onKeyUp_down(event) {
    event.preventDefault();
    this.__onKey__Next();
  }
  _pickList__onKeyUp_left(event) {
    event.preventDefault();
    this.__shiftFocusUp();
  }
  _pickList__onKeyUp_right(event) {
    event.preventDefault();
    this.__onKey__Next();
  }
  _pickList__onKeyUp_delete(event) {
    if (this.kind === "selected") {
      event.preventDefault();
      this.__pushSelected();
    }
  }
  _pickList__onKeyUp_backspace(event) {
    if (this.kind === "selected") {
      event.preventDefault();
      this.__pushSelected();
    }
  }
  _pickList__onKeyUp_home() {
    this.__shiftFocusStart();
  }
  _pickList__onKeyUp_end() {
    this.__shiftFocusEnd();
  }
  __onKey__Next() {
    this.__shiftFocusDown();
  }
  __dndStart(positionIndex, event) {
    this.__setFocusIndex(positionIndex);
    this.__toggleFocusedItem(true);
    this.__addClass("li", event, "drag-item");
    this.__informUser("GRAB");
  }
  __dndEnter(event, dragInstanceId) {
    if (dragInstanceId !== this.uniqueId) {
      return;
    }
    this.__addClass("ul", event, "drag-hover");
    event.preventDefault();
  }
  __dndOver(event, dragInstanceId) {
    if (dragInstanceId !== this.uniqueId) {
      return;
    }
    this.__addClass("ul", event, "drag-hover");
    event.preventDefault();
  }
  __dndLeave(event, dragInstanceId) {
    if (dragInstanceId !== this.uniqueId) {
      return;
    }
    this.__removeClass("ul", event, "drag-hover");
  }
  __dndEnd(event, dragInstanceId) {
    if (dragInstanceId !== this.uniqueId) {
      return;
    }
    this.__removeClass("li", event, "drag-item");
  }
  __dndDrop(event, dragInstanceId) {
    if (dragInstanceId !== this.uniqueId) {
      return;
    }
    this.__removeClass("ul", event, "drag-item", "drag-hover");
    this.__toggleFocusedItem(false);
    this.__pullSelected();
    this.__informUser("DROP");
  }
  __sortListAndRender() {
    this.__updateSelected();
    this.requestUpdate();
  }
  __emitUpdateEvent(eventName) {
    this._dispatchEvent(eventName, {
      value: this.selected
    });
  }
  __pushSelected() {
    this.__emitUpdateEvent("picklist-push-selected");
  }
  __pullSelected() {
    this.__emitUpdateEvent("picklist-pull-selected");
  }
  get __hasOptions() {
    return this.list.length > 0;
  }
  get __lastIndex() {
    return this.list.length - 1;
  }
  __addClass(target = "li", event, ...classes) {
    const eventTarget = event.target;
    const closestListItem = eventTarget?.closest(target);
    if (closestListItem) {
      closestListItem.classList.add(...classes);
    }
  }
  __removeClass(target = "li", event, ...classes) {
    const eventTarget = event.target;
    const closestListItem = eventTarget?.closest(target);
    if (closestListItem) {
      closestListItem.classList.remove(...classes);
    }
  }
  __toggleFocusedItem(force = false) {
    const currentItem = this.list[this.focusedIndex];
    if (currentItem) {
      currentItem.selected = force ? true : !currentItem.selected;
    }
    this.requestUpdate();
  }
  __initFocusIndex() {
    this.__setFocusIndex(0);
  }
  __resetFocusIndex() {
    this.__setFocusIndex(-1);
  }
  __setFocusIndex(index) {
    return this.focusedIndex = index;
  }
  __shiftFocusDown() {
    if (this.focusedIndex !== this.__lastIndex) {
      this.__setFocusIndex(this.focusedIndex + 1);
    }
    return this.focusedIndex;
  }
  __shiftFocusUp() {
    if (this.focusedIndex !== 0) {
      this.__setFocusIndex(this.focusedIndex - 1);
    }
    return this.focusedIndex;
  }
  __shiftFocusStart() {
    this.__setFocusIndex(0);
    return this.focusedIndex;
  }
  __shiftFocusEnd() {
    this.__setFocusIndex(this.__lastIndex);
    return this.focusedIndex;
  }
  __informUser(action) {
    const totalCount = this.list.length;
    const itemPosition = this.focusedIndex + 1;
    const instructions = "Press up and down arrow keys to change position, spacebar to drop, escape key to cancel.";
    let message = ``;
    switch (action) {
      case "GRAB":
        message = `Option ${itemPosition} grabbed, current position ${itemPosition} of ${totalCount}. ${instructions}`;
        break;
      case "DROP":
        message = `Option ${itemPosition} dropped, final position ${itemPosition} of ${totalCount}`;
        break;
      case "MOVE":
        message = `Option ${itemPosition} moved, new position ${itemPosition} of ${totalCount}. ${instructions}`;
        break;
    }
    HIP.fn.announce(message);
  }
  focusItem(index) {
    return this.__setFocusIndex(index);
  }
  shiftFocusDown() {
    return this.__shiftFocusDown();
  }
  shiftFocusUp() {
    return this.__shiftFocusUp();
  }
  selectItem(index) {
    this.focusItem(index);
    return this.__toggleFocusedItem(true);
  }
  reset() {
    this.__resetFocusIndex();
    this.__sortListAndRender();
    return this.selected;
  }
  clear() {
    this.selected = void 0;
    return this.selected;
  }
}
HIP.factory.element(HiPicklist);
export default "hi-picklist";
