
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import { BehaviorContainer } from '../../../eip/element/hi-behavior-container/hi-behavior-container.js';
import { BehaviorFlyout } from '../../../eip/element/hi-behavior-flyout/hi-behavior-flyout.js';
import {
  AsHIC,
  HIP,
  BehaviorA11yKeyboard,
  BehaviorBrowserEvents,
  BehaviorCustomizable,
  BehaviorElement
} from '../../../eip/element/hi-component/hi-component.js';
import '../../../eip/element/hi-icon/hi-icon.js';
export class FlyoutElement extends BehaviorCustomizable(
  BehaviorFlyout(BehaviorContainer(BehaviorA11yKeyboard(BehaviorBrowserEvents(BehaviorElement(AsHIC(PolymerElement))))))
) {
  constructor() {
    super();
    this.defaultLabel = "Actions";
    this._kindFlyout = "sticky";
    this.value = "gear";
  }
  static get is() {
    return "hi-flyout";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          --hi-flyout--height: var(--hi-el--height);
          --hi-flyout--container-height: calc(100vh - 10rem); /* FullPage without headers */
          position: relative;
          display: inline-block;
          border-radius: var(--hi-shape--radius);
          vertical-align: middle;
        }

        :host([size='small']) {
          --hi-flyout--height: var(--hi-el--height-sml);
        }

        :host(:focus:not([disabled])),
        :host(:focus-within:not([disabled])) {
          outline: var(--hi-outline--width) var(--hi-outline--line) var(--hi-clr--outline);
          outline-offset: var(--hi-outline--offset);
        }

        #flyoutValue,
        #flyoutContainer {
          box-sizing: border-box;
          width: auto;
          outline: none;
        }

        #flyoutValue {
          font-size: 100%;
          text-transform: none;
          will-change: box-shadow;
          transition: box-shadow 0.2s cubic-bezier(0.4, 0, 0.2, 1);
          padding: var(--hi-px-2) var(--hi-px-4);
          min-height: var(--hi-flyout--height);
          min-width: var(--hi-flyout--height);
          display: flex;
          justify-content: center;
          align-items: center;
          -webkit-appearance: none;
          -moz-appearance: none;
          appearance: none;
          cursor: default;
        }

        #flyoutValue ::slotted(*) {
          font-size: var(--hi-font-size--text);
        }

        :host([aesthetic='chrome']) #flyoutValue {
          box-shadow: var(--hi-clr--shadow-clickable);
          border: var(--hi-px-1) solid var(--hi-clr--theme-3);
          border-radius: var(--hi-shape--radius);
          background-color: var(--hi-clr--white);
        }

        :host([aesthetic='chrome']) #flyoutValue ::slotted(hi-icon) {
          --hi-icon--color: var(--hi-clr--theme-5);
        }

        :host([aesthetic='chrome']:hover) #flyoutValue,
        :host([opened][aesthetic='chrome']) #flyoutValue {
          box-shadow: var(--hi-clr--shadow-down-3);
        }

        :host([aesthetic='chrome']:hover) #flyoutValue {
          background-color: var(--hi-clr--theme-1);
        }

        :host([disabled]) #flyoutValue {
          --hi-clr--theme-3: var(--hi-clr--disabled);
          --hi-clr--theme-5: var(--hi-clr--disabled);
          -moz-user-focus: ignore !important;
          pointer-events: none !important;
          user-select: none !important;
        }

        #flyoutContainer {
          position: absolute;
          display: none;
          background: var(--hi-clr--white);
          height: auto;
          z-index: var(--hi-depth--window-hover);
          transform: translate3d(0, 0, 0);
          transition: opacity var(--hi-animation--speed) var(--hi-animation--accel);
          margin: var(--hi-px-4) 0;
          min-height: var(--hi-px-32);
          min-width: 12em;
          max-height: 30em;
          border: var(--hi-px-1) solid var(--hi-clr--theme-3);
          border-radius: var(--hi-shape--radius-2);
          box-shadow: var(--hi-clr--shadow-down-3);
        }

        :host([opened]) #flyoutContainer {
          display: block;
        }

        :host([cap-height]) #flyoutContainer {
          max-height: var(--hi-flyout--container-height);
          overflow-y: auto;
        }

        :host([aesthetic='text']) #flyoutValue ::slotted(*) {
          border-bottom: var(--hi-px-1) dashed var(--hi-clr--theme-3) !important;
        }

        @media (hover: hover) {
          :host([aesthetic='ghost']:hover) #flyoutValue,
          :host([aesthetic='text']:hover) #flyoutValue {
            text-shadow: var(--hi-clr--text-shadow-1);
          }
        }
      </style>
      <div id="flyoutValue" role="button" aria-label$="[[ __AXLabel ]]" tabindex="0" aria-controls="flyoutContainer">
        <slot id="anchor" name="anchor">
          <hi-icon kind$="[[ icon ]]" theme$="[[ theme ]]"> </hi-icon>
          <span x-style="hidden">Actions</span>
        </slot>
      </div>
      <div id="flyoutContainer">
        <slot id="items"></slot>
      </div>
    `;
  }
  static get properties() {
    return {
      kind: {
        type: String,
        observer: "__kindChanged"
      },
      heading: {
        type: String,
        reflectToAttribute: true
      },
      _iconDirection: {
        type: String,
        computed: "__iconDirectionComputed(opened)"
      },
      aesthetic: {
        type: String,
        reflectToAttribute: true,
        value: "chrome"
      },
      icon: {
        type: String
      },
      label: {
        type: String
      }
    };
  }
  static get keyBindings() {
    return {
      "enter:keydown:flyoutValue": "_onKeyAny__preventDefault",
      "space:keydown:flyoutValue": "_noop",
      "enter:keyup:flyoutValue": "_flyout__onKeyUp_enter",
      "space:keyup:flyoutValue": "_flyout__onKeyUp_space",
      "esc:keyup:flyoutValue": "_flyout__onKeyUp_escape",
      "left:keyup:flyoutValue": "_noop"
    };
  }
  static get browserBindings() {
    return {
      "click:flyoutValue": "_flyout__onTap"
    };
  }
  static get api() {
    return ["toggle", "close", "open"];
  }
  static get observers() {
    return ["_flyout___openedWatcher(opened)"];
  }
  static get autoBinding() {
    return {
      signal: "opened-changed",
      field: "opened"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    dom.setAttr(this, "focusable", true);
    this.__readContentToAriaLabel();
  }
  _flyout__onKeyUp_space(event) {
    event.preventDefault();
    const targeted = this.__getAppropriateTarget(event);
    if (targeted.localName === "hi-flyout") {
      this.toggle();
    }
  }
  __readContentToAriaLabel() {
    const allAnchors = Array.from(this.querySelectorAll('[slot="anchor"]'));
    const slottedAnchor = allAnchors.reduce((text, el) => `${text}${el.textContent}`, "").replace(/\r?\n|\r/g, "").trim();
    const label = (this.label || "").trim();
    this.__AXLabel = label ? label : slottedAnchor ? slottedAnchor : this.defaultLabel;
  }
  __onSlotChanged() {
    this._flyout__onSlotChanged();
  }
  __kindChanged(kind) {
    this._kindFlyout = kind;
  }
  _onValueNotified(event) {
    super._flyout___onValueNotified(event);
  }
  __iconDirectionComputed(opened) {
    return opened ? "down" : "up";
  }
  toggle() {
    this._flyout__toggle();
  }
}
HIP.factory.element(FlyoutElement);
