
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorContainer } from '../../../eip/element/hi-behavior-container/hi-behavior-container.js';
import { BehaviorInputContainer } from '../../../eip/element/hi-behavior-input/hi-behavior-input-container.js';
import { inputService } from '../../../eip/element/hi-behavior-input/hi-service-input.js';
export const styles = css`
  :host {
    display: block;
    --hi-fieldset--indent: var(--hi-px-16);
  }

  fieldset {
    padding: 0;
    border: 0;
  }

  legend {
    margin-bottom: var(--hi-px-16);
    margin-top: var(--hi-px-16);
    display: block;
    width: 100%;
  }

  legend span {
    display: block;
    padding-bottom: var(--hi-px-4);
    border-bottom: var(--hi-px-1) solid var(--hi-clr--gray-7);
  }

  #container {
    margin-top: calc(-1 * var(--hi-px-24));
  }

  :host(:not([alignment='inline'])) fieldset {
    margin-top: var(--hi-px-24);
  }

  :host([alignment='inline']) legend {
    display: none;
  }

  :host([alignment='inline']) #container {
    display: flex;
    flex-direction: row;
    align-items: end;
    gap: var(--hi-px-12);
  }

  :host([flush][alignment='inline']) #container {
    margin-top: 0;
  }

  :host([alignment='inline']) #container ::slotted(hi-input-group) {
    flex: 1 1 0;
    width: 0;
  }

  @media (min-width: 0px) and (max-width: 40rem) {
    :host([alignment='inline']:not([disallow-responsive])) #container {
      flex-direction: column;
    }
  }
`;
export class FieldsetElement extends BehaviorInputContainer(BehaviorContainer(BehaviorElement(AsHIC(LitElement)))) {
  constructor() {
    super(...arguments);
    this.novalidate = false;
  }
  static get is() {
    return "hi-fieldset";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      heading: {
        type: String
      },
      alignment: {
        type: String,
        reflect: true
      }
    };
  }
  static get autoBinding() {
    return {
      signal: "heading-changed",
      field: "heading"
    };
  }
  updated(props) {
    if (props.has("novalidate")) {
      this.__validationWatcher(this.novalidate);
    }
  }
  render() {
    return html`<fieldset>
      <legend>
        <span>${this.heading}</span>
        <slot name="description"></slot>
      </legend>
      <div id="container">
        <slot id="items"></slot>
      </div>
    </fieldset> `;
  }
  __validationWatcher(novalidate) {
    inputService.applyValidation(this, novalidate);
  }
  _onSlotChanged() {
    super._onSlotChanged();
    inputService.initializeInputs(this);
  }
}
export default HIP.factory.element(FieldsetElement);
