
import { afterNextRender } from '../../../hip/library/polymer/polymer.js';
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { timeOut } from '../../../hip/library/polymer/polymer.js';
import { Debouncer } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorElement, HIP, InternalError } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorContainer } from '../../../eip/element/hi-behavior-container/hi-behavior-container.js';
import { inputService } from '../../../eip/element/hi-behavior-input/hi-service-input.js';
import '../../../eip/element/hi-value-group/hi-value-group.js';
import '../../../eip/element/hi-button/hi-button.js';
import '../../../eip/element/hi-http/hi-http.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import $ip from '../../../pip/library/pi-ip/pi-ip.js';
export class FormElement extends BehaviorContainer(BehaviorElement(AsHIC(PolymerElement))) {
  constructor() {
    super(...arguments);
    this.novalidate = false;
  }
  static get is() {
    return "hi-form";
  }
  static get properties() {
    return {
      kind: {
        type: String,
        value: "submit"
      },
      aesthetic: {
        type: String,
        reflectToAttribute: true,
        observer: "__aestheticChanged"
      },
      alignment: {
        type: String,
        value: "stacked",
        reflectToAttribute: true,
        observer: "__alignmentChanged"
      },
      columnCount: {
        type: Number
      },
      disableHttp: {
        type: Boolean
      },
      input: {
        type: Object,
        observer: "__inputChanged"
      },
      output: {
        type: Object,
        notify: true
      },
      form: {
        type: String,
        value: () => `form-${f1.id()}`
      },
      auto: {
        type: Boolean
      },
      uri: {
        type: String
      },
      method: {
        type: String,
        value: "post"
      },
      endpoint: {
        type: String,
        value: "form"
      },
      externalSubmit: {
        type: String,
        observer: "__externalSubmitChanged"
      },
      externalReset: {
        type: String,
        observer: "__externalResetChanged"
      },
      novalidate: {
        type: Boolean,
        reflectToAttribute: true
      },
      mode: {
        type: String
      },
      secure: {
        type: Boolean
      }
    };
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          --hi-form--input-column-width: auto;
          --hi-form--background-color: var(--hi-clr--white);

          background-color: var(--hi-form--background-color);
          display: block;
        }

        #controlsContainer {
          clear: both;
          padding-top: var(--hi-px-32);
          padding-bottom: var(--hi-px-12);
          background-color: var(--hi-form--background-color);
          justify-content: flex-start;
          flex-direction: row;
          display: none;
        }

        :host([has-controls]) #controlsContainer {
          display: flex;
        }

        :host([hidden-controls]) #controlsContainer {
          position: absolute;
          visibility: hidden;
        }

        #controlsContainer ::slotted(*) {
          order: 99;
          margin-right: var(--hi-px-8);
        }

        #controlsContainer ::slotted([type='reset']),
        :host([reverse-controls]) #controlsContainer ::slotted([type='submit']) {
          order: 101;
        }

        #controlsContainer ::slotted([type='submit']),
        :host([reverse-controls]) #controlsContainer ::slotted([type='reset']) {
          order: 90;
        }

        :host([aesthetic='invisible']) {
          --hi-form--background-color: var(--hi-clr--transparent);
        }

        :host([aesthetic='tight']) {
          border: var(--hi-px-1) solid var(--hi-clr--gray-6);
          border-radius: var(--hi-shape--radius);
        }

        :host([alignment='inline']) #container,
        :host([alignment='inline']) #controlsContainer,
        :host([alignment='hort']) #container,
        :host([alignment='hort']) #controlsContainer {
          align-items: baseline;
          flex-direction: row;
          display: flex;
          height: 100%;
        }

        :host([alignment='hort']) #container,
        :host([alignment='hort']) #controlsContainer {
          flex-grow: 1;
        }

        :host([alignment='inline']) #container,
        :host([alignment='hort']) #container {
          gap: var(--hi-px-12);
          flex-wrap: wrap;
          width: 100%;
        }

        :host([alignment='inline']) #controlsContainer {
          margin-top: auto;
          margin-left: auto;
          margin-bottom: var(--hi-px-8);
          align-self: flex-end;
          padding-top: 0;
          padding-bottom: 0;
        }

        :host([alignment='inline']) #formContainer {
          align-items: flex-end;
          display: inline-flex;
          width: 100%;
          flex: 1 1 0;
        }

        :host([alignment='stacked']) ::slotted(hi-input-group) {
          width: 100%;
          display: block;
        }

        :host([alignment='stacked']),
        :host([alignment='stacked-inline']) {
          width: 100%;
        }

        :host([alignment='inline']:not([column-count])) #container ::slotted(hi-input-group),
        :host([alignment='hort']:not([column-count])) #container ::slotted(hi-input-group) {
          flex: 1 1 0;
          width: 0;
        }

        :host([alignment='inline']) #container ::slotted(hi-input-group),
        :host([alignment='hort']) #container ::slotted(hi-input-group) {
          min-width: calc(var(--hi-px-48) * 2);
        }

        :host([alignment='hort'][column-count]) #container ::slotted(hi-input-group) {
          width: var(--hi-form--input-column-width);
        }

        :host([flush]) #container {
          padding: 0;
        }

        :host([inline-edit]) #formContainer {
          display: inline-flex;
        }

        :host([inline-edit]) #controlsContainer {
          margin: 0;
          justify-content: center;
          align-self: center;
          display: flex;
        }

        @media (min-width: 0px) and (max-width: 40rem) {
          :host([alignment='inline']) #container,
          :host([alignment='inline']) #controlsContainer,
          :host([alignment='hort']:not([column-count])) #container,
          :host([alignment='hort']:not([column-count])) #controlsContainer {
            flex-direction: column;
          }

          :host([alignment='inline']) #formContainer {
            flex-direction: column;
            display: flex;
          }
        }
      </style>
      <hi-value-group id="valueGroup" ns="[[ ns ]]" form="[[ form ]]" input="[[ input ]]" secure="[[ secure ]]>
        <slot id="modifiers" name="modifiers"></slot>
      </hi-value-group>
      <div id="formContainer">
        <hi-validation-notice value="[[ __additionalValidations ]]"></hi-validation-notice>
        <div id="container">
          <slot id="items"></slot>
        </div>
        <div id="controlsContainer">
          <slot id="controls" name="controls"></slot>
        </div>
      </div>
    `;
  }
  static get api() {
    return ["submit", "reset", "invoke", "refresh", "cancel"];
  }
  static get autoBinding() {
    return {
      signal: "output-changed",
      field: "input"
    };
  }
  static get observers() {
    return [
      "__submitOutputWatcher(output, disableHttp)",
      "__calculateInputColumnWatcher(columnCount)",
      "__validationWatcher(novalidate)",
      "__kindModeWatcher(kind, mode)",
      "__methodWatcher(method)"
    ];
  }
  connectedCallback() {
    super.connectedCallback();
    this.__onEvaluateEnded__valueGroupBound = this.__onEvaluateEnded__valueGroup.bind(this);
    this.__onSlotChanged__controlsBound = this.__onSlotChanged__controls.bind(this);
    this.__onTriggerEnded__submitBound = this.__onTriggerEnded__submit.bind(this);
    this.__onTriggerEnded__resetBound = this.__onTriggerEnded__reset.bind(this);
    this.__onTriggerEnded__cancelBound = this.__onTriggerEnded__cancel.bind(this);
    this.addEventListener("keyup", this.__onKeyUp__enter);
    this.$.valueGroup.addEventListener("evaluate-ended", this.__onEvaluateEnded__valueGroupBound);
    this.$.valueGroup.addEventListener("submit-ended", (event) => {
      event.stopPropagation();
    });
    this.$.valueGroup.addEventListener("reset-ended", (event) => {
      event.stopPropagation();
    });
    this.$.controls.addEventListener("slotchange", this.__onSlotChanged__controlsBound);
    afterNextRender(this, this.__onSlotChanged__controlsBound);
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    this.$.valueGroup.removeEventListener("evaluate-ended", this.__onEvaluateEnded__valueGroupBound);
    this.$.controls.removeEventListener("slotchange", this.__onSlotChanged__controlsBound);
    this.__cleanupDynamicHTTP();
  }
  get inputs() {
    return HIP.db.form.queryAll(this.form, this.ns);
  }
  get __inputs() {
    return this.inputs;
  }
  evaluate() {
    return this.$.valueGroup.evaluate("form-submitted");
  }
  validateCustomValidity(errors) {
    f1.each((el) => {
      const elName = el.name.charAt(0) === "." ? el.name.slice(1) : el.name;
      const errorList = errors[elName];
      if (el.hidden != true && errorList != null && errorList.length > 0) {
        errorList.__used__ = true;
        el.setCustomValidity(errorList.join(";"));
        el.validate(false);
      } else {
        el.resetCustomValidity();
        el.hideValidation();
        el.invalid = false;
      }
    }, this.inputs);
    this.__writeVariablesToAdditionalValidations(errors);
    this.__focusOnFirstInvalidElement();
  }
  invoke() {
    const output = this.output;
    let action;
    if ($ip.flags.is("/eip/hi-form/default-to-rest", true) || this.disableHttp) {
      action = Promise.resolve(output);
    } else if (f1.is.empty(output)) {
      action = Promise.reject(new Error("hi-http: empty form"));
    } else {
      const hadHTTP = !!this.$.http;
      const httpEl = this.$.http = this.$.http || document.createElement("hi-http");
      httpEl.ns = this.ns;
      httpEl.uri = this.uri;
      httpEl.method = this.method;
      httpEl.kind = this.endpoint;
      if (!hadHTTP) {
        this.appendChild(this.$.http);
      }
      action = this.$.http.invoke(output);
    }
    return action;
  }
  clearValidations() {
    this.__additionalValidations = [];
    f1.each((el) => {
      el.resetCustomValidity();
      el.hideValidation();
      el.invalid = false;
    }, this.inputs);
  }
  submit() {
    this.__propagateNotRequired();
    this.validateInputs();
    this.__focusOnFirstInvalidElement();
    this.__submitToValueGroup();
    return this.output;
  }
  validateInputs() {
    this.inputs.forEach((inputEl) => {
      inputEl.checkValidity();
    });
  }
  reset() {
    this.clearValidations();
    this.__firstInput()?.focus();
    this.$.valueGroup.reset();
  }
  refresh(_input = this.input) {
    this.$.valueGroup.refresh(_input);
    this._dispatchProcess("update", {
      value: this.input
    });
    return this.input;
  }
  cancel() {
    return this.__slotChildValue;
  }
  _onSlotChanged() {
    super._onSlotChanged();
    inputService.initializeInputs(this);
    this.refresh();
    this.__propagateNotRequired();
  }
  __inputChanged(input) {
    return this.refresh(input);
  }
  __externalSubmitChanged(externalSubmit) {
    this.$.submitButton = HIP.browser.queryElement(externalSubmit, this.ns);
  }
  __externalResetChanged(externalReset) {
    this.$.resetButton = HIP.browser.queryElement(externalReset, this.ns);
  }
  __kindModeWatcher(kind, mode) {
    const kindToRole = {
      filter: "search",
      submit: "form"
    };
    const role = mode === "display" ? "article" : kindToRole[kind] || "form";
    this.setAttribute("role", role);
  }
  __calculateInputColumnWatcher(columnCount) {
    const safeColumnCount = columnCount <= 0 ? 1 : columnCount;
    const columnWidth = 100 / safeColumnCount;
    this.style.setProperty("--hi-form--input-column-width", `calc(${columnWidth}% - 1rem)`);
  }
  __validationWatcher(novalidate) {
    inputService.applyValidation(this, novalidate);
  }
  __submitOutputWatcher(output) {
    if (output != null) {
      this.invoke();
    }
  }
  __methodWatcher(method) {
    this.$.valueGroup.kind = method.toLowerCase() === "patch" ? "recent" : "all";
  }
  __aestheticChanged(aesthetic) {
    if (aesthetic == "ghost") {
      this.aesthetic = "invisible";
    }
  }
  __alignmentChanged(alignment) {
    if (alignment === "vert") {
      this.alignment = "stacked";
    }
  }
  __writeVariablesToAdditionalValidations(errors) {
    this.__additionalValidations = f1.map(
      (errorList, variable) => {
        const from = variable.split("_").map(f1.capitalize).join(" ");
        return errorList.map((str) => `${from} - ${str}`);
      },
      f1.filter((errorList) => {
        return !errorList.__used__;
      }, errors)
    );
  }
  __propagateNotRequired() {
    if (this.kind !== "filter") {
      return;
    }
    f1.each((el) => {
      el.required = false;
    }, this.inputs);
  }
  __submitToValueGroup() {
    const submission = this.$.valueGroup.submit();
    const submissionValid = submission != null;
    if (submissionValid) {
      this.output = submission;
    } else {
      throw new InternalError(`form submission failed`, this.$.valueGroup.submission);
    }
  }
  __focusOnFirstInvalidElement() {
    const firstInvalidInput = this.__firstInvalidInput();
    if (firstInvalidInput) {
      firstInvalidInput.focus();
    }
  }
  __firstInvalidInput() {
    return this.inputs.find((el) => el.invalid || el.hasAttribute("invalid"));
  }
  __firstInput() {
    const directInputEl = this.inputs.find((el) => typeof el.validate === "function");
    const inputGroupEl = this.inputs.find((el) => typeof f1.read("/set/input/validate", null, el) === "function");
    return directInputEl || inputGroupEl?.grouping.input;
  }
  __onKeyUp__enter(event) {
    if (event.key !== "Enter") {
      return;
    }
    const keyElement = event.target?.closest("[x-hip]");
    const allowEnter = keyElement.disallowEnter == null || keyElement.disallowEnter === false;
    if (allowEnter) {
      this.submit();
    }
  }
  __onEvaluateEnded__valueGroup(event) {
    const { valueAll } = event.detail.value;
    if (this.kind === "filter") {
      this.__assignFilterOutput = Debouncer.debounce(this.__assignFilterOutput, timeOut.after(64), () => {
        this.output = valueAll;
      });
    }
  }
  __onTriggerEnded__submit() {
    this.submit();
  }
  __onTriggerEnded__reset() {
    this.reset();
  }
  __onTriggerEnded__cancel() {
    this.cancel();
  }
  __cleanupDynamicHTTP() {
    if (this.$.http) {
      this.$.http.disconnectedCallback();
      this.$.http = void 0;
    }
  }
  __onSlotChanged__controls() {
    this.removeAttribute("has-controls");
    this.$.controls.assignedNodes({
      flatten: false
    }).filter(HIP.util.isNode).filter((el) => !el.hasAttribute("exclude")).forEach((el) => {
      let eventListener;
      const attr = el.getAttribute("type");
      if (attr === "submit") {
        eventListener = this.__onTriggerEnded__submitBound;
        this.setAttribute("has-controls", "");
        this.$.submitButton = el;
      } else if (attr === "cancel") {
        eventListener = this.__onTriggerEnded__cancelBound;
        this.setAttribute("has-controls", "");
        this.$.cancelButton = el;
      } else if (attr === "reset") {
        eventListener = this.__onTriggerEnded__resetBound;
        this.setAttribute("has-controls", "");
        this.$.resetButton = el;
      }
      if (eventListener) {
        el.addEventListener(el.autoBinding.signal, eventListener);
      }
    });
  }
}
export default HIP.factory.element(FormElement);
