
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { idlePeriod, timeOut } from '../../../hip/library/polymer/polymer.js';
import { Debouncer } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorContainer } from '../../../eip/element/hi-behavior-container/hi-behavior-container.js';
import { BehaviorIO } from '../../../eip/element/hi-behavior-io/hi-behavior-io.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import '../../../eip/element/hi-checkbox/hi-checkbox-group.js';
import '../../../eip/element/hi-object/hi-object-stamp.js';
import '../../../eip/element/hi-checkbox/hi-checkbox.js';
import '../../../eip/element/hi-button/hi-button.js';
import '../../../eip/element/hi-icon/hi-icon.js';
import './hi-column.js';
import './hi-row.js';
export class GridElement extends BehaviorIO(BehaviorContainer(BehaviorElement(AsHIC(PolymerElement)))) {
  constructor() {
    super();
    this.respondToModifiers = true;
  }
  static get is() {
    return "hi-grid";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          --hi-grid--row-line-height: var(--hi-table--row-height);
          --hi-grid--height: calc(var(--hi-table--row-height) + var(--hi-table--row-height) * 8);
          --hi-grid-background: var(--hi-clr--white);
        }

        :host {
          display: block;
          background: var(--hi-grid-background);
        }

        table {
          border-collapse: separate;
          border-spacing: 0;
          width: 100%;
        }

        :host([sticky-header]) table {
          position: relative;
        }

        :host([sticky-header]) th {
          background: var(--hi-clr--white);
          z-index: var(--hi-depth--content);
          position: sticky;
          top: 0;
        }

        th,
        td {
          border-right: var(--hi-px-1) solid var(--hi-clr--gray-4);
          font-size: var(--hi-font-size--text);
          line-height: var(--hi-font-lh--text);
          color: var(--hi-clr--text);
          text-align: left;
        }

        th:last-of-type,
        td:last-of-type {
          border-right: none;
        }

        th {
          letter-spacing: 0.025em;
          white-space: nowrap;
          padding: 0 var(--hi-px-6);
          text-transform: capitalize;
          font-variant: small-caps;
        }

        tr {
          background-color: var(--hi-clr--white);
        }

        :host(:not([aesthetic='invisible'])) tr:nth-child(even) {
          background-color: var(--hi-clr--stripe);
        }

        td {
          padding: 0 var(--hi-px-6);
        }

        th:last-of-type,
        td:last-of-type {
          border-right: none;
        }

        tr,
        td,
        th {
          height: var(--hi-table--row-height);
        }

        td {
          font-weight: var(--hi-font--weight-light);
        }

        thead th {
          border-bottom: var(--hi-px-1) solid var(--hi-clr--gray-6);
        }

        :host([aesthetic='invisible'][row-stripe='2']) tbody tr:nth-child(2n) {
          background-color: var(--hi-clr--stripe);
        }

        :host([aesthetic='invisible'][row-stripe='3']) tbody tr:nth-child(3n) {
          background-color: var(--hi-clr--stripe);
        }

        :host([aesthetic='invisible'][row-stripe='5']) tbody tr:nth-child(5n) {
          background-color: var(--hi-clr--stripe);
        }

        #container {
          display: none;
        }

        :host([aesthetic='invisible'][allow-sort]) th {
          cursor: default;
        }

        th[allow-sort],
        th[allow-sort] span {
          cursor: default;
        }

        th:not([allow-sort]) [purpose='columnSort'] {
          display: none;
        }

        th [x-style='columnHint'] {
          position: relative;
          top: var(--hi-px-2);
        }

        hi-button[purpose='columnSort'] {
          margin-left: var(--hi-px-2);
        }

        caption {
          font: 0 / 0 a;
          color: transparent;
          text-shadow: none;
          background-color: transparent;
          border: 0;
        }

        [x-row-select],
        [x-row-activate] {
          text-align: center;
          display: none;
          width: 0;
        }

        :host([allow-row-select]) [x-row-select],
        :host([allow-row-activate]) [x-row-activate] {
          display: table-cell;
          width: 3rem;
        }

        [x-row-select] [internal-grid] {
          margin-bottom: var(--hi-px-3);
        }

        :host([aesthetic='standard']) table {
          border: var(--hi-px-1) solid var(--hi-clr--gray-4);
        }
      </style>
      <hi-checkbox-group
        id="rowSelected__checkboxGroup"
        on-state-changed="__rowSelected__checkboxGroup__onStateChanged"
        group$="[[ gridAutoId ]]"
        max="[[ maxRowSelect ]]"
        ns$="[[ ns ]]"
      >
      </hi-checkbox-group>
      <table id="table" role="grid">
        <caption id="caption" hidden$="[[ !caption ]]">
          [[ caption ]]
        </caption>
        <thead>
          <tr>
            <th x-row-select scope="col" class$="[[ headerClass ]]">
              <hi-checkbox
                id="rowSelected__checkboxControl"
                on-state-changed="__controlCheckbox__onStateChanged"
                kind="group"
                ns="[[ ns ]]"
                theme="[[ theme ]]"
                internal-grid
              >
              </hi-checkbox>
            </th>
            <th x-row-activate scope="col" class$="[[ headerClass ]]"></th>
            <template
              id="headersEl"
              is="dom-repeat"
              index-as="headerIndex"
              items="[[ __columns ]]"
              as="header"
              rendered-item-count="[[renderedItemCount]]"
              target-framerate="15"
              initial-count="[[initialCount]]"
              delay="128"
            >
              <th
                id$="[[ header.id ]]"
                scope="col"
                width$="[[ __headerWidth(header) ]]"
                allow-sort$="[[ __headerAllowSort(header) ]]"
                data-sort-focus$="[[ __headerSortFocus(header, sortBy, sortOrder) ]]"
                on-click="__onClick_header"
                class$="[[ __headerClassName(header) ]]"
                aria-sort$="[[ __headerAriaSort(header, sortBy, sortOrder) ]]"
              >
                <span> [[ header.heading ]] </span>
                <span x-style="columnHint" hidden$="[[ !header.hint ]]">
                  <hi-icon
                    id$="[[ header.id ]]_column_hint"
                    gid$="[[ header.id ]]_column_hint"
                    kind="info_circle"
                    mode="content"
                    type="material"
                  ></hi-icon>
                  <hi-tooltip for$="[[ header.id ]]_column_hint" placement="top">[[ header.hint ]]</hi-tooltip>
                </span>
                <hi-button aesthetic="icon" purpose="columnSort" value="[[ header.id ]]" ns="[[ ns ]]" size="small">
                  <hi-icon
                    x-style="sort-button"
                    aria-label$="Toggle sort for column [[ header.heading ]]"
                    kind$="[[ __headerSortIcon(header, sortBy, sortOrder) ]]"
                    direction$="[[ __headerSortDirection(header, sortBy, sortOrder) ]]"
                    type="material"
                  >
                  </hi-icon>
                </hi-button>
              </th>
            </template>
          </tr>
        </thead>
        <tbody id="contentArea" x-style="scroll-container">
          <template
            id="rowsEl"
            is="dom-repeat"
            as="row"
            index-as="rowIndex"
            rendered-item-count="[[renderedItemCount]]"
            target-framerate="15"
            initial-count="[[initialCount]]"
            delay="128"
            items="[[ output ]]"
          >
            <tr id$="[[ __selectedRowId(rowIndex) ]]" data-qa-id$="[[ __rowId(row) ]]">
              <td id$="[[ __cellSelectId(rowIndex, row) ]]" x-row-select>
                <hi-checkbox
                  group$="[[ gridAutoId ]]"
                  ns$="[[ ns ]]"
                  theme="[[ theme ]]"
                  data-index$="[[ __rowIndex(row) ]]"
                  checked="[[ __isRowSelected(row) ]]"
                  on-checked-changed="__onRowSelectionChanged"
                  internal-grid
                  aria-labelledby$="[[ __selectedRowId(rowIndex) ]]"
                >
                </hi-checkbox>
              </td>
              <td id$="[[ __cellActivateId(rowIndex, row) ]]" x-row-activate>
                <hi-button
                  ns="[[ ns ]]"
                  aesthetic="icon"
                  theme="[[ theme ]]"
                  purpose="rowActivate"
                  value="[[ __rowIndex(row) ]]"
                >
                  <hi-icon kind="[[ rowActivateIcon ]]"> </hi-icon>
                </hi-button>
              </td>
              <template is="dom-repeat" items="[[ __columns ]]" index-as="cellIndex" as="cell">
                <td
                  id$="[[ __cellId(rowIndex, cellIndex, row, cell) ]]"
                  data-column$="[[ __columnName(cell) ]]"
                  data-qa$="[[ __cellQA(rowIndex, cellIndex, row, cell) ]]"
                >
                  <hi-object-stamp
                    template="[[ __columnToTemplate(cell) ]]"
                    cell_index="[[ cellIndex ]]"
                    row_index="[[ rowIndex ]]"
                    reference="[[ cell ]]"
                    object="[[ row ]]"
                    ns="[[ ns ]]"
                  >
                  </hi-object-stamp>
                </td>
              </template>
            </tr>
          </template>
        </tbody>
      </table>

      <!-- collect all column and row tags -->
      <div id="container">
        <slot id="items"></slot>
      </div>
    `;
  }
  static get properties() {
    return {
      __forceGrid: {
        type: Number,
        value: 0
      },
      caption: {
        type: String,
        value: ""
      },
      values: {
        type: Array,
        notify: true,
        computed: "__valuesCompute(selectedIndices, output)"
      },
      selectedIndices: {
        type: Array,
        computed: "__selectedIndicesCompute(__selectedIndices, allowRowSelect, __forceGrid)"
      },
      __selectedIndices: {
        type: Object,
        value: () => {
        }
      },
      allowColumns: {
        type: Array,
        value: () => []
      },
      orderColumns: {
        type: Array,
        value: () => []
      },
      __columnsDB: {
        type: Object,
        computed: "__columnsDBCompute(__columns, allowColumns)"
      },
      __columns: {
        type: Array,
        value: () => []
      },
      __rows: {
        type: Array,
        value: () => []
      },
      __modifiersDB: {
        type: Object,
        computed: "__modifiersDBCompute(__modifiers)"
      },
      __modifiers: {
        type: Array,
        value: () => []
      },
      input: {
        type: Array,
        value: () => []
      },
      inputSize: {
        type: Number,
        notify: true,
        computed: "__inputSizeCompute(input)"
      },
      outputSize: {
        type: Number,
        notify: true,
        computed: "__outputSizeCompute(output)"
      },
      sortBy: {
        type: String
      },
      sortOrder: {
        type: String,
        value: "asc"
      },
      allowRowSelect: {
        type: Boolean,
        reflectToAttribute: true
      },
      rowSelectMax: {
        type: Number,
        value: -1
      },
      maxRowSelect: {
        type: Number,
        value: -1
      },
      allowRowActivate: {
        type: Boolean,
        reflectToAttribute: true
      },
      disallowSort: {
        type: Boolean,
        reflectToAttribute: true
      },
      rowActivateIcon: {
        type: String,
        value: "eye"
      },
      refreshDelay: {
        type: Number,
        value: 64
      },
      rowStripe: {
        type: Number,
        reflectToAttribute: true,
        value: 2
      },
      renderedItemCount: {
        type: Number,
        reflectToAttribute: true,
        value: 100
      },
      initialCount: {
        type: Number,
        reflectToAttribute: true,
        value: 50
      },
      stickyHeader: {
        type: String
      },
      headerClass: {
        type: String,
        value: "",
        reflectToAttribute: true
      },
      gridAutoId: {
        type: String,
        value: () => `grid__${f1.id()}`
      },
      aesthetic: {
        type: String,
        value: "standard",
        reflectToAttribute: true
      }
    };
  }
  static get observers() {
    return [
      "__rowsWatcher(__rows)",
      "__outputWatcher(output)",
      "__gridWatcher(allowColumns, orderColumns)",
      "__columnsWatcher(__columns, allowRowActivate, allowRowSelect)",
      "__deprecatedWatcher(allowRowActivate, rowActivateIcon, rowSelectMax)"
    ];
  }
  static get api() {
    return ["trigger", "pump", "clearValues"];
  }
  static get valueEvent() {
    return "output-changed";
  }
  static get autoBinding() {
    return {
      signal: "output-changed",
      field: "input"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "group");
    this.addEventListener("child-attribute-notified", this.__onChildAttributeNotified);
    this.addEventListener("modified-notified", this.__onModifierModified);
    this.addEventListener("value-notified", this.__onValueNotified);
    this.__onValueFocusOutBound = this.__onValueFocusOut.bind(this);
    this.$.contentArea.addEventListener("focusout", this.__onValueFocusOutBound, true);
    this._includeGlobalStyles();
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    this.$.contentArea.removeEventListener("focusout", this.__onValueFocusOutBound, true);
    this.__onValueFocusOutBound = void 0;
  }
  __onChildAttributeNotified() {
    this.__onSlotChanged();
    this.__forceGrid += 1;
  }
  __onSlotChanged() {
    this.refresh();
    this.__modifiers = [];
    this.__columns = [];
    this.__rows = [];
    this.__onSlotChanged_debouncer = Debouncer.debounce(this.__onSlotChanged_debouncer, idlePeriod, () => {
      this.__modifiers = this.__getModifiers();
      this.__columns = this.__getColumns();
      this.__rows = this.__getRows();
    });
  }
  __rowsWatcher(__rows) {
    this.input = this.input.length <= 0 ? __rows.filter((el) => el.evaluate).map((el) => el.evaluate()) : this.input;
  }
  __deprecatedWatcher(allowRowActivate, rowActivateIcon, rowSelectMax) {
    if (rowSelectMax != null && rowSelectMax >= 0) {
      this.maxRowSelect = rowSelectMax;
    }
  }
  __outputWatcher(output) {
    if (!output || output.length <= 0 || !Array.isArray(output)) {
      return;
    }
    const selectedIndices = this.__selectedIndices = this.__selectedIndices || {};
    output.filter((o) => o).forEach((o, index) => {
      const isSelected = selectedIndices[index];
      selectedIndices[index] = isSelected == null ? false : isSelected;
      o.__index__ = index;
    });
    this.$.table.setAttribute("aria-rowcount", output.length + "");
  }
  __onModifierModified() {
    this.__updateInterface();
  }
  __updateInterface() {
    this.pump();
    this.$.headersEl.render();
    this.$.rowsEl.render();
  }
  __outputSizeCompute(output) {
    return !output ? 0 : output.length;
  }
  __inputSizeCompute(input) {
    HIP.fn.announce(`${this.caption} grid recently updated.`, "assertive");
    return !input ? 0 : input.length;
  }
  __getModifiers() {
    return this.slotChildren.filter((node) => typeof node.modify === "function");
  }
  __getRows() {
    return this.slotChildren.filter((node) => node.nodeName === "HI-ROW" || node.nodeName === "HI-OBJECT");
  }
  __gridWatcher() {
    this.__columns = this.__getColumns();
  }
  __getColumns() {
    const allowColumns = this.allowColumns;
    const orderColumns = this.orderColumns;
    return this.slotChildren.filter((node) => node.nodeName === "HI-COLUMN").filter((node) => !node.hidden).filter((node) => allowColumns.length <= 0 ? true : allowColumns.indexOf(node.id) >= 0).map((node) => {
      if (orderColumns.length > 0) {
        const orderIndex = orderColumns.indexOf(node.id);
        node.order = orderIndex >= 0 ? orderIndex : 0;
      }
      return node;
    }).sort((a, b) => a.order - b.order);
  }
  __columnsWatcher(_columns, allowRowActivate, allowRowSelect) {
    const _columnCount = (_columns || []).length || 1;
    let actionColumns = 0;
    if (allowRowActivate) {
      actionColumns += 1;
    }
    if (allowRowSelect) {
      actionColumns += 1;
    }
    this.$.table.setAttribute("aria-colcount", _columnCount + actionColumns + "");
  }
  __onValueFocusOut(e) {
    const value = e.target.value;
    const el = e.srcElement;
    if (el === this) {
      return;
    }
    e.preventDefault();
    if (el.hasAttribute("internal-grid")) {
      return;
    } else if (!el.__dataHost) {
      return;
    }
    this.__renderColumn(this.__updateInputCell(value, el));
  }
  __renderColumn(columnEl) {
    if (columnEl && columnEl.hasAttribute("force-render")) {
      this.__onForceRender_debouncer = Debouncer.debounce(
        this.__onForceRender_debouncer,
        timeOut.after(this.refreshDelay),
        () => {
          this.__updateInterface();
        }
      );
    }
  }
  __controlCheckbox__onStateChanged(e) {
    this.$.rowSelected__checkboxGroup.state = e.detail.value;
  }
  __rowSelected__checkboxGroup__onStateChanged(e) {
    this.$.rowSelected__checkboxControl.state = e.detail.value;
  }
  __onValueNotified(e) {
    this.__onValueNotified_debouncer = Debouncer.debounce(this.__onValueNotified_debouncer, idlePeriod, () => {
      this.__onValueNotifiedDebounced(e);
    });
  }
  __onValueNotifiedDebounced(e) {
    const detail = e.detail;
    const el = detail.srcElement;
    if (el === this) {
      return;
    }
    e.stopPropagation();
    e.preventDefault();
    if (el.nodeName === "HI-BUTTON" && el.getAttribute("purpose") === "rowActivate") {
      const rowIndex = parseInt(el.value);
      return this._dispatchProcess("activate-row", {
        row: this.output[rowIndex],
        rowIndex
      });
    } else if (el.nodeName === "HI-BUTTON" && el.getAttribute("purpose") === "columnSort") {
      const columnId = el.value;
      return this.toggleSort(columnId, this.__columnsDB[columnId]);
    } else if (el.hasAttribute("internal-grid")) {
      return;
    } else if (!el.__dataHost) {
      return;
    }
    this.__renderColumn(this.__updateInputCell(detail.value, el));
  }
  __updateInputCell(value, el) {
    const objectTemplateEl = el.__dataHost || {};
    const tdEl = el.parentElement || objectTemplateEl.parentElement;
    if (!tdEl) {
      return value;
    }
    const tdId = tdEl.getAttribute("id");
    if (!tdId) {
      return value;
    }
    const [_0, row, col, index, cellId] = tdId.split("-");
    this.set(`output.${index}.${cellId}`, value);
    return this.__columnsDB[cellId];
  }
  __columnToTemplate(cell) {
    return this.__columnsDB[cell.id].children.length > 0 ? this.__columnsDB[cell.id].children[0] : void 0;
  }
  __modifiersDBCompute(__modifiers) {
    const db = {};
    __modifiers.forEach((modifier) => {
      db[modifier.purpose] = modifier;
    });
    return db;
  }
  __columnsDBCompute(columns) {
    const db = {};
    columns.forEach((column) => {
      db[column.id] = column;
    });
    return db;
  }
  __cellId(rowIndex, cellIndex, row, cell) {
    return row && cell ? `cell-${rowIndex}-${cellIndex}-${row.__index__}-${cell.id}` : "";
  }
  __cellQA(rowIndex, cellIndex, row, cell) {
    return row && cell ? row[cell.id] : "";
  }
  __columnName(cell) {
    return cell ? cell.id : "";
  }
  __cellSelectId(rowIndex, row) {
    return row ? `cellSelect-${rowIndex}-${row.__index__}-select` : "";
  }
  __cellActivateId(rowIndex, row) {
    return row ? `cellActivate-${rowIndex}-${row.__index__}-activate` : "";
  }
  __rowId(row) {
    return row ? row.id : "";
  }
  __selectedRowId(rowIndex) {
    return `rowSelected-${rowIndex}`;
  }
  __rowIndex(row) {
    return row.__index__;
  }
  __isRowSelected(row) {
    return this.__selectedIndices[row.__index__];
  }
  __valuesCompute(selectedIndices) {
    const output = f1.safe({}, this.output);
    return f1.filter(
      f1.identity,
      f1.map((index) => output[index], selectedIndices)
    );
  }
  __selectedIndicesCompute(__selectedIndices, allowRowSelect) {
    const selectedIndices = [];
    if (!allowRowSelect) {
      return selectedIndices;
    }
    for (const index in __selectedIndices) {
      if (__selectedIndices[index]) {
        selectedIndices.push(index);
      }
    }
    return selectedIndices;
  }
  __onRowSelectionChanged(e) {
    if (this.__selectedIndices == null) {
      return;
    }
    if (e.target) {
      const index = e.target.getAttribute("data-index");
      const value = e.detail.value;
      this.__selectedIndices[index] = value;
      this.__forceGrid += 1;
    }
  }
  __headerSortFocus(header, sortBy) {
    return header.id === sortBy;
  }
  __headerAriaSort(header, sortBy, sortOrder) {
    return header.id === sortBy ? sortOrder === "asc" ? "ascending" : "descending" : "none";
  }
  __headerAllowSort(header) {
    return !this.disallowSort && header.allowSort;
  }
  __headerWidth(header) {
    return header.width != null ? header.width : "";
  }
  __headerSortIcon(header, sortBy) {
    return header.id === sortBy ? "chevron_up" : "expand_vertical";
  }
  __headerSortDirection(header, sortBy, sortOrder) {
    if (header.id !== sortBy) {
      return "up";
    }
    return sortOrder === "asc" ? header.id === sortBy ? "up" : "down" : header.id === sortBy ? "down" : "up";
  }
  __headerClassName(header) {
    return !!header && !!header.className ? header.className : this.headerClass;
  }
  __onClick_header(e) {
    this.toggleSort(e.currentTarget.id, this.__columnsDB[e.currentTarget.id]);
  }
  clearValues() {
    this.$.rowSelected__checkboxControl.value = false;
  }
  clearSelections() {
    this.$.rowSelected__checkboxGroup.values = [];
  }
  toggleSort(columnId, column) {
    if (this.disallowSort) {
      return;
    } else if (!column.allowSort) {
      return;
    }
    this.set("sortBy", columnId);
    this.set("sortOrder", this.sortOrder === "asc" ? "desc" : "asc");
    this._dispatchProcess("request-sort", {
      sortOrder: this.sortOrder,
      sortBy: this.sortBy
    });
    const sortOrder = this.sortOrder;
    if (this.__modifiersDB["sort"]) {
      const sortModifier = this.__modifiersDB["sort"];
      sortModifier.set("order", sortOrder);
      sortModifier.set("field", columnId);
    }
  }
}
HIP.factory.element(GridElement);
