
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { afterNextRender } from '../../../hip/library/polymer/polymer.js';
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { BehaviorElement, AsHIC, HIP } from '../../../eip/element/hi-component/hi-component.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
export const styles = css`
  :host {
    --hi-hint--max-width: 30em;
    --hi-hint--min-width: 25ch;
    --hi-hint--min-height: calc(1em + 2 * var(--hi-px-4));
  }

  :host {
    display: block;
    position: absolute;
    border: var(--hi-px-1) solid var(--hi-clr--theme-5);
    border-radius: var(--hi-shape--radius-2);
    background-color: var(--hi-clr--white);
    color: var(--hi-clr--text);
    min-height: var(--hi-hint--min-height);
    max-width: var(--hi-hint--max-width);
    min-width: var(--hi-hint--min-width);
    font-weight: var(--hi-font--weight-light);
    font-size: var(--hi-font-size--text);
    box-shadow: var(--hi-clr--shadow-down-3);
    z-index: var(--hi-depth--above-all);
    padding: var(--hi-px-12);
    text-rendering: optimizeLegibility;
    pointer-events: none;
    animation: tooltip-fade 200ms cubic-bezier(0.2, 0, 0.38, 0.9);
  }

  :host(:not([multiline])) {
    --hi-hint--min-width: 1ch;
    white-space: nowrap;
    display: inline-block;
    padding: var(--hi-px-4) var(--hi-px-8);
  }

  :host(:not([multiline])) #container {
    line-height: var(--hi-font-lh--small);
  }

  :host([theme]) {
    background-color: var(--hi-clr--theme-3);
    color: var(--hi-clr--theme-text-3);
  }

  :host([placement^='top']) {
    transform: translate(-50%, -100%);
    top: calc(-1 * var(--hi-px-8));
    left: 50%;
  }

  :host([placement^='bottom']) {
    transform: translate(-50%, var(--hi-px-8));
    top: 100%;
    left: 50%;
  }

  :host([placement^='right']) {
    transform: translate(var(--hi-px-8), -50%);
    left: 100%;
    top: 50%;
  }

  :host([placement^='left']) {
    transform: translate(calc(-1 * var(--hi-px-8)), -50%);
    right: 100%;
    top: 50%;
  }

  @keyframes tooltip-fade {
    0% {
      opacity: 0;
    }
    100% {
      opacity: 1;
    }
  }
`;
export class HintElement extends BehaviorElement(AsHIC(LitElement)) {
  constructor() {
    super(...arguments);
    this.placement = "top";
  }
  static get is() {
    return "hi-hint";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      placement: {
        type: String,
        reflect: true
      },
      multiline: {
        type: Boolean,
        reflect: true
      },
      content: {
        type: String
      }
    };
  }
  updated(changedProperties) {
    if (changedProperties.has("content")) {
      this.__contentWatcher(this.content);
    }
  }
  connectedCallback() {
    super.connectedCallback();
    dom.write(() => {
      dom.ensureAttr(this, "id");
      dom.setAttrs(this, {
        hover: "child",
        role: "tooltip"
      });
      const parentEl = this["parentElement"];
      if (parentEl) {
        parentEl.setAttribute("hover", "parent");
        if (!parentEl.hasAttribute("x-hip")) {
          parentEl.style.textDecoration = "underline dashed";
          parentEl.style["textDecorationSkipInk"] = "none";
        }
      }
    });
    afterNextRender(this, () => {
      this.__applyLabelToIconParent();
    });
  }
  render() {
    return html`
      <div id="container" mixin="text-natural">
        <slot id="content"></slot>
      </div>
    `;
  }
  __contentWatcher(content) {
    const len = `${content}`.length;
    const max = 25;
    this.multiline = len > max;
    this.innerHTML = content;
  }
  __applyLabelToIconParent() {
    const parentEl = this.parentElement;
    if (!parentEl) {
      return;
    }
    const hasIconAesthetic = parentEl.getAttribute("layout") === "icon" || parentEl.getAttribute("aesthetic") === "icon";
    if (!hasIconAesthetic) {
      return;
    }
    dom.includeAttrList(parentEl, "aria-label", this.innerHTML);
  }
}
export default HIP.factory.element(HintElement);
