
import { afterNextRender } from '../../../hip/library/polymer/polymer.js';
import { dedupingMixin } from '../../../hip/library/polymer/polymer.js';
import { ServiceNamespace } from './hi-service-namespace.js';
import { HIP } from './hi-component-base.js';
import ess from '../../../pip/library/pi-ess/pi-ess.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export function BaseBehaviorElement(Base) {
  class BehaviorElement2 extends Base {
    static get properties() {
      return {
        theme: {
          type: String,
          reflectToAttribute: true,
          reflect: true
        },
        layout: {
          type: String,
          reflectToAttribute: true,
          reflect: true
        },
        size: {
          type: String,
          reflectToAttribute: true,
          reflect: true
        },
        ns: {
          type: String,
          reflectToAttribute: true,
          observer: "_nsChanged",
          reflect: true
        },
        disabled: {
          type: Boolean,
          observer: "_disabledChanged",
          reflectToAttribute: true,
          reflect: true,
          notify: true
        },
        hidden: {
          type: Boolean,
          reflectToAttribute: true,
          reflect: true
        },
        readonly: {
          type: Boolean,
          reflectToAttribute: true,
          reflect: true
        },
        autoId: {
          type: Boolean,
          observer: "_autoIdChanged",
          attribute: "auto-id",
          reflect: true
        },
        bind: {
          type: String,
          observer: "_bindChanged",
          reflect: true
        },
        gid: {
          type: String,
          observer: "_gidChanged",
          reflectToAttribute: true,
          reflect: true
        },
        purpose: {
          type: String,
          reflectToAttribute: true,
          reflect: true
        }
      };
    }
    static get api() {
      return ["announce", "warn"];
    }
    connectedCallback() {
      super.connectedCallback();
      dom.write(() => {
        dom.setAttr(this, "x-hip", this.localName);
        HIP.db.gid.registerElementByQuery(this);
        HIP.db.gid.registerWith(this, this.gid);
      });
    }
    disconnectedCallback() {
      super.disconnectedCallback();
      HIP.db.gid.unregisterElementByQuery(this);
      HIP.db.gid.unregisterWith(this, this.gid);
      this.unbind();
    }
    unbind() {
      if (HIP.fn.unbind) {
        this.bindings.forEach((binding) => {
          HIP.fn.unbind(binding);
        });
      }
      this.__bindings = void 0;
    }
    reflection() {
      const Class = this.constructor;
      return {
        gestureBindings: Class.gestureBindings,
        browserBindings: Class.browserBindings,
        keyBindings: Class.keyBindings,
        properties: Class.properties,
        behaviors: this.behaviors,
        api: Class.api,
        valueProperty: this.valueProperty,
        autoBinding: this.autoBinding,
        valueEvent: this.valueEvent
      };
    }
    get autoBinding() {
      return this.constructor.autoBinding || {};
    }
    get valueEvent() {
      return this.constructor.valueEvent || "value-changed";
    }
    get valueProperty() {
      return this.constructor.valueProperty || "value";
    }
    get behaviors() {
      return this.constructor.behaviors || {};
    }
    get componentName() {
      return this.localName;
    }
    get bindings() {
      return this.__bindings || [];
    }
    get siblings() {
      const parentEl = this.parentElement;
      if (!parentEl) {
        return [];
      }
      const __siblings = parentEl.children || parentEl["slotChildren"] || [];
      const _siblings = Array.from(__siblings);
      return _siblings.filter((el) => el !== this);
    }
    announce(text) {
      if (HIP.fn.announce) {
        HIP.fn.announce(text);
      }
    }
    announceNow(text) {
      if (HIP.fn.announceNow) {
        HIP.fn.announceNow(text);
      }
    }
    warn(warning, reporter = console) {
      return HIP.fn.warn.call(this, warning, reporter);
    }
    update(props) {
      super.update(props);
      const elProps = this.constructor.elementProperties;
      if (!elProps) {
        return;
      }
      props.forEach((valueOld, prop) => {
        this.__invokeReactiveProperty(elProps, prop, valueOld, this[prop]);
      });
    }
    __invokeReactiveProperty(elProps, prop, valueOld, value) {
      const hasMeta = elProps.get(prop);
      if (!hasMeta) {
        return;
      }
      if (hasMeta.observer && f1.is.string(hasMeta.observer) && value !== valueOld) {
        this[hasMeta.observer].call(this, value, valueOld);
      }
      if (hasMeta.notify) {
        this._dispatchEvent(`${f1.form("camel", "kebab", prop)}-changed`, { value });
      }
    }
    updated(props) {
    }
    _watchChanges(props, watchers) {
      props.forEach((_, prop) => {
        watchers.filter((v) => v.properties.includes(prop)).forEach((v) => {
          this[v.method](...v.properties.map((k) => this[k]));
        });
      });
    }
    _nsChanged(ns, nsOld) {
      const hasInvalidGidForReg = !f1.is.string(this.gid);
      if (hasInvalidGidForReg) {
        return;
      }
      const hasInvalidOldNS = f1.is.nil(nsOld);
      if (hasInvalidOldNS) {
        nsOld = ServiceNamespace["defaultNS"];
      }
      HIP.db.gid.unregisterWith(this, this.gid, nsOld);
      HIP.db.gid.registerWith(this, this.gid, ns);
    }
    _gidChanged(gid, gidOld) {
      HIP.db.gid.unregisterWith(this, gidOld || gid);
      HIP.db.gid.registerWith(this, gid);
    }
    _bindChanged(rules) {
      this.unbind();
      if (HIP.fn.bind) {
        this.__bindings = ((rules || "").replace(/\s/g, "").split(";") || []).map((rule, index) => {
          const [sink, field, signal] = rule.split(":");
          return HIP.fn.bind(this, void 0, signal, sink, field);
        });
      } else {
        setTimeout(() => this._bindChanged(rules), 32);
      }
    }
    _disabledChanged(disabled) {
    }
    _autoIdChanged(autoId) {
      const autoIdPrefix = `${dom.constants.autoPrefix}-`;
      const hasId = this.hasAttribute("id");
      if (autoId && !hasId) {
        dom.ensureAttr(this, "id");
      } else if (!autoId && hasId) {
        if (`${this.getAttribute(`id`)}`.indexOf(autoIdPrefix) === 0) {
          this.removeAttribute(`id`);
        }
      }
    }
    _dispatchNotification(type, detail = {}) {
      return this._dispatchBubblingEvent(`${type}-notified`, detail);
    }
    _dispatchBubblingEvent(type, detail = {}) {
      if (!detail.srcElement) {
        detail.srcElement = this;
      }
      return this.dispatchEvent(
        new CustomEvent(type, {
          composed: true,
          bubbles: true,
          detail
        })
      );
    }
    _dispatchFailure(type, detail = {}) {
      return this._dispatchEvent(`${type}-failed`, detail);
    }
    _dispatchProcess(type, detail = {}) {
      return this._dispatchEvent(`${type}-ended`, detail);
    }
    _dispatchModification(el, detail = {}) {
      if (!detail.srcElement) {
        detail.srcElement = this;
      }
      return el.dispatchEvent(
        new CustomEvent("external-modification", {
          bubbles: false,
          composed: true,
          detail
        })
      );
    }
    _dispatchEvent(type, detail = {}) {
      if (!detail.srcElement) {
        detail.srcElement = this;
      }
      return this.dispatchEvent(
        new CustomEvent(type, {
          bubbles: false,
          composed: true,
          detail
        })
      );
    }
    _getParentsOnEvent(event) {
      if (f1.is.in("path", event)) {
        return event.path;
      } else if (f1.is.in("composedPath", event) && f1.is.function(event.composedPath)) {
        return event.composedPath();
      } else if (f1.is.in("target", event)) {
        return this.__createParentsOnEvent(event.target);
      }
      return [];
    }
    createParentsOnEvent(element) {
      return this.__createParentsOnEvent(element);
    }
    __createParentsOnEvent(target) {
      const path = [];
      while (target && target.parentNode) {
        path.push(target);
        target = target.parentNode;
      }
      path.push(document);
      return path;
    }
    _includeGlobalStyles() {
      if (this.__includeGlobalStylesGuard) {
        return;
      }
      const styleEl = Array.from(document.querySelectorAll("link[rel=stylesheet]"));
      const isToolkitCSS = /\/.*?yip\/.*?.css/;
      styleEl.filter((e) => e.href.match(isToolkitCSS)).forEach((styleEl2) => {
        if (styleEl2 && this.shadowRoot) {
          this.shadowRoot.prepend(styleEl2.cloneNode(true));
          this.__includeGlobalStylesGuard = true;
        }
      });
    }
    _onKeyAny__preventDefault(event) {
      event.preventDefault();
    }
    _noop() {
    }
    _deprecated(reason) {
      ess.ele(this, {
        action: "deprecated",
        data: reason
      });
      return this;
    }
    _includeQAMarker() {
      if (this.hasAttribute("data-qa")) {
        return;
      }
      afterNextRender(this, function() {
        const innerText = this.innerText;
        if (innerText && innerText.trim() !== "") {
          this.setAttribute("data-qa", innerText.trim());
        }
      });
    }
    toString() {
      return this.componentName;
    }
  }
  return HIP.factory.mixin(BehaviorElement2);
}
export const BehaviorElement = dedupingMixin(BaseBehaviorElement);
HIP["BehaviorElement"] = BehaviorElement;
