
import { idlePeriod, microTask, animationFrame } from '../../../hip/library/polymer/polymer.js';
import { setPassiveTouchGestures } from '../../../hip/library/polymer/polymer.js';
import { afterNextRender } from '../../../hip/library/polymer/polymer.js';
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { Debouncer } from '../../../hip/library/polymer/polymer.js';
import { ServiceBrowser } from './hi-service-browser.js';
import { ServiceHooks } from './hi-service-hooks.js';
import { ServiceDatabases } from './hi-service-db.js';
import { ServiceNamespace } from './hi-service-namespace.js';
import { HIServiceFactory } from './hi-service-factory.js';
import { HIVisitor } from './hi-service-visitor.js';
import { ServiceUtilities } from './hi-service-utilities.js';
import { Kernel } from './hi-service-data.js';
import { ServiceAOP } from './hi-service-aop.js';
import $ip from '../../../pip/library/pi-ip/pi-ip.js';
import ess from '../../../pip/library/pi-ess/pi-ess.js';
setPassiveTouchGestures(true);
class HIPGlobal {
  constructor() {
    this.__elements = {};
    this.instances = {};
    this.a11y = {};
    this.visitor = new HIVisitor();
    this.kernel = new Kernel();
    this.data = new Kernel();
    this.util = new ServiceUtilities();
    this.browser = new ServiceBrowser();
    this.hooks = new ServiceHooks();
    this.db = new ServiceDatabases();
    this.ns = new ServiceNamespace();
    this.factory = new HIServiceFactory();
    this.aop = new ServiceAOP();
    this._ = $ip.configuration;
    this.extern = {
      Polymer: {
        PolymerElement,
        afterNextRender,
        template: {
          polymer: html
        },
        debounce(el, debouncer, fn, rule = "idlePeriod") {
          let ruleFn = microTask;
          const ruleFns = {
            animationFrame,
            idlePeriod,
            microTask
          };
          if (ruleFns[rule]) {
            ruleFn = ruleFns[rule];
          }
          return el[debouncer] = Debouncer.debounce(el[debouncer], ruleFn, fn);
        }
      }
    };
    this.fn = {
      warn(warning, reporter = console) {
        ess.ele(this, {
          action: "warn",
          reason: warning,
          status: "warning"
        });
        return false;
      },
      instance(Element) {
        const elementName = Element.is;
        const tagNameList = document.getElementsByTagName(elementName);
        if (tagNameList && tagNameList.length > 0) {
          return tagNameList[0];
        }
        const elements = document.querySelectorAll(elementName);
        if (elements && elements.length > 0) {
          return elements[0];
        }
        const element = document.createElement(elementName);
        document.body.appendChild(element);
        return element;
      },
      unbind(bindingEl) {
      },
      bind(el, source, signal, sink, field) {
        return el;
      },
      announce(text, politeness) {
      },
      announceNow(text) {
      },
      format(value, formatter, formatterRule = {}) {
        return "";
      },
      parse(value, parser, parserRule = {}) {
      },
      prompt(confirmation) {
        return new Promise((response) => {
        });
      },
      confirmation(confirmation) {
        return new Promise((response) => {
        });
      },
      notify(packet) {
        return new Promise((response) => {
        });
      },
      safeEval() {
      },
      createSubView(viewEl, subViewRequest) {
      }
    };
    this.Polymer = this.extern.Polymer;
    this.version = "24.2.4";
    this.release_date = "2/21/2024, 3:22:13 PM";
  }
  ver() {
    return Promise.resolve(this.whoami());
  }
  whoami() {
    const whoami = `hip ${this.version} on ${this.release_date}`;
    console.debug(whoami);
    return whoami;
  }
}
const isDefined = window["HIP"] != null;
const HIP = isDefined ? window.HIP : new HIPGlobal();
if (!isDefined) {
  HIP.whoami();
  Object.defineProperty(window, "HIP", {
    configurable: false,
    enumerable: false,
    value: HIP
  });
}
export { HIP };
