
import '../../../hip/library/polymer/polymer.js';
import ess from '../../../pip/library/pi-ess/pi-ess.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class InternalError extends Error {
  constructor(message, context) {
    super(message);
    this.context = context;
  }
}
export class ServiceAOP {
  hijack(elClass, method, hijackFn) {
    return this.pointCut(elClass, method, hijackFn);
  }
  pointCut(elClass, method, adviceFn) {
    const proto = typeof elClass === "string" ? window.HIP.factory.queryOne(elClass).prototype : elClass.prototype;
    if (f1.is.function(proto[method])) {
      proto[method] = adviceFn(proto[method]);
    }
    return proto[method];
  }
  makeReactive(elClass) {
    if (!elClass.hasOwnProperty("api")) {
      return elClass;
    }
    const proto = elClass.prototype;
    const api = elClass.api || [];
    const camelToKebab = f1.form("camel", "kebab");
    api.forEach((methodName) => {
      proto[methodName] = function(_methodEventName, fn) {
        return function(...args) {
          const event = {
            ns: this.ns,
            site: this.localName,
            source: this.id || this.gid || this.toString(),
            action: methodName,
            producer: this,
            data: {
              error: void 0,
              value: void 0,
              args
            }
          };
          let value;
          try {
            value = fn.apply(this, args);
          } catch (error) {
            event.data.error = error;
            value = error.context;
            ess.send(event, "failure");
            this._dispatchFailure(_methodEventName, {
              value,
              args
            });
            return value;
          }
          event.data.value = value || this.__data;
          if (value instanceof Promise) {
            return value.then((_value) => {
              event.data.value = _value;
              ess.send(event, "info");
              this._dispatchProcess(_methodEventName, {
                value: _value,
                args
              });
              return _value;
            }).catch((error) => {
              event.data.error = error;
              ess.send(event, "failure");
              this._dispatchFailure(_methodEventName, {
                error
              });
              return Promise.reject(error);
            });
          } else {
            ess.send(event, "info");
            this._dispatchProcess(_methodEventName, {
              value,
              args
            });
            return value;
          }
        };
      }(camelToKebab(methodName), proto[methodName]);
    });
    return elClass;
  }
  valueNotifier(elClass) {
    const classProto = elClass.prototype;
    const _connectedCallback = classProto.connectedCallback;
    if (!classProto.valueEvent) {
      return elClass;
    }
    classProto.connectedCallback = function connectedCallbackValueNotifier(...args) {
      _connectedCallback.apply(this, args);
      this.addEventListener(this.valueEvent, (e) => {
        const detail = e.detail;
        const value = detail.value;
        if (value != null && value !== "") {
          this.dirty = true;
        }
        this._dispatchBubblingEvent("value-notified", {
          valueFormatted: detail.valueFormatted || void 0,
          value
        });
        if (this.valueEventChannel) {
          const channelAttr = this.getAttribute(this.valueEventChannel);
          if (channelAttr) {
            window.HIP.hooks.invoke(`value-notified-${channelAttr}`, this, e);
          }
        }
      });
    };
    return elClass;
  }
}
