
import '../../../hip/library/polymer/polymer.js';
import ess from '../../../pip/library/pi-ess/pi-ess.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class ServiceBrowser {
  constructor() {
    this.__queryGlobal = {
      document: [document],
      window: [window],
      tt: [globalThis["tt"]]
    };
  }
  queryOne(query, ns, root = document.body) {
    if (!query) {
      return void 0;
    } else if (typeof query !== "string") {
      return query;
    }
    const nodes = this.queryAll(query, ns, root);
    if (nodes.length > 1) {
      ess.send(
        {
          ns: `${ns}`,
          site: "hi-component",
          source: "service-browser",
          action: "queryOne",
          data: {
            reason: "greater than one match",
            query
          }
        },
        "warning"
      );
    }
    return f1.nth(nodes.length - 1, nodes);
  }
  queryAll(query, ns, root = document.body) {
    try {
      if (!query) {
        return [];
      } else if (typeof query !== "string") {
        return query;
      }
      const _query = query.trim();
      if (_query === "hip_fn") {
        return [window.HIP.fn];
      }
      const globalQuery = this.__queryGlobal[_query];
      if (globalQuery) {
        return globalQuery;
      }
      const directQuery = window.HIP.db.gid.queryAll(_query, ns);
      if (directQuery.length > 0) {
        return directQuery;
      }
      const elementById = document.getElementById(_query);
      if (elementById) {
        return [elementById];
      }
      const documentQuery = document.querySelectorAll(_query);
      if (!!documentQuery && documentQuery.length > 0) {
        return Array.from(documentQuery);
      }
      const elementQuery = this.__querySelectorParentSiblings(_query, root);
      if (elementQuery) {
        return [elementQuery];
      }
      return [];
    } catch (e) {
      return [];
    }
  }
  __querySelectorParentSiblings(query, root) {
    let element;
    if (!query) {
      return null;
    }
    let _root = root;
    while (_root) {
      element = _root.host && `#${_root.host.id}` === query ? _root.host : _root.querySelector(query);
      _root = _root.parentElement || _root.parentNode;
      if (element) {
        return element;
      }
    }
    return null;
  }
  queryElementId(query, ns) {
    return this.queryOne(this.__ensureId(query), this.__getNS(ns), this);
  }
  __ensureId(query) {
    if (!query) {
      return "";
    } else if (query[0] === "#") {
      return query;
    } else {
      return `#${query}`;
    }
  }
  queryElement(query, ns) {
    return this.queryOne(query, this.__getNS(ns), this);
  }
  queryElements(query, ns) {
    return this.queryAll(query, this.__getNS(ns), this);
  }
  __getNS(ns) {
    return f1.is.string(ns) ? ns : this.ns;
  }
  setFocusOnMain(selector) {
    const target = document.querySelectorAll(selector)[0];
    if (!target) {
      return;
    }
    dom.write(() => {
      dom.setAttr(target, "tabindex", "-1");
    });
    target.addEventListener(
      "blur focusout",
      function() {
        this.removeAttribute("tabindex");
      },
      {
        once: true
      }
    );
    target.focus();
  }
  setFocusOnFirstOf(selector, rootEl = document.body) {
    const firstFocusEl = rootEl.querySelector(selector);
    if (firstFocusEl) {
      this.setFocusOnFirstOfRoot(firstFocusEl);
    }
  }
  setFocusOnFirstOfRoot(rootEl = document.body) {
    const children = dom.queryFocusable(rootEl);
    if (children.length >= 0 && children[0]) {
      children[0].focus();
    }
  }
  findFocusable(rootEl = document.body) {
    return dom.queryFocusable(rootEl);
  }
  get queryMain() {
    return dom.query.main;
  }
  get queryFocusable() {
    return dom.query.focusable;
  }
}
