
import '../../../hip/library/polymer/polymer.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import ifn from '../../../pip/library/pi-ifn/pi-ifn.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
export class Kernel {
  assignTo(el, key, value, event) {
    return this.assign(el, key, value, event);
  }
  evaluateExpression(expression, hostEl, selfInstance) {
    return this.resolveExpression(expression, hostEl, selfInstance);
  }
  coerce(value, type) {
    switch (type) {
      case "number":
        return Number.parseFloat(`${value}`);
      case "string":
        return (value || "") + "";
      case "boolean":
        return !!value;
      default:
        return value;
    }
  }
  get symbolTuple() {
    return ":";
  }
  get symbolDot() {
    return ".";
  }
  isInstruction(ref) {
    return f1.is.string(ref);
  }
  isExpression(ref) {
    return f1.is.string(ref) && this.hasExpressionForm(ref);
  }
  isReference(ref) {
    return f1.is.string(ref) && this.hasReferenceForm(ref) && this.hasReferenceCharacters(ref);
  }
  isPath(ref) {
    return f1.is.string(ref) && ref.match(/^\.[a-z0-9_.-]*$/) != null;
  }
  hasExpressionForm(ref) {
    return ref.match(/\(\(.*?:.*?:.*?\)\)/) != null;
  }
  hasReferenceForm(ref) {
    return ref.match(/:.*?:/) != null;
  }
  hasReferenceCharacters(ref) {
    return ref.match(/[a-z0-9:._-]*/i) != null;
  }
  assign(el, key, value, event) {
    if (key[0] === "_") {
      return;
    } else if (key === "text") {
      return el.textContent = `${value}`;
    } else if (key === "html") {
      return el.innerHTML = `${value}`;
    }
    const isComplexKey = key.indexOf(".") >= 0;
    const [subKey, ...propertiesList] = key.split(".") || [];
    const properties = (propertiesList || []).join(".");
    const camelKey = `${f1.form("kebab", "camel", subKey)}.${properties}`;
    const subObject = f1.read(f1.path(camelKey), void 0, el);
    const subKeyObject = f1.read(f1.path(subKey), void 0, el);
    const complexValue = isComplexKey ? f1.write(f1.path(properties), value, {}) : value;
    if (f1.is.function(subKeyObject)) {
      return subKeyObject.call(el, complexValue, event);
    } else if (f1.is.function(subObject)) {
      return subObject.call(el, complexValue, event);
    } else if (isComplexKey) {
      return f1.write(f1.path(camelKey), value, el);
    }
    const kebabKey = f1.form("camel", "kebab", key);
    const hasHTMLContract = !!el.setAttribute;
    if (!hasHTMLContract) {
      return f1.write(f1.path(camelKey), value, el);
    } else if (value == null) {
      el.removeAttribute(kebabKey);
    } else if (f1.is.string(value) || f1.is.number(value)) {
      dom.setAttr(el, kebabKey, `${value}`);
    } else if (f1.is.boolean(value)) {
      dom.flipAttr(el, kebabKey, value, kebabKey);
    } else {
      return f1.write(f1.path(camelKey), value, el);
    }
    return value;
  }
  concatenate(original, inclusion) {
    const detectValue = original || inclusion;
    if (this.__plainObject(detectValue)) {
      return ifn.deepMerge({}, ifn.deepMerge(original || {}, inclusion || {}));
    } else if (f1.is.array(detectValue)) {
      return [...original || [], ...inclusion || []];
    } else if (f1.is.string(detectValue)) {
      return (original || "") + inclusion;
    } else if (f1.is.number(detectValue)) {
      return (original || 0) + inclusion;
    } else if (f1.is.boolean(detectValue)) {
      return original || inclusion;
    } else {
      return original;
    }
  }
  attributeToValue(attrValue) {
    if (attrValue === "true") {
      return true;
    } else if (attrValue === "false") {
      return false;
    } else if (attrValue === "undefined") {
      return void 0;
    } else if (attrValue === "null") {
      return null;
    } else {
      const _valueNum = parseFloat(attrValue);
      if (_valueNum === _valueNum) {
        return _valueNum;
      }
    }
    return attrValue;
  }
  sortComparator(a, b, property, order, type) {
    let _a = property ? a[property] : a;
    let _b = property ? b[property] : b;
    const _typeA = type || typeof _a;
    const _typeB = type || typeof _b;
    if (_a == null) {
      _a = 0;
    }
    if (_b == null) {
      _b = 0;
    }
    const isBoolOrNumA = (_typeA === "boolean" || _typeA === "number") && _a === _a;
    const isBoolOrNumB = (_typeB === "boolean" || _typeB === "number") && _b === _b;
    if (isBoolOrNumA || isBoolOrNumB) {
      _a += 1;
      _b += 1;
      return order === "asc" ? _a - _b : _b - _a;
    } else if (_typeA === "string" || _typeB === "string") {
      _a = (_a + "").toLowerCase();
      _b = (_b + "").toLowerCase();
      return order === "asc" ? _a < _b ? -1 : _a > _b ? 1 : 0 : _a > _b ? -1 : _a < _b ? 1 : 0;
    } else {
      return 0;
    }
  }
  filterComparator(o, property, value, comparator) {
    return this.runComparator(comparator, value, property ? o[property] : o);
  }
  runComparator(comparator, leftValue, rightValue) {
    const comparatorDefault = (comparator || "fallback").trim().toLowerCase();
    const isNot = comparatorDefault.indexOf("not") >= 0;
    const comparatorRequest = isNot ? comparatorDefault.replace("not", "") : comparatorDefault;
    if (null == leftValue || null == rightValue) {
      return leftValue == rightValue;
    }
    if ("string" === typeof leftValue) {
      leftValue = leftValue.toLowerCase();
    }
    if ("string" === typeof rightValue) {
      rightValue = rightValue.toLowerCase();
    }
    const comparatorRenderType = {
      render_type: comparatorRequest.trim()
    };
    const comparatorVisitor = window.HIP.visitor.queryField(
      comparatorRenderType,
      "comparator"
    );
    if (!comparatorVisitor) {
      return false;
    }
    const result = comparatorVisitor.compare(leftValue, rightValue);
    return isNot ? !result : result;
  }
  eventAsValueOf(eventKey, object, defaultValue) {
    const isNotifiedEvent = eventKey.indexOf("-notified") > 0;
    const isChangedEvent = eventKey.indexOf("-changed") > 0;
    if (!isNotifiedEvent && !isChangedEvent) {
      return defaultValue;
    }
    const eventAttribute = eventKey.replace("-changed", "").replace("-notified", "");
    return this.invokeOn(eventAttribute, object, defaultValue);
  }
  eventAsMethodOf(eventKey, object, defaultValue) {
    const isMethodEvent = eventKey.indexOf("-ended") > 0;
    if (!isMethodEvent) {
      return defaultValue;
    }
    const eventAttribute = eventKey.replace("-ended", "");
    return this.invokeOn(eventAttribute, object, defaultValue);
  }
  invokeOn(eventKey, object, defaultValue) {
    const eventProperty = f1.form("kebab", "camel", eventKey);
    let value;
    value = f1.read(f1.path(eventProperty), void 0, object);
    if (typeof value === "function") {
      return value.call(object);
    } else if (value != null) {
      return value;
    }
    value = f1.read(f1.path(`output.${eventProperty}`), void 0, object);
    if (typeof value === "function") {
      return value.call(object);
    } else if (value != null) {
      return value;
    }
    return defaultValue;
  }
  applyConditions(children, input, owner) {
    return this.__collectElementsOf(children, "condition").reduce((_output, mod) => {
      if (_output == null) {
        return void 0;
      }
      try {
        return mod.condition(_output, owner);
      } catch (e) {
        return void 0;
      }
    }, input);
  }
  applyModifiers(children, input, owner) {
    return this.__collectElementsOf(children, "modify").reduce(
      (_output, mod) => mod.modify(_output, owner),
      window.HIP.util.clone(input)
    );
  }
  applyExpressions(el) {
    return this.__collectElementsOf(el.children, "evaluate").forEach((o) => {
      o.evaluate(el);
    });
  }
  hasMutator(children) {
    return this.__collectElementsOf(children, "modify").reduce(
      (_hasMutator, child) => _hasMutator || !!child.mutator,
      false
    );
  }
  runTasks(children, value, event) {
    return this.__collectElementsOf(children, "invoke").reduce(
      (promise, invocation) => promise.then(() => invocation.invoke(value, event)),
      Promise.resolve(void 0)
    ).then(() => value);
  }
  resolveExpression(expression, hostEl, selfInstance) {
    const isExpression = f1.is.string(expression) || f1.is.array(expression);
    if (!isExpression) {
      return expression;
    }
    const expressionList = this.normalizeExpression(expression);
    if (expressionList.length <= 0) {
      return void 0;
    }
    const expressionValues = expressionList.map((v) => {
      return f1.is.string(v) ? v : !v.instance ? f1.read(f1.path(`${v.field}${v.property}`), void 0, f1.safe({}, selfInstance)) : this.resolveReference(v, hostEl);
    });
    return expressionValues.reduce((s, value) => {
      if (s == null) {
        return s;
      } else if (value instanceof Error) {
        return void 0;
      }
      return s + value;
    }, "");
  }
  normalizeExpression(expression) {
    if (f1.is.array(expression)) {
      return expression;
    } else if (!f1.is.string(expression)) {
      return [];
    }
    const findReference = /\(\(.*?\)\)/g;
    const references = expression.match(findReference);
    const literals = expression.split(findReference);
    const n = literals.length;
    const list = [];
    let i = 0;
    for (; i < n; i += 1) {
      if (!literals || literals.length <= 0) {
        break;
      }
      const literal = literals.shift();
      if (literal != null) {
        list.push(literal);
      }
      if (!references || references.length <= 0) {
        break;
      }
      const reference = references.shift();
      if (reference == null) {
        break;
      }
      list.push(this.normalizeReference(reference));
    }
    return list;
  }
  assignToReference(ns, reference, value) {
    const { instance, field, property } = this.normalizeReference(reference);
    const instanceEl = window.HIP.browser.queryOne(instance, ns);
    const fieldPath = field || instanceEl && instanceEl.autoBinding.field || "";
    const fullPath = property ? `${fieldPath}.${property}` : fieldPath;
    return this.assignTo(instanceEl, fullPath, value);
  }
  normalizeReference(reference) {
    if (!f1.is.string(reference)) {
      return reference;
    }
    const symbolStartReference = "((";
    const symbolEndReference = "))";
    const symbolTuple = ":";
    const variable = reference.slice(reference.indexOf(symbolStartReference) + 2, reference.indexOf(symbolEndReference)).trim();
    const [instance, field, property] = variable.split(symbolTuple);
    return {
      property,
      instance,
      field
    };
  }
  promotePathToReference(reference) {
    return this.isPath(reference) ? `((::${reference}))` : reference;
  }
  resolveReference(reference, hostEl) {
    const { instance, field, property = "" } = this.parseReference(reference);
    const instanceEl = window.HIP.browser.queryOne(instance, hostEl.ns);
    if (instanceEl == null) {
      return new Error(`${instance}: no-instance`);
    }
    const isAIExpression = hostEl.parentElement && hostEl.parentElement.id === instanceEl.id;
    const defaultField = isAIExpression ? "argument" : f1.read("/constructor/autoBinding/signal", "output", instanceEl);
    const peekField = field || defaultField;
    const safeField = peekField.indexOf("-ended") >= 0 ? "output" : peekField;
    const expressionField = safeField.replace("-changed", "").replace("-ended", "");
    const _fieldValue = f1.read(`/${expressionField}`, null, instanceEl);
    if (_fieldValue == null) {
      return new Error(`${field}: no-field`);
    }
    const fieldValue = f1.is.function(_fieldValue) ? _fieldValue.call(instanceEl) : _fieldValue;
    if (fieldValue == null) {
      return new Error(`${field}: no-field-function`);
    }
    const value = property.length > 0 ? f1.read(f1.path(property), void 0, fieldValue) : fieldValue;
    return value == null ? new Error(`${property}: no-property`) : value;
  }
  parseReference(reference) {
    if ("string" === typeof reference) {
      const [instance, field, property] = reference.replace("((", "").replace("))", "").trim().split(window.HIP.kernel.symbolTuple);
      return {
        instance,
        field,
        property
      };
    }
    return reference;
  }
  __collectElementsOf(children, methodName) {
    return Array.from(children || []).filter((o) => !o.disabled).filter((o) => typeof o[methodName] === "function");
  }
  __plainObject(u) {
    return !!u && u === u && !f1.is.array(u) && !f1.is.function(u) && Object(u).constructor === Object;
  }
}
