
import '../../../hip/library/polymer/polymer.js';
import './hi-service-namespace.js';
import ess from '../../../pip/library/pi-ess/pi-ess.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class ServiceDatabase {
  constructor(attr) {
    this.__attr = attr;
    this.clean();
  }
  list(ns) {
    return f1.safe( new Map(), this.__db.get(ns));
  }
  lists() {
    return [...this.__db.keys()];
  }
  clean() {
    this.__db =  new Map();
    return this;
  }
  removeDatabase(ns) {
    this.__db.delete(ns);
  }
  forEach(fn, namespace = "default") {
    this.__forEach(window.HIP.ns.getDatabase(this.__db, namespace), fn);
    return this;
  }
  __forEach(db, fn) {
    for (const [k, _] of db) {
      const list = db.get(k);
      if (list) {
        list.forEach(fn);
      }
    }
  }
  registerWith(element, key, ns) {
    const db = this.getDatabase(element, ns);
    this.__parseKey(key).forEach((_key) => {
      this.__addKey(_key, element, db);
    });
    return this;
  }
  registerElementByQuery(element) {
    const db = this.getDatabase(element);
    this.__parseElementKey(element).forEach((_key) => {
      this.__addKey(_key, element, db);
    });
    return this;
  }
  unregisterWith(element, key, ns) {
    this.__parseKey(key).forEach((_key) => {
      this.__removeKey(_key, element, this.getDatabase(element, ns));
    });
    return this;
  }
  unregisterElementByQuery(element) {
    const db = this.getDatabase(element);
    this.__parseElementKey(element).forEach((_key) => {
      this.__removeKey(_key, element, db);
    });
    return this;
  }
  getDatabase(element, ns) {
    return window.HIP.ns.getDatabase(this.__db, window.HIP.ns.getNamespace(element, ns));
  }
  __parseElementKey(element) {
    return this.__parseKey(element[this.__attr] || element.getAttribute(this.__attr));
  }
  __parseKey(key) {
    return `${key}`.replace(/\s/g, "").split(",");
  }
  addKey(key, element, ns) {
    return this.__addKey(key, element, this.getDatabase(element, ns));
  }
  __addKey(key, element, db) {
    if (!key) {
      return;
    } else if (db.has(key)) {
      db.set(
        key,
        f1.push(
          element,
          f1.remove((el) => el === element, db.get(key))
        )
      );
    } else {
      db.set(key, [element]);
    }
  }
  removeKey(key, element, ns) {
    return this.__removeKey(key, element, this.getDatabase(element, ns));
  }
  __removeKey(key, element, db) {
    if (!key) {
      return;
    }
    const list = db.get(key);
    if (list) {
      const index = list.indexOf(element);
      if (index >= 0) {
        list.splice(index, 1);
      }
      if (list.length <= 0) {
        db.delete(key);
      }
    }
  }
  __includeDOMObjects(o) {
    return !(o instanceof HTMLElement) || o instanceof HTMLElement && o.isConnected;
  }
  queryActive(query) {
    const db = this.__db;
    let queried;
    let k;
    for ([k] of this.__db) {
      queried = this.__queryKey(this.__prepareQuery(query), db.get(k));
      if (queried.length === 0) {
        continue;
      }
      return queried && queried.length === 1 ? f1.nth(0, queried) : queried;
    }
    return [];
  }
  queryOne(query, ns = "default") {
    if (typeof query !== "string") {
      return query;
    }
    const elements = this.queryBy(query, ns);
    if (elements.length > 1) {
      ess.send(
        {
          reason: `A query against multiple results will cause run-time bugs.`,
          ns: `${ns}`,
          site: "hi-component",
          source: "service-database",
          action: "queryOne",
          data: {
            elements,
            query
          }
        },
        "fatal"
      );
    }
    return f1.nth(0, elements);
  }
  queryBy(query, ns) {
    return this.queryAll(query, ns);
  }
  queryAll(query, ns = "default") {
    if (typeof query !== "string") {
      return query;
    }
    const preparedQuery = this.__prepareQuery(query);
    if (f1.is.exists(ns)) {
      return this.__queryKey(preparedQuery, window.HIP.ns.getDatabase(this.__db, ns));
    }
    const database = this.__db;
    let queries = [];
    for (const [_, db] of database) {
      queries = queries.concat(this.__queryKey(preparedQuery, db));
    }
    return queries;
  }
  __prepareQuery(query) {
    query = query + "";
    query = query.indexOf("@") === 0 || query.indexOf("#") === 0 ? query.slice(1) : query;
    return query;
  }
  __queryKey(query, db) {
    return db ? db.get(query) || [] : [];
  }
  exists(query, ns) {
    return this.queryBy(query, ns).length > 0;
  }
}
export class ServiceDatabases {
  constructor() {
    this.form = new ServiceDatabase("form");
    this.group = new ServiceDatabase("group");
    this.gid = new ServiceDatabase("gid");
    this.binding = new ServiceDatabase("binding");
  }
  create(dbName) {
    return window.HIP.db[dbName] = new ServiceDatabase(dbName);
  }
  get(dbName) {
    return window.HIP.db[dbName] = window.HIP.db[dbName] || new ServiceDatabase(dbName);
  }
}
