
import '../../../hip/library/polymer/polymer.js';
export class ServiceHooks {
  constructor() {
    this.clean();
  }
  registerElement(el, customHooks, ns) {
    const hooks = customHooks || el.constructor.hooks || {};
    for (const hook in hooks) {
      if (el[hooks[hook]]) {
        this.add(el, hooks, hook, ns);
      }
    }
  }
  unregisterElement(el, customHooks, ns) {
    const hooks = customHooks || el.constructor.hooks || {};
    for (const hook in hooks) {
      const hookKey = this.__hookBinding(hooks, hook);
      if (el[hookKey]) {
        this.remove(el, hook, hookKey, ns);
      }
    }
  }
  clean() {
    this.__hooks =  new Map();
  }
  add(el, hooks, hook, ns) {
    const fn = el[this.__hookBinding(hooks, hook)] = el[hooks[hook]].bind(el);
    const hookListeners = this.lookupHooks(el, hook, ns);
    const doesNotExist = hookListeners.indexOf(fn) < 0;
    if (doesNotExist) {
      hookListeners.push(fn);
      this.updateHooks(el, hook, hookListeners);
    }
    return this;
  }
  remove(el, hookKey, fnKey, ns) {
    const hookListeners = this.lookupHooks(el, hookKey, ns);
    const fn = el[fnKey];
    if (hookListeners.length <= 0) {
      return this;
    }
    const index = hookListeners.indexOf(fn);
    const exists = index >= 0;
    if (exists) {
      hookListeners.splice(index, 1);
      this.updateHooks(el, hookKey, hookListeners);
    }
    el[fnKey] = void 0;
    return this;
  }
  exists(hookKey, el, fn) {
    return this.lookupHooks(el, hookKey).indexOf(fn) >= 0;
  }
  invoke(hookKey, el, ...args) {
    this.lookupHooks(el, hookKey).forEach(function(fn) {
      fn(hookKey, ...args);
    });
  }
  updateHooks(el, hookKey, hookListeners, ns) {
    this.getHooks(el, ns).set(hookKey, hookListeners);
  }
  lookupHooks(el, hookKey, ns) {
    return this.getHooks(el, ns).get(hookKey) || [];
  }
  getHooks(el, ns = window.HIP.ns.getNamespace(el)) {
    return window.HIP.ns.getDatabase(this.__hooks, ns);
  }
  __hookBinding(hooks, hook) {
    return `${hooks[hook]}BoundHook`;
  }
}
