
import '../../../hip/library/polymer/polymer.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
const _ServiceUtilities = class {
  isNode(node) {
    return dom.is.node(node);
  }
  isTag(tagName, node) {
    return dom.is.tag(tagName, node);
  }
  isTags(tagNames, node) {
    return dom.is.someTag(tagNames, node);
  }
  isCustomElement(el) {
    return dom.is.customElement(el);
  }
  isValidElement(el) {
    return dom.is.element(el);
  }
  isIncluded(el) {
    return !el.hasAttribute("exclude");
  }
  isExcluded(el) {
    return el.hasAttribute("exclude");
  }
  ensureId(el) {
    return dom.ensureAttr(el, "id");
  }
  ensureGid(el) {
    return dom.ensureAttr(el, "gid");
  }
  toggleAttribute(tag, attributeName, isOn, attributeValue) {
    return dom.flipAttr(tag, attributeName, isOn, attributeValue);
  }
  __getNewAttributeValue(el, attributeName, attributeValue) {
    const currentAttributeValue = el.getAttribute(attributeName);
    if (!currentAttributeValue) {
      return attributeValue;
    }
    if (currentAttributeValue.split(" ").includes(attributeValue)) {
      return currentAttributeValue;
    }
    return `${currentAttributeValue} ${attributeValue}`;
  }
  addAttributeValue(el, attributeName, attributeValue) {
    const newAttributeValue = this.__getNewAttributeValue(el, attributeName, attributeValue);
    dom.setAttr(el, attributeName, newAttributeValue);
    return el.getAttribute(attributeName) || "";
  }
  generateUniqueId() {
    return f1.id();
  }
  rewriteTagContent(parent, content, tagBuilder = (i) => "div") {
    const children = parent.children || [];
    const contentSize = (content || []).length;
    const childrenSize = (children || []).length;
    if (contentSize < childrenSize) {
      for (let i = childrenSize - 1; i >= contentSize; i--) {
        parent.children[i].remove();
      }
    } else if (contentSize > childrenSize) {
      for (let i = childrenSize; i < contentSize; i++) {
        parent.appendChild(document.createElement(tagBuilder(i)));
      }
    }
    Array.from(parent.children).forEach((child, index) => child.textContent = content[index]);
    return parent;
  }
  goToHref(href, target = "_self", options, replace) {
    window.open(href, target, options, replace);
    return this;
  }
  guid() {
    return f1.uuid();
  }
  doesExtend(node, behaviorName) {
    return !!f1.read(`/behaviors/${behaviorName}`, false, node);
  }
  camelToKebabCase(s) {
    return f1.form("camel", "kebab", s);
  }
  capitalize(s) {
    return f1.capitalize(s);
  }
  lowercaseFirst(s) {
    return s && s.length > 0 ? s[0].toLowerCase() + s.slice(1) : "";
  }
  setDefault(o, prop, defaultValue) {
    if (o[prop] == null) {
      o[prop] = defaultValue;
    }
  }
  clone(input) {
    if (typeof input === "function") {
      return input;
    } else if (f1.is.array(input)) {
      return Object.assign([], input);
    } else if (f1.is.object(input)) {
      return Object.assign({}, input);
    } else {
      return input;
    }
  }
  deepClone(input) {
    return f1.clone(input);
  }
  deepMerge(...objects) {
    return objects.reduce((obj, next) => f1.merge("over", obj, next), {});
  }
  existsInside(container, el) {
    return dom.existsIn(container, el);
  }
  getAllParents(el) {
    return dom.parentsOf(el);
  }
  setUniversityColor(primary, text, secondary) {
    dom.write(() => {
      const bodyStyle = document.body.style;
      if (primary) {
        bodyStyle.setProperty("--hi-clr--university-primary", primary);
        bodyStyle.setProperty("--hi-clr--university-3", primary);
        bodyStyle.setProperty("--hi-clr--university", primary);
      }
      if (text) {
        bodyStyle.setProperty("--hi-clr--university-text", text);
        bodyStyle.setProperty("--hi-clr--university-5", text);
        secondary = secondary ? secondary : primary;
      }
      if (secondary) {
        bodyStyle.setProperty("--hi-clr--university-secondary", secondary);
        bodyStyle.setProperty("--hi-clr--university-bkg", secondary);
        bodyStyle.setProperty("--hi-clr--university-1", secondary);
      }
    });
  }
  setAttributes(element, attributes = {}) {
    return dom.setAttrs(element, attributes);
  }
  applyNS(children, ns) {
    for (let i = 0, n = children.length; i < n; i++) {
      const el = children[i];
      el.setAttribute("ns", ns);
      if (el.children) {
        this.applyNS(Array.from(el.children), ns);
      }
    }
  }
  includeListAttribute(element, attributeName, value) {
    return dom.includeAttrList(element, attributeName, value);
  }
  excludeListAttribute(element, attributeName, value) {
    return dom.excludeAttrList(element, attributeName, value);
  }
  waitForLibrary(library, globalVar, options = {}) {
    const retries = _ServiceUtilities.__retries;
    const loading = _ServiceUtilities.__loading;
    const opts = {
      parent: globalThis,
      useIPToLoad: true,
      retries: 7,
      delay: 100,
      ...options
    };
    const waitForLibraryPromise = (resolve, reject) => {
      try {
        if (opts.parent[globalVar]) {
          return resolve();
        }
        if (retries[library] > opts.retries) {
          return reject();
        }
        retries[library] += 1;
        if (!opts.parent[globalVar] && loading[library]) {
          setTimeout(() => {
            waitForLibraryPromise(resolve, reject);
          }, opts.delay);
        } else if (!loading[library]) {
          loading[library] = true;
          if (opts.useIPToLoad) {
            globalThis["$ip"].require(library).then(resolve).catch(reject);
          }
        }
      } catch {
        reject();
      }
    };
    return new Promise(waitForLibraryPromise);
  }
};
export let ServiceUtilities = _ServiceUtilities;
ServiceUtilities.__retries = {};
ServiceUtilities.__loading = {};
