
import '../../../hip/library/polymer/polymer.js';
const DEFAULT_ENTITY = "global";
const FALLBACK = "fallback";
export class HIVisitor {
  constructor() {
    this._db = {};
  }
  add(v) {
    const db = this.__baselineDB(v);
    db[v.entity][v.render_type] = v;
    return this;
  }
  remove(v) {
    const db = this.__baselineDB(v);
    db[v.entity][v.render_type] = void 0;
    return this;
  }
  queryField(v, entity = v.entity || DEFAULT_ENTITY) {
    this.setFieldTypeDefaults(v);
    return this.query(v, entity);
  }
  query(v, entity = v.entity || DEFAULT_ENTITY) {
    const entityDB = this._db[entity];
    if (!entityDB) {
      return void 0;
    }
    const visitor = entityDB[`${v.visitor_type}`] || entityDB[v.render_type] || entityDB[FALLBACK];
    return visitor;
  }
  setFieldTypeDefaults(fieldType) {
    return this.setRequestDefaults(fieldType);
  }
  setRequestDefaults(fieldType) {
    const setDefault = window.HIP.util.setDefault;
    setDefault(fieldType, "entity", DEFAULT_ENTITY);
    setDefault(fieldType, "render_type", FALLBACK);
    setDefault(fieldType, "nullable", false);
    setDefault(fieldType, "blankable", false);
    setDefault(fieldType, "attributes", {});
    const titleSetting = fieldType.title || "";
    const nameSetting = fieldType.name || "";
    const idSetting = fieldType.id || "";
    const defaultName = `${nameSetting || idSetting || titleSetting}`.replace(/\s/g, "_").toLowerCase();
    const defaultTitle = `${titleSetting || nameSetting}`.replace(/_/g, " ");
    setDefault(fieldType, "name", defaultName);
    setDefault(fieldType, "title", defaultTitle);
    if (fieldType.render_type === "number") {
      setDefault(fieldType, "maximum", Number.MAX_SAFE_INTEGER);
      setDefault(fieldType, "minimum", 0);
    }
    return fieldType;
  }
  __baselineDB(visitorRequest) {
    const setDefault = window.HIP.util.setDefault;
    const db = this._db;
    setDefault(visitorRequest, "entity", DEFAULT_ENTITY);
    setDefault(visitorRequest, "render_type", FALLBACK);
    db[visitorRequest.entity] = db[visitorRequest.entity] || {};
    db[visitorRequest.entity][visitorRequest.render_type] = db[visitorRequest.entity][visitorRequest.render_type] || {};
    return db;
  }
}
