
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import { idlePeriod, microTask } from '../../../hip/library/polymer/polymer.js';
import { Debouncer } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import { BehaviorIO } from '../../../eip/element/hi-behavior-io/hi-behavior-io.js';
import navigation from '../../../pip/library/pi-navigation/pi-navigation.js';
import ess from '../../../pip/library/pi-ess/pi-ess.js';
import $ip from '../../../pip/library/pi-ip/pi-ip.js';
import io from '../../../pip/library/pi-io/pi-io.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import { XSSPurify } from '../../../pip/library/pi-io/pi-io.js';
export class HttpElement extends BehaviorIO(BehaviorElement(AsHIC(PolymerElement))) {
  constructor() {
    super(...arguments);
    this.xssPurify = new XSSPurify();
  }
  static get is() {
    return "hi-http";
  }
  static get properties() {
    return {
      kind: {
        type: String,
        value: "api"
      },
      ioProxy: {
        type: String
      },
      uri: {
        type: String,
        observer: "__uriChanged",
        reflectToAttribute: true
      },
      method: {
        type: String,
        value: "get",
        observer: "__methodChanged",
        reflectToAttribute: true
      },
      headers: {
        type: Object,
        value: () => {
          return {};
        }
      },
      params: {
        type: Object
      },
      body: {
        type: Object
      },
      disallowContext: {
        type: Boolean,
        reflectToAttribute: true
      },
      allowRedirect: {
        type: Boolean,
        reflectToAttribute: true
      },
      _requestPacket: {
        type: Object
      },
      cachePolicy: {
        type: String,
        value: "default",
        reflectToAttribute: true
      },
      responseType: {
        type: String,
        reflectToAttribute: true
      },
      responseStatus: {
        type: Number,
        notify: true,
        reflectToAttribute: true
      },
      credentials: {
        type: String,
        value: "same-origin",
        reflectToAttribute: true
      },
      auto: {
        type: Boolean,
        reflectToAttribute: true
      },
      manual: {
        type: Boolean,
        reflectToAttribute: true
      },
      modeCors: {
        type: String,
        value: "cors",
        reflectToAttribute: true
      },
      redirect: {
        type: String,
        value: "manual",
        reflectToAttribute: true
      },
      referrerPolicy: {
        type: String,
        value: "origin-when-cross-origin",
        reflectToAttribute: true
      },
      metadata: {
        type: Object,
        notify: true
      },
      allowFail: {
        type: Boolean,
        reflectToAttribute: true
      },
      response: {
        type: Object
      },
      strictUri: {
        type: Boolean,
        reflectToAttribute: true
      }
    };
  }
  static get api() {
    return ["invoke", "invokeLink"];
  }
  static get observers() {
    return ["__invokeDataWatcher(params, body, manual)"];
  }
  static get valueEvent() {
    return "output-changed";
  }
  static get autoBinding() {
    return {
      signal: "output-changed",
      field: "invoke"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("hidden", "hidden");
  }
  __uriChanged(uri, uriOld) {
    const isResourceRequest = uri.indexOf(".") >= 0;
    const hasQueryParams = uri.indexOf("?") >= 0;
    const endsWithSlash = uri.endsWith("/");
    if (!this.strictUri && !hasQueryParams && !isResourceRequest && !endsWithSlash) {
      this.uri = `${uri}/`;
    }
    if (this.manual) {
      return;
    } else if (!this.auto && uriOld == null) {
      return;
    }
    this.__uriInvokeDebouncer = Debouncer.debounce(this.__uriInvokeDebouncer, idlePeriod, () => {
      this.invoke();
    });
  }
  __invokeDataWatcher(params, body, manual) {
    if (manual) {
      return;
    }
    this.__autoInvokeDebouncer = Debouncer.debounce(this.__autoInvokeDebouncer, idlePeriod, () => {
      this.invoke();
    });
  }
  __methodChanged(_method) {
    if (_method) {
      this.method = (_method + "").toLowerCase();
    }
  }
  invoke(_input) {
    if (this.disabled) {
      return Promise.resolve();
    }
    const requestPacket = f1.is.record(_input) || f1.is.array(_input) ? _input : this.params || this.body || {};
    this._requestPacket = requestPacket;
    if (this._applyArgument) {
      this._applyArgument(requestPacket);
    }
    if (!this.uri) {
      return Promise.resolve();
    }
    if (this.uri.indexOf(".json") > 0) {
      this.disallowContext = true;
    }
    if (this.confirmation == null) {
      return this._invoke();
    }
    return HIP.fn.confirmation(this.confirmation).then(() => this._invoke()).catch(() => null);
  }
  _invoke() {
    return new Promise((resolve, reject) => {
      this.__invokeJob__ = Debouncer.debounce(this.__invokeJob__, microTask, () => {
        const isIOProxied = f1.is.exists(this.ioProxy);
        const isRedirected = this.kind === "redirect";
        const rootIO = io.root("api");
        const httpResponse = f1.is.exists(rootIO) ? this._invokeIOProxy(io.rootId("api")) : isIOProxied ? this._invokeIOProxy(this.ioProxy) : isRedirected ? this._invokeRedirectForm() : this._invokeHTTP();
        httpResponse.then(resolve).catch(reject);
      });
    });
  }
  _invokeIOProxy(ioProxy) {
    return io.create("api").include(ioProxy).invoke(this.uri, this.__requestOptions).then(({ data, metadata }) => {
      this.input = data;
      this.metadata = metadata;
      return { data, metadata };
    });
  }
  _invokeRedirectForm() {
    const requestPacket = this._requestPacket;
    const uri = this._fetchURI(this.uri);
    const formEl = document.createElement("form");
    formEl.action = uri;
    formEl.method = this.method;
    formEl.hidden = true;
    let inputEl;
    let value;
    let key;
    for (key in requestPacket) {
      value = requestPacket[key];
      inputEl = document.createElement("input");
      inputEl.type = "text";
      inputEl.value = value;
      inputEl.name = key;
      formEl.appendChild(inputEl);
    }
    document.body.appendChild(formEl);
    formEl.submit();
    return new Promise((resolve) => {
      formEl.remove();
      resolve();
    });
  }
  async _invokeHTTP() {
    const shouldPurify = $ip.flags.is("/eip/hi-http/enable-xss-purification-request");
    const requestOptions = shouldPurify ? await this.xssPurify.purify(this.__requestOptions) : this.__requestOptions;
    return fetch(this._fetchURI(this.uri), requestOptions).then((response) => {
      this.response = response;
      if (this.kind === "form" && response.redirected && response.url) {
        this.redirectedFetch = response.redirected;
        this.redirectedUrl = response.url;
      } else if (this.allowRedirect && response.type === "opaqueredirect" && response.url) {
        this.redirectedUrl = response.url;
        this.redirectedFetch = true;
      }
      this.responseStatus = response.status;
      return response;
    }).then((response) => {
      const NO_RESOURCE = 404;
      const noResponse = [NO_RESOURCE];
      const NO_CONTENT = 204;
      const emptyResponses = [NO_CONTENT];
      if (noResponse.indexOf(response.status) >= 0) {
        return Promise.reject({
          uri: this.uri,
          gid: this.gid,
          id: this.id,
          response
        });
      } else if (emptyResponses.indexOf(response.status) >= 0) {
        return {};
      }
      const responseType = this.responseType || "text";
      try {
        return response[responseType]().catch((e) => this.__repairEmptyResponseBody(responseType, e));
      } catch (e) {
        return this.__repairEmptyResponseBody(responseType, e);
      }
    }).then((value) => {
      if (this.redirectedFetch && this.kind === "form") {
        const newDOM = new DOMParser().parseFromString(value, "text/html");
        document.body.replaceWith(newDOM.body);
        document.title = newDOM.title;
        window.history.pushState(newDOM, newDOM.title, navigation.uri.write(this.redirectedUrl));
        return this.output;
      } else if (this.redirectedFetch && this.kind === "api") {
        window.open(navigation.uri.write(this.redirectedUrl), "_self");
        return this.output;
      }
      const lowestErrorResponseCode = 400;
      const isErrorResponse = this.responseStatus >= lowestErrorResponseCode;
      const isDjangoErrorResponse = isErrorResponse && value.detail != null;
      if (isDjangoErrorResponse) {
        value.metadata = {
          errors: value
        };
      }
      const metadata = value.hasOwnProperty("metadata") ? value.metadata : {};
      const data = value.hasOwnProperty("data") ? value.data : value;
      if (metadata.errors != null) {
        metadata.errors = io.fn.shared.response.metadataErrors.conformErrors(metadata.errors);
      }
      this.metadata = metadata;
      if (this.allowFail && isErrorResponse) {
        return Promise.reject({
          metadata,
          data
        });
      }
      const shouldPurify2 = $ip.flags.is("/eip/hi-http/enable-xss-purification-response");
      if (shouldPurify2) {
        return this.xssPurify.purify(data).then((data2) => {
          this.input = data2;
          return this.output;
        });
      }
      this.input = data;
      return this.output;
    });
  }
  __repairEmptyResponseBody(responseType, e) {
    switch (responseType) {
      case "json":
        return Promise.resolve({});
      case "text":
        return Promise.resolve("");
      default:
        return Promise.reject(e);
    }
  }
  _fetchURI(_uri) {
    const isFormPost = this.kind === "form" && this.method === "post";
    const isApiAndNotGet = this.kind === "api" && this.method !== "get";
    const parameters = this.disallowContext || isFormPost || isApiAndNotGet ? {} : this._requestPacket;
    return navigation.uri.write(_uri, parameters);
  }
  get __requestOptions() {
    const requestPacket = this._requestPacket;
    const defaultHeaderOptions = this.__getDefaultHeaderOptions(this.kind);
    const defaultHeaders = {
    };
    const overrideHeaders = {
      "Content-Type": this.headers["Content-Type"] || defaultHeaderOptions.contentType
    };
    this.responseType = this.responseType || defaultHeaderOptions.responseType;
    const request = {
      headers: Object.assign(defaultHeaders, this.headers, overrideHeaders),
      referrerPolicy: this.referrerPolicy,
      method: this.method.toUpperCase(),
      credentials: this.credentials,
      cache: this.cachePolicy,
      redirect: this.redirect,
      mode: this.modeCors
    };
    if (this.disallowContext) {
      return request;
    }
    if (this.kind === "form" && this.method === "post") {
      request.body = navigation.uri.parameters(requestPacket) || "";
    } else if (this.kind === "api" && this.method !== "get") {
      try {
        request.body = JSON.stringify(requestPacket) || {};
      } catch (e) {
        request.body = {};
        ess.ele(this, {
          status: "failure",
          action: "request-options",
          data: requestPacket
        });
      }
    } else if (this.kind === "api" && this.method === "get") {
      request.params = requestPacket || {};
    }
    return request;
  }
  __getDefaultHeaderOptions(kind) {
    const kindHeaderOptions = {
      form: {
        contentType: "application/x-www-form-urlencoded",
        responseType: "text"
      },
      api: {
        contentType: "application/json",
        responseType: "json"
      },
      failover: {
        contentType: "application/json",
        responseType: "json"
      }
    };
    return f1.read(`/${kind}`, kindHeaderOptions["failover"], kindHeaderOptions);
  }
}
HIP.factory.element(HttpElement);
