
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import { IconDBElement } from './hi-icon-db.js';
import './hi-icon-db.js';
export const styles = css`
  :host {
    --hi-icon--color: var(--hi-clr--theme-3);
  }

  :host {
    -webkit-tap-highlight-color: rgba(0, 0, 0, 0);
    -webkit-font-smoothing: antialiased;

    color: var(--hi-icon--color);
    fill: var(--hi-icon--color);
    vertical-align: baseline;
    display: inline-block;
    line-height: 1;
    font-size: 1em;
    height: 1em;
    width: 1em;
  }

  :host([inherit-color]) {
    --hi-icon--color: currentColor;
    color: currentColor;
    fill: currentColor;
  }

  :host([align='top']) {
    vertical-align: top;
  }

  :host([align='middle']) {
    vertical-align: middle;
  }

  :host([align='bottom']) {
    vertical-align: bottom;
  }

  svg {
    transition: transform var(--hi-animation--speed) var(--hi-animation--accel);
    transform-origin: center;
    vertical-align: inherit;
    height: 1em;
    width: 1em;
  }

  @media screen and (-ms-high-contrast: active) {
    :host,
    svg {
      filter: brightness(10) contrast(5);
    }
  }

  .material-icons-outlined {
    font-family: 'Material Icons Outlined';
    font-weight: normal;
    font-display: swap;
    font-style: normal;
    font-size: inherit;
    letter-spacing: normal;
    text-transform: none;
    display: inline-block;
    white-space: nowrap;
    word-wrap: normal;
    direction: ltr;
    font-feature-settings: 'liga';
    -webkit-font-feature-settings: 'liga';
    -webkit-font-smoothing: antialiased;
    color: var(--hi-icon--color);
    height: 1em;
    width: 1em;
    transition: transform var(--hi-animation--speed) var(--hi-animation--accel);
  }

  :host([theme='warning']),
  :host([theme='orange']),
  :host([theme='teal']),
  :host([theme='info']) {
    --hi-icon--color: var(--hi-clr--theme-5);
  }

  :host([size='small']) {
    font-size: var(--hi-px-13) !important;
  }

  :host([size='large-x']) {
    font-size: var(--hi-px-24) !important;
  }
`;
const _IconElement = class extends BehaviorElement(AsHIC(LitElement)) {
  static get is() {
    return "hi-icon";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      kind: {
        type: String,
        reflect: true
      },
      rotation: {
        type: Number,
        reflect: true
      },
      direction: {
        type: String,
        reflect: true
      },
      mode: {
        type: String,
        reflect: true
      },
      ariaLabel: {
        type: String,
        attribute: "aria-label"
      },
      type: {
        type: String,
        reflect: true
      },
      glyph: {
        type: String
      }
    };
  }
  static get autoBinding() {
    return {
      signal: "kind-changed",
      field: "kind"
    };
  }
  get value() {
    return this.kind;
  }
  iconDB() {
    return _IconElement.iconDB();
  }
  static iconDB() {
    return HIP.fn.instance(IconDBElement);
  }
  update(props) {
    super.update(props);
    if (props.has("glyph")) {
      this.kind = f1.safe(this.glyph, _IconElement.iconsStandard[this.glyph]);
    }
  }
  render() {
    if (!this.kind) {
      return nothing;
    }
    const type = this.type;
    const kind = this.kind;
    const requestedLegacy = type !== "material";
    const requestedIcon = f1.safe(kind, _IconElement.iconsExtended[kind]);
    const useUniqueSVG = _IconElement.iconsSVG.includes(requestedIcon);
    const svgIcon = requestedLegacy || useUniqueSVG ? this.iconDB().svg[kind] : void 0;
    return svgIcon ? svgIcon : html`<span class="material-icons-outlined">${requestedIcon}</span>`;
  }
  updated(props) {
    super.updated(props);
    if (props.has("direction") || props.has("rotation")) {
      this.__computeDirectionRotation();
    }
    dom.write(() => {
      this.__styleParentButton();
      if (props.has("mode") || props.has("ariaLabel")) {
        this.__reflectA11yRules();
      }
    });
  }
  __computeDirectionRotation() {
    const requestedDirection = _IconElement.compass[this.direction];
    if (f1.is.number(requestedDirection)) {
      this.rotation = requestedDirection;
    }
    const rootEl = this.shadowRoot;
    const iconEl = (rootEl.children || [])[0];
    if (iconEl) {
      iconEl.style.transform = `rotate(${this.rotation}deg)`;
    }
  }
  __styleParentButton() {
    const $parent = this.parentElement;
    if (!$parent || $parent.localName !== "hi-button") {
      return;
    }
    const isOnlyOneIcon = $parent.children.length === 1;
    if (isOnlyOneIcon) {
      $parent.setAttribute("layout", "icon");
    }
    if ($parent.hasAttribute("layout-icon")) {
      $parent.setAttribute("layout", "icon");
    }
    if ($parent.hasAttribute("layout") && $parent.getAttribute("layout") === "icon") {
      $parent.setAttribute("data-qa", this.kind);
    }
    if (!this.hasAttribute("theme")) {
      this.setAttribute("inherit-color", "inherit-color");
    }
  }
  __reflectA11yRules() {
    const mode = this.mode || (this.ariaLabel ? "content" : "presentation");
    switch (mode) {
      case "content":
        this.setAttribute("role", "img");
        this.removeAttribute("aria-hidden");
        break;
      case "presentation":
      default:
        this.removeAttribute("role");
        this.setAttribute("aria-hidden", "true");
        break;
    }
  }
  toString() {
    return `${this.kind || "unknown"}${this.title != null ? ", " : ""}${this.title || ""}`.replace(/_/g, " ").trim();
  }
};
export let IconElement = _IconElement;
IconElement.compass = {
  north: 0,
  "north-west": 45,
  west: 90,
  "south-west": 135,
  south: 180,
  "south-east": 225,
  east: 270,
  "north-east": 315,
  left: -90,
  right: 90,
  up: 0,
  top: 0,
  bottom: 180,
  down: 180
};
IconElement.iconsStandard = {
  add: "add_circle_outline",
  cancel: "cancel",
  clear: "layers_clear",
  close: "close",
  collapse: "expand_less",
  create: "add_to_photos",
  delete: "delete",
  download: "save_alt",
  edit: "edit",
  expand: "expand_more",
  favorite: "favorite_border",
  filter: "filter_alt",
  filter_filled: "filter_alt_off",
  flag: "flag",
  go_back: "arrow_back_ios",
  join: "group_add",
  logout: "logout",
  message: "message",
  move: "open_with",
  open_new_window: "open_in_new",
  print: "print",
  remove: "remove_circle_outline",
  refresh: "refresh",
  reply: "reply",
  search: "search",
  sort: "unfold_more",
  star: "star_rate",
  view: "visibility",
  zoom_in: "zoom_in",
  zoom_out: "zoom_out",
  unknown: "question_mark"
};
IconElement.iconsExtended = {
  "9_dots": "apps",
  account_box: "account_box",
  accessibility: "accessible_forward",
  alarm_clock: "alarm",
  app_switcher: "apps",
  arrow_first: "first_page",
  arrow_last: "last_page",
  arrow_solid: "play_arrow",
  bell: "notifications",
  bell_alert: "notifications_active",
  bell_off: "notifications_off",
  bell_on: "notifications_active",
  bookmark: "book",
  bubble_question: "live_help",
  calender: "today",
  cellphone: "smartphone",
  check_circle_filled: "check_box",
  checkbox_checked: "check_box",
  checkbox_checked_fill: "check_box",
  checkbox_indeterminate: "indeterminate_check_box",
  checkbox_unchecked: "check_box_outline_blank",
  chevron_down: "expand_more",
  chevron_up: "expand_less",
  circle_checked: "radio_button_checked",
  clock: "schedule",
  cross: "clear",
  cross_filled: "cancel",
  dash: "horizontal_rule",
  description: "description",
  draggable: "open_with",
  ellipsis: "more_horiz",
  empty_calendar: "calendar_today",
  expand_vertical: "unfold_more",
  external: "launch",
  external2: "launch",
  external2_copy: "launch",
  eye: "visibility",
  fact_check: "fact_check",
  folder_shared: "folder_shared",
  filter: "filter_alt",
  graduation: "school",
  heart: "favorite_border",
  heart_filled: "favorite",
  home: "home",
  info_circle: "info",
  institution_report: "newspaper",
  light_bulb: "lightbulb",
  link_broken: "link_off",
  link_cross: "link_off",
  map_marker: "room",
  minus: "remove",
  mobile: "smartphone",
  more: "more_horiz",
  network: "public",
  notebook_edit: "create",
  notebook_with_person: "contacts",
  notifications: "notifications",
  notification: "notifications",
  open_book: "local_library",
  paper_plane: "send",
  paper_plane_filled: "send",
  paperclip: "attach_file",
  people: "people",
  person_gear: "manage_accounts",
  piggy_bank: "savings",
  plus: "add",
  place: "place",
  printer: "print",
  puzzle: "extension",
  question_circle_filled: "help",
  radio_checked: "radio_button_checked",
  radio_unchecked: "radio_button_unchecked",
  reminder: "rate_review",
  road: "add_road",
  road_filled: "add_road",
  robot: "smart_toy",
  savings: "savings",
  search_v2: "search",
  share: "share",
  shrink_vertical: "unfold_less",
  sign: "directions",
  skating: "skateboarding",
  smiley: "mood",
  sort_down: "arrow_drop_down",
  sort_up: "arrow_drop_up",
  spinner: "update",
  star: "star_border",
  star_filled: "star",
  statistics: "insights",
  stop: "do_not_disturb",
  study_buddies: "group",
  success: "done_all",
  sync_cross: "sync_disabled",
  sync_warning: "sync_problem",
  telephone: "phone",
  user: "person",
  users: "group",
  utilization_report: "bar_chart",
  waving_hand: "waving_hand",
  warning: "warning_amber",
  warning_filled: "warning",
  warning_octagon: "report"
};
IconElement.iconsSVG = [
  "advanced_search",
  "advising",
  "analytics",
  "appily",
  "apple_and_checkmark",
  "blended",
  "book_with_watch",
  "calendar",
  "campaign",
  "career_ascent",
  "dashboard",
  "edit_copy",
  "envelope",
  "expand_horizontal",
  "flag_filled",
  "gear",
  "home_filled",
  "info_circle_filled",
  "list",
  "minimize",
  "pushpin",
  "question_circle",
  "report",
  "requisite",
  "rocket",
  "rocket_filled",
  "search",
  "shrink_horizontal",
  "sign_filled",
  "sort_off",
  "trash",
  "uploaded_report",
  "user_filled"
];
export default HIP.factory.element(IconElement);
