
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { ifDefined as ifDef } from '../../../hip/library/lit-html/directives/if-defined.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { BehaviorElement, AsHIC, HIP } from '../../../eip/element/hi-component/hi-component.js';
import $ip from '../../../pip/library/pi-ip/pi-ip.js';
export const styles = css`
  :host {
    position: relative;
    display: inline-block;
    min-height: var(--hi-px-24);
    min-width: var(--hi-px-24);
    max-width: 100%;
  }

  #image,
  #status_loading {
    display: none;
    width: 100%;
    height: 100%;
  }

  #image {
    margin: 0;
    max-width: 100%;
  }

  #status_loading {
    position: absolute;
    top: 50%;
    left: 50%;
    transform: translate(calc(var(--hi-px-16) / -1), calc(var(--hi-px-16) / -1));
  }

  #status_loading:after {
    content: ' ';
    display: block;
    width: var(--hi-px-16);
    height: var(--hi-px-16);
    margin: var(--hi-px-4);
    border-radius: 50%;
    border: var(--hi-px-4) solid var(--hi-clr--theme-3);
    border-color: var(--hi-clr--theme-3) var(--hi-clr--theme-3) var(--hi-clr--theme-3) transparent;
    animation: ani_loading_spinner 0.9s linear infinite;
  }

  @keyframes ani_loading_spinner {
    0% {
      transform: rotate(0deg);
    }
    100% {
      transform: rotate(360deg);
    }
  }

  :host([status='loaded']) #image {
    display: block;
    width: 100%;
  }

  :host([status='loading']) #status_loading {
    display: block;
  }
`;
export class ImageElement extends BehaviorElement(AsHIC(LitElement)) {
  constructor() {
    super(...arguments);
    this.alt = "";
  }
  static get is() {
    return "hi-image";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      uri: {
        type: String
      },
      src: {
        type: String
      },
      alt: {
        type: String
      },
      crossorigin: {
        type: String
      },
      status: {
        type: String,
        reflect: true,
        notify: true
      },
      width: {
        type: String
      },
      height: {
        type: String
      }
    };
  }
  updated(props) {
    if (props.has("uri")) {
      this.__uriChanged(this.uri);
    }
    if (props.has("src")) {
      this.__srcChanged();
    }
    if (props.has("height") || props.has("width")) {
      this.__sizeWatcher(this.height, this.width);
    }
  }
  render() {
    return html`
      <div id="status_loading" aria-label="loading"></div>
      <img
        id="image"
        src=${this.src}
        alt=${this.alt}
        crossorigin=${ifDef(this.crossorigin)}
        @load="${this.__onLoad_image}"
        @error="${this.__onError_image}"
      />
    `;
  }
  __uriChanged(uri) {
    this.src = uri;
  }
  __srcChanged() {
    this.status = "loading";
  }
  __sizeWatcher(height, width) {
    if (this.hasAttribute("height")) {
      this.style.height = `${height}px`;
    }
    if (this.hasAttribute("width")) {
      this.style.width = `${width}px`;
    }
  }
  __onLoad_image() {
    this.status = "loaded";
    this.__sizeWatcher(this.height, this.width);
    this._dispatchProcess("load", {
      status: true
    });
  }
  __onError_image() {
    this.status = "failed";
    this.src = $ip.path(`@eab-eip/assets/hi-image/404.png`);
    this._dispatchProcess("load", {
      status: false
    });
  }
}
export default HIP.factory.element(ImageElement);
