
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { BehaviorContainer } from '../../../eip/element/hi-behavior-container/hi-behavior-container.js';
import { AsHIC, BehaviorBrowserEvents, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import '../../../eip/element/hi-button/hi-button.js';
import '../../../eip/element/hi-icon/hi-icon.js';
import '../../../eip/element/hi-form/hi-form.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class InlineEditElement extends BehaviorBrowserEvents(
  BehaviorContainer(BehaviorElement(AsHIC(PolymerElement)))
) {
  static get is() {
    return "hi-inline-edit";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          display: block;
          --hi-inline-edit--height: var(--hi-px-40);
        }

        :host([auto-height]) {
          --hi-inline-edit--height: auto;
        }

        #displayTemplate {
          height: var(--hi-inline-edit--height);
          align-items: center;
          position: relative;
          border: var(--hi-px-1) solid var(--hi-clr--transparent);
          padding: var(--hi-px-8);
          border-radius: var(--hi-shape--radius);
        }

        #displayTemplate:hover,
        #displayTemplate:focus {
          border-color: var(--hi-clr--gray-7);
        }

        #displayTemplate::after {
          content: '';
          width: var(--hi-px-44);
        }

        #displayTemplate hi-button {
          position: absolute;
          right: 0;
          opacity: 0;
          z-index: -1000;
          left: -100000px;
          top: -100000px;
          height: 1px;
          width: 1px;
        }

        #displayTemplate:hover hi-button,
        #displayTemplate hi-button:focus {
          opacity: 1;
          z-index: 0;
          left: auto;
          top: auto;
          height: auto;
          width: auto;
        }

        #editMode {
          margin-right: var(--hi-px-2);
        }

        :host([kind='edit']) #displayTemplate {
          display: none;
        }

        :host([kind='edit']) #editTemplate {
          display: flex;
          font-size: inherit;
        }

        :host([kind='display']) #editTemplate {
          display: none;
        }

        :host([kind='display']) #displayTemplate {
          display: flex;
        }

        hi-button[slot='controls'] {
          height: max-content;
        }
      </style>
      <div id="displayTemplate">
        <slot name="display" id="display"></slot>
        <hi-button id="editMode" aesthetic="icon" on-trigger-ended="__editButton__onTriggerEnded">
          <hi-icon kind="edit" aria-label="Click to edit"></hi-icon>
        </hi-button>
      </div>
      <hi-form id="editTemplate" aesthetic="invisible" alignment="inline" inline-edit flush disable-http>
        <slot name="edit" id="edit"></slot>
        <hi-button
          type="submit"
          aesthetic="icon"
          slot="controls"
          theme="success"
          on-trigger-ended="__submitButton__onTriggerEnded"
        >
          <hi-icon kind="check" theme="success" aria-label="Save Changes"></hi-icon>
        </hi-button>
        <hi-button aesthetic="icon" slot="controls" theme="error" on-trigger-ended="__cancelButton__onTriggerEnded">
          <hi-icon kind="cross" theme="error" aria-label="Cancel Changes"></hi-icon>
        </hi-button>
      </hi-form>
      <slot></slot>
    `;
  }
  static get properties() {
    return {
      kind: {
        type: String,
        reflectToAttribute: true,
        observer: "__onKindChanged"
      },
      value: {
        type: Object,
        notify: true,
        observer: "__onValueChanged"
      },
      valueKey: {
        type: String,
        notify: true
      },
      autoHeight: {
        type: Boolean,
        reflectToAttribute: true
      }
    };
  }
  static get browserBindings() {
    return {
      dblclick: "__onDoubleClick"
    };
  }
  static get valueEvent() {
    return "value-changed";
  }
  static get autoBinding() {
    return {
      signal: "value-changed",
      field: "value"
    };
  }
  static get api() {
    return ["submit", "cancel", "toggle"];
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "group");
    if (!this.kind) {
      this.kind = "";
    }
    this.__onSlotChanged__updateBound = this.__onSlotChanged.bind(this);
    this.$.display.addEventListener("slotchange", this.__onSlotChanged__updateBound);
    this.$.edit.addEventListener("slotchange", this.__onSlotChanged__updateBound);
    this.$.editTemplate.addEventListener("submit-ended", (event) => {
      event.stopPropagation();
    });
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    this.$.display.removeEventListener("slotchange", this.__onSlotChanged__updateBound);
    this.$.edit.removeEventListener("slotchange", this.__onSlotChanged__updateBound);
  }
  __onSlotChanged() {
    this.__updateChildValue();
  }
  __onValueChanged() {
    this.__updateChildValue();
  }
  __updateChildValue() {
    const [editEl] = this.__querySlots();
    const valueKey = this.valueKey;
    const value = this.value;
    if (valueKey) {
      editEl.name = valueKey;
    }
    const childValue = f1.is.string(valueKey) ? f1.read(f1.path(this.valueKey), void 0, value) : value;
    const children = f1.to.array(this.children);
    f1.each((el) => el.value = childValue, children);
  }
  __submitButton__onTriggerEnded() {
    this.submit();
  }
  submit() {
    const [editEl, displayEl] = this.__querySlots();
    const editorRecord = f1.safe({}, this.value);
    const modifiedValue = editEl.tagName === "HI-COMBOBOX" ? editEl?.values || [] : editEl.value;
    const saveToKey = this.valueKey;
    displayEl.value = editEl.value;
    this.value = this.valueKey ? f1.copy(f1.write(f1.path(saveToKey), modifiedValue, editorRecord)) : modifiedValue;
    this.toggle();
    this.$.editMode.focus();
    return this.value;
  }
  __cancelButton__onTriggerEnded() {
    this.cancel();
  }
  cancel() {
    const [editEl, displayEl] = this.__querySlots();
    editEl.value = displayEl.value;
    this.toggle();
    this.$.editMode.focus();
    return this.value;
  }
  __editButton__onTriggerEnded() {
    this.toggle();
    this.__addFormAttribute();
  }
  __onDoubleClick() {
    if (this.kind === "display") {
      this.kind = "edit";
    }
  }
  toggle() {
    return this.kind = this.kind === "edit" ? "display" : "edit";
  }
  __addFormAttribute() {
    const [editEl] = this.__querySlots();
    editEl.setAttribute("form", this.$.editTemplate.$.valueGroup.form);
  }
  __onKindChanged(kind) {
    this.kind = kind === "edit" || kind === "display" ? kind : "display";
    this.__focusOnEdit();
  }
  __focusOnEdit() {
    if (this.kind === "edit") {
      const [editEl] = this.__querySlots();
      editEl.focus();
    }
  }
  __querySlots() {
    return [this.querySelector('[slot="edit"]'), this.querySelector('[slot="display"]')];
  }
}
HIP.factory.element(InlineEditElement);
