
import { html, css, LitElement } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { HIP, AsLitHIC, BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorInput } from '../../../eip/element/hi-behavior-input/hi-behavior-input.js';
import '../../../eip/element/hi-form/hi-form.js';
import '../../../eip/element/hi-input/hi-input.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export const styles = css`
  .input-list {
    list-style: none;
    padding: 0;
    margin: 0;
  }

  .input-list-item {
    display: flex;
    gap: var(--hi-px-5);
  }

  .input-list-item > hi-input {
    flex-basis: 100%;
  }

  .full-width {
    width: 100%;
  }

  .list_headers {
    width: 50%;
  }

  .list_labels {
    visibility: hidden;
    height: 0;
  }

  #header {
    display: flex;
    justify-content: flex-start;
  }
`;
export class InputListElement extends BehaviorInput(BehaviorElement(AsLitHIC(LitElement))) {
  constructor() {
    super(...arguments);
    this.kindStrategy = f1.strategy({
      array: {
        renderHeader() {
          return nothing;
        },
        renderKey(el, value, form, ns, index) {
          return nothing;
        },
        fromInternalFormat(val) {
          return val.filter(([_, value]) => value !== "").map(([_, value]) => value);
        }
      },
      dictionary: {
        renderHeader() {
          return html`
          <span class="list_headers"><strong id="Key_header">Key</strong></span>
          <span class="list_headers"><strong id="Value_header">Value</strong></span>
        `;
        },
        renderKey(el, value, form, ns, index) {
          return el.__renderInput({ value, label: "Key", form, ns, index });
        },
        fromInternalFormat(val) {
          return Object.fromEntries(val.filter((_) => _[0] !== ""));
        }
      }
    });
  }
  static get is() {
    return "hi-input-list";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      value: {
        type: Object,
        notify: true
      },
      kind: {
        type: String
      }
    };
  }
  render() {
    const ns = `input-list-${f1.safe(f1.id(), this.ns)}`;
    const form = `input-list-${f1.safe(f1.id(), this.ns)}`;
    if (f1.is.nil(this.value)) {
      return nothing;
    }
    const kindAlgorithm = this.kindStrategy(this.kind);
    return html`
      <hi-form .ns=${ns} form=${form} disable-http kind="filter">
        <div id="header">${kindAlgorithm.renderHeader()}</div>
        <ul class="input-list full-width">
          ${f1.is.array(this.__internalValue) && this.__internalValue.map((item, index) => {
      return html`<li class="input-list-item">
              ${kindAlgorithm.renderKey(this, item[0], form, ns, index)}
              ${this.__renderInput({ value: item[1], label: "Value", form, ns, index })}
            </li>`;
    })}
        </ul>
      </hi-form>
    `;
  }
  __renderInput(input) {
    const inputLabel = `${input.label} ${input.index + 1}`;
    return html`
      <hi-input-group class="full-width" .ns=${input.ns}>
        <hi-label .ns=${input.ns} class="list_labels">${inputLabel}</hi-label>
        <hi-input
          type="text"
          aria-describedby="${input.label}_header"
          name="${input.label}-${input.index}"
          .ns=${input.ns}
          .value=${input.value}
          form=${input.form}
          @commit-ended=${this.__input__onCommitEnded}
        >
        </hi-input>
      </hi-input-group>
    `;
  }
  update(props) {
    super.update(props);
    if (props.has("value")) {
      this.__internalValue = this.__toInternalFormat(f1.clone(this.value));
      this.requestUpdate();
    }
  }
  clear() {
    super.clear();
    return this.value = this.kind === "dictionary" ? {} : [];
  }
  __input__onCommitEnded(event) {
    this.__handleInputValueChanged(event);
    this.requestUpdate();
    const exportValue = this.__fromInternalFormat(this.__internalValue);
    this.value = exportValue;
    this._dispatchEvent("value-changed", {
      value: exportValue
    });
    this.commit(exportValue);
  }
  __toInternalFormat(val) {
    val[""] = "";
    return Object.entries(val);
  }
  __fromInternalFormat(val) {
    return this.kindStrategy(this.kind).fromInternalFormat(val);
  }
  __handleInputValueChanged(event) {
    const inputEl = event.target;
    const splitName = inputEl.name.split("-");
    const itemToUpdate = f1.head(splitName).indexOf("Key") > -1 ? 0 : 1;
    const inputControlIndex = Number(f1.last(splitName));
    let entries = this.__internalValue;
    entries[inputControlIndex][itemToUpdate] = inputEl.value;
    entries = this.kind === "dictionary" ? entries.filter((_) => _[0] !== "" || _[1] !== "") : entries.filter((_) => _[1] !== "");
    entries.forEach((_) => _[1] = this.__assignValueType(_[1]));
    entries.push(["", ""]);
    return this.__internalValue = entries;
  }
  __assignValueType(val) {
    if (f1.is.boolean(val) || val === "true" || val === "false") {
      return f1.cast.boolean(val);
    } else if (!isNaN(Number(val))) {
      return f1.cast.number(val);
    } else {
      return val;
    }
  }
  get validity() {
    return this._computeValidity({
      valueMissing: this.required && f1.is.empty(this.value)
    });
  }
}
export default HIP.factory.element(InputListElement);
