
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import {
  AsHIC,
  BehaviorElement,
  BehaviorCustomizable,
  HIP
} from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorInputContainer } from '../../../eip/element/hi-behavior-input/hi-behavior-input-container.js';
import { BehaviorContainer } from '../../../eip/element/hi-behavior-container/hi-behavior-container.js';
import '../../../eip/element/hi-behavior-input/hi-service-input.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class InputGroupElement extends BehaviorCustomizable(
  BehaviorInputContainer(BehaviorContainer(BehaviorElement(AsHIC(PolymerElement))))
) {
  constructor() {
    super(...arguments);
    this.__grouping = {};
  }
  static get is() {
    return "hi-input-group";
  }
  static get template() {
    return html`<slot id="items"></slot>`;
  }
  static get properties() {
    return {
      excluded: {
        type: Boolean,
        reflectToAttribute: true
      }
    };
  }
  static get observers() {
    return ["__formWatcher(form, __slotChildren)", "__nsWatcher(ns, __slotChildren)", "__excludedWatcher(excluded)"];
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "group");
  }
  _disabledChanged(disabled) {
    super._disabledChanged();
    if (this.grouping.input) {
      this.grouping.input.disabled = disabled;
    }
  }
  __excludedWatcher(excluded) {
    if (this.grouping.input) {
      this.grouping.input.excluded = excluded;
    }
  }
  __onSlotChanged() {
    const slotChildren = Array.from(this.children);
    const inputHIP = HIP["input"];
    this.__grouping = {
      validation: f1.nth(0, slotChildren.filter(inputHIP.isValidationNotice)),
      label: f1.nth(0, slotChildren.filter(inputHIP.isLabel)),
      input: f1.nth(0, slotChildren.filter(inputHIP.isInput))
    };
    inputHIP.tetherInputSet(this.grouping, this);
    this.__tetherStyling(this.grouping);
  }
  get grouping() {
    return this.__grouping || {};
  }
  __tetherStyling(grouping) {
    if (grouping.input) {
      this.setAttribute("input-type", grouping.input.localName);
    }
  }
  __formWatcher() {
    if (this.grouping.input) {
      HIP["input"].tetherInput(this.grouping.input, this);
      this.grouping.input.form = this.form;
    }
    if (this.grouping.label) {
      this.grouping.label.setAttribute("form", this.form);
    }
    if (this.grouping.validation) {
      this.grouping.validation.setAttribute("form", this.form);
    }
  }
  __nsWatcher(ns) {
    f1.each((el) => {
      el.ns = ns;
    }, f1.filter(f1.is.exists, this.grouping));
  }
  _attachDom(dom) {
    this.appendChild(dom);
    return null;
  }
}
export default HIP.factory.element(InputGroupElement);
