
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { BehaviorInputEditable } from '../../../eip/element/hi-behavior-input/hi-behavior-input-editable.js';
import { BehaviorValidated } from '../../../eip/element/hi-behavior-validation/hi-behavior-validated.js';
import { BehaviorItemGroup } from '../../../eip/element/hi-behavior-group/hi-behavior-item-group.js';
import { BehaviorMirror } from '../../../eip/element/hi-behavior-mirror/hi-behavior-mirror.js';
import {
  AsHIC,
  BehaviorA11yKeyboard,
  BehaviorBrowserEvents,
  BehaviorElement,
  a11yLabel,
  HIP
} from '../../../eip/element/hi-component/hi-component.js';
import '../../../eip/element/hi-validation-notice/hi-validation-notice.js';
import '../../../eip/element/hi-button/hi-button.js';
import '../../../eip/element/hi-icon/hi-icon.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class InputElement extends BehaviorItemGroup(
  BehaviorValidated(
    BehaviorInputEditable(
      BehaviorMirror(BehaviorA11yKeyboard(BehaviorBrowserEvents(BehaviorElement(AsHIC(PolymerElement)))))
    )
  )
) {
  constructor() {
    super();
    this.__value_mutex = false;
    this.ignoreKeystrokeCodes = [];
    this.groupName = "input";
  }
  static get is() {
    return "hi-input";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          --hi-input--height: var(--hi-el--height);
          --hi-input--border-color: var(--hi-clr--theme-3);
          --hi-input--color: var(--hi-clr--gray-9);
          --hi-input--placeholder-color: var(--hi-clr--gray-7);
        }

        :host {
          display: block;
          background-color: var(--hi-clr--transparent);
          width: auto;
        }

        :host([type='hidden']) {
          -moz-user-focus: ignore !important;
          pointer-events: none !important;
          user-select: none !important;
          cursor: default !important;
          visibility: hidden !important;
          position: relative !important;
          left: -99999px !important;
          width: 0 !important;
          height: 0 !important;
        }

        :host([aesthetic='inline']) {
          display: inline-block;
        }

        #inputIcon {
          display: inline-block;
          margin-left: var(--hi-px-8);
          font-size: var(--hi-px-20);
        }

        #inputClear {
          margin-right: var(--hi-px-4);
          visibility: hidden;
          opacity: 0;
        }

        :host(:not([type='search'])) #inputClear {
          display: none;
        }

        #inputGroup {
          display: flex;
          align-items: center;
          height: var(--hi-input--height);
          background: var(--hi-clr--white);
          outline: none;

          border: var(--hi-px-1) solid var(--hi-input--border-color);

          transition: box-shadow 0.2s cubic-bezier(0.4, 0, 0.2, 1);
        }

        @media (hover: hover) {
          :host(:hover:not(:disabled):not([disabled]):not([readonly])) #inputGroup {
            box-shadow: var(--hi-clr--halo-2);
          }
        }

        :host([validated][invalid]):not([silent-validate]) #inputGroup,
        :host([invalid]:not([focused]):not([silent-validate])) #inputGroup {
          border-color: var(--hi-clr--red-5);
        }

        :host([readonly]) {
          --hi-input--border-color: var(--hi-clr--transparent);
          --hi-input--color: var(--hi-clr--gray-9);
        }

        :host([readonly]) #inputGroup,
        :host([disabled]) #inputGroup {
          pointer-events: none !important;
        }

        :host([disabled]) {
          --hi-input--border-color: var(--hi-clr--gray-ax);
          --hi-input--color: var(--hi-clr--gray-7);
          pointer-events: all !important;
        }

        #input {
          background-color: var(--hi-clr--transparent);
          color: var(--hi-input--color);
          text-align: inherit;
          box-shadow: none;
          display: block;
          outline: none;
          border: none;
          font-family: inherit;
          font-weight: var(--hi-font--weight-light);
          font-size: var(--hi-font-size--text);
          flex-grow: 1;
          padding: var(--hi-px-4) var(--hi-px-8) var(--hi-px-4);
          height: 100%;
          width: 100%;
        }

        :host,
        #input,
        #inputGroup {
          border-radius: var(--hi-shape--radius);
        }

        #input[type='password'] {
          font-size: var(--hi-font-size--large-x);
        }

        #input::placeholder,
        #input:placeholder-shown {
          color: var(--hi-input--placeholder-color);
          font-weight: var(--hi-font--weight-light);
          opacity: 1;
        }

        [type='search'] #input {
          -webkit-appearance: searchfield;
        }

        #input:active,
        #input:focus {
          cursor: text;
        }

        #colorInput {
          border: 0;
          width: var(--hi-px-28);
          height: var(--hi-px-28);
          outline: none;
          background: transparent;
          padding: 0;
          margin-right: var(--hi-px-4);
        }

        #colorInput:focus {
          outline: var(--hi-outline--width) var(--hi-outline--line) var(--hi-clr--outline);
          outline-offset: var(--hi-outline--offset);
          perspective: 1000px;
        }

        :host([aesthetic='minimal']) #inputGroup {
          border: none;
          border-radius: none;
        }

        :host([aesthetic='minimal']:hover) #inputGroup {
          box-shadow: none !important;
        }

        :host([aesthetic='minimal']) #inputGroup:focus,
        :host([aesthetic='minimal']) #inputGroup:focus-within {
          border: none;
          outline: none;
        }

        #input[type='search']::-webkit-search-decoration,
        #input[type='search']::-webkit-search-cancel-button,
        #input[type='search']::-webkit-search-results-button,
        #input[type='search']::-webkit-search-results-decoration {
          -webkit-appearance: none;
        }

        #input::-webkit-search-cancel-button {
          display: none;
        }

        #input::-ms-clear {
          display: none;
          height: 0;
          width: 0;
        }

        /* this is some kind of hack to support reveal styling */
        [reveal='true'] {
          visibility: visible !important;
          opacity: 1 !important;
        }
      </style>

      <div id="inputGroup">
        <hi-icon id="inputIcon" size="medium" theme="[[ theme ]]" kind="[[ icon ]]"></hi-icon>
        <input
          id="input"
          tabindex="0"
          mixin="input-placeholder"
          autofocus$="[[ autofocus ]]"
          disabled$="[[ disabled ]]"
          max$="[[ max ]]"
          maxlength$="[[ maxlength ]]"
          min$="[[ min ]]"
          minlength$="[[ minlength ]]"
          name$="[[ name ]]"
          pattern$="[[ pattern ]]"
          placeholder$="[[ placeholder ]]"
          readonly$="[[ readonly ]]"
          required$="[[ required ]]"
          title$="[[ title ]]"
          step$="[[ step ]]"
          type$="[[ __type ]]"
          autocomplete$="[[ autocomplete ]]"
          on-keydown="__onKeydown_input"
          on-change="__onChange_input"
          on-focus="__onFocus_input"
          on-blur="__onBlur_input"
          on-input="__onInput_input"
        />
        <hi-button id="inputClear" aesthetic="icon" size="small" on-trigger-ended="clear">
          <hi-icon kind="cross" aria-label="clear search input"> </hi-icon>
        </hi-button>
        <slot name="action"></slot>
      </div>
    `;
  }
  static get autocompleteType() {
    return {
      username: "username",
      email: "email",
      telephone: "tel",
      tel: "tel",
      password: "current-password"
    };
  }
  static get permittedTypes() {
    return {
      "datetime-local": "datetime-local",
      date_time: "datetime-local",
      password: "password",
      percentage: "number",
      currency: "number",
      telephone: "tel",
      search: "search",
      integer: "number",
      decimal: "number",
      number: "number",
      hidden: "hidden",
      username: "text",
      email: "email",
      month: "month",
      color: "text",
      date: "date",
      text: "text",
      week: "week",
      time: "time",
      link: "url",
      tel: "tel",
      url: "url"
    };
  }
  static get properties() {
    return {
      aesthetic: {
        type: String,
        reflectToAttribute: true
      },
      icon: {
        type: String,
        observer: "__iconChanged",
        value: ""
      },
      ariaDescribedby: {
        type: String
      },
      ariaLabel: {
        type: String
      },
      ariaLabelledby: {
        type: String
      },
      autofocus: {
        type: Boolean,
        value: false
      },
      mask: {
        type: String
      },
      max: {
        type: String
      },
      min: {
        type: String,
        value: 0
      },
      pattern: {
        type: String
      },
      step: {
        type: Number,
        reflectToAttribute: true,
        computed: "__stepComputed(step, type)"
      },
      type: {
        type: String,
        observer: "__typeChanged",
        reflectToAttribute: true,
        value: "text",
        notify: true
      },
      __type: {
        type: String,
        computed: "__typeComputed(type)"
      },
      value: {
        type: Object,
        observer: "__valueChanged",
        reflectToAttribute: true,
        notify: true
      },
      ignoreKeystrokeCodes: {
        type: Array,
        value: []
      },
      timeZone: {
        type: String
      }
    };
  }
  static get observers() {
    return ["__typeOptionsWatcher(type)", "_ariaLabelTextWatcher(ariaLabel, ariaLabelledby, ariaDescribedby)"];
  }
  static get api() {
    return ["clear"];
  }
  static get keyBindings() {
    return {
      "esc:keyup": "__onKeyUp_escape",
      "enter:keyup": "__onKeyUp_enter",
      "up:keyup": "_noop",
      "down:keyup": "_noop",
      "backspace:keyup": "_noop"
    };
  }
  static get browserBindings() {
    return {
      click: "_noop"
    };
  }
  static get valueEvent() {
    return "value-changed";
  }
  static get autoBinding() {
    return {
      signal: "value-changed",
      field: "value"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    dom.write(() => {
      dom.setAttrs(this, {
        role: this.getAttribute("role") || "textbox",
        focusable: "focusable",
        "aria-multiline": "false"
      });
    });
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    if (this.$.colorInput) {
      this.$.colorInput.remove();
    }
    if (this.$.__hiddenInput) {
      this.$.__hiddenInput.remove();
    }
  }
  get inputMirrorType() {
    return this.type;
  }
  __onKeyUp_escape() {
    if (this.type === "search") {
      this.value = null;
    }
  }
  __onKeyUp_enter(event) {
    super._inputEditable_onKeyUp_enter(event);
    if (this.type === "search") {
      this.__onSearch_input(event);
    }
  }
  clear() {
    const _return = super.clear();
    if (this.type === "search") {
      this.$.input.focus();
      this.value = null;
      this._dispatchEvent("search", {
        value: this.value
      });
    }
    return _return;
  }
  blur() {
    super.blur();
    this.$.input.blur();
    this.__onBlur_input(new CustomEvent("blur"));
  }
  __onChange_input(event) {
    super._inputEditable_onChange_input(event);
  }
  __onFocus_input() {
    super._inputEditable_onFocus_input();
    this.$.inputGroup.setAttribute("focused", "focused");
    this.setAttribute("focused", "focused");
  }
  __onBlur_input(event) {
    super._inputEditable_onBlur_input(event);
    this.$.inputGroup.removeAttribute("focused");
  }
  __onSearch_input(event) {
    event.preventDefault();
    if (this.validate()) {
      this._dispatchEvent("search", {
        value: this.value
      });
    }
  }
  __onInput_input(event) {
    super._inputEditable_onInput_input(event);
    if (this.type === "search") {
      this.validate();
    }
  }
  __onKeydown_input(e) {
    if ((this.ignoreKeystrokeCodes || []).includes(e.code)) {
      e.preventDefault();
    }
  }
  _maxlengthChanged(maxlength) {
    super._maxlengthChanged(maxlength);
    if (this.getAttribute("aesthetic") === "inline") {
      this.style.width = `calc(2ch * ${maxlength})`;
    }
  }
  __iconChanged(icon) {
    this.$.inputIcon.setAttribute("show", `${icon.length > 0}`);
  }
  __typeChanged(type) {
    if ((type === "datetime-local" || type === "date_time") && this.value) {
      this.value = f1.format("iso_date_time", { format_type: "html5", time_zone: this.timeZone }, this.value);
    }
    if (!InputElement.permittedTypes[type]) {
      this.type = "text";
      return;
    }
  }
  __stepComputed(step, type) {
    const noStepAttribute = f1.is.nil(step);
    const isStepDecimal = step % 1 != 0;
    switch (type) {
      case "decimal":
        return noStepAttribute ? "0.01" : step;
      case "integer":
        if (isStepDecimal && !noStepAttribute) {
          this.type = "number";
        }
        return noStepAttribute ? 1 : step;
      case "number":
      default:
        return noStepAttribute ? 1 : step;
    }
  }
  __typeOptionsWatcher(type) {
    this.hidden = type === "hidden";
    if (type === "date") {
      console.warn("Input Type Date: The format for min and max date is [year|month|day]");
      if (!this.max) {
        this.max = "9999-12-31";
      }
      if (!this.min) {
        this.min = "1000-01-01";
      }
    } else if (type === "color") {
      const colorInput = document.createElement("input");
      dom.setAttrs(colorInput, {
        "aria-labelledby": "input",
        "aria-label": "Choose a color",
        id: "colorInput",
        type: "color"
      });
      colorInput.addEventListener("change", () => {
        if (this.kind === "edit") {
          this._value = colorInput.value;
        } else {
          this.value = colorInput.value;
        }
      });
      this.$.colorInput = colorInput;
      this.$.inputGroup.appendChild(colorInput);
    } else if (this.$.colorInput) {
      this.$.colorInput.remove();
    } else if (type === "username") {
      this.name = "username";
    } else if (type === "password") {
      const hiddenInput = dom.create(null, "input", {
        hidden: true,
        type
      });
      this.$.__hiddenInput = hiddenInput;
      document.body.appendChild(hiddenInput);
    }
    if (this.autocomplete === "off") {
      this.autocomplete = InputElement.autocompleteType[type] || "off";
    }
  }
  __typeComputed(type) {
    return InputElement.permittedTypes[type] ? InputElement.permittedTypes[type] : "text";
  }
  __valueChanged(value, oldValue) {
    if (this.__value_mutex) {
      return;
    }
    this.__value_mutex = true;
    value = value != null ? value : null;
    oldValue = oldValue != null ? oldValue : null;
    const valueSize = ((value || "") + "").length;
    const oldValueSize = ((oldValue || "") + "").length;
    if (value !== oldValue) {
      const isValueSizeIncreasing = valueSize > oldValueSize;
      if (isValueSizeIncreasing) {
        value = this._applyValidationAlterations(value);
      }
    }
    if (this.type === "search") {
      this.$.inputClear.setAttribute("reveal", `${valueSize > 0}`);
    } else if (this.type === `color` && this.$.colorInput) {
      this.$.colorInput.value = value;
    } else if (this.type === "number" && value === "-") {
      return;
    } else if (value === Infinity) {
      value = this.min;
    }
    super._inputEditable_valueChanged(value);
    this.__value_mutex = false;
  }
  __decimalPresent(value) {
    return value.includes(".");
  }
  __decimalAllowed(type = this.type) {
    return type === "decimal";
  }
  _coerceValue(value) {
    const htmlInputType = this.__type;
    const isInputOfTypeNumber = htmlInputType === "number";
    const hasCharacters = value !== "" && value != null;
    const ifValueCanBeCoerced = isInputOfTypeNumber && hasCharacters;
    if (!ifValueCanBeCoerced) {
      return value;
    }
    if (!f1.is.string(value)) {
      return value;
    }
    try {
      value = parseFloat(value);
    } catch (e) {
      value = this.min;
    }
    return value;
  }
  _ariaLabelTextWatcher(label, labelledBy, describedBy) {
    a11yLabel.writeA11yLabel(this.$.input, label, labelledBy, describedBy, { ns: this.ns });
  }
}
export default HIP.factory.element(InputElement);
