
import { repeat } from '../../../hip/library/lit-html/directives/repeat.js';
import { html, LitElement } from '../../../hip/library/lit-element/lit-element.js';
import { render } from '../../../hip/library/lit-html/lit-html.js';
import { AsLitHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import $ip from '../../../pip/library/pi-ip/pi-ip.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import './hi-iterator-generator-list.js';
import './hi-iterator-iterator-list.js';
export class IteratorElement extends BehaviorElement(AsLitHIC(LitElement)) {
  static get is() {
    return "hi-iterator";
  }
  static get properties() {
    return {
      for: {
        type: String
      },
      generator: {
        type: String
      },
      iterator: {
        type: String
      },
      input: {
        type: Array
      },
      context: {
        type: Object
      },
      template: {
        type: Object,
        attribute: false
      }
    };
  }
  createRenderRoot() {
    return this;
  }
  update(props) {
    super.update(props);
    const iterator = $ip.di.inject("iterator", this.iterator);
    const template = f1.safe(this.template, iterator);
    if (!template) {
      return;
    }
    const generator = $ip.di.inject("generator", this.generator);
    const input = generator ? generator.create() : f1.safe([], this.input);
    if (!template) {
      return;
    }
    const requestedForEl = this.for instanceof HTMLElement ? this.for : HIP.browser.queryOne(this.for, this.ns);
    const forEl = f1.safe(this.parentElement, requestedForEl);
    if (!forEl) {
      return;
    }
    const readIdFn = f1.read("/value", 0);
    const context = f1.merge("over", { ns: this.ns }, f1.safe({}, this.context));
    const templateFn = template.render(context);
    const renderEl = forEl;
    const renderTemplate = html` ${repeat(
      input,
      readIdFn,
      (item, index) => html`${templateFn(item, index, templateFn)}`
    )}`;
    render(renderTemplate, renderEl);
  }
  static generatorList(generator) {
    const generatorArtifact = f1.safe(
      { id: "auto-created-noop", create: f1.always([]) },
      $ip.di.inject("generator", generator)
    );
    return generatorArtifact.create();
  }
  static generatorDict(generator) {
    return f1.map(
      ({ title }) => title,
      f1.to.record(() => "value", this.generatorList(generator))
    );
  }
}
export default HIP.factory.element(IteratorElement);
