
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { AsLitHIC, BehaviorBrowserEvents, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export const styles = css`
  :host {
    position: relative;
    display: inline-block;
    color: var(--hi-clr--theme-3);
    line-height: var(--hi-font-lh--text);
    font-size: var(--hi-font-size--text);
    font-weight: var(--hi-font--weight-light);
    cursor: default;
    width: auto;
  }

  :host([aesthetic='inline']) {
    width: auto;
  }

  :host(:not([theme])) {
    color: var(--hi-clr--gray-9);
  }

  #required {
    display: none;
    position: relative;
    padding: var(--hi-px-6);
    color: var(--hi-clr--red-3);
    text-decoration: none;
    font-size: 1.125em;
    bottom: -0.2em;
    line-height: 0;
  }
  #controls,
  #container {
    display: inline-flex;
    align-items: center;
  }
  :host([required]) #required {
    display: inline;
  }

  :host ::slotted(hi-input) {
    margin-top: var(--hi-px-4);
  }

  :host(:not([status])) #labelStatusMarker {
    display: none;
  }

  :host([status]) #labelStatusMarker {
    margin: 0 0 var(--hi-px-2) var(--hi-px-3);
  }

  :host([status='valid']) #labelStatusMarker {
    --hi-icon--color: var(--hi-clr--green-3);
  }

  :host([status='invalid']) #labelStatusMarker {
    --hi-icon--color: var(--hi-clr--red-3);
  }

  @media (hover: hover) {
    :host([for]:hover:not([disabled])) {
      text-shadow: var(--hi-clr--text-shadow-1);
    }
  }
`;
const _LabelElement = class extends BehaviorBrowserEvents(BehaviorElement(AsLitHIC(LitElement))) {
  static get is() {
    return "hi-label";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      for: {
        type: String,
        reflect: true
      },
      action: {
        type: String
      },
      value: {
        type: String
      },
      required: {
        type: Boolean
      },
      status: {
        type: String,
        reflect: true
      }
    };
  }
  static get api() {
    return ["trigger"];
  }
  static get browserBindings() {
    return {
      click: "__onClick"
    };
  }
  static get autoBinding() {
    return {
      signal: "trigger-ended",
      field: "trigger"
    };
  }
  update(props) {
    super.update(props);
    this._watchChanges(props, [
      {
        properties: ["for"],
        method: "__applyAXMarkersToTargetWatcher"
      }
    ]);
  }
  render() {
    return html`
      <div id="container">
        <slot>${this.value}</slot>
        ${this.__renderRequiredMarker()} ${this.__renderStatusIcon()}
        <div id="controls">
          <slot name="controls"></slot>
        </div>
      </div>
    `;
  }
  __updateTarget() {
    this.__applyAXMarkersToTargetWatcher(this.for);
  }
  __onClick(event) {
    const target = event.currentTarget;
    const isLabel = target ? target.localName === "hi-label" : false;
    const isParentLabel = !isLabel && target.closest("hi-label") != null;
    if (isLabel || isParentLabel) {
      this.trigger(event);
    }
  }
  trigger(event) {
    if (!this.__target) {
      this.__updateTarget();
    }
    if (!this.__target) {
      return;
    } else if (event && event.currentTarget === this.__target) {
      return;
    }
    const target = this.__target;
    if (target.hasAttribute("disabled")) {
      return;
    }
    if (target.hasAttribute("replay-tap")) {
      return;
    }
    if (target.readonly) {
      target.focus();
      return;
    }
    if (this.action && typeof target[this.action] === "function") {
      target[this.action]();
    } else if (typeof target.activate === "function") {
      target.activate();
    } else if (typeof target.toggle === "function") {
      target.toggle();
    }
    target.focus();
    return target;
  }
  __renderStatusIcon() {
    const statusStrategy = _LabelElement.statusStrategy;
    const statusOptions = f1.is.string(this.status) ? statusStrategy(this.status) : {};
    const kind = f1.read("/kind", void 0, statusOptions);
    if (!f1.is.string(kind)) {
      return nothing;
    }
    return html`<hi-icon id="labelStatusMarker" align="middle" type="material" kind="${kind}"></hi-icon>`;
  }
  __renderRequiredMarker() {
    return this.required ? html`<abbr id="required" title="required">*</abbr>` : nothing;
  }
  __applyAXMarkersToTargetWatcher(_for) {
    this.__removeAXMarkers();
    this.__findNewAXTarget(_for);
    this.__addAXMarkers();
    this.__addRequiredField();
  }
  __removeAXMarkers() {
    const target = this.__target;
    if (!target) {
      return;
    }
    dom.excludeAttrList(target, "aria-labelledby", dom.getAttr(this, "id", ""));
    target.labelEl = void 0;
  }
  __findNewAXTarget(_for) {
    const newTarget = this.__findTarget(_for);
    this.__target = this.inputEl ? this.inputEl.isSameNode(newTarget) ? this.inputEl : newTarget : newTarget;
  }
  __addAXMarkers() {
    const target = this.__target;
    if (!target) {
      return;
    }
    dom.ensureAttr(this, "id");
    dom.includeAttrList(target, "aria-labelledby", dom.getAttr(this, "id", ""));
    target.labelEl = this;
  }
  __findTarget(_for) {
    const idLookup = `#${_for}`.replace("##", "#");
    if (this.hasAttribute("for") && _for === "") {
      return void 0;
    } else if (_for && _for.length > 0) {
      return HIP.browser.queryElement(idLookup, this.ns);
    }
    const isNotLabel = (o) => !HIP.util.isTags(["label", "hi-label", "dom-bind"], o);
    const nonLabelChildren = Array.from(this.children).filter(isNotLabel);
    if (nonLabelChildren.length >= 1) {
      return nonLabelChildren[0];
    }
    const nonLabelSiblings = this.siblings.filter(isNotLabel);
    if (nonLabelSiblings.length >= 1) {
      return nonLabelSiblings[0];
    }
    const hiButton = HIP.browser.queryElement("hi-button", this.ns);
    if (hiButton) {
      return hiButton;
    }
    const htmlButton = HIP.browser.queryElement("button", this.ns);
    if (htmlButton) {
      return htmlButton;
    }
    return this.firstElementChild;
  }
  __addRequiredField() {
    if (this.__target && this.__target.hasAttribute("required")) {
      this.setAttribute("required", "required");
    }
  }
};
export let LabelElement = _LabelElement;
LabelElement.statusStrategy = f1.strategy({
  valid: {
    kind: "done"
  },
  invalid: {
    kind: "error_outline"
  }
});
export default HIP.factory.element(LabelElement);
