
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { BehaviorSetSelect } from '../../../eip/element/hi-behavior-set-select/hi-behavior-set-select.js';
import {
  AsHIC,
  BehaviorA11yKeyboard,
  BehaviorBrowserEvents,
  BehaviorInteractiveElement,
  BehaviorElement,
  HIP
} from '../../../eip/element/hi-component/hi-component.js';
import './hi-link.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
export class LinkGroupElement extends BehaviorSetSelect(
  BehaviorBrowserEvents(BehaviorA11yKeyboard(BehaviorInteractiveElement(BehaviorElement(AsHIC(PolymerElement)))))
) {
  static get is() {
    return "hi-link-group";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          display: block;
          z-index: 1;
          border: none;
          width: 100%;
          text-align: center;
        }

        :host(:focus) {
          outline: none;
        }

        #container {
          display: flex;
        }

        #container ::slotted(hi-link) {
          transition: border-color var(--hi-animation--speed-normal), background-color var(--hi-animation--speed-normal);
        }

        #container ::slotted(hi-link:hover),
        #container ::slotted(hi-link:focus-within) {
          border-color: var(--hi-clr--gray-5);
          background: var(--hi-clr--gray-2);
        }

        #container ::slotted(hi-link[active]) {
          border-color: var(--hi-clr--blue-3);
          background: var(--hi-clr--blue-1);
          --hi-link--color: var(--hi-clr--theme-5);
          font-weight: var(--hi-font--weight-bold);
        }

        :host([disabled]) #container ::slotted(hi-link[active]) {
          border-color: var(--hi-clr--disabled);
          background: var(--hi-clr--gray-2);
        }

        :host(:not([kind='stacked'])) #container ::slotted(hi-link) {
          border-bottom: var(--hi-px-4) solid var(--hi-clr--transparent);
        }

        :host(:not([kind='stacked'])) #container ::slotted(hi-link[active]) {
          border-bottom-color: var(--hi-clr--theme-3);
        }

        #container ::slotted(hi-link[active]:hover) {
          text-shadow: none;
        }

        :host([kind='stacked']) #container {
          flex-direction: column;
        }

        :host([kind='stacked']) #container ::slotted(hi-link) {
          line-height: var(--hi-px-32);
          padding: 0 var(--hi-px-4);
        }

        :host([kind='stacked']) #container ::slotted(hi-link[active]) {
          border-right: none;
          border-bottom: var(--hi-px-1) solid var(--hi-clr--gray-3);
          border-left: var(--hi-px-3) solid var(--hi-clr--blue-3);
        }

        :host([disabled]) #container ::slotted(*) {
          color: var(--hi-clr--disabled);
        }

        :host([disabled]) #container ::slotted(hi-link[active]) {
          border-color: var(--hi-clr--disabled);
          color: var(--hi-clr--disabled);
          background: var(--hi-clr--gray-2);
        }

        :host([stretched]) #container ::slotted(hi-link) {
          flex: 1 1;
        }
      </style>

      <div id="container">
        <slot id="items"></slot>
      </div>
    `;
  }
  static get properties() {
    return {
      kind: {
        type: String,
        reflectToAttribute: true,
        value: "standard"
      },
      stretched: {
        type: Boolean
      },
      activeMode: {
        type: String,
        reflectToAttribute: true,
        value: "manual"
      }
    };
  }
  static get api() {
    return ["shiftFocusDown", "shiftFocusUp", "focusIndex", "focusItem", "selectIndex", "selectItem"];
  }
  static get keyBindings() {
    return {
      "space:keydown": "_onKeyAny__preventDefault",
      "space:keyup": "_setSelect__onKeyUp_space",
      "enter:keyup": "_setSelect__onKeyUp_enter",
      "up:keydown": "_onKeyAny__preventDefault",
      "down:keydown": "_onKeyAny__preventDefault",
      "left:keydown": "_onKeyAny__preventDefault",
      "right:keydown": "_onKeyAny__preventDefault",
      "right:keyup": "_setFocus__onKeyUp_right",
      "down:keyup": "_setFocus__onKeyUp_down",
      "left:keyup": "_setFocus__onKeyUp_left",
      "up:keyup": "_setFocus__onKeyUp_up"
    };
  }
  static get browserBindings() {
    return {
      click: "_setSelect__onTap",
      focus: "_setFocus__onFocus"
    };
  }
  static get autoBinding() {
    return {
      signal: "selected-index-changed",
      field: "selectedIndex"
    };
  }
  static get observers() {
    return ["__activeModeWatcher(activeMode)"];
  }
  connectedCallback() {
    super.connectedCallback();
    const isInsideAFlyout = HIP.util.doesExtend(this.parentElement, "BehaviorFlyout");
    dom.write(() => {
      this.setAttribute("role", isInsideAFlyout ? "listbox" : "navigation");
    });
  }
  setActive() {
    if (this.activeMode !== "automatic") {
      return;
    }
    const currentChildren = this._readSlotChildren();
    const locationHref = window.location.href;
    currentChildren.forEach((link) => {
      const activePathRegex = link.activePathRegex();
      const isActive = !f1.is.nil(activePathRegex) ? activePathRegex.test(locationHref) : !f1.is.nil(this.selectedIndex) ? this.selectedIndex === link.index : false;
      dom.setAttr(link, "active", isActive);
    });
  }
  _onSlotChanged() {
    super._onSlotChanged();
    this._setSelect__onSlotChanged();
    this.setActive();
  }
  __activeModeWatcher() {
    this.setActive();
  }
  __setSelect__selectedIndexChanged(_index, _oldIndex) {
    super.__setSelect__selectedIndexChanged(this.selectedIndex, _oldIndex);
    dom.write(() => {
      this.setActive();
    });
  }
}
export default HIP.factory.element(LinkGroupElement);
