
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { ifDefined as ifDef } from '../../../hip/library/lit-html/directives/if-defined.js';
import {
  BehaviorA11yKeyboard,
  BehaviorBrowserEvents,
  BehaviorCustomizable,
  BehaviorElement,
  AsHIC,
  HIP
} from '../../../eip/element/hi-component/hi-component.js';
import '../../../eip/element/hi-icon/hi-icon.js';
import navigation from '../../../pip/library/pi-navigation/pi-navigation.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export const styles = css`
  :host {
    --hi-link--color: var(--hi-clr--theme-3);
    --hi-link--font-size: var(--hi-font-size--text);
    --hi-link--line-height: var(--hi-font-lh--medium);
    --hi-link--font-weight: var(--hi-font--weight-normal);
    --hi-link--text-decoration: underline;
  }

  :host {
    display: inline-flex;
    align-items: center;
    line-height: var(--hi-link--line-height);
    font-weight: var(--hi-link--font-weight);
    font-size: var(--hi-link--font-size);
    color: var(--hi-link--color);
  }

  #link {
    display: inline-flex;
    align-items: center;
    color: var(--hi-link--color);
    word-break: normal;
    letter-spacing: normal;
    text-decoration: var(--hi-link--text-decoration);
    text-decoration-skip-ink: auto;
  }

  :host([aesthetic='fill']) #link {
    justify-content: center;
    align-items: center;
    display: flex;
    height: 100%;
    width: 100%;
  }

  :host([aesthetic='button']) #link,
  :host([aesthetic='icon']) #link {
    display: inline-flex;
    align-items: center;
    justify-content: center;
    border-radius: var(--hi-shape--radius);
    min-width: var(--hi-px-24);
    font-weight: var(--hi-font--weight-light);
    white-space: nowrap;
    user-select: none;
    color: var(--hi-clr--theme-5);
  }

  :host([aesthetic='button']) #link {
    will-change: box-shadow;
    transition: box-shadow 0.2s cubic-bezier(0.4, 0, 0.2, 1);
    box-shadow: var(--hi-clr--shadow-clickable);
    border: var(--hi-px-1) solid var(--hi-clr--theme-3);
    background-color: var(--hi-clr--theme-1);
    height: var(--hi-el--height);
    padding: var(--hi-px-8);
  }

  :host([aesthetic='button'][external]) #link {
    padding-right: var(--hi-px-20);
  }

  :host([aesthetic='icon']) #link {
    border: var(--hi-px-1) solid var(--hi-clr--transparent);
    background-color: var(--hi-clr--transparent);
    height: var(--hi-px-34);
    padding: var(--hi-px-8);
    vertical-align: middle;
  }

  @media (hover: hover) {
    #link[href]:hover {
      --hi-link--color: var(--hi-clr--theme-5);
      --hi-link--text-decoration: none;
    }
  }

  @media (hover: hover) {
    :host([aesthetic='button']) #link[href]:hover,
    :host([aesthetic='icon']) #link[href]:hover {
      background-color: var(--hi-clr--theme-1);
      border-color: var(--hi-clr--theme-5);
      box-shadow: var(--hi-clr--shadow-down-2);
      text-shadow: var(--hi-px-0) var(--hi-px-0) var(--hi-px-1) var(--hi-clr--theme-5);
      filter: contrast(115%);
    }
  }

  :host(:not([aesthetic])) #link[href]:active,
  :host([aesthetic='button']) #link[href]:active,
  :host([aesthetic='icon']) #link[href]:active,
  :host([aesthetic='navigation']) #link[href]:active {
    text-shadow: var(--hi-clr--text-shadow-2);
  }

  :host(:focus-within) #link {
    outline: var(--hi-outline--width) var(--hi-outline--line) var(--hi-clr--outline);
    outline-offset: var(--hi-outline--offset);
  }

  :host([aesthetic='navigation']) #link {
    display: flex;
    text-decoration: none !important;
  }

  #link[href],
  #link ::slotted(*) {
    cursor: pointer;
  }

  :host([disabled]) #link {
    text-shadow: none;
  }

  :host(:not([show-icon])) hi-icon {
    display: none;
  }

  #link ::slotted(hi-icon),
  hi-icon {
    --hi-icon--color: currentColor;
  }

  :host([aesthetic='navigation'][external]),
  :host([aesthetic='button'][external]),
  :host([aesthetic='icon'][external]) {
    position: relative;
  }

  :host([aesthetic='button'][external]) #__externalLinkIcon {
    position: absolute;
    top: 0px;
    right: 8px;
  }

  :host([layout='navigation'][external]) #__externalLinkIcon {
    position: absolute;
    right: var(--hi-px-2);
    top: var(--hi-px-2);
  }

  :host([allow-ellipsis]) #link {
    display: block;
    overflow: hidden;
    text-overflow: ellipsis;
    white-space: nowrap;
  }
`;
const R = {
  isExternal: /^(https?|mailto|tel):/
};
export class LinkElement extends BehaviorCustomizable(
  BehaviorA11yKeyboard(BehaviorBrowserEvents(BehaviorElement(AsHIC(LitElement))))
) {
  constructor() {
    super(...arguments);
    this.target = "_self";
    this.__internalClickMutex = false;
  }
  static get is() {
    return "hi-link";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      aesthetic: {
        type: String,
        reflect: true
      },
      external: {
        type: Boolean,
        reflect: true
      },
      disabled: {
        type: Boolean,
        reflect: true
      },
      download: {
        type: String,
        reflect: true
      },
      href: {
        type: String,
        reflect: true
      },
      safeHref: {
        type: String,
        reflect: true,
        attribute: "safe-href"
      },
      target: {
        type: String,
        reflect: true
      },
      safeTarget: {
        type: String,
        reflect: true,
        attribute: "safe-target"
      },
      value: {
        type: Object
      },
      activePath: {
        type: String,
        attribute: "active-path"
      },
      ariaLabel: {
        type: String,
        attribute: "aria-label"
      },
      ariaLabelledby: {
        type: String,
        attribute: "aria-labelledby"
      },
      ariaDescribedby: {
        type: String,
        attribute: "aria-describedby"
      },
      showIcon: {
        type: Boolean,
        reflect: true,
        attribute: "show-icon"
      },
      tabindex: {
        type: Number
      }
    };
  }
  static get api() {
    return ["trigger"];
  }
  static get autoBinding() {
    return {
      signal: "trigger-ended",
      field: "trigger"
    };
  }
  get context() {
    return {
      safeHref: this.safeHref,
      target: this.target,
      value: this.value,
      href: this.href
    };
  }
  connectedCallback() {
    super.connectedCallback();
    dom.write(() => {
      dom.setAttrs(this, {
        focusable: true,
        role: "article"
      });
      this._includeQAMarker();
    });
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    const linkEl = this.__linkEl();
    if (linkEl) {
      this.removeEventListener("click", this.__link__onClickParentBound);
      linkEl.removeEventListener("keydown", this.__link__onKeyDownBound);
      linkEl.removeEventListener("keyup", this.__link__onKeyUpBound);
      linkEl.removeEventListener("click", this.__link__onClickBound);
    }
  }
  attributeChangedCallback(name, valueOld, value, ns) {
    super.attributeChangedCallback(name, valueOld, value, ns);
    if (name === "tabindex") {
      dom.setAttr(this, "tabindex", null);
    }
  }
  update(props) {
    super.update(props);
    if (props.has("href")) {
      this.__hrefWatcher(this.href);
    }
  }
  firstUpdated() {
    this.__link__onClickParentBound = this.__link__onClickParent.bind(this);
    this.__link__onKeyDownBound = this.__link__onKeyDown.bind(this);
    this.__link__onKeyUpBound = this.__link__onKeyUp.bind(this);
    this.__link__onClickBound = this.__link__onClick.bind(this);
    const linkEl = this.__linkEl();
    if (linkEl) {
      this.addEventListener("click", this.__link__onClickParentBound);
      linkEl.addEventListener("keydown", this.__link__onKeyDownBound);
      linkEl.addEventListener("keyup", this.__link__onKeyUpBound);
      linkEl.addEventListener("click", this.__link__onClickBound);
    }
  }
  updated(props) {
    if (props.has("external")) {
      this.__relStatusWatcher(this.external);
    }
    if (props.has("disabled")) {
      this._disabledChanged(this.disabled);
    }
  }
  render() {
    const safeHref = this.__safeHrefCompute(this.href);
    const safeTarget = this.__safeTargetCompute(this.target, this.href);
    const showIcon = this.__showIconCompute(this.external, this.safeTarget);
    const ariaLabelIcon = this.__ariaLabelIconCompute(this.external, this.safeTarget);
    const ariaDescribedbyIcon = this.__ariaDescribedbyIconCompute(
      this.external,
      this.safeTarget,
      this.ariaDescribedby
    );
    this.safeHref = f1.is.empty(safeHref) ? "" : safeHref;
    this.safeTarget = safeTarget;
    this.showIcon = showIcon;
    return html`
      <a
        id="link"
        href=${ifDef(this.safeHref)}
        target=${ifDef(safeTarget)}
        download=${ifDef(this.download)}
        aria-label=${ifDef(this.ariaLabel)}
        aria-describedby=${ifDef(f1.is.empty(ariaDescribedbyIcon) ? void 0 : ariaDescribedbyIcon)}
        ?disabled=${this.disabled}
      >
        <slot></slot>
      </a>
      <hi-icon
        id="__externalLinkIcon"
        type="material"
        kind="north_east"
        theme=${ifDef(this.theme)}
        aria-label=${ifDef(f1.is.empty(ariaLabelIcon) ? void 0 : ariaLabelIcon)}
      ></hi-icon>
    `;
  }
  activePathRegex() {
    return this.activePath ? new RegExp(this.activePath) : new RegExp(this.href);
  }
  trigger(callingAgent) {
    if (this.disabled) {
      return;
    }
    if (callingAgent !== "by-link" && this.href) {
      this.__internalClickMutex = true;
      this.__linkEl().click();
      this.__internalClickMutex = false;
    }
    this.focus();
    this._dispatchBubblingEvent("value-notified", this.context);
    return this.value != null ? this.value : true;
  }
  focus() {
    this.__linkEl().focus();
  }
  blur() {
    this.__linkEl().blur();
  }
  _disabledChanged(disabled) {
    super._disabledChanged(disabled);
    dom.flipAttr(this.__linkEl(), "tabindex", disabled, "-1");
  }
  __linkEl() {
    return this.shadowRoot?.getElementById("link") || document.createElement("a");
  }
  __link__onKeyDown(event) {
    if (event.key === "Enter" || event.key === " ") {
      event.preventDefault();
    }
  }
  __link__onKeyUp(event) {
    if (event.key === "Enter" || event.key === " ") {
      this.trigger();
    }
  }
  __link__onClickParent(event) {
    if (this.__internalClickMutex) {
      return;
    }
    const targetEl = event.target;
    const ifClickedLinkContainer = this === targetEl || this === targetEl.closest("hi-link");
    if (ifClickedLinkContainer) {
      this.trigger();
    }
  }
  __link__onClick(event) {
    if (!this.href) {
      event.preventDefault();
      return;
    }
    if (this.href === "#") {
      event.preventDefault();
    }
    event.stopPropagation();
    this.trigger("by-link");
  }
  __relStatusWatcher(external) {
    dom.flipAttr(this.__linkEl(), "rel", external, "external nofollow noopener noreferrer");
  }
  __hrefWatcher(href) {
    if (href.length > 0 && href.match(R.isExternal) && href.indexOf(location.hostname) < 0) {
      this.external = true;
    }
  }
  __safeHrefCompute(href) {
    if (!f1.is.string(href)) {
      return "";
    }
    const isMail = href.includes("mailto:");
    const isTel = href.includes("tel:");
    if (isMail || isTel) {
      return href;
    }
    return navigation.uri.write(href);
  }
  __safeTargetCompute(target, href) {
    return href != null ? target : void 0;
  }
  __showIconCompute(external, safeTarget) {
    return this.__hasExternalIcon(external, safeTarget);
  }
  __ariaLabelIconCompute(external, safeTarget) {
    return `${external ? "External link." : ""} ${safeTarget === "_blank" || safeTarget === "_system" ? "Opens in a new tab or window." : ""}`.trim();
  }
  __ariaDescribedbyIconCompute(external, safeTarget, ariaDescribedby) {
    return `${f1.safe("", ariaDescribedby)} ${this.__hasExternalIcon(external, safeTarget) ? "__externalLinkIcon" : ""}`.trim();
  }
  __hasExternalIcon(external = this.external, safeTarget = this.safeTarget) {
    return external || safeTarget === "_blank" || safeTarget === "_system";
  }
}
export default HIP.factory.element(LinkElement);
