
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import {
  AsHIC,
  HIP,
  BehaviorBrowserEvents,
  BehaviorA11yKeyboard,
  BehaviorInteractiveElement
} from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorSetFocus } from '../../../eip/element/hi-behavior-set-focus/hi-behavior-set-focus.js';
import './hi-item.js';
export class ListElement extends BehaviorSetFocus(
  BehaviorBrowserEvents(BehaviorA11yKeyboard(BehaviorInteractiveElement(BehaviorElement(AsHIC(PolymerElement)))))
) {
  static get is() {
    return "hi-list";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          display: block;
          height: 100%;
          width: 100%;
          border: 0;
        }

        :host(:focus) {
          outline: none;
        }

        #container ::slotted(hi-item) {
          outline-offset: calc(-1 * var(--hi-px-4));
        }

        :host([allow-remote-focus]) #container ::slotted([focused]:not([disabled])),
        :host(:focus) #container ::slotted([focused]:not([disabled])) {
          outline: var(--hi-outline--width) var(--hi-outline--line) var(--hi-clr--outline);
          background-color: var(--hi-clr--theme-1);
        }

        :host(:not([fit-width])) #container ::slotted(hi-item) {
          white-space: nowrap;
        }
      </style>
      <div id="container">
        <slot id="items"></slot>
      </div>
    `;
  }
  static get api() {
    return ["shiftFocusDown", "shiftFocusUp", "focusIndex", "focusItem"];
  }
  static get keyBindings() {
    return {
      "home:keyup": "_setFocus__onKeyUp_home",
      "end:keyup": "_setFocus__onKeyUp_end",
      "home:keydown": "_onKeyAny__preventDefault",
      "end:keydown": "_onKeyAny__preventDefault",
      "space:keyup": "_setFocus__onKeyUp_space",
      "enter:keyup": "_setFocus__onKeyUp_enter",
      "up:keydown": "_onKeyAny__preventDefault",
      "down:keydown": "_onKeyAny__preventDefault",
      "right:keyup": "_setFocus__onKeyUp_right",
      "down:keyup": "_setFocus__onKeyUp_down",
      "left:keyup": "_setFocus__onKeyUp_left",
      "up:keyup": "_setFocus__onKeyUp_up"
    };
  }
  static get browserBindings() {
    return {
      click: "_setFocus__onTap",
      focus: "_setFocus__onFocus"
    };
  }
  static get valueEvent() {
    return "chosen-ended";
  }
  static get autoBinding() {
    return {
      signal: "focused-index-changed",
      field: "focusedIndex"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "listbox");
  }
  __onSlotChanged() {
    this._setFocus__onSlotChanged();
  }
}
export default HIP.factory.element(ListElement);
