
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { afterNextRender } from '../../../hip/library/polymer/polymer.js';
import { BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import {
  AsHIC,
  HIP,
  BehaviorBrowserEvents,
  BehaviorA11yKeyboard,
  BehaviorInteractiveElement,
  BehaviorCustomizable
} from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorSetTree } from '../../../eip/element/hi-behavior-set-tree/hi-behavior-set-tree.js';
import { BehaviorPopout } from '../../../eip/element/hi-behavior-popout/hi-behavior-popout.js';
import { LayoutScreenQueryElement as LSQ } from '../../../yip/element/yi-layout-screen-query/yi-layout-screen-query.js';
import './hi-sub-menu.js';
import './hi-pick.js';
export class MenuElement extends BehaviorCustomizable(
  BehaviorPopout(
    BehaviorSetTree(
      BehaviorA11yKeyboard(BehaviorBrowserEvents(BehaviorInteractiveElement(BehaviorElement(AsHIC(PolymerElement)))))
    )
  )
) {
  static get is() {
    return "hi-menu";
  }
  static get properties() {
    return {
      capHeight: {
        type: Boolean
      },
      for: {
        type: String
      },
      subtitle: {
        type: String
      }
    };
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          min-width: var(--hi-menu-min-width, 10em);
          max-width: var(--hi-menu-max-width, 24em);
          box-shadow: var(--hi-clr--shadow-down-3);
          z-index: var(--hi-depth--above-all);
          background-color: var(--hi-clr--white);
          border: var(--hi-px-1) solid var(--hi-clr--theme-5);
          border-radius: var(--hi-shape--radius);
          padding: var(--hi-px-8);
          outline: none;
        }

        :host(:focus) {
          outline: none;
        }

        :host(:not([opened])) {
          display: none;
        }

        :host([cap-height]) {
          max-height: var(--hi-menu--max-height, 70vh);
          overflow: auto;
        }

        #menuBackdrop {
          display: none;
        }

        #menuHeader {
          display: none;
          position: sticky;
          top: 0;
          left: 0;
          z-index: var(--hi-depth--above-all);
          background-color: var(--hi-clr--white);
        }

        @media only screen and (max-width: 40rem) {
          :host {
            height: 100vh;
            width: 100vw;
            position: fixed;
            top: 0;
            left: 0;
            min-width: unset;
            max-width: unset;
            background-color: unset;
            border: unset;
            box-shadow: unset;
            padding: 0;
            border-radius: unset;
          }

          :host([cap-height]) {
            max-height: unset;
            overflow: unset;
          }

          #menuBackdrop {
            display: block;
            background: rgba(0.2, 0.2, 0.2, 0.2);
          }

          #menuContainer {
            position: fixed;
            bottom: 0;
            left: 0;
            z-index: var(--hi-depth--modal);
            width: 100%;
            max-height: 50vh;
            overflow-y: auto;
            border-radius: var(--hi-shape--radius-4) var(--hi-shape--radius-4) 0 0;
            background-color: var(--hi-clr--white);
          }

          #menuHeader {
            display: block;
          }

          :host ::slotted(*) {
            margin: var(--hi-px-8);
          }
        }
      </style>
      <hi-backdrop
        id="menuBackdrop"
        opened="[[opened]]"
        on-opened-changed="_mobileMenuBackdrop__onOpenedChanged"
      ></hi-backdrop>
      <div id="menuContainer">
        <div id="menuHeader">
          <hi-subtitle title="[[subtitle]]" on-close-ended="__subtitle__onCloseEnded"></hi-subtitle>
        </div>
        <slot id="items"></slot>
      </div>
    `;
  }
  static get api() {
    return ["expand", "collapse", "activate", "deactivate"];
  }
  static get keyBindings() {
    return {
      "up:keydown": "_onKeyAny__preventDefault",
      "down:keydown": "_onKeyAny__preventDefault",
      "right:keyup": "_setTree__onKeyUp_right",
      "down:keyup": "_setTree__onKeyUp_down",
      "left:keyup": "_setTree__onKeyUp_left",
      "up:keyup": "_setTree__onKeyUp_up",
      "end:keyup": "_setFocus__onKeyUp_end",
      "home:keyup": "_setFocus__onKeyUp_home",
      "esc:keyup": "_setTree__onKeyUp_esc",
      "enter:keyup": "__onKeyUp__enter",
      "space:keyup": "__onKeyUp__enter"
    };
  }
  static get browserBindings() {
    return {
      focusout: "_setTree__onFocusout",
      focus: "_setFocus__onFocus"
    };
  }
  static get valueEvent() {
    return "selected-ended";
  }
  static get autoBinding() {
    return {
      signal: "opened-changed",
      field: "activate"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "menubar");
    this.setAttribute("aria-orientation", "vertical");
    this._isCollapsible = true;
    afterNextRender(this, () => {
      this._forEl();
    });
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    super._popout_destroyPopout();
  }
  _forEl() {
    const forEl = HIP.browser.queryOne(this.for, this.ns);
    if (forEl) {
      forEl.setAttribute("aria-haspopup", "menu");
    }
    return forEl;
  }
  _setTree__openedChanged(_opened) {
    if (_opened) {
      this.__lastActiveElement = document.activeElement;
      this._enableMenu();
    } else if (this.__lastActiveElement) {
      this.__lastActiveElement.focus();
    }
    super._setTree__openedChanged(_opened);
  }
  _enableMenu(event) {
    if (LSQ.instance.isDesktop()) {
      this._popout_createPopout(event, void 0);
    } else {
      this._popout_destroyPopout();
    }
    this.forceTabIndex(0);
    this.focus();
    this.focusIndex(this.focusedIndex || 0);
  }
  __onSlotChanged() {
    this._setTree__onSlotChanged();
  }
  __subtitle__onCloseEnded(evt) {
    evt.preventDefault();
    this.collapse();
  }
  _setTree__onKeyUp_enter() {
    super._setTree__onKeyUp_enter();
    this.opened = !this.opened;
  }
  _setTree__onKeyUp_left(e) {
    e.preventDefault();
    const focusedItem = this._indexToItem(this.focusedIndex);
    if (focusedItem && focusedItem.parentElement) {
      const parent = focusedItem.parentElement;
      parent.opened = false;
      parent.focus();
    }
  }
  _setTree__onKeyUp_up(e) {
    super._setTree__onKeyUp_up(e);
    this._setFocus_scrollIntoView();
  }
  _setTree__onKeyUp_down(e) {
    super._setTree__onKeyUp_down(e);
    this._setFocus_scrollIntoView();
  }
  _setFocus_scrollIntoView() {
    const focusedItem = this._indexToItem(this.focusedIndex);
    if (focusedItem) {
      focusedItem.scrollIntoView();
    }
  }
  _mobileMenuBackdrop__onOpenedChanged(e) {
    const opened = e.detail.value;
    if (!opened) {
      super._setTree__onFocusout();
    }
  }
  activate(value, event) {
    super.activate(value, event);
    if (this.opened && LSQ.instance.isDesktop()) {
      this._popout_createPopout(event, void 0);
    }
  }
  _popout_createPopout(event, to) {
    const sourceEl = event ? event.detail.srcElement : this._forEl();
    super._popout_createPopout(event, to, sourceEl, this, {
      placement: "right-start"
    });
    this.scrollTop = 0;
  }
  __onKeyUp__enter() {
    const focusedPickEl = this._indexToItem(this.focusedIndex);
    if (focusedPickEl) {
      focusedPickEl.notify();
    }
  }
  _setFocus__onItemFocused(e) {
    super._setFocus__onItemFocused(e);
    this.__announceFocus(e.target);
  }
  __announceFocus(focusedItem) {
    if (focusedItem) {
      HIP.fn.announce(this._readValueFormatted(focusedItem));
    }
  }
}
export default HIP.factory.element(MenuElement);
