
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { BehaviorItemTree } from '../../../eip/element/hi-behavior-set-tree/hi-behavior-item-tree.js';
import { AsHIC, HIP, BehaviorBrowserEvents } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import treeService from '../../../eip/element/hi-behavior-set-tree/hi-service-tree.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
const styles = css`
  :host(:not([theme])) {
    --hi-clr--theme-1: transparent;
    --hi-clr--theme-3: transparent;
    --hi-clr--theme-5: transparent;
    --hi-clr--theme-text-1: var(--hi-clr--text);
    --hi-clr--theme-text-3: var(--hi-clr--text);
    --hi-clr--theme-text-5: var(--hi-clr--text);
  }

  :host {
    display: list-item;
    list-style: none;
    background-color: var(--hi-clr--theme-1);
    font-size: var(--hi-font-size--text);
    color: var(--hi-clr--theme-text-1);
    padding: var(--hi-px-2);
    box-sizing: border-box;
    line-height: var(--hi-px-28);
    min-height: var(--hi-px-32);
    overflow-wrap: break-word;
    word-wrap: break-word;
    cursor: default;
    hyphens: auto;
    display: block;
    border-bottom: var(--hi-px-1) solid var(--hi-clr--gray-4);
  }

  :host(:hover:not([disabled])) {
    background-color: var(--hi-clr--gray-2);
    text-shadow: var(--hi-clr--text-shadow-1);
  }

  :host([disabled]) {
    color: var(--hi-clr--disabled) !important;
    pointer-events: all !important;
  }

  :host(:focus),
  :host([focused]) {
    outline: var(--hi-outline--width) var(--hi-outline--line) var(--hi-clr--outline);
    outline-offset: var(--hi-outline--offset);
  }

  :host(:hover) {
    background-color: var(--hi-clr--gray-3);
    color: var(--hi-clr--text);
  }

  @media only screen and (max-width: 40rem) {
    :host {
      display: flex;
      align-items: center;
      min-height: var(--hi-el--height-mobile);
      border-bottom: none;
    }
  }
`;
export class PickElement extends BehaviorItemTree(BehaviorBrowserEvents(BehaviorElement(AsHIC(LitElement)))) {
  static get is() {
    return "hi-pick";
  }
  static get properties() {
    return {
      heading: {
        type: String
      },
      value: {
        type: Object,
        reflect: true,
        converter: {
          fromAttribute: (value) => {
            return `${value}`;
          },
          toAttribute: (value) => {
            if (value == null) {
              return null;
            } else if (f1.is.string(value)) {
              return value;
            } else {
              try {
                return JSON.stringify(value);
              } catch (e) {
                return `${value}`;
              }
            }
          }
        }
      }
    };
  }
  static get browserBindings() {
    return {
      click: "__onClick"
    };
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  render() {
    return html`<slot id="items">${this.heading}</slot> `;
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "menuitem");
  }
  __onClick(e) {
    super._itemTree__onTap(e);
  }
  notify() {
    super.trigger();
    this.focused = true;
    treeService.notifyOfSelection(this.tree, this);
    this.tree.opened = false;
  }
}
export default HIP.factory.element(PickElement);
