
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import { AsHIC, HIP, BehaviorBrowserEvents } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorPopout } from '../../../eip/element/hi-behavior-popout/hi-behavior-popout.js';
import { BehaviorNodeTree } from '../../../eip/element/hi-behavior-set-tree/hi-behavior-node-tree.js';
import { BehaviorScreenQuery } from '../../../yip/element/yi-behavior-screen-query/yi-behavior-screen-query.js';
import { LayoutScreenQueryElement as LSQ } from '../../../yip/element/yi-layout-screen-query/yi-layout-screen-query.js';
import '../../../eip/element/hi-icon/hi-icon.js';
import './hi-pick.js';
export class SubMenuElement extends BehaviorNodeTree(
  BehaviorPopout(BehaviorBrowserEvents(BehaviorElement(BehaviorScreenQuery(AsHIC(PolymerElement)))))
) {
  constructor() {
    super(...arguments);
    this.__iconDirection = "top";
  }
  static get is() {
    return "hi-sub-menu";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          position: relative;
          display: block;
          height: 100%;
          border: 0;
          --hi-sub-menu--page-safe-padding: 10rem; /*Avoid headers & footers*/
          --hi-sub-menu--max-height: calc(100vh - var(--hi-sub-menu--page-safe-padding));
        }

        .menu_item {
          display: flex;
          align-items: center;
          outline: none;
          cursor: default;
          line-height: var(--hi-px-28);
          font-size: var(--hi-font-size--text);
          min-width: var(--hi-menu-min-width, 14em);
          min-height: var(--hi-px-32);
          color: var(--hi-clr--text);
          padding: var(--hi-px-2);
          overflow-wrap: break-word;
          word-wrap: break-word;
          hyphens: auto;
          border-bottom: 1px solid var(--hi-clr--gray-4);
        }

        :host([disabled]) > .menu_item {
          color: var(--hi-clr--disabled) !important;
        }

        :host([active]) > .menu_item {
          background: var(--hi-clr--theme-1);
          color: var(--hi-clr--theme-5);
        }

        :host(:focus) > .menu_item,
        :host([focused]) > .menu_item {
          outline: var(--hi-outline--width) var(--hi-outline--line) var(--hi-clr--outline);
          outline-offset: var(--hi-outline--offset);
        }

        .menu_item:hover {
          background-color: var(--hi-clr--gray-2);
          text-shadow: var(--hi-clr--text-shadow-1);
        }

        .container_list {
          min-width: var(--hi-menu-min-width, 10em);
          max-width: var(--hi-menu-max-width, 24em);
          border: var(--hi-px-1) solid var(--hi-clr--theme-5);
          border-radius: var(--hi-shape--radius-2);
          padding: var(--hi-px-8);
          z-index: var(--hi-depth--above-all);
          background-color: var(--hi-clr--white);
          box-shadow: var(--hi-clr--shadow-down-3);
          line-height: var(--hi-px-28);
          min-height: var(--hi-px-32);
        }

        :host(:not([opened])) > .container_list {
          display: none;
        }

        :host([cap-height]) .container_list {
          max-height: var(--hi-sub-menu--max-height);
          overflow: auto;
        }

        #heading {
          flex-grow: 1;
        }

        hi-icon {
          margin: 0 var(--hi-px-8);
          height: var(--hi-px-16);
          width: var(--hi-px-16);
          overflow: visible;
        }

        :host([disabled]) > hi-icon {
          --hi-icon--color: var(--hi-clr--theme-5);
        }

        @media only screen and (max-width: 40rem) {
          :host {
            --hi-sub-menu--page-safe-padding: unset;
            --hi-sub-menu--max-height: unset;
          }

          :host([cap-height]) .container_list {
            max-height: unset;
            overflow: unset;
          }

          .container_list {
            min-width: unset;
            max-width: unset;
            border: unset;
            border-radius: unset;
            box-shadow: unset;
            min-height: unset;
          }

          :host > .menu_item {
            min-height: var(--hi-px-48);
            border-bottom: unset;
          }
        }
      </style>
      <div id="menu_item" class="menu_item" role="menuitem" aria-owns="container_list">
        <span id="heading">
          <slot name="heading"> [[ heading ]] </slot>
        </span>
        <hi-icon id="indicator" kind$="[[ indicator ]]" theme$="[[ indicatorTheme ]]"> </hi-icon>
        <hi-icon
          id="icon"
          theme$="[[ __iconTheme ]]"
          kind$="[[ __iconKind ]]"
          direction$="[[ __iconDirection ]]"
          disabled$="[[ disabled ]]"
        ></hi-icon>
      </div>
      <div id="container_list" class="container_list" role="menu">
        <slot id="items"></slot>
      </div>
    `;
  }
  static get properties() {
    return {
      heading: {
        type: String
      },
      __iconKind: {
        type: String
      },
      __iconTheme: {
        type: String
      },
      __iconDirection: {
        type: String
      },
      indicator: {
        type: String
      },
      indicatorTheme: {
        type: String
      },
      placement: {
        type: String,
        value: "right-start"
      },
      capHeight: {
        type: Boolean
      }
    };
  }
  static get observers() {
    return ["__visualIconStateObserver(opened, disabled)", "__labelSubMenu(heading)"];
  }
  static get browserBindings() {
    return {
      click: "__onClick",
      mouseleave: "__onMouseLeave"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.__menuItem = this.$.menu_item;
    this.__containerList = this.$.container_list;
    const id = this.getAttribute("id");
    this.removeAttribute("id");
    this.setAttribute("active-id", id);
    this.menuItem.setAttribute("id", id);
    this.menuItem.setAttribute("aria-haspopup", "true");
    this.menuItem.setAttribute("aria-expanded", `${this.opened}`);
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    super._popout_destroyPopout();
  }
  __onSlotChanged() {
    this._setTree__onSlotChanged();
    this.__visualIconStateObserver(this.opened, this.disabled);
  }
  _nodeTree__openedChanged(opened) {
    super._nodeTree__openedChanged(opened);
    if (this.menuItem) {
      this.menuItem.setAttribute("aria-expanded", `${opened}`);
    }
    if (opened && LSQ.instance.isDesktop()) {
      this._popout_createPopout();
    }
    return opened;
  }
  _includeNodes(root) {
    this.tree.__slotChildren = root.slotChildren;
    return this.tree.slotChildren.indexOf(root.children[0]);
  }
  _excludeNodes(root) {
    this.tree.__slotChildren = root.parentElement.__originalChildren || root.parentElement.slotChildren;
    return this.tree.slotChildren.indexOf(root);
  }
  __onMouseLeave() {
    if (LSQ.instance.isDesktop()) {
      this.opened = false;
    }
  }
  __onClick(e) {
    e.preventDefault();
    e.stopPropagation();
    if (LSQ.instance.isDesktop()) {
      this.opened = true;
    } else {
      this.opened = !this.opened;
    }
  }
  notify() {
    this.opened = true;
  }
  __visualIconStateObserver(_opened, _disabled) {
    if (_disabled) {
      this.__iconKind = "dash";
      this.__iconTheme = this.theme;
    } else if (this.slotChildren.length > 0) {
      this.__iconKind = "chevron_right";
      this.__iconTheme = this.theme;
    } else {
      this.__iconKind = "chevron_right";
      this.__iconTheme = "gray";
    }
    if (this.menuItem) {
      this.menuItem.setAttribute("aria-expanded", `${this.opened}`);
    }
    if (LSQ.instance.isMobile()) {
      this.__iconDirection = this.opened ? "right" : "top";
    }
  }
  __labelSubMenu(heading) {
    this.setAttribute("label", heading);
  }
  _popout_createPopout(from, to) {
    super._popout_createPopout(from, to, this.menuItem, this.containerList, {
      placement: "right-start",
      strategy: "absolute"
    });
  }
  get menuItem() {
    return this.__menuItem;
  }
  get containerList() {
    return this.__containerList;
  }
}
export default HIP.factory.element(SubMenuElement);
