
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import { AsHIC, HIP } from '../../../eip/element/hi-component/hi-component.js';
export class MeterElement extends BehaviorElement(AsHIC(PolymerElement)) {
  static get is() {
    return "hi-meter";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          --meter--height: var(--hi-px-16);
          -webkit-print-color-adjust: exact;
          color-adjust: exact;
        }

        meter {
          -webkit-appearance: meter;
          overflow: hidden;
          box-sizing: border-box;
          display: block;
          width: 100%;
          background: none;
          height: var(--meter--height);
        }

        /* HACK Browser hacks to target firefox & safari only*/
        @-moz-document url-prefix() {
          meter {
            border: var(--hi-px-1) solid var(--hi-clr--theme-3);
          }
        }

        @media not all and (min-resolution: 0.001dpcm) {
          meter {
            border: var(--hi-px-1) solid var(--hi-clr--theme-5);
          }
        }

        meter::-webkit-meter-bar {
          border: var(--hi-px-1) solid var(--hi-clr--theme-5);
          background-color: var(--hi-clr--gray-1);
          border-color: var(--hi-clr--theme-3);
        }

        :host([kind='banding']) meter::-webkit-meter-bar {
          border-color: var(--hi-clr--gray-8);
        }

        meter::-webkit-meter-bar {
          background-size: var(--hi-px-34) var(--meter--height);
          background-image: linear-gradient(
            135deg,
            transparent,
            transparent 33%,
            var(--hi-clr--gray-3) 34%,
            var(--hi-clr--gray-3) 67%,
            transparent 67%
          );
        }

        meter::-webkit-meter-inner-element {
          display: block;
        }

        :host([kind='value']) :-moz-meter-optimum::-moz-meter-bar,
        :host([kind='value']) :-moz-meter-sub-optimum::-moz-meter-bar {
          background: var(--hi-clr--theme-3);
        }

        :host([kind='value']) meter::-webkit-meter-optimum-value,
        :host([kind='value']) meter::-webkit-meter-suboptimum-value {
          background: var(--hi-clr--theme-3);
        }

        :host([kind='banding']) meter::-webkit-meter-optimum-value {
          background: var(--hi-clr--green-3);
        }

        :host([kind='banding']) meter::-webkit-meter-suboptimum-value {
          background: var(--hi-clr--yellow-3);
        }

        :-moz-meter-optimum::-moz-meter-bar {
          background: var(--hi-clr--green-3);
        }

        :host([kind='banding']) :-moz-meter-sub-optimum::-moz-meter-bar {
          background: var(--hi-clr--yellow-3);
        }
      </style>
      <div id="meterGroup">
        <meter
          id="meterId"
          name$="[[ name ]]"
          min$="[[ min ]]"
          low$="[[ low ]]"
          high$="[[ high ]]"
          max$="[[ max ]]"
          value$="[[ value ]]"
          optimum$="[[ optimum ]]"
        ></meter>
      </div>
    `;
  }
  static get properties() {
    return {
      kind: {
        type: String,
        reflectToAttribute: true,
        value: "value"
      },
      min: {
        type: Number,
        value: 0
      },
      max: {
        type: Number,
        value: 100
      },
      low: {
        type: Number
      },
      high: {
        type: Number
      },
      optimum: {
        type: Number
      },
      value: {
        type: Number
      }
    };
  }
}
HIP.factory.element(MeterElement);
