
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { AsLitHIC, HIP, BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import '../../../eip/element/hi-breadcrumb/hi-breadcrumb.js';
import '../../../eip/element/hi-link/hi-link.js';
import navigationService from './hi-service-navigation.js';
export const styles = css`
  :host {
    display: block;
    background: var(--hi-clr--white);
    min-height: var(--hi-screen--nav-hierarchy);
    border-bottom: var(--hi-px-1) solid var(--hi-clr--gray-3);
    padding: var(--hi-px-4) var(--hi-px-16) var(--hi-px-4) calc(var(--hi-screen--nav-tertiary) + var(--hi-px-16));
  }
`;
export class HierarchyNavigationElement extends BehaviorElement(AsLitHIC(LitElement)) {
  static get is() {
    return "hi-navigation-hierarchy";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      steps: {
        type: Array
      }
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "region");
    navigationService.toggleMicroNavigationStyling(this, "hierarchy", true);
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    navigationService.toggleMicroNavigationStyling(this, "hierarchy", false);
  }
  render() {
    const steps = f1.filter(
      (_) => !f1.is.empty(_.uri) && !f1.is.empty(_.title),
      f1.safe([], this.steps)
    );
    return html`<hi-breadcrumb
      >${steps.map(
      (step, index) => html`<hi-crumb .selected=${this.__isLast(index, this.steps)}>
          <hi-link href=${step.uri}>${step.title}</hi-link>
        </hi-crumb>`
    )}</hi-breadcrumb
    >`;
  }
  __isLast(index, steps) {
    return index >= steps.length - 1;
  }
}
export default HIP.factory.element(HierarchyNavigationElement);
