
import { AsHIC, HIP, BehaviorElement, BehaviorA11yKeyboard } from '../../../eip/element/hi-component/hi-component.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { ifDefined } from '../../../hip/library/lit-html/directives/if-defined.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import '../../../eip/element/hi-avatar/hi-avatar.js';
import '../../../eip/element/hi-icon/hi-icon.js';
import '../../../eip/element/hi-eab/hi-eab.js';
import '../../../eip/element/hi-flyout/hi-flyout.js';
import '../../../eip/element/hi-list/hi-list.js';
import '../../../eip/element/hi-tab/hi-tab.js';
import '../../../eip/element/hi-tab/hi-tab-group.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import navigationService from './hi-service-navigation.js';
export const styles = css`
  :host {
    --hi-clr--panel-bkg: #fcfcfc;
    --hi-clr--panel-shadow: rgb(0 0 0 / 35%) 0px 2px 10px 5px;
  }

  :host {
    background: var(--hi-clr--white);
    display: block;
    border-bottom: var(--hi-px-1) solid var(--hi-clr--gray-3);
    height: var(--hi-screen--nav-secondary);
  }

  :host([layout='center']) #secondaryContainer {
    width: var(--hi-site--narrow-width);
  }

  #secondaryContainer {
    margin: var(--hi-px-0) auto;
    display: flex;
    align-items: center;
    justify-content: space-between;
    height: 100%;
    width: calc(var(--hi-site--full-width) - var(--hi-px-24));
  }

  #left-side {
    display: block !important;
    margin-left: var(--hi-px-8);
  }

  #left-side,
  #right-side,
  #logo-slot {
    display: inline-flex;
    align-items: center;
  }

  #left-side > * ~ *,
  #right-side > * ~ * {
    margin-left: var(--hi-px-24);
  }

  #center {
    flex: 1;
  }

  #leftControls ::slotted([slot='left-controls']),
  #rightControls ::slotted([slot='right-controls']) {
    display: flex;
  }

  #sectionActions__tabGroup hi-link {
    padding: var(--hi-px-16) var(--hi-px-8) var(--hi-px-8);
  }

  #siteMap__menu {
    margin: var(--hi-px-7) 0 0 var(--hi-px-4);
  }

  #siteMap__menu hi-tab[active] {
    border-bottom: var(--hi-px-3) solid var(--hi-clr--theme-3);
    outline: var(--hi-outline--width) var(--hi-outline--line) var(--hi-clr--outline);
    border-top: none;
  }

  #siteMap__menu hi-tab:not([active]) {
    background-color: var(--hi-clr--gray-1);
    border: var(--hi-px-1) solid var(--hi-clr--gray-3);
    font-weight: var(--hi-font--weight-light);
    color: var(--hi-clr--text);
  }

  #siteMap__menu hi-tab:not([active]):hover {
    text-shadow: var(--hi-clr--text-shadow-1);
  }

  #siteMap__panel {
    position: absolute;
    z-index: var(--hi-depth--above-all);
    padding: var(--hi-px-24);
    min-height: 10rem;
    box-shadow: var(--hi-clr--panel-shadow);
    border-radius: var(--hi-shape--radius-3);
    display: flex;
    justify-content: center;
    background-color: var(--hi-clr--panel-bkg);
    margin: var(--hi-px-4) var(--hi-px-48) 0 var(--hi-px-24);
  }

  #siteMap__subMenu {
    display: flex;
    flex-wrap: wrap;
  }

  #siteMap__subMenuClose {
    align-self: flex-end;
  }

  #siteMap__subMenu ul {
    display: flex;
    flex-direction: column;
    list-style: none;
    margin: var(--hi-px-16);
    padding: var(--hi-px-0);
    max-width: 11rem;
    width: 100%;
  }

  #siteMap__subMenu hi-link {
    line-height: 1.5;
    font-size: var(--hi-font-size--medium);
  }

  #siteMap__subMenuItem {
    margin-top: var(--hi-px-8);
  }

  #siteMap__subMenuCloseButton {
    position: absolute;
    right: var(--hi-px-16);
    top: var(--hi-px-16);
  }

  #logo {
    display: inline-flex !important;
    align-items: center;
  }

  #app-title {
    display: inline-block !important;
    margin-left: var(--hi-px-4) !important;
    font-size: var(--hi-font-size--h4);
    font-weight: var(--hi-font--weight-bold);
    color: var(--hi-clr--theme-3);
    text-transform: uppercase;
    white-space: nowrap;
  }

  /* Remove this styling once all icons are updated with same size */
  #anchorIcon {
    padding-left: var(--hi-px-1);
  }

  .accountActionTitle__navigationSecondary {
    margin-left: var(--hi-px-3);
  }
`;
export class NavigationSecondaryElement extends BehaviorA11yKeyboard(BehaviorElement(AsHIC(LitElement))) {
  constructor() {
    super(...arguments);
    this.title = "Navigate";
    this.accountAvatar = {};
    this.fallbackAccountAvatarInitials = "?";
  }
  static get is() {
    return "hi-navigation-secondary";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      title: {
        type: String
      },
      index: {
        type: Number
      },
      activeIndex: {
        type: Number
      },
      accountAvatar: {
        type: Object,
        attribute: "account-avatar"
      },
      fallbackAccountAvatarInitials: {
        readOnly: true,
        type: String,
        attribute: "fallback-account-avatar-initials"
      },
      sectionActions: {
        type: Array,
        attribute: "section-actions"
      },
      topLevelActions: {
        type: Array,
        attribute: "top-level-actions"
      },
      accountActions: {
        type: Array,
        attribute: "account-actions"
      },
      siteMap: {
        type: Array,
        attribute: "site-map"
      }
    };
  }
  static get keyBindings() {
    return {
      "esc:keyup": "__onKeyUp_escape"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    dom.setAttrs(this, {
      role: "navigation",
      "aria-label": "Main Menu"
    });
    navigationService.toggleMicroNavigationStyling(this, "secondary", true);
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    navigationService.toggleMicroNavigationStyling(this, "secondary", false);
  }
  render() {
    const showAccountAvatar = !f1.is.empty(this.accountActions);
    const showSectionActions = !f1.is.empty(this.sectionActions);
    const showTopLevelActions = !f1.is.empty(this.topLevelActions);
    const showSiteMap = !f1.is.empty(this.siteMap);
    const accountAvatarInitials = this.accountAvatar.initials != null ? this.accountAvatar.initials : this.fallbackAccountAvatarInitials;
    const accountAvatarImage = this.accountAvatar.pictureURI != null ? this.accountAvatar.pictureURI : void 0;
    const accountAvatarBorderColor = this.accountAvatar.theme != null ? this.accountAvatar.theme : "blue";
    const accountAvatarTitle = this.accountAvatar.title != null ? this.accountAvatar.title : "Help Menu";
    return html`
      <section id="secondaryContainer">
        <div id="logo-slot">
          <slot name="logo" id="logo">
            <hi-eab disllow-animation></hi-eab>
            <p id="app-title">${this.title}</p>
          </slot>
        </div>
        <div id="left-side">
          ${showSectionActions ? html`<hi-link-group id="sectionActions__tabGroup" active-mode="automatic">
                ${this.sectionActions?.map((sectionAction, index) => {
      return html`
                    <hi-link
                      id="sectionActions__tab_${index}"
                      value=${index}
                      active-path=${ifDefined(sectionAction.uri)}
                      ?disabled=${sectionAction.disabled}
                      aesthetic="navigation"
                      href=${ifDefined(sectionAction.href)}
                    >
                      ${sectionAction.title}
                    </hi-link>
                  `;
    })}
              </hi-link-group>` : nothing}
          ${showSiteMap ? html`
                <hi-tab-group
                  id="siteMap__menu"
                  @commit-ended=${() => this.__siteMap__onCommitEndedMenu(0)}
                  @focusout=${this.__siteMap__onFocusOutTabGroupClose}

                  .ns=${this.ns}
                  ><hi-tab .ns=${this.ns}>Menu</hi-tab>
                </hi-tab-group>
                <hi-panel
                  id="siteMap__panel"
                  .ns=${this.ns}
                  @mouseleave=${this.__siteMap__onTriggerEndedClose}
                  @focusout=${this.__siteMap__onFocusOutSiteMapClose}
                  hidden>
                  <div id="siteMap__subMenu">
                    ${this.siteMap.map((sectionAction) => {
      const navItems = f1.safe([], sectionAction.navigation_items);
      return html`<ul id="siteMap__subMenuItemContainer">
                        <li>
                          <h6>${sectionAction.title}</h6>
                        </li>
                        ${navItems.map(
        (navItem) => html`<li id="siteMap__subMenuItem">
                            <hi-link
                              aesthetic="navigation"
                              .ns=${this.ns}
                              href=${ifDefined(navItem.uri)}
                              @commit-ended=${this.__siteMap__onTriggerEndedClose}
                              >${navItem.title}</hi-link
                            >
                          </li>`
      )}
                      </ul>`;
    })}
                  </div>
                  <hi-close
                    id="siteMap__subMenuCloseButton"
                    @trigger-ended=${this.__siteMap__onTriggerEndedClose}
                    size="medium"
                    theme="gray-dark"
                    .ns=${this.ns}
                  >
                  </hi-close>
                </hi-panel>
              </div>` : nothing}
          <div id="leftControls">
            <slot name="left-controls"></slot>
          </div>
        </div>

        <div id="center">
          <div id="centerControls">
            <slot name="center-controls"></slot>
          </div>
        </div>

        <div id="right-side">
          <div id="rightControls">
            <slot name="right-controls"></slot>
          </div>
          ${showTopLevelActions ? html`<hi-flyout id="topLevelActions__flyout" aesthetic="ghost" alignment="left">
                <hi-icon id="anchorIcon" slot="anchor" kind="9_dots" align="middle"></hi-icon>
                <hi-link-group @selected-ended=${this.__topLevelAction__onChosenEnded} kind="stacked">
                  ${this.topLevelActions?.map((action, index) => {
      return html`
                      <hi-link
                        value=${index}
                        aesthetic="navigation"
                        href=${ifDefined(action.href)}
                        target=${ifDefined(action.target)}
                        ?disabled=${action.disabled}
                        ?external=${action.external}
                      >
                        ${action.icon ? html`<hi-icon kind=${action.icon}></hi-icon>` : nothing}
                        <span>${action.title}</span>
                      </hi-link>
                    `;
    })}
                </hi-link-group>
              </hi-flyout>` : nothing}
          ${showAccountAvatar ? html`<hi-flyout id="accountAvatar__flyout" aesthetic="ghost" alignment="left">
                <span slot="anchor">
                  <hi-avatar
                    size="small"
                    allow-border-color
                    interactive
                    theme=${ifDefined(accountAvatarBorderColor)}
                    image=${ifDefined(accountAvatarImage)}
                    initials=${ifDefined(accountAvatarInitials)}
                    aria-label=${accountAvatarTitle}
                  >
                  </hi-avatar>
                </span>
                <hi-link-group @selected-ended=${this.__accountAction__onChosenEnded} kind="stacked">
                  ${this.accountActions?.map((action, index) => {
      return html`
                      <hi-link
                        value=${index}
                        href=${ifDefined(action.href)}
                        target=${ifDefined(action.target)}
                        ?disabled=${action.disabled}
                        ?download=${action.download}
                        ?external=${action.external}
                        file-name=${ifDefined(action.fileName)}
                      >
                        ${action.icon ? html`<hi-icon kind=${action.icon} type="material"></hi-icon>` : nothing}
                        <span class="accountActionTitle__navigationSecondary">${action.title}</span>
                      </hi-link>
                    `;
    })}
                </hi-link-group>
              </hi-flyout>` : nothing}
        </div>
      </section>
    `;
  }
  __siteMapHide(siteMapPanelEl, selTab) {
    dom.setAttrs(siteMapPanelEl, {
      hidden: true
    });
    dom.setAttrs(selTab, {
      active: false
    });
  }
  __siteMapShow(siteMapPanelEl, selectedTabEl, firstSubMenuEl) {
    dom.setAttrs(siteMapPanelEl, {
      hidden: false
    });
    dom.setAttrs(selectedTabEl, {
      active: true
    });
    firstSubMenuEl.focus();
  }
  __onKeyUp_escape(event, keyboardEvent) {
    if (!this.shadowRoot) {
      return;
    }
    const shadowRoot = this.shadowRoot;
    const siteMapPanelEl = shadowRoot.getElementById("siteMap__panel");
    const menuEl = shadowRoot.getElementById("siteMap__menu");
    if (!menuEl || !siteMapPanelEl) {
      return;
    }
    const tabsEls = Array.from(menuEl.querySelectorAll("hi-tab"));
    const selectedTabEl = tabsEls[this.activeIndex];
    if (!siteMapPanelEl.hidden && keyboardEvent.code === "Escape") {
      this.__siteMapHide(siteMapPanelEl, selectedTabEl);
      selectedTabEl.focus();
    }
  }
  __siteMap__onCommitEndedMenu(index) {
    if (!this.shadowRoot) {
      return;
    }
    const shadowRoot = this.shadowRoot;
    const siteMapPanelEl = shadowRoot.getElementById("siteMap__panel");
    const menuEl = shadowRoot.getElementById("siteMap__menu");
    if (!menuEl || !siteMapPanelEl) {
      return;
    }
    const tabsEls = Array.from(menuEl.querySelectorAll("hi-tab"));
    const selectedTabEl = tabsEls[index];
    const subMenuEls = Array.from(siteMapPanelEl.querySelectorAll("hi-link"));
    const firstSubMenuEl = subMenuEls[0];
    if (siteMapPanelEl.hidden) {
      this.__siteMapShow(siteMapPanelEl, selectedTabEl, firstSubMenuEl);
      this.activeIndex = index;
      this.index = index;
    } else if (!siteMapPanelEl.hidden) {
      this.__siteMapHide(siteMapPanelEl, selectedTabEl);
      selectedTabEl.focus();
    }
  }
  __siteMap__onTriggerEndedClose(e) {
    if (!this.shadowRoot) {
      return;
    }
    const shadowRoot = this.shadowRoot;
    const siteMapPanelEl = shadowRoot.getElementById("siteMap__panel");
    const menuEl = shadowRoot.getElementById("siteMap__menu");
    if (!menuEl || !siteMapPanelEl) {
      return;
    }
    const tabsEls = Array.from(menuEl.querySelectorAll("hi-tab"));
    const selectedTabEl = tabsEls[this.activeIndex];
    const relatedTarget = e.relatedTarget;
    const onHoverNavigationSecondary = this.contains(relatedTarget);
    if (!onHoverNavigationSecondary) {
      this.__siteMapHide(siteMapPanelEl, selectedTabEl);
      selectedTabEl.focus();
    }
  }
  __siteMap__onFocusOutSiteMapClose(e) {
    if (!this.shadowRoot) {
      return;
    }
    const shadowRoot = this.shadowRoot;
    const siteMapPanelEl = shadowRoot.getElementById("siteMap__panel");
    const menuEl = shadowRoot.getElementById("siteMap__menu");
    if (!siteMapPanelEl || !menuEl) {
      return;
    }
    const tabsEls = Array.from(menuEl.querySelectorAll("hi-tab"));
    const selectedTabEl = tabsEls[this.activeIndex];
    const currentTarget = e.currentTarget;
    const relatedTarget = e.relatedTarget;
    const closeSiteMapPanel = !currentTarget.contains(relatedTarget);
    if (!relatedTarget) {
      return;
    }
    const onSelectedTabEl = relatedTarget.contains(selectedTabEl);
    if (closeSiteMapPanel && !onSelectedTabEl) {
      this.__siteMapHide(siteMapPanelEl, selectedTabEl);
    }
  }
  __siteMap__onFocusOutTabGroupClose(e) {
    if (!this.shadowRoot) {
      return;
    }
    const shadowRoot = this.shadowRoot;
    const siteMapPanelEl = shadowRoot.getElementById("siteMap__panel");
    const menuEl = shadowRoot.getElementById("siteMap__menu");
    const relatedTarget = e.relatedTarget;
    if (!siteMapPanelEl || !menuEl) {
      return;
    }
    const tabsEls = Array.from(menuEl.querySelectorAll("hi-tab"));
    const selectedTabEl = tabsEls[this.activeIndex];
    const focusOutSiteMapPanel = !siteMapPanelEl.contains(relatedTarget);
    const focusOutTabGroup = !menuEl.contains(relatedTarget);
    const closeSiteMapPanel = focusOutSiteMapPanel && focusOutTabGroup;
    if (closeSiteMapPanel) {
      this.__siteMapHide(siteMapPanelEl, selectedTabEl);
    }
  }
  __accountAction__onChosenEnded(e) {
    this._dispatchProcess(
      "account-action",
      Object.assign(e.detail, {
        value: this.accountActions[e.detail.value]
      })
    );
  }
  __topLevelAction__onChosenEnded(e) {
    this._dispatchProcess(
      "top-level-action",
      Object.assign(e.detail, {
        value: this.topLevelActions[e.detail.value]
      })
    );
  }
  __sectionAction__onSelectedEnded(e) {
    this._dispatchProcess(
      "section-action",
      Object.assign(e.detail, {
        value: this.sectionActions[e.detail.value]
      })
    );
  }
}
export default HIP.factory.element(NavigationSecondaryElement);
