
import { LayoutScreenQueryElement as LSQ } from '../../../yip/element/yi-layout-screen-query/yi-layout-screen-query.js';
import { AsHIC, HIP, BehaviorElement, BehaviorA11yKeyboard } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorContainer } from '../../../eip/element/hi-behavior-container/hi-behavior-container.js';
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { ifDefined as ifDef } from '../../../hip/library/lit-html/directives/if-defined.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import '../../../eip/element/hi-button/hi-button.js';
import '../../../eip/element/hi-image/hi-image.js';
import '../../../eip/element/hi-icon/hi-icon.js';
import '../../../eip/element/hi-hint/hi-hint.js';
import '../../../eip/element/hi-link/hi-link.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import navigationService from './hi-service-navigation.js';
export class TertiaryNavigationElement extends BehaviorContainer(
  BehaviorA11yKeyboard(BehaviorElement(AsHIC(LitElement)))
) {
  constructor() {
    super(...arguments);
    this.toggleButtonText = "Navigation Menu";
    this.opened = false;
    this.screenQuery = LSQ.instance;
  }
  static get is() {
    return "hi-navigation-tertiary";
  }
  static get properties() {
    return {
      navigationActions: {
        type: Array,
        attribute: "navigation-actions"
      },
      toggleButtonText: {
        type: String,
        attribute: "toggle-button-text"
      },
      opened: {
        type: Boolean,
        reflectToAttribute: true,
        notify: true
      },
      alwaysOpened: {
        type: Boolean,
        attribute: "always-opened",
        reflect: true
      },
      activeMode: {
        type: String,
        attribute: "active-mode"
      },
      selectedNavigationItem: {
        type: Object,
        attribute: "selected-navigation-item"
      }
    };
  }
  static get keyBindings() {
    return {
      "esc:keyup": "__closeTertiaryNavigation"
    };
  }
  static get api() {
    return ["toggle", "close", "open"];
  }
  static get autoBinding() {
    return {
      signal: "opened-changed",
      field: "opened"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    navigationService.toggleMicroNavigationStyling(this, "tertiary", true);
    this.screenQuery.subscribe(this);
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    navigationService.toggleMicroNavigationStyling(this, "tertiary", false);
    this.screenQuery.unsubscribe(this);
  }
  mediaDeviceHook(mediaName) {
    if (this.alwaysOpened && mediaName !== "mobile") {
      this.open();
    } else if (mediaName === "mobile") {
      this.close();
    }
  }
  update(props) {
    super.update(props);
    if (props.has("alwaysOpened") && this.alwaysOpened && !this.screenQuery.isMobile()) {
      this.opened = true;
    }
  }
  render() {
    const getFirstLetter = (navigationItem) => f1.cut(0, 1, f1.read("/title", "", navigationItem));
    const isLinkActive = (navigationItem) => {
      return this.activeMode !== "automatic" && this.selectedNavigationItem && navigationItem.title === this.selectedNavigationItem.title;
    };
    const navItems = f1.safe([], this.navigationActions);
    const navBadges = f1.map(
      (navItem) => f1.read("/badge_count", void 0, navItem),
      navItems
    );
    const hasBadges = f1.some((badgeCount) => f1.is.value(badgeCount), navBadges);
    const totalBadgeCount = f1.reduce(
      (total, badgeCount) => total + f1.to.integer(f1.safe(0, badgeCount)),
      0,
      navBadges
    );
    return html`
      ${this._renderStyle()}
      ${!this.opened && !f1.is.empty(this.navigationActions) ? html`<hi-button
            id="navMenuControlMobile"
            x-style="nav-menu-control-mobile"
            aesthetic="icon"
            theme="white"
            aria-label="Open ${this.toggleButtonText}"
            aria-controls="tertiaryNavigation"
            aria-pressed=${this.opened}
            aria-expanded=${this.opened}
            @trigger-ended=${this.__navMenuControlMobile__onTriggerEnded}
          >
            <hi-icon kind="menu" mode="presentation"></hi-icon>
            ${!this.opened && hasBadges ? html` <hi-hint for="container" placement="right"> Open ${this.toggleButtonText} </hi-hint>
                  <hi-badge class="bdr-none" dock="top-right" aesthetic="dark" theme="red"
                    >${totalBadgeCount}</hi-badge
                  >` : nothing}
          </hi-button>` : nothing}
      <nav
        id="tertiaryNavigation"
        x-style="tertiary-nav-container"
        ?opened=${this.opened}
        ?always-opened=${this.alwaysOpened}
        aria-label="Features Menu"
      >
        <hi-link-group
          id="hi-nav-tertiary-link-group"
          kind="stacked"
          x-style="tertiary-navigation-items-container"
          active-mode=${ifDef(this.activeMode)}
        >
          ${f1.safe([], this.navigationActions).map((navigationItem, index) => {
      return html`
              <hi-link
                id="hi-nav-tertiary-link-${index}"
                layout="navigation"
                aesthetic="navigation"
                href=${navigationItem.uri || ""}
                value=${index}
                active-path=${ifDef(navigationItem.uri)}
                x-style="nav-list-item"
                x-qa="hi-nav-tertiary-link-${ifDef(navigationItem.key)}"
                ?disabled=${navigationItem.disabled}
                ?active=${isLinkActive(navigationItem)}
                ?external=${navigationItem.external}
                @trigger-ended=${this.__navigationItemSelected}
                target=${navigationItem.target || "_self"}
              >
                <div x-style="nav-link-item-wrapper">
                  ${navigationItem.icon ? html`<hi-icon
                        glyph=${ifDef(navigationItem.icon)}
                        theme=${ifDef(navigationItem.icon_theme || navigationItem.theme)}
                        mode="presentation"
                        size="large-x"
                      >
                      </hi-icon>` : nothing}
                  ${navigationItem.image ? html`<hi-image
                        src=${ifDef(navigationItem.image)}
                        alt="Image of ${f1.word(navigationItem.image)}"
                        x-style="nav-item-image"
                        size="large-x"
                        width="24"
                        height="24"
                      >
                      </hi-image>` : nothing}
                  ${!navigationItem.icon && !navigationItem.image ? html`<span x-style="menu_letter_icon" class="text"> ${getFirstLetter(navigationItem)} </span>` : nothing}
                  <span class=${this.opened ? "text-ellipsis clr-gray_100-txt ml-5" : "show-ax"}>
                    ${navigationItem.title}
                  </span>
                  ${!navigationItem.badge_count ? nothing : this.opened ? html`
                        <hi-badge
                          theme=${navigationItem.badge_theme || navigationItem.theme || "red"}
                          aesthetic="dark"
                          class="ml-4"
                          heading="notifications"
                          aria-label="${navigationItem.title} needs attention"
                          >${navigationItem.badge_count}</hi-badge
                        >
                      ` : html`<hi-badge class="bdr-none" dock="top-right" aesthetic="dark" theme="red" size="small"
                        >${navigationItem.badge_count}</hi-badge
                      >`}
                </div>
                ${navigationItem.release_status && this.opened ? html` <hi-badge
                      theme=${navigationItem.status_theme || navigationItem.theme || "gray"}
                      size="small"
                      class="ml-4"
                      heading="${navigationItem.release_status} badge"
                      aria-label="Menu status is ${navigationItem.release_status}"
                      >${f1.word(f1.safe("", navigationItem.release_status))}</hi-badge
                    >` : nothing}
              </hi-link>
            `;
    })}
        </hi-link-group>
        ${f1.safe([], this.navigationActions).map((navigationItem, index) => {
      return html`<hi-tooltip for="hi-nav-tertiary-link-${index}" placement="right">
            <span aria-hidden="true">${navigationItem.title}</span>
          </hi-tooltip>`;
    })}
        ${(!this.alwaysOpened || this.screenQuery.isMobile()) && !f1.is.empty(this.navigationActions) ? html`<hi-button
              id="navToggleBtn"
              aesthetic="ghost"
              stretched
              x-style="nav-list-item"
              aria-controls="tertiaryNavigation"
              aria-pressed=${this.opened}
              aria-expanded=${this.opened}
              aria-label="Close ${this.toggleButtonText}"
              @trigger-ended=${this.__navToggleBtn__onTriggerEnded}
            >
              <div id="container">
                <hi-icon direction=${this.opened ? "left" : "right"} glyph="chevron_up" mode="presentation"></hi-icon>
                ${this.opened ? html` <span class="text-nowrap ml-3"> Close ${this.toggleButtonText} </span> ` : nothing}
              </div>
            </hi-button>` : nothing}
      </nav>
    `;
  }
  _renderStyle() {
    return html`
      <style>
        hi-navigation-tertiary {
          --bottom-button-height: var(--hi-px-48);
          --hi-nav-tertiary-top-offset: var(
            --hi-screen--top,
            calc(var(--hi-screen--nav-primary) + var(--hi-screen--nav-secondary))
          );
        }

        hi-navigation-tertiary #navToggleBtn {
          border-top: var(--hi-px-1) solid var(--hi-clr--gray-3);
        }

        hi-navigation-tertiary #navToggleBtn {
          position: absolute;
          bottom: 0;
          border-radius: 0;
          box-shadow: none;
          height: var(--bottom-button-height);
        }

        hi-navigation-tertiary #navToggleBtn #container {
          display: flex;
          align-items: center;
          justify-content: flex-start;
          width: 100%;
          margin-left: var(--hi-px-8);
        }

        hi-navigation-tertiary :where(hi-image, hi-link, hi-icon) {
          cursor: pointer;
        }

        [x-style='nav-list-item'] {
          display: block;
          border-left: var(--hi-px-2) solid transparent;
        }

        hi-link[x-style='nav-list-item'] {
          height: calc(var(--hi-px-40) + var(--hi-px-12));
          padding: var(--hi-px-16);
          line-height: var(--hi-px-24);
        }

        [x-style='nav-list-item'][active] {
          background: var(--hi-clr--blue-1);
          border-left-color: var(--hi-clr--blue-3);
          --hi-link--color: var(--hi-clr--theme-5);
        }

        [x-style='nav-list-item']:not([disabled]):hover,
        [x-style='nav-list-item']:not([disabled]):focus-within {
          background-color: var(--hi-clr--gray-2);
          border-left-color: var(--hi-clr--gray-6);
        }

        [x-style='nav-list-item'][disabled] {
          border-left-color: var(--hi-clr--gray-8);
        }

        [x-style='tertiary-nav-container'],
        [x-style='nav-menu-control-mobile'] {
          top: var(--hi-nav-tertiary-top-offset);
          position: fixed;
          z-index: 1000;
        }

        [x-style='nav-menu-control-mobile'] {
          border: var(--hi-px-1) solid var(--hi-clr--black);
          background: var(--hi-clr--white);
          left: var(--hi-px-4);
          display: none;
        }

        [x-style='nav-menu-control-mobile'] * {
          margin-right: 0 !important;
        }

        [x-style='tertiary-nav-container'] {
          bottom: 0;
          left: 0;
          width: var(--hi-screen--nav-tertiary);
          border-right: var(--hi-px-1) solid var(--hi-clr--gray-3);
          transition: width var(--hi-animation--speed) var(--hi-animation--accel);
          background: white;
        }

        [x-style='nav-link-item-wrapper'] {
          position: relative;
          display: flex;
          align-items: center;
          z-index: var(--hi-depth--content);
        }

        [x-style='tertiary-nav-container'][opened] {
          width: var(--hi-screen--nav-tertiary-opened);
        }

        [x-style='tertiary-navigation-items-container'] {
          overflow-y: auto;
          overflow-x: hidden;
          position: absolute;
          top: 0;
          left: 0;
          right: 0;
          bottom: var(--bottom-button-height);
          scrollbar-width: thin;
        }

        [x-style='tertiary-navigation-items-container']::-webkit-scrollbar {
          width: var(--hi-px-4);
          background: var(--hi-clr--gray-2);
        }

        [x-style='tertiary-navigation-items-container']::-webkit-scrollbar-thumb {
          background: var(--hi-clr--gray-7);
        }

        [x-style='tertiary-nav-container'][always-opened]:not(:has(#navToggleBtn))
          [x-style='tertiary-navigation-items-container'] {
          bottom: 0;
        }

        @media only screen and (min-width: 48rem) {
          [x-style='tertiary-nav-container'] {
            width: 4rem;
          }
        }

        @media only screen and (max-width: 40rem) {
          [x-style='tertiary-nav-container'] {
            opacity: 0;
            width: 0;
            visibility: hidden;
            transition: width var(--hi-animation--speed) var(--hi-animation--accel);
          }

          [x-style='tertiary-nav-container'][opened] {
            right: 0;
            width: 100%;
            opacity: 1;
            visibility: visible;
          }

          [x-style='nav-menu-control-mobile']:not([opened]) {
            display: flex;
          }

          [x-style='nav-menu-control-mobile'][opened] {
            visibility: hidden;
          }
        }
      </style>
    `;
  }
  createRenderRoot() {
    return this;
  }
  __navToggleBtn__onTriggerEnded() {
    this.toggle();
  }
  __navMenuControlMobile__onTriggerEnded() {
    this.toggle();
  }
  updateLinkGroupActivePath() {
    const linkGroup = document.getElementById("hi-nav-tertiary-link-group");
    if (linkGroup) {
      linkGroup.setActive();
    }
  }
  toggle() {
    if (this.opened) {
      this.close();
    } else {
      this.open();
    }
    return this.opened;
  }
  close() {
    return this.opened = false;
  }
  open() {
    return this.opened = true;
  }
  __navigationItemSelected(e) {
    const isNavigationItemDisabled = this.navigationActions[e.detail.value]["disabled"];
    if (isNavigationItemDisabled) {
      return;
    }
    this.__setActiveElement(e);
    this.__closeTertiaryNavigation();
    this.__dispatchNavigationItemEvent(e);
  }
  __setActiveElement(e) {
    const selectedNavigationItem = this.navigationActions[e.detail.value];
    if (selectedNavigationItem) {
      this.selectedNavigationItem = selectedNavigationItem;
    }
  }
  __closeTertiaryNavigation() {
    const isAllowedToClose = !this.alwaysOpened;
    const isOverrideAlwaysOpenedInMobile = this.alwaysOpened && this.screenQuery.isMobile();
    if (this.opened && (isAllowedToClose || isOverrideAlwaysOpenedInMobile)) {
      this.close();
    }
  }
  __dispatchNavigationItemEvent(e) {
    this._dispatchProcess(
      "navigation-action",
      Object.assign(e.detail, {
        value: this.navigationActions[e.detail.value]
      })
    );
  }
}
export default HIP.factory.element(TertiaryNavigationElement);
