
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorContainer } from '../../../eip/element/hi-behavior-container/hi-behavior-container.js';
import './hi-notification.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class NotificationGroupElement extends BehaviorContainer(BehaviorElement(AsHIC(PolymerElement))) {
  static get is() {
    return "hi-notification-group";
  }
  static get template() {
    return html`
      <style>
        :host {
          --hi-notification--position-top: var(
            --hi-screen--top,
            calc(var(--hi-screen--nav-primary) + var(--hi-screen--nav-secondary))
          );
          --hi-notification--position-z-index: var(--hi-depth--above-all);
          --hi-notification--position-bottom: var(--hi-px-16);
        }

        :host {
          display: block;
          position: fixed;
          z-index: var(--hi-notification--position-z-index);
          right: var(--hi-px-16);
        }

        :host([position='top']) {
          top: var(--hi-notification--position-top);
          margin-top: var(--hi-px-16);
        }

        :host([position='bottom']) {
          bottom: var(--hi-notification--position-bottom);
        }

        @media screen and (max-width: 40rem) {
          :host {
            width: 100%;
            left: 0;
            right: 0;
            padding: 0 var(--hi-px-16);
          }
        }
      </style>
      <slot></slot>
    `;
  }
  static get properties() {
    return {
      messages: {
        type: Array,
        value: () => []
      },
      defaultDelay: {
        type: Number,
        observer: "__defaultDelayChanged",
        value: 21e3
      },
      position: {
        type: String,
        reflectToAttribute: true,
        value: "bottom"
      }
    };
  }
  static get api() {
    return ["notify"];
  }
  static get autoBinding() {
    return {
      signal: "notify-ended",
      field: "notify"
    };
  }
  __defaultDelayChanged(defaultDelay) {
    if (defaultDelay < 21e3) {
      defaultDelay = 21e3;
    }
  }
  __createNotification(requestObj) {
    return new Promise((resolve) => {
      const notificationEl = document.createElement("hi-notification");
      notificationEl.message = requestObj;
      notificationEl.position = this.position;
      if (this.position === "bottom") {
        this.prepend(notificationEl);
      } else {
        this.append(notificationEl);
      }
      resolve({
        hasNotified: true
      });
    });
  }
  notify(packet) {
    if (f1.is.empty(packet)) {
      return Promise.resolve({
        hasNotified: false
      });
    }
    const prepareMsg = typeof packet === "string" ? { message: packet } : packet;
    if (!prepareMsg.delay) {
      prepareMsg.delay = this.defaultDelay;
    }
    if (f1.is.string(prepareMsg.position)) {
      this.position = prepareMsg.position;
    }
    if (!prepareMsg.details) {
      prepareMsg.details = "";
    }
    const duplicateMsg = this.__getDuplicateIndex(prepareMsg);
    if (duplicateMsg > -1) {
      const childEls = document.querySelectorAll("hi-notification");
      if (childEls.length > 1 && duplicateMsg > 0) {
        childEls[0].before(childEls[duplicateMsg]);
      }
      return Promise.resolve({
        hasNotified: false
      });
    }
    return this.__createNotification(prepareMsg);
  }
  static get instance() {
    return HIP.fn.instance(NotificationGroupElement);
  }
  static get instanceBottom() {
    return this.instance;
  }
  __getDuplicateIndex(curr) {
    const notifications = f1.to.array(document.querySelectorAll("hi-notification")) || [];
    const { message, title, details } = curr;
    const duplicateIndex = notifications.findIndex(({ message: msg }) => {
      const doesMessageMatch = message === msg.message;
      const doesTitleAndDetailsMatch = !!title && title === msg.title && details === msg.details;
      return doesMessageMatch || doesTitleAndDetailsMatch;
    });
    if (duplicateIndex > -1) {
      const identicalMsg = notifications[duplicateIndex];
      identicalMsg.duplicateCount += 1;
      identicalMsg.countdown = parseInt((curr.delay || 21e3) / 1e3);
    }
    return duplicateIndex;
  }
}
HIP.fn.notify = function(packet) {
  if (packet && packet["position"] === "top") {
    if (!NotificationGroupElement.instanceTop) {
      NotificationGroupElement.instanceTop = new NotificationGroupElement();
      document.body.append(NotificationGroupElement.instanceTop);
    }
    return NotificationGroupElement.instanceTop.notify(packet);
  }
  return NotificationGroupElement.instance.notify(packet);
};
HIP.factory.element(NotificationGroupElement);
