
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import '../../../xip/element/xi-a11y-announcer/xi-a11y-announcer.js';
import '../../../eip/element/hi-button/hi-close.js';
import '../../../eip/element/hi-badge/hi-badge.js';
import '../../../eip/element/hi-icon/hi-icon.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class NotificationElement extends BehaviorElement(AsHIC(PolymerElement)) {
  static get is() {
    return "hi-notification";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          will-change: transform, opacity;
          display: flex;
          align-items: flex-start;
          border-radius: var(--hi-shape--radius-2);
          width: calc(22.5 * var(--hi-px-16)); /* 360px */
          background-color: var(--hi-clr--white);
          border: var(--hi-px-1) solid var(--hi-clr--theme-3);
          border-top-width: var(--hi-px-4);
          box-shadow: 0 0 0 1px transparent, 0 1px 2px rgba(0, 0, 0, 0.1), 0 2px 6px 0 rgba(0, 0, 0, 0.2);
          margin: var(--hi-px-16) 0;
        }

        :host([position='top']) {
          animation: topEnter var(--hi-animation--speed) cubic-bezier(0.39, 0.575, 0.565, 1) both;
        }

        :host(.animate-down[position='top']) {
          animation: topExit var(--hi-animation--speed) cubic-bezier(0.39, 0.575, 0.565, 1) both;
        }

        :host([position='bottom']) {
          animation: bottomEnter var(--hi-animation--speed) cubic-bezier(0.39, 0.575, 0.565, 1) both;
        }

        :host(.animate-down[position='bottom']) {
          animation: bottomExit var(--hi-animation--speed) cubic-bezier(0.39, 0.575, 0.565, 1) both;
        }

        @media (prefers-reduced-motion) {
          :host {
            animation: none;
          }
        }

        @media screen and (max-width: 40rem) {
          :host {
            width: 100%;
          }
        }

        #body {
          flex-grow: 1;
        }

        #header {
          padding: var(--hi-px-16) var(--hi-px-16) var(--hi-px-16) 0;
          display: flex;
          align-items: center;
          line-height: var(--hi-font-lh--medium);
          letter-spacing: 0.02em;
        }

        #title {
          font-weight: var(--hi-font--weight-heavy);
          margin-right: auto;
        }

        #title,
        #title a {
          color: var(--hi-clr--text);
          font-size: var(--hi-px-14);
        }

        #closeButton {
          color: var(--hi-clr--text);
          margin-right: calc(-1 * var(--hi-px-8));
          min-width: var(--hi-close--size);
        }

        #count {
          display: block;
          position: absolute;
          top: 0;
          left: 0;
          transform: translate(-50%, -50%);
        }

        #status {
          margin: auto var(--hi-px-16);
        }

        #themeIcon {
          fill: var(--hi-clr--theme-3);
        }

        #details {
          padding: var(--hi-px-8) var(--hi-px-16) var(--hi-px-16) 0;
          display: block;
          overflow-y: auto;
          max-height: calc(var(--hi-px-32) * 5);
          margin-top: calc(-1 * var(--hi-px-8));
        }

        #details:empty {
          display: none;
        }

        @keyframes bottomExit {
          0% {
            transform: translateY(0);
            opacity: 1;
          }
          100% {
            transform: translateY(25px);
            opacity: 0;
          }
        }

        @keyframes topExit {
          0% {
            transform: translateY(0);
            opacity: 1;
          }
          100% {
            transform: translateY(-25px);
            opacity: 0;
          }
        }

        @keyframes bottomEnter {
          0% {
            transform: translateY(25px);
            opacity: 0;
          }
          100% {
            transform: translateY(0);
            opacity: 1;
          }
        }

        @keyframes topEnter {
          0% {
            transform: translateY(-25px);
            opacity: 0;
          }
          100% {
            transform: translateY(0);
            opacity: 1;
          }
        }
      </style>
      <div id="status">
        <hi-icon
          id="themeIcon"
          kind$="[[ __iconKind ]]"
          aria-label$="[[message.status]]"
          type="material"
          size="large"
          theme="[[theme]]"
          type="material"
        >
        </hi-icon>
      </div>
      <div id="body">
        <header id="header">
          <div id="title"></div>
          <hi-badge id="count" aria-hidden="true" theme="[[theme]]">[[ duplicateCount ]]</hi-badge>
          <hi-close id="closeButton" on-trigger-ended="close" size="small"></hi-close>
        </header>
        <p id="details"></p>
      </div>
    `;
  }
  static get properties() {
    return {
      message: {
        type: Object,
        reflectToAttribute: true,
        observer: "__messageChanged"
      },
      countdown: {
        type: Number,
        observer: "__countdownChanged",
        reflectToAttribute: true,
        notify: true
      },
      position: {
        type: String,
        reflectToAttribute: true
      },
      __announceCountdown: {
        type: Number
      },
      __iconKind: {
        type: String,
        computed: "__iconKindCompute(theme)"
      },
      duplicateCount: {
        type: Number,
        value: 1,
        notify: true
      }
    };
  }
  static get api() {
    return ["close"];
  }
  static get autoBinding() {
    return {
      signal: "close-ended",
      field: "close"
    };
  }
  static get observers() {
    return ["__duplicateCountWatcher(duplicateCount)"];
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "alert");
    this.setAttribute("aria-relevant", "additions");
    this.__computeAriaLabel();
  }
  __messageChanged(message) {
    message.delay = message.delay || 21e3;
    this.countdown = parseInt(message.delay / 1e3);
    this.theme = message.status || "primary";
    message.status = this.theme || message.status || "info";
    message.message = message.message || "";
    this.$.details.innerHTML = message.details != null ? message.details : "";
    this.$.title.innerHTML = message.title != null ? message.title : message.message;
  }
  __countdownChanged(countdown) {
    if (countdown > 0) {
      setTimeout(() => {
        this.countdown -= 1;
        if (this.countdown % 7 === 0) {
          this.__announceCountdown = this.countdown;
          HIP.fn.announce(`${this.__announceCountdown} until close.`);
        }
      }, 1e3);
    }
    if (countdown <= 0) {
      HIP.fn.announce(`notification dismissed`);
      this.close();
    }
  }
  __iconKindCompute(theme) {
    const themeToIcon = {
      info: "info",
      warning: "warning",
      error: "stop",
      success: "check_circle"
    };
    return themeToIcon[theme] != null ? themeToIcon[theme] : "info";
  }
  __duplicateCountWatcher() {
    const { title, message, details } = f1.safe({}, this.message);
    const announcement = `${title || message} ${details}`;
    const hasDuplicates = this.duplicateCount > 1;
    this.$.count.style.display = hasDuplicates ? "block" : "none";
    if (hasDuplicates) {
      HIP.fn.announce(announcement);
    }
  }
  close() {
    this.removeFromDOM().then(() => {
      this._dispatchNotification("notification-done", {
        value: this.message
      });
    });
  }
  removeFromDOM() {
    return new Promise((resolve) => {
      this.classList.add("animate-down");
      const fallBackTimer = setTimeout(() => {
        this.remove();
        resolve();
      }, 500);
      this.addEventListener("animationend", () => {
        this.remove();
        clearTimeout(fallBackTimer);
        resolve();
      });
    });
  }
  __computeAriaLabel() {
    const ariaLabel = this.message.status ? `Close ${this.message.status} Notification` : `Close Primary Notification`;
    this.$.closeButton.setAttribute("aria-label", ariaLabel);
  }
}
HIP.factory.element(NotificationElement);
