
import { microTask, Debouncer, PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import '../../../eip/element/hi-icon/hi-icon.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class PaginationElement extends BehaviorElement(AsHIC(PolymerElement)) {
  static get is() {
    return "hi-pagination";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          display: block;
          margin: 0;
          --pagination--height: calc(var(--hi-el--height) - var(--hi-px-6));
        }

        [x-style='page-list'] {
          display: inline-flex;
          flex-direction: row;
          list-style: none;
          gap: var(--hi-px-2);
          margin: 0;
          padding: var(--hi-px-2) 0;
          border-radius: var(--hi-shape--radius-2);
          border: var(--hi-px-1) solid var(--hi-clr--theme-1);
        }

        [x-style='page-list']:hover {
          border-color: var(--hi-clr--theme-3);
          box-shadow: var(--hi-clr--halo-1);
        }

        :host([disabled]) [x-style='page-list']:hover {
          box-shadow: none !important;
        }

        li button {
          display: block;
          height: var(--pagination--height);
          min-width: calc(var(--pagination--height) + var(--hi-px-1));
          line-height: var(--pagination--height);
          border: var(--hi-px-1) solid transparent;
          border-radius: var(--hi-shape--radius-2);
          background-color: var(--hi-clr--white);
          color: var(--hi-clr--theme-5);
          appearance: button;
          text-align: center;
          cursor: pointer;
        }

        li button:not([disabled]):hover > hi-icon {
          cursor: pointer;
        }

        li:first-of-type button {
          width: calc(var(--pagination--height) + var(--hi-px-2));
        }

        li:last-of-type button {
          width: calc(var(--pagination--height) + var(--hi-px-2));
        }

        [x-style='page-number'] > span {
          display: block;
          margin-top: calc(-1 * var(--hi-px-2));
        }

        button:not([disabled]):hover {
          border: var(--hi-px-1) solid var(--hi-clr--theme-5);
          background-color: var(--hi-clr--theme-1);
          color: var(--hi-clr--theme-5);
          text-shadow: var(--hi-px-0) var(--hi-px-0) var(--hi-px-1) var(--hi-clr--theme-5);
        }

        button#firstPage:not([disabled]):hover > hi-icon,
        button#previousPage:not([disabled]):hover > hi-icon,
        button#nextPage:not([disabled]):hover > hi-icon,
        button#lastPage:not([disabled]):hover > hi-icon {
          --hi-icon--color: var(--hi-clr--theme-5);
        }

        button#firstPage:active > hi-icon,
        button#previousPage:active > hi-icon,
        button#nextPage:active > hi-icon,
        button#lastPage:active > hi-icon {
          --hi-icon--color: var(--hi-clr--theme-3);
        }

        button:focus {
          position: relative;
          outline: var(--hi-outline--width) var(--hi-outline--line) var(--hi-clr--outline);
          outline-offset: var(--hi-outline--offset);
          z-index: 100;
        }

        [hidden] {
          display: none;
        }

        button:disabled,
        button[disabled],
        :host([disabled]) button,
        :host([disabled]) button > hi-icon {
          box-shadow: none !important;
          color: var(--hi-clr--disabled-text) !important;
          -webkit-filter: grayscale(100%);
          -moz-filter: grayscale(100%);
          filter: url(desaturate.svg#greyscale);
          filter: gray;
          filter: grayscale(100%);
          -moz-user-focus: ignore !important;
          pointer-events: none !important;
          cursor: not-allowed !important;
          user-select: none !important;
        }

        button:disabled[selected],
        button[disabled][selected] {
          background: var(--hi-clr--gray-2);
        }

        button[selected]:hover,
        button[selected] {
          box-shadow: none;
          background: var(--hi-clr--theme-3);
          color: var(--hi-clr--white);
        }

        @media only screen and (max-width: 40rem) {
          :host {
            width: 100%;
          }

          [x-style='page-list'] {
            display: flex;
            justify-content: space-between;
          }
        }
      </style>
      <ul x-style="page-list">
        <li x-style="first">
          <button id="firstPage" data-id="first" on-click="_firstPage__onClick" disabled$="[[ disabled ]]">
            <hi-icon kind="arrow_left_with_line" theme$="[[ theme ]]" size="small"></hi-icon>
            <span mixin="show-ax"> Go to First Page </span>
          </button>
        </li>
        <li x-style="previous">
          <button id="previousPage" data-id="previous" on-click="_previousPage__onClick" disabled$="[[ disabled ]]">
            <hi-icon kind="chevron_left" theme$="[[ theme ]]" size="small"></hi-icon>
            <span mixin="show-ax"> Go to Previous Page </span>
          </button>
        </li>
        <template is="dom-repeat" items="[[ pages ]]" as="page">
          <li>
            <button
              data-id$="[[ page ]]"
              on-click="_pageNumber__onClick"
              x-style="page-number"
              selected$="[[ _isSelectedState(page, pageCurrent, input, steps) ]]"
              aria-current$="[[ _isSelectedState(page, pageCurrent, input, steps) ]]"
              aria-label$="Go to {{ labelContext }} {{ page }}"
              disabled$="[[ disabled ]]"
            >
              <span data-id$="[[ page ]]">[[ page ]]</span>
            </button>
          </li>
        </template>
        <li x-style="next">
          <button id="nextPage" data-id="next" on-click="_nextPage__onClick" disabled$="[[ disabled ]]">
            <hi-icon kind="chevron_right" theme$="[[ theme ]]" size="small"></hi-icon>
            <span mixin="show-ax"> Go to Next Page </span>
          </button>
        </li>
        <li x-style="last">
          <button id="lastPage" data-id="last" on-click="_lastPage__onClick" disabled$="[[ disabled ]]">
            <hi-icon kind="arrow_right_with_line" theme$="[[ theme ]]" size="small"></hi-icon>
            <span mixin="show-ax"> Go to Last Page </span>
          </button>
        </li>
      </ul>
    `;
  }
  static get properties() {
    return {
      kind: {
        type: String,
        notify: true,
        value: "determinate"
      },
      pageIndex: {
        type: Number,
        reflectToAttribute: true,
        notify: true
      },
      pageCurrent: {
        type: Number,
        computed: "__pageCurrentCompute(pageIndex, pageCurrentOverride)"
      },
      pageCurrentOverride: {
        type: Number
      },
      max: {
        type: Number
      },
      pageSize: {
        type: Number
      },
      pages: {
        type: Array
      },
      rangeSize: {
        type: Number,
        value: 5
      },
      labelContext: {
        type: String,
        value: "Page"
      },
      input: {
        type: Object,
        reflectToAttribute: true,
        notify: true
      },
      steps: {
        type: Object,
        reflectToAttribute: true,
        notify: true
      }
    };
  }
  static get api() {
    return [
      "goToPrevious",
      "goToFirst",
      "goToLast",
      "goToNext",
      "goToPage",
      "jumpTo",
      "toggleDisabledDeterminate",
      "toggleDisabledIndeterminate"
    ];
  }
  static get valueEvent() {
    return "pageIndex";
  }
  static get autoBinding() {
    return {
      signal: "page-index-changed",
      field: "pageIndex"
    };
  }
  static get observers() {
    return ["__pagesWatcher(kind, pageSize, max, pageCurrent, input, steps)"];
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "navigation");
  }
  __pagesWatcher(kind, pageSize, max, pageCurrent, input, steps) {
    if (kind !== "determinate") {
      return;
    }
    const { pageCount, pages } = this.hasSteps() ? this.__pagesFromLinks(this.readSteps()) : pageSize && max ? this.__pagesFromRange(pageSize, max, pageCurrent) : { pageCount: 0, pages: [] };
    this._pageCount = pageCount;
    this.set("pages", pages);
    this.toggleDisabledDeterminate(pageCurrent, pageCount, input);
  }
  readSteps() {
    return this.steps || this.input?.links || {};
  }
  hasSteps() {
    return !!(this.steps || this.input?.links);
  }
  __pagesFromLinks(links) {
    const pages = Object.keys(links).filter(f1.to.integer).map(f1.to.integer);
    return {
      pageCount: pages.length,
      pages
    };
  }
  __pagesFromRange(pageSize, max, pageCurrent) {
    const pageCount = Math.ceil(max / (pageSize || 1));
    const range = this._computePageRange(pageCurrent, pageCount);
    const pages = [...Array(range.high - (range.low - 1))].map((_, index) => range.low + index);
    return {
      pageCount,
      pages
    };
  }
  _computePageRange(pageCurrent, pageCount) {
    const roundMean = Math.round(this.rangeSize / 2);
    const floorMean = Math.floor(this.rangeSize / 2);
    if (pageCount < this.rangeSize) {
      return {
        high: pageCount,
        low: 1
      };
    } else if (pageCurrent < roundMean) {
      return {
        high: this.rangeSize,
        low: 1
      };
    } else if (pageCurrent >= roundMean && pageCurrent <= pageCount - floorMean) {
      return {
        high: pageCurrent + floorMean,
        low: pageCurrent - floorMean
      };
    } else if (pageCurrent > pageCount - floorMean) {
      return {
        high: pageCount,
        low: pageCount - (this.rangeSize - 1)
      };
    } else {
      return {
        high: 0,
        low: 0
      };
    }
  }
  __pageCurrentCompute(pageIndex, pageCurrentOverride) {
    return f1.is.number(pageCurrentOverride) ? pageCurrentOverride : (pageIndex || 0) + 1;
  }
  _firstPage__onClick() {
    this.goToFirst();
  }
  _lastPage__onClick() {
    this.goToLast();
  }
  _nextPage__onClick() {
    this.goToNext();
  }
  _previousPage__onClick() {
    this.goToPrevious();
  }
  goToFirst() {
    if (this.isDeterminate()) {
      this.pageIndex = 0;
    } else {
      this.jumpTo("first");
    }
    this.__triggerEvent_onPaginate("first");
    return this.pageIndex;
  }
  goToLast() {
    if (this.isDeterminate()) {
      this.pageIndex = this._pageCount - 1;
    } else {
      this.jumpTo("last");
    }
    this.__triggerEvent_onPaginate("last");
    return this.pageIndex;
  }
  goToNext() {
    if (this.isDeterminate()) {
      const pageCurrentIndex = this.pageCurrentIndex();
      if (pageCurrentIndex < this._pageCount) {
        this.pageIndex = pageCurrentIndex;
      }
    } else {
      this.jumpTo("next");
    }
    this.__triggerEvent_onPaginate("next");
    return this.pageIndex;
  }
  goToPrevious() {
    if (this.isDeterminate()) {
      const pageCurrentIndex = this.pageCurrentIndex();
      if (pageCurrentIndex > 1) {
        this.pageIndex = pageCurrentIndex - 2;
      }
    } else {
      this.jumpTo("previous");
    }
    this.__triggerEvent_onPaginate("previous");
    return this.pageIndex;
  }
  isDeterminate() {
    return this.kind == "determinate" && !this.hasSteps();
  }
  isIndeterminate() {
    return !this.isDeterminate();
  }
  goToPage(pageCurrent) {
    if (pageCurrent === this.pageCurrent) {
      return;
    }
    this.pageIndex = pageCurrent - 1;
    this.__setPageFocusInMiddle();
    return this.pageIndex;
  }
  pageCurrentIndex() {
    return this.pageCurrent == null ? 1 : this.pageCurrent;
  }
  _pageNumber__onClick(e) {
    const index = f1.to.integer(e.target.getAttribute("data-id"));
    if (!this.hasSteps()) {
      e.preventDefault();
      this.goToPage(index);
    } else {
      e.preventDefault();
      this.jumpTo(index);
    }
    this.__triggerEvent_onPaginate(f1.to.string(index));
  }
  __triggerEvent_onPaginate(value) {
    this._dispatchProcess("paginate", {
      steps: this.steps,
      pageIndex: this.pageIndex,
      pageSize: this.pageSize,
      value
    });
  }
  jumpTo(position) {
    return position;
  }
  toggleDisabledDeterminate(pageCurrent, pageCount, input) {
    if (this.kind !== "determinate") {
      return;
    }
    const controlledByAPI = this.hasSteps();
    if (!controlledByAPI && pageCurrent) {
      this.__disabledDebounce1 = Debouncer.debounce(this.__disabledDebounce1, microTask, () => {
        dom.flipAttr(this.$.firstPage, "disabled", pageCurrent <= 1, "disabled");
        dom.flipAttr(this.$.previousPage, "disabled", pageCurrent <= 1, "disabled");
        dom.flipAttr(this.$.nextPage, "disabled", pageCurrent >= pageCount, "disabled");
        dom.flipAttr(this.$.lastPage, "disabled", pageCurrent >= pageCount, "disabled");
      });
    } else if (controlledByAPI) {
      const links = this.readSteps();
      this.__disabledDebounce1 = Debouncer.debounce(this.__disabledDebounce1, microTask, () => {
        dom.flipAttr(this.$.firstPage, "disabled", links.first == null, "disabled");
        dom.flipAttr(this.$.previousPage, "disabled", links.previous == null, "disabled");
        dom.flipAttr(this.$.nextPage, "disabled", links.next == null, "disabled");
        dom.flipAttr(this.$.lastPage, "disabled", links.last == null, "disabled");
      });
    }
  }
  toggleDisabledIndeterminate(controlFlags) {
    if (this.kind !== "indeterminate") {
      return;
    }
    this.__disabledDebounce2 = Debouncer.debounce(this.__disabledDebounce2, microTask, () => {
      ["first", "previous", "next", "last"].forEach((controlName) => {
        dom.flipAttr(this.$[`${controlName}Page`], "disabled", !controlFlags[controlName], "disabled");
      });
    });
  }
  __setPageFocusInMiddle() {
    const indexEnds = 4;
    const middleButtonIndex = 4;
    const pageCurrent = this.pageCurrent;
    const isInMiddleOfPageIndices = pageCurrent <= this._pageCount - indexEnds && pageCurrent > indexEnds;
    if (isInMiddleOfPageIndices) {
      const buttonList = this.shadowRoot.querySelectorAll("button");
      const middleButton = buttonList[middleButtonIndex];
      if (middleButton) {
        middleButton.focus();
      }
    }
  }
  _isSelectedState(page, pageCurrent, input, steps) {
    if (!this.hasSteps() && page && pageCurrent) {
      return page === pageCurrent;
    } else if (input && page) {
      const inputCurrentPage = input.current_page;
      return page === inputCurrentPage;
    } else if (this.hasSteps()) {
      return page === this.pageCurrent;
    } else {
      return;
    }
  }
  toString() {
    const curPageLastIndex = (this.pageIndex + 1) * this.pageSize;
    const upperLimitText = f1.min(curPageLastIndex, this.max);
    return `${this.pageIndex * this.pageSize + 1} to ${upperLimitText} of ${this.max}`;
  }
}
HIP.factory.element(PaginationElement);
