
import { afterNextRender } from '../../../hip/library/polymer/polymer.js';
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import { AsHIC, HIP } from '../../../eip/element/hi-component/hi-component.js';
import './hi-tickmarks.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class ProgressElement extends BehaviorElement(AsHIC(PolymerElement)) {
  static get is() {
    return "hi-progress";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          --hi-progress--height: var(--hi-px-36);
          --hi-progress--label-height: var(--hi-px-24);
          --hi-progress--border-radius: var(--hi-px-1);
          --hi-progress--container-height: var(--hi-px-16);
        }

        :host {
          display: flex;
          flex-direction: column;
          justify-content: center;
          align-self: center;
          position: relative;
          width: 100%;
          height: var(--hi-progress--container-height);
          border: var(--hi-px-1) solid var(--hi-clr--gray-5);
          box-shadow: var(--hi-clr--shadow-inset-3);
          background-color: var(--hi-clr--gray-1);
          box-sizing: border-box;
          border-radius: var(--hi-progress--border-radius);
          -webkit-print-color-adjust: exact;
          color-adjust: exact;
        }

        #progressBarContainer {
          overflow: hidden;
          height: var(--hi-progress--container-height);
        }

        #progressBar {
          border-radius: var(--hi-progress--border-radius);
          background-color: var(--hi-clr--theme-3);
          transition-property: transform;
          transition-duration: var(--hi-animation--speed);
          will-change: transform;
          backface-visibility: hidden;
          position: absolute;
          box-sizing: border-box;
          width: 100%;
          height: 100%;
          z-index: 1;
          transform-origin: 0 0;
          transform: scaleX(0);
        }

        :host(:not([tickmarks])) hi-tickmarks {
          display: none;
        }

        :host([aesthetic='round']) {
          border-radius: var(--hi-shape--pill);
        }

        :host([aesthetic='round']) #progressBar {
          border-radius: var(--hi-shape--pill);
          transform-origin: 0 0;
        }
      </style>
      <div id="progressBarContainer">
        <div id="progressBar"></div>
      </div>
      <hi-tickmarks
        value="[[ tickmarks ]]"
        theme="[[ theme ]]"
        size="[[ size ]]"
        min="[[ min ]]"
        max="[[ max ]]"
        visible-step-labels$="[[ visibleStepLabels ]]"
      >
      </hi-tickmarks>
    `;
  }
  static get properties() {
    return {
      value: {
        type: Number,
        reflectToAttribute: true,
        notify: true
      },
      min: {
        type: Number,
        reflectToAttribute: true,
        value: 0
      },
      max: {
        type: Number,
        reflectToAttribute: true,
        value: 100
      },
      __ratio: {
        type: Number,
        computed: "__ratioCompute(value, min, max)"
      },
      tickmarks: {
        type: Array
      },
      visibleStepLabels: {
        type: Boolean
      }
    };
  }
  static get observers() {
    return ["__dimensionObserver(value, min, max)"];
  }
  static get autoBinding() {
    return {
      signal: "value-changed",
      field: "value"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "progressbar");
    this.setAttribute("aria-label", "progress indicator");
    afterNextRender(this, function() {
      this.__calculateProgressBar();
    });
  }
  __dimensionObserver(_value, _min, _max) {
    this.min = _min = this.__fixMinBound(_value, _min, _max);
    this.value = _value = this.__clipValue(_value, _min, _max);
    if (!f1.is.nil(_value)) {
      this.setAttribute("aria-valuenow", _value);
    }
    this.setAttribute("aria-valuemax", _max);
    this.setAttribute("aria-valuemin", _min);
    this.__calculateProgressBar();
  }
  __fixMinBound(_value, _min, _max) {
    if (_min > _max) {
      _min = _max - 1;
    }
    return _min;
  }
  __clipValue(_value, _min, _max) {
    if (_value < _min) {
      _value = _min;
    } else if (_value > _max) {
      _value = _max;
    }
    return _value;
  }
  __ratioCompute(_value, _min, _max) {
    const ratio = (_value - _min) / (_max - _min);
    return ratio > 1 && !isNaN(ratio) ? 1 : ratio;
  }
  __calculateProgressBar() {
    this.$.progressBar.style.transform = `scaleX(${this.__ratio})`;
  }
}
HIP.factory.element(ProgressElement);
