
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import { AsHIC, HIP } from '../../../eip/element/hi-component/hi-component.js';
export class TickmarksElement extends BehaviorElement(AsHIC(PolymerElement)) {
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          position: relative;
          height: var(--hi-px-16);
        }

        :host(:not([visible-step-labels])) [x-style='step-label'] {
          display: none;
        }

        [x-style='step'] {
          position: absolute;
          top: 0;
        }

        [x-style='step-label'] {
          position: absolute;
          left: 0;
          top: var(--hi-px-8);
          transform: translateX(-40%);
          font-size: var(--hi-px-12);
          color: var(--hi-clr--text);
        }

        [x-style='step-line'] {
          position: absolute;
          height: var(--hi-px-16);
          width: var(--hi-px-1);
          background-color: var(--hi-clr--theme-text-3);
          top: calc(-1 * var(--hi-px-8));
          border-bottom: 0;
          border-top: 0;
          z-index: 1;
        }
      </style>
      <template is="dom-repeat" items="[[ value ]]" as="percentage">
        <div x-style="step" style$="left: [[ __indexToLocation(index, value) ]]%">
          <span x-style="step-label" class="text"> [[ __format(percentage) ]]% </span>
          <div x-style="step-line"></div>
        </div>
      </template>
    `;
  }
  static get is() {
    return "hi-tickmarks";
  }
  static get properties() {
    return {
      value: {
        type: Array,
        defaultValue: function() {
          return [];
        }
      },
      min: {
        type: Number,
        value: 0
      },
      max: {
        type: Number,
        value: 100
      }
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "status");
  }
  __indexToLocation(index, list) {
    return list[index] / (this.max - this.min) * 100;
  }
  __format(percentage) {
    return parseInt(percentage);
  }
}
HIP.factory.element(TickmarksElement);
