
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import { HIP, AsLitHIC } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorContainer } from '../../../eip/element/hi-behavior-container/hi-behavior-container.js';
import { BehaviorCheckedSetGroup } from '../../../eip/element/hi-behavior-group/hi-behavior-checked-set-group.js';
import { BehaviorInput } from '../../../eip/element/hi-behavior-input/hi-behavior-input.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export const styles = css`
  :host {
    --hi-radio--box-size: calc(var(--hi-px-16) + var(--hi-px-4));
    --hi-radio-group--padding: var(--hi-px-12);

    display: block;
  }

  :host(:focus) {
    outline: none;
  }

  #container ::slotted([x-hip]),
  :host([aesthetic='vertical']) #container ::slotted(*) {
    margin-bottom: var(--hi-px-4);
    margin-top: var(--hi-px-4);
    vertical-align: top;
  }

  #container ::slotted(hi-label) {
    width: calc(100% - var(--hi-radio--box-size) - var(--hi-radio-group--padding));
    line-height: var(--hi-radio--box-size);
  }

  #container ::slotted(hi-radio) {
    width: var(--hi-radio--box-size);
  }

  :host([aesthetic='horizontal']) #container,
  :host([aesthetic='inline']) #container {
    display: flex;
  }

  :host([aesthetic='horizontal']) #container ::slotted(hi-label),
  :host([aesthetic='inline']) #container ::slotted(hi-label) {
    width: auto;
    margin-right: var(--hi-px-16);
  }

  :host([disabled]),
  :host([readonly]) {
    pointer-events: all !important;
    opacity: 1;
  }

  :host([disabled]) {
    cursor: not-allowed;
  }

  :host([disabled]) #container ::slotted(hi-radio) {
    --hi-radio--background-color: var(--hi-clr--disabled);
    --hi-radio--border-color: var(--hi-clr--disabled);
    --hi-clr--theme-3: var(--hi-clr--disabled);
  }
`;
export class RadioGroupElement extends BehaviorCheckedSetGroup(
  BehaviorContainer(BehaviorInput(BehaviorElement(AsLitHIC(LitElement))))
) {
  constructor() {
    super();
    this.groupName = "radio";
    this.aesthetic = "vertical";
  }
  static get is() {
    return "hi-radio-group";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      value: {
        type: String,
        reflect: true,
        notify: true
      },
      selectedIndex: {
        type: Object,
        reflect: true,
        notify: true,
        attribute: "selected-index"
      },
      aesthetic: {
        type: String,
        reflect: true
      }
    };
  }
  static get valueEvent() {
    return "value-changed";
  }
  static get autoBinding() {
    return {
      signal: "selected-index-changed",
      field: "selectedIndex"
    };
  }
  render() {
    return html`
      <div id="container">
        <slot id="items"></slot>
      </div>
    `;
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "radiogroup");
  }
  update(props) {
    super.update(props);
    if (props.has("value")) {
      this.__valueChanged(this.value);
    }
    if (props.has("selectedIndex")) {
      this.__selectedIndexChanged(this.selectedIndex);
    }
  }
  _groupChanged(group, groupOld) {
    super._groupChanged(group, groupOld);
    this.__reflectOnValueChange(this.value);
    this.__reconcileNamedChildren();
  }
  _onSlotChanged() {
    this._group__slotChange();
    this.__reflectOnValueChange(this.value);
    this.__reconcileNamedChildren();
  }
  __reconcileNamedChildren() {
    let selectedIndex = this.selectedIndex;
    let anyFocusable = false;
    let anyChecked = false;
    const group = this._group__groupSet();
    group.forEach((child, index) => {
      if (child.checked) {
        selectedIndex = this.selectedIndex = index;
      }
      if (typeof selectedIndex === "number") {
        child.checked = index === selectedIndex;
      }
      anyChecked = anyChecked || child.checked;
      anyFocusable = anyFocusable || child.getAttribute("tabindex") === "0";
      if (child.checked) {
        this.__proxyValueToGroup(child);
      }
    });
    if ((!anyFocusable || !anyChecked) && group.length >= 1) {
      group[0].setAttribute("checked", "checked");
      group[0].setAttribute("tabindex", "0");
    }
  }
  __reflectOnValueChange(value) {
    if (!f1.is.exists(this.value)) {
      return;
    }
    const valueStr = `${value}`;
    const valueAttr = this.valueAttr;
    this._group__groupSet().forEach((child) => {
      child.checked = child.getAttribute(valueAttr) === valueStr;
    });
  }
  __valueChanged(_value) {
    this.__reflectOnValueChange(_value);
  }
  __selectedIndexChanged(_selectedIndex) {
    if (!f1.is.number(_selectedIndex)) {
      return;
    }
    this._group__groupSet().forEach((child, index) => {
      child.checked = index === _selectedIndex;
    });
  }
  _group__itemChanged(hook, element) {
    this.__proxyValueToGroup(element);
  }
  __proxyValueToGroup(radioEl) {
    if (radioEl && radioEl.checked) {
      this.selectedIndex = this._group__groupSet().findIndex((el) => el === radioEl);
      this.value = f1.safe(f1.safe("", this[this.valueAttr]), radioEl.getAttribute(this.valueAttr));
    }
  }
  get validity() {
    return this._computeValidity({
      valueMissing: this.required && this.value == null
    });
  }
  toString() {
    return `selected: ${this.value}`;
  }
}
export default HIP.factory.element(RadioGroupElement);
