
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { BehaviorItemGroup } from '../../../eip/element/hi-behavior-group/hi-behavior-item-group.js';
import { BehaviorMirror } from '../../../eip/element/hi-behavior-mirror/hi-behavior-mirror.js';
import {
  AsLitHIC,
  BehaviorA11yKeyboard,
  BehaviorInteractiveElement,
  BehaviorBrowserEvents,
  BehaviorElement,
  HIP
} from '../../../eip/element/hi-component/hi-component.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export const styles = css`
  :host {
    --hi-radio--box-size: calc(var(--hi-px-16) + var(--hi-px-4));
    --hi-radio--border-color: var(--hi-clr--gray-7);
    clear: both;
    display: inline-flex;
    box-sizing: border-box;
    margin-right: var(--hi-px-8);
  }

  :host,
  #innerRing,
  #outerRing {
    border-radius: var(--hi-shape--pill);
  }

  #outerRing {
    display: flex;
    justify-content: center;
    align-items: center;
    height: var(--hi-radio--box-size);
    width: var(--hi-radio--box-size);
    color: var(--hi-clr--gray-7);
    border: var(--hi-px-2) solid var(--hi-radio--border-color);
    will-change: box-shadow;
    transition: box-shadow var(--hi-animation--speed) var(--hi-animation--accel);
    background: var(--hi-clr--white);
  }

  #innerRing {
    width: calc(var(--hi-radio--box-size) - var(--hi-px-8));
    height: calc(var(--hi-radio--box-size) - var(--hi-px-8));
    background: var(--hi-radio--background-color, var(--hi-clr--theme-3));
    border: var(--hi-px-2) solid var(--hi-clr--transparent);
    transform: scale(0);
    opacity: 0;
    transition: transform var(--hi-animation--speed) var(--hi-animation--accel),
      opacity var(--hi-animation--speed) var(--hi-animation--accel);
  }

  :host([checked]) #innerRing {
    opacity: 1;
    transform: scale(1);
  }

  :host([checked]:not([disabled])) #outerRing {
    --hi-radio--border-color: var(--hi-clr--theme-3);
  }

  :host(:hover:not([disabled])) #outerRing {
    border-color: var(--hi-clr--theme-3);
    box-shadow: var(--hi-clr--shadow-down-2);
  }

  :host([disabled]) {
    pointer-events: all !important;
  }

  :host([disabled]) * {
    pointer-events: none !important;
  }

  :host([disabled]) #outerRing,
  :host([disabled]) #innerRing {
    --hi-radio--background-color: var(--hi-clr--disabled);
    --hi-radio--border-color: var(--hi-clr--disabled);
  }
`;
export class RadioElement extends BehaviorItemGroup(
  BehaviorMirror(
    BehaviorA11yKeyboard(BehaviorBrowserEvents(BehaviorInteractiveElement(BehaviorElement(AsLitHIC(LitElement)))))
  )
) {
  constructor() {
    super();
    this.groupName = "radio";
  }
  static get is() {
    return "hi-radio";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  render() {
    return html`
      <div id="outerRing">
        <div id="innerRing"></div>
      </div>
    `;
  }
  static get properties() {
    return {
      checked: {
        type: Boolean,
        reflect: true,
        notify: true
      },
      value: {
        type: String,
        reflect: true,
        notify: true
      }
    };
  }
  static get api() {
    return ["toggle"];
  }
  static get keyBindings() {
    return {
      "home:keyup": "__onKeyUp_home",
      "end:keyup": "__onKeyUp_end",
      "home:keydown": "_onKeyAny__preventDefault",
      "end:keydown": "_onKeyAny__preventDefault",
      "space:keydown": "_onKeyAny__preventDefault",
      "space:keyup": "__onKeyUp_space",
      "right:keyup": "__onKeyUp_right",
      "left:keyup": "__onKeyUp_left",
      "down:keydown": "_onKeyAny__preventDefault",
      "up:keydown": "_onKeyAny__preventDefault",
      "down:keyup": "__onKeyUp_down",
      "up:keyup": "__onKeyUp_up"
    };
  }
  static get browserBindings() {
    return {
      click: "__onClick"
    };
  }
  static get valueProperty() {
    return "checked";
  }
  static get autoBinding() {
    return {
      signal: "checked-changed",
      field: "checked"
    };
  }
  get inputMirrorType() {
    return "radio";
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "radio");
    this.__assignFocusRule();
  }
  update(props) {
    super.update(props);
    if (props.has("checked") || props.has("group")) {
      this.__stateChanged(this.checked, this.group);
    }
  }
  commit() {
    if (this.parentElement && f1.is.in("commit", this.parentElement) && f1.is.function(this.parentElement.commit)) {
      this.parentElement.commit(this.value);
    }
  }
  toggle(checked = !this.checked) {
    this.__setChecked(checked);
  }
  __onClick(event) {
    event.stopPropagation();
    this.__setChecked(!this.checked);
  }
  __onKeyUp_home(event) {
    this.__moveTo(event, "home");
  }
  __onKeyUp_end(event) {
    this.__moveTo(event, "end");
  }
  __onKeyUp_space() {
    this.__setChecked(!this.checked);
  }
  __onKeyUp_left(event) {
    this.__moveTo(event, "previous");
  }
  __onKeyUp_right(event) {
    this.__moveTo(event, "next");
  }
  __onKeyUp_up(event) {
    this.__moveTo(event, "previous");
  }
  __onKeyUp_down(event) {
    this.__moveTo(event, "next");
  }
  __moveTo(event, direction) {
    const group = this._group__groupSet();
    const lastItem = this;
    event.preventDefault();
    const directionType = `__${direction}`;
    this[directionType](group, this, (nextItem) => {
      lastItem.setAttribute("tabindex", "-1");
      nextItem.__setChecked(true);
    });
  }
  __next(list, o, callback) {
    return this.__doesElExist(list, o, (index, list2) => {
      return callback(list2[(index + 1) % list2.length]);
    });
  }
  __previous(list, o, callback) {
    return this.__doesElExist(list, o, (index, list2) => {
      index = index - 1;
      if (index < 0) {
        index = list2.length - 1;
      }
      return callback(list2[index]);
    });
  }
  __home(list, o, callback) {
    return this.__doesElExist(list, o, (index, list2) => {
      return callback(list2[0]);
    });
  }
  __end(list, o, callback) {
    return this.__doesElExist(list, o, (index, list2) => {
      return callback(list2[list2.length - 1]);
    });
  }
  __doesElExist(list, o, ifExists) {
    const index = this.__findIndex(list, o);
    if (index >= 0) {
      return ifExists(index, list, o);
    } else {
      return o;
    }
  }
  __findIndex(list, o) {
    return list ? list.indexOf(o) : -1;
  }
  __setChecked(_newChecked) {
    if (this.disabled) {
      return;
    }
    const newChecked = this.hasAttribute("group") ? true : !!_newChecked;
    if (newChecked !== this.checked) {
      this.checked = newChecked;
    }
    this.focus();
    this.commit();
  }
  __assignFocusRule() {
    const group = this._group__groupSet();
    if (!group) {
      return;
    }
    const checkedEl = group.find((node) => node.checked);
    if (checkedEl) {
      checkedEl.setAttribute("tabindex", "0");
    }
    group.filter((node) => !node.checked).forEach((node) => {
      node.setAttribute("tabindex", "-1");
    });
    group.filter((node) => node.disabled).forEach((node) => {
      node.removeAttribute("tabindex");
    });
    const isChecked = !f1.is.nil(checkedEl);
    if (isChecked) {
      return;
    }
    const firstFocusableRadio = group.find((node) => !node.disabled);
    if (firstFocusableRadio) {
      firstFocusableRadio.setAttribute("tabindex", "0");
    }
  }
  __stateChanged(_checked, _group) {
    const group = this._group__groupSet();
    this.setAttribute("aria-checked", `${f1.to.boolean(_checked)}`);
    if (_group && group && !RadioElement.__mutex__ && group.length > 0) {
      RadioElement.__mutex__ = true;
      this.checked = _checked;
      this._group__stateChanged();
      this.__assignFocusRule();
      RadioElement.__mutex__ = false;
    }
  }
  toString() {
    return `${super.toString()} ${this.checked}`;
  }
}
RadioElement.__mutex__ = false;
export default HIP.factory.element(RadioElement);
