
import { html, css, LitElement } from '../../../hip/library/lit-element/lit-element.js';
import { ref, createRef } from '../../../hip/library/lit-html/directives/ref.js';
import { repeat } from '../../../hip/library/lit-html/directives/repeat.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { BehaviorInput } from '../../../eip/element/hi-behavior-input/hi-behavior-input.js';
import { BehaviorForm } from '../../../eip/element/hi-behavior-form/hi-behavior-form.js';
import {
  HIP,
  AsLitHIC,
  BehaviorA11yKeyboard,
  BehaviorInteractiveElement,
  BehaviorElement
} from '../../../eip/element/hi-component/hi-component.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import '../../../eip/element/hi-tooltip/hi-tooltip.js';
const sortDescFn = (a, b) => a.order - b.order;
const mapToRenderFields = (input, rankAttr) => {
  const mapFn = (item) => ({
    title: item.title || item.label || "",
    name: item.name || item.title || "",
    value: item.value || item.id || "",
    hidden: item.hidden || false,
    order: item[rankAttr] || 0,
    __source__: item
  });
  return f1.map(mapFn, input);
};
const styles = css`
  :host {
    display: block;
    width: auto;
  }

  :host(:focus),
  :host([focused]) {
    outline: none;
  }

  .checkbox-control {
    padding: var(--hi-px-16);
    padding-left: var(--hi-px-48);
  }

  .list {
    list-style: none;
    padding: 0;
    margin: 0;
    width: 100%;
  }

  .list-item {
    display: flex;
    align-items: center;
    color: var(--hi-clr--text);
    line-height: var(--hi-font-lh--medium);
    font-size: var(--hi-font-size--text);
    padding: var(--hi-px-1) 0;
    border-left: var(--hi-px-2) solid var(--hi-clr--transparent);
  }

  .list-item[selected] {
    font-weight: var(--hi-font--weight-bold);
    position: relative;
  }

  .list-item[focused] {
    background: var(--hi-clr--blue-1);
    border-left-color: var(--hi-clr--blue-3);
  }

  :host([aesthetic='standard']) .list {
    border: var(--hi-px-1) solid var(--hi-clr--gray-4);
  }

  :host([aesthetic='standard']) .list-item:not(:last-child) {
    border-bottom: var(--hi-px-1) solid var(--hi-clr--gray-4);
  }

  :host([aesthetic='list']) .list {
    border: none;
  }

  :host([aesthetic='list']) .list-item {
    border: var(--hi-px-1) solid var(--hi-clr--gray-4);
    border-bottom-color: var(--hi-clr--gray-5);
    margin-bottom: var(--hi-px-4);
    border-radius: var(--hi-shape--radius-2);
  }

  .list-item [drag-handle] {
    cursor: move;
    align-self: stretch;
    display: flex;
    align-items: center;
    justify-content: center;
    color: var(--hi-clr--blue-3);
  }

  [drag-handle] hi-icon {
    cursor: move;
  }

  .list-item-body {
    padding: var(--hi-px-8);
    padding-left: 0;
    flex-grow: 1;
  }

  .arrow-buttons {
    display: inline-block;
  }

  .list-header {
    display: flex;
    align-items: center;
  }

  .list-item [drag-handle] {
    padding: var(--hi-px-16);
  }

  .drag-item {
    background: var(--hi-clr--blue-1);
  }

  .list-item.drag-hover,
  :host([aesthetic='list']) .list-item.drag-hover {
    border: 2px dashed var(--hi-clr--blue-3) !important;
  }

  .list-item.drag-item .arrow-buttons {
    display: none;
  }

  hi-button[disabled],
  hi-button[disabled] hi-icon {
    visibility: hidden;
  }
`;
const _RankList = class extends BehaviorInput(
  BehaviorForm(BehaviorA11yKeyboard(BehaviorInteractiveElement(BehaviorElement(AsLitHIC(LitElement)))))
) {
  constructor() {
    super(...arguments);
    this.input = [];
    this.selectedIndex = -1;
    this.focusedIndex = -1;
    this.rankAttr = "order";
    this.aesthetic = "standard";
    this.allowInclusion = false;
    this.disallowJumpUp = false;
    this.disallowJumpDown = false;
    this.disallowJumpTop = false;
    this.disallowJumpBottom = false;
    this.selectionsDB = [];
    this.checkboxControlRef = createRef();
    this.checkboxGroupRef = createRef();
    this.__list = [];
    this.__draggedId = null;
  }
  static get is() {
    return "hi-rank-list";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      input: {
        type: Object
      },
      value: {
        type: Array,
        notify: true
      },
      selectedIndex: {
        type: Number,
        reflect: true,
        attribute: "selected-index"
      },
      focusedIndex: {
        type: Number,
        reflect: true,
        attribute: "focused-index"
      },
      rankAttr: {
        type: String,
        reflect: true,
        attribute: "rank-attr"
      },
      aesthetic: {
        type: String,
        reflect: true
      },
      allowInclusion: {
        type: Boolean,
        reflect: true,
        attribute: "allow-inclusion"
      },
      allowJumpUp: {
        type: Boolean,
        reflect: true,
        attribute: "allow-jump-up"
      },
      allowJumpDown: {
        type: Boolean,
        reflect: true,
        attribute: "allow-jump-down"
      },
      allowJumpTop: {
        type: Boolean,
        reflect: true,
        attribute: "allow-jump-top"
      },
      allowJumpBottom: {
        type: Boolean,
        reflect: true,
        attribute: "allow-jump-bottom"
      },
      disallowJumpUp: {
        type: Boolean,
        reflect: true,
        attribute: "disallow-jump-up"
      },
      disallowJumpDown: {
        type: Boolean,
        reflect: true,
        attribute: "disallow-jump-down"
      },
      disallowJumpTop: {
        type: Boolean,
        reflect: true,
        attribute: "disallow-jump-top"
      },
      disallowJumpBottom: {
        type: Boolean,
        reflect: true,
        attribute: "disallow-jump-bottom"
      },
      selectionsDB: {
        type: Object
      }
    };
  }
  static get valueEvent() {
    return "value-changed";
  }
  static get api() {
    return [
      "clear",
      "reset",
      "moveUp",
      "moveDown",
      "shiftFocusDown",
      "shiftFocusUp",
      "shiftFocusStart",
      "shiftFocusEnd",
      "focusItem",
      "selectItem"
    ];
  }
  static get autoBinding() {
    return {
      signal: "value-changed",
      field: "value"
    };
  }
  static get keyBindings() {
    return {
      "home:keyup": "_rankList__onKeyUp_home",
      "home:keydown": "_onKeyAny__preventDefault",
      "end:keyup": "_rankList__onKeyUp_end",
      "end:keydown": "_onKeyAny__preventDefault",
      "shift+space:keyup": "_rankList__onKeyUp_space",
      "shift+space:keydown": "_onKeyAny__preventDefault",
      "up:keyup": "_rankList__onKeyUp_up",
      "up:keydown": "_onKeyAny__preventDefault",
      "down:keyup": "_rankList__onKeyUp_down",
      "down:keydown": "_onKeyAny__preventDefault",
      "right:keyup": "_rankList__onKeyUp_right",
      "right:keydown": "_onKeyAny__preventDefault",
      "left:keyup": "_rankList__onKeyUp_left",
      "left:keydown": "_onKeyAny__preventDefault",
      "esc:keyup": "_rankList__onKeyUp_esc",
      "esc:keydown": "_onKeyAny__preventDefault"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.addEventListener("focus", this.__initFocusIndex);
    this.addEventListener("focusout", this.__resetFocusIndex);
    dom.setAttrs(this, {
      role: "group",
      "aria-describedby": "instructions"
    });
  }
  update(changedProperties) {
    super.update(changedProperties);
    if (changedProperties.has("allowJumpTop")) {
      this.disallowJumpTop = !this.allowJumpTop;
    }
    if (changedProperties.has("allowJumpBottom")) {
      this.disallowJumpBottom = !this.allowJumpBottom;
    }
    if (changedProperties.has("allowJumpUp")) {
      this.disallowJumpUp = !this.allowJumpUp;
    }
    if (changedProperties.has("allowJumpDown")) {
      this.disallowJumpDown = !this.allowJumpDown;
    }
  }
  render() {
    const ns = this.ns;
    this.__calculateAriaActiveDescendant();
    return html`
      <div id="dragContainer" role="presentation">
        <p id="instructions" class="show-ax" mixin="show-ax">${_RankList.__instructionsText}</p>
        ${this.allowInclusion ? html`<div class="checkbox-control">
              <hi-checkbox
                ${ref(this.checkboxControlRef)}
                id="itemSelected__checkboxControl"
                kind="group"
                for-label="checkbox-control-label"
                .ns=${ns}
                @state-changed=${this.__checkboxControl__onStateChanged}
                @commit-ended=${this.__checkboxControl__onCommitEnded}
              >
              </hi-checkbox>
              <hi-label gid="checkbox-control-label" .ns=${ns}></hi-label>
            </div>` : nothing}
        ${this.allowInclusion ? html`<hi-checkbox-group
              ${ref(this.checkboxGroupRef)}
              id="itemSelected__checkboxGroup"
              group="group__rankList"
              .values=${this.selectionsDB}
              .ns=${ns}
              @state-changed=${this.__checkboxGroup__onStateChanged}
            ></hi-checkbox-group>` : nothing}
        <ul id="list" class="list" role="listbox" aria-labelledby="instructions">
          ${repeat(
      this.__list,
      (item) => item.name,
      (item, index) => html`
              <li
                class="list-item"
                id="listItem_${index}"
                ?focused=${index === this.focusedIndex}
                ?selected=${index === this.selectedIndex}
                aria-selected="${index === this.selectedIndex}"
                role="option"
                @dragstart="${(e) => this.__dndStart(e, index)}"
                @dragenter="${(e) => this.__dndEnter(e, index)}"
                @dragover="${(e) => this.__dndOver(e)}"
                @dragend="${(e) => this.__dndEnd(e, index)}"
                @dragleave="${(e) => this.__dndLeave(e)}"
                @drop="${(e) => this.__dndDrop(e, index)}"
                draggable="true"
              >
                <div drag-handle>
                  <hi-icon kind="draggable" size="small" inherit-color></hi-icon>
                </div>
                ${this.allowInclusion ? html` <span>
                      <hi-checkbox
                        id="checkbox--${index}"
                        role="checkbox"
                        aria-checked=${!!this.selectionsDB[item.name]}
                        group="group__rankList"
                        aria-label="Toggle Item Selection"
                        data-index=${index}
                        value=${item.name}
                        .ns=${ns}
                        .item=${item}
                        @toggle-ended=${this.__itemSelector__onToggleEnded}
                      ></hi-checkbox>
                    </span>` : nothing}

                <span class="list-item-body">${item.title}</span>
                <span class="arrow-buttons">
                  ${!this.disallowJumpTop ? html`<hi-button
                        aesthetic="icon"
                        @trigger-ended=${() => this.__moveFirst(index)}
                        @focusout=${this.__button__onFocusOut}
                        ?disabled=${index === 0}
                      >
                        <hi-hint>Jump to Top</hi-hint>
                        <hi-icon type="material" kind="arrow_first" rotation="90"></hi-icon>
                        <span mixin="show-ax"> Jump to Top </span>
                      </hi-button>` : nothing}
                  ${!this.disallowJumpUp ? html`<hi-button
                        aesthetic="icon"
                        @trigger-ended=${() => this.__move(index, index - 1)}
                        @focusout=${this.__button__onFocusOut}
                        ?disabled=${index === 0}
                      >
                        <hi-hint>Jump Up</hi-hint>
                        <hi-icon type="material" kind="chevron_up"></hi-icon>
                        <span mixin="show-ax"> Jump Up</span>
                      </hi-button>` : nothing}
                  ${!this.disallowJumpDown ? html` <hi-button
                        aesthetic="icon"
                        @trigger-ended=${() => this.__move(index, index + 1)}
                        @focusout=${this.__button__onFocusOut}
                        ?disabled=${index === this.__lastIndex}
                      >
                        <hi-hint>Jump Down</hi-hint>
                        <hi-icon type="material" kind="chevron_down"></hi-icon>
                        <span mixin="show-ax"> Jump Down </span>
                      </hi-button>` : nothing}
                  ${!this.disallowJumpBottom ? html` <hi-button
                        aesthetic="icon"
                        @trigger-ended=${() => this.__moveLast(index)}
                        @focusout=${this.__button__onFocusOut}
                        ?disabled=${index === this.__lastIndex}
                      >
                        <hi-hint>Jump to Bottom</hi-hint>
                        <hi-icon type="material" kind="arrow_last" rotation="90"></hi-icon>
                        <span mixin="show-ax"> Jump to Bottom </span>
                      </hi-button>` : nothing}
                </span>
              </li>
            `
    )}
        </ul>
      </div>
    `;
  }
  updated(changedProperties) {
    super.updated(changedProperties);
    if (changedProperties.has("input")) {
      this.__transformInput();
      this.__sortListAndRender();
      this.__calculateSelectionsDB();
    }
  }
  __transformInput() {
    const _input = f1.to.array(f1.read("/properties", this.input, this.input));
    this.__list = mapToRenderFields(_input, this.rankAttr);
    return this.__list;
  }
  __calculateSelectionsDB() {
    const filteredList = f1.filter((item) => !item.hidden, this.__list);
    this.selectionsDB = f1.map((item) => item.name, filteredList);
  }
  __calculateAriaActiveDescendant() {
    const activeDescendant = this.focusedIndex >= 0 ? `listItem_${this.focusedIndex}` : null;
    dom.setAttr(this, "aria-activedescendant", activeDescendant);
  }
  _rankList__onKeyUp_space() {
    this.__toggleFocusedItem();
  }
  _rankList__onKeyUp_left(event) {
    event.preventDefault();
    this.__onKey__Previous();
  }
  _rankList__onKeyUp_up(event) {
    event.preventDefault();
    this.__onKey__Previous();
  }
  _rankList__onKeyUp_right(event) {
    event.preventDefault();
    this.__onKey__Next();
  }
  _rankList__onKeyUp_down(event) {
    event.preventDefault();
    this.__onKey__Next();
  }
  _rankList__onKeyUp_esc() {
    this.__resetSelection();
  }
  _rankList__onKeyUp_home() {
    if (this.__hasSelectedItem()) {
      this.__moveFirst();
    } else {
      this.shiftFocusStart();
    }
  }
  _rankList__onKeyUp_end() {
    if (this.__hasSelectedItem()) {
      this.__moveLast();
    } else {
      this.shiftFocusEnd();
    }
  }
  __toggleFocusedItem() {
    if (this.selectedIndex !== this.focusedIndex) {
      this.__selectFocusedItem();
    } else {
      this.__resetSelection();
    }
    if (this.__hasSelectedItem()) {
      this.__informUser("GRAB");
    } else {
      this.__informUser("DROP");
    }
  }
  __hasSelectedItem() {
    return this.selectedIndex >= 0;
  }
  __resetSelection() {
    this.selectedIndex = -1;
  }
  __selectFocusedItem() {
    this.selectedIndex = this.focusedIndex;
  }
  __onKey__Next() {
    if (this.__hasSelectedItem()) {
      this.__moveNext();
    } else {
      this.shiftFocusDown();
    }
  }
  __onKey__Previous() {
    if (this.__hasSelectedItem()) {
      this.__movePrevious();
    } else {
      this.shiftFocusUp();
    }
  }
  __button__onFocusOut(event) {
    event.stopPropagation();
  }
  shiftFocusDown() {
    if (this.focusedIndex !== this.__lastIndex) {
      this.__setFocusIndex(this.focusedIndex + 1);
    }
    return this.focusedIndex;
  }
  shiftFocusUp() {
    if (this.focusedIndex !== 0) {
      this.__setFocusIndex(this.focusedIndex - 1);
    }
    return this.focusedIndex;
  }
  shiftFocusStart() {
    this.__setFocusIndex(0);
    return this.focusedIndex;
  }
  shiftFocusEnd() {
    this.__setFocusIndex(this.__lastIndex);
    return this.focusedIndex;
  }
  __setFocusIndex(index) {
    return this.focusedIndex = index;
  }
  __move(from, to) {
    this.selectedIndex = from;
    return this.__moveItem(to);
  }
  __movePrevious() {
    if (this.selectedIndex === 0) {
      return;
    }
    return this.__moveItem(f1.max(0, this.selectedIndex - 1));
  }
  __moveNext() {
    if (this.selectedIndex === this.__lastIndex) {
      return;
    }
    return this.__moveItem(f1.min(this.__lastIndex, this.selectedIndex + 1));
  }
  __moveFirst(index = this.selectedIndex) {
    this.selectedIndex = index;
    for (let i = index; i > 0; i--) {
      this.__moveItem(i - 1);
    }
  }
  __moveLast(index = this.selectedIndex) {
    this.selectedIndex = index;
    for (let i = index; i < this.__lastIndex; i++) {
      this.__moveItem(i + 1);
    }
  }
  __moveItem(to) {
    const currentSelectedIndex = this.selectedIndex;
    const actionName = this.__findActionName(to, currentSelectedIndex);
    const selectedItem = this.__list[currentSelectedIndex];
    const nextSibling = this.__list[to];
    this.__swapItemOrder(selectedItem, nextSibling);
    this.selectedIndex = to;
    this.__setFocusIndex(this.selectedIndex);
    this.__sortListAndRender();
    this.__informUser("MOVE");
    const eventDetails = {
      row: selectedItem,
      input: this.__list,
      action: actionName
    };
    this._dispatchNotification("commit", eventDetails);
    this._dispatchProcess("commit", eventDetails);
    return this.selectedIndex;
  }
  __findActionName(toIndex, fromIndex) {
    const indexDiff = toIndex - fromIndex;
    const absIndexDiff = Math.abs(indexDiff);
    return absIndexDiff === 1 ? indexDiff === 1 ? "move_down" : "move_up" : absIndexDiff > 1 && indexDiff > 1 ? "move_bottom" : "move_top";
  }
  __swapItemOrder(selectedItem, targetItem) {
    const currentOrder = selectedItem.order;
    selectedItem.order = targetItem.order;
    targetItem.order = currentOrder;
  }
  __sortListAndRender() {
    this.__sortList();
    this.requestUpdate();
    this.__updateValue();
  }
  __sortList() {
    this.__list = f1.sort(sortDescFn, this.__list);
  }
  __updateValue() {
    this.value = f1.map((item) => {
      return {
        ...item.__source__,
        [this.rankAttr]: item.order
      };
    }, this.__list);
  }
  __informUser(action) {
    const totalCount = this.__list.length;
    const itemPosition = this.focusedIndex + 1;
    const instructions = "Press up and down aritem keys to change position, shift & spacebar to drop, escape key to cancel.";
    let message = "";
    switch (action) {
      case "GRAB":
        message = `Option ${itemPosition} grabbed, current position ${itemPosition} of ${totalCount}. ${instructions}`;
        break;
      case "DROP":
        message = `Option ${itemPosition} dropped, final position ${itemPosition} of ${totalCount}`;
        break;
      case "MOVE":
        message = `Option ${itemPosition} moved, new position ${itemPosition} of ${totalCount}. ${instructions}`;
        break;
    }
    HIP.fn.announce(message);
  }
  __dndStart(event, positionIndex) {
    this.__draggedId = positionIndex;
    this.__setFocusIndex(positionIndex);
    this.__toggleFocusedItem();
    this.__addClass(event, "drag-item");
    const closestListItem = this.__findClosestListItem(event);
    if (closestListItem) {
      event.dataTransfer.setDragImage(closestListItem, 0, 0);
    }
    this.__informUser("GRAB");
  }
  __dndEnter(event, positionIndex) {
    if (this.__draggedId == null) {
      return;
    }
    this.__addClass(event, "drag-hover");
    event.preventDefault();
  }
  __dndOver(event) {
    event.preventDefault();
  }
  __dndLeave(event) {
    if (this.__draggedId == null) {
      return;
    }
    this.__removeClass(event, "drag-hover");
  }
  __dndDrop(event, positionIndex) {
    if (this.__draggedId == null) {
      return;
    }
    const draggedItem = this.__list[this.__draggedId];
    this.__removeClass(event, "drag-item", "drag-hover");
    this.__slotItemAtPosition(this.__draggedId, positionIndex);
    this.__reIndexList();
    this.__setFocusIndex(positionIndex);
    this.__toggleFocusedItem();
    this.__sortListAndRender();
    this.__informUser("DROP");
    const eventDetails = {
      row: draggedItem,
      input: this.__list,
      action: "drag"
    };
    this._dispatchNotification("commit", eventDetails);
    this._dispatchProcess("commit", eventDetails);
  }
  __dndEnd(event, positionIndex) {
    if (this.__draggedId == null) {
      return;
    }
    this.__removeClass(event, "drag-item");
    this.__draggedId = null;
    this.__toggleFocusedItem();
  }
  __slotItemAtPosition(itemIndex, targetIndex) {
    const draggedItem = this.__list[itemIndex];
    this.__list.splice(itemIndex, 1);
    this.__list = [...this.__list.slice(0, targetIndex), draggedItem, ...this.__list.slice(targetIndex)];
  }
  __reIndexList() {
    this.__list.forEach((item, idx) => {
      item.order = idx;
    });
  }
  __addClass(event, ...classes) {
    const closestListItem = this.__findClosestListItem(event);
    if (closestListItem) {
      dom.addClasses(closestListItem, classes);
    }
  }
  __removeClass(event, ...classes) {
    const closestListItem = this.__findClosestListItem(event);
    if (closestListItem) {
      dom.removeClasses(closestListItem, classes);
    }
  }
  __findClosestListItem(event) {
    const eventTarget = event.target;
    return eventTarget?.closest("li");
  }
  get __lastIndex() {
    return this.__list.length - 1;
  }
  __initFocusIndex() {
    this.__setFocusIndex(this.selectedIndex === -1 ? 0 : this.selectedIndex);
  }
  __resetFocusIndex() {
    this.__setFocusIndex(-1);
  }
  focusItem(index) {
    return this.__setFocusIndex(index);
  }
  selectItem(index) {
    this.focusItem(index);
    return this.__toggleFocusedItem();
  }
  moveUp() {
    return this.__movePrevious();
  }
  moveDown() {
    return this.__moveNext();
  }
  reset() {
    this.__transformInput();
    this.__sortListAndRender();
    return this.value;
  }
  clear() {
    this.value = void 0;
    return this.value;
  }
  get validity() {
    return this._computeValidity({
      valueMissing: this.required && f1.is.nil(this.value)
    });
  }
  __checkboxGroup__onStateChanged(e) {
    const { checkboxGroupEl, checkboxEl } = this.__getCheckboxes();
    if (!checkboxEl || !checkboxGroupEl) {
      return;
    }
    checkboxEl.state = checkboxGroupEl.state;
    const checkboxControlEl = this.checkboxControlRef.value;
    if (checkboxControlEl) {
      checkboxControlEl.state = e.detail.value;
    }
  }
  __checkboxControl__onStateChanged(e) {
    const checkboxGroupEl = this.checkboxGroupRef.value;
    if (checkboxGroupEl) {
      checkboxGroupEl.state = e.detail.value;
    }
  }
  __checkboxControl__onCommitEnded(e) {
    const checkboxControlEl = this.checkboxControlRef.value;
    if (!checkboxControlEl) {
      return;
    }
    const isSelectAll = checkboxControlEl.state === "checked";
    this.selectionsDB = isSelectAll ? f1.map(f1.read("/name", ""), this.input) : [];
    const eventDetails = {
      input: this.__list,
      action: isSelectAll ? "includeAll" : "excludeAll"
    };
    this._dispatchNotification("commit", eventDetails);
    this._dispatchProcess("commit", eventDetails);
  }
  __getCheckboxes() {
    return {
      checkboxEl: this.shadowRoot.getElementById("itemSelected__checkboxControl"),
      checkboxGroupEl: this.shadowRoot.getElementById("itemSelected__checkboxGroup")
    };
  }
  __itemSelector__onToggleEnded(event) {
    if (!event.target) {
      return;
    }
    const itemEl = event.target;
    const itemEvent = itemEl.checked ? "included" : "excluded";
    const item = itemEl.item;
    const eventDetails = {
      row: item,
      input: this.__list,
      action: itemEvent
    };
    if (itemEl.checked) {
      this.selectionsDB.push(item.name);
    } else {
      this.selectionsDB = f1.remove((selection) => selection === item.name, this.selectionsDB);
    }
    this._dispatchNotification("commit", eventDetails);
    this._dispatchProcess("commit", eventDetails);
  }
};
export let RankList = _RankList;
RankList.__instructionsText = `Press shift and spacebar to toggle drag drop mode, use arrow keys to move selected elements. Tab to quick move buttons or include/exclude checkboxes.`;
export default HIP.factory.element(RankList);
