
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import { HIP, AsHIC } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorTemplateElement } from '../../../eip/element/hi-behavior-template/hi-behavior-template.js';
export class ReadyElement extends BehaviorTemplateElement(AsHIC(PolymerElement)) {
  static get is() {
    return "hi-ready";
  }
  static get properties() {
    return {
      kind: {
        type: String
      },
      mode: {
        type: String
      }
    };
  }
  attributeChangedCallback(name, old, value, namespace) {
    super.attributeChangedCallback(name, old, value, namespace);
    this.mutableData = true;
  }
  connectedCallback() {
    super.connectedCallback();
    this.style.display = "none";
    const renderTemplate = function hiReadyReady() {
      this.render();
    }.bind(this);
    if (window["hiReady"]) {
      if ("crates" === this.mode) {
        if (document.body.getAttribute("x-crates") === "initialized") {
          renderTemplate();
        } else {
          document.addEventListener("crates-initialized", renderTemplate);
        }
      } else {
        hiReady(renderTemplate);
      }
    } else {
      const cancel = setTimeout(() => {
        clearTimeout(cancel);
        renderTemplate();
      }, 3e3);
      window.addEventListener("WebComponentsReady", () => {
        clearTimeout(cancel);
        renderTemplate();
      });
    }
  }
  render() {
    this._renderTemplate();
    let template = this.querySelector("template");
    if (!this.__children && !template) {
      const observer = new MutationObserver(() => {
        template = this.querySelector("template");
        if (template) {
          observer.disconnect();
          this.render();
        } else {
          throw new Error("hi-ready requires a <template> or it has no usefulness");
        }
      });
      observer.observe(this, {
        childList: true
      });
    } else {
      this.setAttribute("ready", "ready");
    }
  }
  _renderTemplate() {
    const kind = this.getAttribute("kind");
    if (kind === "lit") {
      this._renderTemplateWithDOM();
    } else {
      super._renderTemplate();
    }
  }
  _renderTemplateWithDOM() {
    const template = this.querySelector("template");
    const templateContent = template.content;
    if (template && templateContent) {
      this.parentNode.insertBefore(templateContent, this);
    }
  }
}
export default HIP.factory.element(ReadyElement);
