
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import { html, svg, render, nothing } from '../../../hip/library/lit-html/lit-html.js';
import { directive } from '../../../hip/library/lit-html/directive.js';
import { unsafeHTML } from '../../../hip/library/lit-html/directives/unsafe-html.js';
import { ifDefined } from '../../../hip/library/lit-html/directives/if-defined.js';
import { styleMap } from '../../../hip/library/lit-html/directives/style-map.js';
import { classMap } from '../../../hip/library/lit-html/directives/class-map.js';
import { repeat } from '../../../hip/library/lit-html/directives/repeat.js';
import { cache } from '../../../hip/library/lit-html/directives/cache.js';
import { guard } from '../../../hip/library/lit-html/directives/guard.js';
import { until } from '../../../hip/library/lit-html/directives/until.js';
import { HIP, AsHIC, BehaviorElement } from '../../../eip/element/hi-hip/hi-hif.js';
import { BehaviorIO } from '../../../eip/element/hi-behavior-io/hi-behavior-io.js';
export class RenderElement extends BehaviorIO(BehaviorElement(AsHIC(PolymerElement))) {
  constructor() {
    super(...arguments);
    this.context = {};
  }
  static get is() {
    return "hi-render";
  }
  static get properties() {
    return {
      for: {
        type: String
      },
      litFn: {
        type: Object
      },
      template: {
        type: Object
      },
      resource: {
        type: Object
      },
      context: {
        type: Object
      }
    };
  }
  static get observers() {
    return ["__outputWatcher(output, for, litFn)", "__renderTemplates(template, resource, context)"];
  }
  static get autoBinding() {
    return {
      signal: "output-changed",
      field: "input"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("hidden", "hidden");
  }
  __outputWatcher(output) {
    const litFn = this.litFn;
    if (typeof litFn !== "function") {
      return;
    }
    const forEl = HIP.browser.queryOne(this.for, this.ns);
    if (forEl && output != null) {
      render(litFn(output), forEl);
    }
  }
  __renderTemplates(template, resource, context) {
    if ((resource || template) && context) {
      this.litFn = resource ? resource.render(context) : template.render(context);
    }
  }
}
HIP.extern["lit"] = Object.freeze({
  directive,
  html,
  svg,
  render,
  nothing,
  directives: {
    unsafeHTML,
    ifDefined,
    styleMap,
    classMap,
    repeat,
    cache,
    guard,
    until
  }
});
HIP.factory.element(RenderElement);
