
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { Debouncer } from '../../../hip/library/polymer/polymer.js';
import { microTask } from '../../../hip/library/polymer/polymer.js';
import { HIP, AsHIC, BehaviorElement, BehaviorBrowserEvents } from '../../../eip/element/hi-component/hi-component.js';
import '../../../eip/element/hi-component/hi-component.js';
import '../../../eip/element/hi-icon/hi-icon.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class RevealElement extends BehaviorBrowserEvents(BehaviorElement(AsHIC(PolymerElement))) {
  static get is() {
    return "hi-reveal";
  }
  static get properties() {
    return {
      revealed: {
        type: Boolean,
        reflectToAttribute: true,
        notify: true
      },
      maxHeight: {
        type: Number,
        value: 2
      },
      kind: {
        type: String
      },
      labelReveal: {
        type: String,
        value: "Show More"
      },
      labelConceal: {
        type: String,
        value: "Show Less"
      },
      revealMore: {
        type: Boolean,
        reflectToAttribute: true
      },
      disallowScrolling: {
        type: Boolean,
        reflectToAttribute: true
      },
      __controlLabel: {
        type: String,
        computed: "__controlLabelCompute(labelReveal, labelConceal, revealed)"
      }
    };
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          display: block;
          font-family: var(--hi-font--family-content);
          font-weight: var(--hi-font--weight-light);
          line-height: var(--hi-font-lh--medium);
          font-size: var(--hi-font-size--text);
        }

        :host(:not([kind='additional']):not([revealed])) #clampContainer {
          display: -webkit-box;
          -webkit-box-orient: vertical;
          overflow: hidden;
          height: 2.75ch;
        }

        #revealControl {
          font-size: var(--hi-font-size--text);
          line-height: var(--hi-font-lh--text);
          margin: var(--hi-px-4) 0 var(--hi-px-8) auto;
          color: var(--hi-clr--theme-3);
          padding-right: var(--hi-px-8);
          border: var(--hi-px-1) solid var(--hi-clr--transparent);
          border-radius: var(--hi-shape--radius-2);
          box-shadow: none;
          background-color: var(--hi-clr--transparent);
          -webkit-appearance: none;
          -moz-appearance: none;
          cursor: default;
          align-items: center;
          text-shadow: var(--hi-clr--text-shadow-2);
          display: none;
        }

        #revealControl:hover {
          text-shadow: none;
          text-decoration: none;
        }

        #revealControl:focus {
          outline: var(--hi-outline--width) var(--hi-outline--line) var(--hi-clr--outline);
          outline-offset: var(--hi-outline--offset);
        }

        #revealControl hi-icon {
          margin-left: var(--hi-px-4);
        }

        :host([kind='additional']) #content {
          display: initial;
        }

        :host([kind='additional']:not([revealed])) #additionalContent {
          display: none;
        }

        :host([reveal-more]) #revealControl,
        :host([revealed]) #revealControl {
          display: flex;
        }
      </style>

      <div id="container">
        <div id="clampContainer">
          <slot id="content"></slot>
        </div>
        <slot id="additionalContent" name="additional-content"></slot>
      </div>

      <button id="revealControl" aria-controls="additionalContent" on-click="__revealControl__onClick">
        <span> [[ __controlLabel ]] </span>
        <hi-icon id="iconDirection" size="small" kind="chevron_up" mode="presentation"> </hi-icon>
      </button>
    `;
  }
  static get api() {
    return ["toggle"];
  }
  static get observers() {
    return ["__revealedWatcher(revealed)", "__revealMoreWatcher(revealMore)"];
  }
  static get browserBindings() {
    return {
      click: "_noop"
    };
  }
  static get autoBinding() {
    return {
      signal: "reveal-changed",
      field: "revealed"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.__onSlotChanged_contentBound = this.__onSlotChanged_content.bind(this);
    this.__onSlotChanged_additionalBound = this.__onSlotChanged_additional.bind(this);
    this.$.content.addEventListener("slotchange", this.__onSlotChanged_contentBound);
    this.$.additionalContent.addEventListener("slotchange", this.__onSlotChanged_additionalBound);
    if (this.kind !== "additional") {
      this.__resizeObserver = new ResizeObserver((rso) => this.__updateRevealMoreOnResize(rso));
      this.__resizeObserver.observe(this.$.clampContainer);
    }
    this.setAttribute("role", "group");
    this._reflectInternalState();
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    this.$.content.removeEventListener("slotchange", this.__onSlotChanged_contentBound);
    this.$.additionalContent.removeEventListener("slotchange", this.__onSlotChanged_additionalBound);
    if (this.__resizeObserver) {
      this.__resizeObserver.unobserve(this.$.clampContainer);
    }
  }
  __controlLabelCompute(labelReveal, labelConceal, revealed) {
    return revealed ? labelConceal : labelReveal;
  }
  __revealedWatcher(revealed) {
    if (this.kind === "additional") {
      dom.setAttr(this.$.iconDirection, "direction", revealed ? "up" : "down");
    }
    this._reflectInternalState();
  }
  __revealMoreWatcher(revealMore) {
    dom.setAttr(this.$.iconDirection, "direction", revealMore ? "down" : "up");
  }
  toggle() {
    if (this.disabled) {
      return;
    }
    this.revealed = !this.revealed;
    this.$.revealControl.focus();
    if (!this.disallowScrolling) {
      this.$.revealControl.scrollIntoView({ block: "center" });
    }
  }
  isContentHidden() {
    return this.$.clampContainer.scrollHeight > this.$.clampContainer.offsetHeight;
  }
  readContent() {
    return this._readSlotNodes("content");
  }
  readAdditionalContent() {
    return this._readSlotNodes("additionalContent");
  }
  _reflectInternalState() {
    this.__reflectAria();
    if (this.kind !== "additional") {
      this.__reflectInternalStateContent();
    } else {
      this.__reflectInternalStateAdditional();
    }
  }
  _readSlotNodes(id) {
    const el = this.$[id];
    const content = el.textContent || el.assignedNodes({ flatten: false }).map((_) => _.textContent).join("");
    return content.trim();
  }
  __updateRevealMoreOnResize(resizeEntry) {
    this.__debounceResize = Debouncer.debounce(this.__debounceResize, microTask, () => {
      if (resizeEntry && resizeEntry[0]) {
        this._reflectInternalState();
      }
    });
  }
  __onSlotChanged_additional() {
    this._reflectInternalState();
  }
  __onSlotChanged_content() {
    this._reflectInternalState();
  }
  __reflectInternalStateContent() {
    this.__reflectContainerHeight();
    this.__reflectRevealMore();
  }
  __reflectInternalStateAdditional() {
    this.__reflectRevealMoreAdditionalMode();
  }
  __reflectAria() {
    dom.setAttrs(this.$.revealControl, {
      "aria-label": this.__controlLabel,
      "aria-expanded": `${f1.safe(false, this.revealed)}`
    });
  }
  __reflectRevealMoreAdditionalMode() {
    if (this.revealed) {
      return this.set("revealMore", false);
    }
    const additionalContent = this.readAdditionalContent();
    const hasAdditionalContent = !f1.is.empty(additionalContent);
    return this.set("revealMore", hasAdditionalContent);
  }
  __reflectRevealMore() {
    this.set("revealMore", this.isContentHidden());
  }
  __reflectContainerHeight() {
    const content = this.readContent();
    const hasNoContent = f1.is.empty(content);
    const defaultLineHeight = 2.75;
    const height = hasNoContent ? "0" : this.revealed ? "auto" : `${defaultLineHeight * this.maxHeight}ch`;
    dom.setAttrs(this.$.clampContainer, {
      style: `height: ${height}`
    });
  }
  __revealControl__onClick() {
    this.toggle();
  }
}
export default HIP.factory.element(RevealElement);
