
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { AsLitHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export const styles = css`
  :host {
    display: block;
  }

  #indexList {
    width: inherit;
    display: flex;
    align-items: center;
  }

  ul,
  li {
    list-style: none;
    padding: 0;
    margin: 0;
  }

  .risk-index-cell {
    text-align: center;
    text-transform: uppercase;
    border: var(--hi-px-1) solid var(--hi-clr--gray-9);
    padding: var(--hi-px-4) var(--hi-px-8);
    line-height: var(--hi-px-16);
  }

  .risk-index-cell.cell-title {
    color: var(--hi-clr--gray-8);
    border-color: var(--hi-clr--gray-9);
    background-color: var(--hi-clr--gray-0);
  }

  .risk-index-cell.no-value {
    color: var(--hi-clr--gray-8);
    border-color: var(--hi-clr--gray-9);
    background-color: var(--hi-clr--white);
  }

  .risk-index-cell > strong {
    letter-spacing: 0.5px;
  }

  .risk-index-cell:not(.with-value) > span,
  .risk-index-cell > strong {
    font-size: var(--hi-font-size--small);
  }

  .risk-index-cell.with-value > span {
    font-size: calc(0.9 * var(--hi-font-size--small));
  }

  .risk-index-cell.cell-title > span {
    font-weight: var(--hi-font--weight-normal);
  }

  .risk-index-cell.cell-title {
    border-right-width: var(--hi-px-2);
  }

  .risk-index-cell:not(:first-child) {
    border-left: none;
  }
`;
export class RiskElement extends BehaviorElement(AsLitHIC(LitElement)) {
  constructor() {
    super(...arguments);
    this.value = null;
    this.noValue = "No Value";
    this.riskIndices = {
      low: {
        color: "teal"
      },
      moderate: {
        color: "blue"
      },
      high: {
        color: "yellow"
      },
      critical: {
        color: "orange"
      }
    };
  }
  static get is() {
    return "hi-risk";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      aesthetic: {
        type: String
      },
      title: {
        type: String
      },
      value: {
        type: String
      },
      noValue: {
        type: String,
        attribute: "no-value"
      },
      riskIndices: {
        type: Object,
        attribute: "risk-indices"
      }
    };
  }
  render() {
    const riskIndices = this.riskIndices;
    const riskKeys = f1.keys(riskIndices);
    const itemWidth = 100 / (riskKeys.length + 1);
    const value = this.value;
    const title = this.title;
    const riskStats = value === null ? [] : riskKeys;
    const widthNoValue = `${100 - itemWidth}%`;
    const width = `${itemWidth}%`;
    return html`
      <ul id="indexList">
        ${!title ? nothing : html`<li class="risk-index-cell cell-title" style="width: ${width}">
              <span>${title}</span>
            </li>`}
        ${value !== null ? nothing : html`<li class="risk-index-cell no-value" style="width: ${widthNoValue}">
              <span>${this.noValue}</span>
            </li>`}
        ${riskStats.map((key) => {
      const riskIndex = riskIndices[key];
      const color = f1.read("/color", "gray", riskIndex);
      const title2 = f1.read("/title", key, riskIndex);
      const isSelected = value === key;
      const baseColorLevel = color === "gray" ? "9" : "5";
      const hiddenColorLevel = color === "gray" ? "1" : "1";
      const colorLevel = this.aesthetic !== "focus" ? baseColorLevel : isSelected ? baseColorLevel : hiddenColorLevel;
      const style = `
            width: ${width};
            color: var(--hi-clr--${color}-${colorLevel});
            background-color: var(--hi-clr--${color}-1);
          `;
      return html`<li class="risk-index-cell with-value" aria-hidden=${!isSelected} style=${style}>
            ${isSelected ? html`<strong>${title2}</strong>` : html`<span>${title2}</span>`}
          </li>`;
    })}
      </ul>
    `;
  }
}
export default HIP.factory.element(RiskElement);
