
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorA11yKeyboard, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorContainer } from '../../../eip/element/hi-behavior-container/hi-behavior-container.js';
import { BehaviorAutoComplete } from '../../../eip/element/hi-behavior-auto-complete/hi-behavior-auto-complete.js';
import { BehaviorForm } from '../../../eip/element/hi-behavior-form/hi-behavior-form.js';
import '../../../eip/element/hi-dropdown/hi-dropdown.js';
import '../../../eip/element/hi-button/hi-close.js';
import '../../../eip/element/hi-input/hi-input.js';
import '../../../eip/element/hi-list/hi-list.js';
import '../../../eip/element/hi-list/hi-item.js';
import '../../../eip/element/hi-repeat/hi-repeat.js';
export class SearchElement extends BehaviorAutoComplete(
  BehaviorContainer(BehaviorA11yKeyboard(BehaviorForm(BehaviorElement(AsHIC(PolymerElement)))))
) {
  static get is() {
    return "hi-search";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          --hi-search-select-height: 15rem;
          --hi-search-popup-list-height: var(--hi-search-select-height);
          --hi-search-input--height: calc(var(--hi-px-32) + var(--hi-px-8));

          min-width: calc(var(--hi-px-32) * 4);
          display: block;
        }

        #valueContainer {
          display: flex;
          flex-wrap: nowrap;
          align-items: center;
          flex-direction: row;
        }

        #dropdown {
          --hi-dropdown--height: auto;
          --hi-dropdown--color-bkg-focus: var(--hi-clr--gray-1);
          width: auto;
          height: auto;
        }

        #popupElement {
          max-height: var(--hi-search-select-height);
        }

        #searchInput {
          border: 0;
          background-color: var(--hi-clr--transparent);
          padding: var(--hi-px-4);
          flex-grow: 1;
          width: 8ch;
          font-weight: var(--hi-font--weight-light);
          text-rendering: optimizeLegibility;
          font-family: var(--hi-font--family-content);
          font-size: var(--hi-font-size--text);
          line-height: var(--hi-font-lh--text);
        }

        #iconContainer {
          width: var(--hi-px-32);
          font-size: var(--hi-px-20);
          text-align: center;
        }

        #searchInput:focus {
          outline: none;
        }

        #statusMessage,
        #resultMessage {
          font-size: var(--hi-font-size--text);
          font-weight: var(--hi-font--weight-heavy);
          background: var(--hi-clr--white);
          padding: var(--hi-px-4) var(--hi-px-8);
          color: var(--hi-clr--gray-9);
          border-bottom: var(--hi-px-1) solid var(--hi-clr--gray-7);
          width: 100%;
          z-index: 3;
        }

        #statusMessage:empty,
        #resultMessage:empty {
          display: none;
        }

        :host([hidden-result-message]) #resultMessage {
          display: none;
        }

        :host([hidden-status-message]) #statusMessage {
          display: none;
        }

        .option-separator-heading {
          font-size: var(--hi-font-size--text);
          font-weight: var(--hi-font--weight-bold);
          color: var(--hi-clr--theme-3);
          border-top: var(--hi-px-1) solid var(--hi-clr--theme-1);
          padding: var(--hi-px-8);
        }

        .option-separator-heading:not([hidden]) ~ hi-item:not([hidden]) {
          padding-left: var(--hi-px-16);
        }

        :host([readonly]),
        :host([readonly]) * {
          opacity: 1;
        }

        :host([disabled]) #searchInput {
          border: var(--hi-px-1) solid var(--hi-clr--gray-3);
        }
      </style>

      <hi-dropdown
        id="dropdown"
        mode="input"
        kind="sticky"
        manual
        cap-width
        flush-width
        disabled-focusable
        disallow-aria-role
        theme$="[[ theme ]]"
        aesthetic$="[[ aesthetic ]]"
        disabled$="[[ disabled ]]"
        readonly$="[[ readonly ]]"
      >
        <div id="valueContainer" slot="anchor">
          <div id="iconContainer">
            <hi-icon kind="search" slot="search" aria-label="search icon" mode="presentation"></hi-icon>
          </div>
          <input
            id="searchInput"
            mixin="input-placeholder"
            type="text"
            role="combobox"
            autocomplete="off"
            aesthetic="minimal"
            aria-autocomplete="none"
            aria-expanded="false"
            aria-haspopup="listbox"
            aria-controls="popupElement"
            disabled$="[[ disabled ]]"
            readonly$="[[ readonly ]]"
            nullable$="[[ nullable ]]"
            placeholder$="[[ placeholder ]]"
            aria-activedescendant$="[[ activeDescendant ]]"
            disallowed-shadow-active-element
          />
          <hi-close
            id="clear"
            size="small"
            theme$="[[ theme ]]"
            disabled$="[[ disabled ]]"
            readonly$="[[ readonly ]]"
            aria-label="Clear Input"
            on-trigger-ended="__onTriggerEnded__clearInput"
            hidden
          ></hi-close>
        </div>
        <p id="resultMessage" slot="status" aria-live="polite" aria-atomic="true"></p>
        <hi-list
          id="popupElement"
          aria-labelledby="valueContainer"
          tabindex="-1"
          allow-remote-focus
          value="[[ value ]]"
          ns$="[[ ns ]]"
          theme$="[[ theme ]]"
          readonly$="[[readonly]]"
          required$="[[ required ]]"
          disabled$="[[ disabled ]]"
          hidden$="[[ _hasEmptyInput(_listResults) ]]"
          aria-activedescendant$="[[ activeDescendant ]]"
          on-focus-index-ended="_onFocusIndexEnded__popupElement"
          on-chosen-ended="__onChosenEnded__popupElement"
        >
          <hi-repeat input="[[ _listResults ]]">
            <template>
              <hi-item
                ns$="[[ ns ]]"
                theme$="[[ theme ]]"
                exclude$="[[ item.heading ]]"
                hidden$="[[ item.heading ]]"
                value$="[[ item.value ]]"
              >
                [[ item.title ]]
              </hi-item>
              <div exclude hidden$="[[ !item.heading ]]" role="separator" class="option-separator-heading">
                [[ item.title ]]
              </div>
            </template>
          </hi-repeat>
        </hi-list>
        <div id="statusMessage" aria-live="polite" aria-atomic="true" hidden$="[[ !_hasEmptyInput(_listResults) ]]">
          <slot name="empty-message">
            <p>No results available.</p>
          </slot>
        </div>
      </hi-dropdown>
    `;
  }
  static get properties() {
    return {
      aesthetic: {
        type: String
      },
      max: {
        type: Number
      }
    };
  }
  static get api() {
    return ["commit"];
  }
  static get observers() {
    return ["__valueWatcher(value)"];
  }
  _onKeyUp_enterKey__input() {
    super._onKeyUp_enterKey__input();
    this._clearSearchInput();
    this.close();
  }
  _onKeyUp_escKey__input() {
    super._onKeyUp_escKey__input();
    this._clearSearchInput();
    this.close();
  }
  __valueWatcher(_value) {
    this.value = _value;
    this.$.dropdown.alignFlyout();
  }
  __onTriggerEnded__clearInput(e) {
    e.stopPropagation();
    this.$.clear.hidden = true;
    this.$.searchInput.dispatchEvent(new Event("input"));
    this.clearInput();
    this.focus();
  }
  __onChosenEnded__popupElement(e) {
    this.value = e.detail.value;
    this._reflectSearchInput();
    this.commit();
    this.close();
  }
  commit() {
    return {
      value: this.value
    };
  }
}
export default HIP.factory.element(SearchElement);
