
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorItemSelect } from '../../../eip/element/hi-behavior-set-select/hi-behavior-item-select.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class OptionElement extends BehaviorItemSelect(BehaviorElement(AsHIC(PolymerElement))) {
  static get is() {
    return "hi-option";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          min-height: var(--hi-el--height);
          padding: var(--hi-px-6) var(--hi-px-8);

          line-height: var(--hi-font-lh--medium);
          font-size: var(--hi-font-size--text);

          border-radius: var(--hi-shape--radius-2);
          box-sizing: border-box;

          border: var(--hi-px-2) solid var(--hi-clr--transparent);
          background-color: var(--hi-clr--transparent);
          color: var(--hi-clr--text);
          display: flex;
          outline: 0;
        }

        @media (hover: hover) {
          :host(:hover:not([disabled])) {
            background-color: var(--hi-clr--gray-1);
            text-shadow: var(--hi-clr--text-shadow-1);
          }
        }

        :host([disabled]) {
          color: var(--hi-clr--disabled);
          pointer-events: all !important;
        }

        #optionContent,
        #container {
          display: flex;
          align-items: center;
          gap: var(--hi-px-8);
        }

        :host([selected]:not([disabled])) #optionContent {
          font-weight: var(--hi-font--weight-heavy);
        }

        :host([selected]:not([disabled]):hover) #optionContent {
          text-shadow: none;
          color: var(--hi-clr--theme-5);
        }

        @media only screen and (max-width: 40rem) {
          :host(:not([disallow-responsiveness])) {
            height: var(--hi-el--height-mobile);
          }
        }
      </style>
      <div id="container">
        <hi-icon
          id="optionIcon"
          glyph$="[[ __iconKind ]]"
          hidden$="[[ __iconHidden ]]"
          readonly$="[[ readonly ]]"
          disabled$="[[ disabled ]]"
          mode="presentation"
          type="material"
          size="large-x"
        ></hi-icon>
        <slot id="optionContent"></slot>
      </div>
    `;
  }
  static get properties() {
    return {
      value: {
        type: Object,
        notify: true
      },
      multiple: {
        type: Boolean
      },
      icon: {
        type: String
      },
      __iconKind: {
        type: String,
        computed: "__computedIconKind(icon, selected, multiple)"
      },
      __iconHidden: {
        type: Boolean,
        computed: "__computedIconHidden(icon, multiple)"
      }
    };
  }
  static get autoBinding() {
    return {
      signal: "selected-changed",
      field: "selected"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "option");
    this.setAttribute("aria-label", this.textContent || "");
    this.multiple = this.multiple || false;
  }
  static get observers() {
    return ["__valueWatcher(value.*)"];
  }
  __valueWatcher(change) {
    let attributeValue = change.base;
    if (typeof change.base !== "string") {
      try {
        attributeValue = JSON.stringify(change.base);
      } catch (e) {
        console.warn("set value is not a valid Object");
      }
    }
    this.setAttribute("value", attributeValue);
  }
  __computedIconKind(icon, selected, multiple) {
    if (!multiple) {
      return f1.safe("", icon);
    } else {
      return selected ? "check_box" : "check_box_outline_blank";
    }
  }
  __computedIconHidden(icon, multiple) {
    return !multiple && !icon;
  }
}
export default HIP.factory.element(OptionElement);
