
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import {
  AsHIC,
  BehaviorA11yKeyboard,
  BehaviorBrowserEvents,
  BehaviorElement,
  BehaviorInteractiveElement,
  HIP
} from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorSetSelect } from '../../../eip/element/hi-behavior-set-select/hi-behavior-set-select.js';
import { BehaviorMirror } from '../../../eip/element/hi-behavior-mirror/hi-behavior-mirror.js';
import { BehaviorInput } from '../../../eip/element/hi-behavior-input/hi-behavior-input.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import './hi-option.js';
export class SelectElement extends BehaviorSetSelect(
  BehaviorBrowserEvents(
    BehaviorInput(
      BehaviorMirror(BehaviorA11yKeyboard(BehaviorInteractiveElement(BehaviorElement(AsHIC(PolymerElement)))))
    )
  )
) {
  constructor() {
    super(...arguments);
    this.canBeInFlyout = true;
  }
  static get is() {
    return "hi-select";
  }
  static get properties() {
    return {
      aesthetic: {
        type: String,
        reflectToAttribute: true
      }
    };
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          display: block;
          border: 0;
          width: auto;
        }

        :host(:focus) {
          outline: none;
        }

        #container ::slotted(hi-option) {
          outline-offset: calc(-1 * var(--hi-px-4));
        }

        :host([allow-remote-focus]) #container ::slotted([focused]:not([disabled])),
        :host(:focus) #container ::slotted([focused]:not([disabled])) {
          outline: var(--hi-outline--width) var(--hi-outline--line) var(--hi-clr--outline);
          background-color: var(--hi-clr--theme-1);
        }

        :host([readonly]),
        :host([disabled]) {
          pointer-events: all !important;
        }

        :host([readonly]) #container ::slotted(hi-option),
        :host([disabled]) #container ::slotted(hi-option) {
          cursor: not-allowed;
        }

        :host([readonly]) #container ::slotted(hi-option) {
          opacity: 1;
        }

        :host([aesthetic*='inline']),
        :host([aesthetic='inline-toggle']) {
          width: auto;
        }

        :host([aesthetic='inline-toggle']) #container ::slotted(hi-option[selected]) {
          background: var(--hi-clr--theme-1) !important;
          color: var(--hi-clr--theme-5) !important;
        }

        @media screen and (min-width: 360px) {
          :host([aesthetic*='inline']) #container {
            display: flex;
          }

          :host([aesthetic='inline-toggle']) #container ::slotted(hi-option) {
            border-color: var(--hi-clr--gray-3);
            padding: var(--hi-px-8) var(--hi-px-16);
          }

          :host([aesthetic='inline-toggle']) #container ::slotted(hi-option:not(:first-of-type)) {
            border-left-width: 0;
          }
        }
      </style>
      <div id="container">
        <slot id="items"></slot>
      </div>
    `;
  }
  static get api() {
    return ["shiftFocusDown", "shiftFocusUp", "focusIndex", "focusItem", "selectIndex", "selectItem"];
  }
  static get keyBindings() {
    return {
      "home:keyup": "_setFocus__onKeyUp_home",
      "end:keyup": "_setFocus__onKeyUp_end",
      "home:keydown": "_onKeyAny__preventDefault",
      "end:keydown": "_onKeyAny__preventDefault",
      "space:keydown": "_onKeyAny__preventDefault",
      "space:keyup": "_setSelect__onKeyUp_space",
      "enter:keyup": "_setSelect__onKeyUp_enter",
      "up:keydown": "_onKeyAny__preventDefault",
      "down:keydown": "_onKeyAny__preventDefault",
      "right:keyup": "_setFocus__onKeyUp_right",
      "down:keyup": "_setFocus__onKeyUp_down",
      "left:keyup": "_setFocus__onKeyUp_left",
      "up:keyup": "_setFocus__onKeyUp_up"
    };
  }
  static get browserBindings() {
    return {
      click: "_setSelect__onTap",
      focus: "_setFocus__onFocus"
    };
  }
  static get valueEvent() {
    return "value-changed";
  }
  static get autoBinding() {
    return {
      signal: "selected-index-changed",
      field: "selectedIndex"
    };
  }
  static get observers() {
    return ["__multipleWatcher(multiple)"];
  }
  get autoBinding() {
    if (this.mode === "value") {
      return {
        signal: "value-changed",
        field: "value"
      };
    } else {
      return {
        signal: "selected-index-changed",
        field: "selectedIndex"
      };
    }
  }
  get inputMirrorType() {
    return "select";
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "listbox");
    this.valueAttr = "value";
  }
  __multipleWatcher(multiple) {
    this.__assignOptionProperties();
  }
  __onSlotChanged() {
    this._setSelect__onSlotChanged();
    this.__assignOptionProperties();
  }
  __assignOptionProperties() {
    this.slotChildren.forEach((el) => {
      dom.flipAttr(el, "multiple", this.multiple);
    });
  }
  get validity() {
    return this._computeValidity({
      valueMissing: this.required && f1.is.empty(this.value)
    });
  }
}
export default HIP.factory.element(SelectElement);
