
import { html, css, LitElement } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { HIP, BehaviorElement, AsLitHIC } from '../../../eip/element/hi-component/hi-component.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export const styles = css`
  :host {
    position: relative;
    min-height: calc(var(--hi-px-48) * 2em);
  }

  :host([busy]) {
    display: block;
    -moz-user-focus: ignore !important;
    pointer-events: none !important;
    user-select: none !important;
    cursor: default !important;
  }

  .ph-item {
    position: relative;
    display: flex;
    flex-wrap: wrap;
    padding: calc(var(--hi-px-15) * 2) var(--hi-px-15) var(--hi-px-15) var(--hi-px-15);
    overflow: hidden;
    background-color: var(--hi-clr--white);
  }

  .ph-item,
  .ph-item *,
  .ph-item ::after,
  .ph-item ::before {
    box-sizing: border-box;
  }

  .ph-item::before {
    content: ' ';
    z-index: 1;
    position: absolute;
    top: var(--hi-px-0);
    right: var(--hi-px-0);
    bottom: var(--hi-px-0);
    left: 50%;
    width: 500%;
    margin-left: -250%;
    -webkit-animation: phAnimation 0.8s linear infinite;
    animation: phAnimation 0.8s linear infinite;
    background: linear-gradient(
        to right,
        rgba(255, 255, 255, 0) 46%,
        rgba(255, 255, 255, 0.35) 50%,
        rgba(255, 255, 255, 0) 54%
      )
      50% 50%;
  }

  .ph-item > * {
    flex: 1 1 auto;
    display: flex;
    flex-flow: column;
    padding-right: var(--hi-px-15);
    padding-left: var(--hi-px-15);
  }

  .ph-row {
    display: flex;
    flex-wrap: wrap;
    margin-bottom: var(--hi-px-8);
  }

  .ph-row div {
    height: var(--hi-px-8);
    margin-bottom: var(--hi-px-8);
    background-color: var(--hi-clr--gray-4);
  }

  .ph-row .big,
  .ph-row.big div {
    height: calc(var(--hi-px-10) * 2);
    margin-bottom: var(--hi-px-15);
  }

  .ph-row .empty {
    background-color: var(--hi-clr--transparent);
  }

  .ph-col-1 {
    flex: 0 0 var(--hi-grid--size-1);
  }

  .ph-col-2 {
    flex: 0 0 var(--hi-grid--size-2);
  }

  .ph-col-3 {
    flex: 0 0 var(--hi-grid--size-3);
  }

  .ph-col-4 {
    flex: 0 0 var(--hi-grid--size-4);
  }

  .ph-col-5 {
    flex: 0 0 var(--hi-grid--size-5);
  }

  .ph-col-6 {
    flex: 0 0 var(--hi-grid--size-6);
  }

  .ph-col-7 {
    flex: 0 0 var(--hi-grid--size-7);
  }

  .ph-col-8 {
    flex: 0 0 var(--hi-grid--size-8);
  }

  .ph-col-9 {
    flex: 0 0 var(--hi-grid--size-9);
  }

  .ph-col-10 {
    flex: 0 0 var(--hi-grid--size-10);
  }

  .ph-col-11 {
    flex: 0 0 var(--hi-grid--size-11);
  }

  .ph-col-12 {
    flex: 0 0 var(--hi-grid--size-12);
  }

  @-webkit-keyframes phAnimation {
    0% {
      transform: translate3d(-30%, 0, 0);
    }
    100% {
      transform: translate3d(30%, 0, 0);
    }
  }

  @keyframes phAnimation {
    0% {
      transform: translate3d(-30%, 0, 0);
    }
    100% {
      transform: translate3d(30%, 0, 0);
    }
  }
`;
export class SkeletalElement extends BehaviorElement(AsLitHIC(LitElement)) {
  constructor() {
    super(...arguments);
    this.busy = false;
    this.rows = [];
    this.defaultRows = [5, 6, 7];
  }
  static get is() {
    return "hi-skeletal";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      busy: {
        type: Boolean,
        reflect: true,
        notify: true
      },
      rows: {
        type: Array
      },
      template: {
        type: Object,
        attribute: false
      },
      when: {
        type: Object,
        attribute: false
      }
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "status");
  }
  render() {
    if (!this.busy) {
      return html`
        ${this.template ? this.template : nothing}
        <slot></slot>
      `;
    }
    const rows = f1.is.empty(this.rows) ? this.defaultRows : this.rows;
    const skeletalInput = rows.map((val) => {
      return {
        length: val,
        empty: 12 - val
      };
    }, []);
    return html`
      <div class="ph-item">
        <div class="ph-col-12">
          <div class="ph-row">
            ${skeletalInput.map(
      (_) => html` <div class="ph-col-${_.length}"></div>
                  <div class="ph-col-${_.empty} empty"></div>`
    )}
          </div>
        </div>
      </div>
    `;
  }
  updated(props) {
    if (props.has("busy")) {
      dom.flipAttr(this, "aria-busy", this.busy, `${this.busy}`);
    }
    if (props.has("when") && this.when) {
      this.waitFor(this.when);
    }
  }
  waitFor(promise) {
    const resetElementAfterPromise = (_) => {
      this.busy = false;
      this.when = void 0;
      return _;
    };
    this.busy = true;
    return promise.then(resetElementAfterPromise).catch(resetElementAfterPromise);
  }
}
export default HIP.factory.element(SkeletalElement);
