
import { afterNextRender } from '../../../hip/library/polymer/polymer.js';
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import {
  AsHIC,
  BehaviorA11yGestures,
  BehaviorA11yKeyboard,
  BehaviorBrowserEvents,
  BehaviorElement,
  BehaviorInteractiveElement,
  HIP
} from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorInput } from '../../../eip/element/hi-behavior-input/hi-behavior-input.js';
import { BehaviorMirror } from '../../../eip/element/hi-behavior-mirror/hi-behavior-mirror.js';
import '../../../eip/element/hi-hint/hi-hint.js';
export class SliderElement extends BehaviorA11yKeyboard(
  BehaviorA11yGestures(
    BehaviorBrowserEvents(
      BehaviorInput(BehaviorMirror(BehaviorInteractiveElement(BehaviorElement(AsHIC(PolymerElement)))))
    )
  )
) {
  static get is() {
    return "hi-slider";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          display: flex;
          height: var(--hi-el--height-sml);
          border-radius: var(--hi-shape--pill);
          background-color: var(--hi-clr--white);
          box-shadow: var(--hi-clr--shadow-inset-2);
          border: var(--hi-px-1) solid var(--hi-clr--theme-3);
          padding-right: var(--hi-px-10);
          padding-left: var(--hi-px-10);
        }

        :host([aesthetic='inline']) {
          width: auto;
        }

        #container:active,
        #sliderTrack:active,
        #sliderThumb:active,
        #sliderThumb:active:after {
          cursor: grabbing;
          cursor: -webkit-grabbing;
        }

        #sliderTrack {
          border-radius: var(--hi-shape--pill);
          width: 100%;
          min-width: calc(var(--hi-px-40) * 2em);
          height: var(--hi-el--height-sml);
        }

        #sliderThumb {
          display: flex;
          flex-direction: row;
          justify-content: flex-end;
          align-self: center;
          border: var(--hi-px-3) solid var(--hi-clr--transparent);
          border-radius: var(--hi-shape--radius);
          overflow: visible;
          will-change: width;
          height: var(--hi-el--height-sml);
          margin-top: calc(-1 * var(--hi-px-1));
        }

        #sliderThumb:after {
          content: ' ';
          cursor: grab;
          cursor: -webkit-grab;
          position: relative;
          align-self: center;
          width: var(--hi-px-16);
          height: var(--hi-px-16);
          min-width: var(--hi-px-16);
          background-color: var(--hi-clr--theme-3);
          margin-right: calc(-1 * var(--hi-px-8));
          border: var(--hi-px-1) solid var(--hi-clr--transparent);
          /* transition: height 0.2s ease, width 0.2s ease, left 0.2s ease; */
          border-radius: var(--hi-shape--pill);
        }

        :host([disabled]) #sliderTrack {
          background-color: var(--hi-clr--gray-2);
          box-shadow: none;
        }

        :host([disabled]) #sliderThumb:after {
          background-color: var(--hi-clr--gray-5);
        }

        #minimumValue,
        #maximumValue {
          display: none;
          align-self: center;
          font-size: var(--hi-px-12);
          font-weight: var(--hi-font--weight-bold);
        }

        #minimumValue {
          margin-right: var(--hi-px-12);
        }

        #maximumValue {
          margin-left: var(--hi-px-12);
        }

        :host([view-max]) #maximumValue {
          display: flex;
        }

        :host([view-min]) #minimumValue {
          display: flex;
        }

        @media (hover: hover) {
          :host(:hover:not([disabled])) {
            box-shadow: var(--hi-clr--shadow-down-2);
          }
          :host(:hover:not([disabled])) #sliderThumb:after {
            box-shadow: var(--hi-clr--shadow-down-3);
          }
        }
      </style>
      <p id="minimumValue">[[ min ]]</p>
      <div id="sliderTrack">
        <div id="sliderThumb" on-track="_onTrack">
          <hi-hint style="margin-left: 50%" placement="top">[[value]]</hi-hint>
        </div>
      </div>
      <p id="maximumValue">[[ max ]]</p>
    `;
  }
  static get properties() {
    return {
      value: {
        type: Number,
        reflectToAttribute: true,
        observer: "__valueChanged",
        notify: true
      },
      min: {
        type: Number,
        observer: "__minChanged",
        value: 0
      },
      max: {
        type: Number,
        observer: "__maxChanged",
        value: 100
      },
      step: {
        type: Number,
        value: 1
      },
      range: {
        type: Number,
        computed: "__rangeCompute(min, max)"
      },
      rangeSteps: {
        type: Number,
        computed: "__rangeStepsCompute(range, step)"
      },
      __stepWidth: {
        type: Number,
        computed: "__stepWidthCompute(__trackWidth, rangeSteps)"
      },
      __trackWidth: {
        type: Number
      },
      __sigDigit: {
        type: Number,
        computed: "__sigDigitCompute(step)"
      },
      boundedValue: {
        type: Number,
        computed: "__boundValueCompute(value, max, min, __sigDigit)"
      }
    };
  }
  static get observers() {
    return ["__valueWatcher(boundedValue, min, range)"];
  }
  static get api() {
    return ["increment", "decrement"];
  }
  static get keyBindings() {
    return {
      "right:keydown": "__onKeyDown_right",
      "down:keydown": "__onKeyDown_down",
      "left:keydown": "__onKeyDown_left",
      "up:keydown": "__onKeyDown_up"
    };
  }
  static get browserBindings() {
    return {
      click: "__onClick"
    };
  }
  static get gestureBindings() {
    return {
      track: "_noop"
    };
  }
  static get valueEvent() {
    return "value-changed";
  }
  static get autoBinding() {
    return {
      signal: "value-changed",
      field: "value"
    };
  }
  get inputMirrorType() {
    return "number";
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "slider");
    afterNextRender(this, () => {
      this.__updateTrackWidth();
    });
  }
  __valueWatcher(boundedValue, min, range) {
    if (Number.isFinite(boundedValue)) {
      this.$.sliderThumb.style.width = `${(boundedValue - min) / range * 100}%`;
    }
  }
  __valueChanged(value) {
    if (Number.isFinite(value)) {
      this.setAttribute("aria-valuenow", `${value}`);
    }
  }
  __rangeCompute(min, max) {
    return max - min;
  }
  __rangeStepsCompute(range, step) {
    return range / step;
  }
  __stepWidthCompute(__trackWidth, rangeSteps) {
    return __trackWidth / (rangeSteps || 1);
  }
  __sigDigitCompute(step) {
    step = step + "";
    return step.slice(step.indexOf("."), -1).length;
  }
  __minChanged(min) {
    this.setAttribute("aria-valuemin", `${min}`);
  }
  __maxChanged(max) {
    this.setAttribute("aria-valuemax", `${max}`);
  }
  __updateTrackWidth() {
    this.__trackWidth = this.$.sliderTrack.offsetWidth;
  }
  __boundValueCompute(value, max, min) {
    return Number.isNaN(value) ? this.boundValue(min + max / 2) : this.boundValue(value || min);
  }
  boundValue(value) {
    return parseFloat(Math.max(Math.min(value, this.max), this.min).toFixed(this.__sigDigit));
  }
  __onClick(event) {
    this.__updateTrackWidth();
    this.value = this.boundedValue + this.__mouseToStep(event) * this.step;
  }
  __mouseToStep(event) {
    const position = event.offsetX;
    const size = 20 - 10;
    return parseFloat(
      ((position - size - this.$.sliderThumb.offsetWidth) / this.__stepWidth).toFixed()
    );
  }
  _onTrack(event) {
    event.preventDefault();
    if (event.detail.state === "start") {
      this.__updateTrackWidth();
      this._startingValue = this.boundedValue;
    } else if (event.detail.state === "end") {
      return;
    }
    this.value = this.boundValue(this._startingValue + this.__trackToStep(event) * this.step);
  }
  __trackToStep(event) {
    return parseFloat((event.detail.dx / this.__stepWidth).toFixed());
  }
  __onKeyDown_up(event) {
    event.preventDefault();
    this.decrement();
  }
  __onKeyDown_right(event) {
    event.preventDefault();
    this.increment();
  }
  __onKeyDown_down(event) {
    event.preventDefault();
    this.increment();
  }
  __onKeyDown_left(event) {
    event.preventDefault();
    this.decrement();
  }
  increment() {
    if (this.disabled) {
      return this;
    }
    this.__updateTrackWidth();
    if (this.value == null) {
      this.value = 0;
    }
    this.value = this.boundValue(this.value + this.step);
    return this;
  }
  decrement() {
    if (this.disabled) {
      return this;
    }
    this.__updateTrackWidth();
    this.value = this.boundValue(this.value - this.step);
    return this;
  }
  get willValidate() {
    return this.novalidate !== true && (this.required != null || this.max != null || this.min != null);
  }
  get validity() {
    const rangeUnderflow = this.value < this.min;
    const rangeOverflow = this.value > this.max;
    const valueMissing = this.required && this.value == null;
    return this._computeValidity({
      rangeUnderflow,
      rangeOverflow,
      valueMissing
    });
  }
}
export default HIP.factory.element(SliderElement);
