
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class StampElement extends BehaviorElement(AsHIC(PolymerElement)) {
  static get is() {
    return "hi-stamp";
  }
  static get properties() {
    return {
      mode: {
        type: String
      },
      input: {
        type: Array,
        observer: "__inputChanged"
      }
    };
  }
  static get autoBinding() {
    return {
      signal: "markup-rendered-ended",
      field: "input"
    };
  }
  static get api() {
    return ["replace", "append"];
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "group");
  }
  append(input) {
    return this.appendUnder(input, this.__getParent());
  }
  appendUnder(input, parent) {
    this.__insert(input, parent);
  }
  replace(input) {
    return this.replaceUnder(input, this.__getParent());
  }
  replaceUnder(input, parent) {
    this.__removeNodes(parent);
    this.__insert(input, parent);
  }
  __insert(input, parent) {
    if (this.mode === "fragment") {
      parent.appendChild(this.__addNodes(f1.safe([], input)));
    } else {
      this.__addNodes(f1.safe([], input), parent);
    }
    this._dispatchProcess("markup-rendered", {
      value: input
    });
  }
  __getParent() {
    return this.parentElement ? this.parentElement : this.parentNode.host;
  }
  __inputChanged(input) {
    this.replace(input);
  }
  __removeNodes(parent) {
    Array.from(parent.children || []).forEach((node) => {
      this.__removeNode(parent, node);
    });
    return parent;
  }
  __removeNode(parent, node) {
    if (node !== this) {
      node.remove();
    }
  }
  __addNodes(children, parent = document.createDocumentFragment()) {
    Array.from(children || []).forEach((node) => {
      this.__addNode(node, parent);
    });
    return parent;
  }
  __addNode(node, parent) {
    const tag = document.createElement(node.tagName);
    const attrs = node.attributes || {};
    if (node.tagName !== "template") {
      if (node.text != null) {
        tag.textContent = node.text;
      } else if (node.html != null) {
        tag.innerHTML = node.html;
      }
    }
    for (const key in attrs) {
      HIP.kernel.assign(tag, key, attrs[key]);
    }
    if (node.tagName === "template") {
      tag.innerHTML = this.__addTemplate(node.children);
    } else {
      this.__addNodes(node.children, tag);
    }
    parent.appendChild(tag);
    return tag;
  }
  __addTemplate(children, htmlString = "") {
    if (!children) {
      return htmlString;
    }
    const collapseNodes = (_htmlString, node) => {
      const tagName = node.tagName;
      const attr = node.attributes || {};
      const text = node.text || "";
      let attributeString = "";
      for (const key in attr) {
        attributeString = `${attributeString} ${key}="${attr[key]}"`;
      }
      return `<${tagName} ${attributeString}>
        ${text}
        ${this.__addTemplate(node.children, _htmlString)}
      </${tagName}>`;
    };
    return f1.safe([], f1.to.array(children)).reduce(collapseNodes, htmlString);
  }
  toString() {
    return JSON.stringify(this.input || {});
  }
}
HIP.factory.element(StampElement);
