
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import {
  BehaviorBrowserEvents,
  AsHIC,
  BehaviorA11yKeyboard,
  BehaviorElement,
  BehaviorInteractiveElement,
  HIP
} from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorTabGroup } from '../../../eip/element/hi-behavior-tab/hi-behavior-tab-group.js';
import './hi-step.js';
export class StepGroupElement extends BehaviorTabGroup(
  BehaviorBrowserEvents(BehaviorA11yKeyboard(BehaviorInteractiveElement(BehaviorElement(AsHIC(PolymerElement)))))
) {
  static get is() {
    return "hi-step-group";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          z-index: 1;
          border: none;
          width: 100%;
          display: block;
          text-align: center;
        }

        :host(:not([aesthetic='stacked'])) #container {
          overflow-x: auto;
          overflow-y: hidden;
          scrollbar-width: thin;
        }

        :host ::-webkit-scrollbar {
          height: var(--hi-px-4);
          background: var(--hi-clr--gray-2);
        }

        :host ::-webkit-scrollbar-thumb {
          background: var(--hi-clr--gray-7);
        }

        #container {
          display: flex;
          flex-flow: row nowrap;
        }

        :host([stretched]) #container ::slotted(hi-step) {
          flex: 1 1;
        }

        @media (hover: hover) {
          :host(:not([disabled])) #container ::slotted(hi-step:not([exclude]):hover) {
            border-color: var(--hi-clr--gray-5);
            text-shadow: var(--hi-clr--text-shadow-1);
            box-shadow: var(--hi-clr--halo-1);
          }
        }

        :host(:not([disabled])) #container ::slotted(hi-step[selected]:hover) {
          text-shadow: none;
        }

        :host(:focus) {
          outline: none;
        }

        #container ::slotted(hi-step[focused]) {
          outline: none;
        }

        :host(:not([disabled]):focus) #container ::slotted(hi-step[focused]:not([disabled])),
        :host(:not([kind='navigation']):focus) #container ::slotted(hi-step[focused]:not([disabled])) {
          outline: var(--hi-outline--width) var(--hi-outline--line) var(--hi-clr--outline);
          outline-offset: calc(-1 * var(--hi-outline--offset));
        }

        :host(:not([disabled])) #container ::slotted(hi-step[selected]:not([disabled])) {
          font-weight: var(--hi-font--weight-bold);
          color: var(--hi-clr--text);
        }

        :host([disabled]) #container ::slotted(*) {
          color: var(--hi-clr--disabled-text);
          opacity: 0.8;
        }

        :host([aesthetic='stacked']) #container {
          display: flex;
          flex-direction: column;
          margin-left: auto;
        }

        :host([aesthetic='stacked']) #container ::slotted(hi-step) {
          text-align: start;
        }

        :host([aesthetic='stacked']) #container ::slotted(hi-step[exclude]) {
          padding: var(--hi-px-20) var(--hi-px-8);
        }

        #container ::slotted(hi-step:last-of-type)::after {
          display: none !important;
        }
      </style>

      <div id="container">
        <slot id="items"> </slot>
      </div>
    `;
  }
  static get properties() {
    return {
      aesthetic: {
        type: String,
        reflectToAttribute: true,
        observer: "__aestheticChanged"
      },
      kind: {
        type: String,
        reflectToAttribute: true,
        observer: "__kindChanged"
      },
      controlMode: {
        type: String,
        reflectToAttribute: true,
        value: "automatic"
      },
      stretched: {
        type: Boolean,
        value: false
      },
      activeIndex: {
        type: Number,
        observer: "__activeIndexChanged"
      },
      manualFurthestIndex: {
        type: Boolean
      }
    };
  }
  static get observers() {
    return ["__deprecatedSettingsWatcher(kind, aesthetic)"];
  }
  static get api() {
    return [
      "assignFurthestIndex",
      "shiftFocusDown",
      "shiftFocusUp",
      "focusIndex",
      "focusItem",
      "selectIndex",
      "selectItem",
      "uncomplete",
      "complete",
      "unlock",
      "lock",
      "unpending",
      "pending",
      "unalert",
      "alert"
    ];
  }
  static get keyBindings() {
    return {
      "space:keydown": "_onKeyAny__preventDefault",
      "space:keyup": "__stepGroup__onKeyUp_space",
      "enter:keyup": "__stepGroup__onKeyUp_enter",
      "up:keydown": "_onKeyAny__preventDefault",
      "down:keydown": "_onKeyAny__preventDefault",
      "right:keyup": "__stepGroup__onKeyUp_right",
      "down:keyup": "__stepGroup__onKeyUp_down",
      "left:keyup": "__stepGroup__onKeyUp_left",
      "up:keyup": "__stepGroup__onKeyUp_up"
    };
  }
  static get browserBindings() {
    return {
      click: "__stepGroup__onTap",
      focus: "_setFocus__onFocus"
    };
  }
  static get autoBinding() {
    return {
      signal: "selected-index-changed",
      field: "selectedIndex"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.__setRole(this.kind);
  }
  _onSlotChanged() {
    super._onSlotChanged();
    this.assignFurthestIndex();
  }
  __deprecatedSettingsWatcher(kind, aesthetic) {
    if (kind === "stacked") {
      this.aesthetic = "stacked";
    }
    if (aesthetic === "title") {
      this.kind = "title";
    }
  }
  __stepGroup__onKeyUp_right(event) {
    if (this.kind !== "navigation") {
      super._setFocus__onKeyUp_right(event);
    }
  }
  __stepGroup__onKeyUp_down(event) {
    if (this.kind !== "navigation") {
      super._setFocus__onKeyUp_down(event);
    }
  }
  __stepGroup__onKeyUp_left(event) {
    if (this.kind !== "navigation") {
      super._setFocus__onKeyUp_left(event);
    }
  }
  __stepGroup__onKeyUp_up(event) {
    if (this.kind !== "navigation") {
      super._setFocus__onKeyUp_up(event);
    }
  }
  __stepGroup__onKeyUp_space(event) {
    this.__stepGroupNavigation__clickLink(event, () => {
      super._setSelect__onKeyUp_space();
    });
  }
  __stepGroup__onKeyUp_enter(event) {
    this.__stepGroupNavigation__clickLink(event, () => {
      super._setSelect__onKeyUp_enter();
    });
  }
  __stepGroup__onTap(event) {
    this.__stepGroupNavigation__clickLink(event, () => {
      super._setSelect__onTap(event);
    });
  }
  __stepGroupNavigation__clickLink(event, defaultFn) {
    if (this.kind === "navigation") {
      const stepEl = event.target;
      if (!stepEl) {
        return;
      }
      const stepLinkEl = stepEl.querySelector("hi-link");
      if (stepLinkEl) {
        stepLinkEl.trigger();
      }
    } else {
      defaultFn.call(this);
    }
  }
  __activeIndexChanged(_activeIndex) {
    this.assignFurthestIndex(_activeIndex);
    this.selectedIndex = _activeIndex;
    this.focusedIndex = _activeIndex;
  }
  __setSelect__selectedIndexChanged(_index, _oldIndex) {
    super.__setSelect__selectedIndexChanged(_index, _oldIndex);
    if ("automatic" === this.controlMode) {
      this.__indexChanged__automaticControlMode(_index);
    }
  }
  __indexChanged__automaticControlMode(_index) {
    this.__slotChildren.forEach((step, stepIndex) => {
      if (!step.state && stepIndex != null && stepIndex === _index) {
        step.visited = true;
      }
    });
  }
  lock(stepIndex) {
    return this.__updateChildrenStatus(stepIndex, "locked", true);
  }
  unlock(stepIndex) {
    return this.__updateChildrenStatus(stepIndex, "locked", false);
  }
  complete(stepIndex) {
    return this.__updateChildrenStatus(stepIndex, "completed", true);
  }
  uncomplete(stepIndex) {
    return this.__updateChildrenStatus(stepIndex, "completed", false);
  }
  pending(stepIndex) {
    return this.__updateChildrenStatus(stepIndex, "pending", true);
  }
  unpending(stepIndex) {
    return this.__updateChildrenStatus(stepIndex, "pending", false);
  }
  alert(stepIndex) {
    return this.__updateChildrenStatus(stepIndex, "alert", true);
  }
  unalert(stepIndex) {
    return this.__updateChildrenStatus(stepIndex, "alert", false);
  }
  __updateChildrenStatus(stepIndex, key, status) {
    const children = this.slotChildren;
    if (children[stepIndex]) {
      children[stepIndex][key] = status;
    }
  }
  assignFurthestIndex(furthestIndex = this.activeIndex) {
    if (this.manualFurthestIndex) {
      return;
    }
    if ("manual" === this.controlMode) {
      return;
    }
    const isValid = typeof furthestIndex === "number" && furthestIndex === furthestIndex;
    if (!isValid) {
      return;
    }
    this.slotChildren.forEach(function(node, index) {
      if (index < furthestIndex) {
        node.state = "completed";
      } else if (index === furthestIndex) {
        node.state = "visited";
      } else {
        node.state = "locked";
      }
    });
    this.selectedIndex = furthestIndex;
  }
  __aestheticChanged(_aesthetic) {
    this._readSlotChildren().forEach((step) => {
      step.setAttribute("aesthetic", _aesthetic);
    });
  }
  __kindChanged(kind) {
    this.__setRole(kind);
  }
  __setRole(kind) {
    const groupRole = kind === "navigation" ? "navigation" : "tablist";
    this.setAttribute("role", groupRole);
  }
}
HIP.factory.element(StepGroupElement);
