
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorBrowserEvents, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorItemSelect } from '../../../eip/element/hi-behavior-set-select/hi-behavior-item-select.js';
import { BehaviorTab } from '../../../eip/element/hi-behavior-tab/hi-behavior-tab.js';
import '../../../eip/element/hi-component/hi-behavior-browser-events.js';
import '../../../eip/element/hi-icon/hi-icon.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class StepElement extends BehaviorTab(
  BehaviorItemSelect(BehaviorBrowserEvents(BehaviorElement(AsHIC(PolymerElement))))
) {
  static get is() {
    return "hi-step";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          --hi-step--height: var(--hi-el--height);
          --hi-step--marker: var(--hi-clr--gray-6);
          --hi-step--marker-size: var(--hi-px-16);
        }

        :host {
          position: relative;
          display: block;
          width: auto;
          height: var(--hi-el--height);
          padding: var(--hi-px-6) var(--hi-px-12);
          letter-spacing: calc(var(--hi-px-1) / 2);
          border: var(--hi-px-1) solid var(--hi-clr--transparent);
          border-radius: var(--hi-shape--radius-2);
          color: var(--hi-clr--text);
          margin-top: var(--hi-px-12);
          margin-right: var(--hi-px-24);
          white-space: nowrap;
          cursor: default;
        }

        #container {
          display: flex;
          flex-flow: row nowrap;
          font-weight: inherit;
          justify-content: flex-start;
          font-size: var(--hi-font-size--text);
          line-height: var(--hi-font-lh--medium);
        }

        :host(:not([aesthetic='stacked']))::after {
          content: '';
          height: var(--hi-px-1);
          min-width: var(--hi-step--marker-size);
          background-color: var(--hi-step--marker);
          position: absolute;
          top: var(--hi-px-15);
          right: -20px;
        }

        :host([aesthetic='stacked'])::after {
          content: '';
          display: block;
          background-color: var(--hi-step--marker);
          width: var(--hi-px-1);
          min-height: var(--hi-step--marker-size);
          margin: var(--hi-px-5) 0 var(--hi-px-5) var(--hi-px-10) !important;
        }

        hi-icon {
          font-size: var(--hi-px-24);
        }

        :host([disabled]) {
          color: var(--hi-clr--disabled-text);
          opacity: 0.8;
        }

        :host([disabled]) hi-icon {
          --hi-icon--color: var(--hi-clr--text);
        }

        :host([selected]) {
          background-color: var(--hi-clr--theme-1);
          border-color: var(--hi-clr--theme-3);
          color: var(--hi-clr--theme-text-1);
        }

        #statusMarker {
          margin-right: var(--hi-px-4);
          vertical-align: middle;
          text-align: center;
        }

        :host(:not([aesthetic='stacked']):not([state])) #statusMarker,
        :host([state='none']:not([aesthetic='stacked'])) #statusMarker {
          display: none;
        }

        :host([aesthetic='stacked']:not([state])) #statusMarker,
        :host([state='none'][aesthetic='stacked']) #statusMarker {
          border: var(--hi-px-1) dashed var(--hi-clr--gray-8);
          border-radius: 1000%;
        }

        :host([kind='title'])::after {
          display: none;
        }

        :host([kind='title']) #text {
          text-transform: uppercase;
        }
      </style>
      <span id="container">
        <hi-icon
          id="statusMarker"
          mode="presentation"
          type="material"
          theme$="[[ theme ]]"
          disabled$="[[ disabled ]]"
        ></hi-icon>
        <div id="text">
          <slot></slot>
        </div>
      </span>
    `;
  }
  static get properties() {
    return {
      locked: {
        type: Boolean,
        reflectToAttribute: true,
        notify: true
      },
      completed: {
        type: Boolean,
        reflectToAttribute: true,
        notify: true
      },
      visited: {
        type: Boolean,
        reflectToAttribute: true
      },
      pending: {
        type: Boolean,
        reflectToAttribute: true,
        notify: true
      },
      alert: {
        type: Boolean,
        reflectToAttribute: true,
        notify: true
      },
      state: {
        type: String,
        reflectToAttribute: true,
        notify: true
      },
      kind: {
        type: String,
        reflectToAttribute: true
      },
      aesthetic: {
        type: String,
        reflectToAttribute: true
      }
    };
  }
  static get observers() {
    return [
      "__statusWatcher(completed, locked, alert, pending, visited, selected)",
      "__deprecatedSettingsWatcher(kind, aesthetic)",
      "__stateWatcher(state, kind)"
    ];
  }
  static get browserBindings() {
    return {
      mouseenter: "_itemFocus__onMouseEnter"
    };
  }
  static get autoBinding() {
    return {
      signal: "selected-changed",
      field: "selected"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.__setRole(this.kind);
  }
  _disabledChanged(disabled) {
    super._disabledChanged();
    if (this.kind === "navigation") {
      f1.each((el) => {
        el.disabled = disabled;
      }, Array.from(this.children));
    }
  }
  __statusWatcher(_completed, _locked, _alert, _pending, _visited, _selected) {
    if (_locked) {
      this.state = "locked";
    } else if (_completed) {
      this.state = "completed";
    } else if (_alert) {
      this.state = "alert";
    } else if (_pending) {
      this.state = "pending";
    } else if (_visited) {
      this.state = "visited";
    }
    dom.flipAttr(this, "aria-current", _selected, "page");
  }
  __stateWatcher(_state, _kind) {
    this.setAttribute("aria-label", `step ${_state || "unmarked"}`);
    if ("locked" === _state) {
      this.$.statusMarker.kind = "lock";
      this.disabled = true;
    } else if ("completed" === _state) {
      this.$.statusMarker.kind = "task_alt";
      this.disabled = false;
    } else if ("alert" === _state) {
      this.$.statusMarker.kind = "priority_high";
      this.disabled = false;
    } else if ("pending" === _state) {
      this.$.statusMarker.kind = "hourglass_empty";
      this.disabled = false;
    } else if ("visited" === _state) {
      this.$.statusMarker.kind = "tour";
      this.disabled = false;
    }
    if ("title" === _kind) {
      this.setAttribute("exclude", "exclude");
      this.$.statusMarker.style.display = "none";
    }
  }
  __deprecatedSettingsWatcher(kind, aesthetic) {
    if (kind === "stacked") {
      this.aesthetic = "stacked";
    }
    if (aesthetic === "title") {
      this.kind = "title";
    }
  }
  __setRole(kind) {
    this.setAttribute("role", this.__groupRole(kind));
  }
  __groupRole(kind) {
    return kind === "navigation" ? "listitem" : "tab";
  }
}
HIP.factory.element(StepElement);
