
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import { BehaviorIO } from '../../../eip/element/hi-behavior-io/hi-behavior-io.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import ess from '../../../pip/library/pi-ess/pi-ess.js';
export class StoreElement extends BehaviorIO(BehaviorElement(AsHIC(PolymerElement))) {
  constructor() {
    super(...arguments);
    this.mode = "local";
  }
  static get is() {
    return "hi-store";
  }
  static get properties() {
    return {
      mode: {
        type: String
      },
      table: {
        type: String,
        observer: "__tableChanged",
        notify: true
      },
      __store: {
        type: Object,
        computed: "__storeCompute(mode)"
      }
    };
  }
  static get api() {
    return ["destroy", "clear"];
  }
  static get valueEvent() {
    return "output-changed";
  }
  static get autoBinding() {
    return {
      signal: "output-changed",
      field: "invoke"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.__initializeValue();
  }
  __initializeValue() {
    const dbResult = this._readDBResult();
    if (dbResult != null) {
      this.set("input", dbResult);
    }
  }
  destroy() {
    this._clearDBResult(this.table);
    this.set("input", null);
  }
  __storeCompute(mode) {
    const store = {
      session: window.sessionStorage,
      failover: window.localStorage,
      local: window.localStorage
    };
    return store[mode] || store["failover"];
  }
  __tableChanged(_table, _tableOld) {
    const carryOverStore = this._readDBResult(_tableOld);
    if (carryOverStore) {
      this._clearDBResult(_tableOld);
      this._writeDBResult(carryOverStore, _table);
    }
  }
  _inputChanged(input) {
    this._writeDBResult(input);
  }
  _readDBResult(table = this.table) {
    if (!table) {
      return void 0;
    }
    const dbResult = this.__store.getItem(table);
    try {
      return JSON.parse(dbResult);
    } catch (e) {
      ess.ele(this, {
        status: "failure",
        action: "read",
        data: dbResult
      });
    }
    return dbResult;
  }
  _writeDBResult(dbResult, table = this.table) {
    if (!table) {
      return this;
    }
    try {
      this.__store.setItem(table, JSON.stringify(dbResult));
    } catch (e) {
      ess.ele(this, {
        status: "failure",
        action: "write",
        data: dbResult
      });
    }
    return this;
  }
  _clearDBResult(table = this.table) {
    try {
      this.__store.removeItem(table);
    } catch (e) {
      ess.ele(this, {
        status: "failure",
        action: "clear"
      });
    }
  }
}
HIP.factory.element(StoreElement);
