
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import {
  BehaviorBrowserEvents,
  AsHIC,
  BehaviorA11yKeyboard,
  BehaviorElement,
  BehaviorInteractiveElement,
  HIP
} from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorTabGroup } from '../../../eip/element/hi-behavior-tab/hi-behavior-tab-group.js';
import '../../../eip/element/hi-behavior-set-select/hi-behavior-set-select.js';
import '../../../eip/element/hi-component/hi-behavior-browser-events.js';
import './hi-tab.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
export class TabGroupElement extends BehaviorTabGroup(
  BehaviorBrowserEvents(BehaviorA11yKeyboard(BehaviorInteractiveElement(BehaviorElement(AsHIC(PolymerElement)))))
) {
  static get is() {
    return "hi-tab-group";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          display: block;
          width: 100%;
          text-align: center;
        }

        :host(:not([aesthetic='stacked'])) #container {
          overflow-x: auto;
          overflow-y: hidden;
          scrollbar-width: thin;
        }

        :host ::-webkit-scrollbar {
          height: var(--hi-px-4);
          background: var(--hi-clr--gray-2);
        }

        :host ::-webkit-scrollbar-thumb {
          background: var(--hi-clr--gray-7);
        }

        :host([aesthetic='stacked']) {
          max-width: 100%;
        }

        #container {
          display: flex;
        }

        :host([aesthetic='wrap']) #container {
          display: block;
        }

        :host([aesthetic='stacked']) #container {
          flex-direction: column;
          margin-left: auto;
          display: flex;
        }

        :host([stretched]) #container ::slotted(hi-tab) {
          flex: 1 1;
        }

        :host(:focus) {
          outline: none;
        }

        #container ::slotted(hi-tab[focused]) {
          outline: none;
        }

        :host(:focus) #container ::slotted(hi-tab[focused]:not([disabled])) {
          outline: var(--hi-outline--width) var(--hi-outline--line) var(--hi-clr--outline);
          outline-offset: calc(-1 * var(--hi-outline--offset));
          z-index: 100;
        }

        :host([disabled]) #container ::slotted(*) {
          color: var(--hi-clr--disabled-text);
          opacity: 0.8;
        }

        :host(:not([aesthetic='stacked'])) #container ::slotted(hi-tab) {
          text-align: center;
          white-space: nowrap;
        }

        :host([aesthetic='stacked']) #container ::slotted(hi-tab) {
          text-align: left;
        }

        :host(:not([aesthetic='stacked'])) #container ::slotted(hi-tab:not(:last-of-type)) {
          border-right: 0;
        }

        :host([aesthetic='stacked']) #container ::slotted(hi-tab:not(:last-of-type)) {
          border-bottom: 0;
        }

        :host(:not([aesthetic='stacked'])) #container ::slotted(hi-tab[selected]) {
          border-top: var(--hi-px-3) solid var(--hi-clr--theme-3);
          border-bottom-color: var(--hi-clr--transparent);
          padding-top: var(--hi-px-8);
        }

        :host([aesthetic='stacked']) #container ::slotted(hi-tab[selected]) {
          border-left: var(--hi-px-3) solid var(--hi-clr--theme-3);
          border-right: none;
        }

        :host([size='small']) #container ::slotted(hi-tab) {
          font-size: var(--hi-px-13) !important;
          padding: var(--hi-px-4) var(--hi-px-12);
        }
      </style>

      <div id="container">
        <slot id="items"></slot>
      </div>
    `;
  }
  static get properties() {
    return {
      stretched: {
        type: Boolean
      },
      aesthetic: {
        type: String,
        reflectToAttribute: true,
        value: "standard"
      }
    };
  }
  static get api() {
    return ["shiftFocusDown", "shiftFocusUp", "focusIndex", "focusItem", "selectIndex", "selectItem"];
  }
  static get keyBindings() {
    return {
      "space:keydown": "_onKeyAny__preventDefault",
      "space:keyup": "_setSelect__onKeyUp_space",
      "enter:keyup": "_setSelect__onKeyUp_enter",
      "up:keydown": "_onKeyAny__preventDefault",
      "down:keydown": "_onKeyAny__preventDefault",
      "left:keydown": "_onKeyAny__preventDefault",
      "right:keydown": "_onKeyAny__preventDefault",
      "right:keyup": "_setFocus__onKeyUp_right",
      "down:keyup": "_setFocus__onKeyUp_down",
      "left:keyup": "_setFocus__onKeyUp_left",
      "up:keyup": "_setFocus__onKeyUp_up"
    };
  }
  static get browserBindings() {
    return {
      click: "_setSelect__onTap",
      focus: "_setFocus__onFocus"
    };
  }
  static get autoBinding() {
    return {
      signal: "selected-index-changed",
      field: "selectedIndex"
    };
  }
  static get observers() {
    return ["__aestheticWatcher(aesthetic)"];
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "tablist");
  }
  __aestheticWatcher(_aesthetic) {
    dom.flipAttr(this, "aria-orientation", _aesthetic === "stacked", "vertical");
  }
  __onSlotChanged() {
    super._onSlotChanged();
  }
}
export default HIP.factory.element(TabGroupElement);
