
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorBrowserEvents, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorItemSelect } from '../../../eip/element/hi-behavior-set-select/hi-behavior-item-select.js';
import { BehaviorTab } from '../../../eip/element/hi-behavior-tab/hi-behavior-tab.js';
export class TabElement extends BehaviorTab(
  BehaviorItemSelect(BehaviorBrowserEvents(BehaviorElement(AsHIC(PolymerElement))))
) {
  static get is() {
    return "hi-tab";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          position: relative;
          display: block;
          width: auto;
          padding: var(--hi-px-8) var(--hi-px-16);
          color: var(--hi-clr--text);
          letter-spacing: calc(var(--hi-px-1) / 2);
          font-size: var(--hi-font-size--text);
          line-height: var(--hi-font-lh--large);
          font-weight: var(--hi-font--weight-light);
          cursor: default;
          border: var(--hi-px-1) solid var(--hi-clr--gray-3);
          background-color: var(--hi-clr--gray-1);
          cursor: default;
          padding-top: var(--hi-px-10);
        }

        :host([size='small']) {
          font-size: var(--hi-px-13) !important;
          padding: var(--hi-px-4) var(--hi-px-12);
        }

        :host([selected]) {
          z-index: 1;
          font-weight: var(--hi-font--weight-bold);
          background-color: var(--hi-clr--white);
          color: var(--hi-clr--theme-5);
        }

        @media (hover: hover) {
          :host(:hover) {
            color: var(--hi-clr--text);
            background-color: var(--hi-clr--gray-2);
            text-shadow: var(--hi-clr--text-shadow-1);
          }

          :host([selected]:hover) {
            background-color: var(--hi-clr--gray-3);
            text-shadow: none;
          }
        }

        :host([disabled]),
        :host([disabled]) #container {
          opacity: 0.8;
          color: var(--hi-clr--disabled-text);
        }

        #container {
          display: flex;
          justify-content: space-between;
          width: 100%;
        }
      </style>
      <span id="container">
        <slot id="content"></slot>
      </span>
    `;
  }
  static get autoBinding() {
    return {
      signal: "selected-changed",
      field: "selected"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "tab");
  }
}
HIP.factory.element(TabElement);
