
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { AsLitHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorSetGroup } from '../../../eip/element/hi-behavior-group/hi-behavior-set-group.js';
import { BehaviorContainer } from '../../../eip/element/hi-behavior-container/hi-behavior-container.js';
import '../../../eip/element/hi-behavior-container/hi-behavior-container.js';
import '../../../eip/element/hi-behavior-group/hi-behavior-set-group.js';
import '../../../eip/element/hi-component/hi-component.js';
import './hi-tag.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export const styles = css`
  :host {
    display: flex;
  }

  #container {
    flex: 1;
    display: flex;
    flex-flow: row wrap;
    justify-content: flex-start;
    align-self: center;
    gap: var(--hi-px-2);
  }
`;
export class TagGroupElement extends BehaviorSetGroup(BehaviorContainer(BehaviorElement(AsLitHIC(LitElement)))) {
  constructor() {
    super();
    this.groupName = "tag";
  }
  static get is() {
    return "hi-tag-group";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      values: {
        type: Array,
        notify: true
      },
      selectedCount: {
        type: Number,
        attribute: "selected-count",
        reflect: true,
        notify: true
      }
    };
  }
  static get api() {
    return ["removeAll", "closeTag"];
  }
  static get autoBinding() {
    return {
      signal: "values-changed",
      field: "values"
    };
  }
  async performUpdate() {
    this.selectedCount = this.__selectedCountCompute(this.setChildren);
    this.values = this.__valuesCompute(this.setChildren);
    return super.performUpdate();
  }
  render() {
    return html`
      <div id="container">
        <slot id="items"></slot>
      </div>
    `;
  }
  removeAll() {
    return this._group__groupSet().forEach((item) => {
      return item.remove();
    });
  }
  closeTag(value) {
    return f1.remove((val) => value == val, this.values);
  }
  __selectedCountCompute(setChildren) {
    return setChildren.length;
  }
  __valuesCompute(setChildren) {
    const valueAttr = this.valueAttr;
    return setChildren.map((item) => {
      return item[valueAttr];
    });
  }
  _onSlotChanged() {
    this._group__slotChange();
  }
}
export default HIP.factory.element(TagGroupElement);
