
import { beforeNextRender } from '../../../hip/library/polymer/polymer.js';
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { stylesTagBadge } from '../../../eip/element/hi-behavior-style/hi-behavior-style.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorItemGroup } from '../../../eip/element/hi-behavior-group/hi-behavior-item-group.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import '../../../eip/element/hi-button/hi-close.js';
import '../../../eip/element/hi-icon/hi-icon.js';
export const styles = css`
  :host {
    --hi-tag--icon-min-width: var(--hi-px-16);
  }

  hi-icon {
    margin-left: var(--hi-px-2);
    margin-right: var(--hi-px-2);
    fill: var(--hi-clr--theme-5);
    min-width: var(--hi-tag--icon-min-width);
  }

  #closeButton {
    width: var(--hi-px-20);
    height: var(--hi-px-20);
    border-radius: var(--hi-shape--rounded);
    color: var(--hi-clr--theme-5);
    margin-left: var(--hi-px-2);
  }

  :host([size='large']) #closeButton {
    min-width: var(--hi-px-24) !important;
    margin-left: var(--hi-px-4);
  }

  :host([size='large']) hi-icon {
    margin-right: var(--hi-px-8);
  }

  :host([allow-close]),
  :host([allow-remove]) {
    padding: var(--hi-px-1) var(--hi-px-4);
    padding-right: var(--hi-px-1);
  }

  :host([aesthetic='dark']) #closeButton {
    color: var(--hi-clr--white);
  }

  :host([aesthetic='dark']) #closeButton:hover {
    border-color: var(--hi-clr--white);
  }

  :host([aesthetic='dark'][theme='warning']) #closeButton,
  :host([aesthetic='dark'][theme='orange']) #closeButton,
  :host([aesthetic='dark'][theme='yellow']) #closeButton,
  :host([aesthetic='dark'][theme='teal']) #closeButton,
  :host([theme='warning']) #closeButton,
  :host([theme='orange']) #closeButton,
  :host([theme='yellow']) #closeButton,
  :host([theme='teal']) #closeButton {
    color: var(--hi-clr--text);
  }

  :host([aesthetic='dark'][theme='warning']) #closeButton:hover,
  :host([aesthetic='dark'][theme='orange']) #closeButton:hover,
  :host([aesthetic='dark'][theme='yellow']) #closeButton:hover,
  :host([aesthetic='dark'][theme='teal']) #closeButton:hover,
  :host([theme='warning']) #closeButton:hover,
  :host([theme='orange']) #closeButton:hover,
  :host([theme='yellow']) #closeButton:hover,
  :host([theme='teal']) #closeButton:hover {
    border-color: var(--hi-clr--text);
  }
`;
export class TagElement extends BehaviorItemGroup(BehaviorElement(AsHIC(LitElement))) {
  constructor() {
    super();
    this.groupName = "tag";
  }
  static get is() {
    return "hi-tag";
  }
  static get styles() {
    return [stylesTheme, stylesTagBadge, styles];
  }
  static get properties() {
    return {
      icon: {
        type: String
      },
      allowClose: {
        type: Boolean,
        attribute: "allow-close"
      },
      allowRemove: {
        type: Boolean,
        attribute: "allow-remove"
      },
      value: {
        type: Object,
        reflect: true
      }
    };
  }
  static get api() {
    return ["close"];
  }
  static get autoBinding() {
    return {
      signal: "close-ended",
      field: "close"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this._includeQAMarker();
  }
  render() {
    return html`
      ${this.__renderIcon()}
      <span id="content"><slot></slot></span>
      ${this.__renderClose()}
    `;
  }
  __renderIcon() {
    if (!this.icon) {
      return nothing;
    }
    return html`<hi-icon kind="${this.icon}" .theme="${this.theme}" ?disabled="${this.disabled}"> </hi-icon>`;
  }
  __renderClose() {
    if (!this.doesAllowClose()) {
      return nothing;
    }
    const ariaLabelText = this.getAttribute("data-qa") || this.value || "tag";
    return html`<hi-close
      id="closeButton"
      .theme="${this.theme}"
      ?disabled="${this.disabled}"
      ?readonly="${this.readonly}"
      @trigger-ended="${this.__onCloseTriggerEnded}"
      aria-label="Close ${ariaLabelText}"
    ></hi-close>`;
  }
  close() {
    if (this.allowRemove) {
      beforeNextRender(this, function() {
        this._dispatchProcess("remove", {});
        this.remove();
      });
    }
    const parentEl = this.parentElement;
    if (parentEl) {
      parentEl.closeTag(this.value);
    }
    return this.value;
  }
  doesAllowClose() {
    return this.allowClose || this.allowRemove;
  }
  __onCloseTriggerEnded(e) {
    e.stopPropagation();
    this.close();
  }
}
export default HIP.factory.element(TagElement);
