
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import { HIP, AsHIC, BehaviorElement } from '../../../eip/element/hi-hip/hi-hif.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class TaskGroupElement extends BehaviorElement(AsHIC(PolymerElement)) {
  static get is() {
    return "hi-task-group";
  }
  static get properties() {
    return {
      kind: {
        type: String,
        value: "serial"
      },
      for: {
        type: String
      }
    };
  }
  static get api() {
    return ["invoke"];
  }
  static get autoBinding() {
    return {
      signal: "invoke-ended",
      field: "invoke"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("hidden", "hidden");
  }
  invoke(input) {
    const taskEl = HIP.browser.queryOne(this.for, this.ns, void 0);
    if (!taskEl) {
      return Promise.reject(`no tasks`);
    }
    const invokeTaskStrategy = {
      race: () => {
        return Promise.race(f1.map((objectInArray) => f1.task(taskEl, objectInArray), input));
      },
      parallel: () => {
        return f1.parallel(
          f1.map((objectInArray) => {
            return () => f1.task(taskEl, objectInArray);
          }, input),
          taskEl
        );
      },
      serial: () => {
        return f1.serial(
          f1.map((objectInArray) => {
            return () => f1.task(taskEl, objectInArray);
          }, input),
          taskEl
        );
      },
      fallback: () => {
        return f1.done(input);
      }
    };
    return f1.strategy(invokeTaskStrategy, this.kind)();
  }
}
HIP.factory.element(TaskGroupElement);
