
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { AsLitHIC, BehaviorBrowserEvents, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { BehaviorFormat } from '../../../eip/element/hi-behavior-format/hi-behavior-format.js';
import { IteratorElement } from '../../../eip/element/hi-iterator/hi-iterator.js';
import '../../../eip/element/hi-tooltip/hi-tooltip.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
const regexMatchAllCharacters = /./g;
export const styles = css`
  :host {
    font-weight: var(--hi-font--weight-light);
    line-height: var(--hi-font-lh--text);
    font-size: var(--hi-font-size--text);
  }

  :host([formatter='json']) {
    font-family: monospace;
    white-space: pre-line;
    /* to compensate for the newlines added by pre-line */
    margin-top: -4.5ch;
    display: block;
  }

  :host([allow-ellipsis]) {
    overflow: hidden;
    text-overflow: ellipsis;
    white-space: nowrap;
  }
`;
const _TextElement = class extends BehaviorFormat(BehaviorBrowserEvents(BehaviorElement(AsLitHIC(LitElement)))) {
  static get is() {
    return "hi-text";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      disallowHint: {
        type: Boolean,
        reflect: true,
        attribute: "disallow-hint"
      },
      allowEllipsis: {
        type: Boolean,
        reflect: true,
        attribute: "allow-ellipsis"
      },
      generator: {
        type: String
      },
      value: {
        type: Object,
        notify: true,
        converter: (value) => {
          if (value === null) {
            return null;
          } else {
            try {
              return JSON.parse(value);
            } catch (e) {
              return `${value}`;
            }
          }
        }
      },
      valueDictionary: {
        type: Object
      },
      secure: {
        type: Boolean,
        reflect: true
      }
    };
  }
  static get browserBindings() {
    return {
      mouseenter: "__onMouseEnter"
    };
  }
  static get autoBinding() {
    return {
      signal: "value-changed",
      field: "value"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.__forceHiddenStyling();
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    this.__destroyExpandedText();
  }
  render() {
    return html`<slot></slot>`;
  }
  updated(props) {
    if (props.has("allowEllipsis")) {
      this.__allowEllipsisWatcher(this.allowEllipsis);
    }
    if (props.has("hidden")) {
      this.__forceHiddenStyling(this.hidden);
    }
    if (props.has("secure")) {
      this.__secureWatcher(this.secure);
    }
    if (props.has("generator")) {
      this.__generatorWatcher();
    }
    if (!f1.is.nil(this.value)) {
      this.__textFormatterWatcher();
    }
  }
  __secureWatcher(secure) {
    if (secure) {
      const content = this.content();
      if (f1.is.nil(content)) {
        return;
      }
      this.innerHTML = f1.replace(regexMatchAllCharacters, "*", content);
    }
  }
  __allowEllipsisWatcher(allowEllipsis) {
    const parentEl = this.parentElement;
    if (parentEl) {
      dom.flipAttr(parentEl, "disallow-ellipsis", allowEllipsis);
    }
  }
  __forceHiddenStyling(hidden = this.hidden) {
    this.style.display = hidden ? "none !important" : "inline !important";
  }
  __textFormatterWatcher() {
    const content = this.content();
    if (null == content) {
      return;
    }
    const secure = this.secure;
    if (secure) {
      return this.__secureWatcher(secure);
    }
    const formatterFn = f1.read(`/formatterFn/${this.formatter}`, f1.skip, _TextElement);
    this.innerHTML = formatterFn ? formatterFn(content) : content;
  }
  __generatorWatcher() {
    this.valueDictionary = IteratorElement.generatorDict(this.generator);
  }
  __onMouseEnter() {
    this.__attachTooltip();
  }
  __attachTooltip() {
    const isDisallowedHint = this.disallowHint;
    if (isDisallowedHint) {
      return;
    }
    const isDisallowedEllipsis = !this.allowEllipsis;
    if (isDisallowedEllipsis) {
      return;
    }
    const isAllTextVisible = !this.isTextCutOff();
    if (isAllTextVisible) {
      return;
    }
    const hasTooltip = this.__tooltipEl;
    if (hasTooltip) {
      return;
    }
    const alreadyHasTooltip = this.nextElementSibling && this.nextElementSibling.localName === "hi-tooltip";
    if (alreadyHasTooltip) {
      return;
    }
    this.__tooltipEl = this.__createExpandedText();
    this.addEventListener("mouseleave", () => this.__onMouseLeave(), {
      once: true
    });
  }
  __createExpandedText() {
    const textId = `unique_text_${f1.id()}`;
    const tooltipEl = dom.create(null, "hi-tooltip", {
      placement: "top",
      opened: true,
      for: textId,
      ns: this.ns
    });
    tooltipEl.textContent = `${this.textContent}`;
    this.gid = textId;
    this.insertAdjacentElement("afterend", tooltipEl);
    return tooltipEl;
  }
  __onMouseLeave() {
    this.__destroyExpandedText();
  }
  __destroyExpandedText() {
    const tooltipEl = this.__tooltipEl;
    if (!tooltipEl) {
      return;
    }
    this.__tooltipEl = void 0;
    const parentEl = tooltipEl.parentElement || (tooltipEl.getRootNode()["host"] || {}).shadowRoot;
    if (parentEl) {
      parentEl.removeChild(tooltipEl);
    }
  }
  isTextCutOff() {
    return dom.is.scrollable(this);
  }
  content() {
    const value = f1.is.nil(this.value) ? this.innerHTML : this.value;
    if (f1.is.nil(value)) {
      return null;
    }
    const safeDictionary = f1.safe({}, this.valueDictionary);
    const assignValue = f1.read(`/${value}`, value, safeDictionary);
    return HIP.fn.format(assignValue, this.formatter, this.formatterRule);
  }
};
export let TextElement = _TextElement;
TextElement.formatterFn = {
  json(content) {
    return `<pre>${content}</pre>`;
  }
};
export default HIP.factory.element(TextElement);
