
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorA11yKeyboard, BehaviorElement, a11yLabel, HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorInputEditable } from '../../../eip/element/hi-behavior-input/hi-behavior-input-editable.js';
import { BehaviorMirror } from '../../../eip/element/hi-behavior-mirror/hi-behavior-mirror.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class TextareaElement extends BehaviorInputEditable(
  BehaviorMirror(BehaviorA11yKeyboard(BehaviorElement(AsHIC(PolymerElement))))
) {
  constructor() {
    super();
    this.__mutex__ = false;
    this.__characterCount = 0;
    this.__remainingCharacters = "unlimited";
    this.rows = 2;
    this.cols = 20;
    this.wrap = "soft";
    this.aesthetic = "open";
    this.__remainingCharacters = this.maxlength || "unlimited";
  }
  static get is() {
    return "hi-textarea";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          --hi-textarea--height: calc(var(--hi-el--height) * 2);
          --hi-textarea--border-color: var(--hi-clr--theme-3);
          --hi-textarea--color: var(--hi-clr--gray-9);
          --hi-textarea--placeholder-color: var(--hi-clr--gray-7);
        }

        :host {
          width: auto;
          display: block;
          border-radius: var(--hi-shape--radius);
          background-color: var(--hi-clr--transparent);
          transition: box-shadow 0.2s cubic-bezier(0.4, 0, 1, 1);
        }

        :host([focused]),
        :host(:focus) {
          outline: none;
        }

        #textAreaGroup {
          display: flex;
          outline: none;
          align-items: center;
          min-height: var(--hi-textarea--height);
          background-color: var(--hi-clr--white);
          border-radius: var(--hi-shape--radius);
          border: var(--hi-px-1) solid var(--hi-textarea--border-color);
          transition: box-shadow 0.2s cubic-bezier(0.4, 0, 0.2, 1);
        }

        #textAreaGroup:focus,
        #textAreaGroup:focus-within,
        :host([focused]) #textAreaGroup {
          outline: var(--hi-outline--width) var(--hi-outline--line) var(--hi-clr--outline);
          outline-offset: var(--hi-outline--offset);
        }

        :host([validated][invalid]):not([silent-validate]) #textAreaGroup,
        :host([invalid]:not([focused]):not([silent-validate])) #textAreaGroup {
          border-color: var(--hi-clr--red-5);
        }

        @media (hover: hover) {
          :host(:hover:not(:disabled):not([disabled]):not([readonly])) #textAreaGroup {
            box-shadow: var(--hi-clr--halo-2);
          }
        }

        :host([readonly]) {
          --hi-textarea--border-color: var(--hi-clr--gray-ax);
          --hi-textarea--color: var(--hi-clr--text);
        }

        :host([readonly]) #inputGroup,
        :host([disabled]) #inputGroup {
          pointer-events: none !important;
        }

        :host([disabled]) {
          --hi-textarea--border-color: var(--hi-clr--gray-ax);
          --hi-textarea--color: var(--hi-clr--gray-7);
          pointer-events: all !important;
        }

        :host([disabled]) :slotted(*) {
          opacity: 1;
        }

        #input {
          -moz-appearance: none;
          -webkit-appearance: none;
          border-radius: 0;
          border: 0;
          box-shadow: none;
          color: var(--hi-textarea--color);
          display: block;
          flex-grow: 1;
          font-family: inherit;
          font-weight: var(--hi-font--weight-light);
          font-size: var(--hi-font-size--text);
          min-height: calc(var(--hi-el--height) * 2);
          line-height: 1.25;
          margin: 0;
          outline: none;
          padding: var(--hi-px-8);
          text-align: inherit;
          width: 100%;
          height: 100%;
          background-color: var(--hi-clr--transparent);
        }

        #counter {
          text-align: right;
          color: var(--hi-clr--gray-8);
        }

        :host(:not([maxlength])) #counter,
        :host(:not([maxlength])) #counterAXLabel {
          display: none;
        }

        #input::placeholder,
        #input:placeholder-shown {
          color: var(--hi-textarea--placeholder-color);
          font-weight: var(--hi-font--weight-light);
          opacity: 1;
        }

        #input::placeholder {
          font-style: normal;
        }

        :host([validated][invalid]):not([silent-validate]),
        :host([dirty][invalid]:not([focused]):not([silent-validate])) {
          border-color: var(--hi-clr--error);
        }

        :host([aesthetic='inline']) {
          display: inline-block;
          width: auto;
        }

        :host([aesthetic='code']) #input {
          font-family: monospace;
        }
      </style>

      <div id="textAreaGroup">
        <textarea
          id="input"
          mixin="input-placeholder"
          autocomplete$="[[autocomplete]]"
          autocorrect$="[[autocorrect]]"
          autofocus$="[[autofocus]]"
          disabled$="[[disabled]]"
          maxlength$="[[maxlength]]"
          minlength$="[[minlength]]"
          placeholder$="[[placeholder]]"
          readonly$="[[readonly]]"
          required$="[[required]]"
          value="[[value::textarea]]"
          name$="[[name]]"
          rows$="[[rows]]"
          cols$="[[cols]]"
          wrap$="[[wrap]]"
          spellcheck$="[[spellcheck]]"
          inputmode$="[[inputmode]]"
          on-change="__onChange_input"
          on-focus="__onFocus_input"
          on-blur="__onBlur_input"
          on-input="__onInput_input"
        ></textarea>
      </div>
      <p id="counter">{{__characterCount}} / [[maxlength]]</p>
      <span id="counterAXLabel" mixin="show-ax" aria-live="polite"
        >You have [[__remainingCharacters]] characters remaining</span
      >
    `;
  }
  static get properties() {
    return {
      autocorrect: {
        type: String
      },
      autofocus: {
        type: Boolean,
        value: false
      },
      ariaLabel: {
        type: String
      },
      ariaLabelledby: {
        type: String
      },
      ariaDescribedby: {
        type: String
      },
      label: {
        type: String
      },
      inputmode: {
        type: String
      },
      value: {
        type: Object,
        observer: "__valueChanged",
        notify: true
      },
      rows: {
        type: Number
      },
      cols: {
        type: Number
      },
      wrap: {
        type: String
      },
      spellcheck: {
        type: Boolean
      },
      aesthetic: {
        type: String,
        reflectToAttribute: true
      }
    };
  }
  static get valueEvent() {
    return "value-changed";
  }
  static get autoBinding() {
    return {
      signal: "value-changed",
      field: "value"
    };
  }
  get inputMirrorType() {
    return "textarea";
  }
  static get observers() {
    return ["_ariaLabelTextWatcher(ariaLabel, ariaLabelledby, ariaDescribedby)"];
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute(`role`, `textbox`);
    this.setAttribute(`aria-multiline`, `true`);
    this.setAttribute("focusable", "focusable");
    this.disallowEnter = true;
    this.addEventListener("keyup", this.__onKeyUp_input);
  }
  __valueChanged(value) {
    if (this.__mutex__) {
      return;
    }
    this.__mutex__ = true;
    const textValue = value == null ? null : f1.format("textarea", {}, value);
    this.__reflectCodeAesthetic(textValue);
    super._inputEditable_valueChanged(textValue);
    if (this.maxlength > 0) {
      this.__updateCharacterCounter(textValue);
    }
    this.__mutex__ = false;
  }
  __reflectCodeAesthetic(textValue) {
    const firstCharacter = `${textValue || ""}`[0];
    if (firstCharacter === "[" || firstCharacter === "{") {
      this.aesthetic = "code";
    } else if (this.aesthetic === "code") {
      this.aesthetic = "open";
    }
  }
  __onKeyUp_input(event) {
    const minNoLinesForMultiLine = 1;
    const multilineEnabled = this.rows > minNoLinesForMultiLine;
    if (multilineEnabled && event.key === "Enter") {
      event.stopPropagation();
    }
    super._inputEditable_onKeyUp_enter();
  }
  __onInput_input(event) {
    super._inputEditable_onInput_input(event);
  }
  __onChange_input(event) {
    super._inputEditable_onChange_input(event);
  }
  __onFocus_input(event) {
    super._inputEditable_onFocus_input();
    this.focused = true;
    this.$.input.setAttribute("focused", "focused");
    this.setAttribute("focused", "focused");
    this._dispatchEvent(`focus`, event);
  }
  __onBlur_input(event) {
    super._inputEditable_onBlur_input(event);
    this.$.input.removeAttribute("focused");
  }
  __updateCharacterCounter(value) {
    this.__characterCount = (value || "").length;
    this.__remainingCharacters = this.maxlength - this.__characterCount;
  }
  _ariaLabelTextWatcher(label, labelledBy, describedBy) {
    a11yLabel.writeA11yLabel(this.$.input, label, labelledBy, describedBy, { ns: this.ns });
  }
}
HIP.factory.element(TextareaElement);
