
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
export class TimerElement extends BehaviorElement(AsHIC(PolymerElement)) {
  constructor() {
    super(...arguments);
    this.time = 0;
    this.count = 0;
    this.delay = 1e3;
  }
  static get is() {
    return "hi-timer";
  }
  static get properties() {
    return {
      time: {
        type: Number,
        reflectToAttribute: true,
        notify: true,
        computed: "__timeCompute(count, delay)"
      },
      count: {
        type: Number,
        reflectToAttribute: true,
        notify: true,
        observer: "__countChanged"
      },
      auto: {
        type: Boolean,
        observer: "__autoChanged"
      },
      delay: {
        type: Number
      },
      output: {
        type: Number,
        notify: true,
        computed: "__outputCompute(time)"
      },
      argument: {
        type: Object
      }
    };
  }
  static get api() {
    return ["restart", "invoke", "start", "reset", "stop", "next"];
  }
  static get autoBinding() {
    return {
      signal: "next-ended",
      field: "invoke"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("exclude", "exclude");
    this.setAttribute("hidden", "hidden");
    this.style.display = "none";
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    this.stop();
  }
  __autoChanged(auto) {
    if (auto) {
      this.restart();
    }
  }
  __timeCompute(count, delay) {
    return 1 * delay * count;
  }
  __outputCompute(time) {
    return { time };
  }
  __countChanged(_count, _countOld) {
    if (_count === _countOld) {
      this.count = _count + 1;
    }
  }
  invoke(argument) {
    this.argument = argument;
    HIP.data.applyExpressions(this);
    return this.restart();
  }
  restart() {
    this.stop();
    return this.start();
  }
  stop() {
    this.pause();
    return this.reset();
  }
  pause() {
    clearInterval(this.__autoInterval);
    return this.count;
  }
  reset() {
    this.__autoInterval = void 0;
    return this.count = this.count === 0 ? -1 : 0;
  }
  start() {
    this.__autoInterval = this.__autoInterval != null ? this.__autoInterval : setInterval(() => this.next(), this.delay);
    return this.output;
  }
  next() {
    this.__next();
    return this.output;
  }
  __next() {
    return this.count = this.count + 1;
  }
  toString() {
    return `time: ${this.time}`;
  }
}
HIP.factory.element(TimerElement);
