
import dayjs from '../../../lip/library/li-dayjs/li-dayjs.js';
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { render } from '../../../hip/library/lit-html/lit-html.js';
import { HIP, AsLitHIC, BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import '../../../eip/element/hi-select/hi-select.js';
import '../../../eip/element/hi-select/hi-option.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class TimesElement extends BehaviorElement(AsLitHIC(LitElement)) {
  constructor() {
    super(...arguments);
    this.timeStart = "8:00 AM";
    this.timeStep = 30;
    this.timeEnd = "5:00 PM";
    this.timeDuration = 0;
    this.value = "";
    this.timeFormat = "h:mm A";
  }
  static get is() {
    return "hi-times";
  }
  static get properties() {
    return {
      value: {
        type: String
      },
      timeStart: {
        type: String,
        reflect: true,
        attribute: "time-start"
      },
      timeStep: {
        type: Number,
        reflect: true,
        attribute: "time-step"
      },
      timeEnd: {
        type: String,
        reflect: true,
        attribute: "time-end"
      },
      timeDuration: {
        type: Number,
        reflect: true,
        attribute: "time-duration"
      },
      timeFormat: {
        type: String,
        reflect: true,
        attribute: "time-format"
      },
      timeZone: {
        type: String,
        reflect: true,
        attribute: "time-zone"
      }
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("exclude", "exclude");
    this.setAttribute("hidden", "hidden");
  }
  updated(changedProperties) {
    const parentElement = this.parentElement;
    if (!parentElement) {
      return;
    }
    if (this.value) {
      this.value = f1.format("iso_time", { raw_format: this.timeFormat, time_zone: this.timeZone }, this.value);
      parentElement.value = this.value;
    }
    const renderOptions = ["timeStart", "timeEnd", "timeFormat", "timeDuration", "timeZone"];
    changedProperties.forEach((_, prop) => {
      if (renderOptions.includes(prop)) {
        render(this._render(), parentElement);
      }
    });
  }
  _render() {
    const times = this.__calculateTimes();
    const optionsList = this.__renderOptionsList(times);
    return optionsList;
  }
  __calculateTimes() {
    const times = [];
    const maxIterations = 250;
    let iterations = 0;
    let minTime = f1.parse("iso_time", { time_zone: this.timeZone }, this.timeStart);
    let maxTime = f1.parse("iso_time", { time_zone: this.timeZone }, this.timeEnd);
    maxTime = dayjs(maxTime).add(this.timeStep, "minute");
    let duration = this.timeDuration > 0 ? this.timeDuration : maxTime.diff(minTime, "minute") / this.timeStep;
    while (duration > 0 && iterations < maxIterations) {
      times.push(f1.format("iso_time", { raw_format: this.timeFormat }, minTime));
      minTime = dayjs(minTime).add(this.timeStep, "minute");
      iterations += 1;
      duration -= 1;
    }
    return times;
  }
  __renderOptionsList(times) {
    return times.map((time) => html`<hi-option value=${time}>${time}</hi-option>`);
  }
  createRenderRoot() {
    return this;
  }
}
HIP.factory.element(TimesElement);
export default TimesElement.is;
