
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { AsLitHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
export const styles = css`
  :host {
    --padding: var(--hi-px-32) var(--hi-px-8);
  }

  #headerTitle {
    margin-right: var(--hi-px-24);
  }

  #closeButton {
    z-index: var(--hi-depth--above-all);
    background: var(--hi-clr--white);
    position: absolute;
    top: var(--hi-px-8);
    right: var(--hi-px-8);
    z-index: var(--hi-depth--above-all);
    overflow: hidden;
    color: var(--hi-clr--text);
    min-width: var(--hi-close--size);
    border-radius: var(--hi-px-2);
  }

  #closeButton:focus {
    outline: var(--hi-outline--width) var(--hi-outline--line) var(--hi-clr--outline);
    outline-offset: var(--hi-outline--offset);
  }

  @media only screen and (max-width: 40rem) {
    :host(:not([disallow-responsiveness])) {
      display: block;
      padding: var(--padding);
      background: var(--hi-clr--white);
      border-top-left-radius: var(--hi-px-12) var(--hi-px-12);
      border-top-right-radius: var(--hi-px-12) var(--hi-px-12);
      border-bottom: var(--hi-px-1) solid var(--hi-clr--gray-6);
    }

    :host([has-title]) {
      padding: var(--hi-px-20) var(--hi-px-8);
    }

    :host([disallow-responsiveness]) {
      display: none;
    }
  }
`;
export class SubtitleElement extends BehaviorElement(AsLitHIC(LitElement)) {
  static get is() {
    return "hi-subtitle";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      title: {
        type: String
      }
    };
  }
  static get api() {
    return ["close"];
  }
  static get autoBinding() {
    return {
      signal: "close-ended",
      field: "close"
    };
  }
  render() {
    return html`
      <p aria-hidden="true" id="headerTitle">${this.title}</p>
      <hi-close
        id="closeButton"
        size="small"
        device="mobile"
        @trigger-ended=${this.__closeButton__onTriggerEnded}
      ></hi-close>
    `;
  }
  close() {
  }
  updated(props) {
    super.updated(props);
    if (props.has("title")) {
      dom.flipAttr(this, "has-title", !!this.title);
    }
  }
  __closeButton__onTriggerEnded(e) {
    e.stopPropagation();
    this.close();
  }
}
export default HIP.factory.element(SubtitleElement);
