
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { AsLitHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import $ip from '../../../pip/library/pi-ip/pi-ip.js';
export class TitleElement extends BehaviorElement(AsLitHIC(LitElement)) {
  constructor() {
    super(...arguments);
    this.level = 1;
  }
  static get is() {
    return "hi-title";
  }
  static get properties() {
    return {
      level: {
        type: Number
      },
      title: {
        type: String
      }
    };
  }
  static get autoBinding() {
    return {
      field: "title",
      signal: ""
    };
  }
  firstUpdated() {
    const flag__scrollToTop = $ip.flags.is("/eip/hi-title/scroll-to-top");
    if (flag__scrollToTop) {
      if (window.scrollY !== 0 && this.level <= 1) {
        window.scrollTo({
          top: 0,
          left: 0,
          behavior: "smooth"
        });
      }
    }
  }
  render() {
    const level = this.level;
    const title = this.title;
    return html`
      ${1 >= level ? html`<h1>${title}</h1>` : nothing}
      ${2 === level ? html`<h2>${title}</h2>` : nothing}
      ${3 === level ? html`<h3>${title}</h3>` : nothing}
      ${4 === level ? html`<h4>${title}</h4>` : nothing}
      ${5 === level ? html`<h5>${title}</h5>` : nothing}
      ${6 === level ? html`<h6>${title}</h6>` : nothing}
      ${6 < level ? html`<span aria-level=${level}>${title}</span>` : nothing}`;
  }
  createRenderRoot() {
    return this;
  }
}
export default HIP.factory.element(TitleElement);
