
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { beforeNextRender } from '../../../hip/library/polymer/polymer.js';
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { AsLitHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorContainer } from '../../../eip/element/hi-behavior-container/hi-behavior-container.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
export const styles = css`
  :host {
    display: inline-block;
  }

  #container {
    display: flex;
    align-items: center;
  }

  #container ::slotted(hi-toggle) {
    margin: 0 var(--hi-px-4);
  }

  #container ::slotted(hi-toggle) {
    top: 0;
  }

  #container ::slotted(hi-label) {
    display: block;
    padding-top: var(--hi-px-3);
  }

  :host([internal-label-count='2'][internal-on='false']) #container ::slotted(:not(hi-toggle):first-of-type),
  :host([internal-label-count='2'][internal-on='true']) #container ::slotted(:not(hi-toggle):last-of-type),
  :host([internal-label-count='1'][internal-on='true']) #container ::slotted(:not(hi-toggle):first-of-type) {
    font-weight: var(--hi-font--weight-heavy) !important;
    text-shadow: none !important;
  }

  :host([layout='icon']) #container ::slotted(:not(hi-toggle)) {
    visibility: hidden;
    width: 0;
    height: 0;
  }
`;
export class ToggleGroupElement extends BehaviorContainer(BehaviorElement(AsLitHIC(LitElement))) {
  static get is() {
    return "hi-toggle-group";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      layout: {
        type: String,
        reflect: true
      }
    };
  }
  render() {
    return html`
      <div id="container">
        <slot id="items"></slot>
      </div>
    `;
  }
  connectedCallback() {
    super.connectedCallback();
    this.__onValueChanged_toggleBound = this.__onValueChanged_toggle.bind(this);
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    const toggle = this.querySelector("hi-toggle");
    if (toggle) {
      toggle.removeEventListener("on-changed", this.__onValueChanged_toggleBound);
    }
  }
  __onValueChanged_toggle() {
    const toggle = this.querySelector("hi-toggle");
    if (toggle) {
      this.setAttribute("internal-on", toggle.on ? "true" : "false");
    }
  }
  _onSlotChanged() {
    const children = this._readSlotChildren();
    const labelsEl = children.filter((o) => dom.is.someTag(["label", "hi-label"], o));
    const togglesEl = children.filter((o) => dom.is.tag("hi-toggle", o));
    const toggleEl = togglesEl.length > 0 ? togglesEl[0] : void 0;
    if (toggleEl) {
      toggleEl.addEventListener("on-changed", this.__onValueChanged_toggleBound);
      beforeNextRender(this, () => {
        this.__onValueChanged_toggleBound();
      });
      this.__updateSlotLabelCount(toggleEl, labelsEl);
      this.__updateSlotLabels(toggleEl, labelsEl);
    }
  }
  __updateSlotLabelCount(toggleEl, labelsEl) {
    this.setAttribute("internal-label-count", `${labelsEl.length}`);
  }
  __updateSlotLabels(toggleEl, labelsEl) {
    const labelCount = labelsEl.length;
    const hasChildrenToLabel = toggleEl && labelCount > 0;
    if (!hasChildrenToLabel) {
      return;
    }
    labelsEl.forEach((labelEl) => {
      dom.ensureAttr(labelEl, "id");
    });
    if (labelCount === 2) {
      labelsEl[0].action = "turnOff";
      labelsEl[1].action = "turnOn";
    } else if (labelCount === 1) {
      labelsEl[0].action = "toggle";
    }
    const labelsForToggle = labelsEl.map((o) => o.getAttribute(`id`));
    const associatedLabels = labelsForToggle.join(` `);
    toggleEl.setAttribute("aria-labelledby", associatedLabels);
  }
  toString() {
    return `${this.getAttribute("internal-label-count")} label(s)`;
  }
}
export default HIP.factory.element(ToggleGroupElement);
