
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { BehaviorInput } from '../../../eip/element/hi-behavior-input/hi-behavior-input.js';
import {
  BehaviorA11yKeyboard,
  AsLitHIC,
  BehaviorBrowserEvents,
  BehaviorElement,
  BehaviorInteractiveElement,
  BehaviorCustomizable,
  HIP
} from '../../../eip/element/hi-component/hi-component.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export const styles = css`
  :host {
    --hi-toggle--track-clearance: var(--hi-px-3);
    --hi-toggle--thumb-size: var(--hi-px-16);
    --hi-toggle--empty-width: var(--hi-px-16);
    --hi-toggle--track-width: calc(
      var(--hi-toggle--empty-width) + var(--hi-toggle--thumb-size) + 2px + var(--hi-toggle--track-clearance)
    );
    /* +4 is 2px for border and 2px for space  */
    --hi-toggle--track-height: var(--hi-el--height-sml);
    --hi-toggle--label-lh: var(--hi-el--height-sml);
  }

  :host {
    display: inline-block;
    height: var(--hi-toggle--track-height);
    position: relative;
    vertical-align: middle;
    border-radius: var(--hi-shape--pill);
  }

  :host,
  #toggleTrack {
    transition: background-color var(--hi-animation--speed) var(--hi-animation--accel);
  }

  #toggleTrack,
  #toggleThumb {
    border-radius: var(--hi-shape--pill);
    display: block;
  }

  @media screen and (-ms-high-contrast: active) {
    #toggleTrack,
    #toggleThumb {
      border: var(--hi-px-1) solid var(--hi-clr--transparent);
    }
  }

  #toggleThumb {
    left: 0;
    position: absolute;
    width: var(--hi-toggle--thumb-size);
    height: var(--hi-toggle--thumb-size);
    top: var(--hi-toggle--track-clearance);
    transition: transform 70ms cubic-bezier(0.2, 0, 1, 0.9);
    border: var(--hi-px-1) solid var(--hi-clr--transparent);
  }

  :host([on]) #toggleThumb {
    transform: translate(var(--hi-toggle--empty-width), 0);
    background-color: var(--hi-clr--theme-1);
  }

  :host(:not([on])) #toggleThumb {
    transform: translate(var(--hi-toggle--track-clearance), 0);
    background-color: var(--hi-clr--white);
    border-color: var(--hi-clr--theme-5);
  }

  #toggleTrack {
    position: relative;
    border: var(--hi-px-1) solid var(--hi-clr--theme-5);
    height: var(--hi-toggle--track-height);
    width: var(--hi-toggle--track-width);
  }

  :host([on]) #toggleTrack {
    background-color: var(--hi-clr--theme-3);
  }

  :host(:not([on])) #toggleTrack {
    background-color: var(--hi-clr--theme-1);
  }

  :host([disabled]) #toggleTrack {
    background: var(--hi-clr--gray-3);
  }

  :host([disabled]) #toggleThumb {
    background: var(--hi-clr--gray-8);
  }

  :host([disabled][on]) #toggleThumb {
    background: var(--hi-clr--gray-8);
  }

  :host([disabled]) {
    pointer-events: all !important;
  }

  :host([disabled]) * {
    cursor: not-allowed;
  }

  :host(:hover:active:not([disabled])),
  :host(:hover:active:not([disabled])) #toggleTrack {
    box-shadow: none;
  }

  :host([size='small']) {
    --hi-toggle--thumb-size: calc(var(--hi-px-8) + var(--hi-px-2));
    --hi-toggle--empty-width: calc(var(--hi-px-8) + var(--hi-px-2));
    --hi-toggle--track-clearance: var(--hi-px-2);
    --hi-toggle--track-height: var(--hi-px-16);
  }

  :host([size='large-x']),
  :host([size='large-xx']) {
    font-size: var(--hi-px-20) !important;
  }

  :host([size='large']),
  :host([size='large-x']),
  :host([size='large-xx']) {
    --hi-toggle--thumb-size: calc(var(--hi-px-12) + var(--hi-px-10));
  }

  :host([size='small'][on]) #toggleThumb {
    transform: translate(var(--hi-toggle--empty-width), 0);
  }

  :host([size='small']:not([on])) #toggleThumb {
    transform: translate(var(--hi-toggle--track-clearance), 0);
  }

  :host([kind='button']) {
    border: var(--hi-px-1) solid var(--hi-clr--transparent);
    background-color: var(--hi-clr--white);
    border-radius: var(--hi-shape--radius);
    box-shadow: 0 0 0 var(--hi-px-1) var(--hi-clr--transparent);
    line-height: var(--hi-font-size--large);
    font-size: var(--hi-px-16);
    padding: var(--hi-px-8);
    height: var(--hi-el--height) !important;
    width: var(--hi-el--height) !important;
  }

  hi-icon {
    display: block;
  }

  :host([kind='button'][size='small']) {
    padding: var(--hi-px-4);
    height: var(--hi-el--height-sml) !important;
    width: var(--hi-el--height-sml) !important;
  }

  @media (hover: hover) {
    :host(:hover:not([disabled])) #toggleTrack {
      box-shadow: var(--hi-clr--shadow-down-2);
    }

    :host([kind='button']:not([disabled]):hover) {
      box-shadow: var(--hi-clr--shadow-down-2);
      background-color: var(--hi-clr--white);
    }

    :host([kind='button']:hover:not([disabled])) {
      box-shadow: var(--hi-clr--shadow-down-2);
      border-color: var(--hi-clr--theme-5);
      fill: var(--hi-clr--theme-5);
    }
  }
`;
export class ToggleElement extends BehaviorCustomizable(
  BehaviorA11yKeyboard(
    BehaviorBrowserEvents(BehaviorInput(BehaviorInteractiveElement(BehaviorElement(AsLitHIC(LitElement)))))
  )
) {
  static get is() {
    return "hi-toggle";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      kind: {
        type: String
      },
      value: {
        type: String,
        notify: true
      },
      on: {
        type: Boolean,
        reflect: true,
        notify: true
      },
      icon: {
        type: String
      },
      caseTrue: {
        type: String,
        attribute: "case-true"
      },
      caseFalse: {
        type: String,
        attribute: "case-false"
      }
    };
  }
  render() {
    const icon = this.__iconCompute(this.on, this.icon);
    const isKindButton = this.kind === "button";
    return isKindButton ? html`<hi-icon glyph=${icon} theme=${this.theme}></hi-icon>` : html`<div id="toggleTrack" aria-hidden="true">
          <div id="toggleThumb"></div>
        </div>`;
  }
  static get api() {
    return ["turnOff", "turnOn", "toggle", "commit", "trigger"];
  }
  static get keyBindings() {
    return {
      "space:keydown": "__onKeyDown_space",
      "space:keyup": "__onKeyUp_space"
    };
  }
  static get browserBindings() {
    return {
      click: "__onClick"
    };
  }
  static get valueProperty() {
    return "on";
  }
  static get valueEvent() {
    return "on-changed";
  }
  static get autoBinding() {
    return {
      signal: "on-changed",
      field: "on"
    };
  }
  update(props) {
    super.update(props);
    if (props.has("kind") && this.kind === "icon") {
      this.kind = "button";
    }
  }
  firstUpdated() {
    this.__reflectState();
  }
  updated(props) {
    super.updated(props);
    if (props.has("on")) {
      this.__reflectState();
    }
  }
  toggle() {
    return this.__setState(!this.on);
  }
  trigger() {
    return this.toggle();
  }
  commit() {
    return { value: this.value };
  }
  activate() {
    return this.turnOn();
  }
  turnOn() {
    return this.__setState(true);
  }
  deactivate() {
    return this.turnOff();
  }
  turnOff() {
    return this.__setState(false);
  }
  get validity() {
    return this._computeValidity({
      valueMissing: this.required && this.on == null
    });
  }
  __iconCompute(on, icon) {
    const baseIcon = (icon || "").replace("_filled", "");
    return on ? `${baseIcon}_filled` : baseIcon;
  }
  __onKeyDown_space(event) {
    event.preventDefault();
    event.stopPropagation();
  }
  __onKeyUp_space(event) {
    event.preventDefault();
    event.stopPropagation();
    this.trigger();
    this.commit();
  }
  __onClick(e) {
    e.preventDefault();
    e.stopPropagation();
    this.trigger();
    this.commit();
  }
  __setState(_on) {
    if (this.disabled) {
      return this.on;
    }
    this.on = _on;
    return this.__reflectState();
  }
  __reflectState() {
    const on = this.on;
    const kind = this.kind;
    const state = kind === "button" ? "pressed" : "checked";
    const role = kind === "button" ? "button" : "checkbox";
    if (on && !f1.is.empty(this.caseTrue)) {
      this.value = f1.safe(true, this.caseTrue);
    } else if (!on && !f1.is.empty(this.caseFalse)) {
      this.value = f1.safe(false, this.caseFalse);
    } else {
      this.value = f1.cast.boolean(on);
    }
    dom.setAttrs(this, {
      [`aria-${state}`]: `${!!on}`,
      role
    });
    return this.value;
  }
  toString() {
    return this.on ? "On" : "Off";
  }
}
export default HIP.factory.element(ToggleElement);
