
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { timeOut } from '../../../hip/library/polymer/polymer.js';
import { Debouncer } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
const styles = css`
  :host {
    --hi-tooltip--max-width: 14rem;
    --hi-tooltip--min-width: 4rem;
    --hi-tooltip--min-height: 2rem;
    --hi-tooltip--max-height: 32vh;
    --hi-tooltip--offset-unit: 25;
  }

  :host {
    display: block;
    position: relative;
    border-radius: var(--hi-shape--radius-2);
    background-color: var(--hi-clr--white);
    border: var(--hi-px-1) solid var(--hi-clr--theme-5);
    color: var(--hi-clr--text);
    max-height: var(--hi-tooltip--max-height);
    min-height: var(--hi-tooltip--min-height);
    max-width: var(--hi-tooltip--max-width);
    min-width: var(--hi-tooltip--min-width);
    font-weight: var(--hi-font--weight-light);
    text-rendering: optimizeLegibility;
    box-shadow: var(--hi-clr--shadow-down-3);
    transform: translate3d(0, 0, 0);
    transition: var(--hi-animation--speed) opacity var(--hi-animation--accel);
    z-index: var(--hi-depth--above-all);
    padding: var(--hi-px-12);
    overflow-y: auto;
    overflow-x: hidden;
    overflow-wrap: break-word;
  }

  :host(:not([theme])) {
    border-color: var(--hi-clr--gray-9);
  }

  #container {
    font-size: var(--hi-font-size--text);
    line-height: var(--hi-font-lh--medium);
  }

  #header {
    padding-bottom: var(--hi-px-4);
    font-size: var(--hi-font-size--h5);
    line-height: var(--hi-font-lh--h5);
    font-weight: var(--hi-font--weight-bold, bold);
  }

  :host([data-popper-placement^='top']) {
    margin-bottom: var(--hi-px-8);
  }

  :host([data-popper-placement^='bottom']) {
    margin-top: var(--hi-px-8);
  }

  :host([data-popper-placement^='right']) {
    margin-left: var(--hi-px-8);
  }

  :host([data-popper-placement^='left']) {
    margin-right: var(--hi-px-8);
  }

  :host([opened]) {
    z-index: var(--hi-depth--above-all);
    pointer-events: all;
    visibility: visible;
  }

  :host(:not([heading])) #header {
    display: none;
  }

  :host(:not([opened])) {
    pointer-events: none !important;
    visibility: hidden !important;
    z-index: -100 !important;
    display: none;
  }
`;
export class TooltipElement extends BehaviorElement(AsHIC(LitElement)) {
  constructor() {
    super();
    this.placement = "auto";
    this.opened = false;
    this.__registerWindowEvents();
  }
  static get is() {
    return "hi-tooltip";
  }
  render() {
    return html`
      <header id="header">${this.heading}</header>
      <div id="container">
        <slot id="content"></slot>
      </div>
    `;
  }
  static get properties() {
    return {
      heading: {
        type: String,
        reflect: true
      },
      __content: {
        type: String
      },
      for: {
        type: String,
        reflect: true
      },
      opened: {
        type: Boolean,
        reflect: true,
        notify: true
      },
      placement: {
        type: String
      },
      disallowNoContent: {
        type: Boolean,
        attribute: "disallow-no-content",
        reflect: true
      }
    };
  }
  update(props) {
    super.update(props);
    this._watchChanges(props, [
      {
        properties: ["for"],
        method: "__createAndAttachTooltip"
      },
      {
        properties: ["_heading", "__content"],
        method: "__autoAttach"
      }
    ]);
  }
  updated(changedProperties) {
    super.updated(changedProperties);
    if (changedProperties.has("opened")) {
      this.__openedChanged(this.opened);
    }
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get api() {
    return ["remove", "close"];
  }
  static get autoBinding() {
    return {
      signal: "opened-changed",
      field: "opened"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    dom.ensureAttr(this, "id");
    dom.setAttr(this, "role", "tooltip");
    if (!this.hasAttribute("theme")) {
      this.theme = "gray";
    }
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    this.__destroyTooltip();
    this.__detachTooltip();
    window.removeEventListener("keyup", this.__onKeyupBound, true);
    window.removeEventListener("unload", this.__onUnloadBound);
  }
  __registerWindowEvents() {
    this.__onKeyupBound = this.__onKeyup.bind(this);
    this.__onUnloadBound = this.__onUnload.bind(this);
    window.addEventListener("keyup", this.__onKeyupBound, true);
    window.addEventListener("unload", this.__onUnloadBound);
  }
  __onUnload() {
    window.removeEventListener("keyup", this.__onKeyupBound, true);
  }
  __onKeyup(event) {
    const ESCAPE_KEY = 27;
    switch (event.keyCode) {
      case ESCAPE_KEY:
        this.opened = false;
        break;
      default:
        return;
    }
  }
  __readTooltipContent() {
    const childNodes = Array.from(this.childNodes);
    childNodes.some((childNode) => {
      if (childNode.textContent && childNode.textContent.trim() !== "") {
        this.__content = childNode.textContent;
        return true;
      }
      return false;
    });
  }
  close() {
    this.opened = false;
  }
  remove() {
    this.__removeSelfIfAllowed();
  }
  __autoAttach(_heading = this.heading, __content = this.__content) {
    const hasNoContent = !this.disallowNoContent ? false : (__content || "").trim().length <= 0 && (_heading || "").trim().length <= 0;
    this.__detachTooltip();
    dom.flipAttr(this, "hidden", hasNoContent);
    if (!hasNoContent) {
      this.__createAndAttachTooltip();
    }
  }
  destroy() {
    this.__detachTooltip();
    this.__destroyTooltip();
  }
  __createAndAttachTooltip() {
    this.__attachTooltip();
  }
  __onFocusOut__capture() {
    this.opened = false;
  }
  __openedChanged(_opened) {
    if (!_opened) {
      this.__destroyTooltip();
    }
    if (_opened) {
      this.__createTooltip();
    }
    this.setAttribute("aria-hidden", !_opened + "");
    this.__readTooltipContent();
  }
  __removeSelfIfAllowed() {
    if (this.allowClose) {
      this.remove();
    }
  }
  __attachTooltip() {
    if (!this.for) {
      return;
    }
    dom.ensureAttr(this, "id");
    const attached = HIP.browser.queryOne(this.for, this.ns, this);
    if (!attached) {
      return;
    }
    this.__attached = attached;
    this.__attached.setAttribute("tabIndex", "0");
    this.__attached.style.cursor = f1.safe("default", this.__attached.style.cursor);
    this.__attachment_onMouseEnterBound = this.__attachment_onMouseEnter.bind(this);
    this.__attachment_onMouseLeaveBound = this.__attachment_onMouseLeave.bind(this);
    this.__popper_onMouseEnterBound = this.__popper_onMouseEnter.bind(this);
    this.__popper_onMouseLeaveBound = this.__popper_onMouseLeave.bind(this);
    this.__attachment_onFocusBound = this.__attachment_onFocus.bind(this);
    this.__attachment_onBlurBound = this.__attachment_onBlur.bind(this);
    this.__attached.addEventListener("mouseenter", this.__attachment_onMouseEnterBound);
    this.__attached.addEventListener("mouseleave", this.__attachment_onMouseLeaveBound);
    this.__attached.addEventListener("focus", this.__attachment_onFocusBound);
    this.__attached.addEventListener("blur", this.__attachment_onBlurBound);
    this.addEventListener("mouseenter", this.__popper_onMouseEnterBound, true);
    this.addEventListener("mouseleave", this.__popper_onMouseLeaveBound, true);
    this.addEventListener("focusout", this.__onFocusOut__capture, true);
    dom.ensureAttr(this.__attached, "id");
    this.__existingLabelledBy = this.__attached.getAttribute("aria-labelledby") || "";
    this.__attached.setAttribute(
      "aria-labelledby",
      (this.__existingLabelledBy + " " + `${this.__attached.getAttribute("id") || ""}`.trim() + ` ${this.getAttribute("id") || ""}`).trim()
    );
  }
  __detachTooltip() {
    const attached = this.__attached;
    if (!attached) {
      return;
    }
    attached.removeEventListener("mouseenter", this.__attachment_onMouseEnterBound);
    attached.removeEventListener("mouseleave", this.__attachment_onMouseLeaveBound);
    attached.removeEventListener("focus", this.__attachment_onFocusBound);
    attached.removeEventListener("blur", this.__attachment_onBlurBound);
    this.removeEventListener("mouseenter", this.__popper_onMouseEnterBound, true);
    this.removeEventListener("mouseleave", this.__popper_onMouseLeaveBound, true);
    this.removeEventListener("focusout", this.__onFocusOut__capture, true);
    dom.flipAttr(attached, "aria-labelledby", !!this.__existingLabelledBy, this.__existingLabelledBy);
    this.__attached = void 0;
  }
  __attachment_onMouseEnter() {
    this.__isHovered = true;
    this.__delayedVisibilityDebouncer = Debouncer.debounce(
      this.__delayedVisibilityDebouncer,
      timeOut.after(250),
      () => {
        this.opened = true;
      }
    );
  }
  __popper_onMouseEnter() {
    this.__attachment_onMouseEnter();
  }
  __attachment_onMouseLeave() {
    this.__isHovered = false;
    if (!this.__isFocused) {
      this.__delayedVisibilityDebouncer = Debouncer.debounce(
        this.__delayedVisibilityDebouncer,
        timeOut.after(300),
        () => {
          this.opened = false;
        }
      );
    }
  }
  __popper_onMouseLeave() {
    this.__attachment_onMouseLeave();
  }
  __attachment_onFocus() {
    this.__isFocused = true;
    this.opened = true;
  }
  __attachment_onBlur(event) {
    this.__isFocused = false;
    if (!this.__isHovered && !HIP.util.existsInside(this, event.relatedTarget)) {
      this.opened = false;
    }
  }
  __createTooltip() {
    this.__destroyTooltip();
    if (!this.__attached) {
      return;
    }
    this.__popper = Popper.createPopper(this.__attached, this, {
      placement: this.placement,
      strategy: "fixed",
      modifiers: [
        {
          name: "eventListeners",
          options: { scroll: false }
        },
        {
          name: "offset",
          options: {
            offset: [0, 8]
          }
        },
        {
          name: "flip",
          options: {
            flipVariations: true
          }
        },
        {
          name: "computeStyles",
          options: {
            gpuAcceleration: true
          }
        },
        {
          name: "preventOverflow",
          options: {
            padding: 8,
            altBoundary: true
          }
        }
      ]
    });
    this.__popper.update();
  }
  __destroyTooltip() {
    if (this.__popper && typeof this.__popper.destroy === "function") {
      this.__popper.destroy();
    }
    this.__popper = void 0;
  }
}
export default HIP.factory.element(TooltipElement);
