
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorBrowserEvents, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorItemTree } from '../../../eip/element/hi-behavior-set-tree/hi-behavior-item-tree.js';
export class LeafElement extends BehaviorItemTree(BehaviorBrowserEvents(BehaviorElement(AsHIC(PolymerElement)))) {
  static get is() {
    return "hi-leaf";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          flex: 0 1 auto;
          display: flex;
          height: 100%;
          align-items: center;
          color: var(--hi-clr--theme-5);
          padding: var(--hi-px-6);
          font-size: var(--hi-font-size--text);
          line-height: var(--hi-font-lh--text);
          margin-bottom: var(--hi-px-4);
          border-left: var(--hi-px-3) solid var(--hi-clr--transparent);
        }

        :host(:not([theme])) {
          color: var(--hi-clr--text);
        }

        :host(:not([leaf-icon])) #leafIcon {
          display: none;
        }

        #leafIcon {
          margin-right: var(--hi-px-8);
          --hi-icon--color: var(--hi-clr--theme-5);
        }

        :host ::slotted([slot='addendum']) {
          margin-left: auto;
          margin-right: var(--hi-px-8);
        }

        :host(:not([readonly])) {
          background-color: var(--hi-clr--transparent);
          cursor: pointer;
          outline: 0;
        }

        :host([disabled]) {
          color: var(--hi-clr--disabled) !important;
        }

        :host(:focus:not([readonly])),
        :host([focused]:not([readonly])) {
          outline: var(--hi-outline--width) var(--hi-outline--line) var(--hi-clr--outline);
          outline-offset: var(--hi-outline--offset);
          text-shadow: var(--hi-clr--text-shadow-1);
        }

        @media (hover: hover) {
          :host(:hover:not([readonly])) {
            text-shadow: var(--hi-clr--text-shadow-1);
          }

          :host(:hover:not([readonly])),
          :host(:hover[focused]) {
            background-color: var(--hi-clr--gray-2);
          }
        }

        :host([focused]:not([disabled])) {
          background-color: var(--hi-clr--gray-2);
          border-left: var(--hi-px-4) solid var(--hi-clr--theme-3);
        }
      </style>
      <hi-icon id="leafIcon" glyph="[[ leafIcon ]]" theme="[[ theme ]]"> </hi-icon>
      <slot id="items"><span> [[ heading ]] </span> </slot>
      <slot name="addendum"></slot>
    `;
  }
  static get properties() {
    return {
      heading: {
        type: String
      },
      leafIcon: {
        type: String
      }
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "treeitem");
    if (!this.readonly) {
      this._createBrowserEvent("click", {
        click: "_itemTree__onTap"
      });
    }
  }
}
export default HIP.factory.element(LeafElement);
