
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorBrowserEvents, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorNodeTree } from '../../../eip/element/hi-behavior-set-tree/hi-behavior-node-tree.js';
import { TreeService } from '../../../eip/element/hi-behavior-set-tree/hi-service-tree.js';
import './hi-leaf.js';
import '../../../eip/element/hi-icon/hi-icon.js';
export class SubTreeElement extends BehaviorNodeTree(BehaviorBrowserEvents(BehaviorElement(AsHIC(PolymerElement)))) {
  static get is() {
    return "hi-sub-tree";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          display: block;
          height: auto;
          border: 0;
          font-size: var(--hi-font-size--text);
          color: var(--hi-clr--theme-5);
        }

        :host(:not([theme])) {
          color: var(--hi-clr--text);
        }

        .group {
          outline: none;
          display: flex;
          align-items: center;
          cursor: default;
          line-height: var(--hi-px-32);
          height: var(--hi-px-32);
          margin-bottom: var(--hi-px-4);
          border-left: var(--hi-px-4) solid var(--hi-clr--transparent);
        }

        :host(:not([node-icon])) #nodeIcon {
          display: none;
        }

        #nodeIcon {
          margin: 0 var(--hi-px-8);
          --hi-icon--color: var(--hi-clr--theme-5);
        }

        @media (hover: hover) {
          .group:hover,
          :host([active]) > .group:hover {
            background-color: var(--hi-clr--gray-2);
          }

          .group:hover {
            text-shadow: var(--hi-clr--text-shadow-1);
          }
        }

        :host([opened]) ::slotted([focused]) {
          outline: var(--hi-outline--width) var(--hi-outline--line) var(--hi-clr--outline);
          outline-offset: var(--hi-outline--offset);
        }

        :host([focused]) > .group,
        :host([active]) > .group {
          border-left: var(--hi-px-4) solid var(--hi-clr--theme-3);
          background-color: var(--hi-clr--gray-2);
          text-shadow: var(--hi-clr--text-shadow-1);
        }

        .container_list {
          margin-left: var(--hi-px-12);
        }

        :host(:not([opened])) > .container_list {
          display: none;
        }

        hi-icon#icon {
          margin: 0 var(--hi-px-4);
          --hi-icon--color: var(--hi-clr--theme-5);
        }

        :host([disabled]) > hi-icon {
          --hi-icon--color: var(--hi-clr--theme-5);
        }

        /* give sub-tree node acceptable height for accessibility */
        @media only screen and (max-width: 40rem) {
          :host([device='mobile']) .group {
            height: var(--hi-el--height-mobile) !important;
            margin-top: var(--hi-px-8);
          }
        }
      </style>
      <div id="group" class="group">
        <hi-icon
          id="icon"
          type="material"
          theme="[[ __iconTheme ]]"
          glyph="[[ __iconKind ]]"
          direction$="[[ __iconDirection ]]"
          disabled$="[[ disabled ]]"
        >
        </hi-icon>
        <slot name="heading">
          <span mixin="ellipsis"> [[ heading ]] </span>
        </slot>
        <hi-icon id="nodeIcon" glyph="[[ nodeIcon ]]" theme$="[[ theme ]]"> </hi-icon>
      </div>
      <div aria-labelledby="group" class="container_list" role="group">
        <slot id="items"></slot>
      </div>
    `;
  }
  static get properties() {
    return {
      heading: {
        type: String
      },
      __iconKind: {
        type: String
      },
      __iconTheme: {
        type: String
      },
      __iconDirection: {
        type: String
      },
      nodeIcon: {
        type: String,
        reflectToAttribute: true
      }
    };
  }
  static get observers() {
    return ["__visualIconStateObserver(opened, disabled)"];
  }
  static get browserBindings() {
    return {
      "click:group": "__onClick"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "treeitem");
  }
  __onSlotChanged() {
    this._setTree__onSlotChanged();
    this.__visualIconStateObserver(this.opened, this.disabled);
  }
  __onClick() {
    this.opened = !this.opened;
  }
  _includeNodes(root) {
    const slotChildren = this.tree.slotChildren;
    const nodeIndex = slotChildren.indexOf(root);
    if (nodeIndex >= 0) {
      this.tree.__slotChildren.splice(nodeIndex, 1, ...root.slotChildren);
    }
    this.tree.__slotChildren.splice(nodeIndex, 0, root);
    this.tree.__slotChildren.forEach((o) => {
      o.focused = false;
    });
    this.slotChildren.filter(TreeService.nodeFilter).forEach((node) => {
      node.setAttribute("aria-hidden", "false");
    });
    return this.tree.slotChildren.indexOf(root);
  }
  _excludeNodes(root) {
    const treeChildren = this.tree.__slotChildren;
    const childNodes = TreeService.traverseNode(
      () => {
      },
      (node) => node,
      this
    );
    const allNodes = childNodes.values;
    allNodes.reverse().forEach((node) => node.opened = false);
    allNodes.forEach((node) => {
      const treeIds = treeChildren.map((o) => o.id);
      const removeId = treeIds.indexOf(node.id);
      if (removeId >= 0) {
        treeChildren.splice(removeId, 1);
      }
    });
    allNodes.forEach((node) => {
      node.setAttribute("aria-hidden", "true");
    });
    this.slotChildren.filter(TreeService.leafFilter).forEach((leaf) => {
      const treeIds = treeChildren.map((o) => o.id);
      const removeId = treeIds.indexOf(leaf.id);
      if (removeId >= 0) {
        treeChildren.splice(removeId, 1);
      }
    });
    return this.tree.slotChildren.indexOf(root);
  }
  __visualIconStateObserver(_opened, _disabled) {
    const hasChildren = this.hasAnySlotChildren;
    if (_disabled) {
      this.__iconKind = "dash";
    } else if (_opened) {
      this.__iconKind = "chevron_up";
      this.__iconDirection = "south";
    } else {
      this.__iconKind = "chevron_up";
      this.__iconDirection = "west";
    }
    if (!_disabled && hasChildren) {
      this.__iconTheme = this.theme;
    } else {
      this.__iconTheme = "black";
    }
  }
  _nodeTree__openedChanged(opened) {
    super._nodeTree__openedChanged(opened);
    this.setAttribute("aria-expanded", `${opened}`);
    return opened;
  }
}
export default HIP.factory.element(SubTreeElement);
