
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import {
  BehaviorA11yKeyboard,
  AsHIC,
  BehaviorBrowserEvents,
  BehaviorElement,
  BehaviorInteractiveElement,
  HIP
} from '../../../eip/element/hi-component/hi-component.js';
import '../../../eip/element/hi-component/hi-behavior-browser-events.js';
import '../../../eip/element/hi-behavior-set-tree/hi-behavior-set-tree.js';
import { BehaviorSetTree } from '../../../eip/element/hi-behavior-set-tree/hi-behavior-set-tree.js';
import './hi-sub-tree.js';
import './hi-leaf.js';
export class TreeElement extends BehaviorSetTree(
  BehaviorA11yKeyboard(BehaviorBrowserEvents(BehaviorInteractiveElement(BehaviorElement(AsHIC(PolymerElement)))))
) {
  static get is() {
    return "hi-tree";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          display: block;
          outline: none !important;
          font-size: var(--hi-font-size--text);
        }
        :host(:focus) ::slotted([focused]) {
          outline: var(--hi-outline--width) var(--hi-outline--line) var(--hi-clr--outline);
          outline-offset: var(--hi-outline--offset);
        }
      </style>
      <slot id="items"></slot>
    `;
  }
  static get api() {
    return ["expand", "collapse"];
  }
  static get keyBindings() {
    return {
      "enter:keyup": "_setTree__onKeyUp_enter",
      "up:keydown": "_onKeyAny__preventDefault",
      "down:keydown": "_onKeyAny__preventDefault",
      "right:keyup": "_setTree__onKeyUp_right",
      "down:keyup": "_setTree__onKeyUp_down",
      "left:keyup": "_setTree__onKeyUp_left",
      "up:keyup": "_setTree__onKeyUp_up",
      "end:keyup": "_setFocus__onKeyUp_end",
      "home:keyup": "_setFocus__onKeyUp_home",
      "*:keyup": "__onKeyUp_asterisk"
    };
  }
  static get browserBindings() {
    return {
      focus: "_setFocus__onFocus"
    };
  }
  static get valueEvent() {
    return "selected-ended";
  }
  static get autoBinding() {
    return {
      signal: "opened-changed",
      field: "opened"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "tree");
  }
  __onSlotChanged() {
    this._setTree__onSlotChanged();
  }
  __onKeyUp_asterisk() {
    const isExpanded = this.slotChildren.reduce((isExpanded2, el) => isExpanded2 || el.opened, false);
    if (isExpanded) {
      this.collapseChildren();
    } else {
      this.expandChildren();
    }
  }
}
export default HIP.factory.element(TreeElement);
