
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import navigation from '../../../pip/library/pi-navigation/pi-navigation.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class UploadElement extends BehaviorElement(AsHIC(PolymerElement)) {
  constructor() {
    super();
    this.method = "POST";
    this.fileNameKey = "name";
  }
  static get is() {
    return "hi-upload";
  }
  static get properties() {
    return {
      method: {
        type: String,
        value: "POST"
      },
      uri: {
        type: String
      },
      fileContentKey: {
        type: String
      },
      fileNameKey: {
        type: String
      },
      headers: {
        type: Object
      },
      parameters: {
        type: Object
      }
    };
  }
  static get observers() {
    return ["__setupFileUploader(fileContentKey)"];
  }
  static get api() {
    return ["invoke"];
  }
  static get autoBinding() {
    return {
      signal: "invoke-ended",
      field: "invoke"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "group");
    this.style.display = "block";
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    this.__destroyFileUploader();
  }
  invoke(fileContents) {
    if (!this.__pond) {
      return;
    }
    const file = f1.is.string(fileContents) ? fileContents : new Blob([JSON.stringify(fileContents)], {
      type: "application/json"
    });
    return this.__pond.addFile(file);
  }
  __setupFileUploader() {
    this.__destroyFileUploader();
    this.__createFileUploader();
  }
  __createFileUploader() {
    const pond = this.__pond = window["FilePond"].create({
      required: true,
      allowDrop: true,
      allowPaste: true,
      allowBrowse: true,
      allowRevert: false,
      allowReplace: false,
      instantUpload: true,
      allowMultiple: true,
      dataMaxFiles: 3,
      server: {
        process: (fieldName, file, metadata, load, error, progress) => {
          const formData = new FormData();
          if (this.fileContentKey != null) {
            formData.append(this.fileContentKey, file, file.name);
          }
          if (this.fileNameKey) {
            formData.append(this.fileNameKey, file.name);
          }
          f1.each((value, property) => {
            formData.append(property, value);
          }, this.parameters);
          const request = new XMLHttpRequest();
          const uri = navigation.uri.write(this.uri);
          request.open(this.method, uri);
          f1.each((value, property) => {
            request.setRequestHeader(property, value);
          }, this.headers);
          request.upload.onprogress = (e) => {
            progress(e.lengthComputable, e.loaded, e.total);
          };
          request.addEventListener("error", (event) => {
            this._dispatchProcess("request-errored", event);
          });
          request.addEventListener("abort", (event) => {
            this._dispatchProcess("request-abort", event);
          });
          request.onload = function() {
            if (request.status >= 200 && request.status < 300) {
              load(request.responseText);
            } else {
              error(request.responseText);
            }
          };
          request.send(formData);
        },
        restore: null,
        revert: null,
        fetch: null,
        load: null
      }
    });
    pond.on("error", (error, file) => {
      try {
        error = JSON.parse(error.body || "{}");
      } catch (e) {
      }
      this._dispatchProcess("file-errored", {
        error,
        file
      });
    });
    pond.on("addfilestart", (file) => {
      this._dispatchProcess("file-received", {
        file
      });
    });
    pond.on("addfile", (file) => {
      this._dispatchProcess("file-delivered", {
        file
      });
    });
    pond.on("processfile", (error, file) => {
      this._dispatchProcess("file-completed", {
        error,
        file
      });
    });
    pond.appendTo(this);
  }
  __destroyFileUploader() {
    if (this.__pond) {
      this.__pond.removeFiles();
      this.__pond.destroy();
      window["FilePond"].destroy(this.__pond.element);
    }
    Array.from(this.children).forEach((el) => {
      el.remove();
    });
    this.__pond = void 0;
  }
}
HIP.factory.element(UploadElement);
export default "hi-upload";
