
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import '../../../eip/element/hi-icon/hi-icon.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class ValidationNoticeElement extends BehaviorElement(AsHIC(PolymerElement)) {
  constructor() {
    super(...arguments);
    this.validation_message__repeat = [];
  }
  static get is() {
    return "hi-validation-notice";
  }
  static get template() {
    return html`
      <style is="custom-style" include="hi-theme"></style>
      <style>
        :host {
          display: flex;
          align-items: center;
          color: var(--hi-clr--red-5);
          z-index: var(--hi-depth--content-hover);
          padding: var(--hi-px-3) 0 var(--hi-px-4) var(--hi-px-2);
          line-height: var(--hi-font-lh--medium);
          font-size: var(--hi-font-size--text);
        }

        #validations {
          list-style: none;
          margin: 0;
          padding: 0;
        }

        #validations li {
          display: flex;
          align-items: center;
        }

        #validations hi-icon {
          --hi-icon--color: var(--hi-clr--red-5);
          font-size: var(--hi-font-size--large);
          margin-right: var(--hi-px-4);
          flex-shrink: 0;
        }
      </style>
      <ul id="validations" aria-live="polite" aria-atomic="true">
        <hi-repeat input="[[ validation_message__repeat ]]" as="validationMessage">
          <template>
            <li>
              <hi-icon kind="error_outline" mode="presentation" type="material"></hi-icon>
              <span>[[ validationMessage ]]</span>
            </li>
          </template>
        </hi-repeat>
      </ul>
      <slot></slot>
    `;
  }
  static get properties() {
    return {
      validationNotice: {
        type: Object
      },
      for: {
        type: Object,
        reflectToAttribute: true,
        observer: "__forChanged"
      },
      separator: {
        type: String,
        value: ";"
      },
      value: {
        type: String,
        observer: "__valueChanged",
        notify: true
      },
      __target: {
        type: Object
      }
    };
  }
  static get autoBinding() {
    return {
      signal: "value-changed",
      field: "value"
    };
  }
  static get observers() {
    return ["__validationNoticeInputChanged(validationNotice, separator)"];
  }
  trigger(value) {
    this.__updateValidationNotice(value);
  }
  __validationNoticeInputChanged(validationNotice, separator) {
    const isHidden = !validationNotice;
    dom.flipAttr(this, "hidden", isHidden);
    this.validation_message__repeat = isHidden ? [] : f1.is.string(validationNotice) ? validationNotice.split(separator) : validationNotice;
  }
  __valueChanged(value) {
    this.__updateValidationNotice(value);
  }
  __updateValidationNotice(value) {
    const validationKey = this.__target ? f1.form("camel", "kebab", this.__target.validationKey || "") : "";
    this.validationNotice = this.hasAttribute(validationKey) ? `${this.getAttribute(validationKey)}` : value;
  }
  __forChanged(_for) {
    if (this.__target) {
      this.__target.__validationNoticeEl = void 0;
    }
    const target = this.__findTarget(_for);
    if (target) {
      target.__validationNoticeEl = this;
      this.__target = target;
    }
    if (this.__target) {
      dom.ensureAttr(this, "id");
      dom.includeAttrList(this.__target, "aria-describedby", dom.getAttr(this, "id", ""));
    }
  }
  __findTarget(_for = this.for) {
    if (_for instanceof HTMLElement) {
      return _for;
    } else if (!this.hasAttribute("for") || _for === "") {
      return void 0;
    } else if (_for && _for.length > 0) {
      return HIP.browser.queryElement(_for, this.ns);
    }
    return void 0;
  }
}
export default HIP.factory.element(ValidationNoticeElement);
