
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import { BaseBehaviorIO } from '../../../eip/element/hi-behavior-io/hi-behavior-io.js';
import { AsHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import { XSSPurify } from '../../../pip/library/pi-io/pi-io.js';
import $ip from '../../../pip/library/pi-ip/pi-ip.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class ValueGroupElement extends BaseBehaviorIO(BehaviorElement(AsHIC(PolymerElement))) {
  static get is() {
    return "hi-value-group";
  }
  static get properties() {
    return {
      kind: {
        type: String,
        value: "all"
      },
      form: {
        type: String,
        reflectToAttribute: true,
        observer: "__formChanged",
        reflect: true
      },
      invalid: {
        type: Object
      },
      secure: {
        type: Boolean
      }
    };
  }
  static get api() {
    return ["evaluate", "submit", "reset"];
  }
  static get valueEvent() {
    return "output-changed";
  }
  static get autoBinding() {
    return {
      signal: "output-changed",
      field: "input"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("hidden", "hidden");
    HIP.util.waitForLibrary("dom-purify", "xss", {
      parent: ValueGroupElement
    });
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    if (this.form) {
      this.__removeForm(this.form);
    }
  }
  submit() {
    const { isValid } = this.evaluate("form-submitted");
    return isValid ? this.output : void 0;
  }
  reset() {
    HIP.db.form.queryAll(this.form, this.ns).forEach((inputEl) => {
      if (inputEl.locked) {
        return;
      }
      if (inputEl.reset) {
        inputEl.reset();
      } else {
        inputEl.value = null;
      }
    });
  }
  __formChanged(_form, _formOld) {
    this.__removeForm(_formOld);
    this.__addForm(_form);
  }
  __removeForm(_formOld) {
    if (_formOld) {
      HIP.hooks.unregisterElement(this, {
        [`value-notified-${_formOld}`]: `_inputChild__valueNotifiedChannel`
      });
    }
  }
  __addForm(_form) {
    if (_form) {
      HIP.hooks.registerElement(this, {
        [`value-notified-${_form}`]: `_inputChild__valueNotifiedChannel`
      });
    }
  }
  _inputChild__valueNotifiedChannel() {
    return this.evaluate("input-changed");
  }
  evaluate(activityType = "input-changed") {
    const invalid = {};
    const validAll = {};
    const valueAll = {};
    const validRecent = {};
    let isValid = true;
    HIP.db.form.queryAll(this.form, this.ns).forEach((inputEl) => {
      const values = inputEl.values;
      const max = inputEl.max;
      const hasMultipleValues = !f1.is.nil(values) && (!f1.is.nil(max) && max > 1 || values.length > 1);
      const useOnlyValue = f1.is.in("onlyValue", inputEl) && f1.is.true(inputEl.onlyValue);
      const hasMultipleAttribute = inputEl.multiple || inputEl.hasAttribute("multiple");
      const actualValue = useOnlyValue ? inputEl.value : hasMultipleAttribute || hasMultipleValues ? values : inputEl.value;
      const value = inputEl.nullable && (f1.is.nil(actualValue) || "" === actualValue) ? null : inputEl.blankable && f1.is.nil(actualValue) ? "" : !inputEl.nullable && !inputEl.blankable && "" === actualValue ? void 0 : actualValue;
      const hasNoValue = hasMultipleValues ? values.length === 0 : inputEl.nullable ? f1.is.undefined(value) : f1.is.nil(value);
      if (inputEl.excluded) {
        return;
      } else if (!inputEl.required && hasNoValue) {
        return;
      }
      const VGE = ValueGroupElement;
      const secureValue = this.secure && VGE.xss && f1.is.string(value) ? VGE.xss.sanitize(value) : value;
      const parsedValue = secureValue;
      const modelPath = this.__readModelPath(inputEl);
      f1.write(modelPath, parsedValue, valueAll);
      const exemptValidation = inputEl.novalidate || !inputEl.willValidate;
      const validateFormSubmission = activityType === "form-submitted";
      if (!validateFormSubmission) {
        return;
      }
      if (exemptValidation) {
        if (!hasNoValue) {
          f1.write(modelPath, value, validAll);
        }
      } else if (inputEl.required && hasNoValue) {
        f1.write(modelPath, inputEl.validationMessage, invalid);
        isValid = false;
        inputEl.validate();
      } else if (inputEl.checkValidity()) {
        f1.write(modelPath, value, validAll);
      } else {
        f1.write(modelPath, inputEl.validationMessage, invalid);
        isValid = false;
      }
      if (!exemptValidation && validateFormSubmission) {
        inputEl.validate();
        this.__reflectValidatedStatus(inputEl);
      }
      const validValue = f1.read(modelPath, void 0, validAll);
      if (validValue != null && inputEl.dirty) {
        f1.write(modelPath, validValue, validRecent);
      }
    });
    const valid = this.kind === "all" ? validAll : validRecent;
    const valuePacket = {
      validRecent,
      validAll,
      valueAll,
      isValid,
      invalid,
      valid
    };
    const useXssSupport = XSSPurify.xss && $ip.flags.is("/eip/hi-value-group/use-xss");
    const submission = useXssSupport ? XSSPurify.purifyObject(valuePacket) : valuePacket;
    this.invalid = invalid;
    this.input = valid;
    return this.submission = submission;
  }
  __reflectValidatedStatus(inputEl) {
    inputEl.setAttribute("validated", "validated");
  }
  update(_input) {
    return this.refresh(_input);
  }
  refresh(_input) {
    if (!_input) {
      return;
    }
    Promise.resolve().then(() => {
      HIP.db.gid.queryAll(`_form__iterator`, this.ns).forEach((iteratorEl) => {
        iteratorEl.input = f1.read(
          f1.path(iteratorEl.getAttribute("value-path") || ""),
          void 0,
          _input
        );
      });
      HIP.db.form.queryAll(this.form, this.ns).forEach((inputEl) => {
        const modelPath = this.__readModelPath(inputEl);
        const value = f1.read(modelPath, void 0, _input);
        if (inputEl.locked) {
          return;
        }
        if (void 0 !== value) {
          inputEl.value = value;
        }
      });
    });
  }
  __readModelPath(inputEl) {
    return f1.path(inputEl.modelPath || inputEl.getAttribute("name") || "");
  }
}
HIP.factory.element(ValueGroupElement);
export default "hi-value-group";
