
import { LitElement } from '../../../hip/library/lit/lit.js';
import { AsLitHIC, BehaviorCustomizable, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import { Ignition } from '../../../pip/library/pi-interop/pi-interop.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import $ip from '../../../pip/library/pi-ip/pi-ip.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class RegionElement extends BehaviorCustomizable(BehaviorElement(AsLitHIC(LitElement))) {
  static get is() {
    return "fi-region";
  }
  static get properties() {
    return {
      uri: {
        type: String
      },
      settings: {
        type: Object
      }
    };
  }
  static get api() {
    return ["invoke"];
  }
  static get autoBinding() {
    return {
      signal: "invoke-ended",
      field: "invoke"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.classList.add("display-block");
    dom.setAttrs(this, {
      role: "region"
    });
  }
  createRenderRoot() {
    return this;
  }
  update(props) {
    super.update(props);
    if (props.has("uri")) {
      this.invoke(this.uri);
    }
  }
  async invoke(uri = this.uri) {
    try {
      const RegionView = await $ip.require(uri);
      const regionView = f1.is.class(RegionView) ? new RegionView() : RegionView;
      const I = Ignition.create();
      await I.startup(regionView, f1.safe({}, this.settings));
      await I.mount(regionView, this);
      return regionView;
    } catch (e) {
      console.warn("view failed to load");
      return {
        id: "non-existent view"
      };
    }
  }
}
export default HIP.factory.element(RegionElement);
