import { noChange } from '../lit-html.js';
import { isPrimitive } from '../directive-helpers.js';
import { AsyncDirective } from '../async-directive.js';
import { PseudoWeakRef, Pauser } from './private-async-helpers.js';
import { directive } from '../directive.js';

const isPromise = (x) => {
    return !isPrimitive(x) && typeof x.then === 'function';
};
const _infinity = 0x3fffffff;
class UntilDirective extends AsyncDirective {
    constructor() {
        super(...arguments);
        this.__lastRenderedIndex = _infinity;
        this.__values = [];
        this.__weakThis = new PseudoWeakRef(this);
        this.__pauser = new Pauser();
    }
    render(...args) {
        var _a;
        return (_a = args.find((x) => !isPromise(x))) !== null && _a !== void 0 ? _a : noChange;
    }
    update(_part, args) {
        const previousValues = this.__values;
        let previousLength = previousValues.length;
        this.__values = args;
        const weakThis = this.__weakThis;
        const pauser = this.__pauser;
        if (!this.isConnected) {
            this.disconnected();
        }
        for (let i = 0; i < args.length; i++) {
            if (i > this.__lastRenderedIndex) {
                break;
            }
            const value = args[i];
            if (!isPromise(value)) {
                this.__lastRenderedIndex = i;
                return value;
            }
            if (i < previousLength && value === previousValues[i]) {
                continue;
            }
            this.__lastRenderedIndex = _infinity;
            previousLength = 0;
            Promise.resolve(value).then(async (result) => {
                while (pauser.get()) {
                    await pauser.get();
                }
                const _this = weakThis.deref();
                if (_this !== undefined) {
                    const index = _this.__values.indexOf(value);
                    if (index > -1 && index < _this.__lastRenderedIndex) {
                        _this.__lastRenderedIndex = index;
                        _this.setValue(result);
                    }
                }
            });
        }
        return noChange;
    }
    disconnected() {
        this.__weakThis.disconnect();
        this.__pauser.pause();
    }
    reconnected() {
        this.__weakThis.reconnect(this);
        this.__pauser.resume();
    }
}
const until = directive(UntilDirective);

export { UntilDirective, until };

//# sourceMappingURL=until.js.map
