import './boot.js';

const policy = window.trustedTypes &&
    trustedTypes.createPolicy('polymer-html-literal', {createHTML: (s) => s});

class LiteralString {
  constructor(strings, values) {
    assertValidTemplateStringParameters(strings, values);
    const string = values.reduce(
        (acc, v, idx) => acc + literalValue(v) + strings[idx + 1], strings[0]);
    this.value = string.toString();
  }
  toString() {
    return this.value;
  }
}

function literalValue(value) {
  if (value instanceof LiteralString) {
    return (value).value;
  } else {
    throw new Error(
        `non-literal value passed to Polymer's htmlLiteral function: ${value}`
    );
  }
}

function htmlValue(value) {
  if (value instanceof HTMLTemplateElement) {
    return (value).innerHTML;
  } else if (value instanceof LiteralString) {
    return literalValue(value);
  } else {
    throw new Error(
        `non-template value passed to Polymer's html function: ${value}`);
  }
}

export const html = function html(strings, ...values) {
  assertValidTemplateStringParameters(strings, values);
  const template =
 (document.createElement('template'));
  let value = values.reduce(
      (acc, v, idx) => acc + htmlValue(v) + strings[idx + 1], strings[0]);
  if (policy) {
    value = policy.createHTML(value);
  }
  template.innerHTML = value;
  return template;
};

const assertValidTemplateStringParameters = (strings, values) => {
  if (!Array.isArray(strings) || !Array.isArray(strings.raw) ||
      (values.length !== strings.length - 1)) {
    throw new TypeError('Invalid call to the html template tag');
  }
};

export const htmlLiteral = function(strings, ...values) {
  return new LiteralString(strings, values);
};

//# sourceMappingURL=html-tag.js.map
