import './boot.js';
import { pathFromUrl } from './resolve-url.js';
export const useShadow = !(window.ShadyDOM) || !(window.ShadyDOM.inUse);
export const useNativeCSSProperties = Boolean(!window.ShadyCSS || window.ShadyCSS.nativeCss);
export const useNativeCustomElements = !(window.customElements.polyfillWrapFlushCallback);
export const supportsAdoptingStyleSheets = useShadow &&
    ('adoptedStyleSheets' in Document.prototype) &&
    ('replaceSync' in CSSStyleSheet.prototype) &&
    (() => {
      try {
        const sheet = new CSSStyleSheet();
        sheet.replaceSync('');
        const host = document.createElement('div');
        host.attachShadow({mode: 'open'});
        host.shadowRoot.adoptedStyleSheets = [sheet];
        return (host.shadowRoot.adoptedStyleSheets[0] === sheet);
      } catch(e) {
        return false;
      }
    })();

export let rootPath = window.Polymer && window.Polymer.rootPath ||
  pathFromUrl(document.baseURI || window.location.href);

export const setRootPath = function(path) {
  rootPath = path;
};

export let sanitizeDOMValue =
  window.Polymer && window.Polymer.sanitizeDOMValue || undefined;

export const setSanitizeDOMValue = function(newSanitizeDOMValue) {
  sanitizeDOMValue = newSanitizeDOMValue;
};

export const getSanitizeDOMValue = function() {
  return sanitizeDOMValue;
};

export let passiveTouchGestures =
  window.Polymer && window.Polymer.setPassiveTouchGestures || false;

export const setPassiveTouchGestures = function(usePassive) {
  passiveTouchGestures = usePassive;
};

export let strictTemplatePolicy =
  window.Polymer && window.Polymer.strictTemplatePolicy || false;

export const setStrictTemplatePolicy = function(useStrictPolicy) {
  strictTemplatePolicy = useStrictPolicy;
};

export let allowTemplateFromDomModule =
  window.Polymer && window.Polymer.allowTemplateFromDomModule || false;

export const setAllowTemplateFromDomModule = function(allowDomModule) {
  allowTemplateFromDomModule = allowDomModule;
};

export let legacyOptimizations =
  window.Polymer && window.Polymer.legacyOptimizations || false;

export const setLegacyOptimizations = function(useLegacyOptimizations) {
  legacyOptimizations = useLegacyOptimizations;
};

export let legacyWarnings =
  window.Polymer && window.Polymer.legacyWarnings || false;

export const setLegacyWarnings = function(useLegacyWarnings) {
  legacyWarnings = useLegacyWarnings;
};

export let syncInitialRender =
  window.Polymer && window.Polymer.syncInitialRender || false;

export const setSyncInitialRender = function(useSyncInitialRender) {
  syncInitialRender = useSyncInitialRender;
};

export let legacyUndefined =
  window.Polymer && window.Polymer.legacyUndefined || false;

export const setLegacyUndefined = function(useLegacyUndefined) {
  legacyUndefined = useLegacyUndefined;
};

export let orderedComputed =
  window.Polymer && window.Polymer.orderedComputed || false;

export const setOrderedComputed = function(useOrderedComputed) {
  orderedComputed = useOrderedComputed;
};

export let cancelSyntheticClickEvents = true;

export const setCancelSyntheticClickEvents = function(useCancelSyntheticClickEvents) {
  cancelSyntheticClickEvents = useCancelSyntheticClickEvents;
};

export let removeNestedTemplates =
  window.Polymer && window.Polymer.removeNestedTemplates || false;

export const setRemoveNestedTemplates = function(useRemoveNestedTemplates) {
  removeNestedTemplates = useRemoveNestedTemplates;
};

export let fastDomIf = window.Polymer && window.Polymer.fastDomIf || false;

export const setFastDomIf = function(useFastDomIf) {
  fastDomIf = useFastDomIf;
};

export let suppressTemplateNotifications =
  window.Polymer && window.Polymer.suppressTemplateNotifications || false;

export const setSuppressTemplateNotifications = function(suppress) {
  suppressTemplateNotifications = suppress;
};

export let legacyNoObservedAttributes =
  window.Polymer && window.Polymer.legacyNoObservedAttributes || false;

export const setLegacyNoObservedAttributes = function(noObservedAttributes) {
  legacyNoObservedAttributes = noObservedAttributes;
};

export let useAdoptedStyleSheetsWithBuiltCSS =
  window.Polymer && window.Polymer.useAdoptedStyleSheetsWithBuiltCSS || false;

export const setUseAdoptedStyleSheetsWithBuiltCSS = function(value) {
  useAdoptedStyleSheetsWithBuiltCSS = value;
};

//# sourceMappingURL=settings.js.map
