
import { html, LitElement } from '../../../hip/library/lit-element/lit-element.js';
import { ref, createRef } from '../../../hip/library/lit-html/directives/ref.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { HIP, AsLitHIC, BehaviorElement } from '../../../eip/element/hi-hip/hi-hif.js';
import '../../../vip/element/vi-table/vi-table.js';
import '../../../eip/element/hi-button/hi-button.js';
import renderer from '../../../vip/library/vi-renderer/vi-renderer.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class ChartElement extends BehaviorElement(AsLitHIC(LitElement)) {
  constructor() {
    super(...arguments);
    this._id = f1.uuid();
    this.library = "chart";
    this.toggleView = "chart";
    this.mode = "hrf";
    this.hoverTarget = -1;
    this.tableElRef = createRef();
  }
  static get is() {
    return "vi-chart";
  }
  static get properties() {
    return {
      tableActionText: {
        type: String,
        reflect: true,
        attribute: "table-action-text"
      },
      allowTableAction: {
        type: Boolean,
        reflect: true,
        attribute: "allow-table-action"
      },
      allowToggle: {
        type: Boolean,
        reflect: true,
        attribute: "allow-toggle"
      },
      autoSize: {
        type: Boolean,
        attribute: "auto-size"
      },
      height: {
        type: String,
        reflect: true
      },
      width: {
        type: String,
        reflect: true
      },
      type: {
        type: Object,
        reflect: true
      },
      input: {
        type: Object,
        reflect: true
      },
      library: {
        type: String,
        reflect: true
      },
      kind: {
        type: String,
        reflect: true
      },
      toggleView: {
        type: String
      },
      mode: {
        type: String,
        reflect: true
      }
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.addEventListener("hover-start-ended", this.__handleHover);
    this.__tableEl__onActionEndedBound = this.__tableEl__onActionEnded.bind(this);
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    const tableEl = this.tableElRef.value;
    tableEl?.removeEventListener("action-ended", this.__tableEl__onActionEndedBound);
    const canvasEl = this.querySelector(`[id='${this._id}_chartContainer']`);
    if (canvasEl) {
      canvasEl.__instance.destroy();
    }
  }
  createRenderRoot() {
    return this;
  }
  render() {
    return this.library === "chart" || this.library === "chart_extended" ? html` ${this._renderToggleButton()} ${this._renderChart()} ${this._renderTable()}` : this._renderGraph();
  }
  _renderGraph() {
    return html`<div id=${this._chartId()} style=${this._chartStyle()}></div>`;
  }
  _renderToggleButton() {
    return this.allowToggle ? html`<hi-button
          id="${this._id}_${this.library}_toggleBtn"
          style="float:right; margin-bottom:1em; margin-top:0.25em;"
          aesthetic="secondary"
          size="small"
          theme="gray_light"
          @trigger-ended="${this.__toggleState__onTriggerEnded}"
        >
          ${this.toggleView === "chart" ? "Show Table" : "Show Chart"}
        </hi-button>` : nothing;
  }
  _renderChart() {
    return this.toggleView !== "chart" ? nothing : this.__chartStrategy();
  }
  _renderTable() {
    return html`<vi-table
      ${ref(this.tableElRef)}
      .type=${this.type}
      .input=${this.input}
      .mode=${this.mode}
      style=${this._tableStyle()}
      ?allow-action=${this.allowTableAction}
      action-text=${this.tableActionText ? this.tableActionText : "Action"}
    >
    </vi-table>`;
  }
  _chartId() {
    return `${this._id}_${this.library}_container`;
  }
  _chartStyle() {
    return this.autoSize ? `width:92%; height:92%; margin: var(--hi-px-8) auto` : `width:${this.width || `352px`}; height:${this.height || `352px`}`;
  }
  _tableStyle() {
    return this.toggleView === "table" ? "" : "position:absolute;width: 1px;height: 1px;padding: 0;margin: -1px;overflow: hidden;clip: rect(0, 0, 0, 0);white-space: nowrap;border: 0;";
  }
  __chartStrategy() {
    return html`<div id="parent" style=${this._chartStyle()}>
      <canvas id=${this._chartId()} role="img" aria-hidden="true"></canvas>
    </div>`;
  }
  updated() {
    this.renderChart();
    if (this.allowTableAction) {
      this.__allowTableActionWatcher();
    }
  }
  async renderChart() {
    const containerEl = this.querySelector(`[id="${this._chartId()}"]`);
    const canRender = this.input != null && containerEl;
    if (!canRender) {
      return;
    }
    this.broker = f1.safe(await renderer.createBroker(this.library), this.broker);
    this.broker.render(containerEl, this.kind, this.type, this.input, this.mode, this.library).then((renderer2) => {
      this.renderer = renderer2;
    });
  }
  __allowTableActionWatcher() {
    const tableEl = this.tableElRef.value;
    if (tableEl && this.toggleView === "table") {
      tableEl.addEventListener("action-ended", this.__tableEl__onActionEndedBound);
    }
  }
  __tableEl__onActionEnded(event) {
    this._dispatchProcess("row-action", {
      detail: event.detail.data
    });
  }
  __handleHover(evt) {
    if (evt.detail.index !== this.hoverTarget && evt.detail.index !== -1) {
      this._dispatchProcess("hover-end", {
        detail: evt.detail
      });
    }
    this.hoverTarget = evt.detail.index;
  }
  __toggleState__onTriggerEnded(evt) {
    this.toggleView = this.toggleView === "chart" ? "table" : "chart";
  }
}
export default HIP.factory.element(ChartElement);
