
import { html, css, LitElement } from '../../../hip/library/lit-element/lit-element.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { HIP, AsLitHIC, BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export const styles = css`
  .columns-container {
    border-right: var(--hi-px-3) solid var(--hi-clr--gray-8);
  }

  .label-unit,
  .label-total {
    font-size: var(--hi-font-size--medium);
    line-height: var(--hi-el--height-sml);
    height: var(--hi-el--height-sml);
  }

  .label-total {
    text-align: end;
    margin-right: var(--hi-fixed--small-xx);
  }

  .progress-bar-container {
    background: var(--hi-clr--white);
    display: flex;
    list-style: none;
    padding: 0;
    margin: 0;
  }

  .column-block {
    line-height: var(--hi-el--height);
    height: var(--hi-el--height);
    font-size: var(--hi-font-size--large);
    border-right: var(--hi-px-3) solid var(--hi-clr--white);
    color: var(--hi-clr--white);
    text-align: center;
    font-weight: bold;
    overflow: auto;
  }

  .label-container {
    list-style: none;
    padding: 0;
    margin: 0;
  }

  .label-unit {
    border-left: var(--hi-px-3) solid var(--hi-clr--transparent);
    padding-left: var(--hi-fixed--small-xx);
  }
`;
export class ColumnsElement extends BehaviorElement(AsLitHIC(LitElement)) {
  static get is() {
    return "vi-columns";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      total: {
        type: Object,
        reflect: true
      },
      input: {
        type: Object,
        reflect: true
      }
    };
  }
  render() {
    const chartData = f1.safe([], f1.read("/data", this.input, this.input));
    const total = this.total;
    return html`
      <section class="columns-container">
        <p class="label-total">${total.title} (${total.value})</p>
        <ul class="progress-bar-container" role="presentation" aria-hidden="true" style="background: transparent;">
          ${f1.map((unit) => {
      const calculateBarWidth = unit.value / total.value * 100;
      const style = `background: ${unit.color}; width:${calculateBarWidth}%`;
      return html`<li class="column-block" style=${style}>${unit.value}</li>`;
    }, chartData)}
        </ul>
      </section>
      <ul class="label-container">
        ${f1.map((unit, i) => {
      const style = `margin-left:${this.__computeLabelPosition(i, chartData)}%; border-left-color:${unit.color};`;
      return html` <li class="label-unit" style=${style}>${unit.title} (${unit.value})</li> `;
    }, chartData)}
      </ul>
    `;
  }
  __computeLabelPosition(index, data) {
    if (index === 0) {
      return 0;
    }
    const total = this.total.value;
    let width = 0;
    let i = 0;
    for (; i < index; i++) {
      width += data[i].value / total * 100;
    }
    return width;
  }
}
export default HIP.factory.element(ColumnsElement);
