
import { html, css, LitElement } from '../../../hip/library/lit-element/lit-element.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { AsLitHIC, BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export const styles = css`
  .funnelUlContainer {
    padding: var(--hi-px-0);
    margin: var(--hi-px-0);
    width: 100%;
  }

  .funnelLiContainer {
    display: flex;
    align-items: center;
    margin: 0;
    padding: 0;
    list-style: none;
    min-width: 70%;
  }

  .funnelBarContainer {
    width: 100%;
  }

  .funnelBar {
    display: flex;
    justify-content: center;
    align-items: center;
    margin: var(--hi-px-4) auto;
    height: calc(var(--hi-px-32) * 2);
    padding: var(--hi-px-24);
    background-color: #2c3f66;
    color: white;
    list-style: none;
    box-shadow: var(--hi-px-0) var(--hi-px-6) #d7e1ff;
  }

  .funnelLiContainer:last-child > .funnelBarContainer > .funnelBar {
    box-shadow: none !important;
  }

  span[id*='title_container'] {
    min-width: 15%;
    max-width: 20%;
    text-overflow: ellipsis;
    overflow-x: clip;
    margin-right: var(--hi-px-4);
  }

  span[id*='percentage_container'] {
    min-width: 15%;
    max-width: 20%;
    margin-left: var(--hi-px-4);
  }
`;
export class FunnelElement extends BehaviorElement(AsLitHIC(LitElement)) {
  constructor() {
    super(...arguments);
    this._id = f1.uuid();
  }
  static get is() {
    return "vi-funnel";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      input: {
        type: Object,
        reflect: true
      }
    };
  }
  render() {
    const hrf = f1.safe({}, this.input);
    const data = f1.to.array(f1.read("/data", [], hrf));
    const sortedHrf = data.sort((a, b) => f1.to.real(b.value) - f1.to.real(a.value));
    const valueOfIndex0 = f1.to.real(f1.read("/0/value", "0.0", sortedHrf));
    return html`
      <ul class="funnelUlContainer">
        ${sortedHrf.map((o, index) => {
      const newIndex = index + 1;
      const value = o.value;
      const calculatenthChildWidth = (value * 100 / valueOfIndex0).toFixed(2);
      const widthStyle = `${calculatenthChildWidth}%`;
      return html`
            <li class="funnelLiContainer" index=${newIndex} id="child_funnel_${newIndex}">
              <span id="title_container_${newIndex}"> ${o.title} </span>
              <div class="funnelBarContainer">
                <div class="funnelBar" style="width:${widthStyle}">${o.value}</div>
              </div>
              <span id="percentage_container_${newIndex}"> ${calculatenthChildWidth}% </span>
            </li>
          `;
    })}
      </ul>
    `;
  }
}
export default HIP.factory.element(FunnelElement);
