
import { Debouncer } from '../../../hip/library/polymer/polymer.js';
import { idlePeriod } from '../../../hip/library/polymer/polymer.js';
import { html, css, LitElement } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { AsLitHIC, BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import { FactoryEditElement } from '../../../aip/element/ai-factory-edit/ai-factory-edit.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import { setAttributes } from '../../../eip/element/hi-lit/hi-lit.js';
import adt from '../../../pip/library/pi-adt/pi-adt.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import { computeCellStyle, gridStyles } from './vi-grid-shared-css.js';
import '../../../eip/element/hi-button/hi-button.js';
import '../../../eip/element/hi-toggle/hi-toggle.js';
import '../../../eip/element/hi-menu/hi-menu.js';
import '../../../eip/element/hi-menu/hi-pick.js';
import '../../../eip/element/hi-hint/hi-hint.js';
import './vi-grid-filter.js';
export const styles = css`
  :host {
    width: 100%;
    display: block;
    height: var(--hi-table--column-height);
  }

  .grid__row {
    height: var(--hi-table--column-height);
    line-height: var(--hi-table--column-height);
  }

  .row__cell {
    position: relative;
    background-color: var(--hi-clr--white);
    border-right: var(--hi-px-1) solid var(--hi-clr--gray-4);
    border-bottom: var(--hi-px-2) solid var(--hi-clr--gray-6);
  }

  .column__title_corner {
    margin-left: var(--hi-px-4);
  }

  .cell__title {
    text-overflow: ellipsis;
    white-space: nowrap;
    overflow: hidden;

    margin-right: auto;

    font-variant: small-caps;
    text-transform: capitalize;
    letter-spacing: 0.025em;
  }

  .cell__menu_list {
    flex-direction: column;
    display: flex;
  }

  .cell__menu_button {
    margin-right: 0.2rem;
  }

  .cell__info_hint {
    margin-left: 0.3rem;
    margin-top: 0.3rem;
  }

  .resizer {
    cursor: ew-resize;
    position: absolute;
    bottom: 0;
    right: 0;
    top: 0;
    width: var(--hi-px-4);
    background: var(--hi-clr--blue-3);
    opacity: 0;
    transition: opacity 0.35s ease-in-out;
  }

  .is-active > .resizer,
  .resizer:hover {
    opacity: 1;
  }

  .row--action > hi-icon {
    margin: auto;
  }
`;
export class GridColumnsElement extends BehaviorElement(AsLitHIC(LitElement)) {
  constructor() {
    super();
    this.editFactory = new FactoryEditElement();
    this.__columnResizer__onMouseMoveBound = this.__columnResizer__onMouseMove.bind(this);
    this.__columnResizer__onMouseUpBound = this.__columnResizer__onMouseUp.bind(this);
  }
  static get is() {
    return "vi-grid-columns";
  }
  static get styles() {
    return [stylesTheme, gridStyles, styles];
  }
  static get properties() {
    return {
      group: {
        type: String
      },
      maxSelections: {
        type: Number
      },
      mutationOptions: {
        type: Object
      },
      fields: {
        type: Array
      },
      input: {
        type: Array
      },
      entireInput: {
        type: Array
      },
      settings: {
        type: Object
      },
      selections: {
        type: Object
      },
      state: {
        type: Object
      },
      rowActions: {
        type: Object
      },
      comparators: {
        type: Object
      }
    };
  }
  render() {
    const emptyRow = {};
    const fields = this.fields;
    const mutationOptions = this.mutationOptions;
    const comparators = this.comparators;
    if (f1.is.empty(fields) || f1.is.empty(mutationOptions)) {
      return nothing;
    }
    return html`<div role="row" class="grid__row">
      ${this.__renderSelector(fields, this.selections, this.mutationOptions)} ${this.__renderAction(fields)}
      ${fields.map((_field, index) => {
      const field = adt.expression.evaluateAll(_field, emptyRow);
      const columnName = field.name;
      const titleId = `title-${columnName}`;
      const frozenCSS = index === 0 && this.settings.frozen_column ? "row--frozen" : "";
      const fieldName = field.name;
      const hasMatchingFieldsFn = f1.find(({ field: field2 }) => field2 === fieldName);
      const hasActiveSort = hasMatchingFieldsFn(mutationOptions.sort_by);
      const hasActiveFilter = hasMatchingFieldsFn(mutationOptions.filter_by);
      const isFieldRankable = f1.is.true(field.rankable);
      const hasExternalComparators = f1.keys(comparators).includes(field.render_type);
      const isFieldFilterable = this.settings.allow.table_filter && (field.filterable || hasExternalComparators);
      return html`<div
          role="columnheader"
          class="row__cell column--${columnName} ${frozenCSS}"
          data-attrs=${setAttributes(field.attributes)}
          style=${computeCellStyle(field)}
          data-column=${columnName}
          data-index=${index}
        >
          ${!hasActiveFilter && !hasActiveSort && !isFieldRankable ? nothing : html`
                <hi-corner class="column__title_corner" dock="top-left" inside-edge>
                  ${!hasActiveFilter ? nothing : html`<hi-icon kind="filter" size="tiny"></hi-icon>`}
                  ${!hasActiveSort ? nothing : html`<hi-icon kind="sort" size="tiny"></hi-icon>`}
                  ${!isFieldRankable ? nothing : html`<hi-icon kind="sort" size="tiny"></hi-icon>`}
                </hi-corner>
              `}
          ${isFieldFilterable ? this.__renderFilterableColumnTitle(titleId, field, mutationOptions, comparators) : this.__renderColumnTitle(titleId, field)}
          ${!field.hint ? nothing : html`<span class="cell__info_hint">
                <span>
                  <hi-icon gid="hint-tooltip-${columnName}" .ns=${this.ns} kind="info" type="material"></hi-icon>
                  <hi-tooltip for="hint-tooltip-${columnName}" .ns=${this.ns} placement="bottom-start"
                    >${field.hint}</hi-tooltip
                  >
                </span>
              </span>`}
          ${this.__renderSortMenu(field, mutationOptions)} ${this.__renderResizer(field, mutationOptions)}
        </div>`;
    })}
    </div>`;
  }
  __renderFilterableColumnTitle(titleId, field, mutationOptions, comparators) {
    return html`
      <hi-flyout id="gridFilterFlyout__${field.name}" aesthetic="text" alignment="bottom" kind="sticky" .ns=${this.ns}>
        <div slot="anchor">${this.__renderColumnTitle(titleId, field)}</div>

        <vi-grid-filter
          .mutationOptions=${mutationOptions}
          .comparators=${comparators}
          .editFactory=${this.editFactory}
          .field=${field}
          .ns=${this.ns}
          @state-action-notified=${this.__gridFilter__onStateActionNotified}
        ></vi-grid-filter>
      </hi-flyout>
    `;
  }
  __renderColumnTitle(titleId, field) {
    return html`<span id="${titleId}" class="cell__title">${field.title}</span>`;
  }
  __renderSelector(fields, selections, mutationOptions) {
    if (!this.settings.allow.row_selections) {
      return nothing;
    }
    const input = this.input;
    const maxInputSize = f1.safe(0, mutationOptions.page_size);
    const selected = f1.intersection(input, selections);
    const totalSelected = f1.size(selected);
    const state = totalSelected <= 0 ? "unchecked" : totalSelected >= maxInputSize ? "checked" : "partial";
    return html` <div role="columnheader" class="row__cell row--selector row--frozen-fixed">
      <hi-checkbox
        id="rowSelected__checkboxControl"
        kind="group"
        aria-label="Toggle Bulk Selection Of Rows"
        .state=${state}
        .ns=${this.ns}
        @toggle-ended=${this.__checkboxControl__onToggleEnded}
      ></hi-checkbox>
    </div>`;
  }
  __renderAction(fields) {
    if (!this.rowActions || this.rowActions.length <= 0) {
      return nothing;
    }
    return html` <div role="columnheader" class="row__cell row--action row--frozen-fixed">
      <hi-icon glyph="menu" type="material" size="large"></hi-icon>
      <hi-hint placement="right">${this.settings.labels.row_action_header}</hi-hint>
    </div>`;
  }
  __renderSortMenu(field, mutationOptions) {
    if (f1.is.true(field.rankable) || f1.is.false(field.sortable) || !this.settings.allow.table_sort) {
      return nothing;
    }
    const columnName = field.name;
    const sortButtonId = `column-sort-button-${columnName}`;
    return html`
      <hi-button
        id=${sortButtonId}
        size="small"
        aesthetic="icon"
        purpose="columnSort"
        class="cell__menu_button"
        .ns=${this.ns}
        .value=${field}
        @trigger-ended=${this.__columnSort__onTriggerEnded}
      >
        <hi-icon
          type="material"
          kind=${this.__columnSortIcon(field, mutationOptions)}
          direction=${this.__columnSortDirection(field, mutationOptions)}
        ></hi-icon>
        <span class="show-ax">Toggle sort for column ${field.title}</span>
      </hi-button>
    `;
  }
  __columnSort__onTriggerEnded(event) {
    const commandName = "sortBy";
    const field = event.detail.value;
    const sortOrder = f1.read("/0/order", "asc", this.mutationOptions.sort_by);
    const mutationOptions = {
      sort_by: [
        {
          order: sortOrder === "asc" ? "desc" : "asc",
          field: field.name
        }
      ]
    };
    this._dispatchNotification("data-action", {
      commandName,
      mutationOptions
    });
  }
  __columnSortIcon(field, mutationOptions) {
    return f1.find((_) => _.field === field.name, mutationOptions.sort_by) ? "chevron_up" : "expand_vertical";
  }
  __columnSortDirection(field, mutationOptions) {
    const sortColumn = f1.find((_) => _.field === field.name, mutationOptions.sort_by);
    if (!sortColumn) {
      return "up";
    }
    return sortColumn.order === "asc" ? "up" : "down";
  }
  __gridFilter__onStateActionNotified(event) {
    const gridFilterEl = event.target;
    const flyoutEl = this.shadowRoot.getElementById(`gridFilterFlyout__${gridFilterEl.field.name}`);
    if (flyoutEl) {
      flyoutEl.close();
    }
  }
  __checkboxControl__onToggleEnded(event) {
    const checkboxEl = this.__getCheckbox();
    if (!checkboxEl) {
      return;
    }
    if ("checked" === checkboxEl.state) {
      this._dispatchNotification("row-select-all-page");
    } else if ("unchecked" === checkboxEl.state) {
      this._dispatchNotification("row-unselect-none-page");
      this._dispatchNotification("view-state", {
        viewState: "list"
      });
    }
  }
  __getCheckbox() {
    return this.shadowRoot.getElementById("rowSelected__checkboxControl");
  }
  __renderResizer(field, mutationOptions) {
    return html`<div class="resizer" .field=${field} @mousedown=${this.__columnResizer__onMouseDown}></div>`;
  }
  __columnResizer__onMouseDown(e) {
    const field = f1.read("/target/field", { name: "" }, e);
    const columnHeaderEl = this.shadowRoot.querySelector(
      `.column--${field.name}`
    );
    if (!columnHeaderEl) {
      return;
    }
    const styles2 = window.getComputedStyle(columnHeaderEl);
    this.__clientX = e.clientX;
    this.__columnWidth = parseInt(styles2.width, 10);
    this.__columnHeaderEl = columnHeaderEl;
    this.__columnHeaderName = columnHeaderEl.getAttribute("data-column") || "";
    this.__columnHeaderEl.classList.add("is-active");
    document.addEventListener("mousemove", this.__columnResizer__onMouseMoveBound);
    document.addEventListener("mouseup", this.__columnResizer__onMouseUpBound);
  }
  __columnResizer__onMouseMove(e) {
    this.__columnResizerDebouncer = Debouncer.debounce(this.__columnResizerDebouncer, idlePeriod, () => {
      const columnMinWidth = 80;
      const fieldName = this.__columnHeaderName;
      const css4Column = `--grid--column-${fieldName}`;
      const dx = e.clientX - this.__clientX;
      const columnWidth = f1.max(columnMinWidth, this.__columnWidth + dx);
      const widthPx = `${columnWidth}px`;
      this._dispatchNotification("resize-column", {
        fieldName,
        css4Column,
        widthPx
      });
    });
  }
  __columnResizer__onMouseUp(e) {
    this.__columnHeaderEl.classList.remove("is-active");
    document.removeEventListener("mousemove", this.__columnResizer__onMouseMoveBound);
    document.removeEventListener("mouseup", this.__columnResizer__onMouseUpBound);
  }
}
export default HIP.factory.element(GridColumnsElement);
