
import { html, css, LitElement } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { AsLitHIC, BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import adt from '../../../pip/library/pi-adt/pi-adt.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import { gridStyles } from './vi-grid-shared-css.js';
import '../../../eip/element/hi-input/hi-input-group.js';
import '../../../eip/element/hi-input/hi-input.js';
import '../../../eip/element/hi-form/hi-form.js';
export const styles = css`
  :host {
    display: block;
    width: inherit;
    padding: var(--hi-px-12);
  }

  #filterComparator {
    max-height: calc(4 * var(--hi-px-48));
  }

  hi-input-group:first-child {
    margin-bottom: var(--hi-px-8);
  }
`;
export class GridFilterElement extends BehaviorElement(AsLitHIC(LitElement)) {
  static get is() {
    return "vi-grid-filter";
  }
  static get styles() {
    return [stylesTheme, gridStyles, styles];
  }
  static get properties() {
    return {
      field: {
        type: Object
      },
      mutationOptions: {
        type: Array
      },
      input: {
        type: Array
      },
      state: {
        type: Object
      }
    };
  }
  render() {
    const mutationOptions = this.mutationOptions;
    const externalComparators = this.comparators;
    if (f1.is.empty(mutationOptions) && f1.is.empty(externalComparators)) {
      return nothing;
    }
    const ns = this.ns;
    const field = this.field;
    const form = `grid-filter-group-${f1.id()}`;
    const fieldName = field.name;
    const filterBy = (this.mutationOptions || {}).filter_by || [];
    const filter = f1.safe(
      {
        field: fieldName
      },
      f1.find((_) => _.field === fieldName, filterBy)
    );
    this.editFactory.form = form;
    this.editFactory.ns = ns;
    const filterType = f1.read("/filter_type", f1.read("/render_type", "unknown", field), field);
    const editArtifact = this.editFactory.renderBy("edit", filterType);
    const editFn = editArtifact.render(
      {
        ...field,
        name: "value",
        read_only: false
      },
      this.editFactory
    );
    const comparators = !f1.is.empty(externalComparators) ? externalComparators[filterType] : adt.filterComparators(filterType);
    return html` <hi-form
      class="filters"
      form=${form}
      .ns=${ns}
      disable-http
      @invoke-ended=${this.__form__onInvokeEnded}
    >
      <hi-input-group>
        <hi-label required .ns=${ns}>Condition</hi-label>
        <hi-dropdown form=${form} .ns=${ns}>
          <hi-select
            id="filterComparator"
            name="comparator"
            required
            mode="value"
            form=${form}
            .ns=${ns}
            .value=${filter.comparator}
          >
            ${comparators.map((c) => html`<hi-option value=${c.id}>${c.title}</hi-option>`)}
          </hi-select>
        </hi-dropdown>
      </hi-input-group>
      <hi-input-group>
        <hi-label required .ns=${ns}>Filter By</hi-label>
        ${editFn(filter.value)}
      </hi-input-group>
      <hi-button type="submit" slot="controls" .ns=${ns}>
        <hi-icon glyph="filter"></hi-icon>
        <span>Apply</span>
      </hi-button>
      <hi-button
        type="clear"
        slot="controls"
        aesthetic="secondary"
        .ns=${ns}
        @trigger-ended=${this.__clear__onTriggerEnded}
        >Clear</hi-button
      >
    </hi-form>`;
  }
  __clear__onTriggerEnded(event) {
    const filter_by = this.filterBy = f1.remove(
      (_) => _.field === this.field.name,
      (this.mutationOptions || {}).filter_by || []
    );
    this._dispatchNotification("state-action", {
      mutationOptions: {
        filter_by
      }
    });
  }
  __form__onInvokeEnded(event) {
    const field = this.field.name;
    const value = event.detail.value[field];
    const existingFilters = f1.remove((_) => _.field === field, (this.mutationOptions || {}).filter_by || []);
    const filter_by = f1.push(
      {
        ...event.detail.value,
        field,
        value
      },
      existingFilters
    );
    this._dispatchNotification("state-action", {
      mutationOptions: {
        page_index: 0,
        filter_by
      }
    });
  }
}
export default HIP.factory.element(GridFilterElement);
