
import { css, LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { AsLitHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { gridStyles } from './vi-grid-shared-css.js';
import '../../../eip/element/hi-pagination/hi-pagination.js';
export const style = css`
  :host {
    width: 100%;
    display: block;
    height: var(--grid--footer-height);
    background-color: var(--hi-clr--white);
    border-top: var(--hi-px-1) solid var(--hi-clr--gray-4);
  }

  .page-size {
    align-items: center;
    display: flex;
    gap: var(--hi-px-6);
  }

  /*
   * Adding styling of hi-label to span to keep the same look, since we replaced it for AX.
   */
  .paginationDetails {
    line-height: var(--hi-font-lh--text);
    font-size: var(--hi-font-size--text);
    font-weight: var(--hi-font--weight-light);
  }

  #pageSizeInput {
    max-width: calc(2 * var(--hi-px-36));
  }
`;
export class GridFooterElement extends BehaviorElement(AsLitHIC(LitElement)) {
  constructor() {
    super(...arguments);
    this.selections = [];
    this.__renderStates = {
      empty(mutationOptions, fields, input) {
        return this.__renderGridFooter(mutationOptions, fields, input);
      },
      error() {
        return nothing;
      },
      columns() {
        return nothing;
      },
      settings() {
        return nothing;
      },
      sort() {
        return nothing;
      },
      filter() {
        return nothing;
      },
      presentation(mutationOptions, fields, input) {
        return this.__renderGridFooter(mutationOptions, fields, input);
      }
    };
  }
  static get is() {
    return "vi-grid-footer";
  }
  static get styles() {
    return [stylesTheme, gridStyles, style];
  }
  static get properties() {
    return {
      totalSize: {
        type: Number
      },
      mutationOptions: {
        type: Object
      },
      fields: {
        type: Array
      },
      input: {
        type: Array
      },
      state: {
        type: Object
      },
      settings: {
        type: Object
      },
      selections: {
        type: Array
      }
    };
  }
  render() {
    const mutationOptions = this.mutationOptions;
    const fields = this.fields;
    const input = this.input;
    if (f1.is.nil(fields) || f1.is.nil(input) || f1.is.nil(mutationOptions)) {
      return nothing;
    }
    const renderFn = this.__renderStates[this.state.render_mode] || this.__renderStates.empty;
    if (!renderFn) {
      return nothing;
    }
    return html`<div class="grid__bar">${renderFn.call(this, mutationOptions, fields, input)}</div>`;
  }
  __renderGridFooter(mutationOptions, fields, input) {
    return html`<div class="grid__bar--left">
        ${this.settings.allow.row_selections ? this.__rowSelectionControls() : nothing}
      </div>
      <div class="grid__bar--right">
        ${this.settings.allow.table_pagination ? this.__paginationControls(mutationOptions, fields, input) : nothing}
      </div>`;
  }
  __rowSelectionControls() {
    if (this.settings.hidden.selection_toggle) {
      return nothing;
    }
    const selectionCount = f1.safe(0, this.selections.length);
    const ns = this.ns;
    const hasNoSelections = selectionCount <= 0;
    const isSelectionsViewVisible = this.state.view_mode === "selections";
    return html`
      <hi-label for="toggleDataView" .ns=${ns}>${selectionCount} selected</hi-label>
      <hi-toggle
        gid="toggleDataView"
        id="toggleDataView"
        size="small"
        aria-label=${isSelectionsViewVisible ? "Show selected rows" : "Show all rows"}
        .ns=${ns}
        .on=${isSelectionsViewVisible}
        @toggle-ended=${this.__toggleDataView__onToggleEnded}
      >
      </hi-toggle>
      <div>
        <hi-icon glyph="info" type="material" style="vertical-align:middle"></hi-icon>
        <hi-hint placement="top">Toggle row selection visibility</hi-hint>
      </div>
      ${hasNoSelections ? nothing : html`
            <hi-button-group layout="icon">
              <hi-button size="small" @trigger-ended=${this.__selectFiltered__onTriggerEnded}>Select All</hi-button>
              <hi-button size="small" @trigger-ended=${this.__deselectFiltered__onTriggerEnded}>Deselect All</hi-button>
            </hi-button-group>
          `}
    `;
  }
  __selectFiltered__onTriggerEnded() {
    this._dispatchNotification("row-select-all");
  }
  __deselectFiltered__onTriggerEnded() {
    this._dispatchNotification("row-unselect-all");
    this._dispatchNotification("view-state", {
      viewState: "list"
    });
  }
  __paginationControls(mutationOptions, fields, input) {
    const pageSize = f1.safe(0, mutationOptions.page_size);
    const pageIndex = f1.safe(0, mutationOptions.page_index);
    const links = mutationOptions.links;
    const currentPage = mutationOptions.current_page;
    const filteredTotalSize = f1.safe(0, input.length);
    const hidden = this.settings.hidden;
    const rules = this.settings.rules;
    const isInfiniteScroll = "infinite" === rules.row_rendering;
    return html`
      <div class="page-size">
        ${hidden.pagination_row_message || isInfiniteScroll ? nothing : html`<span class="paginationDetails">
              ${pageSize * pageIndex + 1} - ${pageSize * pageIndex + filteredTotalSize} of ${this.totalSize}
            </span>`}
        ${hidden.pagination_row_size ? nothing : html` <hi-input
              gid="pageSizeInput"
              id="pageSizeInput"
              type="number"
              min="10"
              .ns=${this.ns}
              .value=${pageSize}
              aria-label="Page Size"
              @commit-ended=${this.__pageSize__onCommitEnded}
            >
            </hi-input>`}
        ${isInfiniteScroll ? html`<hi-label for="pageSizeInput" .ns=${this.ns}> ${this.totalSize} Enitities </hi-label>` : nothing}
        ${hidden.pagination_controls || isInfiniteScroll ? nothing : html`<hi-pagination
              kind="determinate"
              page-index=${pageIndex}
              page-size=${pageSize}
              max=${this.totalSize}
              .pageCurrentOverride=${currentPage}
              .steps=${links}
              .ns=${this.ns}
              @paginate-ended=${this.__pagination__onPaginateEnded}
              @page-index-changed=${this.__pagination__onPageIndexChanged}
            ></hi-pagination>`}
      </div>
    `;
  }
  __pageSize__onCommitEnded(evt) {
    const pageSize = evt.detail.value.value;
    if (this.mutationOptions.page_size !== pageSize) {
      this.mutationOptions = {
        ...this.mutationOptions,
        page_size: pageSize,
        page_index: 0
      };
      this._dispatchNotification("state-action", {
        mutationOptions: this.mutationOptions
      });
    }
  }
  __pagination__onPaginateEnded(event) {
    const { steps, pageIndex, pageSize } = event.detail;
    const linkUri = !f1.is.empty(steps) ? steps[event.detail.value] : "";
    this._dispatchProcess("pagination", {
      pageIndex,
      pageSize,
      linkUri
    });
  }
  __pagination__onPageIndexChanged(event) {
    const currentPageIndex = this.mutationOptions.page_index;
    const changedPageIndex = event.detail.value;
    if (currentPageIndex === changedPageIndex) {
      return;
    }
    const commandName = "pageTo";
    const mutationOptions = {
      page_index: changedPageIndex
    };
    this._dispatchNotification("data-action", {
      commandName,
      mutationOptions
    });
  }
  __toggleDataView__onToggleEnded(event) {
    const viewState = event.detail.value ? "selections" : "list";
    this._dispatchNotification("view-state", {
      viewState
    });
  }
}
export default HIP.factory.element(GridFooterElement);
