
import { css, LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { AsLitHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import { gridStyles } from './vi-grid-shared-css.js';
import adt from '../../../pip/library/pi-adt/pi-adt.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import '../../../eip/element/hi-button/hi-button.js';
import '../../../eip/element/hi-menu/hi-menu.js';
import '../../../eip/element/hi-menu/hi-pick.js';
export const style = css`
  :host {
    width: 100%;
    display: block;
    height: var(--grid--header-height);
    background-color: var(--hi-clr--white);
    border-bottom: var(--hi-px-1) solid var(--hi-clr--gray-4);
  }
`;
export class GridHeaderElement extends BehaviorElement(AsLitHIC(LitElement)) {
  constructor() {
    super(...arguments);
    this.selections = [];
  }
  static get is() {
    return "vi-grid-header";
  }
  static get styles() {
    return [stylesTheme, gridStyles, style];
  }
  static get properties() {
    return {
      mutationOptions: {
        type: Object
      },
      fields: {
        type: Object
      },
      input: {
        type: Array
      },
      state: {
        type: Object
      },
      settings: {
        type: Object
      },
      bulkActions: {
        type: Array
      },
      selections: {
        type: Array
      }
    };
  }
  render() {
    const fields = this.fields;
    const input = this.input;
    if (f1.is.nil(fields) || f1.is.nil(input)) {
      return nothing;
    }
    const bulkActions = f1.safe([], this.bulkActions);
    const messages = f1.read("/messages", {}, this.settings);
    const hidden = f1.read("/hidden", {}, this.settings);
    const labels = f1.read("/labels", {}, this.settings);
    const allow = f1.read("/allow", {}, this.settings);
    const ns = this.ns;
    const renderMode = this.state.render_mode;
    const hasBulkActions = !f1.is.empty(bulkActions) && renderMode === "presentation";
    const selectionCount = f1.safe(0, this.selections.length);
    return html`<div class="grid__bar">
      <div class="grid__bar--left">
        ${!hasBulkActions || selectionCount <= 0 ? nothing : bulkActions.length === 1 ? html`<hi-button
              id="bulkActions"
              gid="bulkActions"
              aesthetic="secondary"
              @trigger-ended=${this.__bulkAction__onSelectedEnded}
              .value=${bulkActions[0].value}
              .ns=${ns}
            >
              <span>${f1.safe(messages.bulk_actions.title, bulkActions[0].title)}</span>
              <hi-hint placement="right">${messages.bulk_actions.description}</hi-hint>
            </hi-button>` : html`
              <hi-button
                id="bulkActions"
                gid="bulkActions"
                aesthetic="secondary"
                bind="bulkActionsMenu:activate"
                .ns=${ns}
              >
                <hi-icon glyph="menu" type="material"></hi-icon>
                <span>${messages.bulk_actions.title}</span>
                <hi-hint placement="right">${messages.bulk_actions.description}</hi-hint>
              </hi-button>
              <hi-menu
                id="bulkActionsMenu"
                gid="bulkActionsMenu"
                for="bulkActions"
                placement="bottom-start"
                cap-height
                .ns=${ns}
                @selected-ended=${this.__bulkAction__onSelectedEnded}
              >
                ${bulkActions.map((field) => html`<hi-pick .value=${field.value} .ns=${ns}>${field.title}</hi-pick>`)}
              </hi-menu>
            `}
      </div>
      <div class="grid__bar--middle">
        ${!allow.table_filter || hidden.global_search ? nothing : this.__renderGlobalSearch(labels)}
      </div>
      <div class="grid__bar--right">
        <div style="display:flex">
          ${hidden.clear_all_limits ? nothing : html`<hi-button
                id="clearAllLimits"
                gid="clearAllLimits"
                aesthetic="icon"
                .ns=${ns}
                @trigger-ended=${this.__clearAllLimits__onTriggerEnded}
              >
                <hi-icon glyph="layers_clear" type="material"></hi-icon>
                <hi-hint placement="left"> Clear all sorts and filters </hi-hint>
              </hi-button>`}
          ${hidden.export ? nothing : html`<hi-button
                id="exportGrid"
                gid="exportGrid"
                aesthetic="icon"
                .ns=${ns}
                @trigger-ended=${this.__exportGrid__onTriggerEnded}
              >
                <hi-icon glyph="download" type="material"></hi-icon>
                <hi-hint placement="left"> Export </hi-hint>
              </hi-button>`}
          ${hidden.add_row ? nothing : html`<hi-button
                id="addRow"
                gid="addRow"
                aesthetic="icon"
                .ns=${ns}
                @trigger-ended=${this.__addRow__onTriggerEnded}
              >
                <hi-icon glyph="add_box" type="material"></hi-icon>
                <hi-hint placement="left"> Add a row </hi-hint>
              </hi-button>`}
          ${hidden.pivot_table ? nothing : html`<hi-button
                id="pivotTable"
                gid="pivotTable"
                aesthetic="icon"
                .ns=${ns}
                @trigger-ended=${this.__pivotTable__onTriggerEnded}
              >
                <hi-icon glyph="query_stats" type="material"></hi-icon>
                <hi-hint placement="left"> Load grid in to a pivot table </hi-hint>
              </hi-button>`}
        </div>
        <hi-button-group layout="icon" .ns=${ns}>
          <hi-button
            id="togglePresentationMode"
            gid="togglePresentationMode"
            aesthetic="icon"
            .ns=${ns}
            ?hidden=${hidden.grid_table}
            ?disabled=${this.state.render_mode === "presentation"}
            @trigger-ended=${(e) => this.__toggleGridMode__onTriggerEnded("presentation", e)}
          >
            <hi-icon glyph="table_chart" type="material"></hi-icon>
            <hi-hint placement="left"> View grid table </hi-hint>
          </hi-button>
          <hi-button
            id="toggleColumnManagerMode"
            gid="toggleColumnManagerMode"
            aesthetic="icon"
            .ns=${ns}
            ?hidden=${hidden.column_manager}
            ?disabled=${this.state.render_mode === "columns"}
            @trigger-ended=${(e) => this.__toggleGridMode__onTriggerEnded("columns", e)}
          >
            <hi-icon glyph="list_alt" type="material"></hi-icon>
            <hi-hint placement="left"> View column manager </hi-hint>
          </hi-button>
          <hi-button
            id="toggleSortManagerMode"
            gid="toggleSortManagerMode"
            aesthetic="icon"
            .ns=${ns}
            ?hidden=${!allow.table_sort || hidden.sort_manager}
            ?disabled=${this.state.render_mode === "sort"}
            @trigger-ended=${(e) => this.__toggleGridMode__onTriggerEnded("sort", e)}
          >
            <hi-icon glyph="sort" type="material"></hi-icon>
            <hi-hint placement="left"> View sort manager </hi-hint>
          </hi-button>
          <hi-button
            id="toggleSettingsMode"
            gid="toggleSettingsMode"
            aesthetic="icon"
            .ns=${ns}
            ?hidden=${hidden.settings_manager}
            ?disabled=${this.state.render_mode === "settings"}
            @trigger-ended=${(e) => this.__toggleGridMode__onTriggerEnded("settings", e)}
          >
            <hi-icon glyph="gear" type="material"></hi-icon>
            <hi-hint placement="left"> View grid settings </hi-hint>
          </hi-button>
        </hi-button-group>
      </div>
    </div>`;
  }
  __renderGlobalSearch(labels) {
    return html`<hi-form alignment="inline" kind="filter" disable-http style="width: 100%; flex-direction: row">
      <hi-input-group .ns=${this.ns} style="flex-grow: 1; margin: 0; width: 100%">
        <hi-label id="filterAllLabel" gid="filterAllLabel" .ns=${this.ns} class="hide-content">Search List</hi-label>
        <hi-input
          id="filterAll"
          gid="filterAll"
          type="search"
          aria-labelledby="filterAllLabel"
          placeholder=${labels.global_search}
          .ns=${this.ns}
          @enter-keyup-ended=${this.__filterAll__onCommitEnded}
          @esc-keyup-ended=${this.__filterAll__onCommitEnded}
          @commit-ended=${this.__filterAll__onCommitEnded}
        ></hi-input>
      </hi-input-group>
      <hi-button type="submit" class="hide-content" slot="controls">
        <hi-icon glyph="search" type="material"></hi-icon>
      </hi-button>
    </hi-form>`;
  }
  __bulkAction__onSelectedEnded(event) {
    const actionName = event.detail.item ? event.detail.item.value : event.detail.value;
    this._dispatchNotification("bulk-action", {
      actionName
    });
  }
  __exportGrid__onTriggerEnded(event) {
    this._dispatchNotification("data-action", {
      commandName: "export"
    });
  }
  __addRow__onTriggerEnded(event) {
    this._dispatchNotification("data-action", {
      commandName: "addRow"
    });
  }
  __pivotTable__onTriggerEnded(event) {
    this._dispatchNotification("data-action", {
      commandName: "pivotTable"
    });
  }
  __clearAllLimits__onTriggerEnded(event) {
    const filterAllEl = this.shadowRoot?.querySelector("#filterAll");
    if (filterAllEl) {
      filterAllEl.clear();
    }
    this._dispatchNotification("data-action", {
      commandName: "clearAllLimits"
    });
  }
  __filterAll__onCommitEnded(event) {
    const value = event.srcElement.value;
    const filterByRules = f1.remove((_) => _.field === "*", (this.mutationOptions || {}).filter_by || []);
    const filter_by = f1.is.empty(value) ? filterByRules : [
      ...filterByRules,
      {
        comparator: adt.comparators.isLike.id,
        field: "*",
        value
      }
    ];
    this._dispatchNotification("state-action", {
      mutationOptions: {
        filter_by,
        page_index: 0
      }
    });
  }
  __toggleGridMode__onTriggerEnded(render_mode, event) {
    this._dispatchNotification("state-action", {
      state: {
        render_mode
      }
    });
  }
}
export default HIP.factory.element(GridHeaderElement);
