
import { html, css, LitElement } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { ifDefined as ifDef } from '../../../hip/library/lit-html/directives/if-defined.js';
import { repeat } from '../../../hip/library/lit-html/directives/repeat.js';
import {
  AsLitHIC,
  BehaviorElement,
  BehaviorA11yKeyboard,
  BehaviorBrowserEvents
} from '../../../eip/element/hi-component/hi-component.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import { setAttributes } from '../../../eip/element/hi-lit/hi-lit.js';
import { computeCellStyle, gridStyles } from './vi-grid-shared-css.js';
import adt from '../../../pip/library/pi-adt/pi-adt.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import '../../../eip/element/hi-input/hi-input.js';
import '../../../eip/element/hi-menu/hi-menu-proxy.js';
import '../../../eip/element/hi-lit-labs/hi-lit-virtualizer.js';
export const styles = css`
  :host {
    display: block;
    width: inherit;
  }

  :host(:focus) {
    outline: none;
  }

  .grid__row {
    height: var(--hi-table--row-height);
    line-height: var(--hi-table--row-height);
  }

  .row__cell {
    font-weight: var(--hi-font--weight-light);
    border-right: var(--hi-px-1) solid var(--hi-clr--gray-4);
  }

  [user_mode='view'] .row__cell:not([disallow-ellipsis]) {
    white-space: nowrap;
    overflow: hidden;
    text-overflow: ellipsis;
  }

  .row__cell.cell--stripe {
    background-color: var(--hi-clr--stripe);
  }

  .row__cell:not([focused]):focus {
    outline: none;
  }

  .row__cell[focused] {
    outline: var(--hi-px-2) dashed var(--hi-clr--gray-7);
    outline-offset: calc(-1 * var(--hi-px-3));
  }

  .row__cell[focused]:focus {
    outline-color: var(--hi-clr--outline);
    outline: var(--hi-outline--width) var(--hi-outline--line) var(--hi-clr--outline);
    z-index: 10;
  }

  .text-ellipsis {
    display: block;
    overflow: hidden;
    text-overflow: ellipsis;
    white-space: nowrap;
  }

  lit-virtualizer {
    height: 100%;
  }
`;
export class GridRowsElement extends BehaviorA11yKeyboard(
  BehaviorBrowserEvents(BehaviorElement(AsLitHIC(LitElement)))
) {
  static get is() {
    return "vi-grid-rows";
  }
  static get styles() {
    return [stylesTheme, gridStyles, styles];
  }
  static get properties() {
    return {
      group: {
        type: String
      },
      selectionsDB: {
        type: Object
      },
      maxSelections: {
        type: Number
      },
      fields: {
        type: Array
      },
      input: {
        type: Array
      },
      state: {
        type: Object
      },
      settings: {
        type: Object
      },
      cellRender: {
        type: Object
      },
      rowActions: {
        type: Object
      },
      mutationOptions: {
        type: Object
      }
    };
  }
  static get keyBindings() {
    return {
      "right:keyup": "__onKeyUp_right",
      "down:keyup": "__onKeyUp_down",
      "left:keyup": "__onKeyUp_left",
      "up:keyup": "__onKeyUp_up",
      "enter:keyup": "__onKeyUp_enter",
      "esc:keyup": "__onKeyUp_esc"
    };
  }
  static get browserBindings() {
    return {
      dblclick: "__gridCell__onDblClick",
      click: "__gridCell__onClick",
      blur: "__input__onBlur"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.__onParent__onScrollBound = this.__onParent__onScroll.bind(this);
    this.parentElement?.addEventListener("scroll", this.__onParent__onScrollBound);
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    this.parentElement?.removeEventListener("scroll", this.__onParent__onScrollBound);
  }
  __onParent__onScroll() {
    const gridEl = this.closest("section.state--presentation");
    if (!gridEl) {
      return;
    }
    const menuEl = gridEl.nextElementSibling;
    if (menuEl) {
      menuEl.deactivate();
    }
  }
  render() {
    const input = this.input;
    if (f1.is.empty(input)) {
      return nothing;
    }
    const rowRenderRule = this.settings.rules.row_rendering;
    const rowRender = rowRenderRule === "paginated" ? this.__renderRowsPaginated : this.__renderRowsVirtualized;
    return rowRender.call(this);
  }
  __renderRowsPaginated() {
    const fields = this.fields;
    const input = this.input;
    const renderItem = this.__renderBodyRow;
    const hasIndex = f1.is.in("id", f1.nth(0, input));
    return html` ${repeat(
      input,
      (item, index) => hasIndex ? item.id : index,
      (item, index) => renderItem.call(this, fields, item, index)
    )}`;
  }
  __renderRowsVirtualized() {
    const fields = this.fields;
    const input = this.input;
    const renderItem = this.__renderBodyRow;
    return html`
      <lit-virtualizer
        scroller
        .items=${input}
        @rangeChanged=${this.__listVirtualizer__onRangeChanged}
        .renderItem=${(item, index) => renderItem.call(this, fields, item, index)}
      ></lit-virtualizer>
    `;
  }
  __renderBodyRow(fields, row, index) {
    const stripeCSS = (index || 0) % 2 === 0 ? "cell--stripe" : "";
    const rowIndex = index || 0;
    const state = this.state;
    const isAllowedEdit = this.settings.allow.cell_edit;
    const modelReadFn = f1.map((field) => f1.read(field.model_path, void 0), fields);
    return html`<div
      role="row"
      class="grid__row"
      data-qa=${row.id}
      user_mode=${state.user_mode}
      aria-rowindex="${rowIndex + 1}"
      data-index=${index}
    >
      ${this.__renderSelector(row, rowIndex, `${stripeCSS}`)} ${this.__renderAction(row, rowIndex, `${stripeCSS}`)}
      ${fields.map((_field, index2) => {
      const cell = modelReadFn[index2](row);
      const field = adt.expression.evaluateAll(_field, row);
      const fieldName = field.name;
      const frozenCSS = index2 === 0 && this.settings.frozen_column ? "row--frozen" : "";
      const isCellAtFocus = state && state.focused_row === rowIndex && state.focused_column === fieldName;
      const isCellAllowedFocus = !field.view_only;
      const isCellInFocus = isCellAllowedFocus && isCellAtFocus;
      const isCellEditable = state.user_mode === "edit" && isCellInFocus;
      const tabindex = !isAllowedEdit ? "-1" : isCellInFocus ? "0" : "-1";
      if (tabindex === "0") {
        const cellEl = this.__findFocusedCell(rowIndex, fieldName);
        if (cellEl && cellEl.getAttribute("tabindex") !== "0") {
          dom.write(() => {
            cellEl.focus();
          });
        }
      }
      field.attributes = f1.omit(["id"], f1.safe({}, field.attributes));
      return html`<div
          role="cell"
          tabindex=${tabindex}
          data-row=${rowIndex}
          data-col=${fieldName}
          data-qa=${ifDef(cell)}
          id="${field.id}-${row.id}"
          style=${computeCellStyle(field)}
          data-attrs=${setAttributes(field.attributes)}
          class="row__cell ${stripeCSS} column-${fieldName} ${frozenCSS}"
          ?focused=${isAllowedEdit && isCellInFocus}
          data-index=${index2}
          .rowId=${row.id}
        >
          ${field.hidden ? nothing : this.cellRender(this, field, isCellEditable, cell, row)}
        </div>`;
    })}
    </div>`;
  }
  __renderSelector(row, rowIndex, cssClasses) {
    if (!this.settings.allow.row_selections) {
      return nothing;
    }
    const id = row.id;
    const ns = this.ns;
    return html`<div role="cell" class="row__cell row--selector row--frozen-fixed ${cssClasses}">
      <hi-checkbox
        id="checkbox--${id}"
        group=${this.group}
        data-id=${id}
        data-index=${rowIndex}
        aria-label="Toggle Row Selection"
        .row=${row}
        .ns=${ns}
        ?checked=${!!this.selectionsDB[id]}
        @toggle-ended=${this.__rowSelector__onToggleEnded}
      ></hi-checkbox>
    </div>`;
  }
  __renderAction(row, rowIndex, cssClasses) {
    if (!this.rowActions || this.rowActions.length <= 0) {
      return nothing;
    }
    return html`<div role="cell" class="row__cell row--action row--frozen-fixed ${cssClasses}">
      <hi-button
        gid="row-action-btn-${row.id}"
        id="row-action-btn-${row.id}"
        aria-label="open row actions"
        enable-isolated-events
        aria-haspopup="true"
        aesthetic="icon"
        .value=${row}
        .ns=${this.ns}
        @trigger-ended=${this.__rowAction__onTriggerEnded}
      >
        <hi-icon glyph="menu" type="material"></hi-icon>
      </hi-button>
    </div>`;
  }
  __rowSelector__onToggleEnded(event) {
    if (!event.target) {
      return;
    }
    const rowEl = event.target;
    const rowEvent = rowEl.checked ? "row-selected" : "row-unselected";
    const rowIndex = rowEl.getAttribute("data-index");
    const rowId = rowEl.getAttribute("data-id") || "?";
    const row = rowEl.row;
    const eventDetails = {
      rowIndex,
      rowId,
      rowEl,
      row
    };
    this._dispatchNotification(rowEvent, eventDetails);
  }
  __rowAction__onTriggerEnded(evt) {
    evt.preventDefault();
    evt.stopImmediatePropagation();
    const menuProxyEl = dom.setAttrs(document.createElement("hi-menu-proxy"), {
      "for-control": evt.target.id,
      "for-menu": "row-menu",
      ns: this.ns,
      value: evt.detail.value
    });
    menuProxyEl.input = f1.read("/detail/value/actions", this.rowActions, evt);
    menuProxyEl.activate();
  }
  __listVirtualizer__onRangeChanged(evt) {
    const isInfiniteRendering = "infinite" === this.settings.rules.row_rendering;
    if (!isInfiniteRendering) {
      return;
    }
    const isLastRow = evt.last >= this.input.length - 1;
    if (!isLastRow) {
      return;
    }
    this._dispatchProcess("list-scroll", {
      pageIndex: this.mutationOptions.current_page,
      pageSize: this.mutationOptions.page_size,
      linkUri: f1.read("/links/next", "", this.mutationOptions)
    });
  }
  __onKeyUp_enter(event) {
    if (!this.settings.allow.cell_edit) {
      return;
    }
    const state = this.state;
    event.preventDefault();
    if (state.user_mode === "view") {
      this.__makeCellEditable();
    } else {
      this.__makeCellViewable();
    }
  }
  __onKeyUp_esc(event) {
    const state = this.state;
    event.preventDefault();
    if (state.user_mode === "edit") {
      this.__makeCellViewable();
    }
  }
  __onKeyUp_up(event) {
    event.preventDefault();
    this.__moveCellFocus("moveFocusOnColumn", "up");
  }
  __onKeyUp_down(event) {
    event.preventDefault();
    this.__moveCellFocus("moveFocusOnColumn", "down");
  }
  __onKeyUp_left(event) {
    event.preventDefault();
    this.__moveCellFocus("moveFocusOnRow", "left");
  }
  __onKeyUp_right(event) {
    event.preventDefault();
    this.__moveCellFocus("moveFocusOnRow", "right");
  }
  __moveCellFocus(commandName, direction) {
    if (!this.settings.allow.cell_edit) {
      return;
    }
    const state = this.state;
    if ("view" !== state.user_mode) {
      return;
    }
    const fieldName = state.focused_column;
    const row = state.focused_row;
    this._dispatchNotification("cell-action", {
      commandName,
      fieldName,
      direction,
      row
    });
  }
  __input__onBlur(event) {
    const getTargetEl = event.target;
    if (!getTargetEl) {
      return;
    }
    this.state.user_mode = "view";
    this.requestUpdate();
  }
  __gridCell__onDblClick(e) {
    if (!this.settings.allow.cell_edit) {
      return;
    }
    const state = this.state;
    if (state.user_mode !== "view") {
      return;
    }
    this.__getFocusedCellPositionAndUpdateState(e);
    this.__makeCellEditable();
  }
  __gridCell__onClick(e) {
    if (!this.settings.allow.cell_edit) {
      return;
    }
    const state = this.state;
    const isStateUpdated = this.__getFocusedCellPositionAndUpdateState(e);
    if (state.user_mode === "view" || state.user_mode === "edit" && isStateUpdated) {
      this.__moveCellFocus("moveFocusOnClicked", "");
      this.__makeCellViewable();
    }
  }
  __getFocusedCellPositionAndUpdateState(e) {
    const cellEl = (e.composedPath() || []).find(
      (el) => el.classList && el.classList.contains("row__cell")
    );
    if (!cellEl) {
      return false;
    }
    const state = this.state;
    const focused_row = f1.to.integer(`${cellEl.getAttribute("data-row")}`);
    const focused_column = `${cellEl.getAttribute("data-col")}`;
    if (focused_column === "null") {
      return false;
    }
    if (focused_row === state.focused_row && focused_column === state.focused_column) {
      return false;
    }
    this.state = {
      ...this.state,
      focused_column,
      focused_row
    };
    return true;
  }
  __makeCellEditable() {
    this.state.user_mode = "edit";
    this.requestUpdate();
    setTimeout(() => {
      const { focused_column, focused_row } = this.state;
      const cellEl = this.__findFocusedCell(focused_row, focused_column);
      if (!cellEl) {
        return;
      }
      const inputEl = cellEl.children[0];
      if (inputEl) {
        inputEl.focus();
      }
    });
  }
  __makeCellViewable() {
    this.state.user_mode = "view";
    this.requestUpdate();
    setTimeout(() => {
      const { focused_column, focused_row } = this.state;
      const cellEl = this.__findFocusedCell(focused_row, focused_column);
      if (cellEl) {
        cellEl.focus();
      }
    });
  }
  __findFocusedCell(row, column) {
    return this.shadowRoot?.querySelector(`[data-row="${row}"][data-col="${column}"]`);
  }
}
export default HIP.factory.element(GridRowsElement);
