
import { html, css, LitElement } from '../../../hip/library/lit-element/lit-element.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { AsLitHIC, BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import { gridStyles } from './vi-grid-shared-css.js';
import '../../../eip/element/hi-form/hi-fieldset.js';
import '../../../eip/element/hi-button/hi-button.js';
import '../../../eip/element/hi-toggle/hi-toggle.js';
import './vi-grid-filter.js';
export const styles = css`
  :host {
    width: 100%;
    display: block;
  }

  hi-input-group {
    max-width: calc(16 * var(--hi-px-32));
    display: block;
  }

  hi-input-group:not(first-child) {
    margin-top: var(--hi-px-24);
  }

  hi-input-group > p {
    margin-top: var(--hi-px-6);
    color: var(--hi-clr--gray-8);
  }
`;
export class GridSettingsElement extends BehaviorElement(AsLitHIC(LitElement)) {
  constructor() {
    super(...arguments);
    this.toggleSettings = [
      {
        path: "/frozen_column",
        title: "Frozen First Column",
        description: "If set, then the first column will not scroll horizontally"
      }
    ];
  }
  static get is() {
    return "vi-grid-settings";
  }
  static get styles() {
    return [stylesTheme, gridStyles, styles];
  }
  static get properties() {
    return {
      settings: {
        type: Object
      },
      fields: {
        type: Object
      }
    };
  }
  render() {
    const settings = this.settings;
    const exportSettings = f1.read("/export", {}, settings);
    const hidden = f1.read(
      "/hidden",
      {},
      this.settings
    );
    const ns = this.ns;
    return html`<hi-form disable-http .ns=${ns} form="grid-settings">
      <hi-fieldset heading="Features" .ns=${ns}>
        ${this.toggleSettings.map(
      ({ path, title, description }) => html` <hi-input-group .ns=${ns}>
            <hi-toggle
              form="grid-settings"
              .ns=${ns}
              .on=${f1.read(path, false, settings)}
              @toggle-ended=${(e) => this.__updateSetting(path, e)}
            ></hi-toggle>
            <hi-label .ns=${ns} aesthetic="inline">${title}</hi-label>
            <p>${description}</p>
          </hi-input-group>`
    )}
      </hi-fieldset>
      <hi-fieldset heading="Export Settings" ?hidden=${!!hidden.export} .ns=${ns}>
        <hi-input-group .ns=${ns}>
          <hi-label .ns=${ns}>Filename of Download</hi-label>
          <hi-input
            type="text"
            form="grid-settings"
            .ns=${ns}
            .value=${exportSettings.filename}
            @commit-ended=${(e) => this.__updateSetting("/export/filename", e)}
          ></hi-input>
        </hi-input-group>
        <hi-input-group .ns=${ns}>
          <hi-label .ns=${ns}>Quote Character</hi-label>
          <hi-input
            type="text"
            form="grid-settings"
            .ns=${ns}
            .value=${exportSettings.quote_char}
            @commit-ended=${(e) => this.__updateSetting("/export/quote_char", e)}
          ></hi-input>
          <p>The character used to quote fields.</p>
        </hi-input-group>
        <hi-input-group .ns=${ns}>
          <hi-label .ns=${ns}>Delimiter</hi-label>
          <hi-input
            type="text"
            form="grid-settings"
            .ns=${ns}
            .value=${exportSettings.delimiter}
            @commit-ended=${(e) => this.__updateSetting("/export/delimiter", e)}
          ></hi-input>
          <p>
            The delimiting character, to separate cells. It can be any character except newline, line break, or
            double-quote.
          </p>
        </hi-input-group>
        <hi-input-group .ns=${ns}>
          <hi-toggle
            form="grid-settings"
            .ns=${ns}
            .on=${exportSettings.skip_empty_lines}
            @toggle-ended=${(e) => this.__updateSetting("/export/skip_empty_lines", e)}
          ></hi-toggle>
          <hi-label .ns=${ns} aesthetic="inline">Skip Empty Lines</hi-label>
          <p>If set, then lines that are completely empty (those which evaluate to an empty string) will be skipped.</p>
        </hi-input-group>
        <hi-input-group .ns=${ns}>
          <hi-toggle
            form="grid-settings"
            .ns=${ns}
            .on=${exportSettings.use_name_header}
            @toggle-ended=${(e) => this.__updateSetting("/export/use_name_header", e)}
          ></hi-toggle>
          <hi-label .ns=${ns} aesthetic="inline">Use Column Data Key for Header</hi-label>
          <p>If set, then the header will use column data key rather than column title.</p>
        </hi-input-group>
      </hi-fieldset>
    </hi-form>`;
  }
  __updateSetting(settingPath, e) {
    const value = f1.read("/detail/value/value", f1.read("/detail/value", void 0, e), e);
    this._dispatchNotification("settings-action", {
      settings: f1.write(settingPath, value, {})
    });
  }
}
export default HIP.factory.element(GridSettingsElement);
