
import { html, css, LitElement } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { AsLitHIC, BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import { gridStyles } from './vi-grid-shared-css.js';
import '../../../eip/element/hi-form/hi-fieldset.js';
import '../../../eip/element/hi-button/hi-button.js';
import '../../../eip/element/hi-toggle/hi-toggle.js';
import '../../../eip/element/hi-hint/hi-hint.js';
import './vi-grid-filter.js';
export const styles = css`
  :host {
    width: 100%;
    display: block;
  }

  .setting_row {
    display: flex;
    gap: 0.5rem;
    align-items: end;
    /* this is similar to rank-list */
    border: var(--hi-px-1) solid var(--hi-clr--gray-4);
    padding: var(--hi-px-16);
  }

  .setting_row:not(:first-child) {
    border-top: 0;
  }

  hi-input-group {
    min-width: 12rem;
    width: 30%;
  }

  #addSort {
    margin-top: var(--hi-px-16);
    margin-bottom: var(--hi-px-48);
  }

  .setting_row > hi-button:first-of-type {
    margin-left: auto;
  }

  h5:not(:first-of-type) {
    margin-top: var(--hi-px-16);
  }
`;
export class GridSortElement extends BehaviorElement(AsLitHIC(LitElement)) {
  static get is() {
    return "vi-grid-sort";
  }
  static get styles() {
    return [stylesTheme, gridStyles, styles];
  }
  static get properties() {
    return {
      mutationOptions: {
        type: Object
      },
      fields: {
        type: Object
      }
    };
  }
  render() {
    const mutationOptions = this.mutationOptions;
    const fields = this.fields;
    const sortBy = f1.safe([], mutationOptions.sort_by);
    const hasSortRules = sortBy && sortBy.length > 0;
    const ns = this.ns;
    const rankFields = f1.filter((field) => field.rankable, fields);
    const sortFields = f1.remove((field) => field.rankable || f1.is.false(field.sortable), fields);
    return html`
      <h5>Rank Fields</h5>
      <ul>
        ${f1.map((field) => {
      return html`<li><hi-text>${field.title}</hi-text></li>`;
    }, rankFields)}
      </ul>
      ${f1.is.exists(rankFields) ? nothing : html`<p>No rank fields are specified.</p>`}

      <h5>Sorted Fields</h5>
      <ul>
        ${f1.map(({ order, field }, index) => {
      return html`
            <li class="setting_row">
              <hi-input-group .ns=${ns}>
                <hi-label .ns=${ns}>Order</hi-label>
                <hi-dropdown .ns=${ns}>
                  <hi-select
                    .ns=${ns}
                    mode="value"
                    .value=${order}
                    @commit-ended=${(e) => this.__sortProperty__onCommitEnded(index, "order", e)}
                  >
                    <hi-option value="asc">Ascending</hi-option>
                    <hi-option value="desc">Descending</hi-option>
                  </hi-select>
                </hi-dropdown>
              </hi-input-group>
              <hi-input-group .ns=${ns}>
                <hi-label .ns=${ns}>Field</hi-label>
                <hi-dropdown .ns=${ns}>
                  <hi-select
                    .ns=${ns}
                    mode="value"
                    .value=${field}
                    @commit-ended=${(e) => this.__sortProperty__onCommitEnded(index, "field", e)}
                  >
                    ${f1.map((field2) => {
        return html`<hi-option .value=${field2.name}>${field2.title}</hi-option>`;
      }, sortFields)}
                  </hi-select>
                </hi-dropdown>
              </hi-input-group>
              <hi-button aesthetic="icon" .ns=${ns} @trigger-ended=${() => this.__trashSort__onTriggerEnded(index)}>
                <hi-icon glyph="trash"></hi-icon>
                <hi-hint placement="left">Delete Sort Rule</hi-hint>
              </hi-button>
            </li>
          `;
    }, sortBy)}
      </ul>
      ${hasSortRules ? nothing : html`
            <hi-alert mode="presentation">
              No Active Sort
              <span slot="details">
                If there is no active sort you can add multiple column sort rules here, or use the sort toggle keys in
                the column headers of the grid.
              </span>
            </hi-alert>
          `}
      <hi-button
        id="addSort"
        aesthetic="secondary"
        size="small"
        .ns=${ns}
        @trigger-ended=${this.__addSort__onTriggerEnded}
      >
        <hi-icon glyph="plus"></hi-icon>
        <span>Add Sort Rule</span>
      </hi-button>
    `;
  }
  __sortProperty__onCommitEnded(index, field, e) {
    const sortBy = f1.write(`/${index}/${field}`, e.detail.value, this.mutationOptions.sort_by);
    this.__updateSortBy(sortBy);
  }
  __trashSort__onTriggerEnded(index) {
    const sortBy = f1.safe([], this.mutationOptions.sort_by);
    const sortByRemovedItem = f1.remove((_, idx) => index === idx, sortBy);
    this.__updateSortBy(sortByRemovedItem);
  }
  __addSort__onTriggerEnded() {
    const sortBy = f1.safe([], this.mutationOptions.sort_by);
    const sortByPlusOne = f1.push({}, sortBy);
    this.__updateSortBy(sortByPlusOne);
  }
  __updateSortBy(sort_by) {
    this._dispatchNotification("data-action", {
      commandName: "sortBy",
      mutationOptions: {
        sort_by
      }
    });
  }
}
export default HIP.factory.element(GridSortElement);
