
export * from './vi-grid-shared.js';
import { idlePeriod, Debouncer } from '../../../hip/library/polymer/polymer.js';
import { html, css, LitElement } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { FactoryViewElement } from '../../../aip/element/ai-factory-view/ai-factory-view.js';
import { FactoryEditElement } from '../../../aip/element/ai-factory-edit/ai-factory-edit.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { AsLitHIC, BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import { BehaviorState } from '../../../eip/element/hi-behavior-state/hi-behavior-state.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import { Ignition } from '../../../pip/library/pi-interop/pi-interop.js';
import adt from '../../../pip/library/pi-adt/pi-adt.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import $ip from '../../../pip/library/pi-ip/pi-ip.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import '../../../eip/element/hi-tooltip/hi-tooltip.js';
import '../../../eip/element/hi-hint/hi-hint.js';
import { gridStyles } from './vi-grid-shared-css.js';
import '../../../eip/element/hi-rank-list/hi-rank-list.js';
import './vi-grid-settings.js';
import './vi-grid-columns.js';
import './vi-grid-header.js';
import './vi-grid-footer.js';
import './vi-grid-sort.js';
import './vi-grid-rows.js';
export const style = css`
  :host {
    --grid--row-height: 3rem;
    --grid--row-selector-width: 3rem;
    --grid--header-height: 3rem;
    --grid--footer-height: 3rem;

    --grid--max-rows: 7;
    --grid--min-rows: 3;

    --grid--max-height: calc(var(--grid--max-rows) * var(--grid--row-height));
    --grid--min-height: calc(var(--grid--min-rows) * var(--grid--row-height));
    --grid--cell-padding: var(--hi-px-6);

    --grid--z-index: 1;
    --border--style: var(--hi-px-1) solid var(--hi-clr--gray-4);
  }

  :host {
    width: 100%;
    height: auto;
    display: grid;
    position: relative;
    border-collapse: collapse;
    background-color: var(--hi-clr--white);
    border: var(--border--style);

    min-height: var(--grid--min-height);
    z-index: var(--grid--z-index);
  }

  :host([fit-height]) {
    height: 100%;
    grid-template: var(--grid--header-height) 1fr var(--grid--footer-height) / 100%;
  }

  :host([fit-height]) .grid {
    max-height: inherit;
  }

  .grid {
    display: block;
    max-height: var(--grid--max-height);
  }

  .grid.state--presentation {
    overflow: auto;
    background: var(--hi-clr--gray-1);
  }

  .grid h3 {
    padding-top: var(--hi-px-16);
  }

  .grid.state--empty,
  .grid.state--error,
  .grid.state--error p,
  .grid.state--empty p {
    padding: var(--hi-px-16);
  }

  .grid.state--sort,
  .grid.state--columns,
  .grid.state--settings {
    padding: var(--hi-px-16);
    overflow-y: auto;
  }

  .grid > p {
    max-width: 70ch;
  }

  :not(.grid.state--presentation) vi-grid-columns,
  :not(.grid.state--presentation) vi-grid-rows {
    width: 100% !important;
  }

  caption {
    font: 0 / 0 a;
    text-shadow: none;
    color: var(--hi-clr--transparent);
    background-color: var(--hi-clr--transparent);
    position: absolute;
    border: 0;
  }

  .grid__footer,
  .grid__header {
    z-index: calc(var(--grid--z-index) + 700);
    right: 0;
    left: 0;
  }

  .grid__columns {
    position: sticky;
  }
  .grid__header {
    top: 0;
  }

  .grid__footer {
    bottom: 0;
  }

  .grid__columns {
    z-index: calc(var(--grid--z-index) + 699);
    top: 0;
  }

  .grid__rows {
    z-index: calc(var(--grid--z-index) + 99);
    min-height: calc(5 * var(--grid--row-height));
    height: calc(100% - var(--hi-table--column-height) - var(--grid--header-height) - var(--grid--footer-height));
    /* needed to make sure the first row can be seen with the scroll snapping */
    padding-top: 0;
  }

  :host([aesthetic='invisible']) {
    --border--style: none;
  }

  #row-menu {
    --hi-menu--max-height: 8em;
  }
`;
const linkRankListEventActionsToGridMethods = {
  includeAll: "showColumnAll",
  excludeAll: "hideColumnAll",
  included: "showColumn",
  excluded: "hideColumn",
  selectAll: "showColumn",
  selectNone: "hideColumn"
};
export class GridElement extends BehaviorState(BehaviorElement(AsLitHIC(LitElement))) {
  constructor() {
    super();
    this.mementoCacheProperties = ["type", "settings", "state", "analytics"];
    this.mementoChangeProperties = ["type"];
    this.mementoVersion = 1;
    this.state = {
      focused_row: 0,
      focused_column: "",
      render_mode: "empty",
      view_mode: "list",
      user_mode: "view"
    };
    this.settings = {};
    this.__internal = {
      pivotTable: {}
    };
    this.defaultAnalytics = {
      pivot_table: {
        version: "24.1.5",
        options: {}
      }
    };
    this.settingsMode = f1.strategy({
      data_set: {
        allow: {
          table_sort: true,
          cell_edit: false,
          table_filter: true
        },
        hidden: {
          export: false,
          add_row: true,
          sort_manager: false,
          global_search: false,
          column_manager: false,
          clear_all_limits: false
        }
      },
      record_set: {
        allow: {
          table_sort: true,
          cell_edit: true,
          table_filter: false
        },
        hidden: {
          export: true,
          add_row: true,
          sort_manager: true,
          global_search: true,
          column_manager: true,
          clear_all_limits: true
        }
      },
      fallback: {}
    });
    this.settingsDefaults = {
      frozen_column: false,
      attributes: {},
      allow: {
        row_selections: true,
        cell_edit: false,
        table_sort: true,
        table_filter: true,
        table_pagination: true,
        remote_export: false
      },
      rules: {
        row_rendering: "paginated"
      },
      export: {
        include_bom: true,
        accepts_preprocessed_table: false,
        skip_empty_lines: false,
        use_name_header: false,
        filename: "report.csv",
        quote_char: '"',
        delimiter: ",",
        escapeFormulae: true
      },
      hidden: {
        export: false,
        add_row: true,
        grid_table: false,
        pivot_table: true,
        sort_manager: false,
        column_manager: false,
        settings_manager: false,
        clear_all_limits: false,
        global_search: false,
        selection_toggle: false,
        pagination_row_message: false,
        pagination_controls: false,
        pagination_row_size: false
      },
      labels: {
        global_search: "Search in Results",
        row_action_header: "Row Action"
      },
      messages: {
        bulk_actions: {
          title: "Actions",
          description: "Perform action on selected row(s)"
        },
        state_error: {
          title: "Error",
          description: "Your content cannot be rendered."
        },
        state_empty: {
          title: "Empty",
          description: "No content."
        }
      }
    };
    this.selectionsDB = {};
    this.selections = [];
    this.allowPreprocessedInput = false;
    this.maxSelections = -1;
    this.selectionGroup = `grid__${f1.id()}`;
    this.caption = "Grid";
    this.__rowsPerPage = 1;
    this.__resizeObserver = null;
    this.editFactory = new FactoryEditElement();
    this.viewFactory = new FactoryViewElement();
    this.hoverableText = {
      attributes: {
        "allow-ellipsis": true
      },
      style: {
        "max-width": "100%",
        height: "var(--hi-font-size--large-x)"
      }
    };
    this.__renderStates = {
      empty() {
        return html`
        <section class="grid state--empty">
          <div style="text-align: center">
            <h3>${this.settings.messages.state_empty.title}</h3>
            <p>${this.settings.messages.state_empty.description}</p>
          </div>
        </section>
      `;
      },
      error() {
        return html`
        <section class="grid state--error">
          <div style="text-align: center">
            <h3>${this.settings.messages.state_error.title}</h3>
            <p>${this.settings.messages.state_error.description}</p>
          </div>
        </section>
      `;
      },
      columns(mutationOptions, fields, input, state2, settings, type) {
        const columnFields = adt.mutation("permissions").includeType(type).invoke(type.fields);
        return html`
        <section class="grid state--columns">
          <h4>Column Manager</h4>
          <p>
            Adjust column order by dragging or clicking each item up or down the list. Use the checkboxes to toggle
            column visibility.
          </p>
          <br />
          <hi-rank-list .input=${columnFields} .ns=${this.ns} allow-inclusion></hi-rank-list>
        </section>
      `;
      },
      settings(mutationOptions, fields, input, state2, settings, type) {
        return html`
        <section class="grid state--settings">
          <h4>Grid Settings</h4>
          <p>Configure how the grid renders and exports data.</p>
          <br />
          <vi-grid-settings .settings=${settings} .ns=${this.ns}></vi-grid-settings>
        </section>
      `;
      },
      sort(mutationOptions, fields, input, state2, settings, type) {
        return html`
        <section class="grid state--sort">
          <h4>Sort Manager</h4>
          <p>
            Create a multi-column sort. If the sort buttons are used on the table then the multi-column sort is reset
            and sorting occurs by only one column.
          </p>
          <br />
          <vi-grid-sort .fields=${fields} .mutationOptions=${mutationOptions}></vi-grid-sort>
        </section>
      `;
      },
      filter(mutationOptions, fields, input, state2, settings, type) {
        return html`<p>Not Implemented</p>`;
      },
      presentation(mutationOptions, fields, input, state2, settings, type, entireInput) {
        const widthOfColumns = `calc(${f1.map((field) => `var(--grid--column-${field.name})`, fields).join(" + ")} + 10rem + ${this.settings.allow.row_selections ? `3rem` : `0rem`})`;
        const comparators = f1.safe({}, type.comparators);
        return html`
        <section
          class="grid state--presentation"
          role="grid"
          aria-colcount=${fields.length}
          aria-rowcount=${input.length}
        >
          <vi-grid-columns
            id="columns"
            role="rowgroup"
            class="grid__columns"
            style="width:${widthOfColumns}"
            .mutationOptions=${mutationOptions}
            .rowActions=${this.rowActions}
            .entireInput=${entireInput}
            .settings=${settings}
            .fields=${fields}
            .input=${input}
            .state=${state2}
            .ns=${this.ns}
            .comparators=${comparators}
            .group=${this.selectionGroup}
            .selections=${this.selections}
            .maxSelections=${this.maxSelections}
          >
          </vi-grid-columns>
          <vi-grid-rows
            id="rows"
            role="rowgroup"
            class="grid__rows"
            style="width:${widthOfColumns}"
            .rowActions=${this.rowActions}
            .settings=${settings}
            .fields=${fields}
            .input=${input}
            .state=${state2}
            .ns=${this.ns}
            .selectionsDB=${this.selectionsDB}
            .group=${this.selectionGroup}
            .maxSelections=${this.maxSelections}
            .cellRender=${this.__renderCell.bind(this)}
            .mutationOptions=${mutationOptions}
          ></vi-grid-rows>
        </section>
        <hi-menu
          gid="row-menu"
          id="row-menu"
          .ns=${this.ns}
          placement="right-start"
          cap-height
          @selected-ended=${this.__rowAction__onSelectedEnded}
        ></hi-menu>
      `;
      }
    };
    this.ns = `vi-grid-${f1.id()}`;
  }
  static get is() {
    return "vi-grid";
  }
  static get properties() {
    return {
      mode: {
        type: String
      },
      caption: {
        type: String
      },
      allowPreprocessedInput: {
        type: Boolean,
        attribute: "allow-preprocessed-input"
      },
      type: {
        type: Object
      },
      exportType: {
        type: Object,
        attribute: "export-type"
      },
      input: {
        type: Array
      },
      state: {
        type: Object
      },
      selections: {
        type: Array
      },
      settings: {
        type: Object
      },
      bulkActions: {
        type: Array,
        attribute: "bulk-actions"
      },
      rowActions: {
        type: Array,
        attribute: "row-actions"
      },
      maxSelections: {
        type: Number,
        attribute: "max-selections"
      },
      selectionGroup: {
        type: String,
        attribute: "selection-group"
      },
      aesthetic: {
        type: String
      }
    };
  }
  static get api() {
    return [
      "clearAllLimits",
      "addRow",
      "pivotTable",
      "pageTo",
      "sortBy",
      "filterBy",
      "updateMutations",
      "updateState",
      "updateSettings",
      "hideColumn",
      "showColumn",
      "export"
    ];
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "group");
    this.__resetFocusState();
    this.__connectFactories();
    this.__includeEventHandlers();
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    this.editFactory.removeEventListener("field-trigger-ended", this.__editableCell__onValueChangedBound);
    if (this.__resizeObserver) {
      this.__resizeObserver.unobserve(this);
      this.__resizeObserver.disconnect();
    }
  }
  __resetFocusState() {
    this.state.focused_column = f1.nth(0, f1.keys(f1.read("/fields", {}, this.type)));
    this.state.focused_row = f1.safe(0, this.state.focused_row);
  }
  __includeEventHandlers() {
    this.__editableCell__onValueChangedBound = this.__editableCell__onValueChanged.bind(this);
    this.editFactory.addEventListener("field-trigger-ended", this.__editableCell__onValueChangedBound);
    this.viewFactory.addEventListener("field-trigger-ended", this.__editableCell__onValueChangedBound);
    this.addEventListener("resize-column-notified", (event) => {
      event.stopPropagation();
      const { fieldName, css4Column, widthPx } = event.detail;
      const field = this.__findField(fieldName);
      this.setColumnWidth(css4Column, widthPx, field);
      this.requestUpdate("type");
    });
    this.addEventListener("settings-action-notified", (event) => {
      const { settings = {} } = event.detail;
      event.stopPropagation();
      this.updateSettings(settings);
    });
    this.addEventListener("state-action-notified", (event) => {
      const { state: state2 = {}, mutationOptions = {} } = event.detail;
      event.stopPropagation();
      this.updateState(state2);
      if (!f1.is.empty(mutationOptions)) {
        this.updateMutations(mutationOptions);
      }
    });
    this.addEventListener("column-action-notified", (event) => {
      const { commandName, fieldName, field } = event.detail;
      event.stopPropagation();
      if (f1.is.function(this[commandName])) {
        this[commandName](fieldName, field);
      }
    });
    this.addEventListener("data-action-notified", (event) => {
      const { commandName, mutationOptions, field } = event.detail;
      event.stopPropagation();
      if (f1.is.function(this[commandName])) {
        this[commandName](mutationOptions, field);
      }
    });
    this.addEventListener("cell-action-notified", (event) => {
      const { commandName, row, fieldName, direction } = event.detail;
      event.stopPropagation();
      if (f1.is.function(this[commandName])) {
        this[commandName](row, fieldName, direction);
      }
    });
    this.addEventListener("row-select-all-page-notified", (evt) => {
      evt.stopPropagation();
      this.selectAllOnPage();
    });
    this.addEventListener("row-unselect-all-page-notified", (evt) => {
      evt.stopPropagation();
      this.deselectAllOnPage();
    });
    this.addEventListener("row-select-all-notified", (evt) => {
      evt.stopPropagation();
      this.selectAll();
    });
    this.addEventListener("row-unselect-all-notified", (evt) => {
      evt.stopPropagation();
      this.deselectAll();
    });
    this.addEventListener("row-selected-notified", (evt) => {
      evt.stopPropagation();
      this.selectRow(evt.detail.row);
    });
    this.addEventListener("row-unselected-notified", (evt) => {
      evt.stopPropagation();
      this.deselectRow(evt.detail.row);
    });
    this.addEventListener("field-trigger-notified", (event) => {
      event.stopPropagation();
      const fieldName = event.detail;
      this._dispatchEvent(`field-${fieldName}-trigger`, event.detail);
      this._dispatchEvent(`field-trigger`, event.detail);
    });
    this.addEventListener("bulk-action-notified", (event) => {
      event.stopPropagation();
      const { actionName: action } = event.detail;
      this._dispatchEvent("bulk-action-ended", {
        value: {
          action,
          selections: this.selections
        }
      });
    });
    this.addEventListener("view-state-notified", (event) => {
      this.state = {
        ...this.state,
        view_mode: event.detail.viewState
      };
      this.type.mutations = {
        ...this.type.mutations,
        page_index: 0
      };
      if (event.detail.viewState === "list") {
        this.input = this.backupGrid || this.input;
      } else {
        this.backupGrid = this.input;
        this.input = this.selections;
      }
      this.requestUpdate("state");
    });
    this.addEventListener("commit-notified", (event) => {
      event.stopPropagation();
      const { row, input, action } = event.detail;
      const fieldName = f1.read("/__source__/name", void 0, row);
      const gridMethod = this[linkRankListEventActionsToGridMethods[action]];
      if (action === "selectAll" || action === "selectNone") {
        input.forEach((field) => {
          gridMethod.call(this, field.name);
        });
      } else if (action === "includeAll" || action === "excludeAll") {
        gridMethod.call(this);
      } else if (action === "included" || action === "excluded") {
        gridMethod.call(this, fieldName);
      } else {
        const fields = this.type.fields;
        input.forEach((item) => {
          const field = this.__findField(item.name);
          if (field) {
            field.order = item.order;
          }
        });
        this.type.fields = { ...fields };
        this.requestUpdate();
      }
    });
  }
  __connectFactories() {
    this.editFactory.ns = this.ns;
    this.viewFactory.ns = this.ns;
  }
  selectAllOnPage() {
    this.__changeSelections(f1.union(f1.safe([], this.selections), f1.copy(this.grid)));
  }
  deselectAllOnPage() {
    this.__changeSelections(f1.difference(f1.copy(this.grid), f1.safe([], this.selections)));
  }
  selectAll() {
    this.__changeSelections(f1.copy(f1.safe(this.input, this.backupGrid)));
  }
  deselectAll() {
    this.__changeSelections([]);
  }
  selectRow(row) {
    this.__changeSelections(f1.push(row, this.selections), "selection-changed");
  }
  deselectRow(row) {
    this.__changeSelections(
      f1.remove((selection) => selection.id === row.id, this.selections),
      "selection-changed"
    );
  }
  __changeSelections(selections, selectionEventName = "selections-changed") {
    selections = this.setSelections(selections);
    this._dispatchEvent(selectionEventName, {
      value: selections
    });
  }
  reorderFieldToLeft(fieldName) {
    return this.reorderField(fieldName, (field) => field.order - 1);
  }
  reorderFieldToRight(fieldName) {
    return this.reorderField(fieldName, (field) => field.order + 1);
  }
  reorderFieldToLeftmost(fieldName) {
    return this.reorderField(fieldName, () => 0);
  }
  reorderFieldToRightmost(fieldName) {
    return this.reorderField(fieldName, () => f1.size(this.type.fields) - 1);
  }
  reorderField(fieldName, reorderFn) {
    const fields = this.type.fields;
    const field = fields[fieldName];
    const oldOrder = field.order;
    const newOrder = f1.clamp(0, f1.size(fields), reorderFn(field));
    const existingOrder = f1.find((field2) => field2.order === newOrder, fields);
    field.order = newOrder;
    if (existingOrder) {
      existingOrder.order = oldOrder;
    }
    this.type = { ...this.type };
    this.requestUpdate("type");
  }
  pageTo(mutationOptions) {
    return this.updateMutations(mutationOptions);
  }
  sortBy(mutationOptions) {
    return this.updateMutations(mutationOptions);
  }
  filterBy(mutationOptions) {
    return this.updateMutations(mutationOptions);
  }
  updateMutations(mutations) {
    this.type.mutations = f1.merge("over", this.type.mutations, mutations);
    this.state = f1.copy(this.state);
    this.requestUpdate("state");
    return this.type.mutations;
  }
  updateState(state2) {
    this.state = f1.merge("over", this.state, state2);
    this.requestUpdate("state");
    return this.state;
  }
  updateSettings(settings) {
    this.settings = f1.merge("over", this.settings, settings);
    this.requestUpdate("settings");
    return this.settings;
  }
  hideColumn(fieldName) {
    this.toggleColumnVisibility(fieldName, false);
  }
  showColumn(fieldName) {
    this.toggleColumnVisibility(fieldName, true);
  }
  hideColumnAll() {
    f1.each((field) => {
      this.toggleColumnVisibility(field.name, false);
    }, f1.read("/fields", {}, this.type));
  }
  showColumnAll() {
    f1.each((field) => {
      this.toggleColumnVisibility(field.name, true);
    }, f1.read("/fields", {}, this.type));
  }
  toggleColumnVisibility(fieldName, isVisible) {
    const field = this.__findField(fieldName);
    if (!field) {
      return;
    }
    field.hidden = !isVisible;
    this.type = { ...this.type };
    this.requestUpdate("type");
  }
  moveFocusOnRow(row, fieldName, direction) {
    const value = direction === "left" ? -1 : 1;
    const fields = f1.filter(
      (_) => !_.hidden,
      f1.sort((a, b) => f1.to.integer(a.order) - f1.to.integer(b.order), f1.to.array(this.type.fields))
    );
    const fieldOrder = fields.findIndex((field) => field.name === fieldName);
    const size = f1.size(fields);
    const newFieldIndex = f1.clamp(0, size, fieldOrder + value);
    const newField = fields[newFieldIndex];
    if (!newField) {
      return;
    }
    const state2 = this.state;
    state2.focused_column = f1.read("/name", "", newField);
    state2.focused_row = f1.safe(0, row);
    state2.user_mode = "view";
    this.state = f1.copy(state2);
    this.requestUpdate("state");
  }
  moveFocusOnColumn(row, fieldName, direction) {
    const value = direction === "up" ? -1 : 1;
    const newRow = row + value;
    const state2 = this.state;
    const pageSize = this.__rowsPerPage;
    if (newRow >= 0 && newRow < pageSize) {
      state2.focused_row = newRow;
      state2.focused_column = fieldName;
      this.state = f1.copy(this.state);
      this.requestUpdate("state");
    }
  }
  addRow() {
    return this.input;
  }
  async pivotTable() {
    const analytics = f1.merge(
      "over",
      f1.clone(this.defaultAnalytics),
      f1.merge(
        "over",
        f1.clone(f1.read("/analytics", {}, this.settings)),
        f1.safe({}, this.analytics)
      )
    );
    const version = analytics.pivot_table.version;
    const [pivotTable] = await $ip.requires([
      `@crate/res/pivot-table/${version}/index`,
      `@crate/res/pivot-table/${version}/index.css`
    ]);
    await Ignition.create().startup(pivotTable, {
      version: analytics.pivot_table.version
    });
    this.addEventListener(
      "analytics-pivot-table-closed",
      () => {
        Ignition.create().shutdown(pivotTable);
        $ip.unrequire(`hip/pivot-table/${version}/index.css`);
      },
      { once: true }
    );
    return await pivotTable.commands.createEditor({
      type: this.type,
      input: this.input,
      state: this.__internal.pivotTable,
      attached_to: document.body,
      event_to: this,
      options: f1.merge(
        "over",
        {
          custom_settings: {
            in_dialog: true
          }
        },
        {
          ...analytics.pivot_table.options
        }
      )
    });
  }
  clearAllLimits() {
    return this.updateMutations({
      page_index: 0,
      filter_by: [],
      sort_by: []
    });
  }
  async export() {
    return this.settings.allow.remote_export ? this.remoteExport() : this.localExport();
  }
  async remoteExport() {
    return "";
  }
  async localExport() {
    const input = f1.to.array(f1.safe([], this.input));
    return this.generateExport(input);
  }
  async generateExport(data, localExport) {
    const csvFile = await this.generateCSV(data, localExport);
    dom.download(this.settings.export.filename, csvFile);
    return csvFile;
  }
  async generateCSV(data, localExport) {
    const prefix = this.settings.export.include_bom ? "\uFEFF" : "";
    const type = f1.mirror(localExport || this.exportType || this.type);
    const csvExport = f1.mirror(this.settings.export);
    const csvSettings = f1.write("/accepts_preprocessed_table", this.allowPreprocessedInput, csvExport);
    const cvsFile = await adt.schemas.csv.write(csvSettings, type, data);
    return `${prefix}${cvsFile}`;
  }
  __renderCell(gridRowsEl, field, isEditable, value, row) {
    return isEditable ? this.__renderEditCell(gridRowsEl, field, value, row) : this.__renderViewCell(gridRowsEl, field, value, row);
  }
  __renderEditCell(gridRowsEl, field, value, row) {
    const editFactory = this.editFactory;
    const editType = editFactory && editFactory.hasRender("edit", field.render_type) ? field.render_type : "unknown";
    const editRenderFn = editFactory.renderBy("edit", editType).render(
      {
        ...field,
        attributes: {
          name: "value",
          required: true,
          focused: true
        }
      },
      editFactory
    );
    return editFactory.renderValue(editRenderFn, field, row, value);
  }
  __renderViewCell(gridRowsEl, field, value, row) {
    const viewFactory = this.viewFactory;
    const viewType = viewFactory && viewFactory.hasRender("view", field.render_type) ? field.render_type : "unknown";
    const viewRenderFn = viewFactory.renderBy("view", viewType).render(
      f1.merge("over", f1.merge("over", {}, field), this.hoverableText),
      viewFactory
    );
    return viewFactory.renderValue(viewRenderFn, field, row, value);
  }
  __editableCell__onValueChanged(event) {
    this._dispatchProcess("field-trigger", event.detail);
    if (!event.target) {
      return;
    }
    const newCellvalue = f1.read("/detail/value", "", event);
    const eventPath = f1.read("/detail/path", [], event);
    const targetEl = event.target;
    const cellEl = dom.is.tag("hi-input", targetEl) ? event.target.parentElement : eventPath.find((el) => el.classList && el.classList.contains("row__cell"));
    const focusedColumn = `${cellEl.getAttribute("data-col")}`;
    const rowId = f1.read("/rowId", "", cellEl);
    const row = f1.find((_) => _.id === rowId, this.input);
    if (!row) {
      return;
    }
    const oldCellValue = row[focusedColumn];
    if (oldCellValue !== newCellvalue && typeof oldCellValue === typeof newCellvalue) {
      row[focusedColumn] = newCellvalue;
      this.requestUpdate("input");
      this._dispatchProcess("commit", {
        row,
        table: this.input,
        value: newCellvalue,
        field: focusedColumn
      });
    }
  }
  moveFocusOnClicked(row, fieldName, direction) {
    this.updateState({
      focused_column: fieldName,
      focused_row: row
    });
  }
  firstUpdated() {
    this.__resizeObserver = new ResizeObserver((entries) => {
      for (const entry of entries) {
        this.__debouncedColumnSizeWatcher(entry);
      }
    });
    this.__resizeObserver.observe(this);
  }
  __debouncedColumnSizeWatcher(entry) {
    this.__resizeObserverDebouncer = Debouncer.debounce(this.__resizeObserverDebouncer, idlePeriod, () => {
      this.__columnSizeWatcher(entry);
    });
  }
  __columnSizeWatcher(entry) {
    const fields = f1.safe([], this.fields);
    const count = fields.length;
    if (count <= 0) {
      return;
    }
    const columnDefaultWidth = 180;
    const reduceWidthOfSelectors = !this.settings.allow.row_selections ? 0 : 3 * 16 / count;
    const { width } = entry.contentRect;
    const columnWidth = width > 0 ? width / count - reduceWidthOfSelectors : columnDefaultWidth;
    const safeColumnWidth = f1.max(columnWidth, columnDefaultWidth);
    f1.map((field) => {
      const css4Variable = `--grid--column-${field.name}`;
      let columnWidth2 = f1.read("/attributes/width", ``, field);
      if (f1.is.empty(columnWidth2)) {
        columnWidth2 = `${safeColumnWidth}px`;
      }
      this.setColumnWidth(css4Variable, columnWidth2, field);
    }, fields);
    this.requestUpdate("type");
  }
  setColumnWidth(css4Variable, widthPx, field) {
    this.style.setProperty(css4Variable, widthPx);
    f1.write("/attributes/width", widthPx, field);
  }
  updated(props) {
    super.updated(props);
    if (props.has("type")) {
      if (!this.state.focused_column) {
        this.__resetFocusState();
      }
    }
    if (props.has("state")) {
      const filterBy = this.type.mutations.filter_by;
      if (!f1.is.empty(filterBy)) {
        this._reconcileSelections(this.grid);
      }
    }
  }
  update(props) {
    super.update(props);
    if (props.has("input")) {
      this._reconcileSelections(this.input);
    }
    if (props.has("selections")) {
      this.setSelectionsDB(this.selections);
    }
    if (props.has("settings")) {
      dom.setAttrs(this, f1.read("/attributes", {}, this.settings));
    }
  }
  _reconcileSelections(gridInput) {
    const selections = f1.safe([], this.selections);
    const input = f1.safe([], gridInput);
    const pullIds = f1.map(f1.read("/id", ""));
    const existingSelections = f1.intersection(pullIds(input), pullIds(selections));
    const selectionsPrime = f1.map((id) => f1.find((_) => _.id === id, input), existingSelections);
    this.setSelections(selectionsPrime);
  }
  clearSelections() {
    return this.setSelections([]);
  }
  setSelections(selections) {
    this.selections = f1.unique(selections);
    this.setSelectionsDB(selections);
    return this.selections;
  }
  setSelectionsDB(selections) {
    this.selectionsDB = f1.map(
      f1.always(true),
      f1.to.record(() => "id", selections)
    );
    this.requestUpdate("selectionsDB");
    return this.selectionsDB;
  }
  render() {
    this.type = f1.safe({}, this.type);
    const state2 = this.state;
    const allFields = this.type.fields;
    const entireInput = state2.view_mode === "selections" ? f1.safe([], this.selections) : f1.to.array(f1.safe([], this.input));
    const mutationOptions = this.type.mutations = f1.merge(
      "over",
      {
        page_size: 100,
        page_index: 0,
        filter_by: [],
        sort_by: []
      },
      this.type.mutations
    );
    const hasGrid = entireInput && allFields && entireInput.length > 0 && f1.is.exists(allFields);
    if (hasGrid && state2.render_mode === "empty") {
      state2.render_mode = "presentation";
    }
    const hasEmptyResults = entireInput && allFields && f1.is.empty(entireInput) && f1.is.exists(allFields);
    if (hasEmptyResults && state2.render_mode === "presentation") {
      state2.render_mode = "empty";
    }
    const renderFn = this.__renderStates[state2.render_mode] || this.__renderStates.presentation;
    if (!renderFn) {
      return nothing;
    }
    const settings = this.settings = f1.merge(
      "over",
      f1.merge("over", f1.clone(this.settingsDefaults), f1.clone(this.settingsMode(this.mode || "fallback"))),
      f1.safe({}, this.settings)
    );
    this.type = f1.safe({}, this.type);
    this.type.fields = f1.safe({}, this.type.fields);
    const tableMutation = adt.mutation("table");
    const fieldMutation = adt.mutation("fields");
    const accepts_preprocessed_table = this.allowPreprocessedInput;
    const fields = fieldMutation.invoke(this.type);
    const input = tableMutation.options({ accepts_preprocessed_table }).includeType(this.type).invoke(entireInput);
    const filteredInputSize = tableMutation.size;
    const typeFields = { ...this.type.fields };
    this.grid = input;
    this.fields = fields;
    dom.write(() => {
      dom.setAttrs(this, {
        "render-mode": state2.render_mode,
        "user-mode": state2.user_mode,
        "view-mode": state2.view_mode
      });
    });
    this.__rowsPerPage = input.length;
    return html`
      <caption>
        ${this.caption}
      </caption>
      <vi-grid-header
        class="grid__header"
        .bulkActions=${this.bulkActions}
        .mutationOptions=${mutationOptions}
        .selections=${this.selections}
        .fields=${typeFields}
        .settings=${settings}
        .input=${input}
        .state=${state2}
        .ns=${this.ns}
      ></vi-grid-header>
      ${renderFn.call(this, mutationOptions, fields, input, state2, settings, this.type, entireInput)}
      <vi-grid-footer
        class="grid__footer"
        .mutationOptions=${mutationOptions}
        .input=${input}
        .fields=${typeFields}
        .state=${state2}
        .settings=${settings}
        .totalSize=${filteredInputSize}
        .selections=${this.selections}
        .ns=${this.ns}
      ></vi-grid-footer>
    `;
  }
  shouldUpdate(props) {
    return props.has("selectionsDB") || props.has("selections") || props.has("input") || props.has("fields") || props.has("state") || props.has("type");
  }
  retry() {
    this.state.render_mode = "presentation";
  }
  __findField(fieldName) {
    const fields = this.type.fields;
    return f1.is.in(fieldName, fields) ? fields[fieldName] : f1.find((field) => field.name === fieldName, fields);
  }
  __rowAction__onSelectedEnded(event) {
    const action = event.detail.item ? event.detail.item.value : event.detail.value;
    const targetEl = HIP.browser.queryOne(event.target.for, this.ns);
    const row = f1.read("/value", null, targetEl);
    this._dispatchEvent("row-action-ended", {
      value: {
        selections: f1.compact([row]),
        action,
        row
      }
    });
  }
  _computeDigestState() {
    const type = f1.read("/type", {}, this);
    return {
      comparators: f1.read("/comparators", {}, type),
      fields: f1.keys(f1.read("/fields", {}, type))
    };
  }
  _save() {
    return f1.mirror({
      internal: {
        flex_monster: f1.read("/pivotTable/report", {}, this.__internal)
      },
      analytics: f1.safe({}, this.analytics),
      settings: this.settings,
      state: this.state,
      type: this.type
    });
  }
  _restore(memento) {
    const { internal, analytics, settings, state: state2, type } = f1.read("/state", {}, memento);
    this.__internal = f1.write(
      "/pivotTable/report",
      f1.read("/flex_monster", f1.read("/pivotTable/report", {}, this.__internal), internal),
      this.__internal
    );
    this.analytics = f1.safe(this.analytics, analytics);
    this.settings = f1.safe(this.settings, settings);
    this.state = f1.safe(this.state, state2);
    this.type = f1.safe(this.type, type);
    return this;
  }
}
GridElement.styles = [stylesTheme, gridStyles, style];
export default HIP.factory.element(GridElement);
