
import { html, LitElement, css } from '../../../hip/library/lit-element/lit-element.js';
import { AsLitHIC, BehaviorElement, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export const styles = css`
  :host {
    --hi-info--per-row: 3; /* Set default value for stats per row */
    --hi-info--gap: var(--hi-px-48); /* 3em - Gap between each info card */
    --hi-info--alignment: center; /* horizontal alignment of info component */
    --hi-info--background-color: var(--hi-clr--white);
    --hi-info--outline-color: var(--hi-clr--transparent);
    --hi-info--value-font: calc(2 * var(--hi-px-28)); /* 3.5em */
    --hi-info--label-font: var(--hi-px-14); /* 0.875em */
    --hi-info--icon-font: var(--hi-px-48); /* 3em */
  }

  .info-container {
    display: grid;
    grid-template-columns: repeat(var(--hi-info--per-row), 1fr);
    gap: var(--hi-info--gap);
    justify-items: var(--hi-info--alignment);
    align-items: end;
    background: var(--hi-info--background-color);
    border: var(--hi-px-1) solid var(--hi-info--outline-color);
    border-top-width: var(--hi-px-4);
  }

  .info-card {
    min-width: calc(2 * var(--hi-px-32));
    max-width: calc(4 * var(--hi-px-32));
    padding: var(--hi-px-8);
    text-align: center;
    box-sizing: border-box;
  }

  .info-value {
    font-family: 'Arial', 'Helvetica', 'Roboto';
    font-size: var(--hi-info--value-font);
    font-weight: bold;
    margin-bottom: var(--hi-px-4);
  }

  .info-label {
    font-size: var(--hi-info--label-font);
    margin-bottom: var(--hi-px-8);
    text-transform: uppercase;
    letter-spacing: var(--hi-px-1);
  }

  hi-icon {
    font-size: var(--hi-info--icon-font) !important;
    margin-bottom: var(--hi-px-4);
  }

  @media (max-width: 767px) {
    .info-container {
      grid-template-columns: 1fr;
    }
  }

  :host([aesthetic='card']) .info-card {
    box-shadow: rgba(0, 0, 0, 0.18) var(--hi-px-0) var(--hi-px-2) var(--hi-px-4);
  }

  :host([aesthetic='tint']) {
    --hi-info--background-color: var(--hi-clr--theme-1);
  }

  :host([aesthetic='outline']) {
    --hi-info--outline-color: var(--hi-clr--theme-3);
  }

  :host([aesthetic='inline']) .info-card,
  :host([aesthetic='inline']) .info-value,
  :host([aesthetic='inline']) .info-container {
    --hi-info--per-row: 1;
    --hi-info--alignment: flex-start;
    display: inherit;
    margin: 0;
    padding: 0;
    min-width: auto;
  }
`;
export class InfoElement extends BehaviorElement(AsLitHIC(LitElement)) {
  static get is() {
    return "vi-info";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      type: {
        type: Object
      },
      input: {
        type: Object
      },
      aesthetic: {
        type: String
      }
    };
  }
  getRenderHints(type) {
    return {
      renderHintsScheme: f1.read(`/render_hints/color/scheme`, void 0, type),
      renderHintsColors: f1.read(`/render_hints/color/values`, ["var(--hi-clr--theme-5)"], type),
      renderHintsLabels: f1.read(`/render_hints/color/labels`, ["var(--hi-clr--gray-10)"], type)
    };
  }
  getCardColor(theme, colors, count, index) {
    if (theme) {
      return "var(--hi-clr--theme-5)";
    } else {
      return colors[index % count];
    }
  }
  setTheme(renderHintsScheme) {
    if (!this.theme) {
      this.theme = renderHintsScheme;
    }
  }
  renderIcon(input, index, valueColor) {
    return input.icon ? html` <div class="info-icon" style="color:${valueColor}">
          ${input.icon ? html`<hi-icon glyph=${input.icon} inherit-color></hi-icon>` : ""}
        </div>` : "";
  }
  renderValue(input, index, valueColor) {
    return input.value ? html`<div class="info-value" style="color:${valueColor}">${input.value}</div>` : "";
  }
  renderLabel(input, index, labelColor) {
    return input.label ? html`<div class="info-label" style="color:${labelColor}">${input.label}</div>` : "";
  }
  renderInfoCards(type, input) {
    const { renderHintsScheme, renderHintsColors, renderHintsLabels } = this.getRenderHints(type);
    const valuesColorsCount = renderHintsColors.length;
    const labelsColorsCount = renderHintsLabels.length;
    this.setTheme(renderHintsScheme);
    return input.map((input2, index) => {
      const valueColor = this.getCardColor(this.theme, renderHintsColors, valuesColorsCount, index);
      const labelColor = this.getCardColor(this.theme, renderHintsLabels, labelsColorsCount, index);
      return html`
        <div class="info-card">
          ${this.renderIcon(input2, index, valueColor)} ${this.renderValue(input2, index, valueColor)}
          ${this.renderLabel(input2, index, labelColor)}
        </div>
      `;
    });
  }
  render() {
    if (!this.input) {
      return null;
    }
    return html` <div class="info-container">${this.renderInfoCards(this.type, this.input)}</div> `;
  }
}
export default HIP.factory.element(InfoElement);
