
import { css, LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { AsLitHIC, BehaviorElement, HIP } from '../../../eip/element/hi-component/hi-component.js';
import adt from '../../../pip/library/pi-adt/pi-adt.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import '../../../eip/element/hi-button/hi-button.js';
import '../../../eip/element/hi-menu/hi-menu.js';
import '../../../eip/element/hi-menu/hi-pick.js';
export const style = css`
  :host {
    width: 100%;
    display: block;
    background-color: var(--hi-clr--white);
    border-bottom: var(--hi-px-1) solid var(--hi-clr--gray-4);
  }

  .list__bar {
    width: 100%;
  }

  .list__bar--top {
    justify-content: space-between;
    border-bottom: var(--border--style);
    width: 100%;
    overflow-x: auto;
  }

  .list__bar--top,
  .list__bar--bottom {
    padding: var(--hi-px-0) var(--hi-px-12);
    display: flex;
    align-items: center;
    height: var(--hi-px-48);
  }

  .list__bar--info {
    display: flex;
    align-items: center;
  }

  .list__bar--title {
    padding-left: var(--hi-px-8);
  }

  .hide-content {
    visibility: hidden;
    position: absolute;
    height: 1px;
    width: 1px;
    overflow: hidden;
    clip: rect(1px, 1px, 1px, 1px);
  }

  .list__bar--right,
  .list__bar--left {
    display: flex;
    align-items: center;
    gap: var(--hi-px-6);
  }

  .list__bar--left,
  .list__bar--middle {
    margin-right: var(--hi-px-8);
  }

  .list__bar--middle {
    flex-grow: 100;
  }

  .cell__title {
    text-overflow: ellipsis;
    white-space: nowrap;
    overflow: hidden;

    margin-right: auto;

    font-variant: small-caps;
    text-transform: capitalize;
    letter-spacing: 0.025em;
  }

  @media only screen and (max-width: 40rem) {
    .list__bar--top,
    .list__bar--bottom {
      height: calc(var(--hi-px-48) + var(--hi-px-14));
    }
  }
`;
export class ListHeaderElement extends BehaviorElement(AsLitHIC(LitElement)) {
  constructor() {
    super(...arguments);
    this.selections = [];
  }
  static get is() {
    return "vi-list-header";
  }
  static get styles() {
    return [stylesTheme, style];
  }
  static get properties() {
    return {
      mutationOptions: {
        type: Object
      },
      fields: {
        type: Object
      },
      input: {
        type: Array
      },
      state: {
        type: String
      },
      settings: {
        type: Object
      },
      bulkActions: {
        type: Array
      },
      selections: {
        type: Array
      }
    };
  }
  render() {
    const fields = this.fields;
    const input = this.input;
    if (f1.is.nil(fields) || f1.is.nil(input)) {
      return nothing;
    }
    const bulkActions = f1.safe([], this.bulkActions);
    const ns = this.ns;
    const hasBulkActions = !f1.is.empty(bulkActions);
    const selectionCount = f1.safe(0, this.selections.length);
    const settings = this.settings;
    return html`<section class="list__bar">
      <div class="list__bar--top">
        ${!hasBulkActions || selectionCount <= 0 ? nothing : html`<div class="list__bar--left">
              ${bulkActions.length === 1 ? html`<hi-button
                    id="bulkActions"
                    gid="bulkActions"
                    aesthetic="secondary"
                    @trigger-ended=${this.__bulkAction__onSelectedEnded}
                    .value=${bulkActions[0].value}
                    .ns=${ns}
                  >
                    <span>${f1.safe(settings.messages.bulk_actions.title, bulkActions[0].title)}</span>
                    <hi-hint placement="right">${settings.messages.bulk_actions.description}</hi-hint>
                  </hi-button>` : html`
                    <hi-button
                      id="bulkActions"
                      gid="bulkActions"
                      aesthetic="secondary"
                      bind="bulkActionsMenu:activate"
                      .ns=${ns}
                    >
                      <span>${settings.messages.bulk_actions.title}</span>
                      <hi-hint placement="right">${settings.messages.bulk_actions.description}</hi-hint>
                    </hi-button>
                    <hi-menu
                      id="bulkActionsMenu"
                      gid="bulkActionsMenu"
                      for="bulkActions"
                      placement="bottom-start"
                      cap-height
                      .ns=${ns}
                      .subtitle=${settings.messages.bulk_actions.title}
                      @selected-ended=${this.__bulkAction__onSelectedEnded}
                    >
                      ${bulkActions.map(
      (field) => html`<hi-pick .value=${field.value} .ns=${ns}>${field.title}</hi-pick>`
    )}
                    </hi-menu>
                  `}
            </div>`}
        <div class="list__bar--middle">
          ${settings.hidden.global_search ? nothing : this.__renderGlobalSearch(settings.labels)}
        </div>
        <div class="list__bar--right">
          <div style="display:flex">
            <hi-button
              id="clearAllLimits"
              gid="clearAllLimits"
              aesthetic="icon"
              .ns=${ns}
              ?hidden=${settings.hidden.clear_all_limits}
              @trigger-ended=${this.__clearAllLimits__onTriggerEnded}
            >
              <hi-icon kind="layers_clear" type="material"></hi-icon>
              <hi-hint placement="left"> Clear all sorts and filters </hi-hint>
            </hi-button>
            <hi-button
              id="addRow"
              gid="addRow"
              aesthetic="icon"
              .ns=${ns}
              ?hidden=${settings.hidden.add_row}
              @trigger-ended=${this.__addRow__onTriggerEnded}
            >
              <hi-icon kind="add_box" type="material"></hi-icon>
              <hi-hint placement="left"> Add a row </hi-hint>
            </hi-button>
          </div>
        </div>
      </div>
      <div class="list__bar--bottom">
        <div class="list__bar--info">
          ${!settings.allow.row_selections ? nothing : html`<hi-checkbox
                id="rowSelected__checkboxControl"
                kind="group"
                aria-label="Toggle Bulk Selection Of Rows"
                .state=${this.state}
                .ns=${this.ns}
                @toggle-ended=${this.__checkboxControl__onToggleEnded}
              ></hi-checkbox>`}
          <span class="list__bar--title cell__title">${this.settings.subtitle}</span>
        </div>
      </div>
    </section>`;
  }
  __renderGlobalSearch(labels) {
    const ns = this.ns;
    return html`<hi-form alignment="inline" kind="filter" hidden-controls disable-http style="width: 100%;" .ns=${ns}>
      <hi-input-group .ns=${ns} style="flex-grow: 1; width:100%; margin: 0">
        <hi-label id="filterAllLabel" gid="filterAllLabel" .ns=${ns} class="hide-content">Search List</hi-label>
        <hi-input
          id="filterAll"
          gid="filterAll"
          type="search"
          aria-labelledby="filterAllLabel"
          placeholder=${labels.global_search}
          .ns=${ns}
          @enter-keyup-ended=${this.__filterAll__onCommitEnded}
          @esc-keyup-ended=${this.__filterAll__onCommitEnded}
          @commit-ended=${this.__filterAll__onCommitEnded}
        ></hi-input>
      </hi-input-group>
      <hi-button type="submit" class="hide-content" slot="controls">
        <hi-icon kind="search" type="material"></hi-icon>
      </hi-button>
    </hi-form>`;
  }
  __checkboxControl__onToggleEnded(event) {
    const checkboxEl = this.__getCheckbox();
    if (!checkboxEl) {
      return;
    }
    if ("checked" === checkboxEl.state) {
      this._dispatchNotification("row-select-all-page");
    } else if ("unchecked" === checkboxEl.state) {
      this._dispatchNotification("row-unselect-all-page");
      this._dispatchNotification("view-state", {
        viewState: "list"
      });
    }
  }
  __getCheckbox() {
    return this.shadowRoot.getElementById("rowSelected__checkboxControl");
  }
  __bulkAction__onSelectedEnded(event) {
    const actionName = event.detail.item ? event.detail.item.value : event.detail.value;
    this._dispatchNotification("bulk-action", {
      actionName
    });
  }
  __addRow__onTriggerEnded(evt) {
    this._dispatchProcess("add-row", {
      type: this.type
    });
  }
  __clearAllLimits__onTriggerEnded(evt) {
    const filterAllEl = this.shadowRoot?.querySelector("#filterAll");
    if (filterAllEl) {
      filterAllEl.clear();
    }
    this._dispatchProcess("clear-all-limits");
  }
  __filterAll__onCommitEnded(evt) {
    const value = evt.srcElement.value;
    const filterByRules = f1.remove((_) => _.field === "*", this.type.mutations.filter_by || []);
    const filter_by = f1.is.empty(value) ? filterByRules : [
      ...filterByRules,
      {
        comparator: adt.comparators.isLike.id,
        field: "*",
        value
      }
    ];
    this._dispatchNotification("state-action", {
      mutationOptions: {
        page_index: 0,
        filter_by
      }
    });
  }
}
export default HIP.factory.element(ListHeaderElement);
