
import { html, css, LitElement } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { AsLitHIC, BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import { FactoryViewElement } from '../../../aip/element/ai-factory-view/ai-factory-view.js';
import adt from '../../../pip/library/pi-adt/pi-adt.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import '../../../aip/element/ai-factory-viewer/ai-factory-viewer.js';
import './vi-list-header.js';
class TileTemplate {
  constructor() {
    this.id = "list-template-tile";
  }
  render({ rowActionsTmplFn, type }) {
    return (entity, index) => {
      const rowActionsTmpl = rowActionsTmplFn(entity);
      return html`<li kind="no-dot" data-index=${index}>
        <hi-tile class="mbm card card--entity">
          ${rowActionsTmpl === nothing ? nothing : html`<div slot="toolbar">${rowActionsTmpl}</div>`}
          <section class="card__details">
            <ai-factory-viewer .type=${type} .value=${entity}></ai-factory-viewer>
          </section>
        </hi-tile>
      </li>`;
    };
  }
}
class ItemTemplate {
  constructor() {
    this.id = "list-template-item";
    this.entityMuation = adt.mutation("entity");
  }
  render({
    titleTmplFn,
    rowActionsTmplFn,
    type
  }) {
    this.entityMuation.includeType(type);
    return (_entity, index) => {
      const entity = this.entityMuation.invoke(_entity);
      const rowActionsTmpl = rowActionsTmplFn(entity);
      const description = f1.read("/description", void 0, entity);
      const subTitle = f1.read("/sub_title", void 0, entity);
      const title = f1.read("/title", "", entity);
      return html`<li kind="no-dot" data-index=${index}>
        <article kind="item">
          <div class="item__title">
            <h6>${titleTmplFn(title, entity)}</h6>
            ${subTitle ? html`<p class="item__content"><em>${subTitle}</em></p>` : nothing}
            ${description ? html`<p class="item__content">${description}</p>` : nothing}
          </div>
          ${rowActionsTmpl === nothing ? nothing : html`<section class="item__actions">${rowActionsTmpl}</section>`}
        </article>
      </li>`;
    };
  }
}
export const style = css`
  :host {
    --list--max-height: 24rem;
    --list--min-height: 12rem;
    --list--tile-height: var(--hi-px-48);
    --border--style: var(--hi-px-1) solid var(--hi-clr--gray-4);
  }

  :host {
    width: 100%;
    overflow: auto;
    height: auto;
    display: grid;
    border-collapse: collapse;
    background-color: var(--hi-clr--white);
    border: var(--border--style);

    grid-template-rows: min-content 1fr min-content;
    min-height: var(--list--min-height);
  }

  .list {
    list-style: none;
    display: block;

    background-color: var(--hi-clr--gray-0);

    max-height: var(--list--max-height);
    padding: var(--hi-px-16);
    margin: 0;
  }

  :host([fit-height]) {
    height: 100%;
  }

  :host([fit-height]) .list {
    max-height: inherit;
  }

  :host([mode='item']) .list {
    padding: 0;
  }

  :host([disallow-header]) {
    grid-template-rows: 1fr min-content;
  }

  .list.state--error,
  .list.state--empty {
    display: flex;
    place-items: center;
    flex-direction: column;
    justify-content: center;
    text-align: center;
  }

  .list.state--empty,
  .list.state--error {
    margin-top: var(--hi-px-40);
  }

  .list.state--presentation {
    padding: var(--hi-px-0);
    overflow-x: none;
    overflow-y: auto;

    scroll-snap-points-y: repeat(var(--list--tile-height));
    scroll-snap-type: y mandatory;
    scroll-snap-type: mandatory;
  }

  .list__footer,
  .list__header {
    left: 0;
    right: 0;
    display: flex;
    align-items: center;
    justify-content: flex-start;
    background-color: var(--hi-clr--white);
    padding: var(--hi-px-8);
    height: var(--hi-px-48);
    gap: var(--hi-px-6);
  }

  .list__header {
    padding: var(--hi-px-0);
    border-bottom: var(--border--style);
    justify-content: start;
    height: min-content;
    overflow: hidden;
    z-index: 800;
    top: 0;
  }

  :host([disallow-header]) .list_header {
    display: none;
  }

  .list__footer {
    height: max-content;
    border-top: var(--border--style);
    justify-content: end;
    z-index: 400;
    bottom: 0;
    display: flex;
    flex-wrap: wrap;
  }

  :host([aesthetic='invisible']) {
    --border--style: none;
  }

  #pageSizeInput {
    max-width: calc(2 * var(--hi-px-36));
  }

  /*
   * Adding styling of hi-label to span to keep the same look, since we replaced it for AX.
   */
  .paginationDetails {
    min-height: auto;
    line-height: var(--hi-font-lh--text);
    font-size: var(--hi-font-size--text);
    font-weight: var(--hi-font--weight-light);
  }
`;
export class ListElement extends BehaviorElement(AsLitHIC(LitElement)) {
  constructor() {
    super();
    this.settings = {};
    this.settingsDefaults = {
      subtitle: "Items",
      allow: {
        row_selections: false
      },
      messages: {
        bulk_actions: {
          title: "Actions",
          description: "Perform action on selected row(s)"
        },
        state_error: {
          title: "Error",
          description: "Your content cannot be rendered."
        },
        state_empty: {
          title: "Empty",
          description: "No content."
        }
      },
      labels: {
        global_search: "Search in Results"
      },
      hidden: {
        add_row: true,
        global_search: false,
        clear_all_limits: true,
        pagination_controls: false,
        pagination_row_size: false,
        pagination_row_message: false
      }
    };
    this.selections = [];
    this.selectionsDB = {};
    this.maxSelections = -1;
    this.selectionGroup = `list__${f1.id()}`;
    this.allowPreprocessedInput = false;
    this.templateStrategy = f1.strategy({
      tile: () => new TileTemplate(),
      item: () => new ItemTemplate(),
      fallback: () => new TileTemplate()
    });
    this.listMutation = adt.mutation("table");
    this.fieldMutation = adt.mutation("fields");
    this.viewFactory = new FactoryViewElement();
    this.ns = `vi-list-${f1.id()}`;
  }
  static get is() {
    return "vi-list";
  }
  static get properties() {
    return {
      mode: {
        type: String,
        reflect: true
      },
      template: {
        type: Object
      },
      type: {
        type: Object
      },
      input: {
        type: Array
      },
      settings: {
        type: Object
      },
      context: {
        type: Object
      },
      bulkActions: {
        type: Array,
        attribute: "bulk-actions"
      },
      rowActions: {
        type: Array,
        attribute: "row-actions"
      },
      maxSelections: {
        type: Number,
        attribute: "max-selections"
      },
      selectionGroup: {
        type: String,
        attribute: "selection-group"
      },
      aesthetic: {
        type: String
      },
      allowPreprocessedInput: {
        type: Boolean,
        attribute: "allow-preprocessed-input"
      },
      disallowHeader: {
        type: Boolean,
        attribute: "disallow-header",
        reflect: true
      }
    };
  }
  static get api() {
    return ["pageTo", "sortBy", "filterBy", "updateMutations", "updateSettings"];
  }
  connectedCallback() {
    super.connectedCallback();
    this.__defaultSettings();
    this.__includeEventHandlers();
    this.__title__onFieldTriggerEndedBound = this.__title__onFieldTriggerEnded.bind(this);
  }
  __includeEventHandlers() {
    this.viewFactory.addEventListener("field-trigger-ended", this.__title__onFieldTriggerEndedBound);
    this.addEventListener("row-select-all-page-notified", (evt) => {
      evt.stopPropagation();
      this.selectAllOnPage();
    });
    this.addEventListener("row-unselect-all-page-notified", (evt) => {
      evt.stopPropagation();
      this.deselectAllOnPage();
    });
    this.addEventListener("row-select-all-notified", (evt) => {
      evt.stopPropagation();
      this.selectAll();
    });
    this.addEventListener("row-unselect-all-notified", (evt) => {
      evt.stopPropagation();
      this.deselectAll();
    });
    this.addEventListener("row-selected-notified", (evt) => {
      evt.stopPropagation();
      this.selectRow(evt.detail.row);
    });
    this.addEventListener("row-unselected-notified", (evt) => {
      evt.stopPropagation();
      this.deselectRow(evt.detail.row);
    });
    this.addEventListener("bulk-action-notified", (event) => {
      event.stopPropagation();
      const { actionName: action } = event.detail;
      this._dispatchEvent("bulk-action-ended", {
        value: {
          action,
          selections: this.selections
        }
      });
    });
    this.addEventListener("state-action-notified", (event) => {
      const { mutationOptions = {} } = event.detail;
      event.stopPropagation();
      if (!f1.is.empty(mutationOptions)) {
        this.updateMutations(mutationOptions);
      }
    });
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    this.viewFactory.removeEventListener("field-trigger-ended", this.__title__onFieldTriggerEndedBound);
  }
  selectAllOnPage() {
    this.__changeSelections(f1.union(f1.safe([], this.selections), f1.copy(this.list)));
  }
  deselectAllOnPage() {
    this.__changeSelections(f1.difference(f1.copy(this.list), f1.safe([], this.selections)));
  }
  selectAll() {
    this.__changeSelections(f1.copy(f1.safe(this.input, this.backupList)));
  }
  deselectAll() {
    this.__changeSelections([]);
  }
  selectRow(row) {
    this.__changeSelections(f1.push(row, this.selections), "selection-changed");
  }
  deselectRow(row) {
    this.__changeSelections(
      f1.remove((selection) => selection.id === row.id, this.selections),
      "selection-changed"
    );
  }
  __changeSelections(selections, selectionEventName = "selections-changed") {
    selections = this.setSelections(selections);
    this._dispatchEvent(selectionEventName, {
      value: selections
    });
  }
  setSelections(selections) {
    this.selections = f1.unique(selections);
    this.setSelectionsDB(selections);
    return this.selections;
  }
  setSelectionsDB(selections) {
    this.selectionsDB = f1.map(
      f1.always(true),
      f1.to.record(() => "id", selections)
    );
    this.requestUpdate("selectionsDB");
    return this.selectionsDB;
  }
  performUpdate() {
    this.iteratorListEl = this.iteratorListEl || dom.create(this, "ul", {
      slot: "list"
    });
    this.iteratorEl = this.iteratorEl || dom.create(this, "hi-iterator", {
      for: this.iteratorListEl
    });
    super.performUpdate();
  }
  render() {
    if (!this.type) {
      return nothing;
    }
    const bulkActions = f1.safe([], this.bulkActions);
    const entireInput = f1.to.array(f1.safe([], this.input));
    const settings = this.settings = f1.merge(
      "over",
      f1.merge("over", {}, this.settingsDefaults),
      f1.safe({}, this.settings)
    );
    if (f1.is.empty(entireInput)) {
      return html`
        <section class="list state--empty">
          <h3>${settings.messages.state_empty.title}</h3>
          <p>${settings.messages.state_empty.description}</p>
        </section>
      `;
    }
    const mutationOptions = this.type.mutations = f1.merge(
      "over",
      {
        page_size: 100,
        page_index: 0,
        filter_by: [],
        sort_by: []
      },
      this.type.mutations
    );
    const accepts_preprocessed_table = this.allowPreprocessedInput;
    const fields = this.fieldMutation.invoke(this.type);
    const input = this.listMutation.options({ accepts_preprocessed_table }).includeType(this.type).invoke(entireInput);
    this.list = input;
    this.fields = fields;
    const rowActions = f1.safe([], this.rowActions);
    const type = this.type;
    const selected = f1.intersection(input, this.selections);
    const totalSelected = f1.size(selected);
    const maxInputSize = f1.safe(0, mutationOptions.page_size);
    const filteredTotalSize = f1.safe(0, input.length);
    const pageIndex = f1.safe(0, mutationOptions.page_index);
    const pageSize = f1.safe(0, mutationOptions.page_size);
    const totalSize = this.listMutation.size;
    const currentPage = mutationOptions.current_page;
    const links = mutationOptions.links;
    const paginationInput = { links, current_page: currentPage };
    const DefaultTemplate = this.templateStrategy(this.mode);
    this.iteratorEl.input = this.list;
    this.iteratorEl.template = f1.safe(DefaultTemplate(), this.template);
    this.iteratorEl.context = f1.merge("over", f1.safe({}, this.context), {
      viewFactory: this.viewFactory,
      titleTmplFn: this.__renderFieldTitleTemplate(),
      rowActionsTmplFn: (entity) => this.__renderRowActions(rowActions, entity),
      bulkActionsTmplFn: (entity, index) => this.__renderBulkActions(bulkActions, entity, index),
      rowActions,
      bulkActions,
      type
    });
    const ns = this.ns;
    const state = totalSelected <= 0 ? "unchecked" : totalSelected >= maxInputSize ? "checked" : "partial";
    return html`
      ${this.disallowHeader ? nothing : html` <vi-list-header
            class="list__header"
            .bulkActions=${this.bulkActions}
            .mutationOptions=${mutationOptions}
            .selections=${this.selections}
            .fields=${this.fields}
            .settings=${settings}
            .type=${this.type}
            .input=${input}
            .state=${state}
            .ns=${this.ns}
          ></vi-list-header>`}
      <section x-hip class="list state--presentation">
        <slot name="list"></slot>
      </section>
      <section class="list__footer">
        ${settings.hidden.pagination_row_message ? nothing : html`<span class="paginationDetails">
              ${pageSize * pageIndex + 1} - ${pageSize * pageIndex + filteredTotalSize} of ${totalSize}
            </span>`}
        ${settings.hidden.pagination_row_size ? nothing : html` <hi-input
              id="pageSizeInput"
              gid="pageSizeInput"
              .ns=${ns}
              .value=${pageSize}
              type="number"
              min="10"
              aria-label="Page Size"
              @commit-ended=${this.__pageSize__onCommitEnded}
            >
            </hi-input>`}
        ${settings.hidden.pagination_controls ? nothing : html`<hi-pagination
              kind="determinate"
              page-index=${pageIndex}
              page-size=${pageSize}
              max=${totalSize}
              .pageCurrentOverride=${currentPage}
              .input=${paginationInput}
              .ns=${ns}
              @paginate-ended=${this.__pagination__onPaginateEnded}
              @page-index-changed=${this.__pagination__onPageIndexChanged}
            ></hi-pagination>`}
      </section>
      <section class="row__action__menu__container">
        <hi-menu
          gid="row-menu"
          id="row-menu"
          .ns=${this.ns}
          cap-height
          @selected-ended=${this.__rowAction__onSelectedEnded}
        >
          ${f1.map(
      (rowAction) => html`<hi-pick .value=${rowAction.value} .ns=${this.ns}> ${rowAction.title} </hi-pick>`,
      rowActions
    )}
        </hi-menu>
      </section>
    `;
  }
  __title__onFieldTriggerEnded(evt) {
    this._dispatchProcess("field-trigger", evt.detail);
  }
  __renderFieldTitleTemplate() {
    const viewFactory = this.viewFactory;
    const titleField = f1.find((field) => field.alias === "title" || field.name === "title", this.fields);
    const renderType = f1.read("/render_type", "text", titleField);
    const viewType = viewFactory && viewFactory.hasRender("view", renderType) ? renderType : "unknown";
    const renderTypeFn = viewFactory.renderBy("view", viewType);
    return (value, entity) => {
      return renderTypeFn.render(adt.expression.evaluateAll(titleField, entity), viewFactory)(
        value,
        entity
      );
    };
  }
  __renderRowActions(rowActions, row) {
    if (rowActions.length <= 0) {
      return nothing;
    }
    return html` <hi-button
      aria-label="open row actions"
      enable-isolated-events
      aria-haspopup="true"
      aesthetic="icon"
      .value=${row}
      .ns=${this.ns}
      @trigger-ended=${(e) => this.__rowAction__onTriggerEnded(e)}
    >
      <hi-icon kind="menu" type="material"></hi-icon>
    </hi-button>`;
  }
  __renderBulkActions(bulkActions, row, index) {
    if (!this.settings.allow.row_selections) {
      return nothing;
    }
    if (bulkActions.length <= 0) {
      return nothing;
    }
    const id = row.id;
    return html` <hi-checkbox
      id="checkbox--${id}"
      group=${this.selectionGroup}
      data-id=${id}
      data-index=${index}
      aria-label="Toggle Row Selection"
      .row=${row}
      .ns=${this.ns}
      ?checked=${!!this.selectionsDB[id]}
      @toggle-ended=${this.__rowSelector__onToggleEnded}
    ></hi-checkbox>`;
  }
  __rowSelector__onToggleEnded(event) {
    if (!event.target) {
      return;
    }
    const rowEl = event.target;
    const rowEvent = rowEl.checked ? "row-selected" : "row-unselected";
    const rowIndex = rowEl.getAttribute("data-index");
    const rowId = rowEl.getAttribute("data-id") || "?";
    const row = rowEl.row;
    const eventDetails = {
      rowIndex,
      rowId,
      rowEl,
      row
    };
    this._dispatchNotification(rowEvent, eventDetails);
  }
  __rowAction__onTriggerEnded(evt) {
    evt.preventDefault();
    evt.stopImmediatePropagation();
    const sectionEl = this.shadowRoot?.querySelector("section.row__action__menu__container");
    if (!sectionEl) {
      return;
    }
    const menuEl = sectionEl.firstElementChild;
    if (menuEl) {
      menuEl["row"] = f1.read("/currentTarget/value", null, evt);
      menuEl.for = evt.target;
      menuEl.activate("--", evt);
    }
  }
  __rowAction__onSelectedEnded(evt) {
    const action = evt.detail.item ? evt.detail.item.value : evt.detail.value;
    const rowValue = f1.read("/target/for/value", null, evt);
    const row = f1.read("/currentTarget/row", null, evt);
    this._dispatchEvent("row-action-ended", {
      value: {
        selections: f1.compact([rowValue]),
        action,
        row
      }
    });
  }
  __defaultSettings() {
    this.settings = f1.merge("over", f1.merge("over", {}, this.settingsDefaults), f1.safe({}, this.settings));
  }
  pageTo(mutationOptions) {
    return this.updateMutations(mutationOptions);
  }
  sortBy(mutationOptions) {
    return this.updateMutations(mutationOptions);
  }
  filterBy(mutationOptions) {
    return this.updateMutations(mutationOptions);
  }
  updateMutations(mutations) {
    this.type.mutations = f1.merge("over", this.type.mutations, mutations);
    this.requestUpdate("type");
    return this.type.mutations;
  }
  updateSettings(settings) {
    this.settings = f1.merge("over", this.settings, settings);
    this.requestUpdate("settings");
    return this.settings;
  }
  update(changedProperties) {
    changedProperties.forEach((value, prop) => {
      if (prop === "settings") {
        this.__defaultSettings();
      }
    });
    super.update(changedProperties);
  }
  __pageSize__onCommitEnded(evt) {
    const pageSize = evt.detail.value.value;
    const currentPageSize = this.type.mutations.page_size;
    if (currentPageSize === pageSize) {
      return;
    }
    this.pageTo({
      page_size: pageSize,
      page_index: 0
    });
  }
  __pagination__onPaginateEnded(event) {
    const { steps, pageIndex, pageSize } = event.detail;
    const linkUri = !f1.is.empty(steps) ? steps[event.detail.value] : "";
    this._dispatchProcess("pagination", {
      pageIndex,
      pageSize,
      linkUri
    });
  }
  __pagination__onPageIndexChanged(evt) {
    const currentPageIndex = this.type.mutations.page_index;
    const changedPageIndex = evt.detail.value;
    if (currentPageIndex === changedPageIndex) {
      return;
    }
    this.pageTo({
      page_index: changedPageIndex
    });
  }
  shouldUpdate(changedProperties) {
    return changedProperties.has("selectionsDB") || changedProperties.has("selections") || changedProperties.has("template") || changedProperties.has("context") || changedProperties.has("fields") || changedProperties.has("input") || changedProperties.has("type") || changedProperties.has("mode");
  }
}
ListElement.styles = [stylesTheme, style];
export default HIP.factory.element(ListElement);
