
import { html, css, LitElement } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { AsLitHIC, BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import '../../../eip/element/hi-badge/hi-badge.js';
import '../../../eip/element/hi-icon/hi-icon.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export const styles = css`
  :host {
    --suffix-prefix--font-size: var(--hi-px-16);
    --theme-color: var(--hi-clr--theme-3);
    --value--font-size: var(--hi-px-40);
  }

  :host {
    display: inline-flex;
    flex-direction: column;
    padding: var(--hi-px-20) var(--hi-px-40) var(--hi-px-20) 0;
  }

  :host([aesthetic='boxed']),
  :host([aesthetic='standard']) {
    padding-left: var(--hi-px-12);
    border: var(--hi-px-1) solid var(--theme-color);
    border-top: var(--hi-px-4) solid var(--theme-color);
    box-shadow: var(--hi-clr--shadow-down-2);
    background-color: var(--hi-clr--gray-1);
    border-radius: var(--hi-px-5);
  }

  #metricHeader,
  #metricLabel {
    margin-left: var(--hi-px-4);
  }

  #metricLabel {
    display: flex;
    align-items: start;
    flex-direction: column;
    margin-top: auto;
  }

  #header {
    font-weight: var(--hi-font--weight-heavy);
    font-size: var(--hi-font-size--normal);
    justify-content: flex-start;
    align-items: center;
    display: flex;
  }

  #header hi-icon {
    --hi-icon--color: var(--theme-color);
    margin-right: var(--hi-px-4);
  }

  #metricDetails {
    padding: 0 0 var(--hi-px-20) var(--hi-px-4);
  }
  #metricValue {
    gap: var(--hi-px-3);
    padding: var(--hi-px-8) var(--hi-px-4) var(--hi-px-8) var(--hi-px-4);
    align-items: center;
    display: flex;
  }

  #metricValue.no-label {
    padding: var(--hi-px-8) var(--hi-px-4) 0 var(--hi-px-4);
  }

  #value {
    letter-spacing: -1px;
    font-size: var(--value--font-size);
  }

  #suffixId,
  #prefixId {
    font-weight: var(--hi-font--weight-heavy);
    font-size: var(--suffix-prefix--font-size);
  }

  #trendIcon {
    display: inline-block;
    margin-right: var(--hi-px-3);
  }

  #labelText {
    font-size: var(--hi-font-size--small);
    margin-left: var(--hi-px-3);
  }

  :host([size='small']) #metricValue {
    padding: var(--hi-px-8) 0;
  }

  :host([size='small']) {
    padding: var(--hi-px-16) var(--hi-px-28) var(--hi-px-20) var(--hi-px-8);
  }

  :host([size='small']) #value {
    font-size: calc(0.75 * var(--value--font-size));
  }

  :host([size='large']) #value {
    font-size: calc(1.5 * var(--value--font-size));
  }
`;
export class MetricElement extends BehaviorElement(AsLitHIC(LitElement)) {
  constructor() {
    super(...arguments);
    this.aesthetic = "minimal";
    this.trend = "neutral";
    this.trendTheme = f1.strategy({
      positive: "green",
      negative: "red",
      fallback: "blue"
    });
    this.trendLabel = f1.strategy({
      positive: html`<span id="trendIcon" aria-label="positive trend" role="presentation">↑</span>`,
      negative: html`<span id="trendIcon" aria-label="negative trend" role="presentation">↓</span>`,
      fallback: nothing
    });
  }
  static get is() {
    return "vi-metric";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      value: {
        type: String,
        notify: true
      },
      header: {
        type: String
      },
      label: {
        type: String
      },
      labelText: {
        type: String,
        attribute: "label-text"
      },
      icon: {
        type: String
      },
      suffix: {
        type: String
      },
      prefix: {
        type: String
      },
      aesthetic: {
        type: String,
        reflect: true
      },
      trend: {
        type: String,
        reflect: true
      },
      customTheme: {
        type: String,
        attribute: "custom-theme",
        reflect: true
      }
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "status");
  }
  render() {
    return html`
      <header id="metricHeader">
        <div id="header">
          ${this.icon ? html`<hi-icon kind="${this.icon}"> </hi-icon>` : nothing}
          <span>${this.header}</span>
        </div>
        <slot name="headerSlot"></slot>
        <slot name="header"></slot>
      </header>
      <section id="metricValue" class="${this.label ? "" : "no-label"}">
        ${this.prefix ? html`<sup id="prefixId">${this.prefix}</sup>` : nothing}
        <span id="value"> ${this.value} </span>
        ${this.suffix ? html`<sub id="suffixId">${this.suffix}</sub>` : nothing}
      </section>
      <section id="metricDetails">
        <slot name="details"></slot>
      </section>
      <footer id="metricLabel">
        <div id="footer">
          ${this.label ? html`<hi-badge theme=${this.trendTheme(this.trend)}>
                  ${this.trendLabel(this.trend)}
                  <span id="label">${this.label}</span>
                </hi-badge>
                <span id="labelText">${this.labelText}</span>` : nothing}
        </div>
        <slot name="footer"></slot>
      </footer>
    `;
  }
  updated(props) {
    if (props.has("customTheme")) {
      this.style.setProperty("--theme-color", this.customTheme);
    }
  }
}
export default HIP.factory.element(MetricElement);
