
import { html, css, LitElement } from '../../../hip/library/lit-element/lit-element.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { HIP, AsLitHIC, BehaviorElement } from '../../../eip/element/hi-hip/hi-hif.js';
import { FactoryEditElement } from '../../../aip/element/ai-factory-edit/ai-factory-edit.js';
import { writeAlways } from '../../../eip/element/hi-lit/hi-lit.js';
import adt from '../../../pip/library/pi-adt/pi-adt.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export const styles = css`
  :host {
    display: block;
  }

  ul {
    margin: 0;
    padding: 0;
  }

  li {
    list-style: none;
  }

  hi-toggle-group {
    margin-right: auto;
  }

  header {
    align-items: center;
    display: flex;
  }

  section {
    position: relative;
    padding-left: var(--hi-px-16);
    border-left: var(--hi-px-2) solid var(--hi-clr--theme-5);
  }

  section::before {
    position: absolute;
    left: 0;
    top: calc(50% - 2ch);
    transform: translateX(-50%) rotateX(360deg);
    writing-mode: vertical-lr;
    font-size: var(--hi-font-size--small);
    font-weight: var(--hi-font--weight-bold);
    letter-spacing: 1.1pt;
    padding: var(--hi-px-3);
    background-color: var(--hi-clr--white);
  }

  section.hide-join::before {
    display: none;
  }

  section.join-and::before {
    content: 'AND';
  }

  section.join-or::before {
    content: 'OR';
  }

  footer {
    padding: var(--hi-px-8);
  }

  footer:not(.is-root) {
    border-top: var(--hi-px-1) solid var(--hi-clr--theme-5);
    background-color: var(--hi-clr--gray-1);
    margin: calc(-1 * var(--hi-px-8));
    margin-left: calc(-1 * var(--hi-px-16));
    margin-top: var(--hi-px-16);
    padding: var(--hi-px-16);
    border-bottom-left-radius: var(--hi-tile--border-radius);
    border-bottom-right-radius: var(--hi-tile--border-radius);
  }

  .conjunction {
    padding-left: var(--hi-px-16);
    border-color: var(--hi-clr--theme-5);
  }

  .conjunction.is-joined::after {
    content: '';
    position: absolute;
    width: 0;
    height: var(--hi-px-24);
    bottom: calc(-1 * var(--hi-px-24));
    border-left: var(--hi-px-2) dotted var(--hi-clr--theme-5);
  }

  .pt_ {
    padding-top: var(--hi-px-12);
  }

  .pb_ {
    padding-bottom: var(--hi-px-12);
  }

  .mr_ {
    margin-right: var(--hi-px-8);
  }

  .mt_ {
    margin-top: var(--hi-px-12);
  }

  .mb_ {
    margin-bottom: var(--hi-px-12);
  }
`;
export class QueryElement extends BehaviorElement(AsLitHIC(LitElement)) {
  constructor() {
    super(...arguments);
    this.settingsDefaults = {
      allow: {},
      hidden: {},
      labels: {
        remove_operator: "Remove Operator",
        remove_operand: "Remove Operand",
        add_operator: "Add Operator",
        add_operand: "Add Operand",
        submit: "Query",
        reset: "Clear"
      }
    };
    this.form = `query-form-${f1.id()}`;
    this.settings = {};
    this.editFactory = new FactoryEditElement();
    this.fieldMutation = adt.mutation("fields");
  }
  static get is() {
    return "vi-query";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      input: {
        type: Object
      },
      type: {
        type: Object
      },
      settings: {
        type: Object
      }
    };
  }
  static get api() {
    return ["commit"];
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "group");
  }
  performUpdate() {
    this.settings = f1.merge("over", f1.merge("over", {}, this.settingsDefaults), f1.safe({}, this.settings));
    return super.performUpdate();
  }
  update(props) {
    super.update(props);
    if (props.has("type")) {
      this.sourceFields = this.__createSourceFields();
    }
    this.input = f1.safe({}, this.input);
  }
  __createSourceFields() {
    const rawSourceFields = f1.read("/fields/field/source/out", {}, this.type);
    const sourceFields = f1.is.array(rawSourceFields) ? rawSourceFields : f1.pipe(
      [
        (_) => this.fieldMutation.invoke({ fields: _ }),
        f1.to.array,
        f1.map((field) => ({
          comparator: f1.read("/filter_type", f1.read("/render_type", "", field), field),
          value: field.name,
          title: field.title,
          value_field: field
        }))
      ],
      rawSourceFields
    );
    return sourceFields;
  }
  render() {
    return html`
      <hi-form
        aesthetic="invisible"
        alignment="stacked"
        kind="submit"
        disable-http
        .form=${this.form}
        .ns=${this.ns}
        @reset-ended=${this.__form__onResetEnded}
        @submit-ended=${this.__form__onSubmitEnded}
        @submit-failed=${this.__form__onSubmitFailed}
      >
        ${this.__renderOperator(this.input, "")}
        <hi-button slot="controls" type="submit" .ns=${this.ns}>${this.settings.labels.submit}</hi-button>
        <hi-button aesthetic="secondary" slot="controls" type="reset" .ns=${this.ns}
          >${this.settings.labels.reset}</hi-button
        >
      </hi-form>
    `;
  }
  __renderOperator(operator, namePrefix, parentOperator) {
    const conditions = f1.read("/conditions", [], operator);
    const isOr = f1.read("/operator", "and", operator) === "or";
    const operatorKey = isOr ? "or" : "and";
    const isRoot = namePrefix === "";
    return html`
      <header class="mt_ mb_ pb_">
        <hi-toggle-group>
          <hi-label>AND</hi-label>
          <hi-toggle
            name="${namePrefix}_operator"
            model-path="${namePrefix}/operator"
            case-true="or"
            case-false="and"
            .value=${operatorKey}
            .form=${this.form}
            .ns=${this.ns}
            .on=${isOr}
            @commit-ended=${(e) => this.__operator__onCommitEnded(e, operator)}
          ></hi-toggle>
          <hi-label>OR</hi-label>
        </hi-toggle-group>
        <hi-button
          class="mr_"
          aesthetic="icon"
          ?hidden=${!parentOperator}
          @trigger-ended=${(e) => this.__removeConjunction__onTriggerEnded(e, operator, parentOperator)}
        >
          <hi-icon kind="remove_circle_outline"></hi-icon>
          <span>${this.settings.labels.remove_operator}</span>
        </hi-button>
      </header>
      <section class="join-${operatorKey} ${conditions.length <= 0 ? "hide-join" : ""}">
        <ul>
          ${conditions.map((_, index) => {
      const subKey = `${namePrefix}/conditions/${index}`;
      const conjunction = this.__isOperator(_) ? this.__renderOperator(_, subKey, operator) : this.__renderOperand(_, operator, subKey);
      return html`<li class="mb_">
              <hi-tile class="conjunction ${index + 1 < conditions.length ? "is-joined" : ""}">${conjunction}</hi-tile>
            </li>`;
    })}
        </ul>
      </section>
      <footer class="${isRoot ? "is-root" : ""}">
        <hi-button
          aesthetic="secondary"
          size="small"
          @trigger-ended=${(e) => this.__addOperand__onTriggerEnded(e, operator, namePrefix)}
        >
          <hi-icon kind="add_circle_outline"></hi-icon>
          <span>${this.settings.labels.add_operand}</span>
        </hi-button>
        <hi-button
          aesthetic="secondary"
          size="small"
          class="mr_"
          @trigger-ended=${(e) => this.__addOperator__onTriggerEnded(e, operator, namePrefix)}
        >
          <hi-icon kind="add_circle_outline"></hi-icon>
          <span>${this.settings.labels.add_operator}</span>
        </hi-button>
      </footer>
    `;
  }
  __renderOperand(operand, operator, namePrefix) {
    const ns = this.ns;
    const { sourceFields, comparators } = this.__expandOperand(operand);
    return html`
      <hi-fieldset alignment="inline" flush disallow-responsive>
        <hi-input-group>
          <hi-label>Field</hi-label>
          <hi-dropdown kind="value" cap-height exclude .ns=${ns}>
            <hi-select
              only-value
              allow-key-search
              mode="value"
              name="${namePrefix}_field"
              model-path="${namePrefix}/field"
              .value=${writeAlways(operand.field)}
              .form=${this.form}
              .ns=${ns}
              @commit-ended=${(e) => this.__fieldOperand__onCommitEnded(e, operand, operator, namePrefix)}
            >
              ${sourceFields.map((field) => html`<hi-option .value=${field.value}>${field.title}</hi-option>`)}
            </hi-select>
          </hi-dropdown>
        </hi-input-group>
        <hi-input-group>
          <hi-label>Condition</hi-label>
          <hi-dropdown kind="value" cap-height exclude .ns=${ns}>
            <hi-select
              only-value
              allow-key-search
              mode="value"
              name="${namePrefix}_comparator"
              model-path="${namePrefix}/comparator"
              .value=${writeAlways(operand.comparator)}
              .form=${this.form}
              .ns=${ns}
              @commit-ended=${(e) => this.__comparatorOperand__onCommitEnded(e, operand, operator, namePrefix)}
            >
              ${comparators.map(
      (comparator) => html`<hi-option .value=${comparator.id}>${comparator.title}</hi-option>`
    )}
            </hi-select>
          </hi-dropdown>
        </hi-input-group>
        <hi-input-group>
          <hi-label>Value</hi-label>
          ${this.__renderOperandValue(operand, operator, namePrefix)}
        </hi-input-group>
        <hi-button
          aesthetic="icon"
          @trigger-ended=${(e) => this.__removeConjunction__onTriggerEnded(e, operand, operator)}
        >
          <hi-icon kind="remove_circle_outline"></hi-icon>
          <span>${this.settings.labels.remove_operand}</span>
        </hi-button>
      </hi-fieldset>
    `;
  }
  __renderOperandValue(operand, operator, namePrefix) {
    const { sourceField } = this.__expandOperand(operand);
    const editFactory = this.editFactory;
    const renderType = f1.read("/field/render_type", "text", sourceField);
    const editType = editFactory && editFactory.hasRender("edit", renderType) ? renderType : "unknown";
    const editFn = editFactory.renderBy("edit", editType).render(
      f1.merge("over", sourceField.value_field, {
        name: `${namePrefix}_value`,
        model_path: `${namePrefix}/value`,
        attributes: {
          ns: this.ns,
          form: this.form
        },
        value: {
          out: operand.value
        }
      }),
      editFactory
    );
    return editFn(operand.value, operand);
  }
  commit(value = this.input) {
    return {
      value
    };
  }
  __form__onSubmitEnded(evt) {
    this.input = evt.detail.value;
    this.commit();
  }
  __form__onResetEnded(evt) {
    this.input = {};
    this.commit();
  }
  __form__onSubmitFailed(evt) {
    this._dispatchFailure("commit", evt.detail.value);
  }
  __fieldOperand__onCommitEnded(evt, operand, operator, namePrefix) {
    evt.stopPropagation();
    operand.field = evt.detail.value;
    const { comparators } = this.__expandOperand(operand);
    operand.comparator = f1.read("/0/value", "", comparators);
    operand.value = void 0;
    this.requestUpdate("input");
  }
  __comparatorOperand__onCommitEnded(evt, operand, operator, namePrefix) {
    evt.stopPropagation();
    operand.comparator = evt.detail.value;
    operand.value = void 0;
    this.requestUpdate("input");
  }
  __operator__onCommitEnded(evt, operator) {
    evt.stopPropagation();
    operator.operator = evt.detail.value.value;
    this.requestUpdate("input");
  }
  __addOperand__onTriggerEnded(evt, operator, namePrefix) {
    f1.write(
      "/conditions",
      f1.push(
        {
        },
        f1.read("/conditions", [], operator)
      ),
      operator
    );
    this.requestUpdate("input");
  }
  __addOperator__onTriggerEnded(e, operator, namePrefix) {
    f1.write(
      "/conditions",
      f1.push(
        {
          operator: "and",
          conditions: []
        },
        f1.read("/conditions", [], operator)
      ),
      operator
    );
    this.requestUpdate("input");
  }
  __removeConjunction__onTriggerEnded(e, conjunction, parentConjunction) {
    if (!parentConjunction) {
      return;
    }
    f1.write(
      "/conditions",
      f1.remove((_) => _ === conjunction, f1.read("/conditions", [], parentConjunction)),
      parentConjunction
    );
    this.requestUpdate("input");
  }
  __expandOperand(operand) {
    const sourceFields = this.sourceFields;
    const sourceField = f1.safe(
      {},
      f1.find((fieldSource) => fieldSource.value === operand.field, sourceFields)
    );
    const comparatorsDB = f1.read("/comparators", void 0, this.type);
    const comparators = !f1.is.nil(comparatorsDB) ? f1.safe([], f1.key(sourceField.comparator, comparatorsDB)) : adt.filterComparators(sourceField.comparator);
    return {
      sourceFields,
      sourceField,
      comparators
    };
  }
  __isOperator(rule) {
    return f1.is.in("operator", rule);
  }
}
export default HIP.factory.element(QueryElement);
