
import { html } from '../../../hip/library/lit-html/lit-html.js';
import { ifDefined as ifDef } from '../../../hip/library/lit-html/directives/if-defined.js';
import { setAttributes } from '../../../eip/library/hi-lit/hi-lit.js';
import { styleMap } from '../../../hip/library/lit-html/directives/style-map.js';
import $ip from '../../../pip/library/pi-ip/pi-ip.js';
$ip.di.register("narrative", "grid", {
  id: `factory-narrative-grid`,
  render(narrative, narrativeEl) {
    return (input) => {
      const { id, attributes, style, type, settings } = narrativeEl.createFields(narrative, "grid");
      return html`
        <vi-grid
          id=${id}
          gid=${id}
          .ns=${narrativeEl.ns}
          .input=${input}
          .type=${type}
          .settings=${settings}
          fit-height
          data-attrs=${setAttributes(attributes)}
          @row-action-ended=${(e) => narrativeEl.narrative__onEventFired(e, narrative)}
          style=${styleMap(style)}
        >
        </vi-grid>
      `;
    };
  }
});
$ip.di.register("narrative", "list", {
  id: `factory-narrative-list`,
  render(narrative, narrativeEl) {
    return (input) => {
      const { id, attributes, style, type, settings } = narrativeEl.createFields(narrative, "list");
      return html`
        <vi-list
          id=${id}
          gid=${id}
          mode="tile"
          .ns=${narrativeEl.ns}
          .input=${input}
          .type=${type}
          .settings=${settings}
          fit-height
          data-attrs=${setAttributes(attributes)}
          @row-action-ended=${(e) => narrativeEl.narrative__onEventFired(e, narrative)}
          style=${styleMap(style)}
        >
        </vi-list>
      `;
    };
  }
});
$ip.di.register("narrative", "query", {
  id: `factory-narrative-query`,
  safeDefaults(narrative) {
    return narrative;
  },
  render(narrative, narrativeEl) {
    return (input) => {
      const { id, attributes, style, type } = narrativeEl.createFields(narrative, "query");
      style["margin"] = "0.5rem";
      return html`
        <vi-query
          id=${id}
          gid=${id}
          .ns=${narrativeEl.ns}
          .input=${input}
          .type=${type}
          data-attrs=${setAttributes(attributes)}
          style=${styleMap(style)}
          @commit-ended=${(e) => narrativeEl.narrative__onEventFired(e, narrative)}
        >
        </vi-query>
      `;
    };
  }
});
$ip.di.register("narrative", "info", {
  id: `factory-narrative-info`,
  render(narrative, narrativeEl) {
    return (input) => {
      const { id, attributes, style, type } = narrativeEl.createFields(narrative, "info");
      return html`
        <vi-info
          id=${id}
          gid=${id}
          .ns=${narrativeEl.ns}
          .type=${ifDef(type)}
          .input=${input}
          data-attrs=${setAttributes(attributes)}
          style=${styleMap(style)}
        >
        </vi-info>
      `;
    };
  }
});
$ip.di.register("narrative", "pivot", {
  id: `factory-narrative-pivot`,
  render(narrative, narrativeEl) {
    return (input) => {
      const { id, attributes, style } = narrativeEl.createFields(narrative, "pivot");
      return html` <div id="pivotContainer" style="margin: 0.5em">
        <eab-hip-pivot-table
          id="pivotTable"
          kind="viewer"
          id=${id}
          gid=${id}
          .ns=${narrativeEl.ns}
          data-attrs=${setAttributes(attributes)}
          style=${styleMap(style)}
        >
        </eab-hip-pivot-table>
      </div>`;
    };
  }
});
[
  ["chart", "bar"],
  ["chart", "stacked"],
  ["chart", "pie"],
  ["chart", "doughnut"],
  ["chart", "area"],
  ["chart", "line"],
  ["chart", "scatter"],
  ["chart", "funnel"],
  ["chart", "bubble"],
  ["chart", "min-bar"],
  ["chart", "min-line"],
  ["chart", "min-area"],
  ["chart", "min-doughnut"],
  ["chart", "min-pie"],
  ["chart", "matrix"],
  ["chart_extended", "sankey"]
].forEach(([library, render_type]) => {
  $ip.di.register("narrative", render_type, {
    id: `factory-narrative-${render_type}`,
    render(narrative, narrativeEl) {
      return (input) => {
        const { id, attributes, style, type } = narrativeEl.createFields(narrative, render_type);
        style["height"] = "100%";
        style["margin"] = "auto";
        style["width"] = "100%";
        return html`
          <vi-chart
            mode="hrf4"
            auto-size
            id=${id}
            gid=${id}
            kind=${render_type}
            library=${library}
            data-attrs=${setAttributes(attributes)}
            style=${styleMap(style)}
            .ns=${narrativeEl.ns}
            .type=${type}
            .input=${input}
            @trigger-ended=${(e) => narrativeEl.narrative__onEventFired(e, narrative)}
          >
          </vi-chart>
        `;
      };
    }
  });
});
