
import { html, LitElement } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import { AsLitHIC, BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import adt from '../../../pip/library/pi-adt/pi-adt.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class TableElement extends BehaviorElement(AsLitHIC(LitElement)) {
  constructor() {
    super(...arguments);
    this._id = f1.uuid();
    this.caption = "";
    this.allowAction = false;
    this.actionText = "Action";
    this.mode = "hrf";
    this.fieldMutation = adt.mutation("fields");
  }
  static get is() {
    return "vi-table";
  }
  createRenderRoot() {
    return this;
  }
  static get properties() {
    return {
      type: {
        type: Object,
        reflect: true
      },
      input: {
        type: Object,
        reflect: true
      },
      caption: {
        type: String,
        reflect: true
      },
      allowAction: {
        type: Boolean,
        reflect: true,
        attribute: "allow-action"
      },
      actionText: {
        type: String,
        reflect: true,
        attribute: "action-text"
      },
      mode: {
        type: String,
        reflect: true
      }
    };
  }
  render() {
    const input = this.input || {};
    const type = f1.safe({}, this.type);
    const typeFields = this.fieldMutation.invoke(type);
    const fields = this.mode === "hil" ? f1.to.array(
      f1.map((field) => {
        return {
          name: field.name,
          title: field.title,
          axis_type: field.axis_type
        };
      }, typeFields)
    ) : input.fields || [];
    const data = this.mode === "hil" ? input : input.data || [];
    const axisTitles = fields.map((o) => o.name);
    return html`
      <table id="${this._id}_table" kind="normal">
        <thead>
          <tr>
            ${axisTitles.map((header) => html` <th scope="col">${header}</th> `)}
            ${this.allowAction ? html`<th scope="col">Action</th>` : nothing}
          </tr>
        </thead>
        <tbody>
          ${data.map(
      (data2, index) => html`
              <tr index=${index}>
                ${axisTitles.map(
        (fieldName, index2) => html` <td index=${index2}>${data2[fieldName]}</td>`
      )}
                ${this.allowAction ? html` <td>
                      <hi-button
                        id="${index}_actionTriggerer"
                        aria-label="${this.actionText} button for extra information"
                        aesthetic="secondary"
                        index="${index}"
                        size="small"
                        .value=${this.input}
                        @trigger-ended="${(e) => this.triggerAction(e, data2)}"
                        >${this.actionText}
                      </hi-button>
                    </td>` : nothing}
              </tr>
            `
    )}
        </tbody>
        <caption>
          ${this.caption}
        </caption>
      </table>
    `;
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "group");
  }
  triggerAction(e, data) {
    this.dispatchEvent(
      new CustomEvent("action-ended", {
        detail: {
          data,
          type: this.type,
          value: this.input,
          srcElement: e.detail.srcElement,
          srcElementDetails: e.detail
        }
      })
    );
  }
}
HIP.factory.element(TableElement);
export default TableElement.is;
