
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __decorateClass = (decorators, target, key, kind) => {
  var result = kind > 1 ? void 0 : kind ? __getOwnPropDesc(target, key) : target;
  for (var i = decorators.length - 1, decorator; i >= 0; i--)
    if (decorator = decorators[i])
      result = (kind ? decorator(target, key, result) : decorator(result)) || result;
  if (kind && result)
    __defProp(target, key, result);
  return result;
};
import { html, css, LitElement } from '../../../hip/library/lit-element/lit-element.js';
import { query } from '../../../hip/library/lit-element/decorators.js';
import { ifDefined as ifDef } from '../../../hip/library/lit-html/directives/if-defined.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { AsLitHIC, BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import { HIP } from '../../../eip/element/hi-component/hi-component.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
import ess from '../../../pip/library/pi-ess/pi-ess.js';
export const style = css`
  :host {
    display: block;
    width: 100%;
    height: 100%;
  }

  .controls-container {
    display: flex;
    justify-content: space-between;
    align-items: center;
    margin: var(--hi-px-12) var(--hi-px-8);
  }

  .tree-container {
    margin: var(--hi-px-12) var(--hi-px-8);
  }
`;
export class AdvancedTreeElement extends BehaviorElement(AsLitHIC(LitElement)) {
  constructor() {
    super();
    this.ns = `vi-tree-${f1.id()}`;
  }
  static get is() {
    return "vi-tree";
  }
  static get properties() {
    return {
      input: {
        type: Array
      }
    };
  }
  static get api() {
    return ["expandAll", "collapseAll", "expandImmediate", "collapseImmediate", "expandChildren", "collapseChildren"];
  }
  connectedCallback() {
    super.connectedCallback();
    this.addEventListener("selected-ended", this._handleSelectedEnded);
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    this.removeEventListener("selected-ended", this._handleSelectedEnded);
  }
  render() {
    return html`
      <div class="controls-container">
        <hi-button id="expandAll-${this.ns}" @trigger-ended="${this.expandAll}" aesthetic="ghost" size="small" bordered>
          <span> Expand All </span>
          <hi-icon glyph="unfold_more" type="material" size="small"></hi-icon>
        </hi-button>

        <hi-button
          id="collapseAll-${this.ns}"
          @trigger-ended="${this.collapseAll}"
          aesthetic="ghost"
          size="small"
          bordered
        >
          <span> Collapse All </span>
          <hi-icon glyph="unfold_less" type="material" size="small"></hi-icon>
        </hi-button>
      </div>
      <hi-tree class="tree-container"> ${this.generateTree(this.input)}</hi-tree>
    `;
  }
  generateTree(treeData) {
    return treeData.map(
      (item) => item.children ? html` <hi-sub-tree
            heading="${item.heading}"
            node-icon="${ifDef(item.nodeIcon)}"
            theme="${ifDef(item.nodeTheme)}"
          >
            ${this.generateTree(item.children)}
          </hi-sub-tree>` : html` <hi-leaf
            value="${item["value"]}"
            leaf-icon="${ifDef(item["leafIcon"])}"
            theme="${ifDef(item["leafTheme"])}"
          >
            ${item["label"]}
          </hi-leaf>`
    );
  }
  expandAll() {
    if (this.tree) {
      this.tree.expand();
    }
  }
  collapseAll() {
    if (this.tree) {
      this.tree.collapse();
    }
  }
  expandImmediate() {
    if (this.tree) {
      this.tree.expandImmediate();
    }
  }
  collapseImmediate() {
    if (this.tree) {
      this.tree.collapseImmediate();
    }
  }
  expandChildren() {
    if (this.tree) {
      this.tree.expandChildren();
    }
  }
  collapseChildren() {
    if (this.tree) {
      this.tree.collapseChildren();
    }
  }
  _handleSelectedEnded(event) {
    const { id, item, value } = event.detail ?? {};
    if (!id || !item || !value) {
      ess.ele(this, {
        status: "failure",
        action: "read",
        reason: "Unexpected event.detail format",
        data: event.detail
      });
      return;
    }
    this.dispatchEvent(
      new CustomEvent("tree-selected-ended", {
        detail: { id, item, srcElement: this, value },
        bubbles: true,
        composed: true
      })
    );
  }
}
AdvancedTreeElement.styles = [stylesTheme, style];
__decorateClass([
  query("hi-tree")
], AdvancedTreeElement.prototype, "tree", 2);
export default HIP.factory.element(AdvancedTreeElement);
