
import { PolymerElement, html } from '../../../hip/library/polymer/polymer.js';
import { animationFrame } from '../../../hip/library/polymer/polymer.js';
import { Debouncer } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, HIP, BehaviorElement } from '../../../eip/element/hi-hip/hi-hif.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class A11yAnnouncerElement extends BehaviorElement(AsHIC(PolymerElement)) {
  static get is() {
    return "xi-a11y-announcer";
  }
  static get template() {
    return html`
      <style>
        :host {
          display: block;
          position: static;
          clip: rect(0, 0, 0, 0);
          height: 1px;
          width: 1px;
          overflow: hidden;
          white-space: nowrap;
        }
      </style>
      <span>[[ text ]]</span>
    `;
  }
  static get properties() {
    return {
      kind: {
        type: String,
        value: "polite",
        observer: "__kindChanged"
      },
      text: {
        type: String,
        notify: true
      },
      speechDelay: {
        type: Number,
        value: 32
      }
    };
  }
  static get autoBinding() {
    return {
      signal: "text-changed",
      field: "announce"
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "status");
    this.setAttribute("aria-atomic", "true");
    this.__onHashChange__windowBound = this.__onHashChange__window.bind(this);
    window.addEventListener("hashchange", this.__onHashChange__windowBound, false);
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    window.addEventListener("hashchange", this.__onHashChange__windowBound, false);
  }
  __onHashChange__window() {
    this.__clearAnnouncements();
  }
  __clearAnnouncements() {
    this.text = "";
  }
  __kindChanged(_kind) {
    this.setAttribute("aria-live", _kind);
  }
  announce(text, politeness = "polite") {
    this.kind = politeness;
    this.__announceDebouncer = Debouncer.debounce(this.__announceDebouncer, animationFrame, () => {
      this.__clearAnnouncements();
      setTimeout(() => {
        this.text = text;
      }, this.speechDelay);
    });
  }
  static get instance() {
    return HIP.fn.instance(A11yAnnouncerElement);
  }
}
HIP.fn.announce = function(text, politeness = "polite") {
  return A11yAnnouncerElement.instance.announce(text, f1.is.string(politeness) ? politeness : "polite");
};
HIP.factory.element(A11yAnnouncerElement);
