
import { LitElement, html, css } from '../../../hip/library/lit-element/lit-element.js';
import { TextSpacingStyle, checkBoxStyles } from './xi-a11y-buddy-styles.js';
import { AsLitHIC, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class DOMServant {
  static dispatchMessage(el, message, detail = {}) {
    el.dispatchEvent(
      new CustomEvent(message, {
        detail,
        bubbles: false,
        composed: true
      })
    );
  }
}
export class A11yBuddyContrast extends AsLitHIC(LitElement) {
  constructor() {
    super(...arguments);
    this.isContrastChecked = false;
  }
  static get is() {
    return "a11y-buddy-contrast";
  }
  static get properties() {
    return {
      isContrastChecked: {
        type: Boolean
      }
    };
  }
  static get styles() {
    return [checkBoxStyles];
  }
  firstUpdated() {
    this.__checkLocalStorage();
  }
  __checkLocalStorage() {
    const userSettings = JSON.parse(localStorage.getItem("a11y-buddy-contrast"));
    if (f1.is.nil(userSettings)) {
      return;
    }
    this.isContrastChecked = userSettings;
    this.__setContrast();
  }
  __checkboxOnClick(event) {
    this.isContrastChecked = event.target.checked;
    this.__setContrast();
    this.__setLocalStorage();
  }
  __setContrast() {
    document.body.style.filter = this.isContrastChecked ? "contrast(100%) grayScale(100%)" : "none";
  }
  __setLocalStorage() {
    localStorage.setItem("a11y-buddy-contrast", JSON.stringify(this.isContrastChecked));
  }
  render() {
    return html`
      <div class="checkbox-wrapper">
        <input
          type="checkbox"
          id="contrast"
          name="contrast"
          ?checked="${this.isContrastChecked}"
          @click="${this.__checkboxOnClick}"
        />
        <label for="contrast">Display Monochrome</label>
      </div>
    `;
  }
}
HIP.factory.element(A11yBuddyContrast);
export class A11yBuddySkipToMain extends AsLitHIC(LitElement) {
  static get is() {
    return "a11y-buddy-skip-to-main";
  }
  static get styles() {
    return css`
      :host {
        position: relative;
      }

      a {
        color: var(--a11y-color–-foreground);
        padding: var(--a11y-sz-4);
      }

      a:hover {
        background: var(--a11y-clr--gray-medium);
      }
    `;
  }
  __skipToMainOnClick(event) {
    event.preventDefault();
    dom.focusOnMain();
    DOMServant.dispatchMessage(this, "triggered");
  }
  render() {
    return html`
      <a id="skipToMain" data-widget="control" class="nudge-right" href="#" @click="${this.__skipToMainOnClick}">
        Skip To Main
      </a>
    `;
  }
}
HIP.factory.element(A11yBuddySkipToMain);
export class A11yBuddyTextSize extends AsLitHIC(LitElement) {
  constructor() {
    super(...arguments);
    this.bodyFontSize = 16;
    this.fontSizeScale = 1;
  }
  static get is() {
    return "a11y-buddy-text-size";
  }
  static get properties() {
    return {
      fontSizeScale: {
        type: Number
      }
    };
  }
  static get styles() {
    return css`
      :host {
        position: relative;
      }
      .text-spacing-group {
        display: flex;
        align-items: center;
      }

      .text-spacing-group button {
        box-sizing: border-box;
        padding: var(--a11y-sz-4) var(--a11y-sz-8);
        margin: var(--a11y-sz-8) 0;
        border-radius: var(--a11y-sz-0);
        border-width: var(--a11y-sz-1);
        border-color: var(--a11y-clr--black);
        font-size: inherit;
        background-color: var(--a11y-clr--white);
        color: var(--a11y-clr--black);
      }

      .text-spacing-group button:hover {
        background-color: var(--a11y-clr--gray-medium);
      }

      .text-spacing-group button:first-of-type {
        margin-right: var(--a11y-sz-4);
      }

      #textSizeLabel {
        margin-right: var(--a11y-sz-8);
      }

      [mixin='show-ax'] {
        position: absolute;
        z-index: -1000;
        left: -100000px;
        top: -100000px;
        height: 1px;
        width: 1px;
        clip: rect(1px, 1px, 1px, 1px);
        clip-path: polygon(0px 0px, 0px 0px, 0px 0px);
        overflow: hidden !important;
        padding: 0;
        border: 0;
        margin: 0;
      }
    `;
  }
  connectedCallback() {
    super.connectedCallback();
    this.bodyFontSize = parseInt(getComputedStyle(document.body).fontSize, 10);
  }
  firstUpdated() {
    this.__checkLocalStorage();
  }
  __checkLocalStorage() {
    const userSettings = JSON.parse(localStorage.getItem("a11y-buddy-font-size"));
    if (f1.is.nil(userSettings)) {
      return;
    }
    this.fontSizeScale = userSettings;
    this.__updateFontSize();
  }
  __changeFontSize(change) {
    return () => {
      this.fontSizeScale = Math.min(Math.max(this.fontSizeScale + change, 1), 2);
      this.__updateFontSize();
    };
  }
  __updateFontSize() {
    const scaledFontSizeInPixels = `${this.fontSizeScale * this.bodyFontSize}px`;
    document.documentElement.style.fontSize = scaledFontSizeInPixels;
    this.__setLocalStorage();
  }
  __setLocalStorage() {
    localStorage.setItem("a11y-buddy-font-size", JSON.stringify(this.fontSizeScale));
  }
  render() {
    return html`
      <div class="text-spacing-group" role="group">
        <p id="textSizeLabel">Text Size</p>
        <button name="decrease" id="decreaseText" @click="${this.__changeFontSize(-0.25)}">
          <span class="text-indicator" aria-hidden="true">▼</span>
          <span mixin="show-ax">Decrease text size by 25 percent</span>
        </button>
        <button name="increase" id="increaseText" @click="${this.__changeFontSize(0.25)}">
          <span class="text-indicator" aria-hidden="true">▲</span>
          <span mixin="show-ax">Increase text size by 25 percent</span>
        </button>
        <span mixin="show-ax" aria-live="polite" aria-atomic="false"
          >Text Size is ${this.fontSizeScale * 100} percent</span
        >
      </div>
    `;
  }
}
HIP.factory.element(A11yBuddyTextSize);
export class A11yBuddyTextSpacing extends AsLitHIC(LitElement) {
  static get is() {
    return "a11y-buddy-text-spacing";
  }
  static get properties() {
    return {
      isTextSpacingChecked: {
        type: Boolean
      }
    };
  }
  static get styles() {
    return [checkBoxStyles];
  }
  connectedCallback() {
    super.connectedCallback();
    const WCAG_1_4_12_CSS = document.createElement("style");
    WCAG_1_4_12_CSS.innerHTML = TextSpacingStyle;
    document.body.appendChild(WCAG_1_4_12_CSS);
  }
  firstUpdated() {
    this.__checkLocalStorage();
  }
  __checkLocalStorage() {
    const userSettings = JSON.parse(localStorage.getItem("a11y-buddy-text-spacing"));
    if (f1.is.nil(userSettings)) {
      return;
    }
    if (userSettings) {
      this.__toggleOnInput();
    } else {
      this.__toggleOffInput();
    }
  }
  __toggleOnInput() {
    this.isTextSpacingChecked = true;
    this.__enableWcag();
  }
  __toggleOffInput() {
    this.isTextSpacingChecked = false;
    this.__removeWcag();
  }
  __toggleTextSpacing(event) {
    this.isTextSpacingChecked = event.target.checked;
    if (this.isTextSpacingChecked) {
      this.__enableWcag();
      this.__setLocalStorage();
    } else {
      this.__removeWcag();
      this.__setLocalStorage();
    }
  }
  __removeWcag() {
    document.body.removeAttribute("x-wcag");
  }
  __enableWcag() {
    document.body.setAttribute("x-wcag", "2.1__1.4.12");
  }
  __setLocalStorage() {
    localStorage.setItem("a11y-buddy-text-spacing", JSON.stringify(this.isTextSpacingChecked));
  }
  render() {
    return html`
      <div class="checkbox-wrapper">
        <input
          type="checkbox"
          id="textSpacing"
          name="textSpacing"
          ?checked="${this.isTextSpacingChecked}"
          @click="${this.__toggleTextSpacing}"
        />
        <label for="textSpacing">Easier Reading</label>
      </div>
    `;
  }
}
HIP.factory.element(A11yBuddyTextSpacing);
