
import { LitElement, html, css } from '../../../hip/library/lit-element/lit-element.js';
import { AsLitHIC, HIP } from '../../../eip/element/hi-hip/hi-hif.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import './xi-a11y-buddy-components.js';
import './xi-a11y-buddy-styles.js';
export class XiA11yBuddy extends AsLitHIC(LitElement) {
  constructor() {
    super(...arguments);
    this.opened = false;
    this.displayModel = "inline";
    this.__windowOnClickBound = this.__windowOnClick.bind(this);
    this.__onFocusOutBound = this.__onFocusOut.bind(this);
    this.__onFocusInBound = this.__onFocusIn.bind(this);
  }
  static get is() {
    return "xi-a11y-buddy";
  }
  static get properties() {
    return {
      opened: {
        type: Boolean,
        reflect: true
      },
      displayModel: { type: String, reflect: true, attribute: "display-model" }
    };
  }
  static get styles() {
    return css`
      :host {
        /**
          * Naming convention for css properties
          * --a11y-<css property name in Kebab case>
         */
        --a11y-clr–-background: #fafafa;
        --a11y-clr–-foreground: #161616;
        --a11y-clr--gray-light: #fafafa;
        --a11y-clr--gray-medium: #e0e0e0;
        --a11y-clr--gray-dark: #333d47;
        --a11y-clr--white: #ffffff;
        --a11y-clr--black: #000000;

        --a11y-sz-0: 0rem;
        --a11y-sz-1: 0.07rem;
        --a11y-sz-2: 0.125rem;
        --a11y-sz-4: 0.25rem;
        --a11y-sz-8: 0.5rem;
        --a11y-sz-12: 0.75rem;
        --a11y-sz-16: 1rem;
        --a11y-sz-20: 1.25rem;

        --a11y-font-lightest: 100;
        --a11y-font-light: 200;
        --a11y-font-heavy: 400;
        --a11y-font-family: Tahoma, Helvetica, sans-serif;

        --a11y-border: var(--a11y-sz-1) solid rgba(0, 0, 0, 0.3);
        --a11y-border-radius: var(--a11y-sz-2);
        --a11y-box-shadow: var(--a11y-sz-0) var(--a11y-sz-2) var(--a11y-sz-0) var(--a11y-sz-2) rgba(0, 0, 0, 0.04);
        --a11y-size--inner-padding: var(--a11y-sz-4);
        --a11y-size--margin: 10px 0;
        --a11y-z-index: 1000;
      }

      :host {
        display: flex;
        align-items: center;
        justify-content: start;
        flex-wrap: wrap;
        width: 100%;
        padding: var(--a11y-sz-4) var(--a11y-sz-12);
        min-height: calc(var(--a11y-sz-20) * 2);
        background-color: var(--a11y-clr–-background);
        color: var(--a11y-clr–-foreground);

        font-family: var(--a11y-font-family);
        font-size: inherit;
        font-weight: var(--a11y-font-heavy);
        line-height: 1.5;
        z-index: var(--a11y-z-index);

        position: absolute;
        top: -10000px;
        margin-top: 0;
      }

      :host(:not([opened])) {
        opacity: 0;
      }

      :host([display-model='fixed']) {
        position: fixed;
        left: 0;
        right: 0;
        z-index: var(--a11y-z-index);
      }

      :host > * {
        margin-right: var(--a11y-sz-8);
      }

      a11y-buddy-text-spacing,
      a11y-buddy-text-size {
        margin-left: calc(2 * var(--a11y-sz-8));
      }

      @media screen and (prefers-reduced-motion) {
        :host {
          transition: none;
        }
      }
    `;
  }
  render() {
    return html`
      <a11y-buddy-skip-to-main id="skipToMain" data-widget="control" @triggered=${this.__closeAllyBuddyOnTriggered}>
      </a11y-buddy-skip-to-main>
      <a11y-buddy-text-size id="fontSpacing" data-widget="control"> </a11y-buddy-text-size>
      <a11y-buddy-text-spacing id="textSpacing" data-widget="control"> </a11y-buddy-text-spacing>
      <a11y-buddy-contrast id="contrast" data-widget="control"> </a11y-buddy-contrast>
    `;
  }
  connectedCallback() {
    super.connectedCallback();
    window.addEventListener("click", this.__windowOnClickBound);
    this.addEventListener("focusout", this.__onFocusOutBound);
    this.addEventListener("focusin", this.__onFocusInBound);
    this.addEventListener("keyup", this.__escKeyListener);
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    window.removeEventListener("click", this.__windowOnClickBound);
    this.removeEventListener("focusout", this.__onFocusOutBound);
    this.removeEventListener("focusin", this.__onFocusInBound);
  }
  firstUpdated() {
    this.__reflectOpened();
  }
  updated(props) {
    if (props.has("opened")) {
      this.__reflectOpened();
    }
  }
  shouldUpdate(changedProperties) {
    return changedProperties.has("opened");
  }
  __escKeyListener(event) {
    if (this.opened && event.code === "Escape") {
      dom.focusOnMain();
      this.hide();
    }
  }
  __closeAllyBuddyOnTriggered() {
    this.hide();
  }
  __windowOnClick(event) {
    if (!this.opened) {
      return;
    }
    const targetEl = event?.target;
    const isNotInside = targetEl && targetEl.localName !== this.localName;
    const isNotChild = !this.__isEventFromWithin(event);
    if (isNotInside && isNotChild) {
      this.hide();
    }
  }
  __onFocusIn() {
    this.show();
  }
  __onFocusOut(event) {
    const targetEl = event?.relatedTarget;
    if (!targetEl) {
      return;
    }
    const isInsideElement = this.shadowRoot?.contains(targetEl);
    const isChild = this.__isEventFromWithin(event);
    if (!isInsideElement || !isChild) {
      this.hide();
    }
  }
  __isEventFromWithin(event) {
    let paths = [];
    if (event.composed) {
      paths = event.composedPath();
    } else if (event.path) {
      paths = event.path;
    } else {
      paths = [];
    }
    return paths.includes(this);
  }
  toggle() {
    if (!this.opened) {
      this.show();
    } else {
      this.hide();
    }
  }
  show() {
    this.opened = true;
  }
  hide() {
    this.opened = false;
  }
  __reflectOpened() {
    if (!this.opened) {
      this.style.position = "absolute";
      this.style.top = "-10000px";
      this.style.marginTop = "0";
    } else {
      this.style.position = this.displayModel === "fixed" ? "fixed" : "static";
      this.style.marginTop = "0";
      this.style.top = "0";
    }
  }
}
HIP.factory.element(XiA11yBuddy);
