
import { PolymerElement } from '../../../hip/library/polymer/polymer.js';
import { AsHIC, HIP, BehaviorElement } from '../../../eip/element/hi-hip/hi-hif.js';
import '../../../xip/element/xi-a11y-announcer/xi-a11y-announcer.js';
export class TitleElement extends BehaviorElement(AsHIC(PolymerElement)) {
  static get is() {
    return "xi-title";
  }
  static get properties() {
    return {
      productTitle: {
        type: String,
        reflectToAttribute: true
      },
      pageTitle: {
        type: String,
        reflectToAttribute: true
      }
    };
  }
  static get observers() {
    return ["__titleWatcher(pageTitle, productTitle)"];
  }
  static __titleWatcher(pageTitle, productTitle) {
    TitleElement.setTitle(pageTitle, productTitle);
  }
  setTitle(pageTitle, productTitle) {
    TitleElement.setTitle(pageTitle, productTitle);
  }
  static setTitle(pageTitle, productTitle) {
    const siteTitle = this.__siteTitle(pageTitle, productTitle);
    const siteTitleAnnouncement = this.__siteTitleAnnouncement(siteTitle);
    const titleEl = this.__guaranteeTitleEl();
    HIP.fn.announce(siteTitleAnnouncement);
    if (titleEl) {
      titleEl.textContent = siteTitle;
    }
  }
  static __guaranteeTitleEl() {
    let titleEl = document.getElementsByTagName("title").item(0);
    const headEl = document.getElementsByTagName("head").item(0);
    if (!titleEl && headEl) {
      titleEl = document.createElement("title");
      headEl.appendChild(titleEl);
    }
    return titleEl;
  }
  static __siteTitleAnnouncement(siteTitle) {
    return `Viewing screen ${siteTitle}`;
  }
  static __siteTitle(pageTitle, productTitle) {
    return [productTitle, pageTitle].filter((_) => _).join(" | ");
  }
}
export default HIP.factory.element(TitleElement);
