
import { LitElement } from '../../../hip/library/lit-element/lit-element.js';
import { HIP, AsHIC, BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import dom from '../../../pip/library/pi-dom/pi-dom.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class LayoutScreenQueryElement extends BehaviorElement(AsHIC(LitElement)) {
  constructor() {
    super(...arguments);
    this.__observers =  new Map();
    this.mediaQueries = [];
  }
  static get is() {
    return "yi-layout-screen-query";
  }
  static get properties() {
    return {
      mediaQueries: {
        type: Array,
        attribute: "media-queries"
      },
      __matchMediaList: {
        type: Array,
        attribute: false
      }
    };
  }
  static get api() {
    return ["invoke"];
  }
  static get instance() {
    return HIP.fn.instance(LayoutScreenQueryElement);
  }
  connectedCallback() {
    super.connectedCallback();
    this._initializeBreakpoints();
    this._initializeMediaQuery();
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    this.__removeEventListeners();
  }
  updated(props) {
    super.updated(props);
    if (props.has("mediaQueries")) {
      this._initializeMediaQuery();
    }
  }
  isDesktop() {
    return "mobile" !== this.screenDevice;
  }
  isMobile() {
    return "mobile" === this.screenDevice;
  }
  _initializeBreakpoints() {
    const style = getComputedStyle(document.documentElement);
    this.__mobileBreakpoint = style.getPropertyValue("--hi-media--mobile").trim();
    this.__tabletBreakpoint = style.getPropertyValue("--hi-media--tablet").trim();
    this.__desktopBreakpoint = style.getPropertyValue("--hi-media--desktop").trim();
  }
  _initializeMediaQuery() {
    if (this.__matchMediaList) {
      this.__removeEventListeners();
    }
    if (this.mediaQueries.length === 0) {
      this.mediaQueries = this.__constructMediaQueryList();
    }
    this.__matchMediaList = this.__constructMatchMediaList();
    this.__getInitialScreenQuery();
    this.__addEventListeners();
  }
  subscribe(ob) {
    this.__observers.set(ob, ob);
    this._invokeObserver(ob, this.screenDevice);
  }
  unsubscribe(ob) {
    this.__observers.delete(ob);
  }
  invoke(screenDevice) {
    this.screenDevice = screenDevice;
    dom.write(() => {
      dom.setAttr(document.body, "screen-device", screenDevice);
      this.__observers.forEach((ob) => {
        this._invokeObserver(ob, screenDevice);
      });
    });
    return screenDevice;
  }
  _invokeObserver(ob, screenDevice) {
    if (f1.is.function(ob.setAttribute)) {
      ob.setAttribute("screen-device", screenDevice);
    }
    if (f1.is.function(ob.mediaDeviceHook)) {
      ob.mediaDeviceHook(screenDevice);
    }
  }
  __constructMediaQueryList() {
    return [
      `(max-width: ${this.__mobileBreakpoint})`,
      `(min-width: ${this.__mobileBreakpoint}) and (max-width: ${this.__tabletBreakpoint})`,
      `(min-width: ${this.__tabletBreakpoint})`
    ];
  }
  __constructMatchMediaList() {
    return this.mediaQueries.map((mediaQuery) => {
      return window.matchMedia(mediaQuery);
    });
  }
  __getInitialScreenQuery() {
    this.__matchMediaList.forEach((matchedMediaQuery) => {
      this.__invokeScreenQuery(matchedMediaQuery);
    });
  }
  __addEventListeners() {
    this.__matchMediaList.forEach((matchedMediaQuery) => {
      matchedMediaQuery.addEventListener("change", this.__onMatchMediaQueryChange);
    });
  }
  __removeEventListeners() {
    this.__matchMediaList.forEach((matchedMediaQuery) => {
      matchedMediaQuery.removeEventListener("change", this.__onMatchMediaQueryChange);
    });
  }
  __onMatchMediaQueryChange(event) {
    LayoutScreenQueryElement.instance.__invokeScreenQuery(event);
  }
  __invokeScreenQuery(matchedMediaQuery) {
    if (matchedMediaQuery.matches) {
      switch (matchedMediaQuery.media) {
        case this.mediaQueries[0]:
          this.invoke("mobile");
          break;
        case this.mediaQueries[1]:
          this.invoke("tablet");
          break;
        case this.mediaQueries[2]:
        default:
          this.invoke("desktop");
          break;
      }
    }
  }
}
export default HIP.factory.element(LayoutScreenQueryElement);
