
import { LitElement, html, css } from '../../../hip/library/lit-element/lit-element.js';
import { HIP, AsLitHIC, BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export const styles = css`
  :host {
    --left-panel-min-width: var(--hi-px-24);
    --left-panel-width: 35%;
    --right-panel-min-width: var(--hi-px-48);
    --right-panel-width: 65%;
    --controls-top-bottom: var(--hi-px-8);
    --controls-left-right: var(--hi-px-24);
  }

  .yi-layout-split-2 {
    width: 100%;
    height: 100%;
  }

  .yi-layout-split-2__content {
    position: relative;
    display: flex;
    width: 100%;
  }

  .yi-layout-split-2__divider {
    display: none;
  }

  .yi-layout-split-2__panel {
    width: 100%;
    position: relative;
  }

  .yi-layout-split-2__panel--button {
    position: absolute;
    top: var(--controls-top-bottom);
    left: var(--controls-left-right);
    z-index: 5;
  }

  .yi-layout-split-2__panel--link-left {
    position: absolute;
    top: var(--controls-top-bottom);
    right: var(--controls-left-right);
    z-index: 5;
  }

  .yi-layout-split-2__panel--link-right {
    position: absolute;
    top: var(--controls-top-bottom);
    left: var(--controls-left-right);
    z-index: 5;
  }

  .yi-layout-split-2__content-panel {
    width: 100%;
  }

  [mobile] {
    display: none;
  }

  :host(:not([screen-device='mobile'])) .yi-layout-split-2__panel--button {
    display: none;
  }

  :host(:not([screen-device='mobile'])) .yi-layout-split-2__divider {
    display: block;
    width: var(--hi-px-4);
    background-color: var(--hi-clr--gray-3);
    margin: var(--hi-px-10);
    border-radius: var(--hi-px-10);
  }

  :host(:not([screen-device='mobile'])) .yi-layout-split-2__panel--link-left {
    display: none;
  }

  :host(:not([screen-device='mobile'])) .yi-layout-split-2__panel--link-right {
    display: none;
  }

  :host(:not([screen-device='mobile'])) .yi-layout-split-2__content-left-panel {
    min-width: var(--left-panel-min-width);
    width: var(--left-panel-width);
  }

  :host(:not([screen-device='mobile'])) .yi-layout-split-2__content-right-panel {
    min-width: var(--right-panel-min-width);
    width: var(--right-panel-width);
  }

  :host(:not([screen-device='mobile'])) [mobile] {
    display: block;
  }
`;
export class LayoutSplit2Element extends BehaviorElement(AsLitHIC(LitElement)) {
  constructor() {
    super();
    this.panelToShow = "left";
    this.__leftPanelLinkTitle = "See Next Panel";
    this.__rightPanelLinkTitle = "See Previous Panel";
    const panelTitles = this.__getPanelTitles();
    this.__leftPanelLinkTitle = panelTitles.__leftPanelLinkTitle;
    this.__rightPanelLinkTitle = panelTitles.__rightPanelLinkTitle;
  }
  static get is() {
    return "yi-layout-split-2";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  __getPanelTitles() {
    const panelTitles = {
      __leftPanelLinkTitle: this.__leftPanelLinkTitle,
      __rightPanelLinkTitle: this.__rightPanelLinkTitle
    };
    f1.pipe(
      [
        f1.values,
        f1.each((node) => {
          const element = node;
          const key = `__${element.slot}LinkTitle`;
          if (node.nodeType === Node.ELEMENT_NODE) {
            panelTitles[key] = element.getAttribute("data-link-title") || panelTitles[key];
          }
        })
      ],
      this.childNodes
    );
    return panelTitles;
  }
  __flipSide(side) {
    return side === "right" ? "left" : "right";
  }
  __handleBackClick() {
    const event = new CustomEvent("trigger-ended");
    this.dispatchEvent(event);
  }
  __handlePanelChange() {
    this.panelToShow = this.__flipSide(this.panelToShow);
  }
  render() {
    return html`
      <div class="yi-layout-split-2" l-kind="vert">
        <div class="yi-layout-split-2__content">
          ${this.drilledIn && !this.auto ? html`
                <div class="yi-layout-split-2__panel--button">
                  <hi-button
                    id="yi-layout-split-2__panel--button"
                    @trigger-ended=${this.__handleBackClick}
                    aesthetic="secondary"
                    size="large"
                  >
                    <hi-icon kind="chevron_left"></hi-icon>
                  </hi-button>
                  <hi-tooltip for="yi-layout-split-2__panel--button" placement=${this.panelToShow}
                    >Click to return to previous panel</hi-tooltip
                  >
                </div>
              ` : html``}
          ${this.auto ? html`
                <div id="yi-layout-split-2__panel--link" class="yi-layout-split-2__panel--link-${this.panelToShow}">
                  <hi-button @trigger-ended=${this.__handlePanelChange} aesthetic="secondary" size="large">
                    <hi-icon kind="chevron_${this.__flipSide(this.panelToShow)}"></hi-icon>
                  </hi-button>
                  <hi-tooltip for="yi-layout-split-2__panel--link" placement=${this.panelToShow}
                    >${this.panelToShow === "right" ? `${this.__rightPanelLinkTitle}` : `${this.__leftPanelLinkTitle}`}</hi-tooltip
                  >
                </div>
              ` : html``}
          <div
            ?mobile=${this.panelToShow === "right"}
            class="yi-layout-split-2__content-left-panel yi-layout-split-2__content-panel"
          >
            <slot name="leftPanel"></slot>
          </div>
          <div class="yi-layout-split-2__divider"></div>
          <div
            ?mobile=${this.panelToShow === "left"}
            class="yi-layout-split-2__content-right-panel yi-layout-split-2__content-panel"
          >
            <slot name="rightPanel"></slot>
          </div>
        </div>
      </div>
    `;
  }
  static get properties() {
    return {
      auto: { type: String, attribute: "auto" },
      panelToShow: { type: String, attribute: "panel-to-show" },
      drilledIn: { type: String, attribute: "drilled-in" }
    };
  }
}
export default HIP.factory.element(LayoutSplit2Element);
