
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { HIP, AsLitHIC, BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { LayoutScreenQueryElement } from '../../../yip/element/yi-layout-screen-query/yi-layout-screen-query.js';
export const styles = css`
  :host {
    --width-right: 17.5rem;
    --width-left: 17.5rem;
  }

  :host {
    width: 100%;
    max-width: 102rem;
    min-width: 16rem;
    height: 100%;
    display: block;
    margin: auto;
  }

  .yi-layout-split-3 {
    position: relative;
  }

  .yi-layout-split-3__controls-flyout {
    position: absolute;
    top: 0;
    right: 0;
    z-index: 3;
  }

  .yi-layout-split-3__section {
    border: var(--hi-px-1) solid var(--hi-clr--gray-6);
    box-shadow: var(--hi-clr--shadow-hover);
    background: var(--hi-clr--white);
    border-radius: var(--hi-px-4);
    margin: var(--hi-px-4);
    position: relative;
  }

  .yi-layout-split-3__controls {
    display: block;
  }

  .yi-layout-split-3__controls--buttons {
    display: flex;
    flex-direction: column;
  }

  .yi-layout-split-3__controls--button {
    margin-top: 0.125rem;
    margin-bottom: 0.125rem;
  }

  .yi-layout-split-3__panels {
    display: flex;
    width: 100%;
    height: 100%;
  }

  .yi-layout-split-3__panels--left,
  .yi-layout-split-3__panels--right {
    z-index: 2;
  }

  .yi-layout-split-3__panels--main {
    z-index: 1;
  }

  :host([screen-device='mobile']) .yi-layout-split-3__panels--right {
    width: var(--width-right);
  }

  :host([screen-device='mobile']) .yi-layout-split-3__panels--left {
    width: var(--width-left);
  }

  :host([screen-device='mobile']) .yi-layout-split-3__panels--main {
    width: calc(100% - var(--hi-px-8));
  }

  :host([screen-device='mobile']):host([mobile-state='right']) .yi-layout-split-3__panels--left,
  :host([screen-device='mobile']):host([mobile-state='right']) .yi-layout-split-3__panels--main {
    display: none;
  }

  :host([screen-device='mobile']):host([mobile-state='right']) .yi-layout-split-3__panels--right {
    width: calc(100% - var(--hi-px-8));
  }

  :host([screen-device='mobile']):host([mobile-state='left']) .yi-layout-split-3__panels--right,
  :host([screen-device='mobile']):host([mobile-state='left']) .yi-layout-split-3__panels--main {
    display: none;
  }

  :host([screen-device='mobile']):host([mobile-state='left']) .yi-layout-split-3__panels--left {
    width: calc(100% - var(--hi-px-8));
  }

  :host([screen-device='mobile']):host([mobile-state='main']) .yi-layout-split-3__panels--right,
  :host([screen-device='mobile']):host([mobile-state='main']) .yi-layout-split-3__panels--left {
    display: none;
  }

  :host([screen-device='mobile']):host([mobile-state='main']) .yi-layout-split-3__panels--main {
    width: calc(100% - var(--hi-px-8));
  }

  :host([screen-device='mobile']) [screen-state='mobile'] {
    display: initial;
  }

  :host([screen-device='mobile']) [screen-state='desktop'] {
    display: none;
  }

  :host(:not([screen-device='mobile'])) .yi-layout-split-3__panels--left {
    width: var(--width-left);
    display: block;
  }

  :host(:not([screen-device='mobile'])) .yi-layout-split-3__panels--main {
    width: calc(100% - var(--width-right) - var(--width-left) - var(--hi-px-8));
    display: block;
  }

  :host(:not([screen-device='mobile'])) .yi-layout-split-3__panels--right {
    width: var(--width-right);
    display: block;
  }

  :host(:not([screen-device='mobile'])):host([desktop-state='main-right']) .yi-layout-split-3__panels--left {
    display: none;
  }

  :host(:not([screen-device='mobile'])):host([desktop-state='main-right']) .yi-layout-split-3__panels--main {
    width: calc(100% - var(--width-right) - var(--hi-px-8));
  }

  :host(:not([screen-device='mobile'])):host([desktop-state='main-left']) .yi-layout-split-3__panels--main {
    width: calc(100% - var(--width-left) - var(--hi-px-8));
  }

  :host(:not([screen-device='mobile'])):host([desktop-state='main-left']) .yi-layout-split-3__panels--right {
    display: none;
  }

  :host(:not([screen-device='mobile'])):host([desktop-state='main']) .yi-layout-split-3__panels--left {
    display: none;
  }

  :host(:not([screen-device='mobile'])):host([desktop-state='main']) .yi-layout-split-3__panels--main {
    width: calc(100% - var(--hi-px-8));
  }

  :host(:not([screen-device='mobile'])):host([desktop-state='main']) .yi-layout-split-3__panels--right {
    display: none;
  }

  :host(:not([screen-device='mobile'])) [screen-state='mobile'] {
    display: none;
  }

  :host(:not([screen-device='mobile'])) [screen-state='desktop'] {
    display: initial;
  }
`;
export class LayoutSplit3Element extends BehaviorElement(AsLitHIC(LitElement)) {
  constructor() {
    super(...arguments);
    this.desktopState = "main-left-right";
    this.mobileState = "main";
    this.useControls = true;
    this.mobile = {
      state: "main",
      showMain: true,
      showRight: false,
      showLeft: false
    };
    this.desktop = {
      state: "main-left-right",
      showMain: true,
      showRight: true,
      showLeft: true
    };
    this.screenQuery = LayoutScreenQueryElement.instance;
  }
  static get is() {
    return "yi-layout-split-3";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      mobileState: {
        type: String,
        attribute: "mobile-state",
        reflect: true
      },
      desktopState: {
        type: String,
        attribute: "desktop-state",
        reflect: true
      },
      useControls: {
        type: Boolean,
        attribute: "use-controls",
        reflect: true
      }
    };
  }
  static get api() {
    return ["invoke", "toggle"];
  }
  connectedCallback() {
    super.connectedCallback();
    this.screenQuery.subscribe(this);
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    this.screenQuery.unsubscribe(this);
  }
  invoke(config) {
    this.__reflectStateToState(config);
    return config;
  }
  toggle(direction) {
    this[`__${direction}Toggle_onTriggerEnded`]();
    return direction;
  }
  __leftToggle_onTriggerEnded() {
    this.__reflectTogglesToState("showLeft");
  }
  __mainToggle_onTriggerEnded() {
    this.__reflectTogglesToState("showMain");
  }
  __rightToggle_onTriggerEnded() {
    this.__reflectTogglesToState("showRight");
  }
  __reflectStateToState(config) {
    const newMobileState = {
      state: config.mobile,
      showLeft: false,
      showMain: false,
      showRight: false
    };
    const newDesktopState = {
      state: `main${config.desktop.left ? "-left" : ""}${config.desktop.right ? "-right" : ""}`,
      showLeft: config.desktop.left,
      showMain: true,
      showRight: config.desktop.right
    };
    switch (config.mobile) {
      case "left":
        newMobileState.showLeft = true;
        break;
      case "right":
        newMobileState.showRight = true;
        break;
      case "main":
      default:
        newMobileState.showMain = true;
        break;
    }
    this.mobile = newMobileState;
    this.mobileState = newMobileState.state;
    this.desktop = newDesktopState;
    this.desktopState = newDesktopState.state;
  }
  __reflectTogglesToState(direction) {
    const screenState = this.getAttribute("screen-device") === "mobile" ? "mobile" : "desktop";
    this[screenState][direction] = !this[screenState][direction];
    const _ = this[screenState];
    const newState = screenState === "mobile" ? this.__mobile_togglesToState(direction) : this.__desktop_togglesToState(_.showLeft, _.showRight);
    this[`${screenState}State`] = _.state = newState;
    this[screenState].showRight = newState.indexOf("right") >= 0;
    this[screenState].showMain = newState.indexOf("main") >= 0;
    this[screenState].showLeft = newState.indexOf("left") >= 0;
  }
  __desktop_togglesToState(showLeft, showRight) {
    const right = showRight ? "right" : "";
    const left = showLeft ? "left" : "";
    const main = "main";
    return [main, left, right].filter((_) => _).join("-");
  }
  __mobile_togglesToState(direction) {
    return direction.replace("show", "").toLowerCase();
  }
  render() {
    return html`
      <div class="yi-layout-split-3">
        ${this.useControls ? html`
              <hi-flyout class="yi-layout-split-3__controls-flyout" aesthetic="chrome">
                <hi-icon slot="anchor" kind="gear"> </hi-icon>
                <div class="yi-layout-split-3__section yi-layout-split-3__controls">
                  <div class="yi-layout-split-3__controls--buttons">
                    ${!this.mobile.showLeft ? html` <hi-button
                          class="yi-layout-split-3__controls--button"
                          screen-state="mobile"
                          aesthetic="secondary"
                          stretched
                          @trigger-ended=${this.__leftToggle_onTriggerEnded}
                        >
                          <hi-icon size="large" kind="chevron_left"></hi-icon>
                          <span> Go to Left </span>
                        </hi-button>` : html``}
                    <hi-button
                      class="yi-layout-split-3__controls--button"
                      screen-state="desktop"
                      aesthetic="secondary"
                      @trigger-ended=${this.__leftToggle_onTriggerEnded}
                    >
                      ${this.desktop.showLeft ? html`<hi-icon kind="chevron_left"></hi-icon>` : html``}
                      <span screen-state="desktop"> ${this.desktop.showLeft ? "Hide Left" : "Show Left"} </span>
                      ${this.desktop.showLeft ? html`` : html`<hi-icon kind="chevron_right"></hi-icon>`}
                    </hi-button>
                    ${!this.mobile.showMain ? html` <hi-button
                          class="yi-layout-split-3__controls--button"
                          screen-state="mobile"
                          aesthetic="secondary"
                          stretched
                          @trigger-ended=${this.__mainToggle_onTriggerEnded}
                        >
                          ${this.mobile.showLeft ? html` <hi-icon size="large" kind="home_filled"></hi-icon> ` : html``}
                          <span screen-state="mobile"> Go to Main </span>
                          ${this.mobile.showRight ? html` <hi-icon size="large" kind="home_filled"></hi-icon> ` : html``}
                        </hi-button>` : html``}
                    ${!this.mobile.showRight ? html` <hi-button
                          class="yi-layout-split-3__controls--button"
                          screen-state="mobile"
                          aesthetic="secondary"
                          stretched
                          @trigger-ended=${this.__rightToggle_onTriggerEnded}
                        >
                          <span> Go to Right </span>
                          <hi-icon size="large" kind="chevron_right"></hi-icon>
                        </hi-button>` : html``}
                    <hi-button
                      class="yi-layout-split-3__controls--button"
                      screen-state="desktop"
                      aesthetic="secondary"
                      stretched
                      @trigger-ended=${this.__rightToggle_onTriggerEnded}
                    >
                      ${this.desktop.showRight ? html`` : html`<hi-icon kind="chevron_left"></hi-icon>`}
                      <span screen-state="desktop"> ${this.desktop.showRight ? "Hide Right" : "Show Right"} </span>
                      ${this.desktop.showRight ? html`<hi-icon kind="chevron_right"></hi-icon>` : html``}
                    </hi-button>
                  </div>
                </div>
              </hi-flyout>
            ` : html``}

        <div class="yi-layout-split-3__panels">
          <div class="yi-layout-split-3__section yi-layout-split-3__panels--left">
            <slot name="leftPanel"></slot>
          </div>
          <div class="yi-layout-split-3__section yi-layout-split-3__panels--main">
            <slot name="mainPanel"></slot>
          </div>
          <div class="yi-layout-split-3__section yi-layout-split-3__panels--right">
            <slot name="rightPanel"></slot>
          </div>
        </div>
      </div>
    `;
  }
}
export default HIP.factory.element(LayoutSplit3Element);
