
import { LitElement, html } from '../../../hip/library/lit-element/lit-element.js';
import { HIP, AsHIC, BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export class LayoutWorkflowControlsElement extends BehaviorElement(AsHIC(LitElement)) {
  constructor() {
    super(...arguments);
    this.layout = "";
  }
  static get is() {
    return "yi-layout-workflow-controls";
  }
  static get properties() {
    return {
      options: {
        type: Array
      },
      for: {
        type: String
      }
    };
  }
  static get api() {
    return ["toggle"];
  }
  render() {
    const options = f1.safe([], this.options);
    return html`
      <hi-button-group aesthetic="chrome">
        ${options.map((option) => {
      const optionParts = option.split("_");
      const optionWords = f1.join(
        " ",
        f1.map((opt) => f1.capitalize(opt), optionParts)
      );
      const optionAbbr = f1.join(
        "/",
        f1.map((opt) => f1.capitalize(f1.nth(0, opt)), optionParts)
      );
      return html`
            <hi-button
              value=${option}
              ?disabled=${this.layout === option}
              @trigger-ended=${this.__toggleLayout__onTriggerEnded}
            >
              <strong>${optionAbbr}</strong>
              <hi-hint placement="top">${optionWords}</hi-hint>
            </hi-button>
          `;
    })}
      </hi-button-group>
    `;
  }
  createRenderRoot() {
    return this;
  }
  toggle(layout) {
    return this.layout = layout;
  }
  __toggleLayout__onTriggerEnded(event) {
    const toggledLayout = event.detail.value;
    this.toggle(toggledLayout);
    const __layoutWorkflowEl = HIP.browser.queryOne(this.for, this.ns);
    if (__layoutWorkflowEl) {
      __layoutWorkflowEl.toggleByLayout(toggledLayout);
    }
  }
}
export default HIP.factory.element(LayoutWorkflowControlsElement);
