
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { nothing } from '../../../hip/library/lit-html/lit-html.js';
import navigation from '../../../pip/library/pi-navigation/pi-navigation.js';
import { HIP, AsLitHIC, BehaviorElement, BehaviorCustomizable } from '../../../eip/element/hi-component/hi-component.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { BehaviorScreenQuery } from '../../../yip/element/yi-behavior-screen-query/yi-behavior-screen-query.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export const styles = css`
  :host {
    --panel--topbar-border: var(--hi-px-1) solid var(--hi-clr--gray-3);
    --panel--mobile-padding: var(--hi-px-8);
  }
  :host {
    width: 100%;
    height: 100%;
  }

  :host([screen-device='desktop']) {
    --width-left: 24%;
    --width-right: 24%;
    --hi-right-panel--width: var(--width-right);
    --hi-left-panel--width: var(--width-left);
    width: 100%;
    height: 100%;
    display: grid;
    grid-template-areas: 'left main right';
  }

  .panel {
    position: relative;
    background: var(--hi-clr--white);
    box-shadow: var(--hi-area--shadow);
    border: var(--hi-area--border) var(--hi-clr--gray-6);
    border-radius: var(--hi-area--border-radius);
    display: grid;
    /* height of .panel--topbar */
    grid-template-rows: calc(var(--hi-px-1) + 2 * var(--hi-px-20)) 1fr;
  }

  :host([disallow-borders]) .panel {
    border: none;
    box-shadow: none;
  }

  :host([disallow-overflow]),
  :host([disallow-overflow]) .panel--right,
  :host([disallow-overflow]) .panel--left,
  :host([disallow-overflow]) .panel--main,
  :host([disallow-overflow]) .panel--mobile {
    overflow: hidden;
  }

  :host([disallow-overflow]) .panel--content {
    overflow-y: auto;
    max-height: 100%;
  }

  .panel--right {
    margin-left: var(--hi-px-4);
    grid-area: right;
  }

  .panel--left {
    margin-right: var(--hi-px-4);
    grid-area: left;
  }

  .panel--main {
    grid-area: main;
  }

  .panel--topbar {
    border-bottom: var(--panel--topbar-border);
    background-color: var(--hi-clr--white);
    padding: var(--hi-px-20);
  }

  .panel--topbar-mobile {
    padding: 0;
    border-bottom: var(--panel--topbar-border);
    overflow-x: auto;
    overflow-y: hidden;
    scrollbar-width: thin;
  }

  .panel--topbar-mobile::-webkit-scrollbar {
    height: var(--hi-px-4);
    background: var(--hi-clr--gray-2);
  }

  .panel--topbar-mobile::-webkit-scrollbar-thumb {
    background: var(--hi-clr--gray-7);
  }

  .dock--button-mobile {
    display: inline-block;
    padding: var(--panel--mobile-padding);
    min-width: 100%;
  }

  .dock--button-mobile hi-button {
    flex: 1 1;
    min-width: unset;
  }
  .dock--button {
    padding: var(--hi-px-8);
  }

  .panel--content {
    padding: var(--panel--mobile-padding);
  }

  :host([disallow-borders]) .panel--topbar {
    border-bottom: none;
  }

  :host,
  :host([layout='left_main_right']) {
    grid-template-columns: var(--hi-left-panel--width) 1fr var(--hi-right-panel--width);
  }

  :host([layout='left_main']) {
    grid-template-columns: var(--hi-left-panel--width) 1fr min-content;
  }

  :host([layout='main_right']) {
    grid-template-columns: min-content 1fr var(--hi-right-panel--width);
  }

  :host([layout='left_right']) {
    grid-template-columns: 1fr min-content 1fr;
  }

  :host([layout='main']) {
    grid-template-columns: min-content 1fr min-content;
  }

  :host([layout='left']) {
    grid-template-columns: 1fr min-content min-content;
  }

  :host([layout='right']) {
    grid-template-columns: min-content min-content 1fr;
  }

  :host([layout='left_main']) .panel--right .panel--content,
  :host([layout='main']) .panel--right .panel--content,
  :host([layout='left']) .panel--right .panel--content {
    display: none;
  }

  :host([layout='main_right']) .panel--left .panel--content,
  :host([layout='main']) .panel--left .panel--content,
  :host([layout='right']) .panel--left .panel--content {
    display: none;
  }

  :host([layout='left_right']) .panel--main .panel--content,
  :host([layout='right']) .panel--main .panel--content,
  :host([layout='left']) .panel--main .panel--content {
    display: none;
  }

  @media only screen and (max-width: 62rem) {
    :host {
      --panel--topbar-border: none;
      --panel--mobile-padding: var(--hi-px-1);
    }
  }
`;
export class LayoutWorkflowElement extends BehaviorCustomizable(BehaviorElement(BehaviorScreenQuery(AsLitHIC(LitElement)))) {
  constructor() {
    super(...arguments);
    this.layout = "left_main_right";
    this.activePanel = "main";
    this.defaultSettings = {
      hidden: {
        visibility_control_left: false,
        visibility_control_main: false,
        visibility_control_right: false
      },
      title: {
        panel_title_left: "",
        panel_title_main: "",
        panel_title_right: "",
        mobile_toggle_left: "Left",
        mobile_toggle_main: "Main",
        mobile_toggle_right: "Right"
      }
    };
    this.settings = {
      hidden: {
        visibility_control_left: false,
        visibility_control_main: false,
        visibility_control_right: false
      },
      title: {
        panel_title_left: "",
        panel_title_main: "",
        panel_title_right: "",
        mobile_toggle_left: "Left",
        mobile_toggle_main: "Main",
        mobile_toggle_right: "Right"
      }
    };
  }
  static get is() {
    return "yi-layout-workflow";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      layout: {
        type: String,
        reflect: true
      },
      settings: {
        type: Object
      },
      activePanel: {
        type: String,
        attribute: "active-panel",
        reflect: true
      },
      uriState: {
        type: String,
        attribute: "uri-state",
        reflect: true
      }
    };
  }
  static get api() {
    return ["toggleByLayout", "toggleByPanel"];
  }
  static get autoBinding() {
    return {
      signal: "trigger-ended",
      field: "activePanel"
    };
  }
  __getURIParameters() {
    const href = navigation.location.active.href;
    const URIParameters = navigation.uri.readParameters(href);
    return URIParameters;
  }
  __getActivePanelUriParam() {
    const queryParameters = this.__getURIParameters();
    const activePanelURIParam = queryParameters[`${this.uriState}`];
    return activePanelURIParam;
  }
  connectedCallback() {
    super.connectedCallback();
    if (this.isScreenDeviceMobileOrTablet()) {
      this.__updateActivePanelIfUriExist();
    }
  }
  __onPanelChanged(panel) {
    this.activePanel = panel;
    const event = new CustomEvent("panel-changed", { detail: panel });
    this.dispatchEvent(event);
    this.__updateUriOnPanelChanged(this.activePanel);
  }
  mediaDeviceHook(mediaName) {
    this.requestUpdate();
  }
  update(props) {
    super.update(props);
    if (props.has("screenDevice") || props.has("forcedScreenDevice")) {
      this.screenDevice = this.forcedScreenDevice ? this.forcedScreenDevice : this.screenDevice;
    }
  }
  render() {
    this.settings = f1.merge("over", f1.clone(this.defaultSettings), this.settings);
    const layoutStrategy = {
      mobile: this.__renderLayoutForMobile(),
      tablet: this.__renderLayoutForMobile(),
      desktop: this.__renderLayoutForDesktop(),
      fallback: this.__renderLayoutForDesktop()
    };
    return this._renderLayoutStrategy(layoutStrategy);
  }
  __shouldRenderPanel(showToggle, panel) {
    return showToggle || this.layout.includes(panel);
  }
  __renderLayoutForDesktop() {
    const showRightToggle = this.__computeHiddenSettings("right");
    const showLeftToggle = this.__computeHiddenSettings("left");
    const showMainToggle = this.__computeHiddenSettings("main");
    const rightTitle = this.__computeTitleSettings("right");
    const leftTitle = this.__computeTitleSettings("left");
    const mainTitle = this.__computeTitleSettings("main");
    return html`
      ${this.__shouldRenderPanel(showLeftToggle, "left") ? html`<section class="panel panel--left">
            <section class="panel--topbar">
              ${showLeftToggle ? html` <hi-corner inside-edge dock="top-left" class="dock--button">
                      <slot name="left_topbar"></slot>
                    </hi-corner>
                    <hi-corner inside-edge dock="top-right" class="dock--button">
                      ${this.__renderCornerButton(this.layout.includes("left"), {
      icon: "west",
      title: "Close Left Panel",
      triggerEnded: () => this.toggleByPanel("left", false)
    })}
                      ${this.__renderCornerButton(!this.layout.includes("left"), {
      icon: "east",
      title: "Open Left Panel",
      triggerEnded: () => this.toggleByPanel("left", true)
    })}
                    </hi-corner>` : nothing}
            </section>
            ${!f1.is.empty(leftTitle) ? html`<h3 class="panel--title">${leftTitle}</h3>` : nothing}
            <section class="panel--content">
              <slot name="left"></slot>
            </section>
          </section>` : nothing}
      ${this.__shouldRenderPanel(showMainToggle, "main") ? html` <section class="panel panel--main">
            <section class="panel--topbar">
              ${showMainToggle ? html` <hi-corner inside-edge dock="top-right" class="dock--button">
                    ${this.__renderCornerButton(this.layout.includes("main"), {
      icon: "close_fullscreen",
      title: "Close Main Panel",
      triggerEnded: () => this.toggleByPanel("main", false)
    })}
                    ${this.__renderCornerButton(!this.layout.includes("main"), {
      icon: "open_in_full",
      title: "Open Main Panel",
      triggerEnded: () => this.toggleByPanel("main", true)
    })}
                    ${this.__renderCornerButton(
      this.layout.includes("main") && (this.layout.includes("left") || this.layout.includes("right")),
      {
        icon: "open_in_full",
        title: "Open Main Panel Full",
        triggerEnded: () => {
          this.toggleByPanel("right", false);
          this.toggleByPanel("left", false);
        }
      }
    )}
                  </hi-corner>` : nothing}
            </section>
            ${!f1.is.empty(mainTitle) ? html`<h3 class="panel--title">${mainTitle}</h3>` : nothing}
            <section class="panel--content">
              <slot></slot>
              <slot name="main"></slot>
            </section>
          </section>` : nothing}
      ${this.__shouldRenderPanel(showRightToggle, "right") ? html` <section class="panel panel--right">
            <section class="panel--topbar" class="dock--button">
              ${showRightToggle ? html` <hi-corner inside-edge dock="top-right">
                      <slot name="right_topbar"></slot>
                    </hi-corner>
                    <hi-corner inside-edge dock="top-left" class="dock--button">
                      ${this.__renderCornerButton(this.layout.includes("right"), {
      icon: "east",
      title: "Close Right Panel",
      triggerEnded: () => this.toggleByPanel("right", false)
    })}
                      ${this.__renderCornerButton(!this.layout.includes("right"), {
      icon: "west",
      title: "Open Right Panel",
      triggerEnded: () => this.toggleByPanel("right", true)
    })}
                    </hi-corner>` : nothing}
            </section>
            ${!f1.is.empty(rightTitle) ? html`<h3 class="panel--title">${rightTitle}</h3>` : nothing}
            <section class="panel--content">
              <slot name="right"></slot>
            </section>
          </section>` : nothing}
    `;
  }
  __renderLayoutForMobile() {
    const title = this.__computeTitleSettings(this.activePanel);
    return html` <section class="panel panel--mobile">
      <section class="panel--topbar panel--topbar-mobile ">
        <div class="dock--button-mobile">${this.__renderMobilePanelToggles()}</div>
      </section>
      ${!f1.is.empty(title) ? html`<h3 class="panel--title">${title}</h3>` : nothing}
      <section class="panel--content">
        ${this.activePanel === "main" ? html` <slot name="main"></slot>
              <slot></slot>` : html`<slot .name=${this.activePanel}></slot>`}
      </section>
    </section>`;
  }
  __renderCornerButton(compare, options) {
    if (!compare) {
      return nothing;
    }
    return html`<hi-button bordered size="small" layout="icon" aesthetic="ghost" @trigger-ended=${options.triggerEnded}>
      <hi-icon size="small" kind=${options.icon} type="material"></hi-icon>
      <span class="show-ax">${options.title}</span>
    </hi-button>`;
  }
  __renderMobilePanelToggles() {
    const showLeftToggle = this.__computeHiddenSettings("left");
    const showMainToggle = this.__computeHiddenSettings("main");
    const showRightToggle = this.__computeHiddenSettings("right");
    const panelToggles = [
      {
        region: "left",
        label: this.settings.title.mobile_toggle_left,
        hidden: !this.__shouldRenderPanel(showLeftToggle, "left"),
        triggerEnded: () => this.__onPanelChanged("left")
      },
      {
        region: "main",
        label: this.settings.title.mobile_toggle_main,
        hidden: !this.__shouldRenderPanel(showMainToggle, "main"),
        triggerEnded: () => this.__onPanelChanged("main")
      },
      {
        region: "right",
        label: this.settings.title.mobile_toggle_right,
        hidden: !this.__shouldRenderPanel(showRightToggle, "right"),
        triggerEnded: () => this.__onPanelChanged("right")
      }
    ];
    return html` <hi-button-group aesthetic="none" layout="tight">
      ${panelToggles.map(
      (panel) => panel.hidden ? nothing : html`<hi-button
              .aesthetic=${panel.region === this.activePanel ? "primary" : "ghost"}
              bordered
              size="small"
              @trigger-ended=${panel.triggerEnded}
              >${panel.label}</hi-button
            >`
    )}
    </hi-button-group>`;
  }
  toggleByLayout(layout) {
    return this.layout = layout;
  }
  toggleByPanel(panel, value) {
    const showRightToggle = this.__computeHiddenSettings("right");
    const showLeftToggle = this.__computeHiddenSettings("left");
    const showMainToggle = this.__computeHiddenSettings("main");
    const layout = this.layout;
    const visibility = {
      left: layout.includes("left"),
      main: layout.includes("main"),
      right: layout.includes("right"),
      [panel]: value
    };
    const requestedLayoutList = f1.compact([
      visibility.left ? "left" : null,
      visibility.main ? "main" : null,
      visibility.right ? "right" : null
    ]);
    const requestedLayout = f1.join("_", requestedLayoutList);
    const defaultLayoutList = f1.compact([
      showLeftToggle ? "left" : null,
      showMainToggle ? "main" : null,
      showRightToggle ? "right" : null
    ]);
    const defaultLayout = f1.join("_", defaultLayoutList);
    this.layout = requestedLayout.trim() === "" ? defaultLayout : requestedLayout;
  }
  __computeHiddenSettings(region) {
    return !f1.is.empty(this.settings.hidden) ? f1.safe(true, !this.settings.hidden["visibility_control_" + region]) : true;
  }
  __computeTitleSettings(region) {
    return !f1.is.empty(this.settings.title) ? f1.safe("", this.settings.title["panel_title_" + region]) : "";
  }
  __updateActivePanelIfUriExist() {
    const requestedActivePanelURIParam = this.__getActivePanelUriParam();
    const currentActivePanel = this.activePanel;
    if (f1.is.falsy(requestedActivePanelURIParam)) {
      return;
    }
    if (f1.is.equal(requestedActivePanelURIParam, currentActivePanel)) {
      return;
    }
    this.activePanel = requestedActivePanelURIParam;
  }
  __updateUriOnPanelChanged(activePanel) {
    const queryParam = this.uriState;
    const queryParameters = this.__getURIParameters();
    queryParameters[`${queryParam}`] = activePanel;
    if (!f1.is.empty(queryParam)) {
      const base_hash = navigation.location.active.base_hash;
      const pathname = navigation.location.active.pathname.replace(/\/$/, "");
      const routeWithQueryParams = navigation.uri.writeQuery("", [], queryParameters);
      const finalRoute = `${pathname}${base_hash}?${routeWithQueryParams}`;
      navigation.location.changeStateWithoutReload({}, "", finalRoute);
    }
  }
}
export default HIP.factory.element(LayoutWorkflowElement);
