
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __decorateClass = (decorators, target, key, kind) => {
  var result = kind > 1 ? void 0 : kind ? __getOwnPropDesc(target, key) : target;
  for (var i = decorators.length - 1, decorator; i >= 0; i--)
    if (decorator = decorators[i])
      result = (kind ? decorator(target, key, result) : decorator(result)) || result;
  if (kind && result)
    __defProp(target, key, result);
  return result;
};
import { LitElement, css, html } from '../../../hip/library/lit-element/lit-element.js';
import { queryAssignedNodes } from '../../../hip/library/lit-element/decorators.js';
import { unsafeHTML } from '../../../hip/library/lit-html/directives/unsafe-html.js';
import { HIP, AsLitHIC, BehaviorElement } from '../../../eip/element/hi-component/hi-component.js';
import { stylesTheme } from '../../../eip/element/hi-component/hi-component-theme.js';
import { BehaviorScreenQuery } from '../../../yip/element/yi-behavior-screen-query/yi-behavior-screen-query.js';
import '../../../eip/element/hi-title/hi-title.js';
import '../../../eip/element/hi-reveal/hi-reveal.js';
import f1 from '../../../pip/library/pi-f1/pi-f1.js';
export const styles = css`
  :host {
    display: flex;
    flex-direction: column;
  }

  #status,
  #toolbar,
  #navigation {
    display: flex;
    gap: var(--hi-px-4);
  }

  #superTitle {
    color: var(--hi-clr--gray-7);
  }

  #status,
  #toolbar {
    margin-top: var(--hi-px-8);
    align-items: center;
  }

  #headerTitle,
  #template,
  #summary,
  #toolbar,
  #status,
  #alert {
    margin-bottom: var(--hi-px-8);
  }

  #headerTitle {
    display: flex;
    gap: var(--hi-px-4);
    align-items: flex-start;
    justify-content: space-between;
  }

  :host([level='1']) #headerTitle {
    padding-top: var(--hi-px-16);
  }

  :host([level='2']) #headerTitle {
    padding-top: var(--hi-px-12);
  }

  :host([level='3']) #headerTitle {
    padding-top: var(--hi-px-8);
  }

  #titleBlock {
    display: flex;
    flex-direction: column;
    align-self: flex-start;
  }

  #navigation {
    margin-left: auto;
  }

  [is-empty-slot] {
    display: none !important;
  }
`;
export class ViewHeaderElement extends BehaviorElement(BehaviorScreenQuery(AsLitHIC(LitElement))) {
  constructor() {
    super(...arguments);
    this.level = 1;
    this.__isEmpty = {
      description: true,
      navigation: true,
      template: true,
      summary: true,
      toolbar: true,
      status: true,
      alert: true
    };
  }
  static get is() {
    return "yi-view-header";
  }
  static get styles() {
    return [stylesTheme, styles];
  }
  static get properties() {
    return {
      title: {
        type: String
      },
      superTitle: {
        type: String,
        attribute: "super-title"
      },
      subTitle: {
        type: String,
        attribute: "sub-title"
      },
      description: {
        type: String
      },
      level: {
        type: Number,
        reflect: true
      },
      __isEmpty: {
        type: Object
      }
    };
  }
  connectedCallback() {
    super.connectedCallback();
    this.setAttribute("role", "banner");
  }
  __onSlotChanged(e, part) {
    if (e.currentTarget.name === part) {
      this.__isEmpty[part] = this[`__${part}Slot`].length <= 0;
      this.requestUpdate("__isEmpty");
    }
  }
  render() {
    const ns = this.ns;
    return html`
      <div id="headerTitle">
        <div id="titleBlock">
          <slot name="super_title">
            <p id="superTitle">${this.superTitle}</p>
            <p id="subTitle">${this.subTitle}</p>
          </slot>
          <slot
            name="title"
            @slotchange=${() => (this.__titleSlot || []).forEach((el) => el.classList.add("view-title"))}
          >
            <hi-title id="title" gid="title" level=${this.level} title=${this.title} .ns=${ns}></hi-title>
          </slot>
        </div>
        <div id="navigation" ?is-empty-slot=${this.__isEmpty["navigation"]}>
          <slot name="navigation" @slotchange=${(e) => this.__onSlotChanged(e, "navigation")}></slot>
        </div>
      </div>
      <div id="status" ?is-empty-slot=${this.__isEmpty["status"]}>
        <slot name="status" @slotchange=${(e) => this.__onSlotChanged(e, "status")}></slot>
      </div>
      <div id="description" ?is-empty-slot=${this.__isEmpty["description"] && f1.is.empty(this.description)}>
        <slot name="description" @slotchange=${(e) => this.__onSlotChanged(e, "description")}>
          <hi-reveal max-height="2" max-content="200" .ns=${ns}>${unsafeHTML(this.description)}</hi-reveal>
        </slot>
      </div>
      <div id="summary" ?is-empty-slot=${this.__isEmpty["summary"]}>
        <slot name="summary" @slotchange=${(e) => this.__onSlotChanged(e, "summary")}></slot>
      </div>
      <div id="toolbar" ?is-empty-slot=${this.__isEmpty["toolbar"]}>
        <slot name="toolbar" @slotchange=${(e) => this.__onSlotChanged(e, "toolbar")}></slot>
      </div>
      <div id="alert" ?is-empty-slot=${this.__isEmpty["alert"]}>
        <slot name="alert" @slotchange=${(e) => this.__onSlotChanged(e, "alert")}></slot>
      </div>
      <div id="template" ?is-empty-slot=${this.__isEmpty["template"]}>
        <slot name="template" @slotchange=${(e) => this.__onSlotChanged(e, "template")}></slot>
      </div>
    `;
  }
  updated(props) {
    super.updated(props);
    const children = [...this.children];
    f1.each(
      (activeSlot) => {
        this.__isEmpty[activeSlot] = false;
      },
      children.map((e) => e.slot)
    );
  }
}
__decorateClass([
  queryAssignedNodes({ slot: "status" })
], ViewHeaderElement.prototype, "__statusSlot", 2);
__decorateClass([
  queryAssignedNodes({ slot: "navigation" })
], ViewHeaderElement.prototype, "__navigationSlot", 2);
__decorateClass([
  queryAssignedNodes({ slot: "summary" })
], ViewHeaderElement.prototype, "__summarySlot", 2);
__decorateClass([
  queryAssignedNodes({ slot: "toolbar" })
], ViewHeaderElement.prototype, "__toolbarSlot", 2);
__decorateClass([
  queryAssignedNodes({ slot: "alert" })
], ViewHeaderElement.prototype, "__alertSlot", 2);
__decorateClass([
  queryAssignedNodes({ slot: "template" })
], ViewHeaderElement.prototype, "__templateSlot", 2);
__decorateClass([
  queryAssignedNodes({ slot: "description" })
], ViewHeaderElement.prototype, "__descriptionSlot", 2);
__decorateClass([
  queryAssignedNodes({ slot: "title" })
], ViewHeaderElement.prototype, "__titleSlot", 2);
export default HIP.factory.element(ViewHeaderElement);
