/**
 * we accept values as strings and expand to `<hi-option>` format for a dropdown
 * @example
 *
 * ```
 * get_notes_filter_options([
     ['s1', 's2']
   ]) => [{ title: 'string', value: 'string' }]
 * ```
 */
HIL.dsl.plugin.register({
  id: 'get_notes_filter_options',
  pointCut: {
    prePostCompile: (script) => {
      script.user_space = {
        ...(script.user_space || {}),
        /**
         *
         * @param {string[][]} nestedNoteReasons
         */
        get_notes_filter_options: function (nestedNoteReasons) {
          return nestedNoteReasons.reduce(
            (optionsList, noteReasons) => {
              return noteReasons.reduce((_optionsList, noteReason) => {
                return [..._optionsList, { title: noteReason, value: noteReason }];
              }, optionsList);
            },
            [{ title: 'No Note Reason', value: 'no note reason' }]
          );
        }
      };

      return script;
    }
  }
});
