export class ReportsServant {
  static toggleStar(status) {
    let inProgress = false;
    const star_report = HIP.db.gid.queryActive('star_report');
    const unstar_report = HIP.db.gid.queryActive('unstar_report');
    return (e) => {
      e.preventDefault();
      e.stopPropagation();
      if (inProgress) {
        HIP.fn.notify({ message: 'Please wait', delay: 1000 });
        return;
      }
      if (status !== 'flagged' && star_report) {
        inProgress = true;
        star_report
          .invoke()
          .then(() => {
            inProgress = false;
            status = 'flagged';
            document.getElementById('report_status').kind = 'star_filled';
          })
          .catch(() => {
            inProgress = false;
            HIP.fn.notify({ message: 'Failed to star report', status: 'error' });
          });
      } else if (status === 'flagged' && unstar_report) {
        inProgress = true;
        unstar_report
          .invoke()
          .then(() => {
            inProgress = false;
            status = 'read';
            document.getElementById('report_status').kind = 'star';
          })
          .catch(() => {
            inProgress = false;
            HIP.fn.notify({ message: 'Failed to unstar report', status: 'error' });
          });
      } else {
        HIP.fn.notify({ message: 'Failed to toggle star', status: 'error' });
      }
    };
  }

  static markReportAsRead(status) {
    let markReadInProgress = false;
    if (status === 'unread' && !markReadInProgress) {
      const read_report = HIP.db.gid.queryActive('read_report');
      if (read_report) {
        markReadInProgress = true;
        read_report
          .invoke()
          .then(() => {
            markReadInProgress = false;
            status = 'read';
          })
          .catch(() => {
            markReadInProgress = false;
            HIP.fn.notify({ message: 'Failed to mark report as read', status: 'error' });
          });
      } else {
        HIP.fn.notify({ message: 'Failed to mark report as read', status: 'error' });
      }
    }
  }

  static getInitial(name) {
    if (name === undefined || name === '') {
      return '';
    }
    var names = name.split(' ');
    return names[0].substring(0, 1).toUpperCase() + names[1].substring(0, 1).toUpperCase();
  }

  static openLink(link, needsJWT, isMobile) {
    const configuration = HIP.db.gid.queryActive('configuration');
    const campus_url =
      configuration && configuration.output && configuration.output['core-campus_url']
        ? configuration.output['core-campus_url']
        : '';
    const studentNk =
      configuration && configuration.output && configuration.output['student_nk']
        ? configuration.output['student_nk']
        : '';
    let inProgress = false;
    return (e) => {
      e.preventDefault();
      e.stopPropagation();
      if (needsJWT) {
        if (inProgress) {
          return;
        }
        const fetch_JWT_API = HIP.db.gid.queryActive('fetch_jwt_token');
        fetch_JWT_API['user-mode'] = 'advisor';
        fetch_JWT_API.headers['student-id'] = studentNk;
        HIP.fn.notify({ message: 'Opening attachment', delay: 5000 });
        if (fetch_JWT_API) {
          // API Call to fetch token
          inProgress = true;
          fetch_JWT_API
            .invoke()
            .then(({ access_token }) => {
              inProgress = false;
              window.open(`${campus_url}${link}?authorization=${access_token}`, '_blank', {}, !isMobile);
            })
            .catch(() => {
              inProgress = false;
              HIP.fn.notify({ message: 'Failed to load attachment', status: 'error' });
            });
        } else {
          HIP.fn.notify({ message: 'Failed to load attachment', status: 'error' });
        }
      } else {
        window.open(link, '_blank', {}, !isMobile);
      }
    };
  }

  static readProgressReportConfigs() {
    let config = window.localStorage.getItem('eab.navigate.progress_report_config');
    return JSON.parse(config);
  }

  static processReportBody(body) {
    if (body == null || body === '') {
      return '';
    }
    var parsedBody = new DOMParser().parseFromString(body, 'text/html');
    parsedBody.querySelectorAll('a').forEach((a) => {
      var link = a.getAttribute('href');
      a.setAttribute('onclick', `window.open('${link}', '_blank');return false;`);
      a.setAttribute('href', '#');
    });
    return parsedBody.body.innerHTML;
  }
}
