const {
  html,
  nothing,
  directives: { unsafeHTML, repeat }
} = HIP.extern.lit;

import { ReportsServant } from '../servants/reports_servant.js';
export const type = 'template';
export const id = 'appt_summary__detail';
export const template = (recordType, typeMap, builderEl) => {
  const isNA = (txt) => (txt == null || txt === 'Invalid date' ? 'N/A' : txt);
  const isMobile = document.getElementsByTagName('html')[0].classList.contains('view-mobile');

  return () => (list) => {
    const o = tt.first(list);
    const templateLabels = tt.filter((v, k) => k.endsWith('_label'), o.template || []);
    const labelValueTemplate = tt.map((label, k) => {
      return {
        label: label,
        value: o.template[k.replace('_label', '')]
      };
    }, templateLabels);

    const labelTextTemplate = tt.filter((u, k) => {
      return tt.is(String, u.value);
    }, labelValueTemplate);

    const labelRadioButtonTemplate = tt.filter((u, k) => {
      return tt.is(Number, u.value);
    }, labelValueTemplate);

    const getInitial = ReportsServant.getInitial;
    const toggleStar = ReportsServant.toggleStar;
    const openLink = ReportsServant.openLink;
    const processReportBody = ReportsServant.processReportBody;
    ReportsServant.markReportAsRead(o.status);

    const favIcon = o.status === 'flagged' ? 'star_filled' : 'star';

    const textTemplate = Object.values(labelTextTemplate);
    const radioButtonTemplate = Object.values(labelRadioButtonTemplate);

    const summaryDetail = html`
      <hi-bar> Summary Details </hi-bar>
      <hi-note class="text-sz-l">
        ${textTemplate.map(
          (u) => html`
            <p class="mtsx text-normal">${u.label}</p>
            <p>${u.value === '' ? 'N/A' : u.value}</p>
          `
        )}
        ${radioButtonTemplate.length > 0
          ? html`
              <p class="mtmx text-normal">Advisors Notes</p>
              <ul kind="no-dot">
                ${radioButtonTemplate.map(
                  (u) => html`
                    <li>
                      ${u.label}: <span class="text-bold">${u.value > 0 ? 'Yes' : u.value < 0 ? 'N/A' : 'No'}</span>
                    </li>
                  `
                )}
              </ul>
            `
          : nothing}
      </hi-note>
    `;

    const suggestedFollowupDetail = html`
      <hi-bar> Suggested Follow-Up </hi-bar>
      <hi-note class="text-sz-l">
        <p class="mbm">
          * This date and time have been added for suggested follow up from this appointment. This does not mean that an
          appointment has been created for this date and time.
        </p>

        <ul kind="no-dot">
          <li><span class="text-bold">Date</span>: ${isNA(moment(o.suggested_followup).format('MM/DD/YYYY'))}</li>
          <li><span class="text-bold">Time</span>: ${isNA(moment(o.suggested_followup).format('h:mm a'))}</li>
        </ul>
      </hi-note>
    `;

    return html`
      <header l-kind="hort" l-width-fixed l-align="center">
        <div class="mrm">
          <hi-avatar
            initials="${getInitial(o.from_name)}"</hi-avatar>
        </div>
        <div l-size="grow">
          <p class="text-sz-l text-normal">From: ${o.from_name}</p>
          <p class="clr-gray_80-txt">${moment(o.created_at).format('ll')}</p>
        </div>
        <div class="mrm">
        <hi-link href="#" @trigger-ended="${toggleStar(o.status)}">
          <hi-icon id="report_status" kind="${favIcon}" theme="orange"></hi-icon>
        </hi-link>
        </div>

      </header>
      <section class="pam"
        l-kind="vert">
        <p class="text-sz-l mtm">${unsafeHTML(processReportBody(o.summary))}</p>
        ${o.attachments.length > 0 ? html`<p class="text-sz-l mtm mbsx text-normal">Attachment(s)</p>` : nothing} 
        <ul data-tap-disabled="true" kind="no-dot">${tt.map((l) => {
          const fileName = l.name != '' ? l.name : l.uri.slice(l.uri.lastIndexOf('/') + 1);
          return html` <li class="display-inline-block mbm">
            <hi-link href="#" @trigger-ended="${openLink(l.uri, true, isMobile)}">
              <hi-icon kind="paperclip" mode="presentation"></hi-icon>
              <span>${fileName}</span>
            </hi-link>
          </li>`;
        }, o.attachments)}</ul>

      </section>
      <hi-accordion>
        ${Object.keys(o.template).length !== 0 ? summaryDetail : html``}
        ${o.suggested_followup ? suggestedFollowupDetail : html``}

        <hi-bar>
          Appointment Details
        </hi-bar>
        <hi-note class="text-sz-l">
          <ul kind="no-dot">
            <li><span class="text-bold">Care Unit</span>: ${isNA(o.care_unit)}</li>
            <li><span class="text-bold">Service</span>: ${isNA(html`${o.service.join(', ')}`)}</li>
            <li><span class="text-bold">Location</span>: ${isNA(o.location)}</li>
            <li><span class="text-bold">Course</span>: ${isNA(o.course)}</li>
            <li><span class="text-bold">Meeting Type</span>: ${isNA(o.meeting_type)}</li>
            <li><span class="text-bold">Date of Visit</span>: ${isNA(moment(o.date_of_visit).format('MM/DD/YYYY'))}</li>
            <li><span class="text-bold">Meeting Start Time</span>: ${isNA(
              moment(o.meeting_start_time).format('h:mm a')
            )}</li>
            <li><span class="text-bold">Meeting End Time</span>: ${isNA(
              moment(o.meeting_end_time).format('h:mm a')
            )}</li>
            <li><span class="text-bold">Attendees</span>: ${html`${o.attendees.join(', ')}`}</li>
            <li><span class="text-bold">Check-in Time</span>: ${isNA(moment(o.check_in_time).format('h:mm a'))}</li>
            <li><span class="text-bold">Check-out Time</span>: ${isNA(moment(o.check_out_time).format('h:mm a'))}</li>
          </ul>
        </hi-note>

      </hi-accordion>
    `;
  };
};
