const { html, nothing } = HIP.extern.lit;

import { ReportsServant } from '../servants/reports_servant.js';
export const type = 'template';
export const id = 'progress_report_detail';
export const template = () => {
  const configuration = HIP.db.gid.queryActive('configuration');
  const firstName =
    configuration && configuration.output && configuration.output['firstName'] ? configuration.output['firstName'] : '';

  return () => (list) => {
    const o = tt.first(list);
    const getInitial = ReportsServant.getInitial;
    const toggleStar = ReportsServant.toggleStar;
    ReportsServant.markReportAsRead(o.status);
    const progressReportConfigs = ReportsServant.readProgressReportConfigs();

    const favIcon = o.status === 'flagged' ? 'star_filled' : 'star';

    const hasCommentsViewPermission = typeof o.comments === 'string';
    const hasAlertReasons = o.alert_reasons && o.alert_reasons.length !== 0;
    const listAlertReasons = function (reasons) {
      return typeof reasons === 'string' ? reasons : reasons.join(', ');
    };

    return html`
        <header l-kind="hort" l-width-fixed l-align="center">
          <div class="mrm">
            <hi-avatar initials="${getInitial(o.from_name)}" </hi-avatar>
          </div>
          <div l-size="grow">
            <p class="text-sz-l text-normal">From: ${o.from_name}</p>
            <p class="clr-gray_80-txt">${moment(o.created_at).format('ll')}</p>
          </div>
          <div class="mrm">
          <hi-link href="#" @trigger-ended="${toggleStar(o.status)}">
            <hi-icon id="report_status" kind="${favIcon}" theme="orange"></hi-icon>
          </hi-link>
          </div>

        </header>
        <section class="pam">
          <p class="text-sz-m mtm">Hi, your instructor ${o.from_name} has the following feedback regarding your
            <span class="text-normal">${
              o.course_name
            }</span> class. If a field says "N/A", this means that your instructor left that field blank.
            </p>
          <dl>
          ${
            hasAlertReasons
              ? html`
                  <div l-kind l-width-fixed l-align="center" class="ptm">
                    <dt class="text-normal prm">
                      ${localStorage.getItem('eab.navigate.alert_name') === 'undefined' ||
                      !localStorage.getItem('eab.navigate.alert_name')
                        ? 'Feedback'
                        : JSON.parse(localStorage.getItem('eab.navigate.alert_name'))}
                    </dt>
                    <dd>${listAlertReasons(o.alert_reasons)}</dd>
                  </div>
                `
              : nothing
          }
          ${
            progressReportConfigs.show_absences
              ? html`
                  <div l-kind l-width-fixed l-align="center" class="ptm">
                    <dt class="text-normal prm">${progressReportConfigs.absences_verbiage || 'How Many Absences?:'}</dt>
                    <dd>${o.number_of_absences === '' ? 'N/A' : o.number_of_absences}</dd>
                  </div>
                `
              : nothing
          }
          ${
            progressReportConfigs.show_grade
              ? html`
                  <div l-kind l-width-fixed l-align="center" class="ptm">
                    <dt class="text-normal prm">${progressReportConfigs.grade_verbiage || 'Current Grade:'}</dt>
                    <dd>${o.current_grade === null ? 'N/A' : o.current_grade}</dd>
                  </div>
                `
              : nothing
          }
          ${
            hasCommentsViewPermission && progressReportConfigs.show_comments
              ? html`
                  <div l-width-fixed l-align="center" class="ptm">
                    <dt class="text-normal prm pbsxx">${progressReportConfigs.comments_verbiage || 'Comments:'}</dt>
                    <dd>${o.comments === '' ? 'N/A' : o.comments}</dd>
                  </div>
                `
              : nothing
          }
          </dl>
        </section>
      `;
  };
};
