import { LitElement, html } from 'https://cdn.eab.com/toolkit/21-4-12/hip/library/lit-element/lit-element.js';
import { unsafeHTML } from 'https://cdn.eab.com/toolkit/21-4-12/hip/library/lit-html/directives/unsafe-html.js';

// PURE FUNCTIONS
const getCurrentLocationParams = () => {
  return window.location.hash ? `?${window.location.hash.split('?')[1]}` : window.location.search;
};
const getURLParams = () => {
  const searchParams = getCurrentLocationParams();
  const formDataFromUrlParams = new URLSearchParams(searchParams);
  let paramsData = {};
  const VALID_PARAMS = [
    'care_unit_id',
    'service_id',
    'begin_date',
    'staff_ids',
    'location_id',
    'pal_guid',
    'meeting_type_id'
  ];
  VALID_PARAMS.forEach((key) => {
    if (formDataFromUrlParams.get(key) != null) {
      paramsData[key] = formDataFromUrlParams.get(key);
    }
  });
  return paramsData;
};

class PrefabAppointmentResultsElement extends HIP.BehaviorPrefab(HIP.BehaviorElement(LitElement)) {
  static get is() {
    return 'ai-prefab-new-appointment';
  }

  constructor() {
    super();
    this.filteredFormDefinition = {};
    this.urlParams = {};
    this.isFetchingResults = true;
    this.show_dropin = false;
    this.can_show_dropin = false;
    this.show_pal_escape = false;
    this.show_request_time = false;
    this.show_success_team = false;
    this.can_show_request_time = false;
    this.can_show_schedule_appointment = false;
    this.can_show_success_team = false;
    this.show_staff_escape = false;
    this.student_scheduler_button_text = 'Find Available Time';
    this.locationHashWatcher = () => {
      this.urlParams = getURLParams();
      this.updateFilterDef(this.urlParams);

      let needsRefresh = false;
      Object.keys(this.apiParams).forEach((paramName) => {
        if (!this.urlParams[paramName]) {
          needsRefresh = true;
        }
      });
      if (needsRefresh) {
        this.getResults(this.urlParams);
      }
    };
    this.is_schedulable = false;
    this.is_requestable = false;
    this.is_visitable = false;
  }

  connectedCallback() {
    super.connectedCallback();
    const contextOutput = this.getInstance('context').output;
    this.is_requestable = !!contextOutput.is_requestable;
    this.is_visitable = !!contextOutput.is_visitable;
    this.is_schedulable = !!contextOutput.is_schedulable;
    this.urlParams = getURLParams();
    this.can_show_success_team = contextOutput.show_success_team === false ? false : true;
    if (this.urlParams.hasOwnProperty('pal_guid')) {
      this.show_pal_escape = true;
    }
    if (this.urlParams.hasOwnProperty('staff_ids')) {
      this.show_staff_escape = true;
    }
    if (!this.show_pal_escape && !this.show_staff_escape) {
      this.show_success_team = this.can_show_success_team;
    }
    this.getInstance('initialize').addEventListener('invoke-ended', () => {});

    this.getInstance('services-record').addEventListener('output-changed', () => {
      setTimeout(() => {
        this.isFetchingResults = false;
        this.updateFilterDef(this.urlParams, true);
      }, 0);
    });

    const schedulable_named_bucket = this.getInstance('store_schedulable_service_ids');
    if (!this.is_schedulable && schedulable_named_bucket !== undefined) {
      schedulable_named_bucket.addEventListener('output-changed', (event) => {
        this.can_show_schedule_appointment = event.detail.value.length > 0;

        if (!this.show_pal_escape && !this.show_staff_escape) {
          this.show_schedule_appointment = this.can_show_schedule_appointment;
        }
        this.requestUpdate(); // Lit-Element native method.
      });
    }

    const visitable_named_bucket = this.getInstance('store_visitable_service_ids');
    if (!this.is_visitable && visitable_named_bucket !== undefined) {
      visitable_named_bucket.addEventListener('output-changed', (event) => {
        this.can_show_dropin = event.detail.value.length > 0;

        if (!this.show_pal_escape && !this.show_staff_escape) {
          this.show_dropin = this.can_show_dropin;
        }
        this.requestUpdate(); // Lit-Element native method.
      });
    }

    const requestable_named_bucket = this.getInstance('store_requestable_service_ids');
    if (!this.is_requestable && requestable_named_bucket !== undefined) {
      requestable_named_bucket.addEventListener('output-changed', (event) => {
        this.can_show_request_time = event.detail.value.length > 0;

        if (!this.show_pal_escape && !this.show_staff_escape) {
          this.show_request_time = this.can_show_request_time;
        }
        this.requestUpdate(); // Lit-Element native method.
      });
    }

    const custom_content_bucket = this.getInstance('custom_content');
    if (custom_content_bucket !== undefined) {
      custom_content_bucket.addEventListener('output-changed', (event) => {
        if (event.detail.value.student_scheduling) {
          this.student_scheduler_button_text = event.detail.value.student_scheduling.student_scheduler_button_text;
          this.view_dropin_label = event.detail.value.student_scheduling.view_dropin_label;
          this.request_appointment_label = event.detail.value.student_scheduling.request_appointment_label;
          this.meet_success_team_label = event.detail.value.student_scheduling.meet_success_team_label;
          this.scheduling_help_text = event.detail.value.student_scheduling.scheduling_help_text?.replace(
            /href=/g,
            'target="_blank" href='
          );
          this.drop_in_help_text = event.detail.value.student_scheduling.drop_in_help_text?.replace(
            /href=/g,
            'target="_blank" href='
          );
          this.request_help_text = event.detail.value.student_scheduling.request_help_text?.replace(
            /href=/g,
            'target="_blank" href='
          );
        }
        //Only need to update filter def, request update
        this.updateFilterDef(this.urlParams); //Lebels are updated here
        this.requestUpdate();
      });
    }

    this.getResults(this.urlParams);
    // Watch location changes and update.
    window.addEventListener('hashchange', this.locationHashWatcher);
  }

  disconnectedCallback() {
    super.disconnectedCallback();
    window.removeEventListener('hashchange', this.locationHashWatcher);
  }

  static get properties() {
    return {
      /**
       * all links are direct and not altered to work as relative links (client-side)
       */
      isDirect: Boolean,
      /**
       * the navigation bucket
       */
      index: Object
    };
  }

  get defaultScript() {
    return {
      system: {
        state: 'main'
      },
      action: {},
      bucket: {
        context: {},
        record: {}
      },
      io: {}
    };
  }

  render() {
    return html`<ai-screen gid="_screen" ns="${this.ns}">
      <ai-state gid="_state__main" ns="${this.ns}" states="main" class="">
        <div l-kind="hort">
          <div class="select_care_unit_service mbl" l-size="12" l-med-size="6" l-lrg-size="6">
            <!-- Filter Form -->
            <h2 class="clr-gray_90-txt mbm">What can we help you find?</h2>
            <hi-reveal max-height="3" class="help_text clr-gray_90-txt mbm">
              ${this.is_schedulable && this.scheduling_help_text ? unsafeHTML(this.scheduling_help_text) : ''}
              ${this.is_requestable && this.request_help_text ? unsafeHTML(this.request_help_text) : ''}
              ${this.is_visitable && this.drop_in_help_text ? unsafeHTML(this.drop_in_help_text) : ''}
            </hi-reveal>
            <div class="form_container clr-gray_30-bdr bdr-round-all bdr-solid bas plm prm">
              ${this.isFetchingResults ? this.__fetchingResultsTemplate() : this.__formTemplate()}
            </div>
          </div>
          <div class="other_options_list" l-size="12" l-med-size="6" l-lrg-size="6">
            ${this.__showOtherOptions() ? this.__otherAppointmentOptionsTemplate() : ``}
          </div>
        </div>
      </ai-state>
    </ai-screen> `;
  }

  getInstance(instanceName) {
    return HIP.db.gid.queryOne(instanceName, this.ns);
  }

  getDefinition(bucketName) {
    const bucket = this.getInstance(bucketName);
    if (bucket) {
      return bucket.output;
    }
  }

  getResults(URLParams) {
    this.isFetchingResults = true;
    this.apiParams = { ...this.urlParams };
    // Only narrow down with staff_id on this page.
    let params = {};
    if (URLParams.staff_ids) {
      params.staff_ids = URLParams.staff_ids;
    }
    if (URLParams.pal_guid) {
      params.pal_guid = URLParams.pal_guid;
    }
    this.getInstance('get_results').invoke(params);
  }

  // OPERATING LOGIC

  __submitForm(event) {
    this.getInstance('submit_record').invoke(event.detail.value);
  }

  __createCategorizedOptions(services, categories) {
    let sequencedOptions = [];
    if (categories && categories.length > 0) {
      const mappedCategories = categories.map((cat) => {
        return {
          ...cat,
          services: services.filter((opt) => cat.service_ids.includes(opt.id))
        };
      });
      mappedCategories.forEach((cat, catIndex) => {
        if (cat.services.length > 0) {
          sequencedOptions.push({
            title: cat.title,
            disabled: true,
            heading: true,
            order: parseInt(`${catIndex.toString().padStart(6, '0')}000000`, 10)
          });
          cat.services.forEach((opt, optIndex) => {
            sequencedOptions.push({
              title: opt.title,
              value: opt.value,
              order: parseInt(`${catIndex.toString().padStart(6, '0')}${optIndex.toString().padStart(6, '0')}`, 10)
            });
          });
        }
      });
      // Uncategorized services
      const uncategorizedServices = services.filter((ser) => ser.service_category_ids.length == 0);
      if (uncategorizedServices.length > 0) {
        if (sequencedOptions.length > 0) {
          sequencedOptions.push({
            title: 'Other',
            value: null,
            disabled: true,
            heading: true,
            order: parseInt(`999999000000`, 10)
          });
        }
        uncategorizedServices.forEach((opt, optIndex) => {
          sequencedOptions.push({
            title: opt.title,
            value: opt.value,
            order: parseInt(`999999${optIndex.toString().padStart(6, '0')}`, 10)
          });
        });
      }
    } else {
      sequencedOptions = services;
    }

    return sequencedOptions;
  }

  updateFilterDef(formData, doAutoSelect = false) {
    const def = this.getDefinition('record_type');
    const care_units = this.getDefinition('care-units-record') || [];
    const services = this.getDefinition('services-record') || [];
    const categories = this.getDefinition('services-categories') || [];
    const staff = this.getDefinition('staff-record') || [];
    const meetingTypes = this.getDefinition('meeting-type-record') || [];
    const availableTimes = this.getDefinition('times-record') || [];
    const customContentText = this.getDefinition('custom_content') || {};
    this.__availableDates = availableTimes;

    /**
     * Below code is mapping dropdown values & setting picked options from URL.
     */
    if (def && def.properties) {
      // Show single choice as user should not update it
      def.properties.care_unit_id.options = care_units;
      if (doAutoSelect && def.properties.care_unit_id.options.length === 1) {
        formData.care_unit_id = def.properties.care_unit_id.options[0].id;
      }

      if (formData.care_unit_id) {
        // def.properties.care_unit_id.options = care_units.filter((c) => c.id == formData.care_unit_id);
        def.properties.care_unit_id.initial_value = [formData.care_unit_id];
      } else {
        def.properties.care_unit_id.initial_value = null;
      }
      // Show single choice as user should not update it
      const filteredServices = formData.care_unit_id
        ? services.filter((s) => s.care_unit_id == formData.care_unit_id)
        : [];
      def.properties.service_id.options = this.__createCategorizedOptions(filteredServices, categories);

      if (formData.care_unit_id == undefined) {
        def.properties.service_id.initial_value = null;
      } else {
        if (doAutoSelect && filteredServices.length === 1) {
          formData.service_id = filteredServices[0].value;
        }
        if (formData.service_id) {
          def.properties.service_id.initial_value = [formData.service_id];
        } else {
          def.properties.service_id.initial_value = null;
        }
      }

      if (def.properties.staff_ids) {
        def.properties.staff_ids.options = staff;
        if (formData.staff_ids) {
          def.properties.staff_ids.initial_value = [formData.staff_ids];
        } else {
          def.properties.staff_ids.initial_value = null;
        }
      }

      if (def.properties.begin_date && def.properties.begin_date.attributes) {
        def.properties.begin_date.attributes['highlight-dates'] = JSON.stringify(
          availableTimes.map((el) => {
            if (el.includes('-')) {
              return el;
            } else {
              // Restore the dashes if the server did not give us dashes
              return `${el.substring(0, 4)}-${el.substring(4, 6)}-${el.substring(6, 8)}`;
            }
          })
        );
      }
      if (formData.begin_date && def.properties.begin_date) {
        def.properties.begin_date.initial_value = formData.begin_date;
      }

      if ('pal_guid' in def.properties && 'pal_guid' in formData) {
        def.properties.pal_guid.initial_value = formData.pal_guid;
      } else if ('pal_guid' in def.properties) {
        def.properties.pal_guid.initial_value = null;
      }
      // Udpate custom labels
      if (customContentText.student_scheduling.care_unit_label) {
        def.properties.care_unit_id.title = customContentText.student_scheduling.care_unit_label;
      }
      if (customContentText.student_scheduling.service_label) {
        def.properties.service_id.title = customContentText.student_scheduling.service_label;
      }

      this.filteredFormDefinition = { ...def };
    }

    this.requestUpdate(); // Lit-Element native method.
  }

  __updateFormFields(e) {
    const formData = e.detail.value.valueAll;
    // if (this.getInstance('selected_form_filters')) {
    //   this.getInstance('selected_form_filters').invoke(formData);
    // }
    this.setURLParams(formData);
    // this.updateFilterDef(formData);
  }

  __showOtherOptions() {
    return (
      this.show_dropin ||
      this.show_request_time ||
      this.show_success_team ||
      this.show_pal_escape ||
      this.show_staff_escape
    );
  }

  setURLParams(data) {
    // Compare to existing params
    const urlParamsKeys = Object.keys(this.urlParams);
    const dataKeys = Object.keys(data);
    const combinedKeys = new Set([...urlParamsKeys, ...dataKeys]);
    let skipHashUpdate = true;
    combinedKeys.forEach((param) => {
      skipHashUpdate = skipHashUpdate && this.urlParams[param] == data[param];
    });

    const def = this.getDefinition('record_type');
    // Only manage params part of the form, Other params should be retained
    Object.keys(def.properties).forEach((propName) => {
      const visibleFormElementKey = def.properties[propName].render_type != 'hidden';
      const hasNoValue = data[propName] === undefined || data[propName] === '';
      if (visibleFormElementKey && hasNoValue) {
        delete data[propName];
        delete this.urlParams[propName];
      }
    });
    this.urlParams = { ...data };

    const currentParams = getCurrentLocationParams().substring(1);
    const serializedQueryParams = window.safe.serializeParameters(this.urlParams);
    // Dont trigger update if nothing changed, prevents unnecessary UI refresh
    if (!skipHashUpdate) {
      const urlPartWithoutParams = window.location.href.split('?')[0];
      window.location.assign(`${urlPartWithoutParams}?${serializedQueryParams}`);
    }
  }

  // TEMPLATE PARTIALS

  __fetchingResultsTemplate() {
    return html`<div class="text-center pamx min-h-360">
      <div l-kind l-width-fixed l-align="center" l-justify="center" class="min-h-120">
        <hi-indicator></hi-indicator>
      </div>
      <p class="text-sz-lx text-bold">Loading Available Services...</p>
      <p class="clr-gray_70-txt"></p>
    </div>`;
  }

  __formTemplate() {
    if (this.servicesRecord.length === 0) {
      return html` <p>Sorry! There are no availabilities supporting the scheduling of Appointments at this time.</p> `;
    } else {
      return html`
        <hi-form
          ns="${this.ns}"
          id="_filters_form"
          gid="_filters_form"
          aesthetic="invisible"
          flush
          has-controls
          disable-http=""
          @evaluate-ended="${this.__updateFormFields}"
          @submit-ended="${this.__submitForm}"
        >
          <ai-builder-form gid="_filters_form_builder" ns="${this.ns}" .input=${this.filteredFormDefinition}>
          </ai-builder-form>
          <hi-button aesthetic="primary" slot="controls" type="submit">Find Available Time</hi-button>
        </hi-form>
      `;
    }
  }

  __otherAppointmentOptionsTemplate() {
    return html`
      <div class="other_options">
        <!-- Appointment Results -->
        <h2 class="clr-gray_90-txt mbm">Other Options</h2>
        <ul kind="no-dot">
          ${this.show_schedule_appointment ? this.__scheduleAppointmentTemplate() : ``}
          ${this.show_dropin ? this.__dropinOptionTemplate() : ``}
          ${this.show_request_time ? this.__requestOptionTemplate() : ``}
          ${this.show_success_team ? this.__successTeamOptionTemplate() : ``}
          ${this.show_pal_escape ? this.__palEscapeOptionTemplate() : ``}
          ${this.show_staff_escape ? this.__staffEscapeOptionTemplate() : ``}
        </ul>
      </div>
    `;
  }

  __scheduleAppointmentTemplate() {
    return html`
      <li>
        <hi-link
          href="#"
          class="other_option_link pam display-block clr-gray_30-bdr bdr-round-all bdr-solid bas mts"
          @trigger-ended="${this.scheduleAppointmentAction}"
          >${this.student_scheduler_button_text}</hi-link
        >
      </li>
    `;
  }

  __dropinOptionTemplate() {
    return html`
      <li>
        <hi-link
          href="#"
          class="other_option_link pam display-block clr-gray_30-bdr bdr-round-all bdr-solid bas mts"
          @trigger-ended="${this.viewDropInsAction}"
          >${this.view_dropin_label}</hi-link
        >
      </li>
    `;
  }

  __requestOptionTemplate() {
    return html`
      <li>
        <hi-link
          href="#"
          class="pam display-block clr-gray_30-bdr bdr-round-all bdr-solid bas mts"
          @trigger-ended="${this.requestAlternateTimeAction}"
          >${this.request_appointment_label}</hi-link
        >
      </li>
    `;
  }

  __successTeamOptionTemplate() {
    return html`
      <li>
        <hi-link
          href="#"
          class="pam display-block clr-gray_30-bdr bdr-round-all bdr-solid bas mts"
          @trigger-ended="${this.meetSuccessTeamAction}"
          >${this.meet_success_team_label}</hi-link
        >
      </li>
    `;
  }

  __palEscapeOptionTemplate() {
    return html`
      <li>
        <div class="bas bdr-solid clr-gray_30-bdr pam masm mtm bdr-round-m">
          <hi-link href="#" @trigger-ended="${this.__escapeHatch}">See All Scheduling Options</hi-link>
          <p>
            You are currently viewing options that are limited to the availabilities in the link you clicked.
            <hi-link href="#" @trigger-ended="${this.__escapeHatch}">Click here</hi-link> to go back to main scheduling
            workflow for all other options.
          </p>
        </div>
      </li>
    `;
  }

  __staffEscapeOptionTemplate() {
    return html`
      <li>
        <div class="bas bdr-solid clr-gray_30-bdr pam masm mtm bdr-round-m">
          <hi-link href="#" @trigger-ended="${this.__escapeHatch}">See All Scheduling Options</hi-link>
          <p>
            You are currently viewing options that are limited to your success team.
            <hi-link href="#" @trigger-ended="${this.__escapeHatch}">Click here</hi-link> to go back to main scheduling
            workflow for all other options.
          </p>
        </div>
      </li>
    `;
  }

  __escapeHatch() {
    this.setURLParams({});
    this.show_pal_escape = false;
    this.show_staff_escape = false;
    this.show_schedule_appointment = this.can_show_schedule_appointment;
    this.show_dropin = this.can_show_dropin;
    this.show_request_time = this.can_show_request_time;
    this.show_success_team = this.can_show_success_team;
    this.goToNewPageAction();
  }

  // ACTIONS
  goToNewPageAction() {
    const newPageAction = this.getInstance('view_new_page');
    newPageAction.invoke({});
  }

  requestAlternateTimeAction() {
    const requestApptAction = this.getInstance('request_alternate');
    requestApptAction.invoke({});
  }

  meetSuccessTeamAction() {
    const meetSuccessTeam = this.getInstance('meet_success_team');
    meetSuccessTeam.invoke({});
  }

  viewDropInsAction() {
    const viewDropIns = this.getInstance('view_drop_ins');
    viewDropIns.invoke({});
  }

  scheduleAppointmentAction() {
    const scheduleAppointment = this.getInstance('view_new_page');
    scheduleAppointment.invoke({});
  }
}

// customElements.define('ai-prefab-appointment-results', PrefabAppointmentResultsElement);
HIP.factory.element(PrefabAppointmentResultsElement);
