const {
  html,
  nothing,
  directives: { unsafeHTML }
} = HIP.extern.lit;
export const id = 'appointment-request__detail';
export const type = 'template';
export const template = (hilType) => {
  return () => {
    // the HIL template where we can pre-calculate information too
    const typeMap = tt.filter((o) => !o.hidden, hilType);
    // ignored for templates because we do not get sub-item templates
    return (o, index) => {
      // called for each item and should return a lit-html template
      const formatDataByFormatType = (data) => {
        return tt.map((value, key) => {
          const fieldType = typeMap[key];
          return fieldType && fieldType.format_type
            ? HIP.fn.format(value, fieldType.format_type, tt.get('$configure.format_type', fieldType))
            : value;
        }, data);
      };
      const formattedData = formatDataByFormatType(o);
      const [date, ...time] = formattedData.start_time.split(' ');
      const staffNames = o.staff.map((s) => s.name).join(',');
      return html`
        <style>
          dt {
            font-weight: var(--hi-font--weight-normal);
            font-size: var(--hi-font-size--normal);
            padding-bottom: var(--hi-size--small-x);
          }
          dl,
          dd {
            font-size: var(--hi-font-size--normal);
          }
          dl > div {
            border-bottom: 1px solid var(--hi-color--gray-30);
            padding-top: var(--hi-size--medium);
            padding-bottom: var(--hi-size--medium);
          }
          dd {
            color: var(--hi-color--gray-80);
            font-weight: var(--hi-font--weight-light);
          }
        </style>
        <div>
          <h2 class="appointment-details-header pam">${o.name} with ${staffNames}</h2>
          <dl l-kind="hort" l-justify="center">
            <div l-size="12" l-sml-size="6">
              <dt>Date</dt>
              <dd>${date}</dd>
            </div>
            <div l-size="12" l-sml-size="6">
              <dt>Time</dt>
              <dd>${time}</dd>
            </div>
            <div l-size="12">
              <dt>Location</dt>
              <dd>${o.location}</dd>
            </div>
            ${o.course
              ? html`<div l-size="12">
                  <dt>Course</dt>
                  <dd>${o.course}</dd>
                </div>`
              : ``}
            <div l-size="12">
              <dt>People</dt>
              <dd>${staffNames}</dd>
            </div>
            <div l-size="12">
              <dt>Details</dt>
              <dd>${unsafeHTML(o.details)}</dd>
            </div>
            <div l-size="12">
              <dt>Comments</dt>
              <dd>${o.comments}</dd>
            </div>
          </dl>
        </div>
      `;
    };
  };
};
