const {
  html,
  nothing,
  directives: { unsafeHTML }
} = HIP.extern.lit;
export const id = 'appointment_campaign_list__item_detail';
export const type = 'template';
export const template = (hilType, hilData, builderEl) => {
  return () => {
    // the HIL template where we can pre-calculate information too
    const typeMap = tt.filter((o) => !o.hidden, hilType);
    // ignored for templates because we do not get sub-item templates
    return (o, index) => {
      // called for each item and should return a lit-html template
      const formatDataByFormatType = (data) => {
        return tt.map((value, key) => {
          const fieldType = typeMap[key];
          return fieldType && fieldType.format_type
            ? HIP.fn.format(value, fieldType.format_type, tt.get('$configure.format_type', fieldType))
            : value;
        }, data);
      };
      const formatDate = (date) => {
        let dateArray = date.split('/');
        const year = dateArray.pop();
        dateArray.unshift(year);
        return dateArray.map((chunk) => chunk.padStart(2, '0')).join('-');
      };
      const formattedData = formatDataByFormatType(o);
      const [startDate] = formattedData.start_time.split(' ');
      const [endDate] = formattedData.end_time.split(' ');
      const staffNames = o.staff.map((s) => s.name).join(',');
      // TODO: Use Moment or DayJS in Future
      const dateFormatted = new Intl.DateTimeFormat('default', {
        weekday: 'long',
        month: 'long',
        day: 'numeric'
      }).format(new Date(endDate));

      const params = {
        care_unit_id: o.care_unit.id,
        service_id: o.services[0].id,
        begin_date: moment.max([moment(o.start_time), moment()]).format('YYYY-MM-DD'),
        staff_ids: o.staff[0].id,
        location_id: o.location.id,
        course_id: o.course_id,
        invitation_id: o.id
      };
      if (o.course_id == null) delete params.course_id;
      if (o.staff.length > 1) delete params.staff_ids;

      HIP.db.gid.queryOne('params', builderEl.ns).input = params;

      return html`
        <style>
          dt {
            font-weight: var(--hi-font--weight-normal);
            font-size: var(--hi-font-size--normal);
            padding-bottom: var(--hi-size--small-x);
          }
          dl,
          dd {
            font-size: var(--hi-font-size--normal);
          }
          dl > div {
            border-bottom: 1px solid var(--hi-color--gray-30);
            padding-top: var(--hi-size--medium);
            padding-bottom: var(--hi-size--medium);
            padding-left: var(--hi-size--medium);
          }
          dd {
            color: var(--hi-color--gray-80);
            font-weight: var(--hi-font--weight-light);
          }
        </style>
        <div class="">
          <p class="pam">Please respond to this appointment by <span class="text-bold">${dateFormatted}</span></p>
          <dl l-kind="hort" l-justify="center">
            <div l-size="12">
              <dt>Service</dt>
              <dd>${o.services[0].name}</dd>
            </div>
            <div l-size="12">
              <dt>Location</dt>
              <dd>${o.location.name}</dd>
            </div>
            <div l-size="12">
              <dt>People</dt>
              <dd>${staffNames}</dd>
            </div>
            ${o.course
              ? html`<div l-size="12">
                  <dt>Course</dt>
                  <dd>${o.course}</dd>
                </div>`
              : ``}
          </dl>
        </div>
      `;
    };
  };
};
