const {
  html,
  nothing,
  directives: { repeat }
} = HIP.extern.lit;
export const id = 'appointments_list__item';
export const type = 'template';

const checkIn = (attendance_id, builderEl) => {
  return (event) => {
    const checkInAction = HIP.db.gid.queryOne('check_in', builderEl.ns);
    if (checkInAction) {
      checkInAction.invoke({ attendance_id });
    }
  };
};
const checkOut = (visit_id, builderEl) => {
  return (event) => {
    const checkInAction = HIP.db.gid.queryOne('check_out', builderEl.ns);
    if (checkInAction) {
      checkInAction.invoke({ id });
    }
  };
};

export const template = (hilType, typeAliases, builderEl) => {
  // the HIL template where we can pre-calculate information too
  const typeMap = tt.filter((o) => !o.hidden, hilType);
  const formatDataByFormatType = (data) => {
    return tt.map((value, key) => {
      const fieldType = typeMap[key];
      return fieldType && fieldType.format_type
        ? HIP.fn.format(value, fieldType.format_type, tt.get('$configure.format_type', fieldType))
        : value;
    }, data);
  };
  return () => {
    // ignored for templates because we do not get sub-item templates
    return (o, index) => {
      // called for each item and should return a lit-html template
      const formattedData = formatDataByFormatType(o);
      let [date, ...time] = formattedData.start_time.split(' ');
      time = `${time[0].substring(0, 5)} ${time[1]}`; // Format 09:00:00AM -> 09:00 AM

      let checkedInTime = moment(formattedData.checked_in_time).format('DD/MM/YYYY h:mm A');

      const detailsLink = `#/my/appointment-details?id=${o.instance_id}`;
      return html`<li class="pam bdr-solid bbs clr-gray_40-bdr" l-kind="hort" l-align="center">
        <div l-size="11">
          <div>
            <hi-link href="${detailsLink}">
              <p class="text-sz-lx pbsx">${o.name} with ${o.staff.map((s) => s.name).join(',')}</p>
            </hi-link>
            <p class="clr-gray_80-txt">Scheduled for ${date} ${time}</p>
            ${o.meeting_type ? html`<p class="clr-gray_80-txt">Meeting Type: ${o.meeting_type}</p>` : ``}
            ${o.feedback_token ? html`<hi-badge>Feedback Request Open</hi-badge>` : ``}
          </div>
          ${o.already_checked_in
            ? html`<p class="clr-gray_80-txt text-italic">(Checked In at ${checkedInTime})</p>`
            : o.can_checkin
            ? html`<hi-button class="mtm" aesthetic="secondary" @trigger-ended="${checkIn(o.attendance_id, builderEl)}"
                >Check In</hi-button
              >`
            : ``}
        </div>
        <div l-size="1" class="text-right">
          <hi-link aesthetic="icon" href="${detailsLink}">
            <hi-icon kind="chevron_right" theme="gray" size="large-x"></hi-icon>
          </hi-link>
        </div>
      </li> `;
    };
  };
};
