const {
  html,
  nothing,
  directives: { unsafeHTML }
} = HIP.extern.lit;
export const id = 'appointments_list__item_detail';
export const type = 'template';
export const template = (hilType, hilData, builderEl) => {
  return () => {
    // the HIL template where we can pre-calculate information too
    const typeMap = tt.filter((o) => !o.hidden, hilType);
    // ignored for templates because we do not get sub-item templates
    return (o, index) => {
      if (Array.isArray(o)) {
        return ``;
      }
      // called for each item and should return a lit-html template
      const formatDataByFormatType = (data) => {
        return tt.map((value, key) => {
          const fieldType = typeMap[key];
          return fieldType && fieldType.format_type
            ? HIP.fn.format(value, fieldType.format_type, tt.get('$configure.format_type', fieldType))
            : value;
        }, data);
      };
      const give_feedback_button_pressed = (event) => {
        HIP.db.gid.queryOne('give_feedback', hilData.properties.details.ns).invoke({ token: o.feedback_token });
      };
      const formattedData = formatDataByFormatType(o);
      let [date, ...time] = formattedData.start_time.split(' ');
      time = `${time[0].substring(0, 5)} ${time[1]}`; // Format 09:00:00AM -> 09:00AM
      const staffNames = o.staff.map((s) => s.name).join(',');
      HIP.db.gid.queryOne('params', builderEl.ns).input = {
        id: o.instance_id
      };

      let group_appointment_text = '';
      if (staffNames && o.is_group_appointment) {
        if (o.other_attendees_count === 0) {
          group_appointment_text = `and up to ${o.total_possible_attendees - 1} other(s)`;
        } else {
          group_appointment_text = `and ${o.other_attendees_count} other(s)`;
        }
      }

      return html`
        <style>
          dt {
            font-weight: var(--hi-font--weight-normal);
            font-size: var(--hi-font-size--normal);
            padding-bottom: var(--hi-size--small-x);
          }
          dl,
          dd {
            font-size: var(--hi-font-size--normal);
          }
          dl > div {
            border-bottom: 1px solid var(--hi-color--gray-30);
            padding-top: var(--hi-size--medium);
            padding-bottom: var(--hi-size--medium);
            padding-left: var(--hi-size--medium);
          }
          dd {
            color: var(--hi-color--gray-80);
            font-weight: var(--hi-font--weight-light);
          }
        </style>
        ${group_appointment_text
          ? html`<hi-alert theme="warning" class="mbm" mode="presentation"
              ><p>This is a group appointment with ${staffNames} ${group_appointment_text}</p></hi-alert
            >`
          : ``}
        <div class="">
          <h2 class="appointment-details-header pam">${o.name} with ${staffNames}</h2>
          ${o.feedback_token
            ? html` <div class="pam">
                <hi-button @trigger-ended="${give_feedback_button_pressed}">
                  <hi-icon glyph="check_circle_outline"></hi-icon>
                  <span>Give appointment feedback</span>
                </hi-button>
              </div>`
            : ``}
          <dl l-kind="hort" l-justify="center">
            <div l-size="12" l-sml-size="6">
              <dt>Care Unit</dt>
              <dd>${o.event_type}</dd>
            </div>
            <div l-size="12" l-sml-size="6">
              <dt>Service</dt>
              <dd>${o.name}</dd>
            </div>
            <div l-size="12" l-sml-size="6">
              <dt>Date</dt>
              <dd>${date}</dd>
            </div>
            <div l-size="12" l-sml-size="6">
              <dt>Time</dt>
              <dd>${time}</dd>
            </div>
            <div l-size="12">
              <dt>Location</dt>
              <dd>${o.location}</dd>
            </div>
            ${o.course
              ? html`<div l-size="12">
                  <dt>Course</dt>
                  <dd>${o.course}</dd>
                </div>`
              : ``}
            <div l-size="12">
              <dt>People</dt>
              <dd>${staffNames}</dd>
            </div>
            ${o.details
              ? html`<div l-size="12" class="availability_details">
                  <dt>Details</dt>
                  <dd>${unsafeHTML(o.details)}</dd>
                </div>`
              : ``}
            ${o.meeting_url_or_phone
              ? html`<div l-size="12">
                  <dt>URL / Phone Number</dt>
                  <dd>${o.meeting_url_or_phone}</dd>
                </div>`
              : ``}
            <div l-size="12">
              <dt>Comments</dt>
              <dd>${o.comments}</dd>
            </div>
            ${o.meeting_type
              ? html`<div l-size="12">
                  <dt>Meeting Type</dt>
                  <dd>${o.meeting_type}</dd>
                </div>`
              : ``}
          </dl>
        </div>
      `;
    };
  };
};
