const {
  html,
  nothing,
  directives: { repeat }
} = HIP.extern.lit;
export const id = 'template-appointment-result';
export const type = 'template';

const avatarsTemplate = (staffs) => {
  return html`<div l-kind l-width-fixed l-align="center" class="ptm">
    <div>
      ${repeat(staffs, (sta) => {
        return html`<hi-avatar
          class="non-interactive"
          size="small"
          layout-inline
          initials="${sta.first_name[0] + sta.last_name[0]}"
          image="${sta.picture}"
        ></hi-avatar>`;
      })}
    </div>
    <span class="text-sz-m mlm">${staffs.length} Staffs</span>
  </div>`;
};

const selectTime = (appointment, builderEl) => {
  return (event) => {
    const selectAppointmentAction = HIP.db.gid.queryOne('select_appointment', builderEl.ns);
    const selected_appointment_values = {
      begin_time: event.detail.value,
      duration: appointment.duration,
      service_id: appointment.service_id,
      location_id: appointment.location_id,
      staff_id: appointment.staffs[0].id,
      care_unit_id: appointment.care_unit_id,
      course_id: appointment.course_id,
      appointment: appointment
    };
    if (selectAppointmentAction) {
      selectAppointmentAction.invoke(selected_appointment_values);
    }
  };
};

const dropinTemplate = (appointment) => {
  return appointment.dropin_available
    ? html` <hi-badge aesthetic="light" theme="cyan" size="small">Drop-in time</hi-badge> `
    : ``;
};

const timesTemplate = (appointment, staffTimes, duration, builderEl) => {
  return html`${repeat(staffTimes, (time, index) => {
    const fromTime = new Date(time);
    const fromTimeUnix = fromTime.getTime();
    const toTime = new Date().setTime(fromTimeUnix + duration * 60000);
    // TODO: Use Moment or DayJS in Future
    const timeFormatter = new Intl.DateTimeFormat('default', {
      hour12: true,
      hour: 'numeric',
      minute: 'numeric'
    });
    const fromTimeFormatted = timeFormatter.format(fromTime);
    const toTimeFormatted = timeFormatter.format(toTime);
    return html`<hi-button
      size="small"
      @trigger-ended="${selectTime(appointment, builderEl)}"
      aesthetic="secondary"
      class="bdr-round-full mrsx mtsx"
      value="${time}"
      >${fromTimeFormatted} - ${toTimeFormatted}</hi-button
    >`;
  })}`;
};
export const template = (hilType, typeAliases, builderEl) => {
  return () => {
    return (appointment) => {
      return html`
        <div class="clr-white-bkg pal bas bdr-solid clr-gray_20-bdr mbl">
          <div class="mbmsx">
            <hi-link href="#"><p class="text-sz-lx">${appointment.location.name}</p></hi-link>
          </div>
          ${dropinTemplate(appointment)} ${avatarsTemplate(appointment.staffs)}
          <div class="ptm">${timesTemplate(appointment, appointment.times, appointment.duration, builderEl)}</div>
        </div>
      `;
    };
  };
};
